#
#    Value                           1 1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 2 2 2 2 3  3       
#                0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0  1              
#     I_1:     |   opcode  |                       LI                      |AA|LK|
#
#     B_1:     |   opcode  |    BO   |    BI   |            BD             |AA|LK|
#
#    SC_1:     |   opcode  |   ///   |   ///   |           ///             |XO|LK|
#    SC_2:     |   opcode  |   ///   |   ///   |  ///  |   LEV       | /// |XO|LK|
#
#     D_1:     |   opcode  |    RT   |    RA   |                D                |
#     D_2:     |   opcode  |    RT   |    RA   |               SI                |
#     D_3:     |   opcode  |    RS   |    RA   |                D                |
#     D_4:     |   opcode  |    RS   |    RA   |               SI                |
#     D_5:     |   opcode  |  BF |/|L|    RA   |               SI                |
#     D_6:     |   opcode  |  BF |/|L|    RA   |               UI                |
#     D_7:     |   opcode  |    TO   |    RA   |               SI                |
#     D_8:     |   opcode  |   FRT   |    RA   |                D                |
#     D_9:     |   opcode  |   FRS   |    RA   |                D                |
#
#    DS_1:     |   opcode  |    RT   |    RA   |               DS          | XO  |
#    DS_2:     |   opcode  |    RS   |    RA   |               DS          | XO  |
#
#    DQ_1:     |   opcode  |    RT   |    RA   |          DQ           |    PT  |
#
#    X_1:      |   opcode  |    RT   |    RA   |    RB   |         XO         |//|
#    X_2:      |   opcode  |    RT   |    RA   |    NB   |         XO         |//|
#    X_3:      |   opcode  |    RS   |    RA   |    RB   |         XO         |Rc|
#    X_4:      |   opcode  |    RS   |    RA   |    NB   |         XO         |//|
#    X_5:      |   opcode  |    RS   |    RA   |    SH   |         XO         |Rc|
#    X_6:      |   opcode  |    RS   |   ///   |   ///   |         XO         |//|
#    X_7:      |   opcode  |  BF |/|L|    RA   |    RB   |         XO         |//|
#    X_8:      |   opcode  |  BF |///|   FRA   |   FRB   |         XO         |//|
#    X_9:      |   opcode  |  BF |///| BFA |///|   ///   |         XO         |//|
#    X_10:     |   opcode  |  BF |///|   ///   |    U  |/|         XO         |Rc|
#    X_11:     |   opcode  |    TO   |    RA   |    RB   |         XO         |//|
#    X_12:     |   opcode  |   FRT   |    RA   |    RB   |         XO         |//|
#    X_13:     |   opcode  |   FRT   |   ///   |   FRB   |         XO         |Rc|
#    X_14:     |   opcode  |   FRS   |    RA   |    RB   |         XO         |//|
#    X_15:     |   opcode  |    BT   |   ///   |   ///   |         XO         |Rc|
#    X_16:     |   opcode  |    RS   |/|   SR  |   ///   |         XO         |//|
#    X_17:     |   opcode  |    RT   |/|   SR  |   ///   |         XO         |//|
#    X_18:     |   opcode  |T|//|STRM|    RA   |    RB   |         XO         |//|
#    X_19:     |   opcode  |    VS   |    RA   |    RB   |         XO         |Rc|
#    X_20:     |   opcode  |    VT   |    RA   |    RB   |           XO        |0|
#
#    XL_1:     |   opcode  |    BT   |    BA   |    BB   |         XO         |//|
#    XL_2:     |   opcode  |    BO   |    RI   |   ///   |         XO         |LK|
#    XL_3:     |   opcode  |  BF |///| BFA |///|   ///   |         XO         |//|
#    XL_4:     |   opcode  |   ///   |   ///   |   ///   |         XO         |//|
#
#   XFX_1:     |   opcode  |    RT   |        SPR        |         XO         |//|
#   XFX_2:     |   opcode  |    RT   |        TBR        |         XO         |//|
#   XFX_3:     |   opcode  |    RS   |/|      FXM      |/|         XO         |//|
#   XFX_4:     |   opcode  |    RS   |        SPR        |         XO         |//|
#   XFX_5:     |   opcode  |   ///   |        XO2        |         XO         |//|
#
#   XFL_1:     |   opcode  |/|      FLM      |/|   FRB   |         XO         |Rc|
#
#    XS_1:     |   opcode  |    RS   |    RA   |    SH   |         XO      |SH|Rc|
#
#    XO_1:     |   opcode  |    RT   |    RA   |    RB   |OE|      XO         |Rc|
#    XO_2:     |   opcode  |    RT   |    RA   |    RB   |//|      XO         |Rc|
#
#     A_1:     |   opcode  |   FRT   |   FRA   |   FRB   |   FRC   |    XO    |Rc| 
#
#     M_1:     |   opcode  |    RS   |    RA   |    RB   |    MB   |    ME    |Rc|
#     M_2:     |   opcode  |    RS   |    RA   |    SH   |    MB   |    ME    |Rc|
#
#    MD_1:     |   opcode  |    RS   |    RA   |    SH   |    MB   |   XO  |SH|Rc|
#    MD_2:     |   opcode  |    RS   |    RA   |    SH   |    ME   |   XO  |SH|Rc|
#
#   MDS_1:     |   opcode  |    RS   |    RA   |    RB   |    MB   |     XO   |Rc|
#   MDS_2:     |   opcode  |    RS   |    RA   |    RB   |    ME   |     XO   |Rc|
#   MDS_3:     |   opcode  |    IS   |    RA   |    IB   | XBI  |//|     XO   |Rc|
#   MDS_4:     |   opcode  |    IS   |    RA   |    RB   | XBI  |//|     XO   |Rc|
#   MDS_5:     |   opcode  |    RS   |    RA   |    IB   | XBI  |//|     XO   |Rc|
#   MDS_6:     |   opcode  |    RS   |    RA   |    RB   | XBI  |//|     XO   |Rc|
#   MDS_3n:    |   opcode  |    IS   |    RA   |    IB   |  CBI  |/|     XO   |Rc|
#   MDS_4n:    |   opcode  |    IS   |    RA   |    RB   |  CBI  |/|     XO   |Rc|
#   MDS_5n:    |   opcode  |    RS   |    RA   |    IB   |  CBI  |/|     XO   |Rc|
#   MDS_6n:    |   opcode  |    RS   |    RA   |    RB   |  CBI  |/|     XO   |Rc|
#
#    TX_1:     |   opcode  |    TO   |        UI         |  XBI |       XO    |Rc|
#
#    VA_1:     |   opcode  |    VT   |    VA   |    VB   |    VC   |     XO      |
#    VA_2:     |   opcode  |    VT   |    VA   |    VB   |/|   SH  |     XO      |
#
#    VX_1:     |   opcode  |    VT   |    VA   |    VB   |           XO          |
#    VX_2:     |   opcode  |    VT   |   UIMM  |    VB   |           XO          |
#    VX_3:     |   opcode  |    VT   |   SIMM  |    VB   |           XO          |
#
#    VXR_1:    |   opcode  |    VT   |    VA   |    VB   |Rc|        XO          |
#


sub DoInitialGenerateAssembly {
    local (*TOFILE) = $_[0];
}

#* Generate OPC_ASM assembly-code macros *#
sub DoGenerateAssemblyForThisInstruction {
    local (*TOFILE) = $_[0];

    printf( TOFILE "int\n");
    printf( TOFILE "opc_as_OPC_%s(\n", $do_id);
    printf( TOFILE "\tchar *\topcode,\n");
    printf( TOFILE "\tint\tnargs,\n");
    printf( TOFILE "\tchar *\targs[],\n");
    printf( TOFILE "\tunsigned *\tp_iword\n");
    printf( TOFILE "\t)\n");
    printf( TOFILE "{\n");
    printf( TOFILE "  unsigned\tiword;\n");
    printf( TOFILE "  char *\tm_arg;\n");
    printf( TOFILE "  unsigned\tm_args_at = 0;\n");
    printf( TOFILE "  int\tm_error;\n");
    printf( TOFILE "  unsigned\tm_errline;\n");
    printf( TOFILE "  char *\tm_arg_end;\n");

    $prefix = "";
    
    $assert = sprintf( "  assert( ");
    if( $do_ppc_name ) {
      $assert = $assert . sprintf( "%s (strcmp(opcode, \"%s\") == 0)", $prefix, $do_ppc_name );
      $prefix = "\n\t||"
    }
    if( ($do_power_name) && ($do_power_name ne $do_ppc_name) ) {
      $assert = $assert . sprintf( "%s (strcmp(opcode, \"%s\") == 0)", $prefix, $do_power_name );
      $prefix = "\n\t||"
    }
    if( ($do_power2_name) && ($do_power2_name ne $do_ppc_name) && ($do_power2_name ne $do_power_name) ) {
      $assert = $assert . sprintf( "%s (strcmp(opcode, \"%s\") == 0)", $prefix, $do_power2_name );
      $prefix = "\n\t||"
    }
    if( ($do_amazon_name) && ($do_amazon_name ne $do_ppc_name) && ($do_amazon_name ne $do_power_name) && ($do_amazon_name ne $do_power2_name)) {
      $assert = $assert . sprintf( "%s (strcmp(opcode, \"%s\") == 0)", $prefix, $do_amazon_name );
      $prefix = "\n\t||"
    }
    if( ($do_ppc32_name) && ($do_ppc32_name ne $do_ppc_name) && ($do_ppc32_name ne $do_power_name) && ($do_ppc32_name ne $do_power2_name) && ($do_ppc32_name ne $do_amazon_name) ) {
      $assert = $assert . sprintf( "%s (strcmp(opcode, \"%s\") == 0)", $prefix, $do_ppc32_name );
      $prefix = "\n\t||"
    }
    if( ($do_ppc40x_name) && ($do_ppc40x_name ne $do_ppc_name) && ($do_ppc40x_name ne $do_power_name) && ($do_ppc40x_name ne $do_power2_name) && ($do_ppc40x_name ne $do_amazon_name) && ($do_ppc40x_name ne $do_ppc32_name) ) {
      $assert = $assert . sprintf( "%s (strcmp(opcode, \"%s\") == 0)", $prefix, $do_ppc40x_name );
      $prefix = "\n\t||"
    }
    
    $assert = $assert . sprintf( " );\n");

    if( $do_full_format eq "I_1" ) {
	printf( TOFILE "  unsigned\tli;\n");
	printf( TOFILE "\n%s\n", $assert);
	printf( TOFILE "  GET_LI(li);\n", $do_id);
	printf( TOFILE "  iword = OPC_ASM_%s(li);\n", $do_id);
    }	

    elsif( $do_full_format eq "B_1" ) {
	printf( TOFILE "  unsigned\tbo;\n");
	printf( TOFILE "  unsigned\tbi;\n");
	printf( TOFILE "  unsigned\tbd;\n");
	printf( TOFILE "\n%s\n", $assert);
	printf( TOFILE "  GET_BO(bo);\n", $do_id);
	printf( TOFILE "  GET_BI(bi);\n", $do_id);
	printf( TOFILE "  GET_BD(bd);\n", $do_id);
	printf( TOFILE "  iword = OPC_ASM_%s(bo,bi,bd);\n", $do_id);
    }

    elsif ($do_full_format eq "SC_1") {
	printf( TOFILE "\n%s\n", $assert);
	printf( TOFILE "    iword = OPC_ASM_%s();\n", $do_id);
    }

    elsif ($do_full_format eq "SC_2") {
	printf( TOFILE "  unsigned\tlev;\n");
	printf( TOFILE "\n%s\n", $assert);
	printf( TOFILE "    GET_LEV(lev);\n", $do_id);
	printf( TOFILE "    iword = OPC_ASM_%s(lev);\n", $do_id);
    }

    elsif ($do_format eq "D_form") {
	if ($do_full_format eq "D_1") {

	    printf( TOFILE "  unsigned\trt;\n");
	    printf( TOFILE "  unsigned\td;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RT(rt);\n", $do_id);
	    printf( TOFILE "    GET_D(d);\n", $do_id);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(rt,d,ra);\n", $do_id);

	}
	elsif ($do_full_format eq "D_2") {

	    printf( TOFILE "  unsigned\trt;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\tsi;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RT(rt);\n", $do_id);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_SI(si);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(rt,ra,si);\n", $do_id);

	}
	elsif ($do_full_format eq "D_3") {

	    printf( TOFILE "  unsigned\trs;\n");
	    printf( TOFILE "  unsigned\td;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RS(rs);\n", $do_id);
	    printf( TOFILE "    GET_D(d);\n", $do_id);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(rs,d,ra);\n", $do_id);

	}
	elsif ($do_full_format eq "D_4") {

	    printf( TOFILE "  unsigned\trs;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\tsi;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RS(rs);\n", $do_id);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_SI(si);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(rs,ra,si);\n", $do_id);
	}
	elsif ($do_full_format eq "D_5") {

	    printf( TOFILE "  unsigned\tbf;\n");
	    printf( TOFILE "  unsigned\tl;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\tsi;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_BF(bf);\n", $do_id);
	    printf( TOFILE "    GET_L(l);\n", $do_id);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_SI(si);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(bf,l,ra,si);\n", $do_id);
	}
	elsif ($do_full_format eq "D_6") {

	    printf( TOFILE "  unsigned\tbf;\n");
	    printf( TOFILE "  unsigned\tl;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\tui;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_BF(bf);\n", $do_id);
	    printf( TOFILE "    GET_L(l);\n", $do_id);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_UI(ui);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(bf,l,ra,ui);\n", $do_id);
	}
	elsif ($do_full_format eq "D_7") {

	    printf( TOFILE "  unsigned\tto;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\tsi;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_TO(to);\n", $do_id);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_SI(si);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(to,ra,si);\n", $do_id);
	}
	elsif ($do_full_format eq "D_8") {

	    printf( TOFILE "  unsigned\tfrt;\n");
	    printf( TOFILE "  unsigned\td;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_FRT(frt);\n", $do_id);
	    printf( TOFILE "    GET_D(d);\n", $do_id);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(frt,d,ra);\n", $do_id);

	}
	elsif ($do_full_format eq "D_9") {

	    printf( TOFILE "  unsigned\tfrs;\n");
	    printf( TOFILE "  unsigned\td;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_FRS(frs);\n", $do_id);
	    printf( TOFILE "    GET_D(d);\n", $do_id);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(frs,d,ra);\n", $do_id);

	}
    }
    elsif ($do_full_format eq "DS_1") {

	printf( TOFILE "  unsigned\trt;\n");
	printf( TOFILE "  unsigned\tds;\n");
	printf( TOFILE "  unsigned\tra;\n");
	printf( TOFILE "\n%s\n", $assert);
	printf( TOFILE "    GET_RT(rt);\n", $do_id);
	printf( TOFILE "    GET_DS(ds);\n", $do_id);
	printf( TOFILE "    GET_RA(ra);\n", $do_id);
	printf( TOFILE "    iword = OPC_ASM_%s(rt,ds,ra);\n", $do_id);
    }
    elsif ($do_full_format eq "DS_2") {

	printf( TOFILE "  unsigned\trs;\n");
	printf( TOFILE "  unsigned\tds;\n");
	printf( TOFILE "  unsigned\tra;\n");
	printf( TOFILE "\n%s\n", $assert);
	printf( TOFILE "    GET_RS(rs);\n", $do_id);
	printf( TOFILE "    GET_DS(ds);\n", $do_id);
	printf( TOFILE "    GET_RA(ra);\n", $do_id);
	printf( TOFILE "    iword = OPC_ASM_%s(rs,ds,ra);\n", $do_id);
    }
    elsif ($do_full_format eq "DQ_1") {

	printf( TOFILE "  unsigned\trt;\n");
	printf( TOFILE "  unsigned\tra;\n");
	printf( TOFILE "  unsigned\tdq;\n");
	printf( TOFILE "  unsigned\tpt;\n");
	printf( TOFILE "\n%s\n", $assert);
	printf( TOFILE "    GET_RT(rt);\n", $do_id);
	printf( TOFILE "    GET_RA(ra);\n", $do_id);
	printf( TOFILE "    GET_DQ(dq);\n", $do_id);
	printf( TOFILE "    GET_PT(pt);\n", $do_id);
	printf( TOFILE "    iword = OPC_ASM_%s(rt,ra,dq,pt);\n", $do_id);
    }
    
    elsif ($do_format eq "X_form") {
	if( $do_full_format eq "X_1" ) {
	    if ( ($do_RT eq "0") && ($do_RA eq "0") && ($do_RB eq "0") ) {

		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    iword = OPC_ASM_%s();\n", $do_id);
	    }
	    elsif ( ($do_RA eq "0") && ($do_RB eq "0") ) {

		printf( TOFILE "  unsigned\trt;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_RT(rt);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(rt);\n", $do_id);
	    }
	    elsif ( ($do_RT eq "0") && ($do_RA eq "0") ) {

		printf( TOFILE "  unsigned\trb;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_RB(rb);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(rb);\n", $do_id);
	    }
	    elsif ($do_RT eq "0") {

		printf( TOFILE "  unsigned\tra;\n");
		printf( TOFILE "  unsigned\trb;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_RA(ra);\n", $do_id);
		printf( TOFILE "    GET_RB(rb);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(ra, rb);\n", $do_id);
	    }

	    elsif ($do_RA eq "0") {

		printf( TOFILE "  unsigned\trt;\n");
		printf( TOFILE "  unsigned\trb;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_RT(rt);\n", $do_id);
		printf( TOFILE "    GET_RB(rb);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(rt, rb);\n", $do_id);
	    }
	    elsif ($do_RB eq "0") {

		printf( TOFILE "  unsigned\trt;\n");
		printf( TOFILE "  unsigned\tra;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_RT(rt);\n", $do_id);
		printf( TOFILE "    GET_RA(ra);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(rt, ra);\n", $do_id);

	    }
	    else {

		printf( TOFILE "  unsigned\trt;\n");
		printf( TOFILE "  unsigned\tra;\n");
		printf( TOFILE "  unsigned\trb;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_RT(rt);\n", $do_id);
		printf( TOFILE "    GET_RA(ra);\n", $do_id);
		printf( TOFILE "    GET_RB(rb);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(rt, ra, rb);\n", $do_id);

	    }
	}
	elsif ($do_full_format eq "X_2") {


	    printf( TOFILE "  unsigned\trt;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\tnb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RT(rt);\n", $do_id);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_NB(nb);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(rt, ra, nb);\n", $do_id);
	}
	elsif ($do_full_format eq "X_3") {
	    if ($do_store) {
		if ( ($do_RA ne "0") && ($do_RA ne "0") && ($do_RB ne "0") ) {

		    printf( TOFILE "  unsigned\trs;\n");
		    printf( TOFILE "  unsigned\tra;\n");
		    printf( TOFILE "  unsigned\trb;\n");
		    printf( TOFILE "\n%s\n", $assert);
		    printf( TOFILE "    GET_RS(rs);\n", $do_id);
		    printf( TOFILE "    GET_RA(ra);\n", $do_id);
		    printf( TOFILE "    GET_RB(rb);\n", $do_id);
		    printf( TOFILE "    iword = OPC_ASM_%s(rs, ra, rb);\n", $do_id);

		}
		else { 
		    printf("Can a store indexed (%s) have RS = %s, RA = %s and RB = %s?\n", $do_id, $do_RS, $do_RA, $do_RB);
		}
	    }
	    else {
		if ( ($do_RS eq "0") && ($do_RB eq "0") ) { 

		    printf( TOFILE "  unsigned\tra;\n");
		    printf( TOFILE "\n%s\n", $assert);
		    printf( TOFILE "    GET_RA(ra);\n" );
		    printf( TOFILE "    iword = OPC_ASM_%s(ra);\n", $do_id);

		}
		elsif ($do_RA eq "0") { 

		    printf( TOFILE "  unsigned\trs;\n");
		    printf( TOFILE "  unsigned\trb;\n");
		    printf( TOFILE "\n%s\n", $assert);
		    printf( TOFILE "    GET_RS(rs);\n" );
		    printf( TOFILE "    GET_RB(rb);\n" );
		    printf( TOFILE "    iword = OPC_ASM_%s(rs, rb);\n", $do_id);

		}
		elsif ($do_RB eq "0") { 

		    printf( TOFILE "  unsigned\trs;\n");
		    printf( TOFILE "  unsigned\tra;\n");
		    printf( TOFILE "\n%s\n", $assert);
		    printf( TOFILE "    GET_RS(rs);\n" );
		    printf( TOFILE "    GET_RA(ra);\n" );
		    printf( TOFILE "    iword = OPC_ASM_%s(rs, ra);\n", $do_id);

		}
		else {

		    printf( TOFILE "  unsigned\trs;\n");
		    printf( TOFILE "  unsigned\tra;\n");
		    printf( TOFILE "  unsigned\trb;\n");
		    printf( TOFILE "\n%s\n", $assert);
		    printf( TOFILE "    GET_RS(rs);\n" );
		    printf( TOFILE "    GET_RA(ra);\n" );
		    printf( TOFILE "    GET_RB(rb);\n" );
		    printf( TOFILE "    iword = OPC_ASM_%s(rs, ra, rb);\n", $do_id);
		}
	    }
	}
	elsif ($do_full_format eq "X_4") {

	    printf( TOFILE "  unsigned\trs;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\tnb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RS(rs);\n" );
	    printf( TOFILE "    GET_RA(ra);\n" );
	    printf( TOFILE "    GET_NB(nb);\n" );
	    printf( TOFILE "    iword = OPC_ASM_%s(rs, ra, nb);\n", $do_id);
	}
	elsif ($do_full_format eq "X_5") {

	    printf( TOFILE "  unsigned\trs;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\tsh;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RS(rs);\n" );
	    printf( TOFILE "    GET_RA(ra);\n" );
	    printf( TOFILE "    GET_SH(sh);\n" );
	    printf( TOFILE "    iword = OPC_ASM_%s(rs, ra, sh);\n", $do_id);
	}
	elsif ($do_full_format eq "X_6") {

	    printf( TOFILE "  unsigned\trs;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RS(rs);\n" );
	    printf( TOFILE "    iword = OPC_ASM_%s(rs);\n", $do_id);
	}
	elsif ($do_full_format eq "X_7") {

	    printf( TOFILE "  unsigned\tbf;\n");
	    printf( TOFILE "  unsigned\tl;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\trb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_BF(bf);\n", $do_id);
	    printf( TOFILE "    GET_L(l);\n", $do_id);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_RB(rb);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(bf,l,ra,rb);\n", $do_id);
	}
	elsif ($do_full_format eq "X_8") {

	    printf( TOFILE "  unsigned\tbf;\n");
	    printf( TOFILE "  unsigned\tfra;\n");
	    printf( TOFILE "  unsigned\tfrb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_BF(bf);\n", $do_id);
	    printf( TOFILE "    GET_FRA(fra);\n", $do_id);
	    printf( TOFILE "    GET_FRB(frb);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(bf,fra,frb);\n", $do_id);
	}
	elsif ($do_full_format eq "X_9") {
	    if ($do_BFA eq "0") {

		printf( TOFILE "  unsigned\tbf;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_BF(bf);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(bf);\n", $do_id);
	    }
	    else {

		printf( TOFILE "  unsigned\tbf;\n");
		printf( TOFILE "  unsigned\tbfa;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_BF(bf);\n", $do_id);
		printf( TOFILE "    GET_BFA(bfa);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(bf, bfa);\n", $do_id);
	    }
	}
	elsif ($do_full_format eq "X_10") {

	    printf( TOFILE "  unsigned\tbf;\n");
	    printf( TOFILE "  unsigned\tu;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_BF(bf);\n", $do_id);
	    printf( TOFILE "    GET_U(u);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(bf, u);\n", $do_id);
	}
	elsif ($do_full_format eq "X_11") {

	    printf( TOFILE "  unsigned\tto;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\trb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_TO(to);\n", $do_id);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_RB(rb);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(to,ra,rb);\n", $do_id);
	}
	elsif( $do_full_format eq "X_12" ) {

	    printf( TOFILE "  unsigned\tfrt;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\trb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_FRT(frt);\n", $do_id);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_RB(rb);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(frt,ra,rb);\n", $do_id);
	}
	elsif ($do_full_format eq "X_13") {
	    if ($do_FRB eq "0") {

		printf( TOFILE "  unsigned\tfrt;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_FRT(frt);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(frt);\n", $do_id);
	    }
	    else {

		printf( TOFILE "  unsigned\tfrt;\n");
		printf( TOFILE "  unsigned\tfrb;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_FRT(frt);\n", $do_id);
		printf( TOFILE "    GET_FRB(frb);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(frt,frb);\n", $do_id);
	    }
	}
	elsif ($do_full_format eq "X_14") {

	    printf( TOFILE "  unsigned\tfrs;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\trb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_FRS(frs);\n", $do_id);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_RB(rb);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(frs,ra,rb);\n", $do_id);
	}
	elsif ($do_full_format eq "X_15") {

	    printf( TOFILE "  unsigned\tbt;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_BT(bt);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(bt);\n", $do_id);
	}
	elsif ($do_full_format eq "X_16") {

	    printf( TOFILE "  unsigned\trs;\n");
	    printf( TOFILE "  unsigned\tsr;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RS(rs);\n", $do_id);
	    printf( TOFILE "    GET_SR(sr);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(rs,sr);\n", $do_id);
	}
	elsif ($do_full_format eq "X_17") {

	    printf( TOFILE "  unsigned\trt;\n");
	    printf( TOFILE "  unsigned\tsr;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RT(rt);\n", $do_id);
	    printf( TOFILE "    GET_SR(sr);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(rt,sr);\n", $do_id);
	}

	if( $do_full_format eq "X_18" ) {
	    printf( TOFILE "  unsigned\tt_val;\n");
	    printf( TOFILE "  unsigned\tstrm;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\trb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_Tval(t_val);\n", $do_id);
	    printf( TOFILE "    GET_STRM(strm);\n", $do_id);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_RB(rb);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(t_val, strm, ra, rb);\n", $do_id);
	}
	elsif ($do_full_format eq "X_19") {
	    if ($do_store) {
		if ( ($do_RA ne "0") && ($do_RA ne "0") && ($do_RB ne "0") ) {
		    printf( TOFILE "  unsigned\tvs;\n");
		    printf( TOFILE "  unsigned\tra;\n");
		    printf( TOFILE "  unsigned\trb;\n");
		    printf( TOFILE "\n%s\n", $assert);
		    printf( TOFILE "    GET_VS(vs);\n", $do_id);
		    printf( TOFILE "    GET_RA(ra);\n", $do_id);
		    printf( TOFILE "    GET_RB(rb);\n", $do_id);
		    printf( TOFILE "    iword = OPC_ASM_%s(vs, ra, rb);\n", $do_id);
		}
		else { 
		    printf("Can a store indexed (%s) have RS = %s, RA = %s and RB = %s?\n", $do_id, $do_RS, $do_RA, $do_RB);
		}
	    }
	    else { 
		printf("Can a X_19 instruction (%s) NOT be a store?\n", $do_id);
	    }
	}
	elsif( $do_full_format eq "X_20" ) {
	    printf( TOFILE "  unsigned\tvt;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\trb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RT(vt);\n", $do_id);
	    printf( TOFILE "    GET_VA(ra);\n", $do_id);
	    printf( TOFILE "    GET_VB(rb);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(vt, ra, rb);\n", $do_id);
	}
    }
    elsif ($do_format eq "XL_form") {
	if ($do_full_format eq "XL_1") {
	    printf( TOFILE "  unsigned\tbt;\n");
	    printf( TOFILE "  unsigned\tba;\n");
	    printf( TOFILE "  unsigned\tbb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_BT(bt);\n", $do_id);
	    printf( TOFILE "    GET_BA(ba);\n", $do_id);
	    printf( TOFILE "    GET_BB(bb);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(bt,ba,bb);\n", $do_id);
	}
	elsif ($do_full_format eq "XL_2") {
	    printf( TOFILE "  unsigned\tbo;\n");
	    printf( TOFILE "  unsigned\tbi;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_BO(bo);\n", $do_id);
	    printf( TOFILE "    GET_BI(bi);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(bo,bi);\n", $do_id);
	}
	elsif ($do_full_format eq "XL_3") {
	    printf( TOFILE "  unsigned\tbf;\n");
	    printf( TOFILE "  unsigned\tbfa;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_BF(bf);\n", $do_id);
	    printf( TOFILE "    GET_BFA(bfa);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(bf,bfa);\n", $do_id);
	}
	elsif ($do_full_format eq "XL_4") {
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    iword = OPC_ASM_%s();\n", $do_id);
	}
    }
    elsif ($do_format eq "XFX_form") {
	if ($do_full_format eq "XFX_1") {
	    printf( TOFILE "  unsigned\trt;\n");
	    printf( TOFILE "  unsigned\tspr;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RT(rt);\n", $do_id);
	    printf( TOFILE "    GET_SPR(spr);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(rt, spr);\n", $do_id);
	}
	elsif ($do_full_format eq "XFX_2") {
	    printf( TOFILE "  unsigned\trt;\n");
	    printf( TOFILE "  unsigned\ttbr;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RT(rt);\n", $do_id);
	    printf( TOFILE "    GET_TBR(tbr);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(rt, tbr);\n", $do_id);
	}
	elsif ($do_full_format eq "XFX_3") {
	    printf( TOFILE "  unsigned\tfxm;\n");
	    printf( TOFILE "  unsigned\trs;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_FXM(fxm);\n", $do_id);
	    printf( TOFILE "    GET_RS(rs);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(fxm, rs);\n", $do_id);
	}
	elsif ($do_full_format eq "XFX_4") {
	    printf( TOFILE "  unsigned\tspr;\n");
	    printf( TOFILE "  unsigned\trs;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_SPR(spr);\n", $do_id);
	    printf( TOFILE "    GET_RS(rs);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(spr, rs);\n", $do_id);
	}
	elsif ($do_full_format eq "XFX_5") {
	    printf( TOFILE "  unsigned\txo2;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_XO2(xo2);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(xo2);\n", $do_id);
	}
    }

    elsif ($do_full_format eq "XFL_1") {
	printf( TOFILE "  unsigned\tflm;\n");
	printf( TOFILE "  unsigned\tfrb;\n");
	printf( TOFILE "\n%s\n", $assert);
	printf( TOFILE "    GET_FLM(flm);\n", $do_id);
	printf( TOFILE "    GET_FRB(frb);\n", $do_id);
	printf( TOFILE "    iword = OPC_ASM_%s(flm, frb);\n", $do_id);
    }

    elsif ($do_full_format eq "XS_1") {
	printf( TOFILE "  unsigned\tra;\n");
	printf( TOFILE "  unsigned\trs;\n");
	printf( TOFILE "  unsigned\tshsh;\n");
	printf( TOFILE "\n%s\n", $assert);
	printf( TOFILE "    GET_RA(ra);\n", $do_id);
	printf( TOFILE "    GET_RS(rs);\n", $do_id);
	printf( TOFILE "    GET_SHSH(shsh);\n", $do_id);
	printf( TOFILE "    iword = OPC_ASM_%s(ra, rs, shsh);\n", $do_id);
    }

    elsif ($do_full_format eq "XO_1") {
	if( $do_RB eq "0") {
	    printf( TOFILE "  unsigned\trt;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RT(rt);\n", $do_id);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(rt, ra);\n", $do_id);
	}
	else {
	    printf( TOFILE "  unsigned\trt;\n");
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\trb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RT(rt);\n", $do_id);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_RB(rb);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(rt, ra, rb);\n", $do_id);
	}
    }
    elsif ($do_full_format eq "XO_2") {
	printf( TOFILE "  unsigned\trt;\n");
	printf( TOFILE "  unsigned\tra;\n");
	printf( TOFILE "  unsigned\trb;\n");
	printf( TOFILE "\n%s\n", $assert);
	printf( TOFILE "    GET_RT(rt);\n", $do_id);
	printf( TOFILE "    GET_RA(ra);\n", $do_id);
	printf( TOFILE "    GET_RB(rb);\n", $do_id);
	printf( TOFILE "    iword = OPC_ASM_%s(rt, ra, rb);\n", $do_id);
    }

    elsif( $do_full_format eq "A_1" ) {
	if( ( $do_FRA eq "0" ) && ( $do_FRC eq "0" ) ) {
	    printf( TOFILE "  unsigned\tfrt;\n");
	    printf( TOFILE "  unsigned\tfrb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_FRT(frt);\n", $do_id);
	    printf( TOFILE "    GET_FRB(frb);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(frt, frb);\n", $do_id);
	}
	elsif( $do_FRB eq "0") {
	    printf( TOFILE "  unsigned\tfrt;\n");
	    printf( TOFILE "  unsigned\tfra;\n");
	    printf( TOFILE "  unsigned\tfrc;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_FRT(frt);\n", $do_id);
	    printf( TOFILE "    GET_FRA(fra);\n", $do_id);
	    printf( TOFILE "    GET_FRC(frc);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(frt, fra, frc);\n", $do_id);
	}
	elsif( $do_FRC eq "0" ) {
	    printf( TOFILE "  unsigned\tfrt;\n");
	    printf( TOFILE "  unsigned\tfra;\n");
	    printf( TOFILE "  unsigned\tfrb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_FRT(frt);\n", $do_id);
	    printf( TOFILE "    GET_FRA(fra);\n", $do_id);
	    printf( TOFILE "    GET_FRB(frb);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(frt, fra, frb);\n", $do_id);
	}
	else {
	    printf( TOFILE "  unsigned\tfrt;\n");
	    printf( TOFILE "  unsigned\tfra;\n");
	    printf( TOFILE "  unsigned\tfrc;\n");
	    printf( TOFILE "  unsigned\tfrb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_FRT(frt);\n", $do_id);
	    printf( TOFILE "    GET_FRA(fra);\n", $do_id);
	    printf( TOFILE "    GET_FRC(frc);\n", $do_id);
	    printf( TOFILE "    GET_FRB(frb);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(frt, fra, frc, frb);\n", $do_id);
	}
    }

    elsif( $do_full_format eq "M_1" ) {
	printf( TOFILE "  unsigned\tra;\n");
	printf( TOFILE "  unsigned\trs;\n");
	printf( TOFILE "  unsigned\trb;\n");
	printf( TOFILE "  unsigned\tmb;\n");
	printf( TOFILE "  unsigned\tme;\n");
	printf( TOFILE "\n%s\n", $assert);
	printf( TOFILE "    GET_RA(ra);\n", $do_id);
	printf( TOFILE "    GET_RS(rs);\n", $do_id);
	printf( TOFILE "    GET_RB(rb);\n", $do_id);
	printf( TOFILE "    GET_MB(mb);\n", $do_id);
	printf( TOFILE "    GET_ME(me);\n", $do_id);
	printf( TOFILE "    iword = OPC_ASM_%s(ra,rs,rb,mb,me);\n", $do_id);
    }
    elsif( $do_full_format eq "M_2" ) {
	printf( TOFILE "  unsigned\tra;\n");
	printf( TOFILE "  unsigned\trs;\n");
	printf( TOFILE "  unsigned\tsh;\n");
	printf( TOFILE "  unsigned\tmb;\n");
	printf( TOFILE "  unsigned\tme;\n");
	printf( TOFILE "\n%s\n", $assert);
	printf( TOFILE "    GET_RA(ra);\n", $do_id);
	printf( TOFILE "    GET_RS(rs);\n", $do_id);
	printf( TOFILE "    GET_SH(sh);\n", $do_id);
	printf( TOFILE "    GET_MB(mb);\n", $do_id);
	printf( TOFILE "    GET_ME(me);\n", $do_id);
	printf( TOFILE "    iword = OPC_ASM_%s(ra,rs,sh,mb,me);\n", $do_id);
    }

    elsif( $do_full_format eq "MD_1" ) {
	printf( TOFILE "  unsigned\tra;\n");
	printf( TOFILE "  unsigned\trs;\n");
	printf( TOFILE "  unsigned\tshsh;\n");
	printf( TOFILE "  unsigned\tmbw;\n");
	printf( TOFILE "\n%s\n", $assert);
	printf( TOFILE "    GET_RA(ra);\n", $do_id);
	printf( TOFILE "    GET_RS(rs);\n", $do_id);
	printf( TOFILE "    GET_SHSH(shsh);\n", $do_id);
	printf( TOFILE "    GET_MBW(mbw);\n", $do_id);
	printf( TOFILE "    iword = OPC_ASM_%s(ra,rs,shsh,mbw);\n", $do_id);
    }
    elsif( $do_full_format eq "MD_2" ) {
	printf( TOFILE "  unsigned\tra;\n");
	printf( TOFILE "  unsigned\trs;\n");
	printf( TOFILE "  unsigned\tshsh;\n");
	printf( TOFILE "  unsigned\tmew;\n");
	printf( TOFILE "\n%s\n", $assert);
	printf( TOFILE "    GET_RA(ra);\n", $do_id);
	printf( TOFILE "    GET_RS(rs);\n", $do_id);
	printf( TOFILE "    GET_SHSH(shsh);\n", $do_id);
	printf( TOFILE "    GET_MEW(mew);\n", $do_id);
	printf( TOFILE "    iword = OPC_ASM_%s(ra,rs,shsh,mew);\n", $do_id);
    }
    
    elsif( $do_format eq "MDS_form" ) {
	if( $do_full_format eq "MDS_1" ) {
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\trs;\n");
	    printf( TOFILE "  unsigned\trb;\n");
	    printf( TOFILE "  unsigned\tmbw;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_RS(rs);\n", $do_id);
	    printf( TOFILE "    GET_RB(rb);\n", $do_id);
	    printf( TOFILE "    GET_MBW(mbw);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(ra,rs,rb,mbw);\n", $do_id);
	}
	elsif( $do_full_format eq "MDS_2" ) {
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\trs;\n");
	    printf( TOFILE "  unsigned\trb;\n");
	    printf( TOFILE "  unsigned\tmew;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_RS(rs);\n", $do_id);
	    printf( TOFILE "    GET_RB(rb);\n", $do_id);
	    printf( TOFILE "    GET_MEW(mew);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(ra,rs,rb,mew);\n", $do_id);
	}
	elsif( $do_full_format eq "MDS_3" ) {
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\tis;\n");
	    printf( TOFILE "  unsigned\tib;\n");
	    printf( TOFILE "  unsigned\txbi;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_IS(is);\n", $do_id);
	    printf( TOFILE "    GET_IB(ib);\n", $do_id);
	    printf( TOFILE "    GET_XBI(xbi);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(ra,is,ib,xbi);\n", $do_id);
	}
	elsif( $do_full_format eq "MDS_3n" ) {
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\tis;\n");
	    printf( TOFILE "  unsigned\tib;\n");
	    printf( TOFILE "  unsigned\tcbi;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_IS(is);\n", $do_id);
	    printf( TOFILE "    GET_IB(ib);\n", $do_id);
	    printf( TOFILE "    GET_CBI(cbi);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(ra,is,ib,cbi);\n", $do_id);
	}
	elsif( $do_full_format eq "MDS_4" ) {
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\tis;\n");
	    printf( TOFILE "  unsigned\trb;\n");
	    printf( TOFILE "  unsigned\txbi;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_IS(is);\n", $do_id);
	    printf( TOFILE "    GET_RB(rb);\n", $do_id);
	    printf( TOFILE "    GET_XBI(xbi);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(ra,is,rb,xbi);\n", $do_id);
	}
	elsif( $do_full_format eq "MDS_4n" ) {
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\tis;\n");
	    printf( TOFILE "  unsigned\trb;\n");
	    printf( TOFILE "  unsigned\tcbi;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_IS(is);\n", $do_id);
	    printf( TOFILE "    GET_RB(rb);\n", $do_id);
	    printf( TOFILE "    GET_CBI(cbi);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(ra,is,rb,cbi);\n", $do_id);
	}
	elsif( $do_full_format eq "MDS_5" ) {
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\trs;\n");
	    printf( TOFILE "  unsigned\tib;\n");
	    printf( TOFILE "  unsigned\txbi;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_RS(rs);\n", $do_id);
	    printf( TOFILE "    GET_IB(ib);\n", $do_id);
	    printf( TOFILE "    GET_XBI(xbi);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(ra,rs,ib,xbi);\n", $do_id);
	}
	elsif( $do_full_format eq "MDS_5n" ) {
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\trs;\n");
	    printf( TOFILE "  unsigned\tib;\n");
	    printf( TOFILE "  unsigned\tcbi;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_RS(rs);\n", $do_id);
	    printf( TOFILE "    GET_IB(ib);\n", $do_id);
	    printf( TOFILE "    GET_CBI(cbi);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(ra,rs,ib,cbi);\n", $do_id);
	}
	elsif( $do_full_format eq "MDS_6" ) {
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\trs;\n");
	    printf( TOFILE "  unsigned\trb;\n");
	    printf( TOFILE "  unsigned\txbi;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_RS(rs);\n", $do_id);
	    printf( TOFILE "    GET_RB(rb);\n", $do_id);
	    printf( TOFILE "    GET_XBI(xbi);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(ra,rs,rb,xbi);\n", $do_id);
	}
	elsif( $do_full_format eq "MDS_6n" ) {
	    printf( TOFILE "  unsigned\tra;\n");
	    printf( TOFILE "  unsigned\trs;\n");
	    printf( TOFILE "  unsigned\trb;\n");
	    printf( TOFILE "  unsigned\tcbi;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_RA(ra);\n", $do_id);
	    printf( TOFILE "    GET_RS(rs);\n", $do_id);
	    printf( TOFILE "    GET_RB(rb);\n", $do_id);
	    printf( TOFILE "    GET_CBI(cbi);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(ra,rs,rb,cbi);\n", $do_id);
	}
    }
    
    elsif( $do_full_format eq "TX_1" ) {
	printf( TOFILE "  unsigned\tto;\n");
	printf( TOFILE "  unsigned\tuil;\n");
	printf( TOFILE "  unsigned\txbi;\n");
	printf( TOFILE "\n%s\n", $assert);
	printf( TOFILE "    GET_TO(to);\n", $do_id);
	printf( TOFILE "    GET_UIL(uil);\n", $do_id);
	printf( TOFILE "    GET_XBI(xbi);\n", $do_id);
	printf( TOFILE "    iword = OPC_ASM_%s(to,uil,xbi);\n", $do_id);
    }

    elsif( $do_full_format eq "VA_1" ) {
	if( ( $do_VA eq "0" ) && ( $do_VC eq "0" ) ) {
	    printf( TOFILE "  unsigned\tvt;\n");
	    printf( TOFILE "  unsigned\tvb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_VT(vt);\n", $do_id);
	    printf( TOFILE "    GET_VB(vb);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(vt, vb);\n", $do_id);
	}
	elsif( $do_VB eq "0") {
	    printf( TOFILE "  unsigned\tvt;\n");
	    printf( TOFILE "  unsigned\tva;\n");
	    printf( TOFILE "  unsigned\tvc;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_VT(vt);\n", $do_id);
	    printf( TOFILE "    GET_VA(va);\n", $do_id);
	    printf( TOFILE "    GET_VC(vc);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(vt, va, vc);\n", $do_id);
	}
	elsif( $do_VC eq "0" ) {
	    printf( TOFILE "  unsigned\tvt;\n");
	    printf( TOFILE "  unsigned\tva;\n");
	    printf( TOFILE "  unsigned\tvb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_VT(vt);\n", $do_id);
	    printf( TOFILE "    GET_VA(va);\n", $do_id);
	    printf( TOFILE "    GET_VB(vb);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(vt, va, vb);\n", $do_id);
	}
	else {
	    printf( TOFILE "  unsigned\tvt;\n");
	    printf( TOFILE "  unsigned\tva;\n");
	    printf( TOFILE "  unsigned\tvc;\n");
	    printf( TOFILE "  unsigned\tvb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_VT(vt);\n", $do_id);
	    printf( TOFILE "    GET_VA(va);\n", $do_id);
	    printf( TOFILE "    GET_VC(vc);\n", $do_id);
	    printf( TOFILE "    GET_VB(vb);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(vt, va, vc, vb);\n", $do_id);
	}
    }
    elsif( $do_full_format eq "VA_2" ) {
	printf( TOFILE "  unsigned\tvt;\n");
	printf( TOFILE "  unsigned\tva;\n");
	printf( TOFILE "  unsigned\tvc;\n");
	printf( TOFILE "  unsigned\tsh;\n");
	printf( TOFILE "\n%s\n", $assert);
	printf( TOFILE "    GET_VT(vt);\n", $do_id);
	printf( TOFILE "    GET_VA(va);\n", $do_id);
	printf( TOFILE "    GET_VC(vc);\n", $do_id);
	printf( TOFILE "    GET_SHB(sh);\n", $do_id);
	printf( TOFILE "    iword = OPC_ASM_%s(vt, va, vc, sh);\n", $do_id);
    }

    elsif ($do_format eq "VX_form") {
	if( $do_full_format eq "VX_1" ) {
	    if ( ($do_VA eq "0") && ($do_VB eq "0") ) {
		printf( TOFILE "  unsigned\tvt;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_VT(vt);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(vt);\n", $do_id);
	    }
	    elsif ( ($do_VT eq "0") && ($do_VA eq "0") ) {
		printf( TOFILE "  unsigned\tvb;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_VB(vb);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(vb);\n", $do_id);
	    }
	    elsif ($do_VT eq "0") {
		printf( TOFILE "  unsigned\tva;\n");
		printf( TOFILE "  unsigned\tvb;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_VA(va);\n", $do_id);
		printf( TOFILE "    GET_VB(vb);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(va, vb);\n", $do_id);
	    }
	    elsif ($do_VA eq "0") {
		printf( TOFILE "  unsigned\tvt;\n");
		printf( TOFILE "  unsigned\tvb;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_VT(vt);\n", $do_id);
		printf( TOFILE "    GET_VB(vb);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(vt, vb);\n", $do_id);
	    }
	    elsif ($do_VB eq "0") {
		printf( TOFILE "  unsigned\tvt;\n");
		printf( TOFILE "  unsigned\tva;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_VT(vt);\n", $do_id);
		printf( TOFILE "    GET_VA(va);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(vt, va);\n", $do_id);
	    }
	    else {
		printf( TOFILE "  unsigned\tvt;\n");
		printf( TOFILE "  unsigned\tva;\n");
		printf( TOFILE "  unsigned\tvb;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_VT(vt);\n", $do_id);
		printf( TOFILE "    GET_VA(va);\n", $do_id);
		printf( TOFILE "    GET_VB(vb);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(vt, va, vb);\n", $do_id);
	    }
	}
	elsif( $do_full_format eq "VX_2" ) {
	    if ($do_VB eq "0") {
		printf( TOFILE "  unsigned\tvt;\n");
		printf( TOFILE "  unsigned\tvui;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_VT(vt);\n", $do_id);
		printf( TOFILE "    GET_UIMM(vui);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(vt, vui);\n", $do_id);
	    }
	    else {
		printf( TOFILE "  unsigned\tvt;\n");
		printf( TOFILE "  int     \tvui;\n");
		printf( TOFILE "  unsigned\tvb;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_VT(vt);\n", $do_id);
		printf( TOFILE "    GET_UIMM(vui);\n", $do_id);
		printf( TOFILE "    GET_VB(vb);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(vt, vui, vb);\n", $do_id);
	    }
	}
	elsif( $do_full_format eq "VX_3" ) {
	    if ($do_VB eq "0") {
		printf( TOFILE "  unsigned\tvt;\n");
		printf( TOFILE "  unsigned\tvsi;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_VT(vt);\n", $do_id);
		printf( TOFILE "    GET_SIMM(vsi);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(vt, vsi);\n", $do_id);
	    }
	    else {
		printf( TOFILE "  unsigned\tvt;\n");
		printf( TOFILE "  int     \tvsi;\n");
		printf( TOFILE "  unsigned\tvb;\n");
		printf( TOFILE "\n%s\n", $assert);
		printf( TOFILE "    GET_VT(vt);\n", $do_id);
		printf( TOFILE "    GET_SIMM(vsi);\n", $do_id);
		printf( TOFILE "    GET_VB(vb);\n", $do_id);
		printf( TOFILE "    iword = OPC_ASM_%s(vt, vsi, vb);\n", $do_id);
	    }
	}
    }

    elsif ($do_format eq "VXR_form") {
	if( $do_full_format eq "VXR_1" ) {
	    printf( TOFILE "  unsigned\tvt;\n");
	    printf( TOFILE "  unsigned\tva;\n");
	    printf( TOFILE "  unsigned\tvb;\n");
	    printf( TOFILE "\n%s\n", $assert);
	    printf( TOFILE "    GET_VT(vt);\n", $do_id);
	    printf( TOFILE "    GET_VA(va);\n", $do_id);
	    printf( TOFILE "    GET_VB(vb);\n", $do_id);
	    printf( TOFILE "    iword = OPC_ASM_%s(vt, va, vb);\n", $do_id);	    
	}
    }

    else {
	print STDERR "Opcode_as.pl: Unknown format ", $do_format, " : ", $do_full_format, " for ", $do_id, " in process_asm\n";
    }

    printf( TOFILE "\n  USED_ALL();\n" );
    printf( TOFILE "  *p_iword = iword;\n");
    printf( TOFILE "  return 0;\n" );
    printf( TOFILE "error_found:;\n");
    printf( TOFILE "  DBG( fprintf(stderr, \"error %%s %%d: error %%d at line %%d\\n\", __FILE__, __LINE__, m_error, m_errline) );\n");
    printf( TOFILE "  return m_error;\n");
    printf( TOFILE "}\n" );

    print TOFILE "\n";
}

sub DoFinalGenerateAssembly {
    local (*TOFILE) = $_[0];

    printf( TOFILE "int\n");
    printf( TOFILE "opc_as_index(\n", $do_id);
    printf( TOFILE "\topc_idx\tidx,\n");
    printf( TOFILE "\tchar *\topcode,\n");
    printf( TOFILE "\tint\tnargs,\n");
    printf( TOFILE "\tchar *\targs[],\n");
    printf( TOFILE "\tunsigned *\tp_iword\n");
    printf( TOFILE "\t)\n");
    printf( TOFILE "{\n");
    printf( TOFILE "\n");
    printf( TOFILE "  switch(idx) {\n");
    for( $i = 1; $i <= $G_num; $i++ ) {
      printf( TOFILE "  case OPC_%s:\n", $G_ids_by_index[$i] );
      printf( TOFILE "    return opc_as_OPC_%s(opcode,nargs,args,p_iword);\n", $G_ids_by_index[$i]);
    }
    printf( TOFILE "  }\n" );
    printf( TOFILE "  return -100;\n");
    printf( TOFILE "}\n" );
    #close(TOFILE);
}

