#
#    Value                           1 1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 2 2 2 2 3  3       
#                0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0  1              
#     I_1:     |   opcode  |                       LI                      |AA|LK|
#
#     B_1:     |   opcode  |    BO   |    BI   |            BD             |AA|LK|
#
#    SC_1:     |   opcode  |   ///   |   ///   |           ///             |XO|LK|
#    SC_2:     |   opcode  |   ///   |   ///   |  ///  |   LEV       | /// |XO|LK|
#
#     D_1:     |   opcode  |    RT   |    RA   |                D                |
#     D_2:     |   opcode  |    RT   |    RA   |               SI                |
#     D_3:     |   opcode  |    RS   |    RA   |                D                |
#     D_4:     |   opcode  |    RS   |    RA   |               SI                |
#     D_5:     |   opcode  |  BF |/|L|    RA   |               SI                |
#     D_6:     |   opcode  |  BF |/|L|    RA   |               UI                |
#     D_7:     |   opcode  |    TO   |    RA   |               SI                |
#     D_8:     |   opcode  |   FRT   |    RA   |                D                |
#     D_9:     |   opcode  |   FRS   |    RA   |                D                |
#
#    DS_1:     |   opcode  |    RT   |    RA   |               DS          | XO  |
#    DS_2:     |   opcode  |    RS   |    RA   |               DS          | XO  |
#
#    DQ_1:     |   opcode  |    RT   |    RA   |          DQ           |    PT  |
#
#    X_1:      |   opcode  |    RT   |    RA   |    RB   |         XO         |//|
#    X_2:      |   opcode  |    RT   |    RA   |    NB   |         XO         |//|
#    X_3:      |   opcode  |    RS   |    RA   |    RB   |         XO         |Rc|
#    X_4:      |   opcode  |    RS   |    RA   |    NB   |         XO         |//|
#    X_5:      |   opcode  |    RS   |    RA   |    SH   |         XO         |Rc|
#    X_6:      |   opcode  |    RS   |   ///   |   ///   |         XO         |//|
#    X_7:      |   opcode  |  BF |/|L|    RA   |    RB   |         XO         |//|
#    X_8:      |   opcode  |  BF |///|   FRA   |   FRB   |         XO         |//|
#    X_9:      |   opcode  |  BF |///| BFA |///|   ///   |         XO         |//|
#    X_10:     |   opcode  |  BF |///|   ///   |    U  |/|         XO         |Rc|
#    X_11:     |   opcode  |    TO   |    RA   |    RB   |         XO         |//|
#    X_12:     |   opcode  |   FRT   |    RA   |    RB   |         XO         |//|
#    X_13:     |   opcode  |   FRT   |   ///   |   FRB   |         XO         |Rc|
#    X_14:     |   opcode  |   FRS   |    RA   |    RB   |         XO         |//|
#    X_15:     |   opcode  |    BT   |   ///   |   ///   |         XO         |Rc|
#    X_16:     |   opcode  |    RS   |/|   SR  |   ///   |         XO         |//|
#    X_17:     |   opcode  |    RT   |/|   SR  |   ///   |         XO         |//|
#    X_18:     |   opcode  |T|//|STRM|    RA   |    RB   |         XO         |//|
#    X_19:     |   opcode  |    VS   |    RA   |    RB   |         XO         |Rc|
#    X_20:     |   opcode  |    VT   |    RA   |    RB   |           XO        |0|
#
#    XL_1:     |   opcode  |    BT   |    BA   |    BB   |         XO         |//|
#    XL_2:     |   opcode  |    BO   |    RI   |   ///   |         XO         |LK|
#    XL_3:     |   opcode  |  BF |///| BFA |///|   ///   |         XO         |//|
#    XL_4:     |   opcode  |   ///   |   ///   |   ///   |         XO         |//|
#
#   XFX_1:     |   opcode  |    RT   |        SPR        |         XO         |//|
#   XFX_2:     |   opcode  |    RT   |        TBR        |         XO         |//|
#   XFX_3:     |   opcode  |    RS   |/|      FXM      |/|         XO         |//|
#   XFX_4:     |   opcode  |    RS   |        SPR        |         XO         |//|
#   XFX_5:     |   opcode  |   ///   |        XO2        |         XO         |//|
#
#   XFL_1:     |   opcode  |/|      FLM      |/|   FRB   |         XO         |Rc|
#
#    XS_1:     |   opcode  |    RS   |    RA   |    SH   |         XO      |SH|Rc|
#
#    XO_1:     |   opcode  |    RT   |    RA   |    RB   |OE|      XO         |Rc|
#    XO_2:     |   opcode  |    RT   |    RA   |    RB   |//|      XO         |Rc|
#
#     A_1:     |   opcode  |   FRT   |   FRA   |   FRB   |   FRC   |    XO    |Rc| 
#
#     M_1:     |   opcode  |    RS   |    RA   |    RB   |    MB   |    ME    |Rc|
#     M_2:     |   opcode  |    RS   |    RA   |    SH   |    MB   |    ME    |Rc|
#
#    MD_1:     |   opcode  |    RS   |    RA   |    SH   |    MB   |   XO  |SH|Rc|
#    MD_2:     |   opcode  |    RS   |    RA   |    SH   |    ME   |   XO  |SH|Rc|
#
#   MDS_1:     |   opcode  |    RS   |    RA   |    RB   |    MB   |     XO   |Rc|
#   MDS_2:     |   opcode  |    RS   |    RA   |    RB   |    ME   |     XO   |Rc|
#   MDS_3:     |   opcode  |    IS   |    RA   |    IB   | XBI  |//|     XO   |Rc|
#   MDS_4:     |   opcode  |    IS   |    RA   |    RB   | XBI  |//|     XO   |Rc|
#   MDS_5:     |   opcode  |    RS   |    RA   |    IB   | XBI  |//|     XO   |Rc|
#   MDS_6:     |   opcode  |    RS   |    RA   |    RB   | XBI  |//|     XO   |Rc|
#   MDS_3n:    |   opcode  |    IS   |    RA   |    IB   |  CBI  |/|     XO   |Rc|
#   MDS_4n:    |   opcode  |    IS   |    RA   |    RB   |  CBI  |/|     XO   |Rc|
#   MDS_5n:    |   opcode  |    RS   |    RA   |    IB   |  CBI  |/|     XO   |Rc|
#   MDS_6n:    |   opcode  |    RS   |    RA   |    RB   |  CBI  |/|     XO   |Rc|
#
#    TX_1:     |   opcode  |    TO   |        UI         |  XBI |       XO    |Rc|
#
#    VA_1:     |   opcode  |    VT   |    VA   |    VB   |    VC   |     XO      |
#    VA_2:     |   opcode  |    VT   |    VA   |    VB   |/|   SH  |     XO      |
#
#    VX_1:     |   opcode  |    VT   |    VA   |    VB   |           XO          |
#    VX_2:     |   opcode  |    VT   |   UIMM  |    VB   |           XO          |
#    VX_3:     |   opcode  |    VT   |   SIMM  |    VB   |           XO          |
#
#    VXR_1:    |   opcode  |    VT   |    VA   |    VB   |Rc|        XO          |


sub DoInitialGenerateAsmMacros {
    local (*TOFILE) = $_[0];
}

#* Generate OPC_ASM assembly-code macros *#
sub DoGenerateAsmMacrosForThisInstruction {
    local (*TOFILE) = $_[0];

    if( $do_full_format eq "I_1" ) {
	print TOFILE "#define OPC_ASM_", $do_id, "(_li) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_AA, "<<1) | ", $do_LK;
	print TOFILE " | OPC_ENC_LI(_li))\n";
    }	
    elsif( $do_full_format eq "B_1" ) {
	print TOFILE "#define OPC_ASM_", $do_id, "(_bo, _bi, _bd) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_AA, "<<1) | ", $do_LK;
	print TOFILE " | OPC_ENC_BO(_bo) | OPC_ENC_BI(_bi) | OPC_ENC_BD(_bd))\n";
    }
    elsif ($do_full_format eq "SC_1") {
	print TOFILE "#define OPC_ASM_", $do_id, "() \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_LK, " )\n";
    }
    elsif ($do_full_format eq "SC_2") {
	print TOFILE "#define OPC_ASM_", $do_id, "(_lev) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_LK, " | OPC_ENC_LEV(_lev) )\n";
    }

    elsif ($do_format eq "D_form") {
	if ($do_full_format eq "D_1") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_rt, _d, _ra) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_RT(_rt) | OPC_ENC_D(_d) | OPC_ENC_RA(_ra))\n";
	}
	elsif ($do_full_format eq "D_2") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_rt, _ra, _si) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_RT(_rt) | OPC_ENC_RA(_ra) | OPC_ENC_SI(_si) )\n";      
	}
	elsif ($do_full_format eq "D_3") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_rs, _d, _ra) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_D(_d) | OPC_ENC_RA(_ra))\n";
	}
	elsif ($do_full_format eq "D_4") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _rs, _si) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_RA(_ra) | OPC_ENC_RS(_rs) | OPC_ENC_SI(_si) )\n";
	}
	elsif ($do_full_format eq "D_5") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_bf, _l, _ra, _si) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_BF(_bf) | OPC_ENC_L(_l) | OPC_ENC_RA(_ra) | OPC_ENC_SI(_si) )\n";
	}
	elsif ($do_full_format eq "D_6") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_bf, _l, _ra, _ui) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_BF(_bf) | OPC_ENC_L(_l) | OPC_ENC_RA(_ra) | OPC_ENC_UI(_ui) )\n";
	}
	elsif ($do_full_format eq "D_7") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_to, _ra, _si) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_TO(_to) | OPC_ENC_RA(_ra) | OPC_ENC_SI(_si) )\n";      
	}
	elsif ($do_full_format eq "D_8") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_frt, _d, _ra) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_FRT(_frt) | OPC_ENC_D(_d) | OPC_ENC_RA(_ra))\n";
	}
	elsif ($do_full_format eq "D_9") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_frs, _d, _ra) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_FRS(_frs) | OPC_ENC_D(_d) | OPC_ENC_RA(_ra))\n";
	}
    }

    elsif ($do_full_format eq "DS_1") {
	print TOFILE "#define OPC_ASM_", $do_id, "(_rt, _ds, _ra) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
	print TOFILE " | OPC_ENC_RT(_rt) | OPC_ENC_RA(_ra) | OPC_ENC_DS(_ds) )\n";
	print TOFILE "#define OPC_ASM_", $do_id, "b(_rt, _ds, _ra) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
	print TOFILE " | OPC_ENC_RT(_rt) | OPC_ENC_RA(_ra) | OPC_ENC_DSb(_ds) )\n";
    }
    elsif ($do_full_format eq "DS_2") {
	print TOFILE "#define OPC_ASM_", $do_id, "(_rs, _ds, _ra) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
	print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) | OPC_ENC_DS(_ds) )\n";
	print TOFILE "#define OPC_ASM_", $do_id, "b(_rs, _ds, _ra) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
	print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) | OPC_ENC_DSb(_ds) )\n";
    }
    elsif ($do_full_format eq "DQ_1") {
	print TOFILE "#define OPC_ASM_", $do_id, "(_rt, _ra, _dq, _pt) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26)";
	print TOFILE " | OPC_ENC_RT(_rt) | OPC_ENC_RA(_ra) | OPC_ENC_DQ(_dq) | OPC_ENC_PT(_pt) )\n";
	print TOFILE "#define OPC_ASM_", $do_id, "b(_rt, _ra, _dq, _pt) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26)";
	print TOFILE " | OPC_ENC_RT(_rt) | OPC_ENC_RA(_ra) | OPC_ENC_DQb(_dq) | OPC_ENC_PT(_pt) )\n";
    }
    
    elsif ($do_format eq "X_form") {
	if( $do_full_format eq "X_1" ) {
	    if ( ($do_RT eq "0") && ($do_RA eq "0") && ($do_RB eq "0") ) {
		print TOFILE "#define OPC_ASM_", $do_id, "() \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " )\n";	
	    }
	    elsif ( ($do_RA eq "0") && ($do_RB eq "0") ) {
		print TOFILE "#define OPC_ASM_", $do_id, "(_rt) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_RT(_rt) )\n";
	    }
	    elsif ( ($do_RT eq "0") && ($do_RA eq "0") ) {
		print TOFILE "#define OPC_ASM_", $do_id, "(_rb) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_RB(_rb) )\n";
	    }
	    elsif ($do_RT eq "0") {
		print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _rb) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb))\n";
	    }
	    elsif ($do_RA eq "0") {
		print TOFILE "#define OPC_ASM_", $do_id, "(_rt, _rb) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_RT(_rt) | OPC_ENC_RB(_rb) )\n";
	    }
	    elsif ($do_RB eq "0") {
		print TOFILE "#define OPC_ASM_", $do_id, "(_rt, _ra) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_RT(_rt) | OPC_ENC_RA(_ra) )\n";
	    }
	    else {
		print TOFILE "#define OPC_ASM_", $do_id, "(_rt, _ra, _rb) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_RT(_rt) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb))\n";
	    }
	}
	elsif ($do_full_format eq "X_2") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_rt, _ra, _nb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RT(_rt) | OPC_ENC_RA(_ra) | OPC_ENC_NB(_nb))\n";	
	}
	elsif ($do_full_format eq "X_3") {
	    if ($do_store) {
		if ( ($do_RA ne "0") && ($do_RA ne "0") && ($do_RB ne "0") ) {
		    print TOFILE "#define OPC_ASM_", $do_id, "(_rs, _ra, _rb) \\\n";
		    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		    print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb))\n";	
		}
		else { 
		    printf("Can a store indexed (%s) have RS = %s, RA = %s and RB = %s?\n", $do_id, $do_RS, $do_RA, $do_RB);
		}
	    }
	    else {
		if ( ($do_RS eq "0") && ($do_RB eq "0") ) { 
		    print TOFILE "#define OPC_ASM_", $do_id, "(_ra) \\\n";
		    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		    print TOFILE " | OPC_ENC_RA(_ra) )\n";	
		}
		elsif ($do_RA eq "0") { 
		    print TOFILE "#define OPC_ASM_", $do_id, "(_rs, _rb) \\\n";
		    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		    print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RB(_rb) )\n";	
		}
		elsif ($do_RB eq "0") { 
		    print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _rs) \\\n";
		    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		    print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) )\n";	
		}
		else {
		    print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _rs, _rb) \\\n";
		    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		    print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb))\n";	
		}
	    }
	}
	elsif ($do_full_format eq "X_4") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_rs, _ra, _nb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) | OPC_ENC_NB(_nb))\n";	
	}
	elsif ($do_full_format eq "X_5") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _rs, _sh) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) | OPC_ENC_SH(_sh))\n";	
	}
	elsif ($do_full_format eq "X_6") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_rs) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RS(_rs) )\n";	
	}
	elsif ($do_full_format eq "X_7") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_bf, _l, _ra, _rb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_BF(_bf) | OPC_ENC_L(_l) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb))\n";	
	}
	elsif ($do_full_format eq "X_8") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_bf, _fra, _frb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_BF(_bf) | OPC_ENC_FRA(_fra) | OPC_ENC_FRB(_frb))\n";	
	}
	elsif ($do_full_format eq "X_9") {
	    if ($do_BFA eq "0") {
		print TOFILE "#define OPC_ASM_", $do_id, "(_bf) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_BF(_bf) )\n";	
	    }
	    else {
		print TOFILE "#define OPC_ASM_", $do_id, "(_bf, _bfa) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_BF(_bf) | OPC_ENC_BFA(_bfa) )\n";	
	    }
	}
	elsif ($do_full_format eq "X_10") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_bf, _u) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_BF(_bf) | OPC_ENC_U(_u))\n";	
	}
	elsif ($do_full_format eq "X_11") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_to, _ra, _nb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_TO(_to) | OPC_ENC_RA(_ra) | OPC_ENC_NB(_nb))\n";	
	}
	elsif( $do_full_format eq "X_12" ) {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_frt, _ra, _rb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_FRT(_frt) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb))\n";	
	}
	elsif ($do_full_format eq "X_13") {
	    if ($do_FRB eq "0") {
		print TOFILE "#define OPC_ASM_", $do_id, "(_frt) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_FRT(_frt) )\n";
	    }
	    else {
		print TOFILE "#define OPC_ASM_", $do_id, "(_frt, _frb) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_FRT(_frt) | OPC_ENC_FRB(_frb))\n";	
	    }
	}
	elsif ($do_full_format eq "X_14") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_frs, _ra, _rb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_FRS(_frs) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb))\n";	
	}
	elsif ($do_full_format eq "X_15") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_bt) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_BT(_bt) )\n";	
	}
	elsif ($do_full_format eq "X_16") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_rs, _sr) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_SR(_sr) )\n";	
	}
	elsif ($do_full_format eq "X_17") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_rt, _sr) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RT(_rt) | OPC_ENC_SR(_sr) )\n";	
	}
	elsif ($do_full_format eq "X_18") {
	    if ( ($do_RA eq "0") && ($do_RB eq "0") ) {
		print TOFILE "#define OPC_ASM_", $do_id, "(_t, _strm) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_Tval(_t) | OPC_ENC_STRM(_strm) )\n";
	    }
	    else {
		print TOFILE "#define OPC_ASM_", $do_id, "(_t, _strm, _ra, _rb) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_Tval(_t) | OPC_ENC_STRM(_strm) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb))\n";
	    }
	}
	elsif ($do_full_format eq "X_19") {
	    if ($do_store) {
		if ( ($do_RS ne "0") && ($do_RA ne "0") && ($do_RB ne "0") ) {
		    print TOFILE "#define OPC_ASM_", $do_id, "(_vs, _ra, _rb) \\\n";
		    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		    print TOFILE " | OPC_ENC_VS(_vs) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb))\n";	
		}
		else { 
		    printf("Can a store indexed (%s) have RS = %s, RA = %s and RB = %s?\n", $do_id, $do_RS, $do_RA, $do_RB);
		}
	    }
	    else {
		print "Can't handle instr $do_id, format $do_format : $do_full_format NOT a store!\n";
	    }
	}
	elsif( $do_full_format eq "X_20" ) {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_vt, _ra, _rb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_VT(_vt) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb))\n";
	}
    }
    elsif ($do_format eq "XL_form") {
	if ($do_full_format eq "XL_1") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_bt, _ba, _bb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) ";
	    print TOFILE " | OPC_ENC_BT(_bt) | OPC_ENC_BA(_ba) | OPC_ENC_BB(_bb) )\n";      
	}
	elsif ($do_full_format eq "XL_2") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_bo, _bi) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_LK;
	    print TOFILE " | OPC_ENC_BO(_bo) | OPC_ENC_BI(_bi) )\n";      
	}
	elsif ($do_full_format eq "XL_3") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_bf, _bfa) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) ";
	    print TOFILE " | OPC_ENC_BF(_bf) | OPC_ENC_BFA(_bfa) )\n";      
	}
	elsif ($do_full_format eq "XL_4") {
	    print TOFILE "#define OPC_ASM_", $do_id, "() \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_LK, " )\n";
	}
    }
    elsif ($do_format eq "XFX_form") {
	if ($do_full_format eq "XFX_1") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_rt, _spr) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1)", $do_Rc;
	    print TOFILE " | OPC_ENC_RT(_rt) | OPC_ENC_SPR(_spr) )\n";
	}
	elsif ($do_full_format eq "XFX_2") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_rt, _tbr) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1)", $do_Rc;
	    print TOFILE " | OPC_ENC_RT(_rt) | OPC_ENC_TBR(_tbr) )\n";
	}
	elsif ($do_full_format eq "XFX_3") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_fxm, _rs) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1)", $do_Rc;
	    print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_FXM(_fxm) )\n";
	}
	elsif ($do_full_format eq "XFX_4") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_spr, _rs) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1)", $do_Rc;
	    print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_SPR(_spr) )\n";
	}
	elsif ($do_full_format eq "XFX_5") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_xo2) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1)", $do_Rc;
	    print TOFILE " | OPC_ENC_XO2(_xo2) )\n";
	}
    }

    elsif ($do_full_format eq "XFL_1") {
	print TOFILE "#define OPC_ASM_", $do_id, "(_flm, _rb) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	print TOFILE " | OPC_ENC_FLM(_flm) | OPC_ENC_RB(_rb) )\n";
    }

    elsif ($do_full_format eq "XS_1") {
	print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _rs, _sh) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<2) | ", $do_Rc;
	print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) | OPC_ENC_SHSH(_sh) )\n";
    }

    elsif ($do_full_format eq "XO_1") {
	if( $do_RB eq "0") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_rt, _ra) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_OE, "<<10) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RT(_rt) | OPC_ENC_RA(_ra) )\n";
	}
	else {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_rt, _ra, _rb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_OE, "<<10) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RT(_rt) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb) )\n";
	}
    }
    elsif ($do_full_format eq "XO_2") {
	print TOFILE "#define OPC_ASM_", $do_id, "(_rt, _ra, _rb) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_OE, "<<10) | (", $do_extended_op, "<<1) | ", $do_Rc;
	print TOFILE " | OPC_ENC_RT(_rt) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb) )\n";
    }

    elsif( $do_full_format eq "A_1" ) {
	if( ( $do_FRA eq "0" ) && ( $do_FRC eq "0" ) ) {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_frt, _frb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_FRT(_frt) | OPC_ENC_FRB(_frb) )\n";
	}
	elsif( $do_FRB eq "0") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_frt, _fra, _frc) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_FRT(_frt) | OPC_ENC_FRA(_fra) | OPC_ENC_FRC(_frc))\n";
	}
	elsif( $do_FRC eq "0" ) {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_frt, _fra, _frb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_FRT(_frt) | OPC_ENC_FRA(_fra) | OPC_ENC_FRB(_frb) )\n";
	}
	else {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_frt, _fra, _frc, _frb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_FRT(_frt) | OPC_ENC_FRA(_fra) | OPC_ENC_FRB(_frb) | OPC_ENC_FRC(_frc))\n";
	}
    }

    elsif( $do_full_format eq "M_1" ) {
	print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _rs, _rb, _mb, _me) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_Rc;
	print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb) | OPC_ENC_MB(_mb) | OPC_ENC_ME(_me) )\n";
    }
    elsif( $do_full_format eq "M_2" ) {
	print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _rs, _sh, _mb, _me) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_Rc;
	print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) | OPC_ENC_SH(_sh) | OPC_ENC_MB(_mb) | OPC_ENC_ME(_me) )\n";
    }

    elsif( $do_full_format eq "MD_1" ) {
	print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _rs, _sh, _mb) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<2) | ", $do_Rc;
	print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) | OPC_ENC_SHSH(_sh) | OPC_ENC_MBW(_mb) )\n";
    }
    elsif( $do_full_format eq "MD_2" ) {
	print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _rs, _sh, _me) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<2) | ", $do_Rc;
	print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) | OPC_ENC_SHSH(_sh) | OPC_ENC_MEW(_me) )\n";
    }
    
    elsif( $do_format eq "MDS_form" ) {
	if( $do_full_format eq "MDS_1" ) {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _rs, _rb, _mb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb) | OPC_ENC_MBW(_mb) )\n";
	}
	elsif( $do_full_format eq "MDS_2" ) {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _rs, _rb, _me) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb) | OPC_ENC_MEW(_me) )\n";
	}
	elsif( $do_full_format eq "MDS_3" ) {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _is, _ib, _xbi) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_IS(_is) | OPC_ENC_RA(_ra) | OPC_ENC_IB(_ib) | OPC_ENC_XBI(_xbi) )\n";
	}
	elsif( $do_full_format eq "MDS_3n" ) {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _is, _ib, _cbi) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_IS(_is) | OPC_ENC_RA(_ra) | OPC_ENC_IB(_ib) | OPC_ENC_CBI(_cbi) )\n";
	}
	elsif( $do_full_format eq "MDS_4" ) {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _is, _rb, _xbi) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_IS(_is) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb) | OPC_ENC_XBI(_xbi) )\n";
	}
	elsif( $do_full_format eq "MDS_4n" ) {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _is, _rb, _cbi) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_IS(_is) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb) | OPC_ENC_CBI(_cbi) )\n";
	}
	elsif( $do_full_format eq "MDS_5" ) {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _rs, _ib, _xbi) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) | OPC_ENC_IB(_ib) | OPC_ENC_XBI(_xbi) )\n";
	}
	elsif( $do_full_format eq "MDS_5n" ) {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _rs, _ib, _cbi) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) | OPC_ENC_IB(_ib) | OPC_ENC_CBI(_cbi) )\n";
	}
	elsif( $do_full_format eq "MDS_6" ) {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _rs, _rb, _xbi) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb) | OPC_ENC_XBI(_xbi) )\n";
	}
	elsif( $do_full_format eq "MDS_6n" ) {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_ra, _rs, _rb, _cbi) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RS(_rs) | OPC_ENC_RA(_ra) | OPC_ENC_RB(_rb) | OPC_ENC_CBI(_cbi) )\n";
	}
    }
    
    elsif( $do_full_format eq "TX_1" ) {
	print TOFILE "#define OPC_ASM_", $do_id, "(_to, _ui, _xbi) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	print TOFILE " | OPC_ENC_TO(_to) | OPC_ENC_UIL(_ui) | OPC_ENC_XBI(_xbi) )\n";
    }

    elsif( $do_full_format eq "VA_1" ) {
	if( ( $do_VA eq "0" ) && ( $do_VC eq "0" ) ) {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_vt, _vb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
	    print TOFILE " | OPC_ENC_VT(_vt) | OPC_ENC_VB(_vb) )\n";
	}
	elsif( $do_VB eq "0") {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_vt, _va, _vc) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
	    print TOFILE " | OPC_ENC_VT(_vt) | OPC_ENC_VA(_va) | OPC_ENC_VC(_vc))\n";
	}
	elsif( $do_VC eq "0" ) {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_vt, _va, _vb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
	    print TOFILE " | OPC_ENC_VT(_vt) | OPC_ENC_VA(_va) | OPC_ENC_VB(_vb) )\n";
	}
	else {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_vt, _va, _vc, _vb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
	    print TOFILE " | OPC_ENC_VT(_vt) | OPC_ENC_VA(_va) | OPC_ENC_VB(_vb) | OPC_ENC_VC(_vc))\n";
	}
    }
    elsif( $do_full_format eq "VA_2" ) {
	print TOFILE "#define OPC_ASM_", $do_id, "(_vt, _va, _vb, _shb) \\\n";
	print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
	print TOFILE " | OPC_ENC_VT(_vt) | OPC_ENC_VA(_va) | OPC_ENC_VB(_vb) | OPC_ENC_SHB(_shb))\n";
    }

    elsif ($do_format eq "VX_form") {
	if( $do_full_format eq "VX_1" ) {
	    if ( ($do_VT eq "0") && ($do_VA eq "0") && ($do_VB eq "0") ) {
		print TOFILE "#define OPC_ASM_", $do_id, "() \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
		print TOFILE " )\n";	
	    }
	    elsif ( ($do_VA eq "0") && ($do_VB eq "0") ) {
		print TOFILE "#define OPC_ASM_", $do_id, "(_vt) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
		print TOFILE " | OPC_ENC_VT(_vt) )\n";
	    }
	    elsif ( ($do_VT eq "0") && ($do_VA eq "0") ) {
		print TOFILE "#define OPC_ASM_", $do_id, "(_vb) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
		print TOFILE " | OPC_ENC_VB(_vb) )\n";
	    }
	    elsif ($do_VT eq "0") {
		print TOFILE "#define OPC_ASM_", $do_id, "(_va, _vb) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
		print TOFILE " | OPC_ENC_VA(_va) | OPC_ENC_VB(_vb))\n";
	    }
	    elsif ($do_VA eq "0") {
		print TOFILE "#define OPC_ASM_", $do_id, "(_vt, _vb) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
		print TOFILE " | OPC_ENC_VT(_vt) | OPC_ENC_VB(_vb) )\n";
	    }
	    elsif ($do_VB eq "0") {
		print TOFILE "#define OPC_ASM_", $do_id, "(_vt, _va) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
		print TOFILE " | OPC_ENC_VT(_vt) | OPC_ENC_VA(_va) )\n";
	    }
	    else {
		print TOFILE "#define OPC_ASM_", $do_id, "(_vt, _va, _vb) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
		print TOFILE " | OPC_ENC_VT(_vt) | OPC_ENC_VA(_va) | OPC_ENC_VB(_vb))\n";
	    }
	}
	elsif( $do_full_format eq "VX_2" ) {
	    if ($do_VB eq "0") {
		print TOFILE "#define OPC_ASM_", $do_id, "(_vt, _uimm) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
		print TOFILE " | OPC_ENC_VT(_vt) | OPC_ENC_UIMM(_uimm) )\n";
	    }
	    else {
		print TOFILE "#define OPC_ASM_", $do_id, "(_vt, _uimm, _vb) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
		print TOFILE " | OPC_ENC_VT(_vt) | OPC_ENC_UIMM(_uimm) | OPC_ENC_VB(_vb))\n";
	    }
	}
	elsif( $do_full_format eq "VX_3" ) {
	    if ($do_VB eq "0") {
		print TOFILE "#define OPC_ASM_", $do_id, "(_vt, _simm) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
		print TOFILE " | OPC_ENC_VT(_vt) | OPC_ENC_SIMM(_simm))\n";
	    }
	    else {
		print TOFILE "#define OPC_ASM_", $do_id, "(_vt, _simm, _vb) \\\n";
		print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
		print TOFILE " | OPC_ENC_VT(_vt) | OPC_ENC_SIMM(_simm) | OPC_ENC_VB(_vb))\n";
	    }
	}
    }

    elsif ($do_format eq "VXR_form") {
	if( $do_full_format eq "VXR_1" ) {
	    print TOFILE "#define OPC_ASM_", $do_id, "(_vt, _va, _vb) \\\n";
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_Rc, "<<10) | ", $do_extended_op;
	    print TOFILE " | OPC_ENC_VT(_vt) | OPC_ENC_VA(_va) | OPC_ENC_VB(_vb))\n";
	}
    }

    else {
	print STDERR "Opcode_asm.pl: Unknown format ", $do_format, " : ", $do_full_format, " for ", $do_id, " in process_asm\n";
    }

    print TOFILE "\n";
}

sub DoFinalGenerateAsmMacros {
    local (*TOFILE) = $_[0];
    #close(TOFILE);
}

