# Formats: I_form, B_form, SC_form, D_form, DS_form, DQ_form, 
#	  X_form, XL_form, XFX_form, XFL_form, XS_form, XO_form, 
#	  A_form, M_form, MD_form, MDS_form, TX_form
#
# Full-Formats:
#
#    Value                           1 1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 2 2 2 2 3  3       
#                0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0  1              
#     I_1:     |   opcode  |                       LI                      |AA|LK|
#
#     B_1:     |   opcode  |    BO   |    BI   |            BD             |AA|LK|
#
#    SC_1:     |   opcode  |   ///   |   ///   |           ///             |XO|LK|
#    SC_2:     |   opcode  |   ///   |   ///   |  ///  |   LEV       | /// |XO|LK|
#
#     D_1:     |   opcode  |    RT   |    RA   |                D                |
#     D_2:     |   opcode  |    RT   |    RA   |               SI                |
#     D_3:     |   opcode  |    RS   |    RA   |                D                |
#     D_4:     |   opcode  |    RS   |    RA   |               UI                |
#     D_5:     |   opcode  |  BF |/|L|    RA   |               SI                |
#     D_6:     |   opcode  |  BF |/|L|    RA   |               UI                |
#     D_7:     |   opcode  |    TO   |    RA   |               SI                |
#     D_8:     |   opcode  |   FRT   |    RA   |                D                |
#     D_9:     |   opcode  |   FRS   |    RA   |                D                |
#
#    DS_1:     |   opcode  |    RT   |    RA   |               DS          | XO  |
#    DS_2:     |   opcode  |    RS   |    RA   |               DS          | XO  |
#
#    DQ_1:     |   opcode  |    RT   |    RA   |          DQ           |    PT  |
#
#    X_1:      |   opcode  |    RT   |    RA   |    RB   |         XO         |//|
#    X_2:      |   opcode  |    RT   |    RA   |    NB   |         XO         |//|
#    X_3:      |   opcode  |    RS   |    RA   |    RB   |         XO         |Rc|
#    X_4:      |   opcode  |    RS   |    RA   |    NB   |         XO         |//|
#    X_5:      |   opcode  |    RS   |    RA   |    SH   |         XO         |Rc|
#    X_6:      |   opcode  |    RS   |   ///   |   ///   |         XO         |//|
#    X_7:      |   opcode  |  BF |/|L|    RA   |    RB   |         XO         |//|
#    X_8:      |   opcode  |  BF |///|   FRA   |   FRB   |         XO         |//|
#    X_9:      |   opcode  |  BF |///| BFA |///|   ///   |         XO         |//|
#    X_10:     |   opcode  |  BF |///|   ///   |    U  |/|         XO         |Rc|
#    X_11:     |   opcode  |    TO   |    RA   |    RB   |         XO         |//|
#    X_12:     |   opcode  |   FRT   |    RA   |    RB   |         XO         |//|
#    X_13:     |   opcode  |   FRT   |   ///   |   FRB   |         XO         |Rc|
#    X_14:     |   opcode  |   FRS   |    RA   |    RB   |         XO         |//|
#    X_15:     |   opcode  |    BT   |   ///   |   ///   |         XO         |Rc|
#    X_16:     |   opcode  |    RS   |/|   SR  |   ///   |         XO         |//|
#    X_16:     |   opcode  |    RT   |/|   SR  |   ///   |         XO         |//|
#    X_17:     |   opcode  |    RT   |/|   SR  |   ///   |         XO         |//|
#    X_18:     |   opcode  |T|//|STRM|    RA   |    RB   |         XO         |//|
#    X_19:     |   opcode  |    VS   |    RA   |    RB   |         XO         |Rc|
#    X_20:     |   opcode  |    VT   |    RA   |    RB   |         XO         |Rc|
#
#    XL_1:     |   opcode  |    BT   |    BA   |    BB   |         XO         |//|
#    XL_2:     |   opcode  |    BO   |    BI   |   ///   |         XO         |LK|
#    XL_3:     |   opcode  |  BF |///| BFA |///|   ///   |         XO         |//|
#    XL_4:     |   opcode  |   ///   |   ///   |   ///   |         XO         |//|
#
#   XFX_1:     |   opcode  |    RT   |        SPR        |         XO         |//|
#   XFX_2:     |   opcode  |    RT   |        TBR        |         XO         |//|
#   XFX_3:     |   opcode  |    RS   |/|      FXM      |/|         XO         |//|
#   XFX_4:     |   opcode  |    RS   |        SPR        |         XO         |//|
#   XFX_5:     |   opcode  |   ///   |        XO2        |         XO         |//|
#
#   XFL_1:     |   opcode  |/|      FLM      |/|   FRB   |         XO         |Rc|
#
#    XS_1:     |   opcode  |    RS   |    RA   |    SH   |         XO      |SH|Rc|
#
#    XO_1:     |   opcode  |    RT   |    RA   |    RB   |OE|      XO         |Rc|
#    XO_2:     |   opcode  |    RT   |    RA   |    RB   |//|      XO         |Rc|
#
#     A_1:     |   opcode  |   FRT   |   FRA   |   FRB   |   FRC   |    XO    |Rc|
#
#     M_1:     |   opcode  |    RS   |    RA   |    RB   |    MB   |    ME    |Rc|
#     M_2:     |   opcode  |    RS   |    RA   |    SH   |    MB   |    ME    |Rc|
#
#    MD_1:     |   opcode  |    RS   |    RA   |    SH   |    MB   |   XO  |SH|Rc|
#    MD_2:     |   opcode  |    RS   |    RA   |    SH   |    ME   |   XO  |SH|Rc|
#
#   MDS_1:     |   opcode  |    RS   |    RA   |    RB   |    MB   |     XO   |Rc|
#   MDS_2:     |   opcode  |    RS   |    RA   |    RB   |    ME   |     XO   |Rc|
#   MDS_3:     |   opcode  |    IS   |    RA   |    IB   | XBI  |//|     XO   |Rc|
#   MDS_4:     |   opcode  |    IS   |    RA   |    RB   | XBI  |//|     XO   |Rc|
#   MDS_5:     |   opcode  |    RS   |    RA   |    IB   | XBI  |//|     XO   |Rc|
#   MDS_6:     |   opcode  |    RS   |    RA   |    RB   | XBI  |//|     XO   |Rc|
#
#    TX_1:     |   opcode  |    TO   |        UI         |  XBI |       XO    |Rc|
#
#    VA_1:     |   opcode  |    VT   |    VA   |    VB   |    VC   |     XO      |
#    VA_2:     |   opcode  |    VT   |    VA   |    VB   |/|   SH  |     XO      |
#
#    VX_1:     |   opcode  |    VT   |    VA   |    VB   |           XO          |
#    VX_2:     |   opcode  |    VT   |   UIMM  |    VB   |           XO          |
#    VX_3:     |   opcode  |    VT   |   SIMM  |    VB   |           XO          |
#
#    VXR_1:    |   opcode  |    VT   |    VA   |    VB   |Rc|        XO          |
#


sub InitializeExecInfo {
    $G_exec_trans{"FPR"}  = "fp%d";
    $G_exec_trans{"GPR"}  = "r%d";
    $G_exec_trans{"0"}    = "";
    $G_exec_trans{"1"}    = "";
    $G_exec_trans{"IS"}   = "%d";
    $G_exec_trans{"IB"}   = "%d";
    $G_exec_trans{"SI"}   = "%d";
    $G_exec_trans{"UI"}   = "%d";
    $G_exec_trans{"UI"}   = "%d";
    $G_exec_trans{"D"}    = "%d";
    $G_exec_trans{"U"}    = "0x%x";
    $G_exec_trans{"DS"}   = "%d";
    $G_exec_trans{"DQ"}   = "%d";
    $G_exec_trans{"SH"}   = "%d";
    $G_exec_trans{"SHw"}  = "%d";
    $G_exec_trans{"NB"}   = "%d";
    $G_exec_trans{"SR"}   = "%d";
    $G_exec_trans{"TO"}   = "%d";
    $G_exec_trans{"SPR"}  = "0x%x";
    $G_exec_trans{"TBR"}  = "0x%x";
    $G_exec_trans{"FXM"}  = "0x%x";
    $G_exec_trans{"FLM"}  = "0x%x";
    $G_exec_trans{"CRB"}  = "cb%d";
    $G_exec_trans{"CRF"}  = "cr%d";
    $G_exec_trans{"CR"}   = "cr%d";
    $G_exec_trans{"MB"}   = "%d";
    $G_exec_trans{"MBw"}  = "%d";
    $G_exec_trans{"ME"}   = "%d";
    $G_exec_trans{"MEw"}  = "%d";

    $G_exec_acc{"FPR"} = "proc_state_FPR(state,";
    $G_exec_acc{"GPR"} = "proc_state_GPR(state,";
    $G_exec_acc{"0"}   = "";
    $G_exec_acc{"1"}   = "";
    $G_exec_acc{"IS"}  = "(";
    $G_exec_acc{"IB"}  = "(";
    $G_exec_acc{"SI"}  = "(";
    $G_exec_acc{"UI"}  = "(";
    $G_exec_acc{"D"}   = "(";
    $G_exec_acc{"U"}   = "(";
    $G_exec_acc{"DS"}  = "(";
    $G_exec_acc{"DQ"}  = "(";
    $G_exec_acc{"SH"}  = "(";
    $G_exec_acc{"NB"}  = "(";
    $G_exec_acc{"SR"}  = "(";
    $G_exec_acc{"TO"}  = "(";
    $G_exec_acc{"SPR"} = "(";
    $G_exec_acc{"TBR"} = "(";
    $G_exec_acc{"FXM"} = "(";
    $G_exec_acc{"FLM"} = "(";
    $G_exec_acc{"CRB"} = "proc_state_CRB(state,";
    $G_exec_acc{"CRF"} = "proc_state_CRF(state,";
    $G_exec_acc{"CR"}  = "cr%d";
    $G_exec_acc{"MB"}  = "(";
    $G_exec_acc{"MBw"} = "(";
    $G_exec_acc{"ME"}  = "(";
    $G_exec_acc{"MEw"} = "(";
    
    $G_exec_set{"FPR"} = "proc_state_FPR(state, %s) = %s";
    $G_exec_set{"GPR"} = "proc_state_GPR(state, %s) = %s";
    $G_exec_set{"0"}   = "";
    $G_exec_set{"1"}   = "";
    $G_exec_set{"IS"}  = "";
    $G_exec_set{"IB"}  = "";
    $G_exec_set{"SI"}  = "";
    $G_exec_set{"UI"}  = "";
    $G_exec_set{"D"}   = "";
    $G_exec_set{"U"}   = "";
    $G_exec_set{"DS"}  = "";
    $G_exec_set{"DQ"}  = "";
    $G_exec_set{"SH"}  = "";
    $G_exec_set{"NB"}  = "";
    $G_exec_set{"SR"}  = "";
    $G_exec_set{"TO"}  = "";
    $G_exec_set{"SPR"} = "";
    $G_exec_set{"TBR"} = "";
    $G_exec_set{"FXM"} = "";
    $G_exec_set{"FLM"} = "";
    $G_exec_set{"CRB"} =  "set_proc_state_CRB(state, %s, %s)";
    $G_exec_set{"CRF"} = "set_proc_state_CRF(state, %s, %s)";
    $G_exec_set{"CR"}  = "set_proc_state_CRF(state, %s, %s)";
    $G_exec_set{"MB"}  = "";
    $G_exec_set{"MBw"} = "";
    $G_exec_set{"ME"}  = "";
    $G_exec_set{"MEw"} = "";
}


sub gen_exec_format3 {
    local( *TOFILE, $id, $arg1, $fld1, $arg2, $fld2, $arg3, $fld3) = @_;
    &gen_exec_format5( TOFILE, $id, $arg1, $fld1, $arg2, $fld2, $arg3, $fld3, "", "", "", "");
}

sub gen_exec_format4 {
    local(*TOFILE, $id, $arg1, $fld1, $arg2, $fld2, $arg3, $fld3, $arg4, $fld4) = @_;
    &gen_exec_format5( TOFILE, $id, $arg1, $fld1, $arg2, $fld2, $arg3, $fld3, $arg4, $fld4,"", "");
}

sub gen_exec_format5 {
    local (*TOFILE) = $_[0];
    local ($id)   = $_[1];
    local ($arg1) = $_[2]; 
    local ($fld1) = $_[3]; 
    local ($arg2) = $_[4]; 
    local ($fld2) = $_[5];
    local ($arg3) = $_[6]; 
    local ($fld3) = $_[7]; 
    local ($arg4) = $_[8]; 
    local ($fld4) = $_[9]; 
    local ($arg5) = $_[10]; 
    local ($fld5) = $_[11];
    
    local($n, $i, @uses, @sets, $use_CA, $set_CA, $set_CR0, $suffix);

    @uses = split( ":", $do_uses);
    $use_CA = 0;
    $use_SO = 0;
    for( $i = 0; $i <= $#uses; $i++ ) {
	if( $uses[$i] eq "SO" ) {
	    $use_SO = 1;
	}
	if( $uses[$i] eq "CA" ) {
	    $use_CA = 1;
	}
    }

    #printf("gen_exec_format5(%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s);\n", $id, $arg1, $fld1, $arg2, $fld2, $arg3, $fld3, $arg4, $fld4, $arg5, $fld5);

    if( $arg1 eq "FPR" ) {
	$G_T0 = "F0";
    }
    else {
	$G_T0 = "T0";
    }
    if( $arg2 eq "FPR" ) {
	$G_T1 = "F1";
    }
    else {
	$G_T1 = "T1";
    }
    if( $arg3 eq "FPR" ) {
	$G_T2 = "F2";
    }
    else {
	$G_T2 = "T2";
    }
    if( $arg4 eq "FPR" ) {
	$G_T3 = "F3";
    }
    else {
	$G_T3 = "T3";
    }
    if( $arg5 eq "FPR" ) {
	$G_T4 = "F4";
    }
    else {
	$G_T4 = "T4";
    }

    $G_arg = "";
    $n = 0;

    if( $do_store && ($G_exec_trans{$arg1} ne "" )) {
	$n ++;
	if( $G_arg ne "" ) {
	    $G_arg = $G_arg . ",";
	}
	$G_arg =  $G_T0;
    }
    if( $G_exec_trans{$arg2} ne "" ) {
	$n ++;
	if( $G_arg ne "" ) {
	    $G_arg = $G_arg . ",";
	}
	$G_arg =  $G_arg . $G_T1;
    }
    if( $G_exec_trans{$arg3} ne "" ) {
	$n ++;
	if( $G_arg ne "" ) {
	    $G_arg = $G_arg . ",";
	}
	$G_arg = $G_arg . $G_T2;
    }
    if( $G_exec_trans{$arg4} ne "" ) {
	$n ++;
	if( $G_arg ne "" ) {
	    $G_arg = $G_arg . ",";
	}
	$G_arg = $G_arg . $G_T3;
    }
    if( $G_exec_trans{$arg5} ne "" ) {
	$n ++;
	if( $G_arg ne "" ) {
	    $G_arg = $G_arg . ",";
	}
	$G_arg = $G_arg . $G_T4;
    }
    if( $use_CA ) {
	$n ++;
	if( $G_arg ne "" ) {
	    $G_arg = $G_arg . ",";
	}
	$G_arg = $G_arg . "CA";
    }
#     if( $use_SO && (!$do_Rc || ($do_id eq "STWCX_")) )
    if( $use_SO ) {
	$n ++;
	if( $G_arg ne "" ) {
	    $G_arg = $G_arg . ",";
	}
	$G_arg = $G_arg . "SO";
    }
    
    @sets = split(":", $do_sets);
    $set_CA = 0;
    $set_CR0 = 0;
    
    for( $i = 0; $i <= $#sets; $i++) {
	if( $sets[$i] eq "CA" ) {
	    $set_CA = 1;
	}
	if( $sets[$i] eq "CR0" ) {
	    $set_CR0 = 1;
	}
    }
    
    #printf("  %d ==> G_exec_acc{%s} = '%s' (%s);\n", 1, $arg1, $G_exec_acc{$arg1}, $fld1);
    #printf("  %d ==> G_exec_acc{%s} = '%s' (%s);\n", 2, $arg2, $G_exec_acc{$arg2}, $fld2);
    #printf("  %d ==> G_exec_acc{%s} = '%s' (%s);\n", 3, $arg3, $G_exec_acc{$arg3}, $fld3);
    #printf("  %d ==> G_exec_acc{%s} = '%s' (%s);\n", 4, $arg4, $G_exec_acc{$arg4}, $fld4);
    #printf("  %d ==> G_exec_acc{%s} = '%s' (%s);\n", 5, $arg5, $G_exec_acc{$arg5}, $fld5);

    #print(TOFILE "\n");
    if( $do_store && ($G_exec_acc{$arg1} ne "") ) {
	if( ($do_id eq "STSWI") || ($do_id eq "STSWX") || ($do_id eq "STSDI") || ($do_id eq "STSDX") || ($do_id eq "STMW") ) {
	    print(TOFILE "      R0 = OPC_DEC_", $fld1, "(iword);\n");
	    print(TOFILE "      T0 = R0;\n");
	}
	elsif( ( $arg1 eq "CRF") ) {
	    print(TOFILE "      R0 = OPC_DEC_", $fld1, "(iword);\n");
	    print(TOFILE "      ", $G_T0, " = ", $G_exec_acc{$arg1}, "R0);\n");
	}
	else {
	    print(TOFILE "      R0 = OPC_DEC_", $fld1, "(iword);\n");
	    print(TOFILE "      ", $G_T0, " = ", $G_exec_acc{$arg1}, "R0);\n");
	}
    }
    
    if( $G_exec_acc{$arg2} ne "" ) {
	if( $arg2 eq "CRF" ) {
	    print(TOFILE "      R1 = OPC_DEC_", $fld2, "(iword);\n");
	}
	else {
	    print(TOFILE "      R1 = OPC_DEC_", $fld2, "(iword);\n");
	}
	if( $do_load || $do_store || $do_cache || $do_RAas0 ) {
	    print(TOFILE "      ", $G_T1, " = R1?", $G_exec_acc{$arg2}, "R1):0;\n");
	}
	else {
	    print(TOFILE "      ", $G_T1, " = ", $G_exec_acc{$arg2}, "R1);\n");
	}
    }
    
    if( $G_exec_acc{$arg3} ne "" ) {
	if( $arg3 eq "CRF" ) {
	    print(TOFILE "      R2 = OPC_DEC_", $fld3, "(iword);\n");
	}
	else {
	    print(TOFILE "      R2 = OPC_DEC_", $fld3, "(iword);\n");
	}
	print(TOFILE "      ", $G_T2, " = ", $G_exec_acc{$arg3}, "R2);\n");
    }
    
    if( $G_exec_acc{$arg4} ne "" ) {
	print(TOFILE "      R3 = OPC_DEC_", $fld4, "(iword);\n");
	print(TOFILE "      ", $G_T3, " = ", $G_exec_acc{$arg4}, "R3);\n");
    }

    if( $G_exec_acc{$arg5} ne "" ) {
	print(TOFILE "      R4 = OPC_DEC_", $fld5, "(iword);\n");
	print(TOFILE "      ", $G_T4, " = ", $G_exec_acc{$arg5}, "R4);\n");
    }

    if( $use_CA ) {
	print(TOFILE "      CA = proc_state_XER_CA(state);\n");
    }
    if( $use_SO ) {
	print(TOFILE "      SO = proc_state_XER_SO(state);\n");
    }

    if( $do_load || $do_store ) {
#    if( $do_load ) {
#      if( $do_id eq "LHZ" || $do_id eq "LHZU" || $do_id eq "LHZUX" || $do_id eq "LHZX" ) {
#	_prefix = "read", $do_dlength "s_";
#      }
#      elsif( $do_float eq "1" && $do_dlength eq "4" ) {
#	_prefix = "readf_"
#      }
#      elsif( $do_float eq "1" && $do_dlength eq "8" ) {
#	_prefix = "readd_"
#      }
#      else {
#	_prefix = "read", $do_dlength "_";
#	_suffix = "";
#      }
#      $suffix = "_for_read";
#    }
#    else {
#      if( $do_float eq "1" && $do_dlength eq "4" ) {
#	_prefix = "writef_"
#      }
#      elsif( $do_float eq "1" && $do_dlength eq "8" ) {
#	_prefix = "writed_"
#      }
#      else {
#	_prefix = "write", $do_dlength "_";
#      }
#      $suffix = "_noval";
#    }

	print(TOFILE "      if((show_access_addrs) || (show_interval_access_addrs && do_interval_output)) {\n");
	if ( ($do_id eq "LSWI") || ($do_id eq "LSDI") || ($do_id eq "STSWI") || ($do_id eq "STSDI")) {
	    print(TOFILE "        fprintf(mystderr, \"             access to (EA = 0x%08x, PA = 0x%08x)\\n\", (T1), translate_memory(state,T1));\n");
	}
	else {
	    print(TOFILE "        fprintf(mystderr, \"             access to (EA = 0x%08x, PA = 0x%08x)\\n\", (T1+T2), translate_memory(state, (T1+T2)));\n");
	}
	print(TOFILE "        /**fflush(mystderr);**/\n");
	print(TOFILE "      }\n");

    }

    if( (!$do_store) && ($G_exec_set{$arg1} ne "") && (!$do_multaccess) && (!$do_string) && ($do_id ne "MFSPR") ) {
	if( ( $arg1 eq "CRF") ) {
	    print(TOFILE "      R0 = OPC_DEC_", $fld1, "(iword);\n");
	}
	else {
	    print(TOFILE "      R0 = OPC_DEC_", $fld1, "(iword);\n");
	}
	if ( $do_id eq "FCMPO" || $do_id eq "FCMPU") {
	    print(TOFILE "      OPC_EXEC_", $id, "(R0,", $G_arg, ");\n");
	}
	else {
	    print(TOFILE "      ", $G_T0, " = OPC_EXEC_", $id, "(", $G_arg, ");\n");
	    if( $do_load ) {
		if ($do_dlength == 1) {
		    print(TOFILE "      if ((show_access_values) || (show_interval_access_values && do_interval_output)){\n");
		    print(TOFILE "        fprintf(mystderr, \"             access  value = 0x%02x\\n\", ", $G_T0, ");\n");
		    print(TOFILE "      }\n");
		}
		elsif ($do_dlength == 2) {
		    print(TOFILE "      if ((show_access_values) || (show_interval_access_values && do_interval_output)){\n");
		    print(TOFILE "        fprintf(mystderr, \"             access  value = 0x%04x\\n\", ", $G_T0, ");\n");
		    print(TOFILE "      }\n");
		}
		elsif ($do_dlength == 4) {
		    print(TOFILE "      if ((show_access_values) || (show_interval_access_values && do_interval_output)){\n");
		    print(TOFILE "        fprintf(mystderr, \"             access  value = 0x%08x\\n\", ", $G_T0, ");\n");
		    print(TOFILE "      }\n");
		}
		elsif ($do_dlength == 8) {
		    print(TOFILE "      if ((show_access_values) || (show_interval_access_values && do_interval_output)){\n");
		    print(TOFILE "        fprintf(mystderr, \"             access  value = 0x%08x\\n\", ", $G_T0, ");\n");
		    print(TOFILE "      }\n");
		}
		else {
		    print(TOFILE "******** Warning: ", $do_id, " has dlength of ", $do_dlength);
		}	  
	    }
	    printf(TOFILE "      ");
	    printf(TOFILE $G_exec_set{$arg1}, "R0", $G_T0);
	    print(TOFILE ";\n");
	}
    }
    elsif( (!$do_store) && ($G_exec_acc{$arg1} ne "") ) {
	if( ( $arg1 eq "CRF") ) { 
	    print(TOFILE "      R0 = OPC_DEC_", $fld1, "(iword);\n");
	}
	else {
	    print(TOFILE "      R0 = OPC_DEC_", $fld1, "(iword);\n");
	}
	if( $G_arg ne "" ) {
	    print(TOFILE "      OPC_EXEC_", $id, "(R0,", $G_arg, ");\n");
	}
	else {
	    print(TOFILE "      OPC_EXEC_", $id, "(R0);\n");
	}
    }
    else {
	print(TOFILE "      OPC_EXEC_", $id, "(", $G_arg, ");\n");
	if( $do_store ) {
	    if ($do_dlength == 1) {
		print(TOFILE "      if ((show_access_values) || (show_interval_access_values && do_interval_output)){\n");
		print(TOFILE "        fprintf(mystderr, \"             access value = 0x%02x\\n\", T0);\n");
		print(TOFILE "      }\n");
	    }
	    elsif ($do_dlength == 2) {
		print(TOFILE "      if ((show_access_values) || (show_interval_access_values && do_interval_output)){\n");
		print(TOFILE "        fprintf(mystderr, \"             access  value = 0x%04x\\n\", T0);\n");
		print(TOFILE "      }\n");
	    }
	    elsif ($do_dlength == 4) {
		print(TOFILE "      if ((show_access_values) || (show_interval_access_values && do_interval_output)){\n");
		print(TOFILE "        fprintf(mystderr, \"             access  value = 0x%08x\\n\", T0);\n");
		print(TOFILE "      }\n");
	    }
	    elsif ($do_dlength == 8) {
		print(TOFILE "      if ((show_access_values) || (show_interval_access_values && do_interval_output)){\n");
		print(TOFILE "        fprintf(mystderr, \"             access  value = 0x%08x\\n\", T0);\n");
		print(TOFILE "      }\n");
	    }
	    else {
		print(TOFILE "      if ((show_access_values) || (show_interval_access_values && do_interval_output)){\n");
		print(TOFILE "        fprintf(mystderr, \"             access  value = <too many>\\n\");\n");
		print(TOFILE "      }\n");
	    }
	}
    }

    if( $do_OE ) {
	print(TOFILE "\n");
	print(TOFILE "      OV = OPC_OVFL_", $id, "(", $G_T0, ",", $G_arg, ");\n");
	print(TOFILE "      or_proc_state_XER_SO(state,OV);\n");
	print(TOFILE "      set_proc_state_XER_OV(state,OV);\n");
    }

    if( $do_id eq "STWCX_" ) {
	print(TOFILE "      /* CR0 should be set in OPC_EXEC_STWCX_() */\n");
    }
    elsif( $do_Rc ) {
	print(TOFILE "\n");
	if( $do_float ) {
#	    print(TOFILE "      SO = proc_state_XER_SO(state);\n");
#	    if( $do_OE  ) {
#		print(TOFILE "      SO |= OV;\n");
#	    }
#	    print(TOFILE "      CR1 = OPC_CMPF(", $G_T0, ",SO);\n");
#	    print(TOFILE "      set_proc_state_CRF(state,1,CR1);\n");
	    print(TOFILE "      /* CR1 should be set in OPC_EXEC() function */\n");
	}
	else {
	    print(TOFILE "      SO = proc_state_XER_SO(state);\n");
	    if( $do_OE  ) {
		print(TOFILE "      SO |= OV;\n");
	    }
	    print(TOFILE "      CR0 = OPC_CMPX(", $G_T0, ",SO);\n");
	    print(TOFILE "      set_proc_state_CRF(state,0,CR0);\n");
	}
    }
    elsif( $set_CR0 ) {
	print(TOFILE "\n");
	print(TOFILE "      SO = proc_state_XER_SO(state);\n");
	print(TOFILE "      CR0 = OPC_CMPX(", $G_T0, ",SO);\n");
	print(TOFILE "      set_proc_state_CRF(state,0,CR0);\n");
    }

    if( $set_CA && $do_id ne "MCRXR" ) {
	print(TOFILE "\n");
	print(TOFILE "      CA = OPC_CARRY_", $id, "(", $G_T0, ",", $G_arg, ");\n");
	print(TOFILE "      set_proc_state_XER_CA(state,CA);\n");
    }

    if( $do_update ) {
	print(TOFILE "\n");
	print(TOFILE "      if( R1 != 0 ) {\n");
	print(TOFILE "        ", $G_T1, " = ", $G_T1, " + ", $G_T2, ";\n");
	print(TOFILE "        proc_state_GPR(state,R1) = ", $G_T1, ";\n");
	print(TOFILE "      }\n");
    }

    print(TOFILE "      proc_state_PC(state) += 4;\n");
}


sub gen_exec_dis {
    local (*TOFILE, $format, $args) = @_;
    if( $format ne "" ) {
	printf(TOFILE "%s\"", $format);
	if( $args ne "" ) {
	    printf(TOFILE ", %s", $args);
	}
    }
    else {
	printf(TOFILE "\"", $format);
    }
}


sub DoInitialExecCode {
    local(*TOFILE) = $_[0];
}

sub DoExecCodeForThisInstruction {
    local ( *TOFILE, $name, $format, $args ) = @_;
    
    if( $do_ppc32_name eq "" ) {
	#print "Skipping instruction $do_id (not in 32-bit ppc set)\n";
	return;
    }

    printf(TOFILE "  case OPC_%s: {\n", $do_id);
    if( $do_float ) { 
	print(TOFILE "    if (reg_MSR_FP(proc_state_MSR(state)) == 0) {\n");
	print(TOFILE "      DEBUG( fprintf(mystderr, \"\\n***  Taking an illegal instruction Program interrupt...\\n\\n\") );\n");
	print(TOFILE "      proc_state_SRR0(state) = (unsigned)proc_state_PC(state); \n");
	print(TOFILE "      proc_state_SRR1(state) = (0x00080000 | (proc_state_MSR(state) & 0x87c0ffff)); \n");
	print(TOFILE "      proc_state_MSR(state) = ((proc_state_MSR(state) & 0x00011040) | reg_MSR_ILE(proc_state_MSR(state)));\n"); 
	print(TOFILE "      proc_state_PC(state) = (reg_MSR_IP(proc_state_MSR(state)) == 0) ? 0x00800 : 0xfff00800;\n");
	print(TOFILE "      break;\n");
	print(TOFILE "    }\n");
    }

    $uselr = 0;
    if(( $do_full_format eq "XL_2" ) || 
	  ( $do_full_format eq "B_1"  ) ) {
	print(TOFILE "      PC = proc_state_PC(state);\n");
	print(TOFILE "      CTR = proc_state_CTR(state);\n");
	print(TOFILE "      R0 = OPC_DEC_BO(iword);\n");
	print(TOFILE "      T0 = R0;\n");
	print(TOFILE "      BO = R0;\n");
	print(TOFILE "      R1 = OPC_DEC_BI(iword);\n");
	print(TOFILE "      T1 = proc_state_CRB(state, R1);\n");

	if( $do_full_format eq "B_1" ) {
	    print(TOFILE "      R2 = OPC_DEC_BD(iword);\n");
	    if( $do_AA ) {
		print(TOFILE "      T2 = 4*R2;\n");
	    }
	    else {
		print(TOFILE "      T2 = PC + 4*R2;\n");
	    }
	}
	elsif( ( $do_id eq "BCCTR" ) || ( $do_id eq "BCCTRL" ) ) {
	    print(TOFILE "      T2 = CTR;\n");
	}
	else {
	    print(TOFILE "      T2 = proc_state_LR(state);\n");
	    $uselr = 1;
	}

	print(TOFILE "\n");
	print(TOFILE "      flag = 1;\n");
	print(TOFILE "      if( (BO & 0x4) == 0 ) {\n");
	print(TOFILE "        CTR = CTR - 1;\n");
	print(TOFILE "        proc_state_CTR(state) = CTR;\n");
	print(TOFILE "        flag = (CTR != 0) ^ ((BO & 0x2)!=0);\n");
	print(TOFILE "      }\n");

	print(TOFILE "\n");
	print(TOFILE "      flag &= ((BO & 0x10)!=0) | (((BO & 0x8)>>3) == T1);\n");

	print(TOFILE "      if( flag ) {\n");
	if( $uselr == 1 ) {
	    print(TOFILE "        OPC_EXEC_USE_LR(PC, T2);\n");
	}
	print(TOFILE "        proc_state_PC(state) = T2;\n");
	print(TOFILE "      }\n");
	print(TOFILE "      else {\n");
	print(TOFILE "        T2 = PC + 4;\n");
	print(TOFILE "        proc_state_PC(state) = T2;\n");
	print(TOFILE "      }\n");

	if( $do_LK ) {
	    print(TOFILE "      LR = PC + 4;\n");
	    print(TOFILE "      proc_state_LR(state) = LR;\n");
	    print(TOFILE "      OPC_EXEC_SET_LR(PC, LR);\n");
	}
    }
    elsif( $do_full_format eq "I_1" ) {
	print(TOFILE "      PC = proc_state_PC(state);\n");
	print(TOFILE "      R0 = OPC_DEC_LI(iword);\n");
	print(TOFILE "      T0 = 4*R0;\n");
	
	if( $do_AA ) {
	    print(TOFILE "      proc_state_PC(state) = T0;\n");
	}
	else {
	    print(TOFILE "      T1 = T0 + PC;\n");
	    print(TOFILE "      proc_state_PC(state) = T1;\n");
	}
	if( $do_LK ) {
	    print(TOFILE "      LR = PC + 4;\n");
	    print(TOFILE "      proc_state_LR(state) = LR;\n");
	    print(TOFILE "      OPC_EXEC_SET_LR(PC, LR);\n");
	}
    }
    elsif (( $do_full_format eq "SC_1" ) ||
	   ( $do_full_format eq "SC_2" ) ) { 
	print(TOFILE "      OPC_EXEC_SC(); \n");
    }

    elsif ( $do_full_format eq "D_1" ) {
	if ( $do_load ) {
	    &gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", $do_RA, "RA", "D", "D");
	}
	else {
	    print "ERROR: full_format $do_full_format MUST be a load\n";
	}
    }
    elsif ( $do_full_format eq "D_2" ) {
	if ( $do_load ) {
	    print "ERROR: full_format $do_full_format MUST NOT be a load\n";
	}
	else {
	    &gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", $do_RA, "RA", "SI", "SI");
	}
    }
    elsif( $do_full_format eq "D_3" ) {
	if( $do_store ) {
	    &gen_exec_format3( TOFILE, $do_id, $do_RS, "RS", $do_RA, "RA", "D", "D");
	}
	else {
	    print "ERROR: full_format $do_full_format MUST be a store\n";
	}
    }
    elsif( $do_full_format eq "D_4" ) {
	if( $do_store ) {
	    print "ERROR: full_format $do_full_format MUST NOT be a store\n";
	}
	else {
	    &gen_exec_format3( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "UI", "UI");
	}
    }
    elsif ( $do_full_format eq "D_5" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_BF, "BF", $do_RA, "RA", "SI", "SI");
    }
    elsif ( $do_full_format eq "D_6" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_BF, "BF", $do_RA, "RA", "UI", "UI");
    }
    elsif ( $do_full_format eq "D_7" ) {
	&gen_exec_format4( TOFILE, $do_id, "", "", "TO", "TO", $do_RA, "RA", "SI", "SI");
    }
    elsif ( $do_full_format eq "D_8" ) {
	if ( $do_load ) {
	    &gen_exec_format3( TOFILE, $do_id, $do_FRT, "FRT", $do_RA, "RA", "D", "D");
	}
	else {
	    print "ERROR: full_format $do_full_format MUST be a load\n";
	}
    }
    elsif( $do_full_format eq "D_9" ) {
	if( $do_store ) {
	    &gen_exec_format3( TOFILE, $do_id, $do_FRS, "FRS", $do_RA, "RA", "D", "D");
	}
	else {
	    print "ERROR: full_format $do_full_format MUST be a store\n";	    
	}
    }

    elsif ( $do_full_format eq "DS_1" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", $do_RA, "RA", "DS", "DS");
    }
    elsif ( $do_full_format eq "DS_2" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RS, "RS", $do_RA, "RA", "DS", "DS");
    }

    elsif ( $do_full_format eq "DQ_1" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", $do_RA, "RA", "DQ", "DQ");
    }




    elsif( $do_full_format eq "X_1" ) {
	if( $do_id eq "RFI" ) {
	    print(TOFILE "      OPC_EXEC_RFI();\n");
	}
	else {
	    &gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", $do_RA, "RA", $do_RB, "RB");
	}
    }
    elsif( $do_full_format eq "X_2" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", $do_RA, "RA", "NB", "NB");
    }
    elsif( $do_full_format eq "X_3" ) {
	if ($do_id eq "MTSRIN") {
	    &gen_exec_format4( TOFILE, $do_id, "", "", $do_RS, "RS", $do_RA, "RA", $do_RB, "RB");
	}
	elsif ($do_store || $do_cache) {
	    &gen_exec_format3( TOFILE, $do_id, $do_RS, "RS", $do_RA, "RA", $do_RB, "RB");
	}
	else {
	    &gen_exec_format3( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", $do_RB, "RB");
	}
    }
    elsif( $do_full_format eq "X_4" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RS, "RS", $do_RA, "RA", "NB", "NB");
    }
    elsif( $do_full_format eq "X_5" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "SH", "SH");
    }
    elsif( $do_full_format eq "X_6" ) {
	&gen_exec_format3( TOFILE, $do_id, "", "", $do_RS, "RS", "", "");
    }
    elsif( $do_full_format eq "X_7" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_BF, "BF", $do_RA, "RA", $do_RB, "RB");
    }
    elsif( $do_full_format eq "X_8" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_BF, "BF", $do_FRA, "FRA", $do_FRB, "FRB");
    }
    elsif( $do_full_format eq "X_9" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_BF, "BF", $do_BFA, "BFA", "", "");
    }
    elsif( $do_full_format eq "X_10" ) {
	&gen_exec_format3( TOFILE, $do_id, "", "", $do_BF, "BF", "U", "U");
    }
    elsif( $do_full_format eq "X_11" ) {
	&gen_exec_format4( TOFILE, $do_id, "", "", "TO", "TO", $do_RA, "RA", $do_RB, "RB");
    }
    elsif( $do_full_format eq "X_12" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_FRT, "FRT", $do_RA, "RA", $do_RB, "RB");
    }
    elsif( $do_full_format eq "X_13" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_FRT, "FRT", $do_FRB, "FRB", "", "");
    }
    elsif( $do_full_format eq "X_14" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_FRS, "RS", $do_RA, "RA", $do_RB, "RB");
    }
    elsif( $do_full_format eq "X_15" ) {
	&gen_exec_format3( TOFILE, $do_id, "", "", $do_BT, "BT", "", "");
    }
    elsif( $do_full_format eq "X_16" ) {
	&gen_exec_format3( TOFILE, $do_id, "", "", $do_RS, "RS", "SR", "SR");
    }
    elsif( $do_full_format eq "X_17" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", "SR", "SR", "", "");
    }
    elsif( $do_full_format eq "X_18" ) {
	# Do nothing for the VMX right now
    }
    elsif( $do_full_format eq "X_19" ) {
	# Do nothing for the VMX right now
    }
    elsif( $do_full_format eq "X_20" ) {
	# Do nothing for the VMX right now
    }

    elsif( $do_full_format eq "XL_1" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_BT, "BT", $do_BA, "BA", $do_BB, "BB");
    }
    elsif( $do_full_format eq "XL_3" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_BT, "BT", $do_BFA, "BFA", "", "");
    }
    elsif( $do_full_format eq "XL_4" ) {
	; # Do Nothing
    }
    
    elsif( $do_full_format eq "XFX_1" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", "SPR", "SPR", "", "");
    }
    elsif( $do_full_format eq "XFX_2" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", "TBR", "TBR", "", "");
    }
    elsif( $do_full_format eq "XFX_3" ) {
	&gen_exec_format3( TOFILE, $do_id, "", "", $do_RS, "RS", "FXM", "FXM");
    }
    elsif( $do_full_format eq "XFX_4" ) {
	&gen_exec_format3( TOFILE, $do_id, "", "", $do_RS, "RS", "SPR", "SPR");
    }
    elsif( $do_full_format eq "XFX_5" ) {
	&gen_exec_format3( TOFILE, $do_id, "", "", "XO2", "XO2", "", "");
    }
    
    elsif( $do_full_format eq "XFL_1" ) {
	&gen_exec_format3( TOFILE, $do_id, "FLM", "FLM", $do_FRB, "FRB", "", "");
    }
    
    elsif( $do_full_format eq "XS_1" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "SHw", "SHW");
    }

    elsif( $do_full_format eq "XO_1" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", $do_RA, "RA", $do_RB, "RB");
    }
    elsif( $do_full_format eq "XO_2" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", $do_RA, "RA", $do_RB, "RB");
    }

    elsif( $do_full_format eq "A_1" ) {
	&gen_exec_format4( TOFILE, $do_id, $do_FRT, "FRT", $do_FRA, "FRA", $do_FRC, "FRC", $do_FRB, "FRB");
    }

    elsif( $do_full_format eq "M_1" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", $do_RB, "RB", "MB", "MB", "ME", "ME");
    }
    elsif( $do_full_format eq "M_2" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "SH", "SH", "MB", "MB", "ME", "ME");
    }

    elsif( $do_full_format eq "MD_1" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "SHw", "SHW", "MBw", "MBW");
    }
    elsif( $do_full_format eq "MD_2" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "SHw", "SHW", "MEw", "MEW");
    }

    elsif( $do_full_format eq "MDS_1" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", $do_RB, "RB", "MBw", "MBW");
    }
    elsif( $do_full_format eq "MDS_2" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", $do_RB, "RB", "MEw", "MEW");
    }
    elsif( $do_full_format eq "MDS_3" ) { 
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", "IS", "IS", "IB", "IB", "XBI", "XBI");	
    }
    elsif( $do_full_format eq "MDS_3n" ) { # THE POWERPC Interpretation Here
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", "IS", "IS", "IB", "IB", "CRB", "CBI");
    }
    elsif( $do_full_format eq "MDS_4" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", "IS", "IS", $do_RB, "RB", "XBI", "XBI");
    }
    elsif( $do_full_format eq "MDS_4n" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", "IS", "IS", $do_RB, "RB", "CRB", "CBI");
    }
    elsif( $do_full_format eq "MDS_5" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "IB", "IB", "XBI", "XBI");
    }
    elsif( $do_full_format eq "MDS_5n" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "IB", "IB", "CRB", "CBI");
    }
    elsif( $do_full_format eq "MDS_6" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "IB", "RB", "XBI", "XBI");
    }
    elsif( $do_full_format eq "MDS_6n" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "IB", "RB", "CRB", "CBI");
    }
    elsif( $do_full_format eq "TX_1" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_TO, "TO", "UI", "UI", $do_XBI, "XBI");
    }

    elsif( $do_full_format eq "VA_1" ) {
	# Do nothing for the VMX right now
    }
    elsif( $do_full_format eq "VA_2" ) {
	# Do nothing for the VMX right now
    }
    elsif( $do_full_format eq "VX_1" ) {
	# Do nothing for the VMX right now
    }
    elsif( $do_full_format eq "VX_2" ) {
	# Do nothing for the VMX right now
    }
    elsif( $do_full_format eq "VX_3" ) {
	# Do nothing for the VMX right now
    }
    elsif( $do_full_format eq "VXR_1" ) {
	# Do nothing for the VMX right now
    }

    else {
	print "Opcode_exec:: Can't handle execution case for $do_id format $do_full_format\n";
    }

    print(TOFILE "      break;\n");
    print(TOFILE "    }\n");

}

sub DoFinalExecCode {
    local(*TOFILE) = $_[0];
}





sub DoInitialExecXcoffCode {
    local(*TOFILE) = $_[0];
}


sub DoExecXcoffCodeForThisInstruction {
    local (*TOFILE, $name, $format, $args ) = @_;

    if( $do_ppc32_name eq "" ) {
	#print "Skipping instruction $do_id (not in 32-bit ppc set)\n";
	return;
    }

    printf(TOFILE "  case OPC_%s: {\n", $do_id);
    if( $do_float ) { 
	print(TOFILE "    if (reg_MSR_FP(proc_state_MSR(state)) == 0) {\n");
	print(TOFILE "      DEBUG( fprintf(mystderr, \"\\n***  Taking an illegal instruction Program interrupt...\\n\\n\") );\n");
	print(TOFILE "      proc_state_SRR0(state) = (unsigned)proc_state_PC(state); \n");
	print(TOFILE "      proc_state_SRR1(state) = (0x00080000 | (proc_state_MSR(state) & 0x87c0ffff)); \n");
	print(TOFILE "      proc_state_MSR(state) = ((proc_state_MSR(state) & 0x00011040) | reg_MSR_ILE(proc_state_MSR(state)));\n"); 
	print(TOFILE "      proc_state_PC(state) = (reg_MSR_IP(proc_state_MSR(state)) == 0) ? 0x00800 : 0xfff00800;\n");
	print(TOFILE "      break;\n");
	print(TOFILE "    }\n");
    }

    if(( $do_full_format eq "XL_2" ) || 
	  ( $do_full_format eq "B_1"  ) ) {
	print(TOFILE "      PC = proc_state_PC(state);\n");
	print(TOFILE "      CTR = proc_state_CTR(state);\n");
	print(TOFILE "      R0 = OPC_DEC_BO(iword);\n");
	print(TOFILE "      T0 = R0;\n");
	print(TOFILE "      BO = R0;\n");
	print(TOFILE "      R1 = OPC_DEC_BI(iword);\n");
	print(TOFILE "      T1 = proc_state_CRB(state, R1);\n");

	if( $do_full_format eq "B_1" ) {
	    print(TOFILE "      R2 = OPC_DEC_BD(iword);\n");
	    if( $do_AA ) {
		print(TOFILE "      T2 = 4*R2;\n");
	    }
	    else {
		print(TOFILE "      T2 = PC + 4*R2;\n");
	    }
	}
	elsif( ( $do_id eq "BCCTR" ) || ( $do_id eq "BCCTRL" ) ) {
	    print(TOFILE "      T2 = CTR;\n");
	}
	else {
	    print(TOFILE "      T2 = proc_state_LR(state);\n");
	}

	print(TOFILE "\n");
	print(TOFILE "      flag = 1;\n");
	print(TOFILE "      if( (BO & 0x4) == 0 ) {\n");
	print(TOFILE "        CTR = CTR - 1;\n");
	print(TOFILE "        proc_state_CTR(state) = CTR;\n");
	print(TOFILE "        flag = (CTR != 0) ^ ((BO & 0x2)!=0);\n");
	print(TOFILE "      }\n");

	print(TOFILE "\n");
	print(TOFILE "      flag &= ((BO & 0x10)!=0) | (((BO & 0x8)>>3) == T1);\n");

	print(TOFILE "      if( flag ) {\n");
	print(TOFILE "        proc_state_PC(state) = T2;\n");
	print(TOFILE "      }\n");
	print(TOFILE "      else {\n");
	print(TOFILE "        T2 = PC + 4;\n");
	print(TOFILE "        proc_state_PC(state) = T2;\n");
	print(TOFILE "      }\n");

	if( $do_LK ) {
	    print(TOFILE "      LR = PC + 4;\n");
	    print(TOFILE "      proc_state_LR(state) = LR;\n");
	    print(TOFILE "      OPC_EXEC_SET_LR(PC, LR);\n");
	}
    }
    elsif( $do_full_format eq "I_1" ) {
	print(TOFILE "      PC = proc_state_PC(state);\n");
	print(TOFILE "      R0 = OPC_DEC_LI(iword);\n");
	print(TOFILE "      T0 = 4*R0;\n");
	
	if( $do_AA ) {
	    print(TOFILE "      proc_state_PC(state) = T0;\n");
	}
	else {
	    print(TOFILE "      T1 = T0 + PC;\n");
	    print(TOFILE "      proc_state_PC(state) = T1;\n");
	}
	if( $do_LK ) {
	    print(TOFILE "      LR = PC + 4;\n");
	    print(TOFILE "      proc_state_LR(state) = LR;\n");
	    print(TOFILE "      OPC_EXEC_SET_LR(PC, LR);\n");
	}
    }
    elsif (( $do_full_format eq "SC_1" ) ||
	   ( $do_full_format eq "SC_2" ) ) { 
	print(TOFILE "      if (proc_state_GPR(state, 2) != the_exit_code_value) {\n");
	print(TOFILE "        copy_memory_to_real();\n");
	print(TOFILE "        do_system_call_routine(state);\n");
	print(TOFILE "        copy_real_to_memory();\n");
	print(TOFILE "        proc_state_PC(state) = proc_state_LR(state);\n");
	print(TOFILE "      }\n");
	print(TOFILE "      else {\n");
	print(TOFILE "        proc_state_PC(state) = 0;\n");
	print(TOFILE "      }\n");
    }

    elsif ( $do_full_format eq "D_1" ) {
	if ( $do_load ) {
	    &gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", $do_RA, "RA", "D", "D");
	}
	else {
	    print "ERROR: full_format $do_full_format MUST be a load\n";
	}
    }
    elsif ( $do_full_format eq "D_2" ) {
	if ( $do_load ) {
	    print "ERROR: full_format $do_full_format MUST NOT be a load\n";
	}
	else {
	    &gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", $do_RA, "RA", "SI", "SI");
	}
    }
    elsif( $do_full_format eq "D_3" ) {
	if( $do_store ) {
	    &gen_exec_format3( TOFILE, $do_id, $do_RS, "RS", $do_RA, "RA", "D", "D");
	}
	else {
	    print "ERROR: full_format $do_full_format MUST be a store\n";
	}
    }
    elsif( $do_full_format eq "D_4" ) {
	if( $do_store ) {
	    print "ERROR: full_format $do_full_format MUST NOT be a store\n";
	}
	else {
	    &gen_exec_format3( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "UI", "UI");
	}
    }
    elsif ( $do_full_format eq "D_5" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_BF, "BF", $do_RA, "RA", "SI", "SI");
    }
    elsif ( $do_full_format eq "D_6" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_BF, "BF", $do_RA, "RA", "UI", "UI");
    }
    elsif ( $do_full_format eq "D_7" ) {
	&gen_exec_format4( TOFILE, $do_id, "", "", "TO", "TO", $do_RA, "RA", "SI", "SI");
    }
    elsif ( $do_full_format eq "D_8" ) {
	if ( $do_load ) {
	    &gen_exec_format3( TOFILE, $do_id, $do_FRT, "FRT", $do_RA, "RA", "D", "D");
	}
	else {
	    print "ERROR: full_format $do_full_format MUST be a load\n";
	}
    }
    elsif( $do_full_format eq "D_9" ) {
	if( $do_store ) {
	    &gen_exec_format3( TOFILE, $do_id, $do_FRS, "FRS", $do_RA, "RA", "D", "D");
	}
	else {
	    print "ERROR: full_format $do_full_format MUST be a store\n";	    
	}
    }

    elsif ( $do_full_format eq "DS_1" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", $do_RA, "RA", "DS", "DS");
    }
    elsif ( $do_full_format eq "DS_2" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RS, "RS", $do_RA, "RA", "DS", "DS");
    }

    elsif ( $do_full_format eq "DQ_1" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", $do_RA, "RA", "DQ", "DQ");
    }




    elsif( $do_full_format eq "X_1" ) {
	if( $do_id == "RFI" ) {
	    print(TOFILE "      OPC_EXEC_RFI();\n");
	}
	else {
	    &gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", $do_RA, "RA", $do_RB, "RB");
	}
    }
    elsif( $do_full_format eq "X_2" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", $do_RA, "RA", "NB", "NB");
    }
    elsif( $do_full_format eq "X_3" ) {
	if ($do_id eq "MTSRIN") {
	    &gen_exec_format4( TOFILE, $do_id, "", "", $do_RS, "RS", $do_RA, "RA", $do_RB, "RB");
	}
	elsif ($do_store || $do_cache) {
	    &gen_exec_format3( TOFILE, $do_id, $do_RS, "RS", $do_RA, "RA", $do_RB, "RB");
	}
	else {
	    &gen_exec_format3( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", $do_RB, "RB");
	}
    }
    elsif( $do_full_format eq "X_4" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RS, "RS", $do_RA, "RA", "NB", "NB");
    }
    elsif( $do_full_format eq "X_5" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "SH", "SH");
    }
    elsif( $do_full_format eq "X_6" ) {
	&gen_exec_format3( TOFILE, $do_id, "", "", $do_RS, "RS", "", "");
    }
    elsif( $do_full_format eq "X_7" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_BF, "BF", $do_RA, "RA", $do_RB, "RB");
    }
    elsif( $do_full_format eq "X_8" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_BF, "BF", $do_FRA, "FRA", $do_FRB, "FRB");
    }
    elsif( $do_full_format eq "X_9" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_BF, "BF", $do_BFA, "BFA", "", "");
    }
    elsif( $do_full_format eq "X_10" ) {
	&gen_exec_format3( TOFILE, $do_id, "", "", $do_BF, "BF", "U", "U");
    }
    elsif( $do_full_format eq "X_11" ) {
	&gen_exec_format4( TOFILE, $do_id, "", "", "TO", "TO", $do_RA, "RA", $do_RB, "RB");
    }
    elsif( $do_full_format eq "X_12" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_FRT, "FRT", $do_RA, "RA", $do_RB, "RB");
    }
    elsif( $do_full_format eq "X_13" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_FRT, "FRT", $do_FRB, "FRB", "", "");
    }
    elsif( $do_full_format eq "X_14" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_FRS, "RS", $do_RA, "RA", $do_RB, "RB");
    }
    elsif( $do_full_format eq "X_15" ) {
	&gen_exec_format3( TOFILE, $do_id, "", "", $do_BT, "BT", "", "");
    }
    elsif( $do_full_format eq "X_16" ) {
	&gen_exec_format3( TOFILE, $do_id, "", "", $do_RS, "RS", "SR", "SR");
    }
    elsif( $do_full_format eq "X_17" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", "SR", "SR", "", "");
    }
    elsif( $do_full_format eq "X_18" ) {
	# Do nothing for the VMX right now
    }
    elsif( $do_full_format eq "X_19" ) {
	# Do nothing for the VMX right now
    }
    elsif( $do_full_format eq "X_20" ) {
	# Do nothing for the VMX right now
    }

    elsif( $do_full_format eq "XL_1" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_BT, "BT", $do_BA, "BA", $do_BB, "BB");
    }
    elsif( $do_full_format eq "XL_3" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_BT, "BT", $do_BFA, "BFA", "", "");
    }
    elsif( $do_full_format eq "XL_4" ) {
	; # Do Nothing
    }
    
    elsif( $do_full_format eq "XFX_1" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", "SPR", "SPR", "", "");
    }
    elsif( $do_full_format eq "XFX_2" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", "TBR", "TBR", "", "");
    }
    elsif( $do_full_format eq "XFX_3" ) {
	&gen_exec_format3( TOFILE, $do_id, "", "", $do_RS, "RS", "FXM", "FXM");
    }
    elsif( $do_full_format eq "XFX_4" ) {
	&gen_exec_format3( TOFILE, $do_id, "", "", $do_RS, "RS", "SPR", "SPR");
    }
    elsif( $do_full_format eq "XFX_5" ) {
	&gen_exec_format3( TOFILE, $do_id, "", "", "XO2", "XO2", "", "");
    }
    
    elsif( $do_full_format eq "XFL_1" ) {
	&gen_exec_format3( TOFILE, $do_id, "FLM", "FLM", $do_FRB, "FRB", "", "");
    }
    
    elsif( $do_full_format eq "XS_1" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "SHw", "SHW");
    }

    elsif( $do_full_format eq "XO_1" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", $do_RA, "RA", $do_RB, "RB");
    }
    elsif( $do_full_format eq "XO_2" ) {
	&gen_exec_format3( TOFILE, $do_id, $do_RT, "RT", $do_RA, "RA", $do_RB, "RB");
    }

    elsif( $do_full_format eq "A_1" ) {
	&gen_exec_format4( TOFILE, $do_id, $do_FRT, "FRT", $do_FRA, "FRA", $do_FRC, "FRC", $do_FRB, "FRB");
    }

    elsif( $do_full_format eq "M_1" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", $do_RB, "RB", "MB", "MB", "ME", "ME");
    }
    elsif( $do_full_format eq "M_2" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "SH", "SH", "MB", "MB", "ME", "ME");
    }

    elsif( $do_full_format eq "MD_1" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "SHw", "SHW", "MBw", "MBW");
    }
    elsif( $do_full_format eq "MD_2" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "SHw", "SHW", "MEw", "MEW");
    }

    elsif( $do_full_format eq "MDS_1" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", $do_RB, "RB", "MBw", "MBW");
    }
    elsif( $do_full_format eq "MDS_2" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", $do_RB, "RB", "MEw", "MEW");
    }
    elsif( $do_full_format eq "MDS_3" ) { 
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", "IS", "IS", "IB", "IB", "XBI", "XBI");
    }
    elsif( $do_full_format eq "MDS_3n" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", "IS", "IS", "IB", "IB", "CRB", "CBI");
    }
    elsif( $do_full_format eq "MDS_4" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", "IS", "IS", $do_RB, "RB", "XBI", "XBI");
    }
    elsif( $do_full_format eq "MDS_4n" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", "IS", "IS", $do_RB, "RB", "CRB", "CBI");
    }
    elsif( $do_full_format eq "MDS_5" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "IB", "IB", "XBI", "XBI");
    }
    elsif( $do_full_format eq "MDS_5n" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "IB", "IB", "CRB", "CBI");
    }
    elsif( $do_full_format eq "MDS_6" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "IB", "RB", "XBI", "XBI");
    }
    elsif( $do_full_format eq "MDS_6n" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_RA, "RA", $do_RS, "RS", "IB", "RB", "CRB", "CBI");
    }

    elsif( $do_full_format eq "TX_1" ) {
	&gen_exec_format5( TOFILE, $do_id, $do_TO, "TO", "UI", "UI", $do_XBI, "XBI");
    }

    elsif( $do_full_format eq "VA_1" ) {
	# Do nothing for the VMX right now
    }
    elsif( $do_full_format eq "VA_2" ) {
	# Do nothing for the VMX right now
    }
    elsif( $do_full_format eq "VX_1" ) {
	# Do nothing for the VMX right now
    }
    elsif( $do_full_format eq "VX_2" ) {
	# Do nothing for the VMX right now
    }
    elsif( $do_full_format eq "VX_3" ) {
	# Do nothing for the VMX right now
    }
    elsif( $do_full_format eq "VXR_1" ) {
	# Do nothing for the VMX right now
    }

    else {
	print "Opcode_exec (2) :: Can't handle execution xcoff case for $do_id format $do_full_format\n";
    }

    print(TOFILE "      break;\n");
    print(TOFILE "    }\n");
}


sub DoFinalExecXcoffCode {
    local(*TOFILE) = $_[0];
}
