#
#    Value                           1 1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 2 2 2 2 3  3       
#                0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0  1              
#     I_1:     |   opcode  |                       LI                      |AA|LK|
#
#     B_1:     |   opcode  |    BO   |    BI   |            BD             |AA|LK|
#
#    SC_1:     |   opcode  |   ///   |   ///   |           ///             |XO|LK|
#    SC_2:     |   opcode  |   ///   |   ///   |  ///  |   LEV       | /// |XO|LK|
#
#     D_1:     |   opcode  |    RT   |    RA   |                D                |
#     D_2:     |   opcode  |    RT   |    RA   |               SI                |
#     D_3:     |   opcode  |    RS   |    RA   |                D                |
#     D_4:     |   opcode  |    RS   |    RA   |               SI                |
#     D_5:     |   opcode  |  BF |/|L|    RA   |               SI                |
#     D_6:     |   opcode  |  BF |/|L|    RA   |               UI                |
#     D_7:     |   opcode  |    TO   |    RA   |               SI                |
#     D_8:     |   opcode  |   FRT   |    RA   |                D                |
#     D_9:     |   opcode  |   FRS   |    RA   |                D                |
#
#    DS_1:     |   opcode  |    RT   |    RA   |               DS          | XO  |
#    DS_2:     |   opcode  |    RS   |    RA   |               DS          | XO  |
#
#    DQ_1:     |   opcode  |    RT   |    RA   |          DQ           |    PT  |
#
#    X_1:      |   opcode  |    RT   |    RA   |    RB   |         XO         |//|
#    X_2:      |   opcode  |    RT   |    RA   |    NB   |         XO         |//|
#    X_3:      |   opcode  |    RS   |    RA   |    RB   |         XO         |Rc|
#    X_4:      |   opcode  |    RS   |    RA   |    NB   |         XO         |//|
#    X_5:      |   opcode  |    RS   |    RA   |    SH   |         XO         |Rc|
#    X_6:      |   opcode  |    RS   |   ///   |   ///   |         XO         |//|
#    X_7:      |   opcode  |  BF |/|L|    RA   |    RB   |         XO         |//|
#    X_8:      |   opcode  |  BF |///|   FRA   |   FRB   |         XO         |//|
#    X_9:      |   opcode  |  BF |///| BFA |///|   ///   |         XO         |//|
#    X_10:     |   opcode  |  BF |///|   ///   |    U  |/|         XO         |Rc|
#    X_11:     |   opcode  |    TO   |    RA   |    RB   |         XO         |//|
#    X_12:     |   opcode  |   FRT   |    RA   |    RB   |         XO         |//|
#    X_13:     |   opcode  |   FRT   |   ///   |   FRB   |         XO         |Rc|
#    X_14:     |   opcode  |   FRS   |    RA   |    RB   |         XO         |//|
#    X_15:     |   opcode  |    BT   |   ///   |   ///   |         XO         |Rc|
#    X_16:     |   opcode  |    RS   |/|   SR  |   ///   |         XO         |//|
#    X_17:     |   opcode  |    RT   |/|   SR  |   ///   |         XO         |//|
#    X_18:     |   opcode  |T|//|STRM|    RA   |    RB   |         XO         |//|
#    X_19:     |   opcode  |    VS   |    RA   |    RB   |         XO         |Rc|
#    X_20:     |   opcode  |    VT   |    RA   |    RB   |           XO        |0|
#
#    XL_1:     |   opcode  |    BT   |    BA   |    BB   |         XO         |//|
#    XL_2:     |   opcode  |    BO   |    RI   |   ///   |         XO         |LK|
#    XL_3:     |   opcode  |  BF |///| BFA |///|   ///   |         XO         |//|
#    XL_4:     |   opcode  |   ///   |   ///   |   ///   |         XO         |//|
#
#   XFX_1:     |   opcode  |    RT   |        SPR        |         XO         |//|
#   XFX_2:     |   opcode  |    RT   |        TBR        |         XO         |//|
#   XFX_3:     |   opcode  |    RS   |/|      FXM      |/|         XO         |//|
#   XFX_4:     |   opcode  |    RS   |        SPR        |         XO         |//|
#   XFX_5:     |   opcode  |   ///   |        XO2        |         XO         |//|
#
#   XFL_1:     |   opcode  |/|      FLM      |/|   FRB   |         XO         |Rc|
#
#    XS_1:     |   opcode  |    RS   |    RA   |    SH   |         XO      |SH|Rc|
#
#    XO_1:     |   opcode  |    RT   |    RA   |    RB   |OE|      XO         |Rc|
#    XO_2:     |   opcode  |    RT   |    RA   |    RB   |//|      XO         |Rc|
#
#     A_1:     |   opcode  |   FRT   |   FRA   |   FRB   |   FRC   |    XO    |Rc| 
#
#     M_1:     |   opcode  |    RS   |    RA   |    RB   |    MB   |    ME    |Rc|
#     M_2:     |   opcode  |    RS   |    RA   |    SH   |    MB   |    ME    |Rc|
#
#    MD_1:     |   opcode  |    RS   |    RA   |    SH   |    MB   |   XO  |SH|Rc|
#    MD_2:     |   opcode  |    RS   |    RA   |    SH   |    ME   |   XO  |SH|Rc|
#
#   MDS_1:     |   opcode  |    RS   |    RA   |    RB   |    MB   |     XO   |Rc|
#   MDS_2:     |   opcode  |    RS   |    RA   |    RB   |    ME   |     XO   |Rc|
#   MDS_3:     |   opcode  |    IS   |    RA   |    IB   | XBI  |//|     XO   |Rc|
#   MDS_4:     |   opcode  |    IS   |    RA   |    RB   | XBI  |//|     XO   |Rc|
#   MDS_5:     |   opcode  |    RS   |    RA   |    IB   | XBI  |//|     XO   |Rc|
#   MDS_6:     |   opcode  |    RS   |    RA   |    RB   | XBI  |//|     XO   |Rc|
#   MDS_3n:    |   opcode  |    IS   |    RA   |    IB   |  CBI  |/|     XO   |Rc|
#   MDS_4n:    |   opcode  |    IS   |    RA   |    RB   |  CBI  |/|     XO   |Rc|
#   MDS_5n:    |   opcode  |    RS   |    RA   |    IB   |  CBI  |/|     XO   |Rc|
#   MDS_6n:    |   opcode  |    RS   |    RA   |    RB   |  CBI  |/|     XO   |Rc|
#
#    TX_1:     |   opcode  |    TO   |        UI         |  XBI |       XO    |Rc|
#
#    VA_1:     |   opcode  |    VT   |    VA   |    VB   |    VC   |     XO      |
#    VA_2:     |   opcode  |    VT   |    VA   |    VB   |/|   SH  |     XO      |
#
#    VX_1:     |   opcode  |    VT   |    VA   |    VB   |           XO          |
#    VX_2:     |   opcode  |    VT   |   UIMM  |    VB   |           XO          |
#    VX_3:     |   opcode  |    VT   |   SIMM  |    VB   |           XO          |
#
#    VXR_1:    |   opcode  |    VT   |    VA   |    VB   |Rc|        XO          |
#


sub DoInitialGenerateSimIword {
    local (*TOFILE) = $_[0];
}

#* Generate OPC_ASM assembly-code macros *#
sub DoGenerateSimIwordForThisInstruction {
    local (*TOFILE) = $_[0];

    printf TOFILE ("case OPC_%-8s :  iword = ", $do_id);

    if( $do_full_format eq "I_1" ) {
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_AA, "<<1) | ", $do_LK;
	print TOFILE " | OPC_ENC_LI(ops[0]))";
    }	
    elsif( $do_full_format eq "B_1" ) {
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_AA, "<<1) | ", $do_LK;
	print TOFILE " | OPC_ENC_BO(ops[0]) | OPC_ENC_BI(ops[1]) | OPC_ENC_BD(ops[2]))";
    }
    elsif ($do_full_format eq "SC_1") {
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_LK, " )\n";
    }
    elsif ($do_full_format eq "SC_2") {
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_LK, " | OPC_ENC_LEV(ops[0]))";
    }

    elsif ($do_format eq "D_form") {
	if ($do_full_format eq "D_1") {
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_RT(ops[0]) | OPC_ENC_D(ops[1]) | OPC_ENC_RA(ops[2]))";
	}
	elsif ($do_full_format eq "D_2") {
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_RT(ops[0]) | OPC_ENC_RA(ops[1]) | OPC_ENC_SI(ops[2]))";      
	}
	elsif ($do_full_format eq "D_3") {
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_RS(ops[0]) | OPC_ENC_D(ops[1]) | OPC_ENC_RA(ops[2]))";
	}
	elsif ($do_full_format eq "D_4") {
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_RS(ops[1]) | OPC_ENC_SI(ops[2]))";
	}
	elsif ($do_full_format eq "D_5") {
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_BF(ops[0]) | OPC_ENC_L(ops[2]) | OPC_ENC_RA(ops[2]) | OPC_ENC_SI(ops[3]))";
	}
	elsif ($do_full_format eq "D_6") {
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_BF(ops[0]) | OPC_ENC_L(ops[2]) | OPC_ENC_RA(ops[2]) | OPC_ENC_UI(ops[3]))";
	}
	elsif ($do_full_format eq "D_7") {
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_TO(ops[0]) | OPC_ENC_RA(ops[1]) | OPC_ENC_SI(ops[2]))";      
	}
	elsif ($do_full_format eq "D_8") {
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_FRT(ops[0]) | OPC_ENC_D(ops[1]) | OPC_ENC_RA(ops[2]))";
	}
	elsif ($do_full_format eq "D_9") {
	    print TOFILE "   ( (", $do_opcode, "<<26)";
	    print TOFILE " | OPC_ENC_FRS(ops[0]) | OPC_ENC_D(ops[1]) | OPC_ENC_RA(ops[2]))";
	}
    }

    elsif ($do_full_format eq "DS_1") {
	print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
	print TOFILE " | OPC_ENC_RT(ops[0]) | OPC_ENC_DS(ops[1]) | OPC_ENC_RA(ops[2]))";
    }
    elsif ($do_full_format eq "DS_2") {
	print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
	print TOFILE " | OPC_ENC_RS(ops[0]) | OPC_ENC_DS(ops[1]) | OPC_ENC_RA(ops[2]))";
    }
    elsif ($do_full_format eq "DQ_1") {
	print TOFILE "   ( (", $do_opcode, "<<26)";
	print TOFILE " | OPC_ENC_RT(ops[0]) | OPC_ENC_RA(ops[1]) | OPC_ENC_DQ(ops[2]) | OPC_ENC_PT(ops[3]))";
    }
    
    elsif ($do_format eq "X_form") {
	if( $do_full_format eq "X_1" ) {
	    if ( ($do_RT eq "0") && ($do_RA eq "0") && ($do_RB eq "0") ) {
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " )\n";	
	    }
	    elsif ( ($do_RA eq "0") && ($do_RB eq "0") ) {
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_RT(ops[0]))";
	    }
	    elsif ( ($do_RT eq "0") && ($do_RA eq "0") ) {
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_RB(ops[0]))";
	    }
	    elsif ($do_RT eq "0") {
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_RB(ops[1]))";
	    }
	    elsif ($do_RA eq "0") {
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_RT(ops[0]) | OPC_ENC_RB(ops[1]))";
	    }
	    elsif ($do_RB eq "0") {
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_RT(ops[0]) | OPC_ENC_RA(ops[1]))";
	    }
	    else {
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_RT(ops[0]) | OPC_ENC_RA(ops[1]) | OPC_ENC_RB(ops[2]))";
	    }
	}
	elsif ($do_full_format eq "X_2") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RT(ops[0]) | OPC_ENC_RA(ops[1]) | OPC_ENC_NB(ops[2]))";	
	}
	elsif ($do_full_format eq "X_3") {
	    if ($do_store) {
		if ( ($do_RA ne "0") && ($do_RA ne "0") && ($do_RB ne "0") ) {
		    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		    print TOFILE " | OPC_ENC_RS(ops[0]) | OPC_ENC_RA(ops[1]) | OPC_ENC_RB(ops[2]))";	
		}
		else { 
		    printf("Can a store indexed (%s) have RS = %s, RA = %s and RB = %s?\n", $do_id, $do_RS, $do_RA, $do_RB);
		}
	    }
	    else {
		if ( ($do_RS eq "0") && ($do_RB eq "0") ) { 
		    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		    print TOFILE " | OPC_ENC_RA(ops[0]))";	
		}
		elsif ($do_RA eq "0") { 
		    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		    print TOFILE " | OPC_ENC_RS(ops[0]) | OPC_ENC_RB(ops[1]))";	
		}
		elsif ($do_RB eq "0") { 
		    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		    print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_RS(ops[1]))";	
		}
		else {
		    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		    print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_RS(ops[1]) | OPC_ENC_RB(ops[2]))";	
		}
	    }
	}
	elsif ($do_full_format eq "X_4") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_RS(ops[1]) | OPC_ENC_NB(ops[2]))";	
	}
	elsif ($do_full_format eq "X_5") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_RS(ops[1]) | OPC_ENC_SH(ops[2]))";	
	}
	elsif ($do_full_format eq "X_6") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RS(ops[0]))";	
	}
	elsif ($do_full_format eq "X_7") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_BF(ops[0]) | OPC_ENC_L(ops[2]) | OPC_ENC_RA(ops[2]) | OPC_ENC_RB(ops[2]))";	
	}
	elsif ($do_full_format eq "X_8") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_BF(ops[0]) | OPC_ENC_FRA(ops[1]) | OPC_ENC_FRB(ops[2]))";	
	}
	elsif ($do_full_format eq "X_9") {
	    if ($do_BFA eq "0") {
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_BF(ops[0]))";	
	    }
	    else {
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_BF(ops[0]) | OPC_ENC_BFA(ops[1]))";	
	    }
	}
	elsif ($do_full_format eq "X_10") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_BF(ops[0]) | OPC_ENC_U(ops[1]))";	
	}
	elsif ($do_full_format eq "X_11") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_TO(ops[0]) | OPC_ENC_RA(ops[1]) | OPC_ENC_NB(ops[2]))";	
	}
	elsif( $do_full_format eq "X_12" ) {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_FRT(ops[0]) | OPC_ENC_RA(ops[1]) | OPC_ENC_RB(ops[2]))";	
	}
	elsif ($do_full_format eq "X_13") {
	    if ($do_FRB eq "0") {
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_FRT(ops[0]))";
	    }
	    else {
		print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
		print TOFILE " | OPC_ENC_FRT(ops[0]) | OPC_ENC_FRB(ops[1]))";	
	    }
	}
	elsif ($do_full_format eq "X_14") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_FRS(ops[0]) | OPC_ENC_RA(ops[1]) | OPC_ENC_RB(ops[2]))";	
	}
	elsif ($do_full_format eq "X_15") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_BT(ops[0]))";	
	}
	elsif ($do_full_format eq "X_16") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RS(ops[0]) | OPC_ENC_SR(ops[1]))";	
	}
	elsif ($do_full_format eq "X_17") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RT(ops[0]) | OPC_ENC_SR(ops[1]))";	
	}
	elsif ($do_full_format eq "X_18") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE ") /* NOT IMPLEMENTED FOR SIM */";	
	}
	elsif ($do_full_format eq "X_19") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE ") /* NOT IMPLEMENTED FOR SIM */";	
	}
	elsif ($do_full_format eq "X_20") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
	    print TOFILE ") /* NOT IMPLEMENTED FOR SIM */";	
	}
    }
    elsif ($do_format eq "XL_form") {
	if ($do_full_format eq "XL_1") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) ";
	    print TOFILE " | OPC_ENC_BT(ops[0]) | OPC_ENC_BA(ops[1]) | OPC_ENC_BB(ops[2]))";      
	}
	elsif ($do_full_format eq "XL_2") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_LK;
	    print TOFILE " | OPC_ENC_BO(ops[0]) | OPC_ENC_BI(ops[1]))";      
	}
	elsif ($do_full_format eq "XL_3") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) ";
	    print TOFILE " | OPC_ENC_BF(ops[0]) | OPC_ENC_BFA(ops[1]))";      
	}
	elsif ($do_full_format eq "XL_4") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_LK, " )\n";
	}
    }
    elsif ($do_format eq "XFX_form") {
	if ($do_full_format eq "XFX_1") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1)", $do_Rc;
	    print TOFILE " | OPC_ENC_RT(ops[0]) | OPC_ENC_SPR(ops[1]))";
	}
	elsif ($do_full_format eq "XFX_2") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1)", $do_Rc;
	    print TOFILE " | OPC_ENC_RT(ops[0]) | OPC_ENC_TBR(ops[1]))";
	}
	elsif ($do_full_format eq "XFX_3") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1)", $do_Rc;
	    print TOFILE " | OPC_ENC_FXM(ops[0]) | OPC_ENC_RS(ops[1]))";
	}
	elsif ($do_full_format eq "XFX_4") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1)", $do_Rc;
	    print TOFILE " | OPC_ENC_SPR(ops[0]) | OPC_ENC_RS(ops[1]))";
	}
	elsif ($do_full_format eq "XFX_5") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1)", $do_Rc;
	    print TOFILE " | OPC_ENC_XO2(32))";
	}
    }

    elsif ($do_full_format eq "XFL_1") {
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	print TOFILE " | OPC_ENC_FLM(ops[0]) | OPC_ENC_RB(ops[1]))";
    }

    elsif ($do_full_format eq "XS_1") {
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<2) | ", $do_Rc;
	print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_RS(ops[1]) | OPC_ENC_SHSH(ops[2]))";
    }

    elsif ($do_full_format eq "XO_1") {
	if( $do_RB eq "0") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_OE, "<<10) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RT(ops[0]) | OPC_ENC_RA(ops[1]))";
	}
	else {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_OE, "<<10) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RT(ops[0]) | OPC_ENC_RA(ops[1]) | OPC_ENC_RB(ops[2]))";
	}
    }
    elsif ($do_full_format eq "XO_2") {
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_OE, "<<10) | (", $do_extended_op, "<<1) | ", $do_Rc;
	print TOFILE " | OPC_ENC_RT(ops[0]) | OPC_ENC_RA(ops[1]) | OPC_ENC_RB(ops[2]))";
    }

    elsif( $do_full_format eq "A_1" ) {
	if( ( $do_FRA eq "0" ) && ( $do_FRC eq "0" ) ) {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_FRT(ops[0]) | OPC_ENC_FRB(ops[1]))";
	}
	elsif( $do_FRB eq "0") {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_FRT(ops[0]) | OPC_ENC_FRA(ops[1]) | OPC_ENC_FRC(ops[2]))";
	}
	elsif( $do_FRC eq "0" ) {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_FRT(ops[0]) | OPC_ENC_FRA(ops[1]) | OPC_ENC_FRB(ops[2]))";
	}
	else {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_FRT(ops[0]) | OPC_ENC_FRA(ops[1]) | OPC_ENC_FRC(ops[2]) | OPC_ENC_FRB(ops[3]))";
	}
    }

    elsif( $do_full_format eq "M_1" ) {
	print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_Rc;
	print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_RS(ops[1]) | OPC_ENC_RB(ops[2]) | OPC_ENC_MB(ops[3]) | OPC_ENC_ME(ops[4]))";
    }
    elsif( $do_full_format eq "M_2" ) {
	print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_Rc;
	print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_RS(ops[1]) | OPC_ENC_SH(ops[2]) | OPC_ENC_MB(ops[3]) | OPC_ENC_ME(ops[4]))";
    }

    elsif( $do_full_format eq "MD_1" ) {
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<2) | ", $do_Rc;
	print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_RS(ops[1]) | OPC_ENC_SHSH(ops[2]) | OPC_ENC_MBW(ops[3]))";
    }
    elsif( $do_full_format eq "MD_2" ) {
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<2) | ", $do_Rc;
	print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_RS(ops[1]) | OPC_ENC_SHSH(ops[2]) | OPC_ENC_MEW(ops[3]))";
    }
    
    elsif( $do_format eq "MDS_form" ) {
	if( $do_full_format eq "MDS_1" ) {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_RS(ops[1]) | OPC_ENC_RB(ops[2]) | OPC_ENC_MBW(ops[3]))";
	}
	elsif( $do_full_format eq "MDS_2" ) {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_RS(ops[1]) | OPC_ENC_RB(ops[2]) | OPC_ENC_MEW(ops[3]))";
	}
	elsif( $do_full_format eq "MDS_3" ) {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_IS(ops[1]) | OPC_ENC_IB(ops[2]) | OPC_ENC_XBI(ops[3]))";
	}
	elsif( $do_full_format eq "MDS_3n" ) {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_IS(ops[1]) | OPC_ENC_IB(ops[2]) | OPC_ENC_CBI(ops[3]))";
	}
	elsif( $do_full_format eq "MDS_4" ) {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_IS(ops[1]) | OPC_ENC_RB(ops[2]) | OPC_ENC_XBI(ops[3]))";
	}
	elsif( $do_full_format eq "MDS_4n" ) {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_IS(ops[1]) | OPC_ENC_RB(ops[2]) | OPC_ENC_CBI(ops[3]))";
	}
	elsif( $do_full_format eq "MDS_5" ) {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_RS(ops[1]) | OPC_ENC_IB(ops[2]) | OPC_ENC_XBI(ops[3]))";
	}
	elsif( $do_full_format eq "MDS_5n" ) {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_RS(ops[1]) | OPC_ENC_IB(ops[2]) | OPC_ENC_CBI(ops[3]))";
	}
	elsif( $do_full_format eq "MDS_6" ) {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_RS(ops[1]) | OPC_ENC_RB(ops[2]) | OPC_ENC_XBI(ops[3]))";
	}
	elsif( $do_full_format eq "MDS_6n" ) {
	    print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	    print TOFILE " | OPC_ENC_RA(ops[0]) | OPC_ENC_RS(ops[1]) | OPC_ENC_RB(ops[2]) | OPC_ENC_CBI(ops[3]))";
	}
    }
    
    elsif( $do_full_format eq "TX_1" ) {
	print TOFILE "   ( (", $do_opcode, "<<26) | (", $do_extended_op, "<<1) | ", $do_Rc;
	print TOFILE " | OPC_ENC_TO(ops[0]) | OPC_ENC_UIL(ops[1]) | OPC_ENC_XBI(ops[2]))";
    }

    elsif ($do_full_format eq "VA_1") {
	print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
	print TOFILE ") /* NOT IMPLEMENTED FOR SIM */";	
    }
    elsif ($do_full_format eq "VA_2") {
	print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
	print TOFILE ") /* NOT IMPLEMENTED FOR SIM */";	
    }
    elsif ($do_full_format eq "VX_1") {
	print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
	print TOFILE ") /* NOT IMPLEMENTED FOR SIM */";	
    }
    elsif ($do_full_format eq "VX_2") {
	print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
	print TOFILE ") /* NOT IMPLEMENTED FOR SIM */";	
    }
    elsif ($do_full_format eq "VX_3") {
	print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op;
	print TOFILE ") /* NOT IMPLEMENTED FOR SIM */";	
    }
    elsif ($do_full_format eq "VXR_1") {
	print TOFILE "   ( (", $do_opcode, "<<26) | ", $do_extended_op, " | (", $do_Rc, " << 10)";
	print TOFILE ") /* NOT IMPLEMENTED FOR SIM */";	
    }

    else {
	print STDERR "Opcode_sim_asm.pl: Unknown format ", $do_format, " : ", $do_full_format, " in process_asm\n";
    }

    print TOFILE "; break;\n";
}

sub DoFinalGenerateSimIword {
    local (*TOFILE) = $_[0];
    #close(TOFILE);
}

