#
#    Value                           1 1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 2 2 2 2 3  3       
#                0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0  1              
#     I_1:     |   opcode  |                       LI                      |AA|LK|
#
#     B_1:     |   opcode  |    BO   |    BI   |            BD             |AA|LK|
#
#    SC_1:     |   opcode  |   ///   |   ///   |           ///             |XO|LK|
#    SC_2:     |   opcode  |   ///   |   ///   |  ///  |   LEV       | /// |XO|LK|
#
#     D_1:     |   opcode  |    RT   |    RA   |                D                |
#     D_2:     |   opcode  |    RT   |    RA   |               SI                |
#     D_3:     |   opcode  |    RS   |    RA   |                D                |
#     D_4:     |   opcode  |    RS   |    RA   |               SI                |
#     D_5:     |   opcode  |  BF |/|L|    RA   |               SI                |
#     D_6:     |   opcode  |  BF |/|L|    RA   |               UI                |
#     D_7:     |   opcode  |    TO   |    RA   |               SI                |
#     D_8:     |   opcode  |   FRT   |    RA   |                D                |
#     D_9:     |   opcode  |   FRS   |    RA   |                D                |
#
#    DS_1:     |   opcode  |    RT   |    RA   |               DS          | XO  |
#    DS_2:     |   opcode  |    RS   |    RA   |               DS          | XO  |
#
#    DQ_1:     |   opcode  |    RT   |    RA   |          DQ           |    PT  |
#
#    X_1:      |   opcode  |    RT   |    RA   |    RB   |         XO         |//|
#    X_2:      |   opcode  |    RT   |    RA   |    NB   |         XO         |//|
#    X_3:      |   opcode  |    RS   |    RA   |    RB   |         XO         |Rc|
#    X_4:      |   opcode  |    RS   |    RA   |    NB   |         XO         |//|
#    X_5:      |   opcode  |    RS   |    RA   |    SH   |         XO         |Rc|
#    X_6:      |   opcode  |    RS   |   ///   |   ///   |         XO         |//|
#    X_7:      |   opcode  |  BF |/|L|    RA   |    RB   |         XO         |//|
#    X_8:      |   opcode  |  BF |///|   FRA   |   FRB   |         XO         |//|
#    X_9:      |   opcode  |  BF |///| BFA |///|   ///   |         XO         |//|
#    X_10:     |   opcode  |  BF |///|   ///   |    U  |/|         XO         |Rc|
#    X_11:     |   opcode  |    TO   |    RA   |    RB   |         XO         |//|
#    X_12:     |   opcode  |   FRT   |    RA   |    RB   |         XO         |//|
#    X_13:     |   opcode  |   FRT   |   ///   |   FRB   |         XO         |Rc|
#    X_14:     |   opcode  |   FRS   |    RA   |    RB   |         XO         |//|
#    X_15:     |   opcode  |    BT   |   ///   |   ///   |         XO         |Rc|
#    X_16:     |   opcode  |    RS   |/|   SR  |   ///   |         XO         |//|
#    X_17:     |   opcode  |    RT   |/|   SR  |   ///   |         XO         |//|
#    X_18:     |   opcode  |T|//|STRM|    RA   |    RB   |         XO         |//|
#    X_19:     |   opcode  |    VS   |    RA   |    RB   |         XO         |Rc|
#    X_20:     |   opcode  |    VT   |    RA   |    RB   |           XO        |0|
#
#    XL_1:     |   opcode  |    BT   |    BA   |    BB   |         XO         |//|
#    XL_2:     |   opcode  |    BO   |    RI   |   ///   |         XO         |LK|
#    XL_3:     |   opcode  |  BF |///| BFA |///|   ///   |         XO         |//|
#    XL_4:     |   opcode  |   ///   |   ///   |   ///   |         XO         |//|
#
#   XFX_1:     |   opcode  |    RT   |        SPR        |         XO         |//|
#   XFX_2:     |   opcode  |    RT   |        TBR        |         XO         |//|
#   XFX_3:     |   opcode  |    RS   |/|      FXM      |/|         XO         |//|
#   XFX_4:     |   opcode  |    RS   |        SPR        |         XO         |//|
#   XFX_5:     |   opcode  |   ///   |        XO2        |         XO         |//|
#
#   XFL_1:     |   opcode  |/|      FLM      |/|   FRB   |         XO         |Rc|
#
#    XS_1:     |   opcode  |    RS   |    RA   |    SH   |         XO      |SH|Rc|
#
#    XO_1:     |   opcode  |    RT   |    RA   |    RB   |OE|      XO         |Rc|
#    XO_2:     |   opcode  |    RT   |    RA   |    RB   |//|      XO         |Rc|
#
#     A_1:     |   opcode  |   FRT   |   FRA   |   FRB   |   FRC   |    XO    |Rc| 
#
#     M_1:     |   opcode  |    RS   |    RA   |    RB   |    MB   |    ME    |Rc|
#     M_2:     |   opcode  |    RS   |    RA   |    SH   |    MB   |    ME    |Rc|
#
#    MD_1:     |   opcode  |    RS   |    RA   |    SH   |    MB   |   XO  |SH|Rc|
#    MD_2:     |   opcode  |    RS   |    RA   |    SH   |    ME   |   XO  |SH|Rc|
#
#   MDS_1:     |   opcode  |    RS   |    RA   |    RB   |    MB   |     XO   |Rc|
#   MDS_2:     |   opcode  |    RS   |    RA   |    RB   |    ME   |     XO   |Rc|
#   MDS_3:     |   opcode  |    IS   |    RA   |    IB   | XBI  |//|     XO   |Rc|
#   MDS_4:     |   opcode  |    IS   |    RA   |    RB   | XBI  |//|     XO   |Rc|
#   MDS_5:     |   opcode  |    RS   |    RA   |    IB   | XBI  |//|     XO   |Rc|
#   MDS_6:     |   opcode  |    RS   |    RA   |    RB   | XBI  |//|     XO   |Rc|
#   MDS_3n:    |   opcode  |    IS   |    RA   |    IB   |  CBI  |/|     XO   |Rc|
#   MDS_4n:    |   opcode  |    IS   |    RA   |    RB   |  CBI  |/|     XO   |Rc|
#   MDS_5n:    |   opcode  |    RS   |    RA   |    IB   |  CBI  |/|     XO   |Rc|
#   MDS_6n:    |   opcode  |    RS   |    RA   |    RB   |  CBI  |/|     XO   |Rc|
#
#    TX_1:     |   opcode  |    TO   |        UI         |  XBI |       XO    |Rc|
#
#    VA_1:     |   opcode  |    VT   |    VA   |    VB   |    VC   |     XO      |
#    VA_2:     |   opcode  |    VT   |    VA   |    VB   |/|   SH  |     XO      |
#
#    VX_1:     |   opcode  |    VT   |    VA   |    VB   |           XO          |
#    VX_2:     |   opcode  |    VT   |   UIMM  |    VB   |           XO          |
#    VX_3:     |   opcode  |    VT   |   SIMM  |    VB   |           XO          |
#
#    VXR_1:    |   opcode  |    VT   |    VA   |    VB   |Rc|        XO          |
#



sub DoInitialSimConvertCode {
    local (*TOFILE)  = $_[0];
    local (*TOFILE2) = $_[1];
    local ($gpr_identifier) = " 1";
    local ($fpr_identifier) = " 2";
    local ($cr_identifier)  = " 3";
    local ($simple_number)  = " 4";

    print TOFILE2 "/* These defines are used to connect the register identifiers used */\n";
    print TOFILE2 "/*  by the opcode simulation code consistent across tools.         */\n";
    print TOFILE2 "#define OPC_GPR_IDENTIFIER $gpr_identifier \n";
    print TOFILE2 "#define OPC_FPR_IDENTIFIER $fpr_identifier \n";
    print TOFILE2 "#define OPC_CR_IDENTIFIER  $cr_identifier \n";
    print TOFILE2 "#define OPC_SIMPLE_VALUE   $simple_number \n";
    print TOFILE2 "\n";

    print TOFILE "/* Format:                                                                  */\n";
    print TOFILE "/* instr_index, instr_name, base_iword, num_operands, list_of_operand_types */\n";
    print TOFILE "/* Note:                                                                         */\n";
    print TOFILE "/*   $gpr_identifier => a GPR descrpitor       */\n";
    print TOFILE "/*   $fpr_identifier => a FPR descrpitor       */\n";
    print TOFILE "/*   $cr_identifier => a CR descriptor        */\n";
    print TOFILE "/*   $simple_number => a simple numeric value */\n";
    print TOFILE "/*                                */\n";
    
    $field_to_irep{"RT"}  = $gpr_identifier;
    $field_to_irep{"RS"}  = $gpr_identifier;
    $field_to_irep{"RA"}  = $gpr_identifier;
    $field_to_irep{"RB"}  = $gpr_identifier;
    $field_to_irep{"RC"}  = $gpr_identifier;

    $field_to_irep{"FRT"} = $fpr_identifier;
    $field_to_irep{"FRS"} = $fpr_identifier;
    $field_to_irep{"FRA"} = $fpr_identifier;
    $field_to_irep{"FRB"} = $fpr_identifier;
    $field_to_irep{"FRC"} = $fpr_identifier;

    $field_to_irep{"BF"}  = $cr_identifier;
    $field_to_irep{"BFA"} = $cr_identifier;

    $field_to_irep{"LI"}  = $simple_number;
    $field_to_irep{"LEV"} = $simple_number;
    $field_to_irep{"D"}   = $simple_number;
    $field_to_irep{"SI"}  = $simple_number;
    $field_to_irep{"UI"}  = $simple_number;
    $field_to_irep{"DS"}  = $simple_number;
    $field_to_irep{"BT"}  = $simple_number;
    $field_to_irep{"BA"}  = $simple_number;
    $field_to_irep{"BB"}  = $simple_number;
    $field_to_irep{"BO"}  = $simple_number;
    $field_to_irep{"BI"}  = $simple_number;
    $field_to_irep{"BD"}  = $simple_number;
    $field_to_irep{"FXM"} = $simple_number;
    $field_to_irep{"FLM"} = $simple_number;
    $field_to_irep{"SH2"} = $simple_number;
    $field_to_irep{"MB"}  = $simple_number;
    $field_to_irep{"MBW"} = $simple_number;
    $field_to_irep{"MEW"} = $simple_number;
    $field_to_irep{"ME2"} = $simple_number;
    $field_to_irep{"ME"}  = $simple_number;
    $field_to_irep{"IS"}  = $simple_number;
    $field_to_irep{"IB"}  = $simple_number;
    $field_to_irep{"XBI"} = $simple_number;
    $field_to_irep{"CBI"} = $simple_number;
    $field_to_irep{"TO"}  = $simple_number;
    $field_to_irep{"PT"}  = $simple_number;
    $field_to_irep{"DQ"}  = $simple_number;
    $field_to_irep{"L"}   = $simple_number;
    $field_to_irep{"NB"}  = $simple_number;
    $field_to_irep{"SH"}  = $simple_number;
    $field_to_irep{"U"}   = $simple_number;
    $field_to_irep{"SR"}  = $simple_number;
    $field_to_irep{"SPR"} = $simple_number;
    $field_to_irep{"TBR"} = $simple_number;
    $field_to_irep{"XO2"} = $simple_number;

    $spaces = "                               ";

}


#* Generate "reverse" OPC_ASM assembly-code macros *#
sub DoSimConvertCodeForThisInstruction {
    local (*TOFILE)  = $_[0];

    if( $do_full_format eq "I_1" ) {
	printf TOFILE ("{%3d, 0x%08x, 1, { %s, -1, -1, -1, -1 } }, /* %-10s */\n", # jdonald putting the braces in
		       $i, (($do_opcode<<26) | $do_AA<<1 | $do_LK),
		       $field_to_irep{"LI"}, $do_id);
    }	

    elsif( $do_full_format eq "B_1" ) {
	printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", # and so on...
		       $i, (($do_opcode<<26) | $do_AA<<1 | $do_LK),
		       $field_to_irep{"BO"}, $field_to_irep{"BI"}, $field_to_irep{"BD"}, $do_id);
    }

    elsif ($do_full_format eq "SC_1") {
	printf TOFILE ("{%3d, 0x%08x, 0, { -1, -1, -1, -1, -1 } }, /* %-10s */\n", 
		       $i, (($do_opcode<<26) | $do_extended_op<<1 | $do_LK), $do_id);
    }
    elsif ($do_full_format eq "SC_2") {
	printf TOFILE ("{%3d, 0x%08x, 1, { %s, -1, -1, -1, -1 } }, /* %-10s */\n", 
		       $i, (($do_opcode<<26) | $do_extended_op<<1 | $do_LK),
		       $field_to_irep{"LEV"}, $do_id);
    }

    elsif ($do_format eq "D_form") {
	if ($do_full_format eq "D_1") {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26),
			   $field_to_irep{"RT"}, $field_to_irep{"D"}, $field_to_irep{"RA"}, $do_id);	
	}
	elsif ($do_full_format eq "D_2") {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26),
			   $field_to_irep{"RT"}, $field_to_irep{"RA"}, $field_to_irep{"SI"}, $do_id);
	}
	elsif ($do_full_format eq "D_3") {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26),
			   $field_to_irep{"RS"}, $field_to_irep{"D"}, $field_to_irep{"RA"}, $do_id);
	}
	elsif ($do_full_format eq "D_4") {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26),
			   $field_to_irep{"RA"}, $field_to_irep{"RS"}, $field_to_irep{"SI"}, $do_id);
	}
	elsif ($do_full_format eq "D_5") {
	    printf TOFILE ("{%3d, 0x%08x, 4, { %s, %s, %s, %s, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26),
			   $field_to_irep{"BF"}, $field_to_irep{"L"}, $field_to_irep{"RA"}, $field_to_irep{"SI"}, $do_id);
	}
	elsif ($do_full_format eq "D_6") {
	    printf TOFILE ("{%3d, 0x%08x, 4, { %s, %s, %s, %s, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26),
			   $field_to_irep{"BF"}, $field_to_irep{"L"}, $field_to_irep{"RA"}, $field_to_irep{"UI"}, $do_id);

	}
	elsif ($do_full_format eq "D_7") {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26),
			   $field_to_irep{"TO"}, $field_to_irep{"RA"}, $field_to_irep{"SI"}, $do_id);
	}
	elsif ($do_full_format eq "D_8") {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26),
			   $field_to_irep{"FRT"}, $field_to_irep{"D"}, $field_to_irep{"RA"}, $do_id);
	}
	elsif ($do_full_format eq "D_9") {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26),
			   $field_to_irep{"FRS"}, $field_to_irep{"D"}, $field_to_irep{"RA"}, $do_id);
	}
    }

    elsif ($do_full_format eq "DS_1") {
	printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
		       $i, ($do_opcode<<26),
		       $field_to_irep{"RT"}, $field_to_irep{"DS"}, $field_to_irep{"RA"}, $do_id);
    }
    elsif ($do_full_format eq "DS_2") {
	printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
		       $i, ($do_opcode<<26),
		       $field_to_irep{"RS"}, $field_to_irep{"DS"}, $field_to_irep{"RA"}, $do_id);
    }
    elsif ($do_full_format eq "DQ_1") {
	printf TOFILE ("{%3d, 0x%08x, 4, { %s, %s, %s, %s, -1 } }, /* %-10s */\n", 
		       $i, ($do_opcode<<26),
		       $field_to_irep{"RT"}, $field_to_irep{"RA"}, $field_to_irep{"DQ"}, $field_to_irep{"PT"}, $do_id);
    }
    
    elsif ($do_format eq "X_form") {
	if( $do_full_format eq "X_1" ) {
	    if ( ($do_RT eq "0") && ($do_RA eq "0") && ($do_RB eq "0") ) {
		printf TOFILE ("{%3d, 0x%08x, 0, { -1, -1, -1, -1, -1 } }, /* %-10s */\n",
			       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc), $do_id);
	    }
	    elsif ( ($do_RA eq "0") && ($do_RB eq "0") ) {
		printf TOFILE ("{%3d, 0x%08x, 1, { %s, -1, -1, -1, -1 } }, /* %-10s */\n", 
			       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			       $field_to_irep{"RT"}, $do_id);
	    }
	    elsif ( ($do_RT eq "0") && ($do_RA eq "0") ) {
		printf TOFILE ("{%3d, 0x%08x, 1, { %s, -1, -1, -1, -1 } }, /* %-10s */\n", 
			       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			       $field_to_irep{"RB"}, $do_id);
	    }
	    elsif ($do_RT eq "0") {
		printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
			       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			       $field_to_irep{"RA"}, $field_to_irep{"RB"}, $do_id);
	    }
	    elsif ($do_RA eq "0") {
		printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
			       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			       $field_to_irep{"RT"}, $field_to_irep{"RB"}, $do_id);
	    }
	    elsif ($do_RB eq "0") {
		printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
			       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			       $field_to_irep{"RT"}, $field_to_irep{"RA"}, $do_id);
	    }
	    else {
		printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			       $field_to_irep{"RT"}, $field_to_irep{"RA"}, $field_to_irep{"RB"}, $do_id);
	    }
	}
	elsif ($do_full_format eq "X_2") {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"RT"}, $field_to_irep{"RA"}, $field_to_irep{"NB"}, $do_id);
	}
	elsif ($do_full_format eq "X_3") {
	    if ($do_store) {
		if ( ($do_RA ne "0") && ($do_RA ne "0") && ($do_RB ne "0") ) {
		    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
				   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
				   $field_to_irep{"RS"}, $field_to_irep{"RA"}, $field_to_irep{"RB"}, $do_id);
		}
		else { 
		    printf("Can a store indexed (%s) have RS = %s, RA = %s and RB = %s?}, /* %-10s */\n", $do_RS, $do_RA, $do_RB, $do_id);
		}
	    }
	    else {
		if ( ($do_RS eq "0") && ($do_RB eq "0") ) { 
		    printf TOFILE ("{%3d, 0x%08x, 1, { %s, -1, -1, -1, -1 } }, /* %-10s */\n", 
				   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
				   $field_to_irep{"RA"}, $do_id);
		}
		elsif ($do_RA eq "0") { 
		    printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
				   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
				   $field_to_irep{"RS"}, $field_to_irep{"RB"}, $do_id);
		}
		elsif ($do_RB eq "0") { 
		    printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
				   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
				   $field_to_irep{"RS"}, $field_to_irep{"RA"}, $do_id);
		}
		else {
		    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
				   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
				   $field_to_irep{"RS"}, $field_to_irep{"RA"}, $field_to_irep{"RB"}, $do_id);
		}
	    }
	}
	elsif ($do_full_format eq "X_4") {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"RS"}, $field_to_irep{"RA"}, $field_to_irep{"NB"}, $do_id);
	}
	elsif ($do_full_format eq "X_5") {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"RS"}, $field_to_irep{"RA"}, $field_to_irep{"SH"}, $do_id);
	}
	elsif ($do_full_format eq "X_6") {
	    printf TOFILE ("{%3d, 0x%08x, 1, { %s, -1, -1, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"RS"}, $do_id);
	}
	elsif ($do_full_format eq "X_7") {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"BF"}, $field_to_irep{"L"}, $field_to_irep{"RA"}, $field_to_irep{"RB"}, $do_id);
	}
	elsif ($do_full_format eq "X_8") {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"BF"}, $field_to_irep{"FRA"}, $field_to_irep{"FRB"}, $do_id);
	}
	elsif ($do_full_format eq "X_9") {
	    if ($do_BFA eq "0") {
		printf TOFILE ("{%3d, 0x%08x, 1, { %s, -1, -1, -1, -1 } }, /* %-10s */\n", 
			       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			       $field_to_irep{"BF"}, $do_id);
	    }
	    else {
		printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
			       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			       $field_to_irep{"BF"}, $field_to_irep{"BFA"}, $do_id);
	    }
	}
	elsif ($do_full_format eq "X_10") {
	    printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"BF"}, $field_to_irep{"U"}, $do_id);	
	}
	elsif ($do_full_format eq "X_11") {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"TO"}, $field_to_irep{"RA"}, $field_to_irep{"NB"}, $do_id);
	}
	elsif( $do_full_format eq "X_12" ) {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"FRT"}, $field_to_irep{"RA"}, $field_to_irep{"RB"}, $do_id);
	}
	elsif ($do_full_format eq "X_13") {
	    if ($do_FRB eq "0") {
		printf TOFILE ("{%3d, 0x%08x, 1, { %s, -1, -1, -1, -1 } }, /* %-10s */\n", 
			       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			       $field_to_irep{"FRT"}, $do_id);
	    }
	    else {
		printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
			       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			       $field_to_irep{"FRT"}, $field_to_irep{"FRB"}, $do_id);
	    }
	}
	elsif ($do_full_format eq "X_14") {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"FRS"}, $field_to_irep{"RA"}, $field_to_irep{"RB"}, $do_id);
	}
	elsif ($do_full_format eq "X_15") {
	    printf TOFILE ("{%3d, 0x%08x, 1, { %s, -1, -1, -1, -1 } }, /* %-10s */\n",
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"BT"}, $do_id);
	}
	elsif ($do_full_format eq "X_16") {
	    printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"RS"}, $field_to_irep{"SR"}, $do_id);
	}
	elsif ($do_full_format eq "X_17") {
	    printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"RT"}, $field_to_irep{"SR"}, $do_id);
	}
	elsif ($do_full_format eq "X_18") {
	    printf TOFILE ("{%3d, 0x%08x, 0, { -1, -1, -1, -1, -1 } }, /* %-10s  NOT IMPLEMENTED FOR SIM */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc), $do_id);
	}
	elsif ($do_full_format eq "X_19") {
	    printf TOFILE ("{%3d, 0x%08x, 0, { -1, -1, -1, -1, -1 } }, /* %-10s  NOT IMPLEMENTED FOR SIM */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc), $do_id);
	}
	elsif( $do_full_format eq "X_20" ) {
	    printf TOFILE ("{%3d, 0x%08x, 0, { -1, -1, -1, -1, -1 } }, /* %-10s  NOT IMPLEMENTED FOR SIM */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op), $do_id);
	}
    }
    elsif ($do_format eq "XL_form") {
	if ($do_full_format eq "XL_1") {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1),
			   $field_to_irep{"BT"}, $field_to_irep{"BA"}, $field_to_irep{"BB"}, $do_id);
	}
	elsif ($do_full_format eq "XL_2") {
	    printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1),
			   $field_to_irep{"BO"}, $field_to_irep{"BI"}, $do_id);
	}
	elsif ($do_full_format eq "XL_3") {
	    printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1),
			   $field_to_irep{"BF"}, $field_to_irep{"BFA"}, $do_id);
	}
	elsif ($do_full_format eq "XL_4") {
	    printf TOFILE ("{%3d, 0x%08x, 0, { -1, -1, -1, -1, -1 } }, /* %-10s */\n",
			   $i, ($do_opcode<<26 | $do_extended_op<<1), $do_id);
	}
    }
    elsif ($do_format eq "XFX_form") {
	if ($do_full_format eq "XFX_1") {
	    printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"RT"}, $field_to_irep{"SPR"}, $do_id);
	}
	elsif ($do_full_format eq "XFX_2") {
	    printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"RT"}, $field_to_irep{"TBR"}, $do_id);
	}
	elsif ($do_full_format eq "XFX_3") {
	    printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"FXM"}, $field_to_irep{"RS"}, $do_id);
	}
	elsif ($do_full_format eq "XFX_4") {
	    printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"RS"}, $field_to_irep{"SPR"}, $do_id);
	}
	elsif ($do_full_format eq "XFX_5") {
	    printf TOFILE ("{%3d, 0x%08x, 0, { -1, -1, -1, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"XO2"}, $do_id);
	}
    }

    elsif ($do_full_format eq "XFL_1") {
		printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
		       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
		       $field_to_irep{"FLM"}, $field_to_irep{"RB"}, $do_id);
    }

    elsif ($do_full_format eq "XS_1") {
		printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
		       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
		       $field_to_irep{"RS"}, $field_to_irep{"RA"}, $field_to_irep{"SH2"}, $do_id);
    }

    elsif ($do_full_format eq "XO_1") {
	if( $do_RB eq "0") {
	    printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"RT"}, $field_to_irep{"RA"}, $do_id);
	}
	else {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"RT"}, $field_to_irep{"RA"}, $field_to_irep{"RB"}, $do_id);
	}
    }
    elsif ($do_full_format eq "XO_2") {
		printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
		       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
		       $field_to_irep{"RT"}, $field_to_irep{"RA"}, $field_to_irep{"RB"}, $do_id);
    }

    elsif( $do_full_format eq "A_1" ) {
	if( ( $do_FRA eq "0" ) && ( $do_FRC eq "0" ) ) {
	    printf TOFILE ("{%3d, 0x%08x, 2, { %s, %s, -1, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"FRT"}, $field_to_irep{"FRB"}, $do_id);
	}
	elsif( $do_FRB eq "0") {
	    printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"FRT"}, $field_to_irep{"FRA"}, $field_to_irep{"FRC"}, $do_id);
	}
	elsif( $do_FRC eq "0" ) {
		printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
		       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
		       $field_to_irep{"FRT"}, $field_to_irep{"FRA"}, $field_to_irep{"FRB"}, $do_id);
	}
	else {
	    printf TOFILE ("{%3d, 0x%08x, 4, { %s, %s, %s, %s, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"FRT"}, $field_to_irep{"FRA"}, $field_to_irep{"FRB"}, $field_to_irep{"FRC"}, $do_id);
	}
    }

    elsif( $do_full_format eq "M_1" ) {
		printf TOFILE ("{%3d, 0x%08x, 5, { %s, %s, %s, %s, %s } }, /* %-10s */\n", 
		       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
		       $field_to_irep{"RS"}, $field_to_irep{"RA"}, $field_to_irep{"RB"}, 
		       $field_to_irep{"MB"}, $field_to_irep{"ME"}, $do_id);
    }
    elsif( $do_full_format eq "M_2" ) {
		printf TOFILE ("{%3d, 0x%08x, 5, { %s, %s, %s, %s, %s } }, /* %-10s */\n", 
		       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
		       $field_to_irep{"RS"}, $field_to_irep{"RA"}, $field_to_irep{"SH"}, 
		       $field_to_irep{"MB"}, $field_to_irep{"ME"}, $do_id);
    }

    elsif( $do_full_format eq "MD_1" ) {
		printf TOFILE ("{%3d, 0x%08x, 4, { %s, %s, %s, %s, -1 } }, /* %-10s */\n", 
		       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
		       $field_to_irep{"RS"}, $field_to_irep{"RA"}, $field_to_irep{"SH"}, $field_to_irep{"MBW"}, $do_id);
    }
    elsif( $do_full_format eq "MD_2" ) {
		printf TOFILE ("{%3d, 0x%08x, 4, { %s, %s, %s, %s, -1 } }, /* %-10s */\n", 
		       $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
		       $field_to_irep{"RS"}, $field_to_irep{"RA"}, $field_to_irep{"SH"}, $field_to_irep{"MEW"}, $do_id);
    }
    
    elsif( $do_format eq "MDS_form" ) {
	if( $do_full_format eq "MDS_1" ) {
	    printf TOFILE ("{%3d, 0x%08x, 4, { %s, %s, %s, %s, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"RS"}, $field_to_irep{"RA"}, $field_to_irep{"RB"}, $field_to_irep{"MB"}, $do_id);
	}
	elsif( $do_full_format eq "MDS_2" ) {
	    printf TOFILE ("{%3d, 0x%08x, 4, { %s, %s, %s, %s, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"RS"}, $field_to_irep{"RA"}, $field_to_irep{"RB"}, $field_to_irep{"ME2"}, $do_id);
	}
	elsif( $do_full_format eq "MDS_3" ) {
	    printf TOFILE ("{%3d, 0x%08x, 4, { %s, %s, %s, %s, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"IS"}, $field_to_irep{"RA"}, $field_to_irep{"IB"}, $field_to_irep{"XBI"}, $do_id);
	}
	elsif( $do_full_format eq "MDS_3n" ) {
	    printf TOFILE ("{%3d, 0x%08x, 4, { %s, %s, %s, %s, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"IS"}, $field_to_irep{"RA"}, $field_to_irep{"IB"}, $field_to_irep{"CBI"}, $do_id);
	}
	elsif( $do_full_format eq "MDS_4" ) {
	    printf TOFILE ("{%3d, 0x%08x, 4, { %s, %s, %s, %s, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"IS"}, $field_to_irep{"RA"}, $field_to_irep{"RB"}, $field_to_irep{"XBI"}, $do_id);
	}
	elsif( $do_full_format eq "MDS_4n" ) {
	    printf TOFILE ("{%3d, 0x%08x, 4, { %s, %s, %s, %s, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"IS"}, $field_to_irep{"RA"}, $field_to_irep{"RB"}, $field_to_irep{"CBI"}, $do_id);
	}
	elsif( $do_full_format eq "MDS_5" ) {
	    printf TOFILE ("{%3d, 0x%08x, 4, { %s, %s, %s, %s, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"RS"}, $field_to_irep{"RA"}, $field_to_irep{"IB"}, $field_to_irep{"XBI"}, $do_id);
	}
	elsif( $do_full_format eq "MDS_5n" ) {
	    printf TOFILE ("{%3d, 0x%08x, 4, { %s, %s, %s, %s, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"RS"}, $field_to_irep{"RA"}, $field_to_irep{"IB"}, $field_to_irep{"CBI"}, $do_id);
	}
	elsif( $do_full_format eq "MDS_6" ) {
	    printf TOFILE ("{%3d, 0x%08x, 4, { %s, %s, %s, %s, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"RS"}, $field_to_irep{"RA"}, $field_to_irep{"RB"}, $field_to_irep{"XBI"}, $do_id);
	}
	elsif( $do_full_format eq "MDS_6n" ) {
	    printf TOFILE ("{%3d, 0x%08x, 4, { %s, %s, %s, %s, -1 } }, /* %-10s */\n", 
			   $i, ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
			   $field_to_irep{"RS"}, $field_to_irep{"RA"}, $field_to_irep{"RB"}, $field_to_irep{"CBI"}, $do_id);
	}
    }
    
    elsif( $do_full_format eq "TX_1" ) {
		printf TOFILE ("{%3d, 0x%08x, 3, { %s, %s, %s, -1, -1 } }, /* %-10s */\n", 
		       $i, 
		       ($do_opcode<<26 | $do_extended_op<<1 | $do_Rc),
		       $field_to_irep{"TO"}, $field_to_irep{"UI"}, $field_to_irep{"XBI"}, $do_id);
    }


    elsif( $do_full_format eq "VA_1" ) {
		printf TOFILE ("{%3d, 0x%08x, 0, { -1, -1, -1, -1, -1 } }, /* %-10s  NOT IMPLEMENTED FOR SIM */\n", 
		       $i, ($do_opcode<<26 | $do_extended_op), $do_id);
    }
    elsif( $do_full_format eq "VA_2" ) {
		printf TOFILE ("{%3d, 0x%08x, 0, { -1, -1, -1, -1, -1 } }, /* %-10s  NOT IMPLEMENTED FOR SIM */\n", 
		       $i, ($do_opcode<<26 | $do_extended_op), $do_id);
    }
    elsif( $do_full_format eq "VX_1" ) {
		printf TOFILE ("{%3d, 0x%08x, 0, { -1, -1, -1, -1, -1 } }, /* %-10s  NOT IMPLEMENTED FOR SIM */\n", 
		       $i, ($do_opcode<<26 | $do_extended_op), $do_id);
    }
    elsif( $do_full_format eq "VX_2" ) {
		printf TOFILE ("{%3d, 0x%08x, 0, { -1, -1, -1, -1, -1 } }, /* %-10s  NOT IMPLEMENTED FOR SIM */\n", 
		       $i, ($do_opcode<<26 | $do_extended_op), $do_id);
    }
    elsif( $do_full_format eq "VX_3" ) {
		printf TOFILE ("{%3d, 0x%08x, 0, { -1, -1, -1, -1, -1 } }, /* %-10s  NOT IMPLEMENTED FOR SIM */\n", 
		       $i, ($do_opcode<<26 | $do_extended_op), $do_id);
    }
    elsif( $do_full_format eq "VXR_1" ) {
		printf TOFILE ("{%3d, 0x%08x, 0, { -1, -1, -1, -1, -1 } }, /* %-10s  NOT IMPLEMENTED FOR SIM */\n", 
		       $i, ($do_opcode<<26 | $do_extended_op), $do_id);
    }

    else {
	print STDERR "Opcode_simcode: Unknown format ", $do_format, " : ", $do_full_format, " in process_simcode for ", $do_id;
    }
}

sub DoFinalSimConvertCode {
    local (*TOFILE)  = $_[0];

    #close(TOFILE);
}

