/*
** COMPONENT_NAME: MET / Aria
**
** GA-0100 (C) Copyright IBM Corp. 1997 - 2003
**         under development at Princeton University 2004 - 2006
**
** IBM Confidential (IBM Confidential-Restricted when combined
**                   with aggregated modules for this product)
** OCO Source Materials
**
** US Government Users Restricted Rights - Use, duplication or
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
**
** The source code for this program is not published or otherwise
** divested of its trade secrets, irrespective of what has been
** deposited within the U.S. Copyright Office.
**
*/

/*******************************************************************
  PROGRAM NAME..... build_sim_file

  FUNCTION ........ Generates an aria simulation file from a user
     provided simtable description.

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 
  CALLED-ROUTINES.. 
  CALLED-BY........ None
  INCLUDED-BY...... 
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... James Donald, Princeton University, March 2006
  
  ********************************************************************/

#include <unistd.h> // jdonald
#include <string.h> // jdonald, for strerror()
#include <sys/types.h>
#include <sys/stat.h> // jdonald, for fchmod()

#include "posix_fcntl.h"

#include "ansi_stdio.h"
#include "ansi_stdlib.h"

#include "ansi_assert.h"

#include "aix_errno.h"

#include "opc.h"

#include "opc_is_implemented_types.h"

/** MACROS **/
#define TAKEN_PATH       0
#define NOT_TAKEN_PATH   1

/** Types **/

/** Forward function declarations **/
void print_usage(int argc, char * argv[]);
int read_input_flags(int argc, char * argv[]);

char * get_opcode_token();
int    get_isa_token();
int    get_operand_token(int operand_type);
int    get_operand_flag(int operand_type);

void add_to_out_buffer(unsigned word);
void force_write_of_out_buffer();
int get_next_infile_block();

int classify_isa_token();

void process_one_simcode_entry();

/** Global Variables **/
extern int errno;

__thread char * outfile_base_name = "simcode.bin";
__thread int outfiled;

__thread char * infile_base_name = NULL;
__thread int    infile_line = 1;
__thread int    infiled;

__thread int run_in_verbose_mode = 1;

__thread unsigned total_bytes_read = 0;

/*******************************************************************
  PROGRAM NAME..... build_sim_file

  FUNCTION ........ 

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 
  CALLED-ROUTINES.. 
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

__thread int eof_found = 0;
__thread int error_occurred = 0;
__thread int end_marker_found = 0;
__thread int isa_marker_found = 0;
__thread int isa_marker_test_val = 0;

__thread char string_buffer[64];
__thread int string_buffer_idx;

#define READ_BUFFER_SIZE 4096
__thread char read_buffer[READ_BUFFER_SIZE];
__thread char * rp;
__thread char * rend;

typedef struct iwords_data_struct_t {
  int index;
  unsigned iword_base;
  int num_operands;
  int op_type[5];
} iwords_data;

__thread iwords_data simcode_info_tbl[OPC_NUM+1] = {
  {0, 0, 0, { 0, 0, 0, 0, 0 } },
  #include "opc_form_simulation_code.gen"
  {0, 0, 0, { 0, 0, 0, 0, 0 } }
  };

#define OUT_BUFFER_SIZE 4096
__thread unsigned out_buffer[OUT_BUFFER_SIZE];
__thread int out_buffer_idx;
__thread int spec_flag;

int
main( int argc, 
      char * argv[]
      )
{
  char ch;
  out_buffer_idx = 0;

  opc_init();

  /** Read in the input (command line) switch values **/
  if (read_input_flags(argc, argv) != 0) {
    fprintf(stderr, "Problem reading the command-line input... Exiting\n");
    print_usage(argc, argv);
    return -1;
  }

  if (run_in_verbose_mode > 1) {fprintf(stderr, "Trying to open input file \"%s\"\n", infile_base_name); }
  infiled = open(infile_base_name, O_RDONLY, 0);
  if (infiled == -1) {
    fprintf(stderr, "Could not open input file \"%s\"\n", infile_base_name);
    fprintf(stderr, "   errno=%d (\"%s\")\n", errno, strerror(errno));
    exit(-1);
  }
  if (run_in_verbose_mode > 1) {fprintf(stderr, "Opened simcode file: \"%s\"\n", infile_base_name); }
  
  if (run_in_verbose_mode > 1) {fprintf(stderr, "Trying to open output file \"%s\"\n", outfile_base_name); }
  outfiled = creat(outfile_base_name, 0);
  if (outfiled == -1) {
    fprintf(stderr, "Could not open output file \"%s\"\n", outfile_base_name);
    fprintf(stderr, "   errno=%d (\"%s\")\n", errno, strerror(errno));
    exit(-1);
  }  
  if (run_in_verbose_mode > 1) {fprintf(stderr, "Opened output file: \"%s\" = %d\n", outfile_base_name, outfiled); }
  if (fchmod(outfiled, 0666) == -1) {
    fprintf(stderr, "Could not fchmod 0x666 output file \"%s\"\n", outfile_base_name);
    fprintf(stderr, "   errno=%d (\"%s\")\n", errno, strerror(errno));
    exit(-1);
  }    
  
  /* Read in the simcode file */
  {
    int n = get_next_infile_block();
    if (n == 0) {
      fprintf(stderr, "Is file \"%s\" empty?\n", infile_base_name);
      return -1;
    }

    do {
      process_one_simcode_entry();
      if (run_in_verbose_mode > 1) {fprintf(stderr, "---------------------------------------- (rp = 0x%08x vs 0x%08x = rend)\n", (unsigned) rp, (unsigned) rend);} // jdonald cast
    } while (eof_found == 0);
    
    /* Clean up the tail of the file */
    while (rp != rend) {
      process_one_simcode_entry();      
      if (run_in_verbose_mode > 1) {fprintf(stderr, "---------------------------------------C (rp = 0x%08x vs 0x%08x = rend)\n", (unsigned) rp, (unsigned) rend);} // jdonald cast
    }      
  }

  close(infiled);

  if (out_buffer_idx != 0) {
    fprintf(stderr, "ERROR: should be nothing left in the output buffer\n");
    fprintf(stderr, "       This occurred near line %d of file \"%s\"\n", infile_line, infile_base_name);
    error_occurred = 1;
    force_write_of_out_buffer();
  }

  /* Write out a zero-word to indicate end-of-file */
  /*    out_buffer[out_buffer_idx++] = 0x00000000; */
  /*    force_write_of_out_buffer(); */

  close(outfiled);  

  if (error_occurred) {
    fprintf(stderr, "\nThere were errors in this run...\n");
    fprintf(stderr, "The resulting output file \"%s\"\n", outfile_base_name);
    fprintf(stderr, " probably contains inconsistent data, and should not be used.\n");
    fprintf(stderr, " Please analyze your input file \"%s\"\n", infile_base_name);
    fprintf(stderr, " and resolve the errors reported above.\n");
  }
  
  return EXIT_SUCCESS; // jdonald
}


int 
get_next_infile_block()
{
  int ret_val;

  if (run_in_verbose_mode > 2) {
    fprintf(stderr, "Getting the next file block: rp = 0x%08x vs 0x%08x = rend\n", (unsigned) rp, (unsigned) rend); // jdonald cast
  }

  ret_val = read( infiled, &read_buffer, sizeof(read_buffer) );
  total_bytes_read += ret_val; 

  if (ret_val == 0) {
    if (run_in_verbose_mode > 2) {
      fprintf(stderr, "Found end of file\n");
    }
    rp = read_buffer;
    rend = read_buffer;
    *rp = 'z';
    eof_found = 1;    
  }

  rp = read_buffer;
  rend = &read_buffer[ret_val];

  if (run_in_verbose_mode > 2) {
    int i;
    fprintf(stderr, "I read the following: (ret_val = %d)\n", ret_val);
    fprintf(stderr, "--------------------------------------------------\n");
    for (i = 0; i < ret_val; i++) {
      fprintf(stderr, "%c", read_buffer[i]);
    }
    fprintf(stderr, "\n--------------------------------------------------\n");
  }

  return ret_val;
}

void 
process_one_simcode_entry()
{
  char * token;
  int iidx;
  int iisa;
  int sidx;
      
  unsigned   num_isas;
  unsigned * num_isas_p;
  unsigned   num_isa_instrs;
  unsigned * num_isa_instrs_p;

  token = get_opcode_token();
  if (eof_found) {
    return;
  }

  if (run_in_verbose_mode) {fprintf(stderr, "Adding simcode for instruction \"%s\"\n", token);}
  
  iidx = opc_index_for_name(token);
  if (run_in_verbose_mode > 1) {fprintf(stderr, "  it has index %d\n", iidx);}
  if ((iidx < 1) || (iidx > OPC_NUM)) {
    fprintf(stderr, "ERROR: Illegal instruction specification: \"%s\"\n", string_buffer);
    fprintf(stderr, "       This occurred near line %d of file \"%s\"\n", infile_line, infile_base_name);
    error_occurred = 1;
  }
  
  add_to_out_buffer(iidx);
  num_isas_p = &out_buffer[out_buffer_idx];
  add_to_out_buffer(0x00000000); /* get a placeholder for the num_isas */
  num_isas = 0;

  iisa = get_isa_token();
  if (eof_found) {
    fprintf(stderr, "ERROR: EOF found in midst of simulation code definition\n");
    fprintf(stderr, "       This occurred near line %d of file \"%s\"\n", infile_line, infile_base_name);
    error_occurred = 1;
    return;
  }

  do {
    if (run_in_verbose_mode > 1) {fprintf(stderr, "      now adding simcode for isa = \"%s\" spec_flag = %d\n", token, spec_flag);}
    
    add_to_out_buffer(iisa);
    add_to_out_buffer(spec_flag);
    num_isas++;
    num_isa_instrs_p = &out_buffer[out_buffer_idx];
    add_to_out_buffer(0x00000000); /* get a placeholder for the num_isa_instrs */

    num_isa_instrs = 0;
    end_marker_found = 0;
    isa_marker_found = 0;

    token = get_opcode_token();
    if (eof_found) {
      fprintf(stderr, "ERROR: EOF found in midst of simulation code definition\n");
      fprintf(stderr, "       This occurred near line %d of file \"%s\"\n", infile_line, infile_base_name);
      error_occurred = 1;
      return;
    }
    while ((eof_found == 0) && (end_marker_found == 0) && (isa_marker_found == 0)) {
      int ii;
      if (run_in_verbose_mode > 1) {fprintf(stderr, "    read in token \"%s\"\n", token);}
      sidx = opc_index_for_name(token);
      add_to_out_buffer(sidx);
      num_isa_instrs++;
      if (run_in_verbose_mode > 1) {fprintf(stderr, "      it has index %d and %d operands\n", 
					    sidx, simcode_info_tbl[sidx].num_operands);}
      for (ii = 0; ii < simcode_info_tbl[sidx].num_operands; ii++) {
	int op_tok_val = get_operand_token(simcode_info_tbl[sidx].op_type[ii]);
	if (eof_found) {
	  fprintf(stderr, "ERROR: EOF found in midst of simulation code definition\n");
	  fprintf(stderr, "       This occurred near line %d of file \"%s\"\n", infile_line, infile_base_name);
	  error_occurred = 1;
	  return;
	}
	if (run_in_verbose_mode > 1) {
	  fprintf(stderr, "      read in operand %d, type = %d, token = \"%s\" = %d\n", 
		  ii, simcode_info_tbl[sidx].op_type[ii], string_buffer, op_tok_val);
	}
	add_to_out_buffer(op_tok_val);
      }
      token = get_opcode_token();
      if (eof_found) {
	fprintf(stderr, "ERROR: EOF found in midst of simulation code definition\n");
	fprintf(stderr, "       This occurred near line %d of file \"%s\"\n", infile_line, infile_base_name);
	error_occurred = 1;
	return;
      }
      if (run_in_verbose_mode > 1) {fprintf(stderr, "                    ----------------------------------------\n");}
    } 

    if (run_in_verbose_mode > 1) {fprintf(stderr, "    read in token \"%s\"\n", token);}
    iisa = isa_marker_test_val;
    *num_isa_instrs_p = num_isa_instrs;
  } while (isa_marker_found);
  
  *num_isas_p = num_isas;

  force_write_of_out_buffer();

  return;
}



#define is_an_opcode_letter(_ch) ((((_ch) >= 'a') && ((_ch) <= 'z')) || \
				  (((_ch) >= '0') && ((_ch) <= '9')) || \
				  ((_ch) == '.') || ((_ch) == '@') )

#define is_an_operand_letter(_ch) ((((_ch) >= 'A') && ((_ch) <= 'Z')) || \
				   (((_ch) >= '0') && ((_ch) <= '9')) || \
				   (((_ch) >= 'a') && ((_ch) <= 'f')) || \
				   ((_ch) == 'x') )

#define is_a_number(_ch) (((_ch) >= '0') && ((_ch) <= '9'))
#define is_whitespace(_ch) (((_ch) == ' ') || ((_ch) == '\t') ||  ((_ch) == ',') || ((_ch) == '\n'))
#define is_comment_char(_ch)  ((_ch) == ';')

void 
skip_comment()
{
  do {
    char * last_rp = rp;
    while ((*last_rp != '\n') && (eof_found == 0)) {
      if (run_in_verbose_mode > 2) {fprintf(stderr, "skip_comment : '%c'\n", *rp);}
      last_rp = rp;
      rp++; 
      if (rp == rend) { 
	int n = get_next_infile_block();
	if (eof_found) {
	  return;
	}
      } 
    } 
    infile_line++;
  } while (is_comment_char(*rp));
}

void 
skip_whitespace()
{
  do {
    if (is_comment_char(*rp)) {
      skip_comment();
    }
    while (is_whitespace(*rp) && (eof_found == 0)) {
      if (*rp == '\n') { 
	infile_line++;
      }
      if (run_in_verbose_mode > 2) {fprintf(stderr, "skip_whitespace: '%c'\n", *rp);}
      rp++; 
      if (rp >= rend) { 
	int n = get_next_infile_block();
	if (eof_found) {
	  return;
	}
      } 
      if (is_comment_char(*rp)) {
	skip_comment();
      }
    } 
  } while (is_whitespace(*rp));
}

char * 
get_opcode_token()
{
  /* Skip whitespace */
  skip_whitespace();

  string_buffer_idx = 0;

  if (eof_found) { 
    string_buffer[string_buffer_idx] = '\0';
    return NULL; // jdonald return NULL instead of nothing
  }
    
  while (is_an_opcode_letter(*rp)) {
    string_buffer[string_buffer_idx++] = *rp;
    if (run_in_verbose_mode > 2) {fprintf(stderr, "get_opcode_token: '%c'\n", *rp);}
    rp++;
    if (rp >= rend) { 
      int n = get_next_infile_block();
      if (eof_found) {
	error_occurred = 1;
	return string_buffer;
      }
    } 
  }
  if (run_in_verbose_mode > 2) {fprintf(stderr, "get_opcode_token: '%c'\n", *rp);}

  string_buffer[string_buffer_idx++] = '\0';

  if (strcmp(string_buffer, "@end") == 0) {
    end_marker_found = 1;
  }
  else {
    isa_marker_test_val = classify_isa_token(&spec_flag);
    if (isa_marker_test_val >= 0) {
      isa_marker_found = 1;      
    }
  }
  return string_buffer;
}
  

int 
get_isa_token()
{
  int ret_val = -1;

  /* Skip whitespace */
  skip_whitespace();

  string_buffer_idx = 0;
  
  if (eof_found) { 
    string_buffer[string_buffer_idx] = '\0';
    return 0; // jdonald return 0, not nothing
  }
    
  while (is_an_opcode_letter(*rp)) {
    string_buffer[string_buffer_idx++] = *rp;
    if (run_in_verbose_mode > 2) {fprintf(stderr, "get_isa_token: '%c'\n", *rp);}
    rp++;
    if (rp >= rend) { 
      int n = get_next_infile_block();
      if (eof_found) {
	error_occurred = 1;
	return ret_val;
      }
    } 
  }
  if (run_in_verbose_mode > 2) {fprintf(stderr, "get_isa_token: '%c'\n", *rp);}

  string_buffer[string_buffer_idx++] = '\0';

  ret_val = classify_isa_token(&spec_flag);

  if (ret_val < 0) {
    fprintf(stderr, "ERROR: Illegal isa specification: \"%s\"\n", string_buffer);
    fprintf(stderr, "       This occurred near line %d of file \"%s\"\n", infile_line, infile_base_name);
    error_occurred = 1;
  }

  return ret_val;
}

int
classify_isa_token(int * spec_flag)
{
  if (strcmp(string_buffer, "simppc") == 0) {
    *spec_flag = TAKEN_PATH;
    return OPC_PPC_ISA;
  }
  else if (strcmp(string_buffer, "simppc_spec") == 0) {
    *spec_flag = NOT_TAKEN_PATH;
    return OPC_PPC_ISA;
  }
  else if (strcmp(string_buffer, "simppc32") == 0) {
    *spec_flag = TAKEN_PATH;
    return OPC_PPC32_ISA;
  }
  else if (strcmp(string_buffer, "simppc32_spec") == 0) {
    *spec_flag = NOT_TAKEN_PATH;
    return OPC_PPC32_ISA;
  }
  else if (strcmp(string_buffer, "simppc40x") == 0) {
    *spec_flag = TAKEN_PATH;
    return OPC_PPC40X_ISA;
  }
  else if (strcmp(string_buffer, "simppc40x_spec") == 0) {
    *spec_flag = NOT_TAKEN_PATH;
    return OPC_PPC40X_ISA;
  }
  else if (strcmp(string_buffer, "simppc601") == 0) {
    *spec_flag = TAKEN_PATH;
    return OPC_PPC601_ISA;
  }
  else if (strcmp(string_buffer, "simppc601_spec") == 0) {
    *spec_flag = NOT_TAKEN_PATH;
    return OPC_PPC601_ISA;
  }
  else if (strcmp(string_buffer, "simpower") == 0) {
    *spec_flag = TAKEN_PATH;
    return OPC_POWER_ISA;
  }
  else if (strcmp(string_buffer, "simpower_spec") == 0) {
    *spec_flag = NOT_TAKEN_PATH;
    return OPC_POWER_ISA;
  }
  else if (strcmp(string_buffer, "simpower2") == 0) {
    *spec_flag = TAKEN_PATH;
    return OPC_POWER2_ISA;
  }
  else if (strcmp(string_buffer, "simpower2_spec") == 0) {
    *spec_flag = NOT_TAKEN_PATH;
    return OPC_POWER2_ISA;
  }
  else if (strcmp(string_buffer, "simamazon") == 0) {
    *spec_flag = TAKEN_PATH;
    return OPC_AMAZON_ISA;
  }
  else if (strcmp(string_buffer, "simamazon_spec") == 0) {
    *spec_flag = NOT_TAKEN_PATH;
    return OPC_AMAZON_ISA;
  }
  else if (strcmp(string_buffer, "simppc64on32") == 0) {
    *spec_flag = TAKEN_PATH;
    return OPC_PPC64on32_ISA;
  }
  else if (strcmp(string_buffer, "simppc64on32_spec") == 0) {
    *spec_flag = NOT_TAKEN_PATH;
    return OPC_PPC64on32_ISA;
  }
  else {
    return -1;
  }
}


int
get_operand_token(int operand_type)
{
  int operand_val = -1;

  /* Skip whitespace */
  skip_whitespace();

  string_buffer_idx = 0;
  
  if (eof_found) { 
    string_buffer[string_buffer_idx] = '\0';
    return 0; // jdonald return 0, instead of nothing
  }
    
  while (is_an_operand_letter(*rp)) {
    string_buffer[string_buffer_idx++] = *rp;
    if (run_in_verbose_mode > 2) {fprintf(stderr, "get_operand_token: '%c'\n", *rp);}
    rp++;
    if (rp >= rend) { 
      int n = get_next_infile_block();
      if (eof_found) {
	error_occurred = 1;
	return operand_val;
      }
    } 
  }
  if (run_in_verbose_mode > 2) {fprintf(stderr, "get_operand_token: '%c'\n", *rp);}

  string_buffer[string_buffer_idx++] = '\0';

  operand_val = get_operand_flag(operand_type);

  return operand_val;
}


#include "opc_reg_ident_defines.gen"
#define NUM_GPR_SCRATCH_REGISTERS 4
#define NUM_FPR_SCRATCH_REGISTERS 4

int 
get_operand_flag(int operand_type) 
{
  int ret_val = 0xdeadbeef;
  char str_buf_02[4];
  str_buf_02[0] = string_buffer[0];
  str_buf_02[1] = string_buffer[1];
  str_buf_02[2] = string_buffer[2];
  str_buf_02[3] = '\0';

  switch (operand_type) {
  case OPC_GPR_IDENTIFIER:
    /* Check for special flags, e.g. RA */
    if      (strcmp(string_buffer, "RT") == 0)   { ret_val = 32; }
    else if (strcmp(string_buffer, "RS") == 0)   { ret_val = 33; }
    else if (strcmp(string_buffer, "RA") == 0)   { ret_val = 34; }
    else if (strcmp(string_buffer, "RB") == 0)   { ret_val = 35; }
    /* Now check for scratch register names, e.g. SCR1 */
    else if (strcmp(str_buf_02, "SCR") == 0) { 
      ret_val = atoi(&string_buffer[3]);
      if ((ret_val < 0) || (ret_val >= NUM_GPR_SCRATCH_REGISTERS)) {
	fprintf(stderr, "ERROR: GPR register specification \"%s\" illegal -- SCR %d does not exist\n", 
		string_buffer, ret_val);
	fprintf(stderr, "       This occurred near line %d of file \"%s\"\n", infile_line, infile_base_name);
	error_occurred = 1;
	ret_val = 0xdeadbeef;
      }
      else {
	ret_val = ret_val + 36;
      }
    }
    /* Or this could be a specific GPR specification, e.g. GPR0, GPR1 */
    else if (strcmp(str_buf_02, "GPR") == 0) { 
      ret_val = atoi(&string_buffer[3]);
      if ((ret_val < 0) || (ret_val > 31)) {
	fprintf(stderr, "ERROR: GPR register specification \"%s\" illegal -- GPR %d does not exist\n", string_buffer, ret_val);
	fprintf(stderr, "       This occurred near line %d of file \"%s\"\n", infile_line, infile_base_name);
	error_occurred = 1;
	ret_val = 0xdeadbeef;
      }
    }
    /* else this is an error */
    else {
      fprintf(stderr, "ERROR: GPR operand token \"%s\" unrecognized\n", string_buffer);
      fprintf(stderr, "       This occurred near line %d of file \"%s\"\n", infile_line, infile_base_name);
      error_occurred = 1;
    }
    break;
    
  case OPC_FPR_IDENTIFIER:
    /* Check for special flags, e.g. RA */
    if      (strcmp(string_buffer, "FRT") == 0)   { ret_val = 32; }
    else if (strcmp(string_buffer, "FRS") == 0)   { ret_val = 33; }
    else if (strcmp(string_buffer, "FRA") == 0)   { ret_val = 34; }
    else if (strcmp(string_buffer, "FRB") == 0)   { ret_val = 35; }
    else if (strcmp(string_buffer, "FRC") == 0)   { ret_val = 36; }
    /* Now check for scratch register names, e.g. SCR1 */
    else if (strcmp(str_buf_02, "FSC") == 0) { 
      ret_val = atoi(&string_buffer[3]);
      if ((ret_val < 0) || (ret_val >= NUM_FPR_SCRATCH_REGISTERS)) {
	fprintf(stderr, "ERROR: FPR register specification \"%s\" illegal -- FSC %d does not exist\n", 
		string_buffer, ret_val);
	fprintf(stderr, "       This occurred near line %d of file \"%s\"\n", infile_line, infile_base_name);
	ret_val = 0xdeadbeef;
	error_occurred = 1;
      }
      else {
	ret_val = ret_val + 37;
      }
    }
    /* Or this could be a specific FPR specification, e.g. FPR0, FPR1 */
    else if (strcmp(str_buf_02, "FPR") == 0) { 
      ret_val = atoi(&string_buffer[3]);
      if ((ret_val < 0) || (ret_val > 31)) {
	fprintf(stderr, "ERROR: FPR register specification \"%s\" illegal -- FPR %d does not exist\n", string_buffer, ret_val);
	fprintf(stderr, "       This occurred near line %d of file \"%s\"\n", infile_line, infile_base_name);
	ret_val = 0xdeadbeef;
	error_occurred = 1;
      }
    }
    /* else this is an error */
    else {
      fprintf(stderr, "ERROR: FPR operand token \"%s\" unrecognized\n", string_buffer);
      fprintf(stderr, "       This occurred near line %d of file \"%s\"\n", infile_line, infile_base_name);
      error_occurred = 1;
    }
    break;

  case OPC_CR_IDENTIFIER:
    str_buf_02[2] = '\0';
    /* Check for special flags, e.g. RA */
    if      (strcmp(string_buffer, "BF") == 0)  { ret_val = 8; }
    else if (strcmp(string_buffer, "BFA") == 0) { ret_val = 9; }
    /* Or this could be a specific CR specification, e.g. CR0, CR1 */
    else if (strcmp(str_buf_02, "CR") == 0) { 
      ret_val = atoi(&string_buffer[2]);
      if ((ret_val < 0) || (ret_val > 7)) {
	fprintf(stderr, "ERROR: CR register specification \"%s\" illegal -- CR %d does not exist\n", string_buffer, ret_val);
	fprintf(stderr, "       This occurred near line %d of file \"%s\"\n", infile_line, infile_base_name);
	ret_val = 0xdeadbeef;
	error_occurred = 1;
      }
    }
    /* else this is an error */
    else {
      fprintf(stderr, "ERROR: CR operand token \"%s\" unrecognized\n", string_buffer);
      fprintf(stderr, "       This occurred near line %d of file \"%s\"\n", infile_line, infile_base_name);
      error_occurred = 1;
    }
    break;
    
  case OPC_SIMPLE_VALUE:
    /* I guess it must be a number? */
    ret_val = atoi(string_buffer);
    break;
    
  default:
    fprintf(stderr, "ERROR: Unknown operand type flag: %d for operand \"%s\"\n", operand_type, string_buffer);
    fprintf(stderr, "       This occurred near line %d of file \"%s\"\n", infile_line, infile_base_name);
    error_occurred = 1;
    break;
  }

  return ret_val;
}


void
add_to_out_buffer(unsigned word)
{
  out_buffer[out_buffer_idx++] = word;
  if (out_buffer_idx >= OUT_BUFFER_SIZE) {
    /* out_buffer is full -- signal error */
    fprintf(stderr, "ERROR: could not fit enough in the out buffer -- increase its size? (from %d bytes)\n", 
	    (int)(sizeof(unsigned) * OUT_BUFFER_SIZE));
    error_occurred = 1; 
    exit(-1);
  }
}

void
force_write_of_out_buffer()
{
  /* write out_buffer */
  int n = write( outfiled, out_buffer, sizeof(unsigned)*out_buffer_idx);
  if (n == -1) {
    fprintf(stderr, "Could not write to file \"%s\", errno=%d (\"%s\")\n", outfile_base_name, errno, strerror(errno));
  }
  if (run_in_verbose_mode > 1) {
    int i;
    fprintf(stderr, "Writing records to the output file: (%d of %d) : n = %d\n", 
	    (int)(sizeof(unsigned)*out_buffer_idx), (int)sizeof(out_buffer), n);
    for (i = 0; i < out_buffer_idx; i++) {
      if ((i > 0) && ((i % 4) == 0)) {
	fprintf(stderr, "\n");
      }
      if ((i % 4) == 0) {
	fprintf(stderr, "   %4d  :  ", i);
      }
      fprintf(stderr, "0x%08x ", out_buffer[i]);
    }
    fprintf(stderr, "\n");
  }
  out_buffer_idx = 0;
}

/*******************************************************************
  PROGRAM NAME..... read_input_flags

  FUNCTION ........ Parses the input command-line arguments, and 
     sets (global) variables to indicate the flag conditions that 
     are requested by the user.

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 0 = good, other = error in parsing
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

int
read_input_flags(int    argc, 
		 char * argv[]
		 ) 
{
  int ret_val = 0;
  int i = 1;
  if (run_in_verbose_mode > 1) { fprintf(stderr, "reading in the input parameters...\n"); } // jdonald removed: i, argv[i]

  /* Check that enough parameters were specified */
  if (argc < 2) {
    return -1;
  }

  while (i < argc) {
    if (run_in_verbose_mode > 1) { fprintf(stderr, "  argv[%d] = \"%s\"\n", i, argv[i]); }
    
    if (strcmp(argv[i], "-o") == 0) {
      i++;
      if (i < argc) {
	outfile_base_name = argv[i];
      }
      else {
	fprintf(stderr, "MUST specify the output file name after the -o flag\n");
	ret_val = 1;
	goto exit_this_while_loop;
      }
    }
    else if ((strcmp(argv[i], "-q") == 0) || (strcmp(argv[i], "-quiet") == 0)) {
      run_in_verbose_mode = 0;
    }
    else if ((strcmp(argv[i], "-v") == 0) || (strcmp(argv[i], "-verbose") == 0)) {
      run_in_verbose_mode = 2;
    }
    else if ((strcmp(argv[i], "-V") == 0) || (strcmp(argv[i], "-Verbose") == 0)) {
      run_in_verbose_mode = 3;
    }
    else if (i == argc-1) {
      infile_base_name = argv[i];
    }
    i++;
  }
  
exit_this_while_loop:

  if (infile_base_name == NULL) {
    ret_val = -1;
  }

  return ret_val;
}


/*******************************************************************
  PROGRAM NAME..... print_usage

  FUNCTION ........ Dumps out information about the invocation of 
     the build_sim_file program.

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

void
print_usage(int argc,
	    char * argv[]
	    )
{
  fprintf(stderr, "usage: %s <options> filename\n", argv[0]);
  fprintf(stderr, "\n  Where the\n");
  fprintf(stderr, "      <options> can be any of:\n");
  fprintf(stderr, "%30s : display simulation file preparation information\n",    "-verbose");
}


