#ifndef H_opc
#define H_opc

/* The number of "distinct" ISAs in this table-set */
#define  OPC_UNION_ISA      0
#define  OPC_PPC_ISA        1
#define  OPC_PPC32_ISA      2
#define  OPC_PPC40X_ISA     3
#define  OPC_PPC601_ISA     4
#define  OPC_POWER_ISA      5
#define  OPC_POWER2_ISA     6
#define  OPC_AMAZON_ISA     7
#define  OPC_PPC64on32_ISA  8
#define  OPC_NUM_ISAs       9

/* Special flags for the dlength */
#define OPC_DLENGTH_MAX_PRESET_LENGTH   16

#define OPC_DLENGTH_RT_R31   128
#define OPC_DLENGTH_NB       129
#define OPC_DLENGTH_RS_R31   130

typedef unsigned short opc_idx;
typedef unsigned char opc_reg;

#include "opc_decl.h"
#include "opc_fields.h"
#include "opc_regs.h"
#include "opc_private.h"
#include "opc_asm.h"
#include "opc_ff32.h"
#include "opc_imul.h"
#include "opc_idiv.h"
#include "opc_memop.h"
#include "opc_instr_info.h"

extern int opc_the_current_active_isa;

char *	opc_reg_names( opc_reg reg );
#define opc_reg_names(reg)	xxopc_reg_names(reg)

/* Instruction Names */
char *  opc_name(int the_ISA, opc_idx index );
#define opc_isa_name(_S,_idx)  xxopc_isa_names(_S,_idx)
#define opc_name(_idx)         xxopc_isa_names(opc_the_current_active_isa,_idx)

char * opc_ppc_name(opc_idx index);
#define opc_ppc_names(_idx)	xxopc_isa_names(OPC_PPC_ISA,_idx)

char * opc_ppc32_name(opc_idx index);
#define opc_ppc32_names(_idx)	xxopc_isa_names(OPC_PPC32_ISA,_idx)

char * opc_ppc601_name(opc_idx index);
#define opc_ppc601_names(_idx)	xxopc_isa_names(OPC_PPC601_ISA,_idx)

char * opc_amazon_name(opc_idx index);
#define opc_amazon_names(_idx)	xxopc_isa_names(OPC_AMAZON_ISA,_idx)

char * opc_power_name(opc_idx index);
#define opc_power_names(_idx)	xxopc_isa_names(OPC_POWER_ISA,_idx)
#define opc_pwr_names(_idx)	xxopc_isa_names(OPC_POWER_ISA,_idx)

char * opc_power2_name(opc_idx index);
#define opc_power2_names(_idx)	xxopc_isa_names(OPC_POWER2_ISA,_idx)
#define opc_pwr2_names(_idx)	xxopc_isa_names(OPC_POWER2_ISA,_idx)

/* The instruction index */
opc_idx opc_index( unsigned insn);
#define opc_index( insn )	xxopc_index(insn)

opc_idx	opc_index_for_name(const char * name);

opc_idx opc_ISA_index( unsigned insn, unsigned the_ISA);
#define opc_ISA_index( insn, isa )	xxopc_index(insn)

int	opc_dlength( opc_idx index);
#define opc_dlength( idx ) xxopc_dlength(idx)
int	opc_iword_dlength( unsigned int iword);

unsigned int opc_opcode( unsigned insn);
#define opc_opcode( insn )      ((insn) >> 26)

int opc_extended_op( unsigned insn);
#define opc_extended_op( insn )	xxopc_extended_op(insn)

int	opc_sets( unsigned insn, opc_reg sets[]);
int	opc_uses( unsigned insn, opc_reg uses[]);

/* These arrays are all now of dimension OPC_NUM_USE_DEF_VECS */
#define OPC_NUM_USE_DEF_VECS   5
void	opc_setvecs( unsigned insn, unsigned sets[]);
void	opc_usevecs( unsigned insn, unsigned uses[]);
void    opc_use_def_vecs( unsigned iword, unsigned uvec[], unsigned dvec[]);
void    opc_full_use_def_vecs( unsigned iword, unsigned uvec[], unsigned dvec[], unsigned tvec[]);

int	opc_legal( unsigned insn );

unsigned opc_form_iword_from_index_and_operands( opc_idx index, int operands[] );

void	opc_dis_isa( unsigned the_ISA, unsigned insn, char * buf );
#define opc_dis(_i,_b)  opc_dis_isa(opc_the_current_active_isa,_i,_b)
#define opc_dis_isa(_S,_i,_b)  opc_dis_isa(_S,_i,_b)

void	opc_disas_isa( unsigned the_ISA, unsigned insn, char * buf );
#define opc_disas(_i,_b)  opc_disas_isa(opc_the_current_active_isa,_i,_b)
#define opc_disas_isa(_S,_i,_b)  opc_disas_isa(_S,_i,_b)

void	opc_ISA_init ( int the_ISA );
void    opc_init( void );


#endif
