#include "ansi_stdlib.h"
#include "ansi_stdio.h"
#include "ansi_assert.h"
#include "ansi_string.h"
#include "opc.h"

__thread int opc_the_current_active_isa = OPC_UNION_ISA;

/**************************************************************/
__thread char opc_tblreg_names[256][8] = {
/*  0 */   "",
/*  1 */  "ctr",
/*  2 */  "xer",
/*  3 */  "lr",
/*  4 */  "fpscr",
/*  5 */  "mq",
/*  6 */  "dsisr",
/*  7 */  "dar",
/*  8 */  "rtcu",
/*  9 */  "rtcl",
/* 10 */  "dec",
/* 11 */  "sdr0",
/* 12 */  "sdr1",
/* 13 */  "sdr2",
/* 14 */  "srr0",
/* 15 */  "srr1",
/* 16 */  "srr2",
/* 17 */  "srr3",
/* 18 */  "msr",
/* 19 */  "tb",
/* 20 */  "tbu",
/* 21 */  "SR0_15",
/* 22 */  "dc",
/* 23 */  "so",
/* 24 */  "ov",
/* 25 */  "ca",
/* 26 */  "oc",
/* 27 */  "fxcc",
/* 28 */  "ds",
/* 29 */  "tgcc",
/* 30 */  "bcnt",
/* 31 */  "bcmp"
};

static void
opc_reg_names_init( void )
{
  int	i;

  for( i = 0; i < 256; i++ )  {
    if( opc_tblreg_names[i][0] == 0 ) {
      sprintf( opc_tblreg_names[i], "?%d", i);
    }
  }

  for( i = 0; i < 8; i++  ) {
    sprintf( opc_tblreg_names[OPC_FIRST_CR+i], "cr%d", i);
  }

  for( i = 0; i < 32; i++  ) {
    sprintf( opc_tblreg_names[OPC_FIRST_CRB+i], "crb%d", i);
  }

  for( i = 0; i < 32; i++  ) {
    sprintf( opc_tblreg_names[OPC_FIRST_GPR+i], "r%d", i);
  }

  for( i = 0; i < 32; i++  ) {
    sprintf( opc_tblreg_names[OPC_FIRST_FPR+i], "fp%d", i);
  }

  for( i = 0; i < 16; i++  ) {
    sprintf( opc_tblreg_names[OPC_SR0+i], "sr%d", i);
  }

  for( i = 0; i < 32; i++  ) {
    sprintf( opc_tblreg_names[OPC_FIRST_VECR+i], "v%d", i);
  }

  sprintf( opc_tblreg_names[OPC_FPSCR_ENR],   "enr"); // jdonald extra arg?
  sprintf( opc_tblreg_names[OPC_FPSCR_FPCC],  "fpcc"); // jdonald extra arg?
  sprintf( opc_tblreg_names[OPC_FPSCR_FRFI],  "frfi"); // jdonald extra arg?
  sprintf( opc_tblreg_names[OPC_FPSCR_EXCPT], "fpexcpt"); // jdonald extra arg?
}

/**************************************************************/
__thread char * opc_tbl_names[OPC_NUM+1][OPC_NUM_ISAs] = {
#include "opc_names.gen"
  { 0 } // jdonald braces
};

char *
(opc_name)(int the_ISA, 
	   opc_idx index
	   )
{
  return opc_tbl_names[index][the_ISA];
}


char *
(opc_ppc_name)(opc_idx index)
{
  return opc_tbl_names[index][OPC_PPC_ISA];
}

char *
(opc_ppc32_name)(opc_idx index)
{
  return opc_tbl_names[index][OPC_PPC32_ISA];
}

char *
(opc_ppc601_name)(opc_idx index)
{
  return opc_tbl_names[index][OPC_PPC601_ISA];
}

char *
(opc_amazon_name)(opc_idx index)
{
  return opc_tbl_names[index][OPC_AMAZON_ISA];
}

char *
(opc_power_name)(opc_idx index)
{
  return opc_tbl_names[index][OPC_POWER_ISA];
}

char *
(opc_power2_name)(opc_idx index)
{
  return opc_tbl_names[index][OPC_POWER2_ISA];
}


/**************************************************************/
__thread char	opc_tbl_dlength[] = {
#include "opc_dlength.gen"
  0
};

int
(opc_dlength)(opc_idx index)
{
  return opc_tbl_dlength[index];
}

int
(opc_iword_dlength)(unsigned int iword) 
{
  opc_idx index = opc_index(iword);
// jdonald setting the_len to unsigned char as it should be
  unsigned char the_len = (unsigned char) opc_tbl_dlength[index];
  if (the_len > OPC_DLENGTH_MAX_PRESET_LENGTH) { 
    switch(the_len) {
    case OPC_DLENGTH_RT_R31:
      the_len = (32 - OPC_DEC_RT(iword)) << 2;
      break;
      
    case OPC_DLENGTH_NB:
      the_len = OPC_DEC_NB(iword);
      break;
      
    case OPC_DLENGTH_RS_R31:
      the_len = (32 - OPC_DEC_RS(iword)) << 2;
      break;
      
    default:
      break;
    }
  }
  return the_len;
}


/**************************************************************/
__thread char	opc_tbl_is_in_ISA[OPC_NUM+1][OPC_NUM_ISAs] = {
#include "opc_is_in_ISA.gen"
  { OPC_IS_NOT_IN_ISA , OPC_IS_NOT_IN_ISA , OPC_IS_NOT_IN_ISA , OPC_IS_NOT_IN_ISA , OPC_IS_NOT_IN_ISA , OPC_IS_NOT_IN_ISA  }
};

int
(opc_is_in_ISA)(int the_ISA, 
		opc_idx index)
{
  return (opc_tbl_is_in_ISA[index][the_ISA] != 0);
}

int
(opc_is_in_ISA_typed)(int the_ISA, 
		      opc_idx index)
{
  return opc_tbl_is_in_ISA[index][the_ISA];
}


/**************************************************************/

__thread unsigned short opc_isa_map_set[OPC_NUM_ISAs] = {0,0,0,0,0,0,1,0};

__thread unsigned short opc_tblidx_index[OPC_NUM_ISA_SETS][OPC_TBL_SIZE];
__thread unsigned short opc_tblidx_init[64];
__thread unsigned short opc_tblidx_mask[64];
__thread unsigned short opc_tblidx_max;

opc_idx
(opc_index)(
	unsigned iword
	)
{
  unsigned	major = iword >> 26;
  unsigned	mask  = opc_tblidx_mask[major];
  unsigned	init  = opc_tblidx_init[major];
  unsigned	minor = iword & mask;
  unsigned	index = init+minor;

  assert( index < opc_tblidx_max );
  // assert( index >= 0 ); // meaningless unsigned assertion -jdonald
  return opc_tblidx_index[(opc_the_current_active_isa == OPC_AMAZON_ISA)][index];
}


opc_idx
(opc_ISA_index)(
	unsigned iword,
	unsigned the_ISA
	)
{
  unsigned	major = iword >> 26;
  unsigned	mask  = opc_tblidx_mask[major];
  unsigned	init  = opc_tblidx_init[major];
  unsigned	minor = iword & mask;
  unsigned	index = init+minor;

  assert( index < opc_tblidx_max );
  // assert( index >= 0 ); // meaningless unsigned assertion -jdonald
  return opc_tblidx_index[(the_ISA == OPC_AMAZON_ISA)][index];
}


static __thread short opc_tblidx_setup[][4] = {
#include "opc_tblsetup.gen"
  {-1, -1, -1, -1}
};

static __thread char opc_tblidx_sizes[] = {
#include "opc_tblsizes.gen"
};

static void
opc_index_init(
	void
	)
{
  int		i;
  unsigned	j;
  unsigned	val;
  unsigned	init;
  unsigned	major;
  unsigned	bits;
  unsigned	minor;
  unsigned	index;
  unsigned	mask;

  /* Set up the opcode masks and pointers */
  init = 0;
  for( i = 0; i < 64; i++ ) {
    val		       = 1 << opc_tblidx_sizes[i];
    mask	       = val - 1;
    opc_tblidx_init[i] = init;
    opc_tblidx_mask[i] = mask;
    init              += val;
    assert( init <= OPC_TBL_SIZE );
  }
  opc_tblidx_max = init;

  /* 
     printf("The table size (max) = %d\n", opc_tblidx_max);
    */

  /* Set up the contents of the index table */
  for( i = 0; opc_tblidx_setup[i][0] != -1; i++ ) {
    major = opc_tblidx_setup[i][0];
    bits  = opc_tblidx_setup[i][1];
    minor = opc_tblidx_setup[i][2];
    index = opc_tblidx_setup[i][3];
    init  = opc_tblidx_init[major];
    mask  = opc_tblidx_mask[major];
    val   = 1 << bits;

    for( j = minor; j <= mask; j += val ) {
      assert( init+j < opc_tblidx_max );
      if (major != 56) {
	opc_tblidx_index[OPC_OTHERS_ISA_SET][init+j] = index;
	opc_tblidx_index[OPC_AMAZON_ISA_SET][init+j] = index;
      }
      else { /* major = 56 => LQ vs. LFQ */
	opc_tblidx_index[OPC_OTHERS_ISA_SET][init+j] = OPC_LFQ;
	opc_tblidx_index[OPC_AMAZON_ISA_SET][init+j] = OPC_LQ;
      }
      /**
	printf("%3d  %3d  index[OPC_AMAZON_ISA_SET][%d] = %d\n", major, minor, (init+j), opc_tblidx_index[OPC_AMAZON_ISA_SET][init+j]);
	**/
    }    
  }
}


/**************************************************************/

unsigned int 
(opc_opcode)(unsigned iword
	     )
{
  unsigned	major = iword >> 26;
  return major;
}



/**************************************************************/

__thread short opc_tbl_extended_op[OPC_NUM+1] = {
#include "opc_extended_op.gen"
  0
};

int
(opc_extended_op)(unsigned iword
		  )
{
  return (int)(opc_tbl_extended_op[opc_index(iword)]);
}


/**************************************************************/

int
opc_legal(
	unsigned	iword
	)
{
  opc_idx	index = opc_index(iword);
  unsigned	ok = 1;

  switch( index ) {
#include "opc_legal.gen"
  default: {
      ok = 0;
      break;
    }
  }
  return ok;
}



/**************************************************************/

__thread struct {
  int		index;
  char *	name;
} opc_spr_tbl_setup[] =  {
  { 0,		"MQ"		},
  { 1,		"XER"		},
  { 8,		"LR"		},
  { 9,		"CTR"		},
  { 18,		"DSISR"		},
  { 19,		"DAR"		},
  { 20,		"RTCU"		},
  { 21,		"RTCL"		},
  { 22,		"DEC"		},
  { 25,		"SDR1"		},
  { 26,		"SRR0"		},
  { 27,		"SRR1"		},
  { 268,	"TBL"		},
  { 269,	"TBU"		},
  { 272,	"SPRG0"		},
  { 273,	"SPRG1"		},
  { 274,	"SPRG2"		},
  { 275,	"SPRG3"		},
  { 282,	"EAR"		},
  { 284,	"TBL"		},
  { 285,	"TBU"		},
  { 287,	"PVR"		},
  { 528,	"IBAT0U"	},
  { 529,	"IBAT0L"	},
  { 530,	"IBAT1U"	},
  { 531,	"IBAT1L"	},
  { 532,	"IBAT2U"	},
  { 533,	"IBAT2L"	},
  { 534,	"IBAT3U"	},
  { 535,	"IBAT3L"	},
  { 536,	"DBAT0U"	},
  { 537,	"DBAT0L"	},
  { 538,	"DBAT1U"	},
  { 539,	"DBAT1L"	},
  { 540,	"DBAT2U"	},
  { 541,	"DBAT2L"	},
  { 542,	"DBAT3U"	},
  { 543,	"DBAT3L"	},
  { 952,	"MMCR0"		},
  { 953,	"PMC1"		},
  { 954,	"PMC2"		},
  { 955,	"SIA"		},
  { 956,	"MMCR1"		},
  { 957,	"PMC3"		},
  { 958,	"PMC4"		},
  { 959,	"SDA"		},
  { 976,	"DMISS"		},
  { 977,	"DCMP"		},
  { 978,	"HASH1"		},
  { 979,	"HASH2"		},
  { 980,	"IMISS"		},
  { 981,	"ICMP"		},
  { 982,	"RPA"		},
  { 1008,	"HID0"		},
  { 1009,	"HID1"		},
  { 1010,	"IABR"		},
  { 1013,	"DABR"		},
  { 1023,	"PIR"		},
  { 0,		0		}
};

__thread char 	opc_spr_tbl[1024][8];

void
opc_spr_init( void )
{
  int	i;

  for( i = 0; i < 1024; i++ ) {
    sprintf(opc_spr_tbl[i], "%d", i);
  }

#if 0
  for( i = 0; opc_spr_tbl_setup[i].name; i++ ) {
    strcpy( opc_spr_tbl[opc_spr_tbl_setup[i].index], opc_spr_tbl_setup[i].name);
    /*printf("opc_spr_tbl[%d] = %s\n", i, opc_spr_tbl[i]);*/
  }
#endif

}

void
opc_dis_isa(unsigned the_ISA,
	    unsigned iword,
	    char *   buf
	    )
{
  opc_idx	index = opc_index(iword);

  switch( index ) {
#include "opc_dis.gen"
    default: {
      sprintf(buf, "???");
    }
  }
}

void
opc_disas_isa(unsigned the_ISA,
	      unsigned iword,
	      char *   buf
	      )
{
  opc_idx index = opc_index(iword);
  
  switch( index ) {
#include "opc_disas.gen"
    default: {
      sprintf(buf, "???");
    }
  }
}


/**************************************************************/

__thread unsigned opc_tblsprenc[1024];

static __thread unsigned opc_tblsprenc_init[][3] = {
  {  0, OPC_MQ     },
  {  1, -1 },  /* Signal problem for XER */
  {  4, OPC_RTCU   },
  {  5, OPC_RTCL   },
  {  8, OPC_LR     },
  {  9, OPC_CTR    },
  { 18, OPC_DSISR  },
  { 19, OPC_DAR    },
  { 20, OPC_RTCU   },
  { 21, OPC_RTCL   },
  { 22, OPC_SDR1   },
  { 26, OPC_SRR0   },
  { 27, OPC_SRR1   },  
  { -1, -1 }
};

__thread unsigned opc_tblxbienc[32];

static __thread unsigned opc_tblxbienc_init[][3] = {
  {  0, OPC_XER_SO },
  {  1, OPC_XER_OV },  
  {  2, OPC_XER_CA },
  {  3, OPC_XER_OC },
  {  4, OPC_XER_FXCC },
  {  5, OPC_XER_FXCC },
  {  6, OPC_XER_FXCC },
  {  7, OPC_XER_FXCC },
  {  8, OPC_XER_DS },
  {  9, OPC_XER_TGCC },
  { 10, OPC_XER_TGCC },
  { 11, OPC_XER_TGCC },
  { 17, OPC_XER_BCMP },
  { 18, OPC_XER_BCMP },
  { 19, OPC_XER_BCMP },
  { 20, OPC_XER_BCMP },
  { 21, OPC_XER_BCMP },
  { 22, OPC_XER_BCMP },
  { 23, OPC_XER_BCMP },
  { 24, OPC_XER_BCMP },
  { 25, OPC_XER_BCNT },
  { 26, OPC_XER_BCNT },
  { 27, OPC_XER_BCNT },  
  { 28, OPC_XER_BCNT },
  { 29, OPC_XER_BCNT },
  { 30, OPC_XER_BCNT },
  { 31, OPC_XER_BCNT },
  { -1, -1 }
};

static void
opc_enctbls_init(
	void
	)
{
  int		i;
  unsigned	value;
  unsigned	reg;

  for( i =  0; opc_tblsprenc_init[i][0] != -1; i++ ) {
    value = opc_tblsprenc_init[i][0];
    reg = opc_tblsprenc_init[i][1];

    opc_tblsprenc[value] = reg;
  }

  for( i =  0; opc_tblxbienc_init[i][0] != -1; i++ ) {
    value = opc_tblxbienc_init[i][0];
    reg = opc_tblxbienc_init[i][1];

    opc_tblxbienc[value] = reg;
  }
}


int
opc_uses(
	unsigned	iword,
	opc_reg		uses[]
	)
{
  opc_idx	index = opc_index(iword);
  unsigned	spr;
  unsigned	xbi;
  unsigned	flm, bf, bfa, bt;
  int		n;
  int		nuses = 0;
  int		i;

  switch( index ) {
#include "opc_uses.gen"
  default:
    break;
  }
  return nuses;
}

int
opc_sets(
	unsigned	iword,
	opc_reg		sets[]
	)
{
  opc_idx	index = opc_index(iword);
  unsigned	spr;
  unsigned	xbi;
  unsigned	flm, bf, bfa, bt;
  int		nsets = 0;
  int		n;
  int		m;
  int		i;
  int		j;

  switch( index ) {
#include "opc_sets.gen"
  default:
    break;
  }
  return nsets;
}

/**************************************************************/

static __thread unsigned opc_tblvecsprenc[1024];
static __thread unsigned opc_tblvecxbienc[32];
static __thread unsigned char crm_swap[256];

static void
opc_vecenctbls_init(
	void
	)
{
  int		i;
  unsigned	value;
  unsigned	reg;

  for( i =  0; opc_tblsprenc_init[i][0] != -1; i++ ) {
    value = opc_tblsprenc_init[i][0];
    reg = opc_tblsprenc_init[i][1];

    opc_tblvecsprenc[value] = 1<<reg;
  }

  for( i =  0; opc_tblxbienc_init[i][0] != -1; i++ ) {
    value = opc_tblxbienc_init[i][0];
    reg = opc_tblxbienc_init[i][1];

    opc_tblvecxbienc[value] = 1<<reg;
  }

}

static void
opc_crm_swap_init(
	void
	)
{
  unsigned	i;
  unsigned	swap;
  unsigned	mask;
  unsigned	val;
  unsigned	j;

  for( i = 0; i < 256; i++ ) {
    swap = 0;
    val = i;
    mask = 1<<7;
    for( j = 0; j < 8; j++ ) {
      if( val & 1 ) {
	swap |= mask;
      }
      val >>= 1;
      mask >>= 1;
    }
    crm_swap[i] = swap;
  }

  #if 0
  for( i = 0; i < 256; i++ ) {
    fprintf(stderr, "%02x %02x\n", i, crm_swap[i]);
  }
  #endif
}

void
opc_usevecs(
	unsigned	iword,
	unsigned	vec[]
	)
{
  opc_idx	index = opc_index(iword);
  unsigned	gpr = 0;
  unsigned	fpr = 0;
  unsigned	cr  = 0;
  unsigned	spr = 0;
  unsigned	xer = 0;
  unsigned	flm, bf, bfa, bt;
  int		n;
  int		m;

  switch( index ) {
#include "opc_usevecs.gen"
  }
  vec[0] = gpr;
  vec[1] = fpr;
  vec[2] = cr;
  vec[3] = spr;
  vec[4] = xer;
}

void
opc_setvecs(
	unsigned	iword,
	unsigned	vec[]
	)
{
  opc_idx	index = opc_index(iword);
  unsigned	gpr = 0;
  unsigned	fpr = 0;
  unsigned	cr  = 0;
  unsigned	spr = 0;
  unsigned	xer = 0;
  unsigned	flm, bf, bfa, bt; // jdonald: bf often (but not always) unused
  int		n;
  int		m;

  switch( index ) {
#include "opc_setvecs.gen"
  }
  vec[0] = gpr;
  vec[1] = fpr;
  vec[2] = cr;
  vec[3] = spr;
  vec[4] = xer;
}


void
opc_use_def_vecs(
		 unsigned	iword,
		 unsigned	uvec[],
		 unsigned	dvec[]
		 )
{
  opc_idx	index = opc_index(iword);
  unsigned	gpr = 0;
  unsigned	fpr = 0;
  unsigned	cr = 0;
  unsigned	spr = 0;
  unsigned	xer = 0;
  unsigned	flm, bf, bfa, bt;
  int		n;
  int		m;

  switch( index ) {
#include "opc_usesetvecs.gen"
  default:
    break;
  }

  dvec[0] = gpr;
  dvec[1] = fpr;
  dvec[2] = cr;
  dvec[3] = spr;
  dvec[4] = xer;
}

void
opc_full_use_def_vecs(
		      unsigned	iword,
		      unsigned	uvec[],
		      unsigned	dvec[],
		      unsigned	tvec[]
	)
{
  opc_idx	index = opc_index(iword);
  unsigned	gpr = 0;
  unsigned	fpr = 0;
  unsigned	cr = 0;
  unsigned	spr = 0;
  unsigned	xer = 0;
  unsigned	flm, bf, bfa, bt;
  int		n;
  int		m;

  switch( index ) {
#include "opc_usesetvecs.gen"
  default:
    break;
  }

  dvec[0] = gpr;
  dvec[1] = fpr;
  dvec[2] = cr;
  dvec[3] = spr;
  dvec[4] = xer;

  tvec[0] = uvec[0] | dvec[0];
  tvec[1] = uvec[1] | dvec[1];
  tvec[2] = uvec[2] | dvec[2];
  tvec[3] = uvec[3] | dvec[3];
  tvec[4] = uvec[4] | dvec[4];
}


/**************************************************************/

#include "opc_memop.h"

__thread unsigned char opc_memop_tbl[] = {
#include "opc_memop.gen"
  OPC_MEMOP_NOT
};

unsigned char
(opc_is_memop)( opc_idx idx)
{
  return opc_is_memop(idx);
}

unsigned char
(opc_is_memop_store)( opc_idx idx)
{
  return opc_is_memop_store(idx);
}

unsigned char
(opc_is_memop_load)( opc_idx idx)
{
  return opc_is_memop_load(idx);
}

unsigned char
(opc_is_memop_cache)( opc_idx idx)
{
  return opc_is_memop_cache(idx);
}


/**************************************************************/

#include "opc_instr_info.h"

__thread unsigned short opc_itypes_tbl[] = {
#include "opc_itypes.gen"
  OPC_ILLEGAL_INSTR
};

__thread unsigned char opc_iformats_tbl[] = {
#include "opc_iformats.gen"
  OPC_ILLEGAL_FORMAT
};

__thread unsigned char opc_ifformats_tbl[] = {
#include "opc_fullformats.gen"
  OPC_ILLEGAL_FORMAT
};

__thread unsigned char opc_is_float_tbl[] = {
#include "opc_ifloats.gen"
  OPC_IS_NOT_A_FLOAT_OP
};

__thread unsigned char opc_is_vmx_tbl[] = {
#include "opc_iVMX.gen"
  OPC_IS_NOT_A_VMX_OP
};

__thread unsigned char opc_does_update_tbl[] = {
#include "opc_iupdates.gen"
  OPC_DOES_NOT_DO_AN_UPDATE
};

__thread unsigned char opc_does_record_tbl[] = {
#include "opc_irecords.gen"
  OPC_DOES_NOT_DO_A_RECORD
};

/**
  __thread unsigned char opc_is_user_PPC_tbl[] = {
  #include "opc_legal_user_ppc.gen"
  OPC_IS_NOT_A_LEGAL_USER_PPC_OP
  };
  
  __thread unsigned char opc_is_user_POWER_tbl[] = {
  #include "opc_legal_user_power.gen"
  OPC_IS_NOT_A_LEGAL_USER_POWER_OP
  };
  **/

unsigned
(opc_itypes)( opc_idx idx)
{
  return opc_itypes_tbl[idx];
}

unsigned
(opc_iword_itypes)( unsigned iword)
{
  return opc_itypes_tbl[opc_index(iword)];
}


unsigned
(opc_iformat)( opc_idx idx)
{
  return opc_iformats_tbl[idx];
}

unsigned
(opc_iword_iformat)( unsigned iword)
{
  return opc_iformats_tbl[opc_index(iword)];
}

unsigned
(opc_ifformat)( opc_idx idx)
{
  return opc_ifformats_tbl[idx];
}

unsigned
(opc_iword_ifformat)( unsigned iword)
{
  return opc_ifformats_tbl[opc_index(iword)];
}

unsigned
(opc_is_float)( opc_idx idx)
{
  return opc_is_float_tbl[idx];
}

unsigned
(opc_iword_is_float)( unsigned iword)
{
  return opc_is_float_tbl[opc_index(iword)];
}

unsigned
(opc_does_update)( opc_idx idx)
{
  return opc_does_update_tbl[idx];
}

unsigned
(opc_iword_does_update)( unsigned iword)
{
  return opc_does_update_tbl[opc_index(iword)];
}

unsigned
(opc_does_record)( opc_idx idx)
{
  return opc_does_record_tbl[idx];
}

unsigned
(opc_iword_does_record)( unsigned iword)
{
  return opc_does_record_tbl[opc_index(iword)];
}

/**
   unsigned
   (opc_is_user_PPC_instr)( opc_idx idx)
   {
   return opc_is_user_PPC_tbl[idx];
   }
   
   unsigned
   (opc_iword_is_user_PPC_instr)( unsigned iword)
   {
   return opc_is_user_PPC_tbl[opc_index(iword)];
   }
   
   unsigned
   (opc_is_user_POWER_instr)( opc_idx idx)
   {
   return opc_is_user_POWER_tbl[idx];
   }
   
   unsigned
   (opc_iword_is_user_POWER_instr)( unsigned iword)
   {
   return opc_is_user_POWER_tbl[opc_index(iword)];
   }
   **/


/**************************************************************/

#include "opc_ff32.h"

__thread unsigned char opc_ff32_cattbl[] = {
#include "opc_ff32cat.gen"
  OPC_FF32_ILLEGAL
};

unsigned char
(opc_ff32_cat)( opc_idx idx)
{
  return opc_ff32_cattbl[idx];
}


/**************************************************************/

__thread unsigned char opc_imul_tbl[] = {
#include "opc_imul.gen"
  0
};

unsigned char
(opc_is_imul)( opc_idx idx)
{
  return opc_imul_tbl[idx];
}



/**************************************************************/

__thread unsigned char opc_idiv_tbl[] = {
#include "opc_idiv.gen"
  0
};

unsigned char
(opc_is_idiv)( opc_idx idx)
{
  return opc_idiv_tbl[idx];
}


/**************************************************************/
unsigned
(opc_form_iword_from_index_and_operands)(opc_idx index, int ops[]) 
{
  unsigned int iword;

  switch (index) {
    #include "opc_integrate_operands_to_iword.gen"
  default:
    fprintf(stderr, "ERROR in opc_form_iword_from_index_and_operands...\n");
    fprintf(stderr, "  Illegal index %d \n", index);
    iword = 0x00000000;
    break;
  }
  
  return iword;
}


/**************************************************************/

#include "strhash.h"

#define OPC_NAME_HASH_SIZE	4096
static __thread struct strhash *	opc_name_hash_tbl;

void
opc_name_hash_init(void)
{
  unsigned	i;
  int		j;
  char *	opcode;
  unsigned		n;
  int		status;

  opc_name_hash_tbl = make_strhash(OPC_NAME_HASH_SIZE);
  if( opc_name_hash_tbl == 0 ) {
    fprintf(stderr, "out of memory trying to build name hash table\n");
  }


  for( i = 1; i < OPC_NUM; i++ ) {
    for( j = 0; j < OPC_NUM_ISAs; j++ ) {
      opcode = opc_tbl_names[i][j];

      if( opcode == 0 || strcmp(opcode, "ILL" ) == 0  || strcmp( opcode, "") == 0 )  {
	continue;
      }

      if( is_in_strhash(opc_name_hash_tbl, opcode) ) {
	n = get_strhash(opc_name_hash_tbl, opcode);
	assert( i == n );
	continue;
      }

      status = update_strhash(opc_name_hash_tbl, opcode, i);
      if( status == 0 ) {
	fprintf(stderr, "out of memory trying to add entry to name hash table\n");
      }
    }
  }
}

opc_idx
opc_index_for_name(
	const char *	opcode
	)
{
  return get_strhash(opc_name_hash_tbl, opcode);
}



/**************************************************************/

void
opc_ISA_init(int the_ISA)
{
  opc_the_current_active_isa = the_ISA;
  opc_init();
}

static __thread int lcl_opc_initialized = 0;
void
opc_init( void )
{
  if( lcl_opc_initialized ) {
    return;
  }

  lcl_opc_initialized = 1;
  opc_index_init();
  opc_enctbls_init();
  opc_spr_init();
  opc_vecenctbls_init();
  opc_crm_swap_init();
  opc_reg_names_init();
  opc_name_hash_init();
}
