#include "opc.h"
#include "opc_as.h"
#include "ansi_stdlib.h"
#include "ansi_assert.h"
#include "ansi_string.h"
#include "ansi_ctype.h"
#include "ansi_stdio.h"

#define OPC_ERROR_AS_TOOFEW		-1
#define OPC_ERROR_AS_TOOMANY		-2
#define OPC_ERROR_AS_NOTGPR_1		-3
#define OPC_ERROR_AS_NOTGPR_2		-4
#define OPC_ERROR_AS_NOTGPR_3		-5

#define OPC_ERROR_AS_NOTFPR_1		-6
#define OPC_ERROR_AS_NOTFPR_2		-7
#define OPC_ERROR_AS_NOTFPR_3		-8

#define OPC_ERROR_AS_NOTCBR_1		-9
#define OPC_ERROR_AS_NOTCBR_2		-10
#define OPC_ERROR_AS_NOTCBR_3		-11

#define OPC_ERROR_AS_NOTCFR_1		-12
#define OPC_ERROR_AS_NOTCFR_2		-13
#define OPC_ERROR_AS_NOTCFR_3		-14

#define OPC_ERROR_AS_NOTNUM		-15
#define OPC_ERROR_AS_URANGE		-16
#define OPC_ERROR_AS_SRANGE		-17
#define OPC_ERROR_AS_OPCODE		-18

#define DEBUG 1

#if( DEBUG)
#define DBG(x)	((void)(x))
#else
#define DBG(x)	((void)0)
#endif

#define GET_GPR(x, str)					\
  {							\
    assert( strcmp(#x, str) == 0);			\
    if( m_args_at >= nargs ) {				\
      m_error = OPC_ERROR_AS_TOOFEW;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    m_arg = args[m_args_at];				\
    if( m_arg[0] != 'r' ) {				\
      DBG(fprintf(stderr, "gpr_1: %s\n", m_arg));	\
      m_error = OPC_ERROR_AS_NOTGPR_1;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    x = strtoul( m_arg+1, &m_arg_end, 10 );		\
    if( *m_arg_end != 0 ) {				\
      m_error = OPC_ERROR_AS_NOTGPR_2;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    if( x >= 32 ) {					\
      m_error = OPC_ERROR_AS_NOTGPR_3;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    m_args_at++;					\
  }

#define GET_FPR(x, str)					\
  {							\
    assert( strcmp(#x, str) == 0);			\
    if( m_args_at >= nargs ) {				\
      m_error = OPC_ERROR_AS_TOOFEW;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    m_arg = args[m_args_at];				\
    if( m_arg[0] != 'f' || m_arg[1] != 'p' ) {		\
      m_error = OPC_ERROR_AS_NOTFPR_1;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    x = strtoul( m_arg+2, &m_arg_end, 10 );		\
    if( *m_arg_end != 0 ) {				\
      m_error = OPC_ERROR_AS_NOTFPR_2;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    if( x >= 32 ) {					\
      m_error = OPC_ERROR_AS_NOTFPR_3;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    m_args_at++;					\
  }

#define GET_CBR(x, str)					\
  {							\
    assert( strcmp(#x, str) == 0);			\
    if( m_args_at >= nargs ) {				\
      m_error = OPC_ERROR_AS_TOOFEW;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    m_arg = args[m_args_at];				\
    if( m_arg[0] != 'c' || m_arg[1] != 'b' ) {		\
      m_error = OPC_ERROR_AS_NOTCBR_1;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    x = strtoul( m_arg+2, &m_arg_end, 10 );		\
    if( *m_arg_end != 0 ) {				\
      m_error = OPC_ERROR_AS_NOTCBR_2;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    if( x >= 32 ) {					\
      m_error = OPC_ERROR_AS_NOTCBR_3;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    m_args_at++;					\
  }

#define GET_CFR(x, str)					\
  {							\
    assert( strcmp(#x, str) == 0);			\
    if( m_args_at >= nargs ) {				\
      m_error = OPC_ERROR_AS_TOOFEW;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    m_arg = args[m_args_at];				\
    if( m_arg[0] != 'c' || m_arg[1] != 'r' ) {		\
      m_error = OPC_ERROR_AS_NOTCFR_1;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    x = strtoul( m_arg+2, &m_arg_end, 10 );		\
    if( *m_arg_end != 0 ) {				\
      m_error = OPC_ERROR_AS_NOTCFR_2;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    if( x >= 8 ) {					\
      m_error = OPC_ERROR_AS_NOTCFR_3;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    m_args_at++;					\
  }

#define GET_VECR(x, str)				\
  {							\
    assert( strcmp(#x, str) == 0);			\
    if( m_args_at >= nargs ) {				\
      m_error = OPC_ERROR_AS_TOOFEW;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    m_arg = args[m_args_at];				\
    if( m_arg[0] != 'v' ) {				\
      DBG(fprintf(stderr, "vecr_1: %s\n", m_arg));	\
      m_error = OPC_ERROR_AS_NOTGPR_1;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    x = strtoul( m_arg+1, &m_arg_end, 10 );		\
    if( *m_arg_end != 0 ) {				\
      m_error = OPC_ERROR_AS_NOTGPR_2;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    if( x >= 32 ) {					\
      m_error = OPC_ERROR_AS_NOTGPR_3;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    m_args_at++;					\
  }

#define GET_UNUM(bits, x, str)				\
  {							\
    assert( strcmp(#x, str) == 0 );			\
    if( m_args_at >= nargs ) {				\
      m_error = OPC_ERROR_AS_TOOFEW;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    m_arg = args[m_args_at];				\
    x = strtoul( m_arg, &m_arg_end, 0 );		\
    if( *m_arg_end != 0 ) {				\
      m_error = OPC_ERROR_AS_NOTNUM;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    if( (x>>bits) != 0 ) {				\
      m_error = OPC_ERROR_AS_URANGE;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    m_args_at++;					\
  }

#define GET_SNUM(bits, x, str)				\
  {							\
    assert( strcmp(#x, str) == 0 );			\
    if( m_args_at >= nargs ) {				\
      m_error = OPC_ERROR_AS_TOOFEW;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    m_arg = args[m_args_at];				\
    x = strtoul( m_arg, &m_arg_end, 0 );		\
    if( *m_arg_end != 0 ) {				\
      m_error = OPC_ERROR_AS_NOTNUM;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    if( ((int)x >> bits) != 0 && ((int)x >> bits) != ~0 ) {					\
    DBG( fprintf(stderr, "bits = %d, arg=%s hi =%d, x = %d 0x%08x\n", bits, str, (1<<bits), (int)x, x) ); /* jdonald wrongvar? */ \
      m_error = OPC_ERROR_AS_SRANGE;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
    m_args_at++;					\
  }


#define USED_ALL()					\
  {							\
    if( m_args_at != nargs ) {				\
      m_error = OPC_ERROR_AS_TOOMANY;			\
      m_errline = __LINE__;				\
      goto error_found;					\
    }							\
  }



#define	GET_RA(x)		GET_GPR(x, "ra")
#define	GET_RB(x)		GET_GPR(x, "rb")
#define	GET_RS(x)		GET_GPR(x, "rs")
#define	GET_RT(x)		GET_GPR(x, "rt")

#define	GET_FRA(x)		GET_FPR(x, "fra")
#define	GET_FRB(x)		GET_FPR(x, "frb")
#define	GET_FRC(x)		GET_FPR(x, "frc")
#define	GET_FRS(x)		GET_FPR(x, "frs")
#define	GET_FRT(x)		GET_FPR(x, "frt")

#define GET_BT(x)		GET_CBR(x, "bt")
#define GET_BA(x)		GET_CBR(x, "ba")
#define GET_BB(x)		GET_CBR(x, "bb")
#define GET_BI(x)		GET_CBR(x, "bi")
#define GET_CBI(x)		GET_CBR(x, "cbi")

#define	GET_VA(x)		GET_VECR(x, "va")
#define	GET_VB(x)		GET_VECR(x, "vb")
#define	GET_VC(x)		GET_VECR(x, "vc")
#define	GET_VS(x)		GET_VECR(x, "vs")
#define	GET_VT(x)		GET_VECR(x, "vt")

#define GET_BF(x)		GET_CFR(x, "bf")
#define GET_BFA(x)		GET_CFR(x, "bfa")

#define GET_LI(x)		GET_SNUM(24, x, "li")
#define GET_UI(x)		GET_UNUM(16, x, "ui")
#define GET_SI(x)		GET_SNUM(16, x, "si")
#define GET_UIMM(x)		GET_UNUM(5,  x, "uimm")
#define GET_SIMM(x)		GET_SNUM(5,  x, "simm")
#define GET_D(x)		GET_SNUM(16, x, "d")
#define GET_DQ(x)		GET_SNUM(16, x, "d")
#define GET_DS(x)		GET_SNUM(14, x, "ds")
#define GET_BD(x)		GET_SNUM(14, x, "bd")
#define GET_IS(x)		GET_SNUM(5, x, "is")
#define GET_IB(x)		GET_SNUM(5, x, "ib")

#define GET_BO(x)		GET_UNUM(5, x, "bo")
#define GET_SH(x)		GET_UNUM(5, x, "sh")
#define GET_MB(x)		GET_UNUM(5, x, "mb")
#define GET_ME(x)		GET_UNUM(5, x, "me")
#define GET_NB(x)		GET_UNUM(5, x, "nb")
#define GET_SHB(x)		GET_UNUM(4, x, "sh")

#define GET_Tval(x)		GET_UNUM(1, x, "t")
#define GET_STRM(x)		GET_UNUM(2, x, "strm")

#define GET_L(x)		GET_UNUM(1, x, "l")
#define GET_TO(x)		GET_UNUM(5, x, "to")
#define GET_SR(x)		GET_UNUM(4, x, "sr")
#define GET_SPR(x)		GET_UNUM(10, x, "spr")
#define GET_TBR(x)		GET_UNUM(10, x, "tbr")
#define GET_FXM(x)		GET_UNUM(8, x, "fxm")
#define GET_FLM(x)		GET_UNUM(8, x, "flm")
#define GET_FLM(x)		GET_UNUM(8, x, "flm")

#define GET_SHSH(x)		GET_UNUM(6, x, "shsh")
#define GET_MBW(x)		GET_UNUM(6, x, "mbw")
#define GET_MEW(x)		GET_UNUM(6, x, "mew")
#define GET_LEV(x)		GET_UNUM(6, x, "lev")
#define GET_PT(x)		GET_UNUM(4, x, "pt")
#define GET_XO2(x)		GET_UNUM(10, x, "xo2")
#define GET_U(x)		GET_UNUM(4, x, "u")
#define GET_XBI(x)		GET_UNUM(4, x, "xbi")
#define GET_UIL(x)		GET_UNUM(10, x, "uil")

#include "opc_as.gen"
  
int
opc_as(
	unsigned *	p_iword,
	char *		opcode,
	int		nargs,
	char *		args[]
	)
{
  opc_idx	idx = opc_index_for_name(opcode);

  return opc_as_index(idx, opcode, nargs, args, p_iword);
}

int
opc_as_args(
	unsigned *	p_iword,
	char *		opcode,
	char *		allargs
	)
{
  char		cpyargs[8][256];
  char *	q;
  char *	args[8];
  int		nargs = 0;
  char *	p = allargs;
  // int		size = 0; // unused?
  int		res;

  /* get rid of initial white space */
  while( *p && !isalnum(*p) && *p != '-') {
    p++;
  }

  while( *p ) {

    /* begin token */
    args[nargs] = cpyargs[nargs];
    q = cpyargs[nargs];
    nargs++;

    while( isalnum(*p) || *p == '-' ) {
      *q = *p;
      p++;
      q++;
    }
    *q = 0;

    /* get rid of whitespace/separators */
    while( *p && !isalnum(*p) && *p != '-' ) {
      p++;
    }
  }
  res = opc_as(p_iword, opcode, nargs, args);
  return res;
}



int
opc_as_insn(
	unsigned *	p_iword,
	char *		insn
	)
{
  char		cpyargs[8][256];
  char *	q;
  char *	args[8];
  int		nargs = 0;
  char *	p = insn;
  // int		size = 0; // unused?
  char 		opcode[256];

  /* get rid of initial white space */
  while( *p && !isalnum(*p) ) {
    p++;
  }

  q = opcode;
  while( isalnum(*p) ) {
    *q = *p;
    p++;
    q++;
  }

  if( *p == '.' ) {
    *q = *p;
    q++; // jdonald: there was no reason to write it as *q++
    p++; // ""
  }
  *q = 0;

  while( *p && !isalnum(*p) && *p != '-' ) {
    p++;
  }

  /* DBG(fprintf(stderr, "opcode = %s\n", opcode)); */

  while( *p ) {

    /* begin token */
    args[nargs] = cpyargs[nargs];
    q = cpyargs[nargs];
    nargs++;


    while( isalnum(*p) || *p == '-' ) {
      *q = *p;
      p++;
      q++;
    }
    *q = 0;
    /* DBG(fprintf(stderr, "arg %d = %s\n", nargs-1, args[nargs-1] )); */

    /* get rid of whitespace/separators */
    while( *p && !isalnum(*p) && *p != '-' ) {
      p++;
    }
  }

  return opc_as(p_iword, opcode, nargs, args);
}
