#ifndef H_opc_field
#define H_opc_field

/*************************************************************/
/*  Decoding macros                                          */
/*************************************************************/

#define OPC_DEC_BITS(x, shift, bits) \
  (((x) >> (shift))&(~(~0<<(bits))))

#define OPC_DEC_IBM_BITS(x, msb, lsb) \
  OPC_DEC_BITS(x, 31-lsb, (lsb-msb)+1)

#define OPC_DEC_SBITS(x, lshift, rshift) \
  (((int)((x) << (lshift))) >> (rshift+lshift))

#define OPC_DEC_IBM_SBITS(x, msb, lsb) \
  OPC_DEC_SBITS(x, msb, 31-lsb)


#define OPC_DEC_OPCD(x)   (((unsigned)(x))>>26)
#define OPC_DEC_XO(x)	  (OPC_DEC_IBM_BITS(x,21,30))

#define OPC_DEC_LI(x)	  (OPC_DEC_IBM_SBITS(x,6,29))
#define OPC_DEC_AA(x)     (OPC_DEC_IBM_BITS(x,30,30))
#define OPC_DEC_LK(x)     (OPC_DEC_IBM_BITS(x,31,31))

#define OPC_DEC_BO(x)	    (OPC_DEC_IBM_BITS(x,6,10))
#define OPC_DEC_BO_bit0(x)  (OPC_DEC_IBM_BITS(x,6,6))
#define OPC_DEC_BI(x)	    (OPC_DEC_IBM_BITS(x,11,15))
#define OPC_DEC_BD(x)	    (OPC_DEC_IBM_SBITS(x,16,29))

#define OPC_DEC_BO2(x)     (OPC_DEC_IBM_BITS(x,8,8))

#define OPC_DEC_LEV(x)	  (OPC_DEC_IBM_BITS(x,20,26))

#define OPC_DEC_RT(x)	  (OPC_DEC_IBM_BITS(x,6,10))
#define OPC_DEC_RS(x)	  (OPC_DEC_IBM_BITS(x,6,10))
#define OPC_DEC_RA(x)	  (OPC_DEC_IBM_BITS(x,11,15))
#define OPC_DEC_RB(x)	  (OPC_DEC_IBM_BITS(x,16,20))

#define OPC_DEC_FRT(x)	  (OPC_DEC_IBM_BITS(x,6,10))
#define OPC_DEC_FRS(x)	  (OPC_DEC_IBM_BITS(x,6,10))
#define OPC_DEC_FRA(x)	  (OPC_DEC_IBM_BITS(x,11,15))
#define OPC_DEC_FRB(x)	  (OPC_DEC_IBM_BITS(x,16,20))
#define OPC_DEC_FRC(x)	  (OPC_DEC_IBM_BITS(x,21,25))

#define OPC_DEC_VS(x)	  (OPC_DEC_IBM_BITS(x,6,10))
#define OPC_DEC_VT(x)	  (OPC_DEC_IBM_BITS(x,6,10))
#define OPC_DEC_VA(x)	  (OPC_DEC_IBM_BITS(x,11,15))
#define OPC_DEC_VB(x)	  (OPC_DEC_IBM_BITS(x,16,20))
#define OPC_DEC_VC(x)	  (OPC_DEC_IBM_BITS(x,21,25))
#define OPC_DEC_Tval(x)	  (OPC_DEC_IBM_BITS(x,6,6))
#define OPC_DEC_STRM(x)	  (OPC_DEC_IBM_BITS(x,9,10))
#define OPC_DEC_SHB(x)	  (OPC_DEC_IBM_BITS(x,22,25))
#define OPC_DEC_SIMM(x)	  (OPC_DEC_IBM_SBITS(x,11,15))
#define OPC_DEC_UIMM(x)	  (OPC_DEC_IBM_BITS(x,11,15))
#define OPC_DEC_VRc(x)	  (OPC_DEC_IBM_BITS(x,20,20))

#define OPC_DEC_BF(x)	  (OPC_DEC_IBM_BITS(x,6,8))
#define OPC_DEC_L(x)	  (OPC_DEC_IBM_BITS(x,10,10))

#define OPC_DEC_D(x)	  (OPC_DEC_IBM_SBITS(x,16,31))
#define OPC_DEC_SI(x)	  (OPC_DEC_IBM_SBITS(x,16,31))
#define OPC_DEC_UI(x)	  (OPC_DEC_IBM_BITS(x,16,31))
#define OPC_DEC_DS(x)	  (OPC_DEC_IBM_SBITS(x,16,29))
#define OPC_DEC_DSb(x)	  (OPC_DEC_IBM_SBITS(x,16,29) << 2)

#define OPC_DEC_DQ(x)	  (OPC_DEC_IBM_SBITS(x,16,27))
#define OPC_DEC_DQb(x)	  (OPC_DEC_IBM_SBITS(x,16,27) << 4)

#define OPC_DEC_PT(x)	  (OPC_DEC_IBM_BITS(x,28,31))

#define OPC_DEC_TO(x)	  (OPC_DEC_IBM_BITS(x,6,10))
#define OPC_DEC_BFA(x)	  (OPC_DEC_IBM_BITS(x,11,13))
#define OPC_DEC_SR(x)	  (OPC_DEC_IBM_BITS(x,12,15))
#define OPC_DEC_NB(x)	  (OPC_DEC_IBM_BITS(x,16,20))
#define OPC_DEC_SH(x)	  (OPC_DEC_IBM_BITS(x,16,20))
#define OPC_DEC_U(x)	  (OPC_DEC_IBM_BITS(x,16,19))
#define OPC_DEC_Rc(x)	  (OPC_DEC_IBM_BITS(x,31,31))

#define OPC_DEC_BT(x)	  (OPC_DEC_IBM_BITS(x,6,10))
#define OPC_DEC_BA(x)	  (OPC_DEC_IBM_BITS(x,11,15))
#define OPC_DEC_BB(x)	  (OPC_DEC_IBM_BITS(x,16,20))

#define OPC_DEC_SPR(x)	  (OPC_DEC_IBM_BITS(x,11,15) | (OPC_DEC_IBM_BITS(x,16,20)<<5))

#define OPC_DEC_TBR(x)	  (OPC_DEC_IBM_BITS(x,11,15) | (OPC_DEC_IBM_BITS(x,16,20)<<5))
#define OPC_DEC_FXM(x)	  (OPC_DEC_IBM_BITS(x,12,19))
#define OPC_DEC_XO2(x)	  (OPC_DEC_IBM_BITS(x,11,20))

#define OPC_DEC_FLM(x)	  (OPC_DEC_IBM_BITS(x,7,14))

#define OPC_DEC_SHSH(x)   (OPC_DEC_IBM_BITS(x,16,20) | (OPC_DEC_IBM_BITS(x,30,30)<<5))

#define OPC_DEC_OE(x)	  (OPC_DEC_IBM_BITS(x,21,21))

#define OPC_DEC_MB(x)	  (OPC_DEC_IBM_BITS(x,21,25))
#define OPC_DEC_ME(x)	  (OPC_DEC_IBM_BITS(x,26,30))

#define OPC_DEC_MBW(x)	  (OPC_DEC_IBM_BITS(x,21,26))
#define OPC_DEC_MEW(x)	  (OPC_DEC_IBM_BITS(x,21,26))

#define OPC_DEC_IS(x)	  (OPC_DEC_IBM_SBITS(x,6,10))
#define OPC_DEC_IB(x)	  (OPC_DEC_IBM_SBITS(x,16,20))
#define OPC_DEC_XBI(x)	  (OPC_DEC_IBM_BITS(x,21,24))
#define OPC_DEC_CBI(x)	  (OPC_DEC_IBM_BITS(x,21,25))

#define OPC_DEC_UIL(x)    (OPC_DEC_IBM_BITS(x,11,20))


/*************************************************************/
/*  Encoding macros                                          */
/*************************************************************/

#define OPC_ENC_BITS(x, shift, bits) \
     (((x) &(~(~0<<(bits)))) << (shift))

#define OPC_ENC_IBM_BITS(x, msb, lsb) \
     OPC_ENC_BITS(x, 31-lsb, (lsb-msb)+1)

#define OPC_ENC_SBITS(x, lshift, rshift) \
     OPC_ENC_BITS(x, lshift, rshift)

#define OPC_ENC_IBM_SBITS(x, msb, lsb) \
     OPC_ENC_SBITS(x, 31-lsb, (lsb-msb)+1)

#define OPC_ENC_OPCD(x)   ((x)<<26)
#define OPC_ENC_XO(x)	  (OPC_DEC_IBM_SBITS(x,21,30))

#define OPC_ENC_LI(x)	  (OPC_ENC_IBM_SBITS(x,6,29))
#define OPC_ENC_AA(x)     (OPC_ENC_IBM_BITS(x,30,30))
#define OPC_ENC_LK(x)     (OPC_ENC_IBM_BITS(x,31,31))

#define OPC_ENC_BO(x)	  (OPC_ENC_IBM_BITS(x,6,10))
#define OPC_ENC_BI(x)	  (OPC_ENC_IBM_BITS(x,11,15))
#define OPC_ENC_BD(x)	  (OPC_ENC_IBM_SBITS(x,16,29))

#define OPC_ENC_BO2(x)     (OPC_ENC_IBM_BITS(x,8,8))

#define OPC_ENC_LEV(x)	  (OPC_ENC_IBM_BITS(x,20,26))

#define OPC_ENC_RT(x)	  (OPC_ENC_IBM_BITS(x,6,10))
#define OPC_ENC_RS(x)	  (OPC_ENC_IBM_BITS(x,6,10))
#define OPC_ENC_RA(x)	  (OPC_ENC_IBM_BITS(x,11,15))
#define OPC_ENC_RB(x)	  (OPC_ENC_IBM_BITS(x,16,20))

#define OPC_ENC_FRT(x)	  (OPC_ENC_IBM_BITS(x,6,10))
#define OPC_ENC_FRS(x)	  (OPC_ENC_IBM_BITS(x,6,10))
#define OPC_ENC_FRA(x)	  (OPC_ENC_IBM_BITS(x,11,15))
#define OPC_ENC_FRB(x)	  (OPC_ENC_IBM_BITS(x,16,20))
#define OPC_ENC_FRC(x)	  (OPC_ENC_IBM_BITS(x,21,25))

#define OPC_ENC_VS(x)	  (OPC_ENC_IBM_BITS(x,6,10))
#define OPC_ENC_VT(x)	  (OPC_ENC_IBM_BITS(x,6,10))
#define OPC_ENC_VA(x)	  (OPC_ENC_IBM_BITS(x,11,15))
#define OPC_ENC_VB(x)	  (OPC_ENC_IBM_BITS(x,16,20))
#define OPC_ENC_VC(x)	  (OPC_ENC_IBM_BITS(x,21,25))
#define OPC_ENC_Tval(x)	  (OPC_ENC_IBM_BITS(x,6,6))
#define OPC_ENC_STRM(x)	  (OPC_ENC_IBM_BITS(x,9,10))
#define OPC_ENC_SHB(x)	  (OPC_ENC_IBM_BITS(x,22,25))
#define OPC_ENC_SIMM(x)	  (OPC_ENC_IBM_SBITS(x,11,15))
#define OPC_ENC_UIMM(x)	  (OPC_ENC_IBM_BITS(x,11,15))
#define OPC_ENC_VRc(x)	  (OPC_ENC_IBM_BITS(x,20,20))

#define OPC_ENC_BF(x)	  (OPC_ENC_IBM_BITS(x,6,8))
#define OPC_ENC_L(x)	  (OPC_ENC_IBM_BITS(x,10,10))

#define OPC_ENC_D(x)	  (OPC_ENC_IBM_SBITS(x,16,31))
#define OPC_ENC_SI(x)	  (OPC_ENC_IBM_SBITS(x,16,31))
#define OPC_ENC_UI(x)	  (OPC_ENC_IBM_BITS(x,16,31))
#define OPC_ENC_DS(x)	  (OPC_ENC_IBM_SBITS(x,16,29))
#define OPC_ENC_DSb(x)	  (OPC_ENC_IBM_SBITS(((x)>>2),16,29))

#define OPC_ENC_DQ(x)	  (OPC_ENC_IBM_SBITS(((x)),16,27))
#define OPC_ENC_DQb(x)	  (OPC_ENC_IBM_SBITS(((x)>>4),16,27))

#define OPC_ENC_PT(x)	  (OPC_ENC_IBM_BITS(x,28,31))

#define OPC_ENC_TO(x)	  (OPC_ENC_IBM_BITS(x,6,10))
#define OPC_ENC_BFA(x)	  (OPC_ENC_IBM_BITS(x,11,13))
#define OPC_ENC_SR(x)	  (OPC_ENC_IBM_BITS(x,12,15))
#define OPC_ENC_NB(x)	  (OPC_ENC_IBM_BITS(x,16,20))
#define OPC_ENC_SH(x)	  (OPC_ENC_IBM_BITS(x,16,20))
#define OPC_ENC_U(x)	  (OPC_ENC_IBM_BITS(x,16,19))
#define OPC_ENC_Rc(x)	  (OPC_ENC_IBM_BITS(x,31,31))

#define OPC_ENC_BT(x)	  (OPC_ENC_IBM_BITS(x,6,10))
#define OPC_ENC_BA(x)	  (OPC_ENC_IBM_BITS(x,11,15))
#define OPC_ENC_BB(x)	  (OPC_ENC_IBM_BITS(x,16,20))

#define OPC_ENC_SPR(x)	  (OPC_ENC_IBM_BITS(x,11,15) | OPC_ENC_IBM_BITS(((x)>>5),16,20))
#define OPC_ENC_TBR(x)	  (OPC_ENC_IBM_BITS(x,11,15) | OPC_ENC_IBM_BITS(((x)>>5),16,20))

#define OPC_ENC_FXM(x)	  (OPC_ENC_IBM_BITS(x,12,19))
#define OPC_ENC_XO2(x)	  (OPC_ENC_IBM_BITS(x,11,20))

#define OPC_ENC_FLM(x)	  (OPC_ENC_IBM_BITS(x,7,14))

#define OPC_ENC_SHSH(x)   ((OPC_ENC_IBM_BITS(((x)&0x1f),16,20)) | OPC_ENC_IBM_BITS((((x)>>5)&0x1),30,30))

#define OPC_ENC_OE(x)	  (OPC_ENC_IBM_BITS(x,21,21))

#define OPC_ENC_MB(x)	  (OPC_ENC_IBM_BITS(x,21,25))
#define OPC_ENC_ME(x)	  (OPC_ENC_IBM_BITS(x,26,30))

#define OPC_ENC_MBW(x)	  (OPC_ENC_IBM_BITS(x,21,26))
#define OPC_ENC_MEW(x)	  (OPC_ENC_IBM_BITS(x,21,26))

#define OPC_ENC_IS(x)	  (OPC_ENC_IBM_SBITS(x,6,10))
#define OPC_ENC_IB(x)	  (OPC_ENC_IBM_SBITS(x,16,20))
#define OPC_ENC_XBI(x)	  (OPC_ENC_IBM_BITS(x,21,24))
#define OPC_ENC_CBI(x)	  (OPC_ENC_IBM_BITS(x,21,25))

#define OPC_ENC_UIL(x)    (OPC_ENC_IBM_BITS(x,11,20))


/*************************************************************/
/*  Insertion macros                                         */
/*************************************************************/

// jdonald changed bits to (bits) to comply with compiler warning

#define OPC_INS_BITS(w, x, shift, bits) \
     ( ((w) & ~((~(~0<<(bits)))<<(shift)) ) | \
       (((x) &(~(~0<<(bits)))) << (shift)) )

#define OPC_INS_IBM_BITS(w, x, msb, lsb) \
     OPC_INS_BITS(w, x, 31-lsb, (lsb-msb)+1)

#define OPC_INS_SBITS(w, x, lshift, rshift) \
     OPC_INS_BITS(w, x, lshift, rshift)

#define OPC_INS_IBM_SBITS(w, x, msb, lsb) \
     OPC_INS_SBITS(w, x, 31-lsb, (lsb-msb)+1)

#define OPC_INS_XO(w,x)   (OPC_INS_IBM_SBITS(w,x,21,30))

#define OPC_INS_LI(w,x)   (OPC_INS_IBM_SBITS(w,x,6,29))
#define OPC_INS_AA(w,x)   (OPC_INS_IBM_BITS(w,x,30,30))
#define OPC_INS_LK(w,x)   (OPC_INS_IBM_BITS(w,x,31,31))

#define OPC_INS_BO(w,x)   (OPC_INS_IBM_BITS(w,x,6,10))
#define OPC_INS_BI(w,x)   (OPC_INS_IBM_BITS(w,x,11,15))
#define OPC_INS_BD(w,x)   (OPC_INS_IBM_SBITS(w,x,16,29))

#define OPC_INS_BO2(w,x)  (OPC_INS_IBM_BITS(w,x,8,8))

#define OPC_INS_LEV(w,x)  (OPC_INS_IBM_BITS(w,x,20,26))

#define OPC_INS_RT(w,x)   (OPC_INS_IBM_BITS(w,x,6,10))
#define OPC_INS_RS(w,x)   (OPC_INS_IBM_BITS(w,x,6,10))
#define OPC_INS_RA(w,x)   (OPC_INS_IBM_BITS(w,x,11,15))
#define OPC_INS_RB(w,x)   (OPC_INS_IBM_BITS(w,x,16,20))

#define OPC_INS_FRT(w,x)  (OPC_INS_IBM_BITS(w,x,6,10))
#define OPC_INS_FRS(w,x)  (OPC_INS_IBM_BITS(w,x,6,10))
#define OPC_INS_FRA(w,x)  (OPC_INS_IBM_BITS(w,x,11,15))
#define OPC_INS_FRB(w,x)  (OPC_INS_IBM_BITS(w,x,16,20))
#define OPC_INS_FRC(w,x)  (OPC_INS_IBM_BITS(w,x,21,25))

#define OPC_INS_VS(x)	  (OPC_INS_IBM_BITS(x,6,10))
#define OPC_INS_VT(x)	  (OPC_INS_IBM_BITS(x,6,10))
#define OPC_INS_VA(x)	  (OPC_INS_IBM_BITS(x,11,15))
#define OPC_INS_VB(x)	  (OPC_INS_IBM_BITS(x,16,20))
#define OPC_INS_VC(x)	  (OPC_INS_IBM_BITS(x,21,25))
#define OPC_INS_Tval(x)	  (OPC_INS_IBM_BITS(x,6,6))
#define OPC_INS_STRM(x)	  (OPC_INS_IBM_BITS(x,9,10))
#define OPC_INS_SHB(x)	  (OPC_INS_IBM_BITS(x,22,25))
#define OPC_INS_SIMM(x)	  (OPC_INS_IBM_SBITS(x,11,15))
#define OPC_INS_UIMM(x)	  (OPC_INS_IBM_BITS(x,11,15))
#define OPC_INS_VRc(x)	  (OPC_INS_IBM_BITS(x,20,20))

#define OPC_INS_BF(w,x)   (OPC_INS_IBM_BITS(w,x,6,8))
#define OPC_INS_L(w,x)    (OPC_INS_IBM_BITS(w,x,10,10))

#define OPC_INS_D(w,x)    (OPC_INS_IBM_SBITS(w,x,16,31))
#define OPC_INS_SI(w,x)   (OPC_INS_IBM_SBITS(w,x,16,31))
#define OPC_INS_UI(w,x)   (OPC_INS_IBM_BITS(w,x,16,31))
#define OPC_INS_DS(w,x)   (OPC_INS_IBM_SBITS(w,x,16,29))
#define OPC_INS_DSb(w,x)   (OPC_INS_IBM_SBITS(w,((x)>>2),16,29)) 

#define OPC_INS_DQ(w,x)   (OPC_INS_IBM_SBITS(w,((x)),16,27))
#define OPC_INS_DQb(w,x)   (OPC_INS_IBM_SBITS(w,((x) >> 4),16,27))

#define OPC_INS_PT(w,x)   (OPC_INS_IBM_SBITS(w,x,28,31))

#define OPC_INS_TO(w,x)   (OPC_INS_IBM_BITS(w,x,6,10))
#define OPC_INS_BFA(w,x)  (OPC_INS_IBM_BITS(w,x,11,13))
#define OPC_INS_SR(w,x)   (OPC_INS_IBM_BITS(w,x,12,15))
#define OPC_INS_NB(w,x)   (OPC_INS_IBM_BITS(w,x,16,20))
#define OPC_INS_SH(w,x)   (OPC_INS_IBM_BITS(w,x,16,20))
#define OPC_INS_U(w,x)    (OPC_INS_IBM_BITS(w,x,16,19))
#define OPC_INS_Rc(w,x)   (OPC_INS_IBM_BITS(w,x,31,31))

#define OPC_INS_BT(w,x)   (OPC_INS_IBM_BITS(w,x,6,10))
#define OPC_INS_BA(w,x)   (OPC_INS_IBM_BITS(w,x,11,15))
#define OPC_INS_BB(w,x)   (OPC_INS_IBM_BITS(w,x,16,20))

#define OPC_INS_SPR(w,x)  (OPC_INS_IBM_BITS(w,x,11,15) | OPC_INS_IBM_BITS(w,((x)>>5),16,20))

#define OPC_INS_TBR(w,x)  (OPC_INS_IBM_BITS(w,x,11,15) | OPC_INS_IBM_BITS(w,((x)>>5),16,20))
#define OPC_INS_FXM(w,x)  (OPC_INS_IBM_BITS(w,x,12,19))
#define OPC_INS_XO2(w,x)  (OPC_INS_IBM_BITS(w,x,11,20))

#define OPC_INS_FLM(w,x)  (OPC_INS_IBM_BITS(w,x,7,14))

#define OPC_INS_SHSH(w,x) (OPC_INS_IBM_BITS(w,((x)&0x1f),16,20) | OPC_INS_IBM_BITS(w,(((x)>>5)&0x1),30,30))

#define OPC_INS_OE(w,x)   (OPC_INS_IBM_BITS(w,x,21,21))

#define OPC_INS_MB(w,x)   (OPC_INS_IBM_BITS(w,x,21,25))
#define OPC_INS_ME(w,x)   (OPC_INS_IBM_BITS(w,x,26,30))

#define OPC_INS_MBW(w,x)  (OPC_INS_IBM_BITS(w,x,21,26))
#define OPC_INS_MEW(w,x)  (OPC_INS_IBM_BITS(w,x,21,26))

#define OPC_INS_IS(w,x)   (OPC_INS_IBM_SBITS(w,x,6,10))
#define OPC_INS_IB(w,x)   (OPC_INS_IBM_SBITS(w,x,16,20))
#define OPC_INS_XBI(w,x)  (OPC_INS_IBM_BITS(w,x,21,24))
#define OPC_INS_CBI(w,x)  (OPC_INS_IBM_BITS(w,x,21,25))

#define OPC_INS_UIL(w,x)  (OPC_INS_IBM_BITS(w,x,11,20)

#endif
