#ifndef H_opc_instr_info_defns
#define H_opc_instr_info_defns

/*****************************************************/
/** These are definitions for the instruction types **/
/*****************************************************/
#define OPC_ILLEGAL_INSTR	  0x0
#define OPC_OTHER_INSTR		  0x1
#define OPC_BRANCH_INSTR	  0x2
#define OPC_FLOW_INSTR		  0x4
#define OPC_SYSTEM_CALL_INSTR	  0x8
#define OPC_LOAD_INSTR	          0x10
#define OPC_MULTI_LOAD_INSTR 	  0x20
#define OPC_STR_LOAD_INSTR 	  0x40
#define OPC_STR_X_LOAD_INSTR 	  0x80
#define OPC_STORE_INSTR	          0x100
#define OPC_MULTI_STORE_INSTR     0x200
#define OPC_STR_STORE_INSTR       0x400
#define OPC_STR_X_STORE_INSTR  	  0x800
#define OPC_CACHE_INSTR  	  0x1000

/* These are the "basic" types */
#define OPC_IS_ILLEGAL(ity)        ((ity) == 0x0) 
#define OPC_IS_AN_OTHER(ity)       ((ity) == 0x1)
#define OPC_IS_A_BRANCH(ity)       ((ity) == 0x2)
#define OPC_ALTERS_FLOW(ity)       ((ity) == 0x4)
#define OPC_IS_A_SYSTEM_CALL(ity)  ((ity) == 0x8)
#define OPC_IS_A_LOAD(ity)         ((ity) == 0x10)
#define OPC_IS_A_MULTI_LOAD(ity)   ((ity) == 0x20)
#define OPC_IS_A_STR_LOAD(ity)     ((ity) == 0x40)
#define OPC_IS_A_STR_X_LOAD(ity)   ((ity) == 0x80)
#define OPC_IS_A_STORE(ity)        ((ity) == 0x100)
#define OPC_IS_A_MULTI_STORE(ity)  ((ity) == 0x200)
#define OPC_IS_A_STR_STORE(ity)    ((ity) == 0x400)
#define OPC_IS_A_STR_X_STORE(ity)  ((ity) == 0x800)
#define OPC_IS_A_CACHE_INSTR(ity)  ((ity) == OPC_CACHE_INSTR)

#define _local_itype_2_ff32_other_mask    (OPC_OTHER_INSTR)
#define _local_itype_2_ff32_branch_mask   (OPC_BRANCH_INSTR | OPC_FLOW_INSTR | OPC_SYSTEM_CALL_INSTR)
#define _local_itype_2_ff32_memop_mask    (OPC_LOAD_INSTR       | OPC_STORE_INSTR       | \
                                           OPC_MULTI_LOAD_INSTR | OPC_MULTI_STORE_INSTR | \
					   OPC_STR_LOAD_INSTR   | OPC_STR_STORE_INSTR   | OPC_CACHE_INSTR)
#define _local_itype_2_ff32_memlen_mask   (OPC_STR_X_LOAD_INSTR | OPC_STR_X_STORE_INSTR)
#define _local_itype_2_ff32_illegal_mask  (OPC_ILLEGAL_INSTR)
#define opc_itype_to_ff32_type(ity) \
     ((((ity) & _local_itype_2_ff32_other_mask ) != 0) ? OPC_FF32_OTHER  : \
      (((ity) & _local_itype_2_ff32_branch_mask) != 0) ? OPC_FF32_BRANCH : \
      (((ity) & _local_itype_2_ff32_memop_mask ) != 0) ? OPC_FF32_MEMOP  : \
      (((ity) & _local_itype_2_ff32_memlen_mask) != 0) ? OPC_FF32_MULTI  : \
      OPC_FF32_ILLEGAL)
     
/* These are some "accumulation" types */
#define OPC_IS_A_LOAD_OP(ity)   \
     (((ity) & (OPC_LOAD_INSTR        | \
		OPC_MULTI_LOAD_INSTR  | \
		OPC_STR_LOAD_INSTR    | \
		OPC_STR_X_LOAD_INSTR  ) \
       ) != 0)

#define OPC_IS_A_STORE_OP(ity)  \
     (((ity) & (OPC_STORE_INSTR        | \
		OPC_MULTI_STORE_INSTR  | \
		OPC_STR_STORE_INSTR    | \
		OPC_STR_X_STORE_INSTR)   \
       ) != 0)

#define OPC_IS_A_MEMORY_ACCESS_OP(ity)  \
     (((ity) & (OPC_LOAD_INSTR        | \
		OPC_MULTI_LOAD_INSTR  | \
		OPC_STR_LOAD_INSTR    | \
		OPC_STR_X_LOAD_INSTR  | \
		OPC_STORE_INSTR       | \
		OPC_MULTI_STORE_INSTR | \
		OPC_STR_STORE_INSTR   | \
		OPC_STR_X_STORE_INSTR | \
		OPC_CACHE_INSTR) \
       ) != 0)

#define OPC_IS_A_NON_CACHE_MEMORY_ACCESS_OP(ity)  \
     (((ity) & (OPC_LOAD_INSTR        | \
		OPC_MULTI_LOAD_INSTR  | \
		OPC_STR_LOAD_INSTR    | \
		OPC_STR_X_LOAD_INSTR  | \
		OPC_STORE_INSTR       | \
		OPC_MULTI_STORE_INSTR | \
		OPC_STR_STORE_INSTR   | \
		OPC_STR_X_STORE_INSTR ) \
       ) != 0)

#define OPC_IS_A_MULTI_REGISTER_MEMORY_ACCESS_OP(ity) \
     (((ity) & (OPC_MULTI_LOAD_INSTR  | \
		OPC_STR_LOAD_INSTR    | \
		OPC_STR_X_LOAD_INSTR  | \
		OPC_MULTI_STORE_INSTR | \
		OPC_STR_STORE_INSTR   | \
		OPC_STR_X_STORE_INSTR) \
       ) != 0)

#define OPC_IS_A_LOAD_STORE_MULTIPLE_OP(ity) \
     (((ity) & (OPC_MULTI_LOAD_INSTR | \
		OPC_MULTI_STORE_INSTR) \
       ) != 0)

#define OPC_IS_A_LOAD_STRING_OP(ity) \
     (((ity) & (OPC_STR_LOAD_INSTR  | \
		OPC_STR_X_LOAD_INSTR) \
       ) != 0)

#define OPC_IS_A_STORE_STRING_OP(ity) \
     (((ity) & (OPC_STR_STORE_INSTR  | \
		OPC_STR_X_STORE_INSTR) \
       ) != 0)

#define OPC_IS_A_LOAD_STORE_STRING_OP(ity) \
     (((ity) & (OPC_STR_LOAD_INSTR   | \
		OPC_STR_X_LOAD_INSTR | \
		OPC_STR_STORE_INSTR  | \
		OPC_STR_X_STORE_INSTR) \
       ) != 0)

#define OPC_IS_A_LOAD_STORE_STRING_INDEXED_OP(ity) \
     (((ity) & (OPC_STR_X_LOAD_INSTR   | \
		OPC_STR_X_STORE_INSTR) \
       ) != 0)

#define OPC_IS_A_MULTIPLE_REGISTER_MEMORY_ACCESS_OP(ity) \
     (((ity) & (OPC_MULTI_LOAD_INSTR  | \
		OPC_STR_LOAD_INSTR    | \
		OPC_STR_X_LOAD_INSTR  | \
		OPC_MULTI_STORE_INSTR | \
		OPC_STR_STORE_INSTR   | \
		OPC_STR_X_STORE_INSTR)  \
       ) != 0)

#define OPC_IS_A_FLOW_CONTROL_OP(ity)   \
     (((ity) & (OPC_BRANCH_INSTR | \
		OPC_FLOW_INSTR   | \
		OPC_SYSTEM_CALL_INSTR)  \
       ) != 0)


/**********************************************/
/** These are instruction format definitions **/
/**********************************************/
#define OPC_ILLEGAL_FORMAT     0
#define OPC_I_FORMAT           1
#define OPC_B_FORMAT           2
#define OPC_SC_FORMAT          3
#define OPC_D_FORMAT           4
#define OPC_DS_FORMAT          5
#define OPC_DQ_FORMAT          6
#define OPC_X_FORMAT           7
#define OPC_XL_FORMAT          8
#define OPC_XFX_FORMAT         9
#define OPC_XFL_FORMAT        10
#define OPC_XS_FORMAT         11
#define OPC_XO_FORMAT         12
#define OPC_A_FORMAT          13
#define OPC_M_FORMAT          14
#define OPC_MD_FORMAT         15
#define OPC_MDS_FORMAT        16
#define OPC_TX_FORMAT         17
#define OPC_VA_FORMAT         18
#define OPC_VX_FORMAT         19
#define OPC_VXR_FORMAT        20

/***************************************************/
/** These are instruction full-format definitions **/
/***************************************************/
#define OPC_ILLEGAL_FORMAT     0
#define OPC_I_1_FORMAT         1
#define OPC_B_1_FORMAT         2
#define OPC_SC_1_FORMAT        3
#define OPC_SC_2_FORMAT        4
#define OPC_D_1_FORMAT         5
#define OPC_D_2_FORMAT         6
#define OPC_D_3_FORMAT         7
#define OPC_D_4_FORMAT         8
#define OPC_D_5_FORMAT         9
#define OPC_D_6_FORMAT        10
#define OPC_D_7_FORMAT        11
#define OPC_D_8_FORMAT        12
#define OPC_D_9_FORMAT        13
#define OPC_DS_1_FORMAT       14
#define OPC_DS_2_FORMAT       15
#define OPC_DQ_1_FORMAT       16
#define OPC_X_1_FORMAT        17
#define OPC_X_2_FORMAT        18
#define OPC_X_3_FORMAT        19
#define OPC_X_4_FORMAT        20
#define OPC_X_5_FORMAT        21
#define OPC_X_6_FORMAT        22
#define OPC_X_7_FORMAT        23
#define OPC_X_8_FORMAT        24
#define OPC_X_9_FORMAT        25
#define OPC_X_10_FORMAT       26
#define OPC_X_11_FORMAT       27
#define OPC_X_12_FORMAT       28
#define OPC_X_13_FORMAT       29
#define OPC_X_14_FORMAT       30
#define OPC_X_15_FORMAT       31
#define OPC_X_16_FORMAT       32
#define OPC_X_17_FORMAT       33
#define OPC_X_18_FORMAT       34
#define OPC_X_19_FORMAT       35
#define OPC_X_20_FORMAT       36
#define OPC_XL_1_FORMAT       37
#define OPC_XL_2_FORMAT       38
#define OPC_XL_3_FORMAT       39
#define OPC_XL_4_FORMAT       40
#define OPC_XFX_1_FORMAT      41
#define OPC_XFX_2_FORMAT      42
#define OPC_XFX_3_FORMAT      43
#define OPC_XFX_4_FORMAT      44
#define OPC_XFX_5_FORMAT      45
#define OPC_XFL_1_FORMAT      46
#define OPC_XS_1_FORMAT       47
#define OPC_XO_1_FORMAT       48
#define OPC_XO_2_FORMAT       49
#define OPC_A_1_FORMAT        50
#define OPC_M_1_FORMAT        51
#define OPC_M_2_FORMAT        52
#define OPC_MD_1_FORMAT       53
#define OPC_MD_2_FORMAT       54
#define OPC_MDS_1_FORMAT      55
#define OPC_MDS_1n_FORMAT     56
#define OPC_MDS_2_FORMAT      57
#define OPC_MDS_2n_FORMAT     58
#define OPC_MDS_3_FORMAT      59
#define OPC_MDS_3n_FORMAT     60
#define OPC_MDS_4_FORMAT      61
#define OPC_MDS_4n_FORMAT     62
#define OPC_MDS_5_FORMAT      63
#define OPC_MDS_5n_FORMAT     64
#define OPC_MDS_6_FORMAT      65
#define OPC_MDS_6n_FORMAT     66
#define OPC_TX_1_FORMAT       67
#define OPC_VA_1_FORMAT       68
#define OPC_VA_2_FORMAT       69
#define OPC_VX_1_FORMAT       70
#define OPC_VX_2_FORMAT       71
#define OPC_VX_3_FORMAT       72
#define OPC_VXR_1_FORMAT      73

/***********************************************************/
/** These are the instruction is a float flag definitions **/
/***********************************************************/
#define OPC_IS_A_FLOAT_OP      1
#define OPC_IS_NOT_A_FLOAT_OP  0

/*********************************************************/
/** These are the instruction is a VMX flag definitions **/
/*********************************************************/
#define OPC_IS_A_VMX_OP      1
#define OPC_IS_NOT_A_VMX_OP  0

/***************************************************************/
/** These are the instruction does an update flag definitions **/
/***************************************************************/
#define OPC_DOES_AN_UPDATE         1
#define OPC_DOES_NOT_DO_AN_UPDATE  0

/***************************************************************/
/** These are the instruction does a record flag definitions  **/
/***************************************************************/
#define OPC_DOES_A_RECORD          1
#define OPC_DOES_NOT_DO_A_RECORD   0

/***************************************************************************************/
/** These flags indicate an instruction is a legal user-mode POWER or PPC instruction **/
/***************************************************************************************/
#define OPC_IS_NOT_A_LEGAL_USER_POWER_OP  0
#define OPC_IS_A_LEGAL_USER_POWER_OP      1

#define OPC_IS_NOT_A_LEGAL_USER_PPC_OP    0
#define OPC_IS_A_LEGAL_USER_PPC_OP        1


/***************************************************************************************/
/** These flags indicate an instruction is a legal user-mode POWER or PPC instruction **/
/***************************************************************************************/
#define OPC_IS_NOT_IN_ISA    0
#define OPC_IS_IN_ISA        1
#define OPC_IS_IN_ISA_PRIV   2


/****************************************************************/
/** These are some useful macros that short-hand certain checks */
/****************************************************************/
/* Report whether a given index is a conditional branch index */
#define OPC_IS_A_CONDITIONAL_BRANCH(_idx) \
         ( (((_idx) >= OPC_BC  ) && ((_idx) <= OPC_BCLA  )) ||  \
	   ((_idx) == OPC_BCCTR) || ((_idx) == OPC_BCCTRL)  ||  \
	   ((_idx) == OPC_BCLR ) || ((_idx) == OPC_BCLRL )    ) \
     
	   
/* Report whether a given conditional branch word does CTR decrement */
#define OPC_COND_BRANCH_DECREMENTS_COUNTER(_iwd) \
	 ( (OPC_DEC_BO(_iwd) & 0x04) == 0 )

#define OPC_BRANCH_THROUGH_REGISTER(_idx) \
         ( ((_idx == OPC_BCCTR) || (_idx == OPC_BCCTRL)) ||  \
	   ((_idx == OPC_BCLR ) || (_idx == OPC_BCLRL ))   ) \
	   

#endif
