#ifndef H_opc_regs
#define H_opc_regs

/* Full Register Names */
/*      NOTE: if the lower 32 (SPR) register positions change */
/*        you must change the opc_reg_names table in opc.c    */
#define OPC_FIRST_SPR    0

#define OPC_CTR          1
#define OPC_XER          2
#define OPC_LR           3
#define OPC_FPSCR        4
#define OPC_MQ           5

#define OPC_DSISR        6
#define OPC_DAR          7
#define OPC_RTCU         8
#define OPC_RTCL         9
#define OPC_DEC         10
#define OPC_SDR0        11
#define OPC_SDR1        12
#define OPC_SDR2        13
#define OPC_SRR0        14
#define OPC_SRR1        15
/* NO! this will push XERs too far out */
/**
#define OPC_SRR2        16
#define OPC_SRR3        17
**/
#define OPC_MSR         16

#define OPC_TB          17
#define OPC_TBU         18

#define OPC_SR0_15      19

#define OPC_FIRST_XER_FIELD (OPC_SR0_15 + 1)

/* XER fields */
#define OPC_XER_DC_OFFSET    0
#define OPC_XER_SO_OFFSET    1
#define OPC_XER_OV_OFFSET    2
#define OPC_XER_CA_OFFSET    3
#define OPC_XER_OC_OFFSET    4
#define OPC_XER_FXCC_OFFSET  5
#define OPC_XER_DS_OFFSET    6
#define OPC_XER_TGCC_OFFSET  7
#define OPC_XER_BCNT_OFFSET  8
#define OPC_XER_BCMP_OFFSET  9
#define OPC_XER_DC          (OPC_FIRST_XER_FIELD + OPC_XER_DC_OFFSET)
#define OPC_XER_SO	    (OPC_FIRST_XER_FIELD + OPC_XER_SO_OFFSET)
#define OPC_XER_OV	    (OPC_FIRST_XER_FIELD + OPC_XER_OV_OFFSET)
#define OPC_XER_CA	    (OPC_FIRST_XER_FIELD + OPC_XER_CA_OFFSET)
#define OPC_XER_OC          (OPC_FIRST_XER_FIELD + OPC_XER_OC_OFFSET)
#define OPC_XER_FXCC	    (OPC_FIRST_XER_FIELD + OPC_XER_FXCC_OFFSET)
#define OPC_XER_TGCC        (OPC_FIRST_XER_FIELD + OPC_XER_TGCC_OFFSET)
#define OPC_XER_DS          (OPC_FIRST_XER_FIELD + OPC_XER_DS_OFFSET)
#define OPC_XER_BCNT	    (OPC_FIRST_XER_FIELD + OPC_XER_BCNT_OFFSET)
#define OPC_XER_BCMP	    (OPC_FIRST_XER_FIELD + OPC_XER_BCMP_OFFSET)

/* Other register number space */
/* NOTE: OPC_FIRST_CR is set at position 32 */
#define OPC_FIRST_CR    32
#define OPC_FIRST_CRB   (OPC_FIRST_CR  +  8)
#define OPC_FIRST_FPR   (OPC_FIRST_CRB + 32)
#define OPC_FIRST_GPR   (OPC_FIRST_FPR + 32)
#define OPC_FIRST_SR    (OPC_FIRST_GPR + 32)
#define OPC_FIRST_VECR  (OPC_FIRST_SR  + 16)

/* The SR Registers */
#define OPC_SR0         (OPC_FIRST_SR)
#define OPC_SR1         (OPC_SR0 +1)
#define OPC_SR2         (OPC_SR1 +1)
#define OPC_SR3         (OPC_SR2 +1)
#define OPC_SR4         (OPC_SR3 +1)
#define OPC_SR5         (OPC_SR4 +1)
#define OPC_SR6         (OPC_SR5 +1)
#define OPC_SR7         (OPC_SR6 +1)
#define OPC_SR8         (OPC_SR7 +1)
#define OPC_SR9         (OPC_SR8 +1)
#define OPC_SR10        (OPC_SR9 +1)
#define OPC_SR11        (OPC_SR10+1)
#define OPC_SR12        (OPC_SR11+1)
#define OPC_SR13        (OPC_SR12+1)
#define OPC_SR14        (OPC_SR13+1)
#define OPC_SR15        (OPC_SR14+1)


#define OPC_FIRST_FPSCR_FIELD (OPC_FIRST_VECR + 32)

#define OPC_FPSCR_ENR_OFFSET    10
#define OPC_FPSCR_FPCC_OFFSET   11
#define OPC_FPSCR_FRFI_OFFSET   12
#define OPC_FPSCR_EXCPT_OFFSET  13
#define OPC_FPSCR_ENR        (OPC_FIRST_FPSCR_FIELD)
#define OPC_FPSCR_FPCC	     (OPC_FPSCR_ENR  + 1)
#define OPC_FPSCR_FRFI	     (OPC_FPSCR_FPCC + 1)
#define OPC_FPSCR_EXCPT	     (OPC_FPSCR_FRFI + 1)
#define OPC_LAST_FPSCR_FIELD (OPC_FPSCR_EXCPT)

#define OPC_RTYPE_TOTAL (OPC_LAST_FPSCR_FIELD + 1)

/* Types of register */
#define OPC_SR_REG_TYPE     1
#define OPC_GPR_REG_TYPE    2
#define OPC_FPR_REG_TYPE    3
#define OPC_CRB_REG_TYPE    4
#define OPC_CR_REG_TYPE     5
#define OPC_SPR_REG_TYPE    6

#define OPC_RTYPE(n) (((n) >= OPC_FIRST_FPSCR_FIELD) ? OPC_FPSCR_FIELD_TYPE : \
                      (((n) >= OPC_FIRST_XER_FIELD) ? OPC_XER_FIELD_TYPE : \
		       (((n) >= OPC_FIRST_SR) ? OPC_SR_REG_TYPE : \
			(((n) >= OPC_FIRST_GPR) ? OPC_GPR_REG_TYPE : \
			 (((n) >= OPC_FIRST_FPR) ? OPC_FPR_REG_TYPE : \
			  (((n) >= OPC_FIRST_CR) ? OPC_CR_REG_TYPE : \
			   OPC_SPR_REG_TYPE) \
			  ) \
			 ) \
			) \
		       ) \
		      )

#define OPC_REG_IS_A_XER_FIELD(_n)   (((_n) >= OPC_FIRST_XER_FIELD)   && ((_n) < OPC_FIRST_CR))
#define OPC_REG_IS_A_FPSCR_FIELD(_n) (((_n) >= OPC_FIRST_FPSCR_FIELD) && ((_n) <= OPC_LAST_FPSCR_FIELD))
#define OPC_REG_IS_A_SPR(_n)         (((_n) < OPC_FIRST_XER_FIELD) || OPC_REG_IS_A_XER_FIELD(_n) || OPC_REG_IS_A_FPSCR_FIELD(_n))
#define OPC_REG_IS_A_CRF(_n)         (((_n) >= OPC_FIRST_CR)          && ((_n) < OPC_FIRST_CRB))
#define OPC_REG_IS_A_CRB(_n)         (((_n) >= OPC_FIRST_CRB)         && ((_n) < OPC_FIRST_FPR))
#define OPC_REG_IS_A_CR(_n)          (((_n) >= OPC_FIRST_CR)          && ((_n) < OPC_FIRST_FPR))
#define OPC_REG_IS_A_FPR(_n)         (((_n) >= OPC_FIRST_FPR)         && ((_n) < OPC_FIRST_GPR))
#define OPC_REG_IS_A_GPR(_n)         (((_n) >= OPC_FIRST_GPR)         && ((_n) < OPC_FIRST_SR))
#define OPC_REG_IS_A_SR(_n)          (((_n) >= OPC_FIRST_SR)          && ((_n) < OPC_FIRST_VECR))
#define OPC_REG_IS_A_VECR(_n)        (((_n) >= OPC_FIRST_VECR)        && ((_n) < OPC_FIRST_FPSCR_FIELD))

/* Register Conversions -- from register number to internal rep */
#define OPC_SPR(_n)	((_n)+OPC_FIRST_SPR)
#define OPC_GPR(_n)	((_n)+OPC_FIRST_GPR)
#define OPC_FPR(_n)	((_n)+OPC_FIRST_FPR)
#define OPC_CR(_n)	((_n)+OPC_FIRST_CR)
#define OPC_CRF(_n)	((_n)+OPC_FIRST_CR)
#define OPC_VECR(_n)	((_n)+OPC_FIRST_VECR)

/* Register Conversions -- from internal rep to register number */
#define OPC_CVT_SPR_RVAL(_x)          ((_x))
#define OPC_CVT_XER_FIELD(_x)         ((_x) - OPX_FIRST_XER_FIELD)
#define OPC_CVT_XER_FIELD_AS_SPR(_x)  ((_x))

#define OPC_CVT_CR_RVAL(_x)    ((_x) - OPC_FIRST_CR)
#define OPC_CVT_CRF_RVAL(_x)   (((_x) - OPC_FIRST_CR)<<2)

#define OPC_CVT_FPR_RVAL(_x)   ((_x) - OPC_FIRST_FPR)
#define OPC_CVT_GPR_RVAL(_x)   ((_x) - OPC_FIRST_GPR)
#define OPC_CVT_SR_RVAL(_x)    ((_x) - OPC_FIRST_SR)

/** MACROS to determine if bit-vector fields are set **/
#define OPC_LR_DEF_USE_MASK    (1 << OPC_LR)
#define OPC_CTR_DEF_USE_MASK   (1 << OPC_CTR)

#define OPC_XER_DEF_USE_MASK   (1 << OPC_XER)
#define OPC_FPSCR_DEF_USE_MASK (1 << OPC_FPSCR)
#define OPC_MQ_DEF_USE_MASK    (1 << OPC_MQ)

#define OPC_XER_FIELDS_MASK    ((1<<OPC_XER_DC) | (1<<OPC_XER_SO) | (1<<OPC_XER_OV) | (1<<OPC_XER_CA) | (1<<OPC_XER_OC) | (1<<OPC_XER_FXCC) | (1<<OPC_XER_TGCC) | (1 << OPC_XER_DS) | (1 << OPC_XER_BCMP) | (1 << OPC_XER_BCNT))

#define OPC_FPSCR_FIELDS_MASK  ((1<<OPC_FPSCR_EXCPT) | (1<<OPC_FPSCR_FRFI) | (1<<OPC_FPSCR_FPCC) | (1<<OPC_FPSCR_ENR))

#endif
