// PROGRAM NAME..... read_sim_bin_file
//
// FUNCTION ........ Probably reads an aria simulation file and provides
//                   API functions for the user to read this information.
//                      -jdonald

#include <unistd.h> // jdonald
#include <string.h> // jdonald, for strcmp()
#include "ansi_stdio.h"
#include "ansi_stdlib.h"

#include "ansi_assert.h"
#include "posix_fcntl.h"

#include "opc.h"

#include "opc_is_implemented_types.h"

/** MACROS **/


/** Types **/

/** Forward function declarations **/
void print_usage(int argc, char * argv[]);
int read_input_flags(int argc, char * argv[]);

int get_next_infile_block();
unsigned next_word();

/** Global Variables **/
__thread char * infile_base_name = NULL;
__thread int    infile_line = 1;
__thread int    infiled;

__thread int run_in_verbose_mode = 0;

__thread unsigned total_bytes_read = 0;

/*******************************************************************
  PROGRAM NAME..... build_sim_file

  FUNCTION ........ 

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 
  CALLED-ROUTINES.. 
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

__thread int eof_found = 0;
__thread int error_occurred = 0;
__thread int end_marker_found = 0;
__thread int isa_marker_found = 0;
__thread int isa_marker_test_val = 0;

__thread char string_buffer[64];
__thread int string_buffer_idx;

#define READ_BUFFER_SIZE 4096
__thread unsigned   read_buffer[READ_BUFFER_SIZE];
__thread unsigned * rp;
__thread unsigned * rend;

typedef struct iwords_data_struct_t {
  int index;
  unsigned iword_base;
  int num_operands;
  int op_type[5];
} iwords_data;

__thread iwords_data simcode_info_tbl[OPC_NUM+1] = {
  {0, 0, 0, { 0, 0, 0, 0, 0 } },
  #include "opc_form_simulation_code.gen"
  {0, 0, 0, { 0, 0, 0, 0, 0 } }
  };

#include "opc_reg_ident_defines.gen"
#define NUM_GPR_SCRATCH_REGISTERS 4
#define NUM_FPR_SCRATCH_REGISTERS 4


static __thread char * reverse_map_isa_name[OPC_NUM_ISAs][2] = {
  {"simunion",     "simunion_spce"},    /*  OPC_UNION_ISA     */
  {"simppc",       "simppc_spec"},      /*  OPC_PPC_ISA       */
  {"simppc32",     "simppc32_spec"},    /*  OPC_PPC32_ISA     */
  {"simppc40x",    "simppc40x_spec"},   /*  OPC_PPC40X_ISA    */
  {"simppc601",    "simppc601_spec"},   /*  OPC_PPC601_ISA    */
  {"simpower",     "simpower_spec"},    /*  OPC_POWER_ISA     */
  {"simpower2",    "simpower2_spec"},   /*  OPC_POWER2_ISA    */
  {"simamazon",    "simamazon_spec"},   /*  OPC_AMAZON_ISA    */
  {"simppc64on32", "simppc64on32_spec"} /*  OPC_PPC64on32_ISA */
};

static __thread char * gpr_field_names[4] = {"RT", "RS", "RA", "RB"};
static __thread char * fpr_field_names[5] = {"FRT", "FRS", "FRA", "FRB", "FRC"};
static __thread char * cr_field_names[2]  = {"BF", "BFA"};


int
main( int argc, 
      char * argv[]
      )
{
  char ch;
  int n;
  
  opc_init();

  /** Read in the input (command line) switch values **/
  if (read_input_flags(argc, argv) != 0) {
    fprintf(stderr, "Problem reading the command-line input... Exiting\n");
    print_usage(argc, argv);
    return -1;
  }

  if (run_in_verbose_mode > 1) {fprintf(stderr, "Trying to open input file \"%s\"\n", infile_base_name); }
  infiled = open(infile_base_name, O_RDONLY, 0);
  if (infiled == -1) {
    fprintf(stderr, "Could not open input file \"%s\"\n", infile_base_name);
    exit(-1);
  }
  if (run_in_verbose_mode > 1) {fprintf(stderr, "Opened simcode file: \"%s\"\n", infile_base_name); }
  
  /* Read in the simcode bin file */
  eof_found = 0;
  n = get_next_infile_block();
  while (eof_found == 0) {
    int instr_idx = next_word();
    if (eof_found == 0) {
      int num_sim_isas = next_word();
      int i;
      if (run_in_verbose_mode) {fprintf(stderr, "Read instr idx = %d = \"%s\", it has %d sim isa descriptions\n", instr_idx, opc_name(instr_idx), num_sim_isas); }

      fprintf(stdout, "%s\n", opc_name(instr_idx)); 
      for (i = 0; i < num_sim_isas; i++) {
	int target_isa = next_word();
	int spec_flag  = next_word();
	int num_sim_instrs = next_word();
	int j;
	if (run_in_verbose_mode) {fprintf(stderr, "  For target_isa %d spec_flag %d there are %d sim instructions\n", target_isa, spec_flag, num_sim_instrs); }
	fprintf(stdout, "%s\n", reverse_map_isa_name[target_isa][spec_flag]); 
	for (j = 0; j < num_sim_instrs; j++) {
	  int sim_instr_idx = next_word();
	  int k;
	  if (run_in_verbose_mode) {fprintf(stderr, "    For target_isa %d instr %d idx = %d = \"%s\" with %d operands\n", 
					    target_isa, j, sim_instr_idx, opc_isa_name(target_isa, sim_instr_idx),
					    simcode_info_tbl[sim_instr_idx].num_operands); }
	  fprintf(stdout, "\t%s\t", opc_isa_name(target_isa, sim_instr_idx));
	  for (k = 0; k < simcode_info_tbl[sim_instr_idx].num_operands; k++) {
	    int op_flag = next_word();
	    if (run_in_verbose_mode) {fprintf(stderr, "      instr %d \"%s\" operand %d flag = %d\n", 
					      j, opc_isa_name(target_isa, sim_instr_idx), k, op_flag); }
	    if (k > 0) { 
	      fprintf(stdout, ", ");
	    }
	    switch (simcode_info_tbl[sim_instr_idx].op_type[k]) {
	    case OPC_GPR_IDENTIFIER:
	      if (op_flag < 32) {
		fprintf(stderr, "GPR%-2d", op_flag);
	      } else if (op_flag < 36) {
		fprintf(stdout, "%s", gpr_field_names[op_flag-32]);
	      }
	      else {
		fprintf(stdout, "SCR%-2d", op_flag-36);
	      }
	      break;
	    case OPC_FPR_IDENTIFIER:
	      if (op_flag < 32) {
		fprintf(stdout, "FPR%-2d", op_flag);
	      } else if (op_flag < 37) {
		fprintf(stdout, "%s", fpr_field_names[op_flag-32]);
	      }
	      else {
		fprintf(stdout, "FSC%-2d", op_flag-37);
	      }
	      break;
	    case OPC_CR_IDENTIFIER:
	      if (op_flag < 8) {
		fprintf(stdout, "CR%-2d", op_flag);
	      } else {
		fprintf(stdout, "%s", cr_field_names[op_flag-8]);
	      }
	      break;
	    case OPC_SIMPLE_VALUE:
	      fprintf(stdout, "0x%x", op_flag);	      
	      break;
	    default:
	      fprintf(stdout, "\?\?=0x%x=\?\?", op_flag); // avoid 'trigraph' confusion -jdonald
	      error_occurred = 1;
	      break;
	    }
	  }
	  fprintf(stdout, "\n");
	}
      }
    }
  }

  close(infiled);
  
  return EXIT_SUCCESS; // jdonald
}


unsigned 
next_word() 
{
  unsigned ret_val = *rp;
  if (run_in_verbose_mode > 2) {
    fprintf(stderr, "0x%08x = 0x%08x   (rend = 0x%08x) words_left = %d\n", (unsigned) rp, *rp, (unsigned) rend, rend-rp); // jdonald cast
  }
  rp++;
  if (rp >= rend) {
    int n = get_next_infile_block();
    if (run_in_verbose_mode > 1) {fprintf(stderr, "read in %d bytes\n", n);} // jdonald: was missing the 'n' for %d!!
    
    if (n == 0) {
      if (eof_found == 1) {
	fprintf(stderr, "ERROR: Premature EOF near byte %d\n", total_bytes_read);
      }
      eof_found = 1;
    }
  }
  return ret_val;
}

int 
get_next_infile_block()
{
  int ret_val;

  if (run_in_verbose_mode > 2) {
    fprintf(stderr, "Getting the next file block: rp = 0x%08x vs 0x%08x = rend\n", (unsigned) rp, (unsigned) rend); // jdonald cast
  }

  ret_val = read( infiled, &read_buffer, sizeof(read_buffer) );
  total_bytes_read += ret_val; 

  rp = read_buffer;
  rend = &read_buffer[ret_val/4];

  if (run_in_verbose_mode > 2) {
    int i;
    fprintf(stderr, "I read the following: (%d bytes = %d words)\n", ret_val, ret_val/4);
    fprintf(stderr, "--------------------------------------------------\n");
    for (i = 0; i < ret_val/4; i++) {
      if ((i > 0) && ((i % 4) == 0)) {
	fprintf(stderr, "\n");
      }
      if ((i % 4) == 0) {
	fprintf(stderr, "   %4d  :  ", i);
      }
      fprintf(stderr, "0x%08x ", read_buffer[i]);
    }
    fprintf(stderr, "\n--------------------------------------------------\n");
  }

  return ret_val;
}


int
classify_isa_token()
{
  if (strcmp(string_buffer, "simppc") == 0) {
    return OPC_PPC_ISA;
  }
  else if (strcmp(string_buffer, "simppc32") == 0) {
    return OPC_PPC32_ISA;
  }
  else if (strcmp(string_buffer, "simppc40x") == 0) {
    return OPC_PPC40X_ISA;
  }
  else if (strcmp(string_buffer, "simppc601") == 0) {
    return OPC_PPC601_ISA;
  }
  else if (strcmp(string_buffer, "simpower") == 0) {
    return OPC_POWER_ISA;
  }
  else if (strcmp(string_buffer, "simpower2") == 0) {
    return OPC_POWER2_ISA;
  }
  else if (strcmp(string_buffer, "simamazon") == 0) {
    return OPC_AMAZON_ISA;
  }
  else if (strcmp(string_buffer, "simppc64on32") == 0) {
    return OPC_PPC64on32_ISA;
  }
  else {
    return -1;
  }
}



/*******************************************************************
  PROGRAM NAME..... read_input_flags

  FUNCTION ........ Parses the input command-line arguments, and 
     sets (global) variables to indicate the flag conditions that 
     are requested by the user.

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 0 = good, other = error in parsing
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

int
read_input_flags(int    argc, 
		 char * argv[]
		 ) 
{
  int ret_val = 0;
  int i = 1;
  if (run_in_verbose_mode > 1) { fprintf(stderr, "reading in the input parameters...\n"); } // jdonald removed: i, argv[i]

  if (argc < 2) {
    return -1;
  }

  while (i < argc) {
    if (run_in_verbose_mode > 1) { fprintf(stderr, "  argv[%d] = \"%s\"\n", i, argv[i]); }
    
    if ((strcmp(argv[i], "-VV") == 0) || (strcmp(argv[i], "-VERBOSE") == 0)) {
      run_in_verbose_mode = 3;
    }
    else if ((strcmp(argv[i], "-v") == 0) || (strcmp(argv[i], "-verbose") == 0)) {
      run_in_verbose_mode = 1;
    }
    else if ((strcmp(argv[i], "-V") == 0) || (strcmp(argv[i], "-Verbose") == 0)) {
      run_in_verbose_mode = 2;
    }
    else if (i == argc-1) {
      infile_base_name = argv[i];
    }
    i++;
  }
  
// exit_this_while_loop: // unused?

  if (infile_base_name == NULL) {
    ret_val = -1;
  }

  return ret_val;
}


/*******************************************************************
  PROGRAM NAME..... print_usage

  FUNCTION ........ Dumps out information about the invocation of 
     the build_sim_file program.

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

void
print_usage(int argc,
	    char * argv[]
	    )
{
  fprintf(stderr, "usage: %s <options> filename\n", argv[0]);
  fprintf(stderr, "\n  Where the\n");
  fprintf(stderr, "      <options> can be any of:\n");
  fprintf(stderr, "%30s : display simulation file preparation information\n",    "-verbose");
}
