#include "ansi_stdlib.h"
#include "ansi_string.h"
#include "ansi_assert.h"
#include "ansi_ctype.h"
#include "strhash.h"

#define SIZE_POOL_STRHASH	(1<<16)
static __thread char pool_strhash[SIZE_POOL_STRHASH];
static __thread char *	base_pool_strhash = NULL; // jdonald changing to a constant initializer, so checking put elsewhere

void *
alloc_pool_strhash(
	unsigned bytes
	)
{
  unsigned	size = (bytes+7)&~0x7;
  void *	res;

  if (base_pool_strhash == NULL) 
	base_pool_strhash = pool_strhash; // jdonald
  if( base_pool_strhash + size >  pool_strhash + SIZE_POOL_STRHASH ) {
    return 0;
  }

  res = (void *) base_pool_strhash;
  base_pool_strhash += size;

  return res;
}

struct strhash *
make_strhash(
	unsigned	size
	)
{
  struct strhash *	tbl;
  int			i;

  assert( ((size - 1)&size) == 0 ); /* power of 2 */

  tbl = (struct strhash *)alloc_pool_strhash( sizeof(struct strhash) + (size-1)*sizeof(struct strhash_cell *));
  if( tbl == 0 ) {
    return 0;
  }
  size_strhash(tbl) = size;
  mask_strhash(tbl) = size-1;
  for( i = 0; i < size; i++ ) {
    cells_strhash(tbl)[i] = 0;
  }

  return tbl;
}

void
destroy_strhash(
	struct strhash *	tbl
	)
{
  unsigned		size = size_strhash(tbl);
  struct strhash_cell *	p;
  struct strhash_cell *	q;
  int			i;

  for( i = 0; i < size; i++ ) {
    for( p = cells_strhash(tbl)[i]; p; p = q ) {
      q = next_strhash_cell(p);
      free(p);
    }
  }
  free(tbl);
}

unsigned
hash_strhash(
	const char *	str
	)
{
  unsigned	hashval;
  unsigned	v;
  int		i;

  hashval = 0;
  for( i = 0; str[i]; i++ ) {
    if( str[i] == '.' ) {
      v = 26;
    }
    else if( isdigit(str[i]) ) {
      v = 27;
    }
    else {
      v = str[i] - 'a';
    }

    hashval = ( hashval << 1 ) ^ v;
  }
  hashval += i;

  return hashval;
}

int
is_in_strhash(
	struct strhash *	tbl,
	const char *		str
	)
{
  unsigned		mask = mask_strhash(tbl);
  unsigned		hashval = hash_strhash(str);
  unsigned		hash = hashval & mask;
  struct strhash_cell *	p;

  for( p = cells_strhash(tbl)[hash]; p; p = next_strhash_cell(p) ) {
    if( hash_strhash_cell(p) == hashval &&
	strcmp(str_strhash_cell(p), str) == 0 ) {
      return 1;
    }
  }
  return 0;
}

unsigned
get_strhash(
	struct strhash *	tbl,
	const char *		str
	)
{
  unsigned		mask = mask_strhash(tbl);
  unsigned		hashval = hash_strhash(str);
  unsigned		hash = hashval & mask;
  struct strhash_cell *	p;

  for( p = cells_strhash(tbl)[hash]; p; p = next_strhash_cell(p) ) {
    if( hash_strhash_cell(p) == hashval &&
	strcmp(str_strhash_cell(p), str) == 0 ) {
      return value_strhash_cell(p);
    }
  }
  return 0;
}

int
update_strhash(
	struct strhash *	tbl,
	const char *		str,
	unsigned		val
	)
{
  unsigned		mask = mask_strhash(tbl);
  unsigned		hashval = hash_strhash(str);
  unsigned		hash = hashval & mask;
  struct strhash_cell *	p;
  unsigned		len;

  for( p = cells_strhash(tbl)[hash]; p; p = next_strhash_cell(p) ) {
    if( hash_strhash_cell(p) == hashval &&
	strcmp(str_strhash_cell(p), str) == 0 ) {
      value_strhash_cell(p) = val;
      return 1;
    }
  }

  len = strlen(str);
  p = (struct strhash_cell *)alloc_pool_strhash(sizeof(struct strhash_cell) + len);
  if( p  == 0 ) {
    return 0;
  }

  next_strhash_cell(p) = cells_strhash(tbl)[hash];
  cells_strhash(tbl)[hash] = p;
  value_strhash_cell(p) = val;
  hash_strhash_cell(p) = hashval;
  strcpy(str_strhash_cell(p), str);
  return 2;
}
