/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... do_lwarx_stwcx_translation
  FUNCTION ........ Translates a set of user code including an 
     lwarx and one (or more?) stwcx. instructions, keeping them 
     close together (and avoiding any writes) so that the 
     reservation (of the lwarx) will not be lost before a following
     stwcx. inapropriately.

  NOTES............ We know this is a "first" translation of the 
     taken-path code, otherwise this code would not be invoked.
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... the next available translation address
  CALLED-ROUTINES.. analyze_branch, bb_fallthrough
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... free_regs_in_8_bits_of_vector.init_dat,
     fill_static_buffer_code.c, xlation_alignment_code.c,
     save_aria_state_code.c, bb_prolog_code.c, opc_idx_dtrb_code.gen,
     bb_epilog_code.c, restore_aria_state_code.c, opc_idx_spec_xform.gen

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... July, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
#include <stdio.h>

#include "aria_xlate_macros.h"

#include "opc.h"

#include "proc_state.h"
#include "aria_state.h"
#include "hashing.h"
#include "analyze_branch.h"

#include "io_msgs.h"

#include "dump_description.h"

#include "globtypes.h"
#include "global_variables.h"

#include "analyze_lwarx_stwcx.h"

#include "canned_code.h"

#include "display_reg_usedef.h"

#include "aria_interface.h"
#include "aria_per_instr_interface.h"

/************************************************************************
 * These are static (local) variable definitions used in analysis
 ***********************************************************************/
static unsigned free_regs_in[257] = {
    #include "free_regs_in_8_bits_of_vector.init_dat"
    0
  };


/** This data structure is used to keep information about the instructions in a block **/
/** MACROS used to shorthand some pieces of code **/

#include "translation_macros.h"

branch_list_type the_branches_list[MAX_NUMBER_OF_BRANCHES_IN_BRANCH_LIST];

/** Helper Functions **/

unsigned * 
do_lwarx_stwcx_translation(unsigned   * next_prog_address,
			   hash_entry * the_hash_entry_ptr,
			   unsigned     generation
			   )
{
  unsigned * last_orig_bb_instr_ptr;
  unsigned * orig_ptr;
  unsigned * xlate_ptr;
  unsigned * temp_ptr;
  unsigned total_instr_in_bb = 0;

  use_def_data_type the_instr_reg_data[ABSOLUTE_MAXIMUM_USER_INSTR_IN_A_BLOCK];
  use_def_data_type the_full_block;
  
  instruction_xlate_type instr_xlate_info[ABSOLUTE_MAXIMUM_USER_INSTR_IN_A_BLOCK];

  unsigned translation_type;
  
  unsigned curr_br_num = 0;

  unsigned * original_bb_start_address   = next_prog_address;
  unsigned * translated_bb_start_address;
  unsigned * microtrace_buffer_address = 0;

  unsigned * prolog_start_address;
  unsigned * epilog_start_address;

  opc_idx last_iword_index;

  short instr_in_prolog = 0;
  short instr_in_epilog = 0;

  unsigned base_register;
  unsigned scratch_reg;
  unsigned dyntr_posn = 0;
  unsigned trbf_base_reg;

  unsigned original_base_register;
  unsigned original_scratch_reg;
  unsigned original_trbf_base_reg;

  unsigned original_br_num_taken;
  unsigned original_br_taken_addr;
  unsigned original_br_fall_thru;

  unsigned top_visible_reg_use_vectors[4] = {0, 0, 0, 0};

  unsigned total_instr_in_xl_bb = 0;

  unsigned block_ending_branch   = 0; 
  unsigned done_with_user_code = 0;

  unsigned saved_do_full_xlate_dump       = do_full_xlate_dump;
  unsigned saved_do_full_execute_dump     = do_full_execute_dump;
  unsigned saved_do_hash_tables_dump      = do_hash_tables_dump;
  unsigned saved_do_proc_state_dump       = do_proc_state_dump;
  unsigned saved_do_bb_regs_use_def_dump  = do_bb_regs_use_def_dump;
  unsigned saved_do_trace_dump            = do_trace_dump;
  unsigned saved_do_no_output_during_runs = do_no_output_during_runs;
  unsigned long long saved_output_after_static_bb_translation = output_after_static_bb_translation;
  unsigned long long saved_output_after_dynamic_bb_execution  = output_after_dynamic_bb_execution;

  /* Now analyze the code for this basic block */
  orig_ptr  = next_prog_address;
  temp_ptr  = next_xlate_address;
  xlate_ptr = next_xlate_address;
  
  /*********************************/
  /** FIND THE END OF BASIC BLOCK **/
  /*************************************************************************/
  /** This code determines the extent of the user-code basic block that will
   *   be translated.  There are three conditions that can end a basic-block:
   *    1. The discovery of a branch instruction
   *    2. The user-code basic block becomes longer than 
   *          the maximum allowed size (in instructions)
   *    3. The register use/def for next instr would not leave enough 
   *          registers for dynamic trace info generation.
   **/
  
  CLEAR_FULL_BLOCK_USE_DEF_TOUCH_VECTORS;
  
  /** We create the trace buffer as we scan the original basic block **/
  /**  To prepare, set a 0x00000000 value as a buffer spot, then     **/
  /**   set teh start of the trace buffer                            **/
  *next_xlate_address = 0x00000000;           next_xlate_address++;
  microtrace_buffer_address = next_xlate_address;
  
  do { 
    unsigned test_vector;
    unsigned tv_0_7  ;
    unsigned tv_8_15 ;
    unsigned tv_16_23;
    unsigned tv_24_31;
    unsigned free_gpr_regs;
    
    /* Trace information */
    unsigned iword         = *orig_ptr; 
    unsigned iword_index   = opc_index(iword);
    unsigned iword_ikind   = opc_itype(iword_index);

    instr_xlate_info[total_instr_in_bb].iaddr = (unsigned)orig_ptr;
    instr_xlate_info[total_instr_in_bb].iword = iword;
    instr_xlate_info[total_instr_in_bb].index = opc_index(iword);
    instr_xlate_info[total_instr_in_bb].ikind = opc_itype(iword_index);

    /**
      instr_xlate_info[total_instr_in_bb].daddr = 0xdeadbeef;
      **/
    instr_xlate_info[total_instr_in_bb].daddr = (do_iaddr_fixup) ? 
      (instr_xlate_info[total_instr_in_bb].iaddr + 4 - global_user_iaddr_offset) : 
      (instr_xlate_info[total_instr_in_bb].iaddr + 4); /* temp set of fallthrough */
    instr_xlate_info[total_instr_in_bb].dlen  = 0;

    /** Determine if the user specified a BEFORE or INSTEAD action for this iword **/
    if ( ((per_instruction_action_time(iword_index) & DO_ARIA_ACTION_BEFORE) != 0) &&
	 (total_instr_in_bb > 0)
	 ) {
      global_user_stop_condition_counter++; 
      warn("*************************************************************************\n");
      warn("I don't know how to handle a user stop condition in a lwarx/stwcx code...\n");
      do_full_xlate_dump      = 1;
      do_full_execute_dump    = 1;
      do_hash_tables_dump     = 1;
      do_proc_state_dump      = 1;
      do_bb_regs_use_def_dump = 1;
      do_trace_dump            = 1;
      do_no_output_during_runs = 0;
      output_after_static_bb_translation = 0;
      output_after_dynamic_bb_execution  = 0;
      goto stop_analysis_because_of_user_action;
    }

    if (OPC_IS_A_SYSTEM_CALL(iword_ikind)) {
      warn("*****************************************************************\n");
      warn("I don't know how to handle a system call in a lwarx/stwcx code...\n");
      do_full_xlate_dump      = 1;
      do_full_execute_dump    = 1;
      do_hash_tables_dump     = 1;
      do_proc_state_dump      = 1;
      do_bb_regs_use_def_dump = 1;
      do_trace_dump            = 1;
      do_no_output_during_runs = 0;
      output_after_static_bb_translation = 0;
      output_after_dynamic_bb_execution  = 0;
      goto block_ends_before_an_svc;
    }
    
    last_iword_index = iword_index;
    
    opc_full_use_def_vecs(iword, the_instr_reg_data[total_instr_in_bb].use_vectors, 
			  the_instr_reg_data[total_instr_in_bb].def_vectors, the_instr_reg_data[total_instr_in_bb].touch_vectors);
    
    /** Check that we would still have one free register **/
    test_vector = ( the_instr_reg_data[total_instr_in_bb].use_vectors[0] | the_full_block.use_vectors[0] |
		    the_instr_reg_data[total_instr_in_bb].def_vectors[0] | the_full_block.def_vectors[0] );
    tv_0_7   = (test_vector & 0x000000f8);
    tv_8_15  = (test_vector & 0x0000ff00) >> 8;
    tv_16_23 = (test_vector & 0x00ff0000) >> 16;
    tv_24_31 = (test_vector & 0xff000000) >> 24;
    free_gpr_regs = (free_regs_in[tv_0_7] + free_regs_in[tv_8_15] 
		     + free_regs_in[tv_16_23] + free_regs_in[tv_24_31]);
    
    /** Determine if three registers are avaiable,   **/
    /**    NOT including registers 0, 1 or 2.        **/
    /**      THREE registers are required to:        **/
    /**        1. Address the dynamic trace buffer   **/
    /**        2. Address the proc state space       **/
    /**        3. use as a scratch register          **/
    /** OR if there are no instructions in the BB    **/
    /**  (this is necessary for lswx, lmw instrs)    **/
    /**  in which case we make this a one-instr BB   **/
    
    if ( free_gpr_regs < 9 ) {
      global_addressibility_counter++;
      warn("**********************************************************************\n");
      warn("I don't know how to handle registers not free in a lwarx/stwcx code...\n");
      do_full_xlate_dump      = 1;
      do_full_execute_dump    = 1;
      do_hash_tables_dump     = 1;
      do_proc_state_dump      = 1;
      do_bb_regs_use_def_dump = 1;
      do_trace_dump            = 1;
      do_no_output_during_runs = 0;
      output_after_static_bb_translation = 0;
      output_after_dynamic_bb_execution  = 0;
      goto block_ends_on_register_addresibility;
    }
    
    /** We know now that this instruction will be added to this basic block **/
    UPDATE_TOTAL_BLOCK_USE_DEF_TOUCH_VECTORS;
    
    /********************************************************/
    /** Set up the trace buffer entry for this instruction **/
    #include "fill_trace_buffer_entry_code.c"

    /** Increment the orig_ptr to point to the next user program word **/
    orig_ptr++; 
    
    /** Increment the number of instructions in the user basic block **/
    total_instr_in_bb++; 
    
    /** Check whether this instruction puts us over the max block size limit **/
    if ( total_instr_in_bb == ABSOLUTE_MAXIMUM_USER_INSTR_IN_A_BLOCK ) { 
      warn("*************************************************************************\n");
      warn("I don't know how to handle a too-large user code in a lwarx/stwcx code...\n");
      do_full_xlate_dump      = 1;
      do_full_execute_dump    = 1;
      do_hash_tables_dump     = 1;
      do_proc_state_dump      = 1;
      do_bb_regs_use_def_dump = 1;
      do_trace_dump            = 1;
      do_no_output_during_runs = 0;
      output_after_static_bb_translation = 0;
      output_after_dynamic_bb_execution  = 0;
      global_max_size_ubb_counter++; 
      done_with_user_code = 1;
    }
    
    /** Check for a user-specified stop-after condition **/
    if ( (per_instruction_action_time(last_iword_index) > DO_NO_ARIA_ACTION) ){
      warn("*************************************************************************\n");
      warn("I don't know how to handle a user stop condition in a lwarx/stwcx code...\n");
      do_full_xlate_dump      = 1;
      do_full_execute_dump    = 1;
      do_hash_tables_dump     = 1;
      do_proc_state_dump      = 1;
      do_bb_regs_use_def_dump = 1;
      do_trace_dump            = 1;
      do_no_output_during_runs = 0;
      output_after_static_bb_translation = 0;
      output_after_dynamic_bb_execution  = 0;
      HashTable_user_stop_iaddr(*the_hash_entry_ptr) = (orig_ptr-1);
      global_user_stop_condition_counter++; 
      goto stop_analysis_because_of_user_action;
    }
    
    if (OPC_IS_A_BRANCH(iword_ikind) ) {
      unsigned all_br_to_stwcx_covered = 1;
      unsigned target_addr   = 0x0;
      unsigned fallthru_addr = 0x0;
      int ii;

      if (curr_br_num == MAX_NUMBER_OF_BRANCHES_IN_BRANCH_LIST) {
	warn("ARIA: Error -- too many branches in lwarx/stwcx. code sequence...\n");
	abort();
      }

      find_relative_branch_target(iword, (orig_ptr-1), &target_addr, &fallthru_addr);

      the_branches_list[curr_br_num].address    = (unsigned)(orig_ptr-1);
      the_branches_list[curr_br_num].num_instr  = total_instr_in_bb;
      the_branches_list[curr_br_num].rel_target = target_addr;
      the_branches_list[curr_br_num].fall_thru  = fallthru_addr;
      the_branches_list[curr_br_num].skip_posn  = 0x0;

      {
	unsigned * taddrp = (unsigned *)target_addr;
	unsigned   tiword = *taddrp;
	unsigned   tiword_index = opc_index(tiword);
      
	if ((tiword_index == OPC_STWCX_) || (tiword_index == OPC_STDCX_)) {
	  the_branches_list[curr_br_num].to_a_stwcx = 1;
	}
	else {
	  the_branches_list[curr_br_num].to_a_stwcx = 0;
	}
      }
      
      for (ii = 0; ii <= curr_br_num; ii++) {
	if ( (the_branches_list[ii].to_a_stwcx == 1) && 
	     (the_branches_list[ii].rel_target > the_branches_list[curr_br_num].address)
	     ) {
	  all_br_to_stwcx_covered = 0;
	  break;
	}
      }

      if ( (the_branches_list[curr_br_num].rel_target < the_branches_list[curr_br_num].address) &&
	   (all_br_to_stwcx_covered == 1) ) {
	done_with_user_code = 1;
      }
      curr_br_num++;
    }

  } while ( !(done_with_user_code) );
  
  if (OPC_IS_A_BRANCH(opc_itype(last_iword_index))) {
    block_ending_branch = 1;
  }
  
block_ends_on_register_addresibility:
block_ends_before_an_svc:
stop_analysis_because_of_user_action:
  /*********************************/
  /** END OF BASIC BLOCK DETECTED **/
  /*********************************/
  
  /** Mark the end of the trace buffer **/
  *next_xlate_address = 0x00000000;           next_xlate_address++;
  
  /* Dump out the initial trace buffer contents */
  /* Dump out the initial trace buffer contents */
  #include "show_trace_buffer_contents.c"

  if ( (do_no_output_during_runs == 0) ) { 
    int iii;
    warn_xlate("------------------------------ (end of buffer)\n");

    warn_xlate("\nThe branch list has %u entries\n", curr_br_num);
    for (iii = 0; iii < curr_br_num; iii++) {
      warn_xlate("  0x%08x  %5u  0x%08x  0x%08x  %3u  0x%08x\n", 
		 the_branches_list[iii].address,
		 the_branches_list[iii].num_instr,
		 the_branches_list[iii].rel_target,
		 the_branches_list[iii].fall_thru,
		 the_branches_list[iii].to_a_stwcx,
		 the_branches_list[iii].skip_posn
		 );
    }
    warn_xlate("------------------------------ (end of branches list)\n");
  } 
  
  /* Update the xlation address alignment for the subsequent translation */
  #include "xlation_alignment_code.c"
  
  /* Set a pointer to the last instruction in the basic block */
  /*  (often this is the ending branch instruction)           */
  last_orig_bb_instr_ptr = orig_ptr - 1; 
  
  /*************************************************************************/
  /*  Now go bottom-up through the BB to determine the outward-visible     */
  /*   (here called top_visible) register uses (in a vector for each type) */ 
  /*************************************************************************/
  {
    unsigned * iptr;
    for (iptr = last_orig_bb_instr_ptr; iptr >= original_bb_start_address; iptr--) {
      unsigned ii;
      unsigned temp_reg_use_vectors[4] = {0, 0, 0, 0};
      unsigned temp_reg_def_vectors[4] = {0, 0, 0, 0};
      opc_usevecs(*iptr, temp_reg_use_vectors);
      opc_setvecs(*iptr, temp_reg_def_vectors);
      
      for (ii = 0; ii < 4; ii++) {
	top_visible_reg_use_vectors[ii] = ((top_visible_reg_use_vectors[ii] & ~temp_reg_def_vectors[ii]) 
					   | temp_reg_use_vectors[ii]);
      }
    }
  }

  /**********************************************************************/
  /* Now select 3 registers to be our base, scratch and dynamic trace   */
  /*  buffer address registers, and include them in the touch vectors,  */
  /*  so that they will retain a consistent value in ARIA's view.       */
  /**********************************************************************/
  {
    int i;
    unsigned regs_to_use[6] = {3, 4, 5, 6, 7, 8};
    unsigned index = 0;
    for (i = 3; i <= 31; i++) {
      if ((the_full_block.touch_vectors[0] & (1 << i)) == 0) {
	regs_to_use[index] = i;
	if (i > 12) {
	  the_full_block.touch_vectors[0] |= (1 << i);
	}
	index++;
      }
      if (index == 6) {
	break;
      }
    }
    original_base_register = regs_to_use[0];
    original_scratch_reg   = regs_to_use[1];
    original_trbf_base_reg = regs_to_use[2];
    original_br_num_taken  = regs_to_use[3];
    original_br_taken_addr = regs_to_use[4];
    original_br_fall_thru  = regs_to_use[5];
  }
  
  if ( (do_no_output_during_runs == 0) ) { 
    warn_xlate("\nThe register assignments are:\n");
    warn_xlate("\toriginal_base_register = %u\n", original_base_register);
    warn_xlate("\toriginal_scratch_reg   = %u\n", original_scratch_reg);
    warn_xlate("\toriginal_trbf_base_reg      = %u\n", original_trbf_base_reg);
    warn_xlate("\toriginal_br_num_taken  = %u\n", original_br_num_taken);
    warn_xlate("\toriginal_br_taken_addr = %u\n", original_br_taken_addr);
    warn_xlate("\toriginal_br_fall_thru  = %u\n", original_br_fall_thru);
    warn_xlate("\n");
  }

  /** Store the information common to all translations (i.g. info about original BB) **/
  HashTable_original_address(*the_hash_entry_ptr)   = original_bb_start_address;
  HashTable_trace_buffer(*the_hash_entry_ptr)       = microtrace_buffer_address;
  HashTable_user_bb_size(*the_hash_entry_ptr)       = total_instr_in_bb;
  HashTable_generation(*the_hash_entry_ptr)         = generation;
  HashTable_overflow(*the_hash_entry_ptr)           = NULL;

  /** Store the information we may need to do the other translation **/
  HashTable_has_dynamic_data(*the_hash_entry_ptr)  = 1;
  HashTable_ends_in_a_branch(*the_hash_entry_ptr)  = block_ending_branch;

  HashTable_base_reg(*the_hash_entry_ptr)    = original_base_register;
  HashTable_scratch_reg(*the_hash_entry_ptr) = original_scratch_reg;
  HashTable_buffer_reg(*the_hash_entry_ptr)  = original_trbf_base_reg;

  HashTable_block_gpr_use(*the_hash_entry_ptr) = the_full_block.use_vectors[0];
  HashTable_block_gpr_def(*the_hash_entry_ptr) = the_full_block.def_vectors[0];
  HashTable_block_fpr_use(*the_hash_entry_ptr) = the_full_block.use_vectors[1];
  HashTable_block_fpr_def(*the_hash_entry_ptr) = the_full_block.def_vectors[1];
  HashTable_block_cr_use(*the_hash_entry_ptr)  = the_full_block.use_vectors[2];
  HashTable_block_cr_def(*the_hash_entry_ptr)  = the_full_block.def_vectors[2];
  HashTable_block_spr_use(*the_hash_entry_ptr) = the_full_block.use_vectors[3];
  HashTable_block_spr_def(*the_hash_entry_ptr) = the_full_block.def_vectors[3];
  HashTable_top_vis_gpr_use(*the_hash_entry_ptr)   = top_visible_reg_use_vectors[0];
  HashTable_top_vis_fpr_use(*the_hash_entry_ptr)   = top_visible_reg_use_vectors[1];
  HashTable_top_vis_cr_use(*the_hash_entry_ptr)    = top_visible_reg_use_vectors[2];
  HashTable_top_vis_spr_use(*the_hash_entry_ptr)   = top_visible_reg_use_vectors[3];

  /** Enter a header for this BB translation **/
  #include "add_xlation_header_code.c"

  /* Output the use and def of registers for this user block */
  if ( (do_no_output_during_runs == 0) ) { 
    /** warn_bb_regs("The total sets of registers touched are:\n"); **/
    display_register_uses_and_defs(the_full_block.use_vectors[0], the_full_block.use_vectors[1], 
				   the_full_block.use_vectors[2], the_full_block.use_vectors[3], 
				   the_full_block.def_vectors[0], the_full_block.def_vectors[1], 
				   the_full_block.def_vectors[2], the_full_block.def_vectors[3], 
				   top_visible_reg_use_vectors); 
  } 

  /** Do the taken path translations **/

  translation_type = TAKEN_XLATE;   
  translated_bb_start_address = next_xlate_address;
  warn_xlate("\nThe taken-path-execution (non-speculative) code is:\n");
  
  /* Set a temp pointer (for output) to the start of the translated block */
  temp_ptr = next_xlate_address;

  /* Store (temporarily) the start of the translated code epilog */
  prolog_start_address = next_xlate_address;
  
  /* Set up some temporary values for base_register and scratch_reg */
  /* to use only while saving off the Aria entry state              */
  base_register = 12;
  scratch_reg   = 11;
  
  /********************************************************/
  /** Save off all of Aria's important (clobbered) state **/
  /********************************************************/
  /* Load GPRbase with the address of our ARIA state saving space */          
  *next_xlate_address = OPC_ASM_ADDIS(base_register,  0, ARIA_STATE_SAVE_HI_16);  next_xlate_address++; 
  
  /* Store the processor state save space address in our reserved work area */
  *next_xlate_address = STORE_REG ( 3, PROC_STATE_OFFSET,  base_register);      next_xlate_address++;
  
  /* If we are calculating not-taken addresses, store the not_taken address receiver address */
  if (not_taken_address_required_flag == 1) {
    *next_xlate_address = STORE_REG ( 4, NOT_TAKEN_RVAL_OFFSET, base_register);   next_xlate_address++;
  }
  
  /** If we are doing dynamic tracing AND have dynamic data in the user BB **/
  /**  Create a pointer to the trace buffer **/
  if ( (do_dynamic_trace_generation) ) {
    unsigned static_hi = ((unsigned)microtrace_buffer_address & 0xffff0000) >> 16;
    unsigned static_lo = ((unsigned)microtrace_buffer_address & 0x0000ffff);
    *next_xlate_address = OPC_ASM_ADDIS( 4, 0, static_hi);                          next_xlate_address++;
    *next_xlate_address = OPC_ASM_ORI  ( 4, 4, static_lo);                          next_xlate_address++;
    *next_xlate_address = STORE_REG  ( 4, PROC_DYN_TRBUF_OFFSET, base_register);  next_xlate_address++;
  }
  
  /********************************************************************************************/
  /** Insert the code to save the Aria state (that may be altered by user program execution) **/
  /********************************************************************************************/
  {
    unsigned gprs = the_full_block.touch_vectors[0];
    unsigned fprs = the_full_block.touch_vectors[1];
    unsigned crs  = the_full_block.touch_vectors[2];
    
    #include "save_aria_state_code.c"
  }
  
  /* Recover the settings for the selected base, scratch and dynamic trace buffer ptrs */
  base_register = original_base_register;
  scratch_reg   = original_scratch_reg;
  trbf_base_reg      = original_trbf_base_reg;
  
  /* Copy the GPR3 (user processor state ptr) value to the base register */
  if (base_register != 3) {
    /* Copy GPR 3 value to the base register */
    *next_xlate_address = OPC_ASM_OR(base_register,  3,  3);  next_xlate_address++;
  }
  
  /****************************************************************************/
  /** Insert the bb prolog, which inserts instructions to restore user state **/
  /****************************************************************************/
  {
    unsigned gpr_uses   = top_visible_reg_use_vectors[0];
    unsigned fpr_uses   = top_visible_reg_use_vectors[1];
    unsigned cr_touches = the_full_block.touch_vectors[2];
    unsigned spr_uses   = the_full_block.use_vectors[3];
    unsigned spr_defs   = the_full_block.def_vectors[3];
    
    #include "bb_prolog_code.c"
  }
  
  /** Calculate the number of instructions in the translated BB prolog **/
  instr_in_prolog = (next_xlate_address - prolog_start_address);

  /** Output the prolog (if appropriate) **/
  if (do_no_output_during_runs == 0) {
    warn_xlate("* (prolog) ----------------------------------------------------------------\n");
    DUMP_DESCRIPTION(temp_ptr, next_xlate_address);    
    warn_xlate("* (translated body) -------------------------------------------------------\n");
  }
  temp_ptr = next_xlate_address;

  /******************************************************************/
  /** Now generate a translated copy of the original (user         **/
  /** instructions) basic block                                    **/
  /******************************************************************/
  
  /** If doing dynamic trace generation, set up the dynamic trace buffer pointer **/
  if ( (do_dynamic_trace_generation) ) {
    temp_ptr = next_xlate_address;       /* save curr start of region (for output)      */
    /** Get the pointer (base address) of the aria save space **/
    *next_xlate_address = OPC_ASM_ADDIS(scratch_reg,  0, ARIA_STATE_SAVE_HI_16);   next_xlate_address++;
    /** Load the dynamic trace buffer pointer from it **/
    *next_xlate_address = LOAD_REG  (trbf_base_reg, PROC_DYN_TRBUF_OFFSET, scratch_reg);  next_xlate_address++;
    /* print out the translation if user desires it */
    if (do_no_output_during_runs == 0) {
      DUMP_DESCRIPTION(temp_ptr, next_xlate_address);
      temp_ptr = next_xlate_address;
    }
  }
  
  /** Zero out the taken branch number register value **/
  *next_xlate_address = OPC_ASM_ADDI(original_br_num_taken, 0, 0);   next_xlate_address++;
  
  /* reset the user instruction position index */
  dyntr_posn = 0;

  /*******************************************************************************/
  /** Do the copying of the original basic block instructions.                  **/
  /*******************************************************************************/

  /*************************************************************/
  /** Now copy over the actual user instructions (unmodified) **/
  /**  and insert dynamic trace code if requested             **/
  /*************************************************************/
  {
    unsigned instr_num;
    unsigned br_instr_num = 0;
    unsigned * start_of_user_code_xlation = next_xlate_address;
    temp_ptr = next_xlate_address;

    for (instr_num = 0; instr_num < total_instr_in_bb; instr_num++) {
      if (OPC_IS_A_BRANCH(instr_xlate_info[instr_num].ikind) ) {
	if (do_no_output_during_runs == 0) {
	  char dis_buf[256];
	  opc_dis(instr_xlate_info[instr_num].iword, dis_buf);
	  warn("for the branch instruction: %s\n", dis_buf);
	}

	insert_embedded_branch_code(instr_xlate_info[instr_num].iword, 
				    br_instr_num, the_branches_list, 
				    base_register, scratch_reg, trbf_base_reg,
				    original_br_num_taken, original_br_taken_addr, original_br_fall_thru);
	br_instr_num++;
	
	if (do_no_output_during_runs == 0) {
	  DUMP_FAKE_DESCRIPTION(temp_ptr, next_xlate_address);
	  temp_ptr = next_xlate_address;
	}	
      }
      else {
	/* Insert the instruction-index-dependent code required to generate a translation */
	/*  Note that this includes:                                                   */
	/*    setting the data access length in the trace buffer (for memory accesses) */
	/*    inserting any dynamic trace generation code required by the instruction  */
	/*    inserting either the original iword or a not-taken path code sequence    */
	if ( do_dynamic_trace_generation ) {
          #include "opc_idx_dtrb_code.gen" 
	}

	if (do_no_output_during_runs == 0) {
	  DUMP_FAKE_DESCRIPTION(temp_ptr, next_xlate_address);
	  temp_ptr = next_xlate_address;
	}      

	*next_xlate_address = instr_xlate_info[instr_num].iword;  next_xlate_address++;
	if (do_no_output_during_runs == 0) {
	  DUMP_XLATE_DESCRIPTION(temp_ptr, next_xlate_address, (unsigned *)instr_xlate_info[instr_num].iaddr);
	  temp_ptr = next_xlate_address;
	}
      }
      temp_ptr = next_xlate_address;
    }

    /* Restore the temp ptr */
    temp_ptr = start_of_user_code_xlation;
  }

  /** Fixup the xlated code by inserting the "skip" branches **/
  {
    int iii;
    for (iii = 0; iii < curr_br_num; iii++) {
      if (the_branches_list[iii].skip_posn != 0x0) {
	if ((next_xlate_address - the_branches_list[iii].skip_posn) == 1) {
	  /* Insert a NOP instruction rather than a branch */
	  *the_branches_list[iii].skip_posn = OPC_ASM_OR(0, 0, 0);
	}
	else {
	  *the_branches_list[iii].skip_posn = OPC_ASM_B((next_xlate_address - the_branches_list[iii].skip_posn));
	}
      }
      else {
	warn("Can I have a 0x0 skip_posn?\n");
      }
    }
  }


  if ( (do_no_output_during_runs == 0) ) { 
    int iii;
    warn_xlate("\nThe branch list has %u entries\n", curr_br_num);
    for (iii = 0; iii < curr_br_num; iii++) {
      warn_xlate("  0x%08x  %5u  0x%08x  0x%08x  %3u  0x%08x\n", 
		 the_branches_list[iii].address,
		 the_branches_list[iii].num_instr,
		 the_branches_list[iii].rel_target,
		 the_branches_list[iii].fall_thru,
		 the_branches_list[iii].to_a_stwcx,
		 the_branches_list[iii].skip_posn
		 );
    }
    warn_xlate("------------------------------ (end of branches list)\n");

    if (do_no_output_during_runs == 0) {
      warn_xlate("\nRepeating the user code body...\n");
      DUMP_FAKE_DESCRIPTION(temp_ptr, next_xlate_address);
      temp_ptr = next_xlate_address;
    }
  } 
  
  /* Output the epilog marker */
  if (do_no_output_during_runs == 0) {
    warn_xlate("* (epilog) {0x%08x} ---------------------------------------------------\n", next_xlate_address);
    temp_ptr = next_xlate_address;
  }
  
  /* Store (temporarily) the start of the translated code epilog */
  epilog_start_address = next_xlate_address;
  
  /*****************************************************************************/
  /** Insert the bb epilog, which inserts instructions to save new user state **/
  /*****************************************************************************/
  {
    unsigned   gpr_defs   = the_full_block.def_vectors[0];
    unsigned   fpr_defs   = the_full_block.def_vectors[1];
    unsigned   cr_touches = the_full_block.touch_vectors[2];
    unsigned   spr_uses   = the_full_block.use_vectors[3];
    unsigned   spr_defs   = the_full_block.def_vectors[3];
    
    #include "bb_epilog_code.c"
  }
  
  if ( (do_no_output_during_runs == 0) ) {
    unsigned iword;
    opc_idx  index;
    char buf[512] = "*illegal instruction (data?)*";
    DUMP_DESCRIPTION(temp_ptr, next_xlate_address);
    warn_xlate("* (branch cleanup) --------------------------------------------------------\n");
    iword  = *last_orig_bb_instr_ptr;
    index  =  opc_index(iword);	
    opc_dis(iword, buf);

    temp_ptr = next_xlate_address;
  }
  
  insert_embedded_branch_cleanup_code(curr_br_num, 
				      the_branches_list, 
				      last_orig_bb_instr_ptr,
				      &base_register, 
				      scratch_reg,
				      trbf_base_reg,
				      original_br_num_taken, 
				      original_br_taken_addr, 
				      original_br_fall_thru
				      );
  
  if (do_no_output_during_runs == 0) {
    DUMP_DESCRIPTION(temp_ptr, next_xlate_address);
    temp_ptr = next_xlate_address;
  }

  /* update the program address pointer */
  next_prog_address = last_orig_bb_instr_ptr + 1;
  
  /********************************/
  /** Now restore the Aria state **/
  /********************************/
  scratch_reg   = 11;
  
  if (scratch_reg == base_register) {
    scratch_reg--;
  }
  
  {
    unsigned gprs = the_full_block.touch_vectors[0];
    unsigned fprs = the_full_block.touch_vectors[1];
    unsigned crs  = the_full_block.touch_vectors[2];
    
    #include "restore_aria_state_code.c"
  }

  /********************************************************************************************/
  /* Replace the branch in the original code with code to return through link register branch */
  /********************************************************************************************/
  *next_xlate_address = OPC_ASM_BCLR (0x14, 0);        next_xlate_address++;
  
  /** Calculate the number of instructions in the translated BB prolog **/
  instr_in_epilog = (next_xlate_address - epilog_start_address);

  /* Determine the total instructions in the translated BB code */
  total_instr_in_xl_bb = (next_xlate_address - translated_bb_start_address);

  /* Output the translated basic block information exit code */
  if (do_no_output_during_runs == 0) {
    DUMP_DESCRIPTION(temp_ptr, next_xlate_address);
    warn_xlate("===============================================================================\n");
    warn_xlate("    end of BB %d [%u user, %u xlated instr] ; oa=%08x xa = %08x\n", 
	       global_bb_index+1, total_instr_in_bb,
	       total_instr_in_xl_bb, original_bb_start_address, translated_bb_start_address);
  }


  /***************************************************************/
  /* Update the addresses for the program and translation space **/
  /***************************************************************/
  #include "xlation_alignment_code.c"

  /* Dump out the initial trace buffer contents */
  #include "show_trace_buffer_contents.c"

  /**********************************************************/
  /* add a hash table entry for the newly translated block **/
  /**********************************************************/
  HashTable_xlation_address(*the_hash_entry_ptr) = translated_bb_start_address;
  HashTable_xlated_bb_size(*the_hash_entry_ptr)  = total_instr_in_xl_bb;
  /* Set the prolog and epilog sizes */
  HashTable_prolog_size(*the_hash_entry_ptr) = instr_in_prolog;
  HashTable_epilog_size(*the_hash_entry_ptr) = instr_in_epilog;
  
  /****************************************************/
  /** Dump the translated code out of the data cache **/
  /****************************************************/
  _sync_cache_range(translated_bb_start_address, 4*(next_xlate_address - translated_bb_start_address));

  do_full_xlate_dump       = saved_do_full_xlate_dump;
  do_full_execute_dump     = saved_do_full_execute_dump;
  do_hash_tables_dump      = saved_do_hash_tables_dump;
  do_proc_state_dump       = saved_do_proc_state_dump;
  do_bb_regs_use_def_dump  = saved_do_bb_regs_use_def_dump;
  do_trace_dump            = saved_do_trace_dump;
  do_no_output_during_runs = saved_do_no_output_during_runs;
  output_after_static_bb_translation = saved_output_after_static_bb_translation;
  output_after_dynamic_bb_execution  = saved_output_after_dynamic_bb_execution;

  return next_xlate_address;
}




