/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/

#include <limits.h>

#include "aix_a.out.h"
#include "aix_errno.h"
#include "aix_ldr.h"
#include "aix_mman.h"
#include "posix_fcntl.h"
#include "ansi_stdio.h"
#include "ansi_stdlib.h"

#include "ansi_assert.h"

#include "aria_state.h"

#include "io_msgs.h"
#include "opc.h"
#include "proc_state.h"
#include "hashing.h"
#include "do_translation.h"
#include "do_svc_translation.h"
#include "save_user_state.h"

#include "globdefs.h"
#include "globtypes.h"

#include "global_variables.h"

#include "interface_globals.h"

/** Forward function declarations **/

#include "aria_interface.h"

/** Some statistics-holding variables **/

unsigned long long global_user_line_index   = 0;
unsigned long long global_xlated_line_index = 0;

unsigned long long global_user_instructions   = 0;
unsigned long long global_xlated_instructions = 0;

/*******************************************************************
  PROGRAM NAME..... dump_aria_run_statistics

  FUNCTION ........ Dumps out information about the number of basic 
     blocks translated, executed, and total user and translated 
     instructions (counted either statically or dynamically),
     as well as the program return value (from the processor state).

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

void 
dump_aria_run_statistics_to(proc_state_type * state_ptr,
			    FILE * fptr
			    )
{
  int i;
  int user_exit_code = proc_state_GPR(state_ptr, 3);

  fprintf(fptr, "User program %s invoked as:\n", user_argv[0]);

  fprintf(fptr, "     %s ", user_argv[0]);
  for (i = 1; i < user_argc; i++) {
    fprintf(fptr, "%s ", user_argv[i]);
  }
  fprintf(fptr, "\n\n");

  fprintf(fptr, "Translation resulted in %llu block translations, and\n", global_bb_index);
  fprintf(fptr, "                        %llu basic block executions\n",  global_bb_executions);
  fprintf(fptr, "\n");
  fprintf(fptr, "There are a total of %llu original (static) instructions,\n", global_user_line_index);
  fprintf(fptr, "      and a total of %llu translated (static) instructions.\n", global_xlated_line_index);
  fprintf(fptr, "\n");
  fprintf(fptr, "There are a total of %llu original instructions executed,\n", global_user_instructions);
  fprintf(fptr, "      and a total of %llu translated instructions executed.\n", global_xlated_instructions);
  fprintf(fptr, "\n");
  fprintf(fptr, "The user program returned an exit code of (%d)\n", user_exit_code);

  fprintf(fptr, "Of the %llu translated basic blocks, \n", global_bb_index);
  fprintf(fptr, "       %u were broken due to maximum code size limit,\n", global_max_size_ubb_counter);
  fprintf(fptr, "       %u were broken due to register addresibility,\n", global_addressibility_counter);
  fprintf(fptr, "   and %u were broken due to user stop conditions.\n",  global_user_stop_condition_counter);
}


void 
dump_aria_run_statistics(proc_state_type * state_ptr
			 )
{
  dump_aria_run_statistics_to(state_ptr, mystderr);
  return;
}


