/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... do_aria_execution, 

  FUNCTION ........ Provides a set of interface routines to aria, 
     comprising the external user interface.

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 
  CALLED-ROUTINES.. 
  CALLED-BY........ None
  INCLUDED-BY...... 
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

#include <sys/time.h>
#include <limits.h>

#include "aix_a.out.h"
#include "aix_errno.h"
#include "aix_ldr.h"
#include "aix_mman.h"
#include "posix_fcntl.h"
#include "ansi_stdio.h"
#include "ansi_stdlib.h"

#include "ansi_assert.h"

#include "aria_debug.h"
#include "aria_state.h"

#include "io_msgs.h"
#include "opc.h"
#include "proc_state.h"
#include "hashing.h"
#include "do_translation.h"
#include "do_svc_translation.h"
#include "save_user_state.h"

#include "globdefs.h"
#include "globtypes.h"

#include "global_variables.h"

#include "interface_globals.h"

/** Forward function declarations **/

#include "aria_interface.h"
#include "aria_per_instr_interface.h"

/** A legal statement that will preside in the libaria.a object file **/
static char legal_statement [24] [73] = {
  "/* \n",
  "** COMPONENT_NAME: MET / Aria                                         \n", 
  "**                                                                    \n", 
  "** Licensed Materials - Property of IBM                               \n",  
  "**                                                                    \n",
  "** IBM Confidential (IBM Confidential-Restricted when combined        \n", 
  "**                   with aggregated modules for this product)        \n", 
  "**                                                                    \n",
  "**                                                                    \n",
  "** OCO Source Materials                                               \n",
  "**                                                                    \n",
  "**                                                                    \n",
  "** GA-0100 (C) Copyright IBM Corp. 1997                               \n",
  "**                                                                    \n",
  "** US Government Users Restricted Rights - Use, duplication or        \n",
  "** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  \n",
  "**                                                                    \n",
  "** The source code for this program is not published or otherwise     \n",
  "** divested of its trade secrets, irrespective of what has been       \n",
  "** deposited within the U.S. Copyright Office.                        \n",
  "**                                                                    \n",
  "** Copyright information in aria.copyright                            \n",
  "**                                                                    \n",
  "*/\n"
};

/*******************************************************************
  PROGRAM NAME..... do_aria_execution

  FUNCTION ........ Executes a user code basic block at a specified 
     address, for a given processor state, generating a new processor
     state, one trace buffer describing the executed basic block, 
     and the next taken and not-taken path addresses.
 
  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

#include "dump_description.h"

unsigned * last_bb_printed = 0;

unsigned saved_do_no_output_during_runs;
unsigned long long saved_output_after_static_bb_translation;
unsigned long long saved_output_after_dynamic_bb_execution;
unsigned long long saved_no_output_after_static_bb_translation;
unsigned long long saved_no_output_after_dynamic_bb_execution;

unsigned saved_do_full_xlate_dump;
unsigned saved_do_full_execute_dump;
unsigned saved_do_trace_dump;
unsigned saved_do_bb_regs_use_def_dump;
unsigned saved_do_proc_state_dump;

unsigned * last_user_space_address;

#define  dummy_speculation_block_trace_size  8
unsigned dummy_speculation_block_trace_buffer[ARIA_TB_PER_INSTR_SIZE * dummy_speculation_block_trace_size] = {
  0x0, OPC_ASM_AND(0,0,0), OPC_OTHER_INSTR, 0x0, 0x0,
  0x0, OPC_ASM_AND(0,0,0), OPC_OTHER_INSTR, 0x0, 0x0,
  0x0, OPC_ASM_AND(0,0,0), OPC_OTHER_INSTR, 0x0, 0x0,
  0x0, OPC_ASM_AND(0,0,0), OPC_OTHER_INSTR, 0x0, 0x0,
  0x0, OPC_ASM_AND(0,0,0), OPC_OTHER_INSTR, 0x0, 0x0,
  0x0, OPC_ASM_AND(0,0,0), OPC_OTHER_INSTR, 0x0, 0x0,
  0x0, OPC_ASM_AND(0,0,0), OPC_OTHER_INSTR, 0x0, 0x0,
  0x0, OPC_ASM_AND(0,0,0), OPC_OTHER_INSTR, 0x0, 0x0
};

unsigned the_stop_address = 0x0; 
unsigned the_restart_address = 0x0;


#define INTERACT_XLATE_MODE 1
  
unsigned * 
do_aria_execution(unsigned *  next_bb_address, 
		  proc_state_type * saved_proc_state_ptr, 
		  unsigned *  trace_size,
		  unsigned ** microtrace_buffer_ptr, 
		  unsigned ** not_taken_bb_address,
		  unsigned    speculative_not_taken_flag
		  )
{
  proc_state_type   old_proc_saved_space;
  proc_state_type * old_saved_proc_state_ptr = &old_proc_saved_space;

  unsigned ubb_size;
  unsigned taken_xbb_size;
  unsigned not_taken_xbb_size;
  unsigned * next_xlated_bb_address;
  unsigned * next_not_taken_xl_bb_address;
  unsigned * temp_ptr;
  
  unsigned * taken_path;
  unsigned * not_taken_path;

  hash_entry * the_hash_entry_ptr = NULL;

  unsigned do_xlation;

  if ((do_uniform_sampling_in_aria) && (curr_window_count >= capture_window_size)) {
    unsigned * new_next_bb_address;
    unsigned  actual_skipped_instrs = 0;
    IFDEBUG(fprintf(mystderr, "Skipping %d instructions at addr %08x...\n", capture_window_size, next_bb_address);
	    fflush(mystderr);
	    );
    if (skip_window_size > 0) {
      new_next_bb_address = do_aria_skip_instructions(next_bb_address, saved_proc_state_ptr, 
						      skip_window_size, &actual_skipped_instrs);
      next_bb_address = new_next_bb_address;
    }
    IFDEBUG(fprintf(mystderr, "  Actually skipped %d instructions to addr %08x...\n", 
		    actual_skipped_instrs, next_bb_address);
	    fflush(mystderr);
	    );
    curr_window_count = 0;
  }

  /* Check for an execution of the exit flag address */
  if ( (((unsigned)next_bb_address) == ARIA_EXIT_FLAG_ADDRESS) ) {
    *trace_size = 0;
    *microtrace_buffer_ptr = NULL;
    *not_taken_bb_address = (unsigned *)ARIA_EXIT_FLAG_ADDRESS;
    return ARIA_EXIT_FLAG_ADDRESS;    
  }

  #ifdef FULL_DEBUG
    if ( (((unsigned)next_bb_address) == the_exit_routine_address) ) {
      fprintf(mystderr, "Hit the exit routine address ...\n");
    }
  #endif

  if (next_xlate_address >= (unsigned *)ARIA_STATE_SAVE_SPACE) {
    fprintf(stderr, "ARIA: ERROR: Too many translations; translation memory is full\n");
    abort();
  }

  IFFULLDEBUG(if (next_bb_address == (unsigned *)the_stop_address) {
    fprintf(stderr, "Found the address sought\n");
    do_full_xlate_dump       = 1;
    do_full_execute_dump     = 1;
    do_hash_tables_dump      = 1;
    do_proc_state_dump       = 1;
    do_bb_regs_use_def_dump  = 1;
    do_trace_dump            = 1;
    do_trace_buffer_dump     = 1;
    do_show_runtime_warnings = 1;
    do_no_output_during_runs = 0;
    output_after_static_bb_translation = 0;
    output_after_dynamic_bb_execution  = 0;
  } );

  /** Check whether we are asked to speculate into an un-mapped section of memory **/
  /**  If we tried even to read that region of memory, we will get a Seg Fault!   **/
  if ( speculative_not_taken_flag && ((unsigned)next_bb_address > 0xdfffffff) ) { 
    int i;
    *microtrace_buffer_ptr = dummy_speculation_block_trace_buffer;
    *trace_size = dummy_speculation_block_trace_size;
    for (i = 0; i < dummy_speculation_block_trace_size; i++) {
      aria_Trace_Iaddr(dummy_speculation_block_trace_buffer, i) = (unsigned)next_bb_address;
      next_bb_address++;
    }
    *not_taken_bb_address = next_bb_address;
    return next_bb_address;
  }

  /** If iaddresses are being adjusted, the user could be sending an   **/
  /**  adjusted address, so Aria checks it and corrects the address    **/
  /**  to recover the appropriate address in Aria's view of the loaded **/
  /**  user program.                                                   **/
  {
    unsigned int temp_addr = (unsigned)next_bb_address;
    if ( (do_iaddr_fixup) && (temp_addr < 0x20000000) && (temp_addr >= 0x00005000) ) {
      next_bb_address = (unsigned *) ((unsigned)next_bb_address + global_user_iaddr_offset);
    }
  }

  /** Check for a translation of this basic block **/
  the_hash_entry_ptr = find_hash_table_entry_for(next_bb_address, INTERACT_XLATE_MODE);

  next_xlated_bb_address       = HashTable_xlation_address(*the_hash_entry_ptr);
  next_not_taken_xl_bb_address = HashTable_NT_xlation_address(*the_hash_entry_ptr);
  *microtrace_buffer_ptr       = HashTable_trace_buffer(*the_hash_entry_ptr);

  ubb_size           = HashTable_user_bb_size(*the_hash_entry_ptr);
  taken_xbb_size     = HashTable_xlated_bb_size(*the_hash_entry_ptr);
  not_taken_xbb_size = HashTable_NT_xlated_bb_size(*the_hash_entry_ptr);

  /** Save the original (user-program) address **/
  temp_ptr = next_bb_address;
    
  do_xlation = (speculative_not_taken_flag) ? 
    (next_not_taken_xl_bb_address == NULL) : (next_xlated_bb_address == NULL);

  if (do_xlation) {
    /** Determine the kind of tranlsation to do... **/
    /** IF the next instruction is a system call, then translate it to a canned sys call block **/
    if ( (global_use_safe_svc_code && (next_bb_address < (unsigned *)0x00005000)) ||
	 (OPC_IS_A_SYSTEM_CALL(opc_iword_itype(*next_bb_address))) ) {
      next_xlate_address = do_one_svc_translation(next_bb_address, the_hash_entry_ptr, 
						  speculative_not_taken_flag, INTERACT_XLATE_MODE);
    }
    /** ELSE this is a normal (user code) basic block, so translate its contents **/
    else {
      next_xlate_address = do_one_translation(next_bb_address, the_hash_entry_ptr, 
					      speculative_not_taken_flag, INTERACT_XLATE_MODE);
    }
      
    next_xlated_bb_address = HashTable_xlation_address(*the_hash_entry_ptr);
    next_not_taken_xl_bb_address = HashTable_NT_xlation_address(*the_hash_entry_ptr);
    *microtrace_buffer_ptr = HashTable_trace_buffer(*the_hash_entry_ptr);
    ubb_size = HashTable_user_bb_size(*the_hash_entry_ptr);
    taken_xbb_size = HashTable_xlated_bb_size(*the_hash_entry_ptr);
    not_taken_xbb_size = HashTable_NT_xlated_bb_size(*the_hash_entry_ptr);
      
    if ( (do_no_output_during_runs == 0) && (do_trace_dump == 1) ) {
      if ( (output_after_dynamic_bb_execution <= global_bb_executions) 
	   && (no_output_after_dynamic_bb_execution  >= global_bb_executions) 
	   && (output_after_static_bb_translation <= global_bb_index) 
	   && (no_output_after_static_bb_translation >= global_bb_index)) {
	warn_trace("  ...translated BB %llu (%llu): Oa = 0x%08x Xa = 0x%08x SXa = 0x%08x Sbf = 0x%08x Us = %u Ts = %u NTs = %u\n", 
		   global_bb_index, global_bb_executions, temp_ptr, next_xlated_bb_address, next_not_taken_xl_bb_address, 
		   *microtrace_buffer_ptr, ubb_size, taken_xbb_size, not_taken_xbb_size); 
      }
    }
      
    global_user_line_index   += ubb_size;
    global_xlated_line_index += taken_xbb_size + not_taken_xbb_size;
    global_bb_index++;
  }

  /* Backup the "old" system state */

  /****************************************/
  /** Execute the translated basic block **/
  /****************************************/
  /** Check whether we are recording system call execution times **/
  if (global_get_system_call_execution_times == 1) {
    proc_state_sc_identifier(saved_proc_state_ptr) = 0x0; 
    if ( ((global_use_safe_svc_code && (next_bb_address <= (unsigned *)0x4095)) ||
	  (OPC_IS_A_SYSTEM_CALL(opc_iword_itype(*next_bb_address))))
	 && (proc_state_GPR(saved_proc_state_ptr, 2) != the_exit_code_value) 
	 ) {
      proc_state_sc_identifier(saved_proc_state_ptr) = proc_state_GPR(saved_proc_state_ptr, 2);
    }
  }

  /** Select either the speculative or true-path translation **/
  /** Also add the executed instructions estimate   **/
  /**   from that path now to save a later if stmnt **/
  if (speculative_not_taken_flag) { 
    pc_func_desc(prog_desc) = next_not_taken_xl_bb_address;
    global_xlated_instructions += not_taken_xbb_size;
  }
  else {
    pc_func_desc(prog_desc) = next_xlated_bb_address;
    global_xlated_instructions += taken_xbb_size;
  }

  /** The copy to run when debugging output desired **/
  if (do_no_output_during_runs == 0) {
    if ( (do_proc_state_dump == 1) || (do_full_execute_dump == 1) ) {
      if ( (output_after_dynamic_bb_execution <= global_bb_executions) 
	   && (no_output_after_dynamic_bb_execution  >= global_bb_executions) 
	   && (output_after_static_bb_translation <= global_bb_index) 
	   && (no_output_after_static_bb_translation >= global_bb_index) ) {
	if (speculative_not_taken_flag) { 
	  warn_proc_state("\nExecuting the not-taken-path (speculation) translated code:\n");
	  warn_exec("\nExecuting the not-taken-path (speculation) translated code:\n");
	}
	else {
	  warn_proc_state("\nExecuting the taken-path translated code:\n");
	  warn_exec("\nExecuting the taken-path translated code:\n");
	}
	warn_exec("     Starting address %08x:\n",   pc_func_desc(prog_desc));
	warn_exec("     TOC address      %08x:\n",   toc_func_desc(prog_desc));
	warn_exec("     Other data       %08x:\n", other_func_desc(prog_desc));
	warn_processor_state(saved_proc_state_ptr, show_proc_float_state);
      }
    }

    taken_path = (unsigned *)prog_ptr(saved_proc_state_ptr, &not_taken_path);
    /* warn("exec at 0x%08x => 0x%08x\n", next_bb_address, taken_path); */
    next_bb_address = taken_path;

    /**
      if (!speculative_not_taken_flag) {
      if (((unsigned)taken_path & 0xf0000000) == 0x10000000) {
      warn("Possible problem: ...Executed.. BB %d, the code for Oa = %08x Xa = %08x and Next = %08x\n",
      global_bb_executions, temp_ptr, pc_func_desc(prog_desc), taken_path);
      }
      }
      **/

    if ( (output_after_dynamic_bb_execution <= global_bb_executions) && 
	 (no_output_after_dynamic_bb_execution  >= global_bb_executions) && 
	 (output_after_static_bb_translation <= global_bb_index) &&
	 (no_output_after_static_bb_translation >= global_bb_index) ) {
      if (do_full_execute_dump) {
	warn_exec("  ... Done. RAW: Next address = %08x (not_taken = %08x)\n", next_bb_address, not_taken_path);
      }

      if (speculative_not_taken_flag) {
	warn_trace("  ... RAW: Speculated BB %llu, the code for Oa = %08x Xa = %08x Next = %08x\n", 
		   global_bb_executions, temp_ptr, pc_func_desc(prog_desc), next_bb_address);
      }
      else {
	warn_trace("  ... RAW: Executed.. BB %llu, the code for Oa = %08x Xa = %08x Next = %08x\n", 
		   global_bb_executions, temp_ptr, pc_func_desc(prog_desc), next_bb_address);
      }
      if (do_proc_state_dump) {
	warn_processor_state(saved_proc_state_ptr, show_proc_float_state);
      }
      /**
	trace_processor_state(saved_proc_state_ptr, old_saved_proc_state_ptr, show_proc_float_state);
	**/
    }

    /** Show what is in the trace buffer after execution **/
    if (do_trace_buffer_dump) {
      unsigned total_instr_in_bb = ubb_size;
      unsigned *microtrace_buffer_address = *microtrace_buffer_ptr;
      #include "show_trace_buffer_contents.c"
    }
  }
  /** The faster copy to run when no debugging output desired **/
  else {
    taken_path = (unsigned *)prog_ptr(saved_proc_state_ptr, &not_taken_path);
    next_bb_address = taken_path;
    /** 
      if (!speculative_not_taken_flag) {
      if (((unsigned)taken_path & 0xf0000000) == 0x10000000) {
      warn("Possible problem: ...Executed.. BB %d, the code for Oa = %08x Xa = %08x and Next = %08x\n",
      global_bb_executions, temp_ptr, pc_func_desc(prog_desc), taken_path);
      }
      }
      **/
  }


  if (proc_state_sc_identifier(saved_proc_state_ptr) != 0) {
    int w1, w2;
    timebasestruct_t * Saddr  = (timebasestruct_t *) (ARIA_STATE_SAVE_SPACE + ARIA_SAVE_START_TIME);      
    timebasestruct_t * Faddr  = (timebasestruct_t *) (ARIA_STATE_SAVE_SPACE + ARIA_SAVE_FINISH_TIME);      
    timebasestruct_t * Naddr  = (timebasestruct_t *) (ARIA_STATE_SAVE_SPACE + ARIA_SAVE_NULL_TIME);      

    time_base_to_time(Saddr, TIMEBASE_SZ);
    time_base_to_time(Faddr, TIMEBASE_SZ);

#ifdef FULL_DEBUG_TRACE
    warn_trace("Saddr = %08x %08x %08x\n", Saddr->flag, Saddr->tb_high, Saddr->tb_low);
    warn_trace("Faddr = %08x %08x %08x\n", Faddr->flag, Faddr->tb_high, Faddr->tb_low);
    warn_trace("Naddr = %08x %08x %08x\n", Naddr->flag, Naddr->tb_high, Naddr->tb_low);
#endif 

    w1 = Faddr->tb_high - Saddr->tb_high;
    w2 = Faddr->tb_low  - Saddr->tb_low;
    if (w2 < 0) { 
      w1 --;
      w2 += 1000000000;
    }

    w1 = w1 - Naddr->tb_high;
    w2 = w2 - Naddr->tb_low;

    if (w2 < 0) {
      warn("ARIA: sc time (with overhead removed) is negative!\n");
    }
      
#ifdef FULL_DEBUG_TRACE 
    warn("For system call %04x : time is %u seconds and %u ns  (last_user_code_addr = 0x%08x)\n", 
	 proc_state_sc_identifier(saved_proc_state_ptr),
	 w1, w2, last_user_space_address);
#endif

    /* Set the value in the proc state space */
    proc_state_sc_exec_time_secs(saved_proc_state_ptr) = w1;
    proc_state_sc_exec_time_ns(saved_proc_state_ptr) = w2;
  }


#ifdef FULL_DEBUG_TRACE 
  if (next_bb_address > (unsigned*)0x4095) {
    last_user_space_address = next_bb_address;
  }
#endif
  
  global_bb_executions++;
  global_user_instructions   += ubb_size;
  
  *trace_size = ubb_size;
  
  if (not_taken_address_required_flag) {
    if ( (do_iaddr_fixup) && ((proc_state_NT_TARGET(saved_proc_state_ptr) & 0xf0000000) == global_user_program_iaddr_mask) ) {
      *not_taken_bb_address = (unsigned *)(proc_state_NT_TARGET(saved_proc_state_ptr) - 
					   global_user_iaddr_offset);
    }
    else {
      *not_taken_bb_address = (unsigned *)proc_state_NT_TARGET(saved_proc_state_ptr);
    }
  }
  
  /** Output the run flag check **/
  if (global_bb_executions > show_run_update_after) {
    if ((global_bb_executions % show_run_update_every) == 0) {
      unsigned curr_orig_address = (unsigned)temp_ptr;
      unsigned next_orig_address = (unsigned)next_bb_address;
      if ( (do_iaddr_fixup) && (((unsigned)next_bb_address & 0xf0000000) == global_user_program_iaddr_mask) ) {
	curr_orig_address -= global_user_iaddr_offset;
	next_orig_address -= global_user_iaddr_offset;
      }
      
      if (speculative_not_taken_flag) {
	warn("  ...Speculated BB %u: Oa= 0x%08x Xa= 0x%08x Nxt= 0x%08x  [%u BB translations]\n", 
	     global_bb_executions, curr_orig_address, pc_func_desc(prog_desc), next_orig_address,
	     global_bb_index);
      }
      else {
	warn("  ...Executed BB %u: Oa= 0x%08x Xa= 0x%08x Nxt= 0x%08x  [%u BB translations]\n", 
	     global_bb_executions, curr_orig_address, pc_func_desc(prog_desc), next_orig_address,
	     global_bb_index);
      }
      
#ifdef FULL_DEBUG_TRACE 
      if (last_bb_printed != temp_ptr) {
	for (i = 0; i < ubb_size; i++) {
	  unsigned * trbuf = *microtrace_buffer_ptr;
	  fprintf(mystderr, "\t0x%08x  0x%08x  %u  0x%08x  %u\n", 
		  trbuf[PER_INSTR_TB_SIZE*i+0], trbuf[PER_INSTR_TB_SIZE*i+1], 
		  trbuf[PER_INSTR_TB_SIZE*i+2], trbuf[PER_INSTR_TB_SIZE*i+3], 
		  trbuf[PER_INSTR_TB_SIZE*i+4]);
	}
      }
      last_bb_printed = temp_ptr;
#endif
    }
  }

  /** Dump final statistics check **/
  if (show_final_aria_statistics == 1) {
    if (next_bb_address == ARIA_EXIT_FLAG_ADDRESS) {
      unsigned save_print = do_proc_state_dump;
      unsigned save_dstart = output_after_dynamic_bb_execution;
      unsigned save_dend   = no_output_after_dynamic_bb_execution;
      unsigned save_sstart = output_after_static_bb_translation;
      unsigned save_send   = no_output_after_static_bb_translation;
      do_proc_state_dump = 1;
      output_after_dynamic_bb_execution = 0;
      no_output_after_dynamic_bb_execution = INT_MAX;
      output_after_static_bb_translation = 0;
      no_output_after_static_bb_translation = INT_MAX;
      if (speculative_not_taken_flag == 1) {
	/**
	  warn("Specultive exit from program detected...\n");
	  warn_processor_state(saved_proc_state_ptr, show_proc_float_state);
	  **/
      }
      else {
	unsigned curr_orig_address = (unsigned)temp_ptr;
	unsigned next_orig_address = (unsigned)next_bb_address;
	if ( (do_iaddr_fixup) && (((unsigned)next_bb_address & 0xf0000000) == global_user_program_iaddr_mask) ) {
	  curr_orig_address -= global_user_iaddr_offset;
	  next_orig_address -= global_user_iaddr_offset;
	}
	warn("Exit from program detected...\n");
	warn("  ** Executed BB %d: Oa= 0x%08x (0x%08x) Xa= 0x%08x Next = 0x%08x (0x%08x)\n", 
	     global_bb_executions, temp_ptr, curr_orig_address, pc_func_desc(prog_desc), 
	     next_bb_address, next_orig_address);
	warn_processor_state(saved_proc_state_ptr, show_proc_float_state);
	
	warn("\n===========================================================================\n");
	warn("And the final statistics (within Aria) are:\n\n");
	dump_aria_run_statistics(saved_proc_state_ptr);
	warn("Returning to timer model...\n");
      }
      
      do_proc_state_dump = save_print;
      output_after_dynamic_bb_execution = save_dstart;
      no_output_after_dynamic_bb_execution = save_dend;
      output_after_static_bb_translation = save_sstart;
      no_output_after_static_bb_translation = save_send;
    }
  }

#ifdef FULL_DEVELOPMENT_DEBUG_CODE
  if (global_bb_executions > global_do_verify_hash_after) {
    if ((global_bb_executions % global_do_verify_hash_every) == 0) {
      verify_hash_table();
    }
  }
#endif

  if ( (do_safe_exit_flag) && (((unsigned)next_bb_address) == the_exit_routine_address) ) {
    #ifdef FULL_DEBUG
      fprintf(mystderr, "<END> Hit the exit routine address ...\n");
    #endif
    return ARIA_EXIT_FLAG_ADDRESS;
  }

  if ( (do_iaddr_fixup) && (((unsigned)next_bb_address & 0xf0000000) == global_user_program_iaddr_mask) ) {
    next_bb_address = (unsigned *)((unsigned)next_bb_address - global_user_iaddr_offset);
  }

  IFFULLDEBUG(if (next_bb_address == (unsigned *)the_restart_address) {
    fprintf(stderr, "Found the restart address sought\n");
    do_full_xlate_dump       = 0;
    do_full_execute_dump     = 0;
    do_hash_tables_dump      = 0;
    do_proc_state_dump       = 0;
    do_bb_regs_use_def_dump  = 0;
    do_trace_dump            = 0;
    do_trace_buffer_dump     = 0;
    do_show_runtime_warnings = 0;
    do_no_output_during_runs = 1;
    output_after_static_bb_translation = 0x7fffffff;
    output_after_dynamic_bb_execution  = 0x7fffffff;
  } );

  curr_window_count += ubb_size;

  return next_bb_address;
}


/*******************************************************************
  PROGRAM NAME..... get_new_process_state_space

  FUNCTION ........ Returns a pointer to a user process state 
     structure, which can then be used by the timer program.

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... NULL indicates failure
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

proc_state_type * 
aria_get_new_process_state_space()
{
  proc_state_type * the_return_value = NULL;

  if ((curr_process_state_space_ptr+1) <= last_process_state_space_position) {
    the_return_value = curr_process_state_space_ptr;
    curr_process_state_space_ptr++;
  }

  the_return_value->_fpr_0_value = 0.0;

  return the_return_value;
}
