/*                                                                     
** COMPONENT_NAME: MET / Aria 
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... aria_main, print_usage, read_input_flags

  FUNCTION ........ Provides the set of routines comprising the 
     local aria main program.

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 
  CALLED-ROUTINES.. 
  CALLED-BY........ None
  INCLUDED-BY...... 
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

#include <limits.h>
#include <posix_stat.h>

#include "aix_systemcfg.h"

#include "aix_a.out.h"
#include "aix_errno.h"
#include "aix_ldr.h"
#include "aix_mman.h"
#include "posix_fcntl.h"
#include "ansi_stdio.h"
#include "ansi_stdlib.h"

#include "ansi_assert.h"

#include "set141516.h"

#include "aria_debug.h"

#include "aria_state.h"

#include "io_msgs.h"

#include "opc.h"

#include "proc_state.h"

#include "globtypes.h"
#include "globdefs.h"

#include "aria_main.h"

#include "opc_is_implemented_types.h"

/** MACROS **/


/** Types **/

/** Forward function declarations **/

/** (Kernel-code independent) Global Variables **/

global_flags_type * the_global_flags = (global_flags_type *)(GLOBAL_FLAGS_STATE_SPACE_START);

unsigned  * prog_start;
unsigned  * xlate_mem_start;
unsigned    user_program_start_address;

aria_func_desc * prog_desc;
func_ptr    prog_ptr;

int     user_argc;
char ** user_argv;
char  * user_envp;

unsigned * user_stack_address_value; 

proc_state_type * curr_process_state_space_ptr;
proc_state_type * last_process_state_space_position;

unsigned global_do_show_system_xlation;
unsigned global_do_show_system_pre_run_proc_state;
unsigned global_do_show_system_proc_state;
unsigned do_interactive_aria_system_run;

unsigned long long show_run_update_every;
unsigned long long show_run_update_after;

unsigned long long capture_window_size;
unsigned long long skip_window_size;
int      do_uniform_sampling_in_aria = 0;
unsigned curr_window_count = 0;

unsigned long long global_do_verify_hash_after;
unsigned long long global_do_verify_hash_every;

/*******************************************************************
  PROGRAM NAME..... aria_main

  FUNCTION ........ Serves as teh local main program, setting up 
     essential information, parsing the input flags, and so forth.
     It then invokes the specified model program, returning any 
     value returned by that program.

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... return value determined by called model program.
  CALLED-ROUTINES.. 
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

int
aria_main( int my_aria_argc, 
	   char * my_aria_argv[], 
	   char * my_aria_envp,
	   unsigned * stack_address,
	   caddr_t xlate_addr
	   )
{
  char *	 argv0_name;
  int		 prog_fd;
  char  	 prog_name[256];
  char *	 prog_name2;
  struct filehdr prog_hdr;
  AOUTHDR	 prog_auxhdr;
  unsigned	 prog_textsize;
  unsigned *	 prog_end;

  func_ptr       model_ptr;
  char *	 model_name;

  int     model_argc = 0;
  char ** model_argv = NULL;
  char  * model_envp = NULL;

  int		 n;

  int user_arg_posn = 0;
  int model_arg_posn = 0;

  int my_stderr_fdes = dup(2);

  int return_value; 
  int i;

  struct stat file_status;

  /* Dump the obligatory legal statement */
  #ifdef EXTERNAL_VERSION
  for (i = 0; i < 24; ++i) {
    printf("%s",legal_statement[i]);
  }
  fflush(stdout);
  #endif

  /* Get a new (aria-only) copy of the stderr path */
  mystderr    = fdopen(my_stderr_fdes, "w");

  IFDEBUG(warn("[aria_main] Entered the aria_main with...\n");
	  warn("[aria_main]      int        argc   value  = %d\n"    , my_aria_argc); 
	  warn("[aria_main]      char     * argv[] value  = 0x%08x\n", my_aria_argv);
	  warn("[aria_main]      char     * envp   value  = 0x%08x\n", my_aria_envp);
	  warn("[aria_main]      unsigned * stack_address = 0x%08x\n", stack_address);
	  warn("[aria_main]      void     * xlate_address = 0x%08x\n", xlate_addr);
	  );

  /** Copy the stack address to global variable space       **/
  /*  If the actual stack address is in 0x2f000000 region,   */
  /*   the speculation code cannot "see" it, so move it down */
  /*   a little, otherwise, keep it unchanged                */
  if (((unsigned)stack_address & 0xff000000) == 0x2f000000) {
    /* Drop the user stack into segment 0x2e000000 */
    user_stack_address_value = (unsigned *) (0x2e000000 | ((unsigned)stack_address & 0x00ffffff));
  }
  else {
    /* Leave the user stack unchanged */
    user_stack_address_value = stack_address;
  }
  IFDEBUG( warn("[aria_main] Set the  user_stack_address_value = 0x%08x\n",  user_stack_address_value); );

  /** Set up the global variables' initial values **/
  the_exit_code_value      = 0x17;
  the_exit_routine_address = 0x0;
  do_safe_exit_flag = 1;

  /** Set up to return a NOP in the trace for illegal instructions **/
  global_return_nop_in_trace_for_illegal_speculatives = 1;
  global_spec_nop_iword = OPC_ASM_OR(0,0,0);
  global_spec_nop_itype = opc_itype(opc_index(global_spec_nop_iword));
  
  /** Default to no internal (Aria) simulation mode **/
  global_do_aria_simulation = 0;

  /* Allow globally-accessible xcoff/timer files */
  allow_global_access_permissions = 0;

  /* Do iaddress fixing (to "match" xcoff offsets) */
  do_iaddr_fixup = 1;

  /* The timer model uses no floating-point registers (don't save in Aria state) */
  no_FP_registers_used_in_model_flag  = 0;
  
  /* The timer model does not need Aria to calculate a not-taken address for each block */
  not_taken_address_required_flag  = 0;

  /* Replace the SVC instructions with BCLR instructions to retain trace semantics consistency */
  turn_svc_into_a_bclr_instruction = 1;
  global_use_safe_svc_code = 0;
  global_get_system_call_execution_times = 0;

  /* Set the user program ISA to be the PowerPC ISA */
  the_user_program_architecture = ARIA_PPC_ISA;

  /* Reset the Arai statistics counters */
  global_bb_index      = 0;
  global_bb_executions = 0;
  global_max_size_ubb_counter  = 0;
  global_user_stop_condition_counter = 0; 
  global_addressibility_counter = 0;

  /* Aria should do not-taken (speculative) translations */
  do_not_taken_path_translations = 1;

  /* Use the Aria canned string instruction code */
  do_canned_string_code = 1;

  /* Generate the dynamic trace buffer information */
  do_dynamic_trace_generation = 1;

  /* Use the older, original user process state save space initialization code option. */
  /*  Note that we DO NOT use this code by default any more                            */
  global_do_old_initial_proc_state_code = 0;

  /** OUTPUT flags: **/

  /* Give no periodic run update messages */
  show_run_update_every = 0;
  show_run_update_after = INT_MAX;

  /* Do verification checks on hash table each cycle */
  global_do_verify_hash_every = 0;
  global_do_verify_hash_after = INT_MAX;

  /* Turn off all output */
  do_no_output_during_runs  = 1;

  /* Set ouput limiters to block all output */
  output_after_static_bb_translation    = INT_MAX;
  no_output_after_static_bb_translation = INT_MAX;

  output_after_dynamic_bb_execution    = INT_MAX;
  no_output_after_dynamic_bb_execution = INT_MAX;

  /* Set all output info flags to OFF */
  do_full_xlate_dump         = 0;
  do_full_execute_dump       = 0;
  do_trace_dump              = 0;
  do_trace_buffer_dump       = 0;
  do_bb_regs_use_def_dump    = 0;
  do_hash_tables_dump        = 0;
  do_proc_state_dump         = 0;
  show_proc_float_state      = 0;
  show_final_aria_statistics = 0;
  /* except the run-time warning messages */
  do_show_runtime_warnings   = 1;

  /* Kernel mode inform globals */
  global_do_show_system_xlation = 0;
  global_do_show_system_pre_run_proc_state = 0;
  global_do_show_system_proc_state = 0;
  do_interactive_aria_system_run = 0;

  /* Default alignment is on a word-boundary */
  the_xlated_code_alignment   = 4; 

  /* Set max block size limit to default to 32 user instructions */
  global_max_block_size_limit = 32;
  /* Set to use limit for all user basic blocks */
  use_block_limit_after_static_bb_translation     = 0;
  use_block_unlimited_after_static_bb_translation = INT_MAX;

  do_aix_4_2_run = 0;

  /* Set the global pointers into the process saved state memory heap space */
  IFDEBUG(warn("[aria_main] Setting  last_process_state_space_position = 0x%08x\n", (proc_state_type *)PROC_STATE_LAST_POSN);
	  warn("[aria_main] Setting  curr_process_state_space_ptr      = 0x%08x\n", (proc_state_type *)PROC_STATE_FIRST_POSN);
	  );

  last_process_state_space_position = (proc_state_type *)PROC_STATE_LAST_POSN;
  curr_process_state_space_ptr      = (proc_state_type *)PROC_STATE_FIRST_POSN;

  IFDEBUG(warn("[aria_main] Have set last_process_state_space_position = 0x%08x\n", last_process_state_space_position);
	  warn("[aria_main] Have set curr_process_state_space_ptr      = 0x%08x\n", curr_process_state_space_ptr     );
	  );

  /* Set the cache flushing/synchronizing function */
  /*  We use the system configuration information  */
  /*  to select whether to use the POWER version   */
  /*  or the PowerPC version.                      */
#ifndef DO_64_BIT_COMPILE
  if ( __power_pc() ) {
    the_machine_isa_type = OPC_PPC_ISA;
    IFDEBUG(warn("Setting the_machine_isa_type to OPC_PPC_ISA = 0x%02x\n", OPC_PPC_ISA););
  }
  else if ( __power_rs() ) {
    if (__power_rs1()) {
      the_machine_isa_type = OPC_POWER_ISA;
      IFDEBUG(warn("Setting the_machine_isa_type to OPC_POWER_ISA = 0x%02x\n", OPC_POWER_ISA););
    }
    else if (__power_rsc()) {
      the_machine_isa_type = OPC_POWER_ISA;
      IFDEBUG(warn("Setting the_machine_isa_type to OPC_POWER_ISA = 0x%02x\n", OPC_POWER_ISA););
    }
    else if (__power_rs2()) {
      the_machine_isa_type = OPC_POWER2_ISA;
      IFDEBUG(warn("Setting the_machine_isa_type to OPC_POWER2_ISA = 0x%02x\n", OPC_POWER2_ISA););
    }
  }
#else
  if ( __power_64() ) {
    the_machine_isa_type = OPC_AMAZON_ISA; /* Set to one that has no simulation code right now */
    IFDEBUG(warn("[aria_main] Setting the_machine_isa_type to 0x%02x\n", the_machine_isa_type); );
  }
  else {
    warn("Unsupported type: POWER architecture for 64-bit mode!\n");
    exit(-1);
  }
#endif

  /***************************************************/
  /** Start analysis of the user input command line **/
  /***************************************************/

  /** Check the number of arguments **/
  argv0_name = my_aria_argv[0];
  if( my_aria_argc < 2 ) {
    print_usage(my_aria_argc, my_aria_argv);
    return EXIT_FAILURE;
  }

  /** Read in the input (command line) switch values **/
  if (read_input_flags(my_aria_argc, my_aria_argv, 
		       &user_arg_posn, &user_argc, 
		       &model_arg_posn, &model_argc) 
      != 0) {
    fprintf(mystderr, "Problem reading the command-line input... Exiting\n");
    print_usage(my_aria_argc, my_aria_argv);
    return EXIT_FAILURE;
  }

  IFDEBUG(warn("[aria_main] Returned from read_input_flags with model argc = %d argv_pos = %d\n", model_argc, model_arg_posn); );

  /** Set pointers into argv[] for user and model sections **/
  model_name = my_aria_argv[model_arg_posn];
  model_argv = &my_aria_argv[model_arg_posn];
  model_envp = my_aria_envp;

  user_envp = my_aria_envp;

  /** Set up the hash table **/
  initialize_hash_table();

  /** Set up the disassembler **/
  opc_init();

  /** Check that these files do not have global access permissions; **/
  /**  NOTE: global permissions allow the file to be loaded into    **/
  /**   segment 0xd, and do not unload the program nicely at exit.  **/
  if (allow_global_access_permissions == 0) {
    /** Get the user program and model program names **/
    user_argv = &my_aria_argv[user_arg_posn];
    prog_name2 = my_aria_argv[user_arg_posn];
    
    if (stat(prog_name2, &file_status) == 0) {
      if ( (file_status.st_mode & S_IROTH) ) {
	warn("ARIA: ERROR: You should not run Aria with a globally-read--enabled xcoff file (%s)\n", prog_name2);
	return EXIT_FAILURE;
      }
    }
    else {
      warn("ARIA: ERROR: Cannot check the status of xcoff file %s, errno=%d \n", prog_name2, errno);
      return EXIT_FAILURE;
    }
  }
    
  if (stat(model_name, &file_status) == 0) {
    if ( (file_status.st_mode & S_IROTH) ) {
      warn("ARIA: ERROR: You should not run Aria with a globally-read--enabled timer model (%s)\n", model_name);
      return EXIT_FAILURE;
    }
  }
  else {
    warn("ARIA: ERROR: Cannot check the status of xcoff file %s\n", model_name);
    return EXIT_FAILURE;
  }

  /*****************************************/
  /** Get the extent of the user program  **/
  /*****************************************/
  user_argv = &my_aria_argv[user_arg_posn];

  {
    char * currpath = getenv("PWD");
    if ((currpath == NULL) || (my_aria_argv[user_arg_posn][0] == '/')) {
      sprintf(prog_name, "%s", my_aria_argv[user_arg_posn]);
    }
    else {
      sprintf(prog_name, "%s/%s", currpath, my_aria_argv[user_arg_posn]);
    }
  }
  
  prog_fd = open( prog_name, O_RDONLY, 0);
  if( prog_fd == -1 ) {
    warn("%s: could not open program %s\n", argv0_name, prog_name);
    return EXIT_FAILURE;
  }
  
  n = read( prog_fd, &prog_hdr, sizeof( prog_hdr) );
  if( n != sizeof(prog_hdr) ) {
    warn("%s: could not read header for %s\n", argv0_name, prog_name);
    return EXIT_FAILURE;
  }
  
  n = read( prog_fd, &prog_auxhdr, sizeof( prog_auxhdr) );
  if( n != sizeof(prog_auxhdr) ) {
    warn("%s: could not read auxiliary header for %s\n", argv0_name, prog_name);
    return EXIT_FAILURE;
  }
  
  /** ASSUME that the text_start is the starting address for this program **/
  /**  NOTE: SHOULD CHANGE THIS TO LOAD AND DECODE THE PROGRAM ENTRY DESCRIPTION **/
  /**  NOTE2: If the text_start is in segment 0x0, then we have a problem, so we     **/
  /**           alter the text_start to be in segment 0x1, and following code works. **/
  if ((prog_auxhdr.text_start & 0xf0000000) == 0x00000000) {
    user_program_start_address = 0x10000000 | (prog_auxhdr.text_start & 0x0fffffff);
  }
  else {
    user_program_start_address = prog_auxhdr.text_start;
  }
  
  prog_textsize = prog_auxhdr.tsize / 4;
  close( prog_fd );
  
  /** Do the load of the program **/
  prog_ptr = load( prog_name, 1, 0);
  if( prog_ptr == 0 ) {
    warn("%s: could not load program %s, errno=%d (\"%s\")\n", argv0_name, prog_name, errno, strerror(errno));
    return EXIT_FAILURE;
  }
  
  prog_desc  = (aria_func_desc *)prog_ptr;
  prog_start = pc_func_desc(prog_desc);
  prog_end   = prog_start + prog_textsize;

  { 
    int lb_res = loadbind(0, (void *)aria_main, (void *)prog_ptr);
    if (lb_res != 0) {
      warn("%s: could not loadbind program %s, errno=%d (\"%s\")\n", argv0_name, prog_name, errno, strerror(errno));
      return EXIT_FAILURE;
    }
  }      
  
  raw_address_of_program_start(curr_process_state_space_ptr) = (unsigned)prog_start;
  raw_address_of_program_end(curr_process_state_space_ptr)   = (unsigned)prog_end;

  global_user_program_iaddr_mask = ((unsigned)prog_start & 0xf0000000);
  global_user_iaddr_offset = (unsigned)prog_start - user_program_start_address;

  IFDEBUG(warn("The global_user_iaddr_offset = 0x%08x\n", global_user_iaddr_offset););

  /**********************************/
  /** Now do the load of the model **/
  /**********************************/
  IFDEBUG(warn("[aria_main] Loading the model \"%s\"\n", model_name); );
  model_ptr  = load( model_name, 1, 0);

  if( model_ptr == 0 ) {
    warn("%s: could not load model %s, errno=%d\n", argv0_name, model_name, errno);
    return EXIT_FAILURE;
  }
  IFDEBUG(else {
            warn("[aria_main] Loaded the model \"%s\" just fine\n", model_name);
           }
	  );

  /** Set the value of some global variables, including **/
  /*    the start of the translation memory              */
  /*    the next translation memory address              */
  /*    the global user line index number                */
  /*    the global translated line index number          */
  /*    the global number of basic blocks translated     */
  xlate_mem_start = (unsigned *)xlate_addr;
  next_xlate_address = xlate_mem_start;
  IFDEBUG(warn("[aria_main] Set next_xlate_address to 0x%08x\n", next_xlate_address); );

  /** Set the _exit system call value automatically **/
  {
    /* Get the function descriptor for the _exit routine */
    /*  (which does the system call)                     */
    unsigned * taddrp = (unsigned *)_exit;

    /* Use the TOC value (GPR 2) as the exit code value */
    the_exit_code_value = *(taddrp+1);

    /* Print debug information if desired */
    IFDEBUG(warn("[aria_main] the _exit descriptor is at 0x%08x\n", taddrp);
	    warn("[aria_main]        _exit address 0x%08x\n",*taddrp);
	    warn("[aria_main]        _exit TOC     0x%08x\n",*(taddrp+1));
	    warn("[aria_main]        _exit other   0x%08x\n",*(taddrp+2));
	    warn("[aria_main]        the_exit_code_value = 0x%08x\n", the_exit_code_value);
	    );
  }

  /** Set the exit routine address (automatically) **/
  {
    /* Get the function descriptor for the exit routine */
    unsigned * taddrp = (unsigned *)exit;

    /* Use the TOC value (GPR 2) as the exit code value */
    the_exit_routine_address = *taddrp;

    /* Print debug information if desired */
    IFDEBUG(warn("[aria_main] the exit routine is at: 0x%08x\n", taddrp);
	    warn("[aria_main]        exit address 0x%08x\n",*taddrp);
	    warn("[aria_main]        exit TOC     0x%08x\n",*(taddrp+1));
	    warn("[aria_main]        exit other   0x%08x\n",*(taddrp+2));
	    );
  }

  /** If the user wants timings of the system calls, **/
  /**  set the address of the read_real_time routine **/
  if (global_get_system_call_execution_times == 1) {
    /* Get the address of the function descriptor save space */
    unsigned * Aaddr  = (unsigned *) (ARIA_STATE_SAVE_SPACE + ARIA_READ_REAL_TIME);
    /* get the read_real_time routine function descriptor */
    unsigned * taddrp = (unsigned *)read_real_time;

    /* Save (copy) the function descriptor values */
    *Aaddr  =*taddrp;
    *(Aaddr+1) = *(taddrp+1);
    *(Aaddr+2) = *(taddrp+2);

    IFDEBUG(warn("[aria_main] Saving read_real_time descriptor to 0x%08x\n", Aaddr);
	    warn("[aria_main]        read_real_time address 0x%08x to 0x%08x = 0x%08x\n",
		 *taddrp, Aaddr, *Aaddr);
	    warn("[aria_main]        read_real_time TOC     0x%08x to 0x%08x = 0x%08x\n",
		 *(taddrp+1), (Aaddr+1), *(Aaddr+1));
	    warn("[aria_main]        read_real_time other   0x%08x to 0x%09x = 0x%08x\n",
		 *(taddrp+2), (Aaddr+2), *(Aaddr+2));
	    );

    /* Pre-set the saved null time to all zeros */
    Aaddr      = (unsigned *) (ARIA_STATE_SAVE_SPACE + ARIA_SAVE_NULL_TIME);
    *Aaddr     = 0x00000000;
    *(Aaddr+1) = 0x00000000;
    *(Aaddr+2) = 0x00000000;
  }    

  IFDEBUG({
            int ii;
	    warn("[aria_main] Calling model_ptr with:\n");
	    warn("model argc = %d and model_argv = 0x%08x  and model_envp = 0x%08x\n", 
		 model_argc, model_argv, model_envp);
	    for (ii = 0; ii < model_argc; ii++) {
	      warn("[aria_main] \targv[%d] = \"%s\"\n", ii, model_argv[ii]);
	    }
          } );

  /** Invoke the user model (driver) program;
   ** NOTE: we have to zap r30 and r31 because of some eccentricity
   ** in the startup code (DONT ASK!!!) (look at __start())
   **/
  if (do_aix_4_2_run) {
    set1415(model_argc, model_argv, model_envp); 
    zap3031();
  }
  return_value = (int) model_ptr(model_argc, model_argv); /* , model_envp); */

  return return_value;
}


/*******************************************************************
  PROGRAM NAME..... read_input_flags

  FUNCTION ........ Parses the input command-line arguments, and 
     sets (global) variables to indicate the flag conditions that 
     are requested by the user.

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 0 = good, other = error in parsing
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

int
read_input_flags(int    argc, 
		 char * argv[],
		 int  * user_arg_posn,
		 int  * u_argc,
		 int  * model_arg_posn,
		 int  * model_argc
		 ) 
{
  int ret_val = 0;
  int flag_type = 0;
  int u_argc_count  = 0;
  int model_argc_count = 0;

  int i = 1;

  while (i < argc) {
    switch (flag_type) {
    case 1:
      if (strcmp(argv[i], "--PROGRAM") == 0) {
	i++;
	if (i < argc) {
	  *user_arg_posn = i;
	  u_argc_count = 1;
	  flag_type = 2;
	}
	else {
	  fprintf(mystderr, "MUST specify the user program name after the --PROGRAM flag\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }
      else {
	model_argc_count++;
      }
      break;

    case 2:
      u_argc_count++;
      break;

    case 0:
      if (strcmp(argv[i], "--MODEL") == 0) {
	i++;
	if (i < argc) {
	  *model_arg_posn = i;
	  flag_type = 1;
	  model_argc_count = 1;
	}
	else {
	  fprintf(mystderr, "MUST specify the model name after the --MODEL flag\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }
      else if (strcmp(argv[i], "-aix42") == 0) {
	do_aix_4_2_run = 1;
      }
      else if (strcmp(argv[i], "-show_all") == 0) {
	do_full_xlate_dump       = 1;
	do_full_execute_dump     = 1;
	do_hash_tables_dump      = 1;
	do_proc_state_dump       = 1;
	do_bb_regs_use_def_dump  = 1;
	do_trace_dump            = 1;
	do_trace_buffer_dump     = 1;
	do_show_runtime_warnings = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }
      else if (strcmp(argv[i], "-show_all_xlate") == 0) {
	do_full_xlate_dump      = 1;
	do_hash_tables_dump     = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }
      else if (strcmp(argv[i], "-show_all_execute") == 0) {
	do_full_execute_dump     = 1;
	do_proc_state_dump       = 1;
	do_bb_regs_use_def_dump  = 1;
	do_show_runtime_warnings = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }
      else if (strcmp(argv[i], "-show_run_trace") == 0) {
	do_trace_dump            = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }
      else if (strcmp(argv[i], "-show_trace_buffer") == 0) {
	do_trace_buffer_dump     = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }
      else if (strcmp(argv[i], "-show_bb_reg_use_def") == 0) {
	do_bb_regs_use_def_dump  = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }
      else if (strcmp(argv[i], "-show_hash_table") == 0) {
	do_hash_tables_dump      = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }
      else if (strcmp(argv[i], "-show_no_hash_table") == 0) {
	do_hash_tables_dump      = 0;
      }
      else if (strcmp(argv[i], "-no_warnings") == 0) {
	do_show_runtime_warnings = 0;
      }
      else if (strcmp(argv[i], "-show_proc_state") == 0) {
	do_proc_state_dump       = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }
      else if (strcmp(argv[i], "-show_proc_floats") == 0) {
	show_proc_float_state   = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }

      else if (strcmp(argv[i], "-show_final_stats") == 0) {
	show_final_aria_statistics = 1;
      }

      else if (strcmp(argv[i], "-verify_hash") == 0) {
#ifndef FULL_DEVELOPMENT_DEBUG_CODE
	warn("Aria does not generally support the -verify_hash option...\n");
	warn("  This option is only for developmental debugging. \n");
	warn("  If you need access to this option, contact the author by email:\n");
	warn("    John-David Wellman : wellman@watson.ibm.com\n");
#else
	i++;
	if (i < argc) {
	  global_do_verify_hash_after = strtoull(argv[i], 0, 0);
	}
	else {
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
	i++;
	if (i < argc) {
	  global_do_verify_hash_every = strtoull(argv[i], 0, 0);
	}
	else {
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
#endif
      }

      else if (strcmp(argv[i], "-update_on_dbbs") == 0) {
	i++;
	if (i < argc) {
	  show_run_update_after = strtoull(argv[i], 0, 0);
	}
	else {
	  ret_val = 1;
	  warn("You must specify  show_after and update_every values for -update_on_dbbs option.");
	  goto exit_this_while_loop;
	}
	i++;
	if (i < argc) {
	  show_run_update_every = strtoull(argv[i], 0, 0);
	}
	else {
	  ret_val = 1;
	  warn("You must specify  show_after and update_every values for -update_on_dbbs option.");
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-usample") == 0) {
	i++;
	if (i < argc) {
	  capture_window_size = strtoull(argv[i], 0, 0);
	}
	else {
	  ret_val = 1;
	  warn("You must specify a capture window size and skip window size for -usample option.");
	  goto exit_this_while_loop;
	}
	i++;
	if (i < argc) {
	  skip_window_size = strtoull(argv[i], 0, 0);
	}
	else {
	  ret_val = 1;
	  warn("You must specify a capture window size and skip window size for -usample option.");
	  goto exit_this_while_loop;
	}
	do_uniform_sampling_in_aria = 1;
      }

      else if (strcmp(argv[i], "-max_block_size") == 0) {
	i++;      
	if (i < argc) {
	  global_max_block_size_limit = strtoul(argv[i], 0, 0);
	  if ( global_max_block_size_limit > ABSOLUTE_MAXIMUM_USER_INSTR_IN_A_BLOCK ) {
	    warn("Cannot specify a max block size larger than %u\n", ABSOLUTE_MAXIMUM_USER_INSTR_IN_A_BLOCK);
	    ret_val = 1;
	    goto exit_this_while_loop;
	  }
	}
	else {
	  warn("Must specify a max block size value with -max_block_size\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }
    
      else if (strcmp(argv[i], "-output_after_sbb") == 0) {
	i++;
	if (i < argc) {
	  output_after_static_bb_translation = strtoull(argv[i], 0, 0);
	}
	else {
	  warn("Must specify a number of static block translations with -output_after_sbb\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-output_after_dbb") == 0) {
	i++;      
	if (i < argc) {
	  output_after_dynamic_bb_execution = strtoull(argv[i], 0, 0);
	}
	else {
	  warn("Must specify a number of dynamic block executions with -output_after_dbb\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-no_output_after_sbb") == 0) {
	i++;
	if (i < argc) {
	  no_output_after_static_bb_translation = strtoull(argv[i], 0, 0);
	}
	else {
	  warn("Must specify a number of static block translations with -no_output_after_sbb\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-no_output_after_dbb") == 0) {
	i++;      
	if (i < argc) {
	  no_output_after_dynamic_bb_execution = strtoull(argv[i], 0, 0);
	}
	else {
	  warn("Must specify a number of dynamic block executions with -no_output_after_dbb\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-output_at_address") == 0) {
	i++;
	if (i < argc) {
	  output_at_original_address = strtoul(argv[i], 0, 0);
	}
	else {
	  warn("Must specify an address with -output_at_address\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-orig_user_stack") == 0) {
	; /** Handled by the main.c code **/
      }

      else if (strcmp(argv[i], "-align_xlated_code") == 0) {
	i++;      
	if (i < argc) {
	  unsigned power_of_two = (strtoul(argv[i], 0, 0) + 2);
	  the_xlated_code_alignment = 1 << power_of_two;
	}
	else {
	  warn("Must specify a word alignment (power of two) with -align_xlated_code\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-block_lim_after_sbb") == 0) {
	i++;
	if (i < argc) {
	  use_block_limit_after_static_bb_translation = strtoull(argv[i], 0, 0); 
	}
	else {
          warn("Must specify a number of static block translations with -block_lim_after_sbb\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }
      else if (strcmp(argv[i], "-block_unlim_after_sbb") == 0) {
	i++;
	if (i < argc) {
	  use_block_unlimited_after_static_bb_translation = strtoull(argv[i], 0, 0); 
	}
	else {
          warn("Must specify a number of static block translations with -no_block_lim_after_sbb\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-speculation_support") == 0) {
	do_not_taken_path_translations = 1;
      }
      else if (strcmp(argv[i], "-no_speculation_support") == 0) {
	do_not_taken_path_translations = 0;
      }

      else if (strcmp(argv[i], "-canned_string_code") == 0) {
	do_canned_string_code = 1;
      }
      else if (strcmp(argv[i], "-no_canned_string_code") == 0) {
	do_canned_string_code = 0;
      }

      else if (strcmp(argv[i], "-dynamic_trace") == 0) {
	do_dynamic_trace_generation = 1;
      }
      else if (strcmp(argv[i], "-no_dynamic_trace") == 0) {
	do_dynamic_trace_generation = 0;
      }

      else if (strcmp(argv[i], "-old_init_proc_state_code") == 0) {
	global_do_old_initial_proc_state_code = 1;
      }

      else if (strcmp(argv[i], "-iaddr_fixup") == 0) {
	do_iaddr_fixup = 1;
      }
      else if (strcmp(argv[i], "-no_iaddr_fixup") == 0) {
	do_iaddr_fixup = 0;
      }

      else if (strcmp(argv[i], "-report_bclr_for_sc") == 0) {
	turn_svc_into_a_bclr_instruction = 1;
      }
      else if (strcmp(argv[i], "-report_svc_for_sc") == 0) {
	turn_svc_into_a_bclr_instruction = 0;
      }
      else if (strcmp(argv[i], "-report_sc_for_sc") == 0) {
	turn_svc_into_a_bclr_instruction = 0;
      }

      else if (strcmp(argv[i], "-raw_spec_code") == 0) {
	global_return_nop_in_trace_for_illegal_speculatives = 0;
      }
      else if (strcmp(argv[i], "-spec_nop_instr") == 0) {
	i++;
	if (i < argc) {
	  global_spec_nop_iword = strtoul(argv[i], 0, 0);
	  global_spec_nop_itype = opc_itype(opc_index(global_spec_nop_iword));
	}
	else {
	  warn("Must specify an iword to use as the speculative path instr for illegals with -spec_nop_instr\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-simulate") == 0) {
	global_do_aria_simulation = 1;
      }
      else if (strcmp(argv[i], "-no_simulate") == 0) {
	global_do_aria_simulation = 0;
      }

      else if (strcmp(argv[i], "-show_sc_instr") == 0) {
	global_use_safe_svc_code = 0;
      }
      else if (strcmp(argv[i], "-safe_sc_code") == 0) {
	global_use_safe_svc_code = 1;
      }

      else if (strcmp(argv[i], "-show_sc_times") == 0) {
	global_get_system_call_execution_times = 1;
      }

      else if (strcmp(argv[i], "-fpr_use_in_model") == 0) {
	no_FP_registers_used_in_model_flag = 0;
      }
      else if (strcmp(argv[i], "-no_fpr_use_in_model") == 0) {
	no_FP_registers_used_in_model_flag = 1;
      }

      else if (strcmp(argv[i], "-no_NT_addr_reqd") == 0) {
	not_taken_address_required_flag = 0;
      }
      else if (strcmp(argv[i], "-NT_addr_reqd") == 0) {
	not_taken_address_required_flag = 1;
      }
      else if (strcmp(argv[i], "-allow_global_access") == 0) {
	allow_global_access_permissions = 1;
      }
    
      else if (strcmp(argv[i], "-full_safe_exit") == 0) {
	do_safe_exit_flag = 1;
      }
      else if (strcmp(argv[i], "-less_safe_exit") == 0) {
	do_safe_exit_flag = 0;
      }
    
      else if (strcmp(argv[i], "-oslevel") == 0) {
	i++;
	if (i < argc) {
	  if ((strcmp(argv[i], "AIX4.1-L") == 0) || (strcmp(argv[i], "AIX4.1.4-L") == 0)) {
	    /*
	      the_exit_code_value = 0x16;
	      */
	  }
	  if ((strcmp(argv[i], "AIX4.1") == 0) || (strcmp(argv[i], "AIX4.1.4") == 0)) {
	    /*
	      the_exit_code_value = 0x17;
	      */
	  }
	  else if (strcmp(argv[i], "AIX4.2") == 0) {
	    /*
	      the_exit_code_value = 0x19;
	      */
	  }
	  else if ((strcmp(argv[i], "AIX3.2") == 0) || (strcmp(argv[i], "AIX3.2.5") == 0)) {
	    /*
	      the_exit_code_value = 0x0d;
	      */
	  }
	  else {
	    fprintf(mystderr, "Illegal flag type in read_input_flags of Aria\n");
	  }
	}
	else {
	  fprintf(mystderr, "Need a flag value for the -oslevel switch...\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-arch") == 0) {
	i++;
	if (i < argc) {
	  if ( (strcmp(argv[i], "POWER") == 0) || (strcmp(argv[i], "PPC") == 0) ) {
	    ; /* Do nothing; this flag is now set automatically */
	  }
	  else {
	    fprintf(mystderr, "Illegal flag type in read_input_flags of Aria:\n");
	    fprintf(mystderr, "   -arch flag cannot have %s flag\n", argv[i]);
	  }
	}
	else {
	  fprintf(mystderr, "Need a flag value for the -arch switch...\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-uarch") == 0) {
       i++;
        if (i < argc) {
          if (strcmp(argv[i], "POWER") == 0) {
            the_user_program_architecture = ARIA_POWER_ISA;
          }
          else if (strcmp(argv[i], "PPC") == 0) {
            the_user_program_architecture = ARIA_PPC_ISA;
          }
          else {
            fprintf(mystderr, "Illegal flag type in read_input_flags of Aria:\n");
            fprintf(mystderr, "   -uarch flag cannot have %s flag\n", argv[i]);
          }
        }
        else {
          fprintf(mystderr, "Need a flag value for the -arch switch...\n");
          ret_val = 1;
          goto exit_this_while_loop;
        }
      }

      else {
        fprintf(mystderr, "Unrecognized switch in Aria read_input_flags: '%s'\n", argv[i]);
	ret_val = 1;
	goto exit_this_while_loop;
      }
      break;

    default:
      fprintf(mystderr, "Illegal flag type in read_input_flags of Aria\n");
      fprintf(mystderr, "   the flag is: '%s'\n", argv[i]);
      ret_val = 1;
      break;
    }
    i++;
  }

exit_this_while_loop:

  if (model_argc_count == 0) {
    fprintf(mystderr, "NOTE: Did not find a --MODEL entry and model name...\n");
    ret_val = 1;
  }

  if (u_argc_count == 0) {
    fprintf(mystderr, "NOTE: Did not find a --PROGRAM entry and user program name...\n");
    ret_val = 1;
  }

  *model_argc = model_argc_count;
  *u_argc  = u_argc_count;

  return ret_val;
}



