/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... do_aria_setup, do_aria_execution, 
                    dump_aria_run_statistics

  FUNCTION ........ Provides a set of interface routines to aria, 
     comprising the external user interface.

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 
  CALLED-ROUTINES.. 
  CALLED-BY........ None
  INCLUDED-BY...... 
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

#include <sys/time.h>
#include <limits.h>

#include "aix_a.out.h"
#include "aix_errno.h"
#include "aix_ldr.h"
#include "aix_mman.h"
#include "posix_fcntl.h"
#include "ansi_stdio.h"
#include "ansi_stdlib.h"

#include "ansi_assert.h"

#include "set141516.h"

#include "aria_debug.h"

#include "aria_state.h"

#include "io_msgs.h"
#include "opc.h"
#include "proc_state.h"
#include "hashing.h"
#include "do_translation.h"
#include "do_svc_translation.h"
#include "save_user_state.h"

#include "globdefs.h"
#include "globtypes.h"

#include "global_variables.h"

#include "interface_globals.h"

/** Forward function declarations **/

#include "aria_interface.h"
#include "aria_per_instr_interface.h"
#include "get_null_sc_timing.h"

/*******************************************************************
  PROGRAM NAME..... do_aria_setup, 

  FUNCTION ........ Takes a pointer to a user process state structure
     and fills in that structure with the initial user process state.

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... returns the starting address of the user program
  CALLED-ROUTINES.. insert_user_proc_state_save_code, 
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

unsigned * 
do_aria_setup_with_blocksize_limit(proc_state_type * saved_proc_state_ptr, 
				   int max_block_size_override
				   )
{
  unsigned * next_bb_address = prog_start;
  unsigned * next_xlated_bb_address = next_xlate_address;
  unsigned * temp_ptr;
  unsigned i;

  IFDEBUG(warn("[aria_setup] In do_aria_setup...\n"););

  if (max_block_size_override > 0) {
    IFDEBUG(warn("driver requested max block size limit of %d\n", max_block_size_override); );
    if (max_block_size_override < global_max_block_size_limit) {
      global_max_block_size_limit = max_block_size_override;
      warn("The current max size (%d) is being reduced to %d\n", global_max_block_size_limit, max_block_size_override);
    }
    IFDEBUG(else { warn("The current max size (%d) is smaller -- retained\n", global_max_block_size_limit); });
  }

  /** Reset the per-instruction type interface table **/
  for (i = 0; i < OPC_NUM+1; i++) {
    per_instruction_action_time(i)          = DO_NO_ARIA_ACTION;
    per_instruction_action_type(i)          = 0;
    per_instruction_before_function_ptr(i)  = 0;
    per_instruction_after_function_ptr(i)   = 0;
  }

  /** Reset the processor saved state area **/
  for (i = 0; i < 16; i++) {
    proc_state_valid_segment(saved_proc_state_ptr,  0, i) = 0; 
    proc_state_valid_segment(saved_proc_state_ptr,  1, i) = 0; 
    proc_state_valid_segment(saved_proc_state_ptr,  2, i) = 1; 
    proc_state_valid_segment(saved_proc_state_ptr,  3, i) = 0; 
    proc_state_valid_segment(saved_proc_state_ptr,  4, i) = 0; 
    proc_state_valid_segment(saved_proc_state_ptr,  5, i) = 0; 
    proc_state_valid_segment(saved_proc_state_ptr,  6, i) = 0; 
    proc_state_valid_segment(saved_proc_state_ptr,  7, i) = 0; 
    proc_state_valid_segment(saved_proc_state_ptr,  8, i) = 0; 
    proc_state_valid_segment(saved_proc_state_ptr,  9, i) = 0; 
    proc_state_valid_segment(saved_proc_state_ptr, 10, i) = 0;
    proc_state_valid_segment(saved_proc_state_ptr, 11, i) = 0; 
    proc_state_valid_segment(saved_proc_state_ptr, 12, i) = 1; 
    proc_state_valid_segment(saved_proc_state_ptr, 13, i) = 0; 
    proc_state_valid_segment(saved_proc_state_ptr, 14, i) = 0; 
    proc_state_valid_segment(saved_proc_state_ptr, 15, i) = 0; 
  }
  proc_state_valid_segment(saved_proc_state_ptr,  2, 15) = 0; 
  proc_state_valid_segment(saved_proc_state_ptr,  12, 15) = 0; 

  proc_state_LR    (saved_proc_state_ptr)  = 0x4;
  proc_state_CTR   (saved_proc_state_ptr)  = 0x5;
  proc_state_CR    (saved_proc_state_ptr)  = 0x6;
  proc_state_XER   (saved_proc_state_ptr)  = 0x7;
  proc_state_TARGET(saved_proc_state_ptr)  = 0x8;

  /** Use the predefined user stack address **/
  pc_func_desc(prog_desc) = next_xlate_address;

  /** Save the global user iaddress offset for this process **/
  proc_state_iaddr_offset(saved_proc_state_ptr) = global_user_iaddr_offset;

  IFDEBUG(warn("[aria_setup]  Calling the insert_user_proc_state_save_code routine...\n"); );
  /**
    next_xlate_address = insert_user_proc_state_save_code(saved_proc_state_ptr, (unsigned *)USER_PROCESS_STACK_ADDRESS);
    **/  
  next_xlate_address = insert_user_proc_state_save_code(saved_proc_state_ptr, user_stack_address_value);
  
  /** And then execute that block... **/
  temp_ptr = next_bb_address;
  
  IFDEBUG(warn("[aria_setup]  Back from insert_user_proc_state_save_code routine... executing it\n"); );
  /** The copy to run when debugging output desired **/
  if (do_no_output_during_runs == 0) {
    if ( (output_after_dynamic_bb_execution <= global_bb_executions) && 
	 (no_output_after_dynamic_bb_execution >= global_bb_executions) && 
	 (output_after_static_bb_translation <= global_bb_index) &&
	 (no_output_after_static_bb_translation >= global_bb_index) ) {
      warn_exec("\nExecuting the translated code:\n");
      warn_exec("     Starting address %016LLx:\n",   pc_func_desc(prog_desc));
      warn_exec("     TOC address      %016LLx:\n",   toc_func_desc(prog_desc));
      warn_exec("     Other data       %016LLx:\n", other_func_desc(prog_desc));
      warn_processor_state(saved_proc_state_ptr, show_proc_float_state);
    }
#ifdef DO_64_BIT_COMPILE
    set1415(user_argc, user_argv, user_envp);
#endif
    next_bb_address = (unsigned *)prog_ptr(user_argc, user_argv, user_envp, saved_proc_state_ptr);  
    if ( (output_after_dynamic_bb_execution <= global_bb_executions) && 
	 (no_output_after_dynamic_bb_execution >= global_bb_executions) && 
	 (output_after_static_bb_translation <= global_bb_index) &&
	 (no_output_after_static_bb_translation >= global_bb_index) ) {
      warn_exec("  ... Done with initial save of user state...next address = %08x\n", next_bb_address);
      warn_trace("Executed initial user state save: Xa = %08x Next = %08x\n", 
		 pc_func_desc(prog_desc), next_bb_address);
      /*
	warn_processor_state(saved_proc_state_ptr, show_proc_float_state);
	*/
      warn_trace("\nStarting dynamic translation\n\n");
    }
  }
  /** The faster copy to run when no debugging output desired **/
  else {
#ifdef DO_64_BIT_COMPILE
    set1415(user_argc, user_argv, user_envp);
#endif
    next_bb_address = (unsigned *)prog_ptr(user_argc, user_argv, user_envp, saved_proc_state_ptr);  
  }

  IFDEBUG(warn("[aria_setup]  Back from inserted code routine..."); );

  if (global_do_old_initial_proc_state_code == 0) {
    int kk;
    /**
      warn("[aria_setup] Doing the new initial state setup code...\n");
      warn("the global flags space is 0x%08x = %d bytes\n", sizeof(global_flags_type), sizeof(global_flags_type));
      **/
    proc_state_GPR(saved_proc_state_ptr, 0) = 0xdeadbeef;
    proc_state_GPR(saved_proc_state_ptr, 1) = (unsigned) user_stack_address_value;
    proc_state_GPR(saved_proc_state_ptr, 2) = (unsigned) toc_func_desc(prog_desc);
    proc_state_GPR(saved_proc_state_ptr, 3) = (unsigned) user_argc;
    proc_state_GPR(saved_proc_state_ptr, 4) = (unsigned) user_argv;
    proc_state_GPR(saved_proc_state_ptr, 5) = (unsigned) user_envp;
    for (kk = 6; kk < 14; kk++) {
      proc_state_GPR(saved_proc_state_ptr, kk) = 0xdeadbeef;
    }
    for (kk = 16; kk < 32; kk++) {
      proc_state_GPR(saved_proc_state_ptr, kk) = 0xdeadbeef;
    }
    for (kk = 0; kk < 32; kk++) {
      proc_state_FPR(saved_proc_state_ptr, kk) = 0x0;
    }
    proc_state_CR(saved_proc_state_ptr)  = (unsigned) 0;
    proc_state_CTR(saved_proc_state_ptr) = (unsigned) 0;
  }

  /** Determine the overhead of our timing measurements **/
  if (global_get_system_call_execution_times == 1) {
    proc_state_type * dummy_proc_state_ptr = aria_get_new_process_state_space();
    unsigned * saved_next_xlate_address = next_xlate_address;
    aria_copy_process_state_from_to(saved_proc_state_ptr, dummy_proc_state_ptr);
    get_null_sc_timing_overhead(dummy_proc_state_ptr);
    next_xlate_address = saved_next_xlate_address;
  }

  next_bb_address = prog_start;
  global_xlated_instructions += (next_xlate_address - next_xlated_bb_address);

  if ( (do_iaddr_fixup) && (((unsigned)next_bb_address & 0xf0000000) == global_user_program_iaddr_mask) ) {
    return (unsigned *) ((unsigned)next_bb_address - global_user_iaddr_offset);
  }
  else {
    return  next_bb_address;
  }
}
