/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... bb_epilog_code.c
  FUNCTION ........ A C code include that contains the code to 
    save off new user process state.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... canned_multiple_code.c, canned_string_code.c,
      do_svc_translation.c, do_translation.c

  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
  #define TEMP_USER_SPOT -212

  int  found_base_register = 0;
  int  fscratch_reg   = 11;
  int  i;

  /** First, determine a scratch_reg register to use: */
  for (i = 3; i < 32; i++)
    {
      if ((gpr_defs & (1 << i)) == 0) {
	base_register = i;
	found_base_register = 1;
	break;
      }
    }

  if (!found_base_register) {
    base_register = 12;
    /* Temporarily store the value of GPR 12 */
    *next_xlate_address = STORE_REG  (base_register, TEMP_USER_SPOT, STACK_PTR);        next_xlate_address++;
  }
  *next_xlate_address = OPC_ASM_ADDIS(base_register,  0, ARIA_STATE_SAVE_HI_16);          next_xlate_address++;
  *next_xlate_address = LOAD_REG  (base_register,  PROC_STATE_OFFSET, base_register);  next_xlate_address++;    
  
  /** Loop through the set of other GPRs and see if we need to store them **/
  for (i = 0; i < 32; i++) {
    if (gpr_defs & (1 << i)) {
      *next_xlate_address = STORE_REG( i, GPR_SAVE_OFFSET+ARIA_GPR_SIZE*i,  base_register); next_xlate_address++;
    }
  }
  
  if (scratch_reg == base_register) { 
    scratch_reg--;
  }

  if (!found_base_register) {
    /* Recover the temp stored value of user GPR12 and store in proper location */
    *next_xlate_address = LOAD_REG  (scratch_reg, TEMP_USER_SPOT, STACK_PTR);  next_xlate_address++;
    *next_xlate_address = STORE_REG  (scratch_reg, GPR_SAVE_OFFSET+ARIA_GPR_SIZE*base_register, base_register); next_xlate_address++;
  }

  if ( (spr_uses & OPC_XER_DEF_USE_MASK) 
       || (spr_defs & OPC_XER_DEF_USE_MASK) ) {
    /** Save the new (user) value of the XER **/
    *next_xlate_address = OPC_ASM_MFSPR(scratch_reg,  1);         next_xlate_address++; 
    *next_xlate_address = STORE_REG  (scratch_reg, XER_SAVE_OFFSET, base_register);    next_xlate_address++;
  }

  if (spr_defs & OPC_LR_DEF_USE_MASK) {
    /** Save the new (user) value of the LR **/
    *next_xlate_address = OPC_ASM_MFSPR(scratch_reg,  8);         next_xlate_address++; 
    *next_xlate_address = STORE_REG  (scratch_reg, LR_SAVE_OFFSET, base_register);    next_xlate_address++;
  }

  if (spr_defs & OPC_CTR_DEF_USE_MASK) {
    /** Save the new (user) value of the CTR **/
    *next_xlate_address = OPC_ASM_MFSPR(scratch_reg,  9);         next_xlate_address++; 
    *next_xlate_address = STORE_REG  (scratch_reg, CTR_SAVE_OFFSET, base_register);    next_xlate_address++;
  }

  if (spr_defs & OPC_MQ_DEF_USE_MASK) {
    if ((the_machine_isa_type == OPC_POWER_ISA) || (the_machine_isa_type == OPC_POWER2_ISA)) {
      /** Save the new (user) value of the MQ register **/
      *next_xlate_address = OPC_ASM_MFSPR(scratch_reg,  0);         next_xlate_address++; 
      *next_xlate_address = STORE_REG  (scratch_reg, MQR_SAVE_OFFSET, base_register);    next_xlate_address++;
    }
  }

  if (cr_touches) {
    /** Save the new (user) value of the condition register (fields) **/
    *next_xlate_address = OPC_ASM_MFCR (scratch_reg);                 next_xlate_address++; 
    *next_xlate_address = STORE_REG  (scratch_reg, CR_SAVE_OFFSET , base_register);    next_xlate_address++;
  }

  /** Loop through the set of FPRs and see if we need to store them **/
  for (i = 0; i < 32; i++) {
    if (fpr_defs & (1 << i)) {
      *next_xlate_address = OPC_ASM_STFD ( i, FPR_SAVE_OFFSET+ARIA_FPR_SIZE*i,  base_register);  next_xlate_address++;
    }
  }

  /** Save the (user) value of the FPSCR **/
  if (spr_defs & OPC_FPSCR_DEF_USE_MASK) {
    *next_xlate_address = OPC_ASM_MFFS (fscratch_reg);                    next_xlate_address++; 
    *next_xlate_address = OPC_ASM_STFD (fscratch_reg, FPSCR_64BIT_OFFSET, base_register);  next_xlate_address++;
  }
  
