/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... bb_epilog_code.c
  FUNCTION ........ A C code include that contains the code to 
    restore the saved user process state (prior to executing 
    the translated code for a user basic block body).

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... canned_multiple_code.c, canned_string_code.c,
      do_svc_translation.c, do_translation.c

  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
  int i;
  unsigned fscratch_reg = 11;

  if (scratch_reg == base_register) {
    scratch_reg--;
  }

  if ( (spr_uses & OPC_XER_DEF_USE_MASK) 
       || (spr_defs & OPC_XER_DEF_USE_MASK) ) {
    /** Restore the saved (user) value of the XER **/
    *next_xlate_address = LOAD_REG  (scratch_reg, XER_SAVE_OFFSET, base_register);     next_xlate_address++;
    *next_xlate_address = OPC_ASM_MTSPR( 1, scratch_reg);                    next_xlate_address++; 
  }

  if (spr_uses & OPC_LR_DEF_USE_MASK) {
    /** Restore the saved (user) value of the LR **/
    *next_xlate_address = LOAD_REG  (scratch_reg, LR_SAVE_OFFSET, base_register);      next_xlate_address++;
    *next_xlate_address = OPC_ASM_MTSPR( 8, scratch_reg);                    next_xlate_address++; 
  }

  if (spr_uses & OPC_CTR_DEF_USE_MASK) {
    /** Restore the saved (user) value of the CTR **/
    *next_xlate_address = LOAD_REG  (scratch_reg, CTR_SAVE_OFFSET, base_register);     next_xlate_address++;
    *next_xlate_address = OPC_ASM_MTSPR( 9, scratch_reg);                                 next_xlate_address++; 
  }

  if (spr_uses & OPC_MQ_DEF_USE_MASK) {
    if ((the_machine_isa_type == OPC_POWER_ISA) || (the_machine_isa_type == OPC_POWER2_ISA)) {
      /** Save the new (user) value of the MQ register **/
      *next_xlate_address = LOAD_REG  (scratch_reg, MQR_SAVE_OFFSET, base_register);     next_xlate_address++;
      *next_xlate_address = OPC_ASM_MTSPR( 0, scratch_reg);                                 next_xlate_address++; 
    }
  }

  if (cr_touches) {
    /** Restore the saved (user) value of the condition register (fields) **/
    *next_xlate_address = LOAD_REG  (scratch_reg, CR_SAVE_OFFSET , base_register);     next_xlate_address++;
    *next_xlate_address = OPC_ASM_MTCRF(0xff, scratch_reg);                               next_xlate_address++; 
  }

  /** Restore the saved (user) value of the FPSCR **/
  if (spr_uses & OPC_FPSCR_DEF_USE_MASK) {
    *next_xlate_address = OPC_ASM_LFD  (fscratch_reg, FPSCR_64BIT_OFFSET, base_register);  next_xlate_address++;
    *next_xlate_address = OPC_ASM_MTFSF(0xff, fscratch_reg);                               next_xlate_address++; 
  }
  
  /** Loop through the set of FPRs and see if we need to load them **/
  for (i = 0; i < 32; i++) {
    if (fpr_uses & (1 << i)) {
      *next_xlate_address = OPC_ASM_LFD  ( i, FPR_SAVE_OFFSET+ARIA_FPR_SIZE*i,  base_register);   next_xlate_address++;
    }
  }
  
  /***********************************************************/
  /** Load the required GPRS of the user's 32 GPR registers **/
  /***********************************************************/

  /** Loop through the set of other GPRs and see if we need to load them **/
  for (i = 0; i < 32; i++) {
    if ((gpr_uses & (1 << i)) && (i != base_register) ) { 
      *next_xlate_address = LOAD_REG  ( i, GPR_SAVE_OFFSET+ARIA_GPR_SIZE*i,  base_register);   next_xlate_address++;
    }
  }

  /* Now load over the base register (if it must be loaded) */
  i = base_register;
  if (gpr_uses & (1 << i)) {
    *next_xlate_address = LOAD_REG  ( i, GPR_SAVE_OFFSET+ARIA_GPR_SIZE*i,  base_register);     next_xlate_address++;
  }

