/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... insert_canned_multiple_code
  FUNCTION ........ Generates code to deal with a load or store 
     multiple instruction that touches too many registers, so 
     Aria cannot maintain free registers to use within the 
     user process state code.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... the next available translation address
  CALLED-ROUTINES.. None
  CALLED-BY........ do_one_translation
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... <see below> plus
      save_aria_state_code.c, bb_prolog_code.c, opc_idx_spec_xform.gen, 
      bb_epilog_code.c, restore_aria_state_code.c, xlation_alignment_code.c


  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
#include <stdio.h>

#include "aria_xlate_macros.h"

#include "opc.h"

#include "proc_state.h"
#include "aria_state.h"
#include "hashing.h"

#include "io_msgs.h"

#include "dump_description.h"

#include "globtypes.h"
#include "global_variables.h"

#include "display_reg_usedef.h"

#define TAKEN_XLATE      1
#define NOT_TAKEN_XLATE  2

/**********************************************************************/
/** THE FOLLOWING ARE THE 32-BIT MACROS FOR CANNED MULTI TRANSLATION **/
/**********************************************************************/
#ifndef DO_64_BIT_COMPILE

/* Store the dynamic trace buffer address (pointer) in our reserved work area */
#define INSERT_DYNAMIC_TRACE_CODE(x) \
      unsigned static_hi = ((unsigned)microtrace_buffer_address & 0xffff0000) >> 16; \
      unsigned static_lo = ((unsigned)microtrace_buffer_address & 0x0000ffff);       \
      *next_xlate_address = OPC_ASM_ADDIS((x), 0, static_hi);  next_xlate_address++;  \
      *next_xlate_address = OPC_ASM_ORI  ((x), (x), static_lo);  next_xlate_address++;  \
      *next_xlate_address = STORE_REG ((x), PROC_DYN_TRBUF_OFFSET, base_register);  next_xlate_address++;

/** Set the fall-through next instruction address **/
#define SET_FALLTHROUGH_PATH \
  {\
    unsigned fall_thru_path = (unsigned)(original_bb_start_address+1); \
    unsigned hi_addr = fall_thru_path >> 16;                           \
    unsigned lo_addr = fall_thru_path & 0x0000ffff;                    \
    *next_xlate_address = OPC_ASM_ADDIS(scratch_reg,  0, hi_addr);          next_xlate_address++; \
    *next_xlate_address = OPC_ASM_ORI  (scratch_reg, scratch_reg, lo_addr); next_xlate_address++; \
  }

/**********************************************************************/
/** THE FOLLOWING ARE THE 64-BIT MACROS FOR CANNED MULTI TRANSLATION **/
/**********************************************************************/
#else

/* Store the dynamic trace buffer address (pointer) in our reserved work area */
#define INSERT_DYNAMIC_TRACE_CODE(x) \
      unsigned addr_0_15  = ((BASE_TYPE)microtrace_buffer_address >> 48) & 0xffffLL; \
      unsigned addr_16_31 = ((BASE_TYPE)microtrace_buffer_address >> 32) & 0xffffLL; \
      unsigned addr_32_47 = ((BASE_TYPE)microtrace_buffer_address >> 16) & 0xffffLL; \
      unsigned addr_48_63 = ((BASE_TYPE)microtrace_buffer_address      ) & 0xffffLL; \
      *next_xlate_address = OPC_ASM_ADDI  ((x), 0, addr_0_15);                           next_xlate_address++; \
      *next_xlate_address = OPC_ASM_RLDICL((x), (x), 16, 0);                               next_xlate_address++; \
      *next_xlate_address = OPC_ASM_ORI   ((x), (x), addr_16_31);                         next_xlate_address++; \
      *next_xlate_address = OPC_ASM_RLDICL((x), (x), 16, 0);                               next_xlate_address++; \
      *next_xlate_address = OPC_ASM_ORI   ((x), (x), addr_32_47);                         next_xlate_address++; \
      *next_xlate_address = OPC_ASM_RLDICL((x), (x), 16, 0);                               next_xlate_address++; \
      *next_xlate_address = OPC_ASM_ORI   ((x), (x), addr_48_63);                         next_xlate_address++; \
      *next_xlate_address = OPC_ASM_STD   ((x), PROC_DYN_TRBUF_OFFSET, base_register);   next_xlate_address++;

/** Set the fall-through next instruction address **/
#define SET_FALLTHROUGH_PATH \
  { \
    BASE_TYPE fall_thru_path = (BASE_TYPE)(original_bb_start_address+1); \
    unsigned addr_0_15  = (fall_thru_path >> 48) & 0xffffLL;             \
    unsigned addr_16_31 = (fall_thru_path >> 32) & 0xffffLL;             \
    unsigned addr_32_47 = (fall_thru_path >> 16) & 0xffffLL;             \
    unsigned addr_48_63 = (fall_thru_path      ) & 0xffffLL;             \
    *next_xlate_address = OPC_ASM_ADDI  (scratch_reg, 0, addr_0_15);              next_xlate_address++; \
    *next_xlate_address = OPC_ASM_RLDICL(scratch_reg, scratch_reg, 16, 0);        next_xlate_address++; \
    *next_xlate_address = OPC_ASM_ORI   (scratch_reg, scratch_reg, addr_16_31);   next_xlate_address++; \
    *next_xlate_address = OPC_ASM_RLDICL(scratch_reg, scratch_reg, 16, 0);        next_xlate_address++; \
    *next_xlate_address = OPC_ASM_ORI   (scratch_reg, scratch_reg, addr_32_47);   next_xlate_address++; \
    *next_xlate_address = OPC_ASM_RLDICL(scratch_reg, scratch_reg, 16, 0);        next_xlate_address++; \
    *next_xlate_address = OPC_ASM_ORI   (scratch_reg, scratch_reg, addr_48_63);   next_xlate_address++; \
  }

#endif


/***********************************************************/
/** THE SUBROUTINE TO INSERT THE CANNED MULTI TRANSLATION **/
/***********************************************************/

unsigned * 
insert_canned_multiple_code(use_def_data_type instr_info[], 
			    instruction_xlate_type instr_xlate_info[],
			    unsigned * original_bb_start_address,
			    unsigned * microtrace_buffer_address,
			    hash_entry * the_hash_entry_ptr
			    )
{
  int i;
  unsigned base_register = 12;
  unsigned scratch_reg   = 11;
  unsigned trbf_base_reg      = 10;

  unsigned original_base_register;
  unsigned original_scratch_reg;
  unsigned original_trbf_base_reg;

  unsigned * translated_bb_start_address = next_xlate_address;
  unsigned * not_taken_translated_bb_start_address = 0;

  unsigned total_instr_in_xl_bb = 0;
  unsigned instr_in_taken_xl_bb = 0;

  unsigned translation_type;
  unsigned * temp_ptr = next_xlate_address;

  unsigned * br_posn;
  unsigned * br_posn_2;

  unsigned iword = *original_bb_start_address;
  unsigned rT   = OPC_DEC_RT(iword);
  unsigned rA   = OPC_DEC_RA(iword);
  int      Dval = OPC_DEC_D(iword);

  unsigned instr_num = 0;


  /** Need to find 3 unused registers in [1-31] **/
  {
    unsigned index = 0;
    unsigned regs_to_use[3];
    for (i = 3; i <= 31; i++) {
      if ((i != 3) && (rA != i) && (rT != i)) {
	regs_to_use[index] = i;
	index++;
      }
      if (index == 3) {
	break;
      }
    }
    original_base_register = regs_to_use[0];
    original_scratch_reg   = regs_to_use[1];
    original_trbf_base_reg      = regs_to_use[2];
  }


  /** Enter a header for this BB translation **/
  {
    unsigned total_instr_in_bb = 1;
    #include "add_xlation_header_code.c"
  }
  
  /************************************************/
  /** Start the translations with the taken path **/
  translation_type = TAKEN_XLATE;   
  translated_bb_start_address = next_xlate_address;

insert_the_translated_code:
  /* Set up some temporary values for base_register and scratch_reg */
  /* to use only while saving off the Aria entry state              */
  base_register = 12;
  scratch_reg   = 11;
  trbf_base_reg      = original_trbf_base_reg;

  /* Load GPRbase with the address of our ARIA state saving space */          
  *next_xlate_address = OPC_ASM_ADDIS(base_register,  0, ARIA_STATE_SAVE_HI_16);   next_xlate_address++; 

  /* Store the processor state save space address in our reserved work area */
  *next_xlate_address = STORE_REG (  3,  PROC_STATE_OFFSET,  base_register);     next_xlate_address++;

  /* Store the dynamic trace buffer address (pointer) in our reserved work area */
  if ( (do_dynamic_trace_generation) ) {
    INSERT_DYNAMIC_TRACE_CODE(4);
  }

  /********************************************************************************************/
  /** Insert the code to save the Aria state (that may be altered by user program execution) **/
  /********************************************************************************************/
  {
    unsigned gprs = instr_info[0].touch_vectors[0];
    unsigned fprs = instr_info[0].touch_vectors[1];
    unsigned crs  = instr_info[0].touch_vectors[2];

    #include "save_aria_state_code.c"
  }

  if (do_dynamic_trace_generation) {
    /** Load the dynamic trace pointer **/
    *next_xlate_address = LOAD_REG   ( trbf_base_reg, PROC_DYN_TRBUF_OFFSET, base_register);  next_xlate_address++;
  }

  /* Recover the settings for the selected base, scratch and dynamic trace buffer ptrs */
  base_register = original_base_register;
  scratch_reg   = original_scratch_reg;

  /* Copy the GPR3 (user process state ptr) value to the base register */
  if (base_register != 3) {
    /* Copy GPR 3 value to the base register */
    *next_xlate_address = OPC_ASM_OR(base_register,  3,  3);  next_xlate_address++;
  }
  
  if (do_dynamic_trace_generation) {
    /** Determine the loaded address **/
    if (rA != 0) {
      *next_xlate_address = LOAD_REG  ( scratch_reg, GPR_SAVE_OFFSET+ARIA_GPR_SIZE*rA, base_register);   next_xlate_address++;
      *next_xlate_address = OPC_ASM_ADDI( scratch_reg, scratch_reg, Dval);                     next_xlate_address++;
    }
    else {
      *next_xlate_address = OPC_ASM_ADDI (scratch_reg, 0, Dval);    next_xlate_address++;
    }
    /** Store the address accessed **/
    *next_xlate_address = STORE_REG ( scratch_reg, 12, trbf_base_reg);   next_xlate_address++;
  }

  /**************************************************************************/
  /* Insert the restoration of user process state code for this basic block */
  /**************************************************************************/
  {
    unsigned gpr_uses   = instr_info[0].use_vectors[0];
    unsigned fpr_uses   = instr_info[0].use_vectors[1];
    unsigned cr_touches = instr_info[0].touch_vectors[2];
    unsigned spr_uses   = instr_info[0].use_vectors[3];
    unsigned spr_defs   = instr_info[0].def_vectors[3];
      
    #include "bb_prolog_code.c"
  }
  
  /** Output the prolog (if appropriate) **/
  if (do_no_output_during_runs == 0) {
    warn_xlate("\nThe taken-path-execution (non-speculative) code is:\n");
    warn_xlate("* (prolog) ----------------------------------------------------------------\n");
    DUMP_DESCRIPTION(temp_ptr, next_xlate_address);    
    warn_xlate("* (translated body) -------------------------------------------------------\n");
    temp_ptr = next_xlate_address;
  }

  /*******************************************************************************/
  /** Insert an instruction (stream) for the multiple memory access instruction **/
  /*******************************************************************************/
  {
    unsigned CR_must_be_saved = 0;

    /* Insert the instruction-index-dependent code required to generate a translation */
    /*  Note that this includes:                                                   */
    /*    setting the data access length in the trace buffer (for memory accesses) */
    /*    inserting any dynamic trace generation code required by the instruction  */
    /*    inserting either the original iword or a not-taken path code sequence    */
    #ifdef DO_64_BIT_COMPILE
      #include "opc_idx_canned_multiple_code_64.gen" 
    #else
      #include "opc_idx_canned_multiple_code.gen" 
    #endif

    if (do_no_output_during_runs == 0) {
      DUMP_FAKE_DESCRIPTION(temp_ptr, next_xlate_address);
      temp_ptr = next_xlate_address;
    }      
  }

  /** Now, determine a new base_register register to use: */
  for (i = 3; i < 32; i++)
    {
      if ((instr_info[0].def_vectors[0] & (1 << i)) == 0) {
	base_register = i;
	break;
      }
    }

  /** Now set up a pointer to the dynamic trace buffer **/
  scratch_reg = 12;

  if (scratch_reg == base_register) {
    scratch_reg--;
  }

  /*****************************************************************************/
  /** Insert the bb epilog, which inserts instructions to save new user state **/
  /*****************************************************************************/
  {
    unsigned gpr_defs   = instr_info[0].def_vectors[0];
    unsigned fpr_defs   = instr_info[0].def_vectors[1];
    unsigned cr_touches = instr_info[0].touch_vectors[2];
    unsigned spr_uses   = instr_info[0].use_vectors[3];
    unsigned spr_defs   = instr_info[0].def_vectors[3];
    
    #include "bb_epilog_code.c"
  }

  /** Set the value of the Aria base register */
  base_register = 4;
  *next_xlate_address = OPC_ASM_ADDIS(base_register,  0, ARIA_STATE_SAVE_HI_16);    next_xlate_address++;    

  if ( (do_no_output_during_runs == 0) ) {
    warn_xlate("* (epilog) ----------------------------------------------------------------\n", next_xlate_address);
    DUMP_DESCRIPTION(temp_ptr, next_xlate_address);
    temp_ptr = next_xlate_address;
  }

  /********************************/
  /** Now restore the Aria state **/
  /********************************/
  {
    unsigned gprs = instr_info[0].touch_vectors[0];
    unsigned fprs = instr_info[0].touch_vectors[1];
    unsigned crs  = instr_info[0].touch_vectors[2];
    
    #include "restore_aria_state_code.c"
  }

  /** Set the fall-through next instruction address **/
  SET_FALLTHROUGH_PATH;

  *next_xlate_address = STORE_REG  (scratch_reg, NEXT_TARGET_OFFSET, base_register);    next_xlate_address++;

  /** Set the next not_taken address return value **/
  if (not_taken_address_required_flag == 1) {
    *next_xlate_address = STORE_REG(scratch_reg, NOT_TAKEN_OFFSET, base_register);    next_xlate_address++;
  }  

  /** Set the next not_taken address return value **/
  if (not_taken_address_required_flag == 1) {
    /* Load GPRbase_register with the address of our ARIA state saving space    */               
    *next_xlate_address = OPC_ASM_ADDIS(base_register, 0, ARIA_STATE_SAVE_HI_16);        next_xlate_address++; 

    /* Load the address to which the not-taken value should be stored */
    *next_xlate_address = LOAD_REG   (base_register, NOT_TAKEN_RVAL_OFFSET, base_register);   next_xlate_address++; 
    *next_xlate_address = STORE_REG  (scratch_reg, 0, base_register);                   next_xlate_address++; 
  }
  
  /** Set the next address return value **/
  /**
   *next_xlate_address = LOAD_REG ( 3,  NEXT_TARGET_OFFSET, base_register);    next_xlate_address++;
   **/
  *next_xlate_address = OPC_ASM_ORI( 3, scratch_reg, 0);                         next_xlate_address++;

  /********************************************************************************************/
  /* Replace the branch in the original code with code to return through link register branch */
  /********************************************************************************************/
  *next_xlate_address = OPC_ASM_BCLR (0x14, 0);        next_xlate_address++;

  /* Determine the total instructions in the translated BB code */
  total_instr_in_xl_bb = (next_xlate_address - translated_bb_start_address);

  /* Output the translated basic block information exit code */
  if ( (do_no_output_during_runs == 0) ) {
    warn_xlate("* (restore) ----------------------------------------------------------------\n", next_xlate_address);
    DUMP_DESCRIPTION(temp_ptr, next_xlate_address);
    warn_xlate("===============================================================================\n");
    if (translation_type == TAKEN_XLATE) {
      warn_xlate("    end of BB %d [%u user, %u xlated instr] ; oa=%08x xa = %08x\n", 
		 global_bb_index+1, 1, total_instr_in_xl_bb, original_bb_start_address, translated_bb_start_address);
    }
    else {
      warn_xlate("    end of speculative BB %d [%u user, %u xlated instr] ; oa=%08x xa = %08x\n", 
		 global_bb_index+1, 1, total_instr_in_xl_bb, original_bb_start_address, not_taken_translated_bb_start_address);
    }
  }

  /***************************************************************/
  /* Update the addresses for the program and translation space **/
  /***************************************************************/
  #include "xlation_alignment_code.c"

  /***************************************************/
  /** If we are also including the not-taken path,  **/
  /** branch back to insert that code at this point **/
  /***************************************************/
  if ( (do_not_taken_path_translations) && (translation_type == TAKEN_XLATE) ) { 
    warn_xlate("\nThe speculative-execution (not-taken path) code is:\n");
    instr_in_taken_xl_bb = total_instr_in_xl_bb;
    total_instr_in_xl_bb = 0;
    translation_type = NOT_TAKEN_XLATE;
    not_taken_translated_bb_start_address = next_xlate_address;
    goto insert_the_translated_code;
  }

  /* Store the appropriate hash table information */
  HashTable_original_address(*the_hash_entry_ptr)   = original_bb_start_address;
  HashTable_xlation_address(*the_hash_entry_ptr)    = translated_bb_start_address;
  HashTable_NT_xlation_address(*the_hash_entry_ptr) = not_taken_translated_bb_start_address;
  HashTable_trace_buffer(*the_hash_entry_ptr)       = microtrace_buffer_address;
  HashTable_user_bb_size(*the_hash_entry_ptr)       = 1;
  HashTable_xlated_bb_size(*the_hash_entry_ptr)     = instr_in_taken_xl_bb;
  HashTable_NT_xlated_bb_size(*the_hash_entry_ptr)  = total_instr_in_xl_bb;
  
  return next_xlate_address;
}

