/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... compare_user_fF.c
  FUNCTION ........ A simple program that compares two fF traces 
     to determine whether they differ significantly

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... <see below>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

#include <stdio.h>
#include "opc.h"

main(int argc, 
     char * argv[]
     )
{
  FILE * ff_file1;
  FILE * ff_file2;

  char buf_1[512];
  char buf_2[512];

  unsigned i, j;

  int iaddr_diff;
  int daddr_diff[16][16];

  unsigned start_addr_1;
  unsigned start_addr_2;
  unsigned tr_instr_index = 0;

  unsigned eof_1;
  unsigned eof_2;

  unsigned iaddr_1;
  unsigned iaddr_2;

  if (argc < 3) {
    printf("Error in invocation; usage is:\n");
    printf("   %s <fF_1> <fF_2>\n", argv[0]);
    return -1;
  }
  
  ff_file1 = fopen(argv[1], "r");
  if (ff_file1 == NULL) {
    printf("Error in invocation: cannot open file %s\n", argv[1]);
  }
  ff_file2 = fopen(argv[2], "r");
  if (ff_file2 == NULL) {
    printf("Error in invocation: cannot open file %s\n", argv[2]);
  }

  /** Initialize the OPC stuff **/
  opc_init();

  /** initialize some local data structures **/
  for (i = 0; i < 16; i++) {
    for (j = 0; j < 16; j++) {
      daddr_diff[i][j] = 0xffffffff;
    }
  }

  
  if (fread(&iaddr_1, 4, 1, ff_file1) == 0) {
    printf("Error: file %s seems empty (can't read first word)\n", argv[1]);
  }
  if (fread(&iaddr_2, 4, 1, ff_file2) == 0) {
    printf("Error: file %s seems empty (can't read first word)\n", argv[2]);
  }
  
  iaddr_diff = iaddr_1 - iaddr_2;

  /** Now compare the trace bodies **/
  do {
    unsigned word_1;
    unsigned word_2;
    eof_1 = fread(&word_1, 4, 1, ff_file1);
    eof_2 = fread(&word_2, 4, 1, ff_file2);

    tr_instr_index++;

    if (eof_1 != eof_2) {
      if (eof_1 == 0) {
	printf("Error: instruction %u file %s has prematurely exited\n", tr_instr_index, argv[1]);
      }
      if (eof_2 == 0) {
	printf("Error: instruction %u file %s has prematurely exited\n", tr_instr_index, argv[2]);
      }
      return -1;
    }

    if (word_1 != word_2) {
      printf("Error: instruction %u file %s has word 0x%08x versus 0x%08x in %s\n", 
	     tr_instr_index, argv[1], word_1, word_2, argv[2]);
      printf("*** Aboritng the comparison at this point...\n");
      return -1;
    }      
    
    {
      opc_idx  w1_index = opc_index(word_1);
      opc_idx  w2_index = opc_index(word_2);
      unsigned w1_itype = opc_itype(w1_index);
      unsigned w2_itype = opc_itype(w2_index);

      unsigned daddr_1;
      unsigned daddr_2;
      unsigned dlen_1;
      unsigned dlen_2;

      if (w1_itype != w2_itype) {
	printf("Error: instruction %u file %s has itype 0x%08x versus 0x%08x in %s\n", tr_instr_index, argv[1], w1_itype, w2_itype, argv[2]);
      }      

      opc_dis(word_1, buf_1);
      opc_dis(word_2, buf_2);

      switch (w1_itype) {
      case OPC_OTHER_INSTR:
	#ifdef DEBUG
	printf("==> %08x 0x%08x              = %s\n vs %08x 0x%08x              = %s\n",
	       iaddr_1, word_1, buf_1, iaddr_2, word_2, buf_2);
	#endif
	break;

      case OPC_BRANCH_INSTR:
      case OPC_FLOW_INSTR:
	eof_1 = fread(&daddr_1, 4, 1, ff_file1);
	eof_2 = fread(&daddr_2, 4, 1, ff_file2);
	if (eof_1 != eof_2) {
	  if (eof_1 == 0) {
	    printf("Error: instruction %u file %s has exited (target_addr)\n", tr_instr_index, argv[1]);
	  }
	  if (eof_2 == 0) {
	    printf("Error: instruction %u file %s has exited (target_addr)\n", tr_instr_index, argv[2]);
	  }
	  return -1;
	}

	if ( (daddr_1 & 0xf0000000) == 0x20000000 ) {
	  if ((daddr_1 - iaddr_diff) != daddr_2) {
	    printf("Error: instruction %u target address misalign: 0x%08x (%08x) vs 0x%08x)\n", 
		   tr_instr_index, daddr_1, (daddr_1-iaddr_diff), daddr_2);
	  }
	}
	else {
	  if (daddr_1 != daddr_2) {
	    printf("Error: instruction %u target address misalign: 0x%08x (not user) vs 0x%08x)\n", 
		   tr_instr_index, daddr_1, daddr_2);
	  }
	}
	#ifdef DEBUG
	printf("==> %08x 0x%08x %08x     = %s\n vs %08x 0x%08x %08x     = %s\n",
	       iaddr_1, word_1, daddr_1, buf_1, iaddr_2, word_2, daddr_2, buf_2);
	#endif
	break;

      case OPC_LOAD_INSTR:
      case OPC_STORE_INSTR:
      case OPC_FLOAT_LOAD_INSTR:
      case OPC_FLOAT_STORE_INSTR: {
	unsigned segment_1;
	unsigned segment_2;
	eof_1 = fread(&daddr_1, 4, 1, ff_file1);
	eof_2 = fread(&daddr_2, 4, 1, ff_file2);
	if (eof_1 != eof_2) {
	  if (eof_1 == 0) {
	    printf("Error: instruction %u file %s has exited (L/S daddr)\n", tr_instr_index, argv[1]);
	  }
	  if (eof_2 == 0) {
	    printf("Error: instruction %u file %s has exited (L/S daddr)\n", tr_instr_index, argv[2]);
	  }
	  return -1;
	}
	segment_1 = (daddr_1 & 0xf0000000) >> 28;
	segment_2 = (daddr_2 & 0xf0000000) >> 28;
	if ((daddr_diff[segment_1][segment_2]) == 0xffffffff) {
	  daddr_diff[segment_1][segment_2] = daddr_1 - daddr_2;
	}
	if ((daddr_1 - daddr_diff[segment_1][segment_2]) != daddr_2) {
	  printf("Error: instruction %u data address misalign: 0x%08x (+ 0x%08x = 0x%08x) vs 0x%08x)\n", 
		 tr_instr_index, daddr_1, daddr_diff[segment_1][segment_2],
		 (daddr_1-daddr_diff[segment_1][segment_2]), daddr_2);
	}

	#ifdef DEBUG
	printf("==> %08x 0x%08x %08x     = %s\n vs %08x 0x%08x %08x     = %s\n",
	       iaddr_1, word_1, daddr_1, buf_1, iaddr_2, word_2, daddr_2, buf_2);
	#endif
	break;
      }
      case OPC_MULTI_LOAD_INSTR:
      case OPC_MULTI_STORE_INSTR: {
	unsigned segment_1;
	unsigned segment_2;
	eof_1 = fread(&daddr_1, 4, 1, ff_file1);
	eof_2 = fread(&daddr_2, 4, 1, ff_file2);
	if (eof_1 != eof_2) {
	  if (eof_1 == 0) {
	    printf("Error: instruction %u file %s has exited (M daddr)\n", tr_instr_index, argv[1]);
	  }
	  if (eof_2 == 0) {
	    printf("Error: instruction %u file %s has exited (M daddr)\n", tr_instr_index, argv[2]);
	  }
	  return -1;
	}
	segment_2 = (daddr_2 & 0xf0000000) >> 28;
	if ((daddr_diff[segment_1][segment_2]) == 0xffffffff) {
	  daddr_diff[segment_1][segment_2] = daddr_1 - daddr_2;
	}
	if ((daddr_1 - daddr_diff[segment_1][segment_2]) != daddr_2) {
	  printf("Error: instruction %u M-data address misalign: 0x%08x (%08x) vs 0x%08x)\n", 
		 tr_instr_index, daddr_1, daddr_diff[segment_1][segment_2],
		 (daddr_1-daddr_diff[segment_1][segment_2]), daddr_2);
	}

	eof_1 = fread(&dlen_1, 4, 1, ff_file1);
	eof_2 = fread(&dlen_2, 4, 1, ff_file2);
	if (eof_1 != eof_2) {
	  if (eof_1 == 0) {
	    printf("Error: instruction %u file %s has exited (Dlength)\n", tr_instr_index, argv[1]);
	  }
	  if (eof_2 == 0) {
	    printf("Error: instruction %u file %s has exited (Dlength)\n", tr_instr_index, argv[2]);
	  }
	  return -1;
	}
	if (dlen_1 != dlen_2) {
	  printf("Error: instruction %u dlengths differ: 0x%08x vs 0x%08x)\n", 
		 tr_instr_index, dlen_1, dlen_2);
	}
	#ifdef DEBUG
	printf("==> %08x 0x%08x %08x %3u = %s\n vs %08x 0x%08x %08x %3u = %s\n",
	       iaddr_1, word_1, daddr_1, dlen_1, buf_1, iaddr_2, word_2, daddr_2, dlen_2, buf_2);
	#endif
	break;
      }

      case OPC_IS_ILLEGAL:
      default:
	printf("ERROR: instruction %u is illegal\n", tr_instr_index);
      }
    }
    
  } while (eof_1 != 0);
}
