/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... display_reg_usedef.c
  FUNCTION ........ Debug routines that format and display the 
      uses and defs information passed to them.  These routines were 
      implemented during Aria debugging.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 
  CALLED-ROUTINES.. 
  CALLED-BY........ None
  INCLUDED-BY...... 
  MACROS-USED...... None
  INCLUDES......... <see below>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

#include <stdio.h>
#include "display_reg_usedef.h"
#include "io_msgs.h"

/*******************************************************************
  PROGRAM NAME..... display_register_uses_and_defs
  FUNCTION ........ A routine that display the uses and defs 
     information passed to it, in a nice way.  This is most useful
     for debugging the translation process.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... <see below>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
void
display_register_uses_and_defs(unsigned gpr_uses,
			       unsigned fpr_uses,
			       unsigned cr_uses,
			       unsigned spr_uses,
			       unsigned gpr_defs,
			       unsigned fpr_defs,
			       unsigned cr_defs,
			       unsigned spr_defs,
			       unsigned use_vectors[]
			       )
{
  int rridx;
  warn_bb_regs("The total sets of registers touched are:\n");
  warn_bb_regs("                     USED                            SET               \n");
  warn_bb_regs(" GPR : ");
  for (rridx = 0; rridx < 32; rridx++) {
    if (gpr_uses & (1 << rridx)) { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }
  warn_bb_regs("     ");
  for (rridx = 0; rridx < 32; rridx++) {
    if (gpr_defs & (1 << rridx)) { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }
  warn_bb_regs("\n top : ");
  for (rridx = 0; rridx < 32; rridx++) {
    if (use_vectors[0] & (1 << rridx)) { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }
  warn_bb_regs(" tch ");
  for (rridx = 0; rridx < 32; rridx++) {
    if ((gpr_uses | gpr_defs) & (1 << rridx)) { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }

  warn_bb_regs("\n FPR : ");
  for (rridx = 0; rridx < 32; rridx++) {
    if (fpr_uses & (1 << rridx)) { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }
  warn_bb_regs("     ");
  for (rridx = 0; rridx < 32; rridx++) {
    if (fpr_defs & (1 << rridx)) { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }
  warn_bb_regs("\n top : ");
  for (rridx = 0; rridx < 32; rridx++) {
    if (use_vectors[1] & (1 << rridx)) { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }
  warn_bb_regs(" tch ");
  for (rridx = 0; rridx < 32; rridx++) {
    if ((fpr_uses | fpr_defs) & (1 << rridx)) { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }

  warn_bb_regs("\n  CR : ");
  for (rridx = 0; rridx < 32; rridx++) {
    if (cr_uses & (1 << rridx))  { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }
  warn_bb_regs("     ");
  for (rridx = 0; rridx < 32; rridx++) {
    if (cr_defs & (1 << rridx)) { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }
  warn_bb_regs("\n top : ");
  for (rridx = 0; rridx < 32; rridx++) {
    if (use_vectors[2] & (1 << rridx)) { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }
  warn_bb_regs(" tch ");
  for (rridx = 0; rridx < 32; rridx++) {
    if ((cr_uses | cr_defs) & (1 << rridx)) { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }

  warn_bb_regs("\n SPR : ");
  for (rridx = 0; rridx < 32; rridx++) {
    if (spr_uses & (1 << rridx)) { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }
  warn_bb_regs("     ");
  for (rridx = 0; rridx < 32; rridx++) {
    if (spr_defs & (1 << rridx)) { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }
  warn_bb_regs("\n top : ");
  for (rridx = 0; rridx < 32; rridx++) {
    if (use_vectors[3] & (1 << rridx)) { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }
  warn_bb_regs(" tch ");
  for (rridx = 0; rridx < 32; rridx++) {
    if ((spr_uses | spr_defs) & (1 << rridx)) { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }

  warn_bb_regs("\n\n");

}





/*******************************************************************
  PROGRAM NAME..... display_one_register_class_uses_and_defs
  FUNCTION ........ A routine that display the uses and defs 
     information passed to it, in a nice way, for just one 
     class of registers.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... <see below>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
void
display_one_register_class_uses_and_defs(unsigned reg_uses,
					 unsigned reg_defs
					 )
{
  int rridx;
  warn_bb_regs(" REG : ");
  for (rridx = 0; rridx < 32; rridx++) {
    if (reg_uses & (1 << rridx)) { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }
  warn_bb_regs("    ");
  for (rridx = 0; rridx < 32; rridx++) {
    if (reg_defs & (1 << rridx)) { warn_bb_regs("1"); }
    else { warn_bb_regs("0"); }
  }
  warn_bb_regs("\n");
}



