/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... dump_one_instruction_description
  FUNCTION ........ Debugging routine implemented to output the 
     translation code (generated for each basic block) in a 
     human-readable way.  There are three flag values that 
     control whether the translated instruction is preceded
     by blank space, an empty parentheses set (to indicate it
     is code associated with an original instruction, but not 
     an original instruction) or an original user program 
     instruction address (indicating it is a copy of the 
     instruction at that address).

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 
  CALLED-ROUTINES.. 
  CALLED-BY........ None
  INCLUDED-BY...... 
  MACROS-USED...... None
  INCLUDES......... <see below>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

#include <stdio.h>

#include "opc.h"

#include "io_msgs.h"

static unsigned global_print_line_index = 0;

#ifndef DO_64_BIT_COMPILE
void 
dump_one_instruction_description(unsigned * first_addr, 
				 unsigned * last_addr,  
				 unsigned * user_addr,
				 int flag
				 )
{ 
  unsigned * temp_ptr = first_addr;

  switch (flag)
    {
    case 0:
      for (temp_ptr = (first_addr); temp_ptr < (last_addr); temp_ptr++)
	{
	  char buf[512] = "*illegal instruction (data?)*";
	  unsigned iword = *temp_ptr;
	  opc_idx  index  = opc_index(iword);	
	  opc_dis(iword, buf);
	  warn_xlate("%6u             -> %08x %08x  (%3d)   =   %s\n", global_print_line_index, 
		     (unsigned)temp_ptr, iword, index, buf);
	  global_print_line_index++;
	}
      break;

    case 1:
      for (temp_ptr = (first_addr); temp_ptr < (last_addr); temp_ptr++)
	{
	  char buf[512] = "*illegal instruction (data?)*";
	  unsigned iword = *temp_ptr;
	  opc_idx  index  = opc_index(iword);	
	  opc_dis(iword, buf);
	  warn_xlate("%6u  (%08x) -> %08x %08x  (%3d)   =   %s\n", global_print_line_index, 
		     (unsigned)user_addr, (unsigned)temp_ptr, *temp_ptr, index, buf);
	  user_addr++;
	  global_print_line_index++;
	}
      break;

    case 2:
      for (temp_ptr = (first_addr); temp_ptr < (last_addr); temp_ptr++)
	{
	  char buf[512] = "*illegal instruction (data?)*";
	  unsigned iword = *temp_ptr;
	  opc_idx  index  = opc_index(iword);	
	  opc_dis(iword, buf);
	  warn_xlate("%6u  (        ) -> %08x %08x  (%3d)   =   %s\n", global_print_line_index, 
		     (unsigned)temp_ptr, iword, index, buf);
	  global_print_line_index++;
	}
      break;
  
    case 3:
      for (temp_ptr = (first_addr); temp_ptr < (last_addr); temp_ptr++)
	{
	  char buf[512] = "*illegal instruction (data?)*";
	  unsigned iword = *temp_ptr;
	  opc_idx  index  = opc_index(iword);	
	  opc_dis(iword, buf);
	  warn("%6u             -> %08x %08x  (%3d)   =   %s\n", global_print_line_index, 
	       (unsigned)temp_ptr, iword, index, buf);
	  global_print_line_index++;
	}
      break;

    case 4:
      {
	char buf[512] = "*illegal instruction (data?)*";
	unsigned iword;
	opc_idx  index;
	temp_ptr = (first_addr);
	iword = *temp_ptr;
	index = opc_index(iword); 
	opc_dis(iword, buf);
	warn_xlate("%6u s(%08x) -> %08x %08x  (%3d)   =   %s\n", global_print_line_index, 
		   (unsigned)user_addr, (unsigned)temp_ptr, *temp_ptr, index, buf);
	user_addr++;
	global_print_line_index++;
	for (temp_ptr++; temp_ptr < (last_addr); temp_ptr++)
	  {
	    iword = *temp_ptr;
	    index = opc_index(iword); 
	    opc_dis(iword, buf);
	    warn_xlate("%6u  ( simul. ) -> %08x %08x  (%3d)   =   %s\n", global_print_line_index, 
		       (unsigned)temp_ptr, iword, index, buf);
	    global_print_line_index++;
	  }
      }
    break;

    default:
      warn("ERROR:: Bad call to dump_one_instruction_description; flag = %d\n", flag);
      break;
    }
}

#else 

void 
dump_one_instruction_description(unsigned * first_addr, 
				 unsigned * last_addr,  
				 unsigned * user_addr,
				 int flag
				 )
{ 
  unsigned * temp_ptr = first_addr;

  switch (flag)
    {
    case 0:
      for (temp_ptr = (first_addr); temp_ptr < (last_addr); temp_ptr++)
	{
	  char buf[512] = "*illegal instruction (data?)*";
	  unsigned iword = *temp_ptr;
	  opc_idx  index  = opc_index(iword);	
	  opc_dis(iword, buf);
	  warn_xlate("%6u                     -> %016LLx %08x  (%3d)   =   %s\n", global_print_line_index, 
		     temp_ptr, iword, index, buf);
	  global_print_line_index++;
	}
      break;

    case 1:
      for (temp_ptr = (first_addr); temp_ptr < (last_addr); temp_ptr++)
	{
	  char buf[512] = "*illegal instruction (data?)*";
	  unsigned iword = *temp_ptr;
	  opc_idx  index  = opc_index(iword);	
	  opc_dis(iword, buf);
	  warn_xlate("%6u  (%016LLx) -> %016LLx %08x  (%3d)   =   %s\n", global_print_line_index, 
		     user_addr, temp_ptr, *temp_ptr, index, buf);
	  user_addr++;
	  global_print_line_index++;
	}
      break;

    case 2:
      for (temp_ptr = (first_addr); temp_ptr < (last_addr); temp_ptr++)
	{
	  char buf[512] = "*illegal instruction (data?)*";
	  unsigned iword = *temp_ptr;
	  opc_idx  index  = opc_index(iword);	
	  opc_dis(iword, buf);
	  warn_xlate("%6u  (                ) -> %016LLx %08x  (%3d)   =   %s\n", global_print_line_index, 
		     temp_ptr, iword, index, buf);
	  global_print_line_index++;
	}
      break;
  
    case 3:
      for (temp_ptr = (first_addr); temp_ptr < (last_addr); temp_ptr++)
	{
	  char buf[512] = "*illegal instruction (data?)*";
	  unsigned iword = *temp_ptr;
	  opc_idx  index  = opc_index(iword);	
	  opc_dis(iword, buf);
	  warn("%6u             -> %016LLx %08x  (%3d)   =   %s\n", global_print_line_index, 
	       temp_ptr, iword, index, buf);
	  global_print_line_index++;
	}
      break;

    case 4:
      {
	char buf[512] = "*illegal instruction (data?)*";
	unsigned iword;
	opc_idx  index;
	temp_ptr = (first_addr);
	iword = *temp_ptr;
	index = opc_index(iword); 
	opc_dis(iword, buf);
	warn_xlate("%6u s(%016LLx) -> %016LLx %08x  (%3d)   =   %s\n", global_print_line_index, 
		   user_addr, temp_ptr, *temp_ptr, index, buf);
	user_addr++;
	global_print_line_index++;
	for (temp_ptr++; temp_ptr < (last_addr); temp_ptr++)
	  {
	    iword = *temp_ptr;
	    index = opc_index(iword); 
	    opc_dis(iword, buf);
	    warn_xlate("%6u  (   simulation   ) -> %016LLx %08x  (%3d)   =   %s\n", global_print_line_index, 
		       temp_ptr, iword, index, buf);
	    global_print_line_index++;
	  }
      }
    break;

    case 5:
      for (temp_ptr = (first_addr); temp_ptr < (last_addr); temp_ptr++)
	{
	  char buf[512] = "*illegal instruction (data?)*";
	  unsigned iword = *temp_ptr;
	  opc_idx  index  = opc_index(iword);	
	  opc_dis(iword, buf);
	  warn_xlate("%6u  (   initialize   ) -> %016LLx %08x  (%3d)   =   %s\n", global_print_line_index, 
		     temp_ptr, iword, index, buf);
	  global_print_line_index++;
	}
      break;

    default:
      warn("ERROR:: Bad call to dump_one_instruction_description; flag = %d\n", flag);
      break;
    }
}

#endif
