#if( BUILD_FFGZ )
#include "zlib.h"
#endif

#include <stdio.h>
#include <unistd.h>
#include "ansi_assert.h"
#include "ansi_stdlib.h"

#include "aria_interface.h"
#include "opc.h"
#include "opc_instr_info.h"

#if( DEBUG )
#define DBG(x)	((void)(x))
#else
#define DBG(x)	((void)(0))
#endif

static FILE * fftrace_fp = 0;
static FILE * mystderr;

int do_xor_iaddress = 0;
int do_xor_daddress = 0;
int do_xor_seg13 = 1;

struct fftrace_state {
  FILE *	fp;
  unsigned	pc;
  char		done;
  unsigned	segs[16];
};

#define BUF_SIZE	16
static	unsigned fftrace_buf[BUF_SIZE * ARIA_TB_PER_INSTR_SIZE];

#define deref_fftrace_state(p)	(p)
#define fp_fftrace_state(p)	(deref_fftrace_state(p)->fp)
#define pc_fftrace_state(p)	(deref_fftrace_state(p)->pc)
#define done_fftrace_state(p)	(deref_fftrace_state(p)->done)
#define segs_fftrace_state(p)	(deref_fftrace_state(p)->segs)
#define seg_fftrace_state(p,i)	(segs_fftrace_state(p)[i])

#if( BUILD_FFGZ )
  #define fget_word(fp, ptr)	(gzread( fp, ptr, 1*sizeof(unsigned))>>2);
#else
  #define fget_word(fp, ptr)	fread( ptr, sizeof(unsigned), 1, fp);
#endif


#define XOR_ADDR( res_, addr_, state_ )			\
  {							\
    unsigned	tmp_addr_;				\
    unsigned	seg_id_;				\
    unsigned	seg_val_;				\
							\
    seg_id_ = (addr_ >> 28)&0xf;			\
    seg_val_ = seg_fftrace_state(state_, seg_id_);	\
    tmp_addr_ = (addr_ &0xfffffff);			\
    tmp_addr_ |= (seg_val_ << 28);			\
    tmp_addr_ ^= (seg_val_ << 8) & 0xfffff000;		\
    res_ = tmp_addr_;					\
  }

void
ffinit(
	FILE *	fp
	)
{

  mystderr = fdopen(dup(2), "w");
  fftrace_fp = fp;
}

unsigned *
do_aria_setup_with_blocksize_limit(
	proc_state_type *	proc_state_ptr,
	int			dummy_param
	)
{
  struct fftrace_state *	ptr = (struct fftrace_state *) proc_state_ptr;
  int				i;
  unsigned			pc;
  int				in;

  opc_init();
  fp_fftrace_state(ptr) = fftrace_fp;
  for( i = 0; i < 16; i++ ) {
    seg_fftrace_state(ptr,i) = i;
  }

  in = fget_word(fftrace_fp, &pc);

  if( in == 0 ){
    DBG( fprintf(mystderr, "empty trace...\n") );
    done_fftrace_state(ptr) = 1;
    return 0;
  }

  done_fftrace_state(ptr) = 0;
  pc_fftrace_state(ptr) = pc;

  DBG( fprintf(mystderr, "leaving setup...\n") );
  return (unsigned *) pc;
}

unsigned *
do_aria_execution(
	unsigned *		bb_address,
	proc_state_type *	proc_state_ptr,
	unsigned *		trace_size,
	unsigned **		static_trbuf_ptr,
	unsigned **		not_taken_bb_address,
	unsigned		not_taken_execution_flag
	)
{
  struct fftrace_state *	ptr = (struct fftrace_state *) proc_state_ptr;
  FILE *			fpin;
  unsigned			pc;
  unsigned			w_special;
  unsigned			w_memop;
  unsigned			w_memlen;
  unsigned			w_multi[2];
  unsigned			w_branch;
  unsigned			w_disp;
  unsigned			iword;
  int				i;
  int				in;
  unsigned			seg_id;
  unsigned			seg_word;
  unsigned			index;
  unsigned			old_pc;
  unsigned			ffkind;
  unsigned			daddr;
  unsigned			iaddr;

  if( not_taken_execution_flag ) {
    DBG( fprintf(mystderr, "not taken exec flag...\n") );
    *trace_size = 0;
    return (unsigned *) pc_fftrace_state(ptr);
  }

  if( done_fftrace_state(ptr) ) {
    DBG( fprintf(mystderr, "already done...\n") );
    *trace_size = 0;
    return 0;
  }

  fpin = fp_fftrace_state(ptr);
  pc = pc_fftrace_state(ptr);

  for( i = 0; i < BUF_SIZE; i++ ) {

    while( 1 ) {
      in = fget_word( fpin, &iword);
      if( in != 1 ) {
	done_fftrace_state(ptr) = 1;
	goto done_reading;
      }

      if( iword != 1 ) {
	break;
      }

      in = fget_word(fpin,  &w_special);
      if( in != 1 ) {
	done_fftrace_state(ptr) = 1;
	goto done_reading;
      }

      seg_id = w_special >> 28;
      seg_word = w_special & 0x00ffffff;

      seg_fftrace_state(ptr, seg_id) = seg_word;
    }

    index = opc_index(iword);

    if( index == OPC_ILLEGAL ) {
      fprintf( mystderr, "insn not recognized: 0x%08x", iword);
      exit(1);
    }

    old_pc = pc;
    ffkind = opc_ff32_cat(index);

    switch( ffkind ) {

    case OPC_FF32_OTHER: {
	pc += 4;

        DBG(fprintf(mystderr, "found other ... %08x\n", iword));

	if( do_xor_iaddress && (do_xor_seg13 || (old_pc >> 28) != 13) ) {
	  XOR_ADDR( iaddr, old_pc, ptr);
	}
	else {
          iaddr = old_pc;
	}

	aria_Trace_Iaddr(fftrace_buf, i) = old_pc;
	aria_Trace_Iword(fftrace_buf, i) = iword;
	aria_Trace_Ikind(fftrace_buf, i) = opc_itype(index);
	break;
      }

    case OPC_FF32_BRANCH: {
	in = fget_word(fpin,  &w_branch);
	if( in != 1 ) {
	  done_fftrace_state(ptr) = 1;
	  goto done_reading;
	}
	pc = w_branch;

	if( do_xor_iaddress && (do_xor_seg13 || (old_pc >> 28) != 13) ) {
	    XOR_ADDR( iaddr, old_pc, ptr);
	}
	else {
          iaddr = old_pc;
	}

        aria_Trace_Iaddr(fftrace_buf, i) = iaddr;
	aria_Trace_Iword(fftrace_buf, i) = iword;
	aria_Trace_Daddr(fftrace_buf, i) = pc;
	aria_Trace_Ikind(fftrace_buf, i) = opc_itype(index);

	i++;
	goto done_reading;
      }

    case OPC_FF32_MEMOP: {
	in = fget_word(fpin,  &w_memop);

	if( in != 1 ) {
	  done_fftrace_state(ptr) = 1;
	  goto done_reading;
	}
	pc += 4;

	if( do_xor_daddress ) {
	  XOR_ADDR( daddr, w_memop, ptr);
	}
	else {
	  daddr = w_memop;
	}

	if( do_xor_iaddress  && (do_xor_seg13 || (old_pc >> 28) != 13) ) {
	  XOR_ADDR( iaddr, old_pc, ptr);
	}
	else {
          iaddr = old_pc;
	}

	aria_Trace_Iaddr(fftrace_buf, i) = old_pc;
	aria_Trace_Iword(fftrace_buf, i) = iword;
	aria_Trace_Daddr(fftrace_buf, i) = daddr;
	aria_Trace_Dlen(fftrace_buf, i) = opc_dlength(index);
	aria_Trace_Ikind(fftrace_buf, i) = opc_itype(index);

	break;
      }

    case OPC_FF32_MULTI: {
	in = fget_word(fpin,  &w_multi[0]);
	in += fget_word(fpin, &w_multi[1]);
	if( in != 2 ) {
	  done_fftrace_state(ptr) = 1;
	  goto done_reading;
	}
	w_memop = w_multi[0];
	w_memlen = w_multi[1];
	pc += 4;

	if( do_xor_daddress ) {
	  XOR_ADDR( daddr, w_memop, ptr);
	}
	else {
	  daddr = w_memop;
	}

	if( do_xor_iaddress && (do_xor_seg13 || (old_pc >> 28) != 13) ) {
	  XOR_ADDR( iaddr, old_pc, ptr);
	}
	else {
          iaddr = old_pc;
	}

	aria_Trace_Iaddr(fftrace_buf, i) = old_pc;
	aria_Trace_Iword(fftrace_buf, i) = iword;
	aria_Trace_Daddr(fftrace_buf, i) = daddr;
	aria_Trace_Dlen(fftrace_buf, i) = w_memlen;
	aria_Trace_Ikind(fftrace_buf, i) = opc_itype(index);

	break;
      }

    default: {
	fprintf(mystderr, "unrecognized word=0x%08x\n", iword);
	pc += 4;
	exit( 1 );
      }
    }
  }

done_reading:;

  pc_fftrace_state(ptr) = pc;
  *trace_size = i;
  *static_trbuf_ptr = fftrace_buf;
  DBG(fprintf(mystderr, "finally i=%d size=%d...\n", i, *trace_size));

  return (unsigned *) pc;
}
