/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... None
  FUNCTION ........ Inlined (#included) code to set up the trace 
      buffer entry data corresponding to one original user instr.
  
  NOTES............

  ASSUMPTIONS... None
  GLOBAL-VARS...  NOTE: all are unsigned values unless noted
    orig_bb_addr  : The original user-program instruction address 
    iword         : The original user-program instruction word
    iword_index   : The original user-program instruction index
    iword_kind    : The original user-program instruction kind
    iword_daddr   : The address of any data the instruction accesses
    iword_dlen    : The number of bytes of memory accessed
    next_xlate_address : holds the next available xlation address
         (this is what must be aligned to the user-specified boundary)
	 [ this is an unsigned * variable]
    do_iaddr_fixup : flag to indicate if "fixed" instruction addresses are 
        to be used; this causes the trace addresses to be offset by a factor
    global_user_iaddr_offset : the instruction address offset to use when 
        fixing the instruction addresses.
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 0= Successful
  CALLED-ROUTINES.. opc_index, opc_itype
  CALLED-BY........ None
  INCLUDED-BY...... do_translation.c
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
  {
    union {
      unsigned long long ll_addr;
      struct {
	unsigned addr_w1;
	unsigned addr_w2;
      } wds;
    } faddr;
    unsigned * nptr = next_xlate_address;

    /** If the instruction address needs to be "fixed up", **/
    /**  do so before putting it in the trace buffer       **/
    if ( (do_iaddr_fixup) && ((instr_xlate_info[total_instr_in_bb].iaddr & 0xf0000000) == global_user_program_iaddr_mask) ) {
      faddr.ll_addr = instr_xlate_info[total_instr_in_bb].iaddr - global_user_iaddr_offset;
    }
    else {
      faddr.ll_addr = instr_xlate_info[total_instr_in_bb].iaddr;
    }

    *nptr  = faddr.wds.addr_w1;    nptr++;
    *nptr  = faddr.wds.addr_w2;    nptr++;

    *nptr  = instr_xlate_info[total_instr_in_bb].iword;    nptr++;
    *nptr  = instr_xlate_info[total_instr_in_bb].ikind;    nptr++;

    faddr.ll_addr = instr_xlate_info[total_instr_in_bb].daddr;
    *nptr  = faddr.wds.addr_w1;    nptr++;
    *nptr  = faddr.wds.addr_w2;    nptr++;

    *nptr  = instr_xlate_info[total_instr_in_bb].dlen;     nptr++;

    /** increment the next_xlate_Address by one buffer entry. **/
    next_xlate_address += ARIA_TB_PER_INSTR_SIZE;
  }  
