/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... hashing.c
  FUNCTION ........ Contains the definitions of the functions used 
      is building and accessing the hash table of translations.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... <see below>
  CALLED-ROUTINES.. None
  CALLED-BY........ do_translation.c, do_svc_translation.c, 
     aria_interface.c, 
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... <see below>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
#include <stdio.h>
#include "ansi_assert.h"

#include "global_variables.h"

#include "hashing.h"

#include "io_msgs.h"

static unsigned next_overflow = 0;

/*******************************************************************
  PROGRAM NAME..... initialize_hash_table
  FUNCTION ........ Initializes the hash table

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ aria_main
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... <see above>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
void
initialize_hash_table()
{
  int i;
  for (i = 0; i < NUMBER_OF_HASH_TABLE_ENTRIES; i++) {
    int j;
    the_hash_table[i].orig_address           = NULL;
    the_hash_table[i].xlated_address         = NULL;
    the_hash_table[i].nt_xlated_address      = NULL;
    the_hash_table[i].microtrace_buffer_addr = NULL;

    the_hash_table[i].overflow = NULL;

    the_hash_table[i].user_stop_iaddr = NULL;

    for (j = 0; j < 12; j++) {
      the_hash_table[i].use_def_vectors[j] = 0x0;
    }

    the_hash_table[i].size_of_prolog = (short)0x0;
    the_hash_table[i].size_of_epilog = (short)0x0;

    the_hash_table[i].user_bb_size         = (short)0x0;
    the_hash_table[i].taken_xl_bb_size     = (short)0x0;
    the_hash_table[i].not_taken_xl_bb_size = (short)0x0;

    the_hash_table[i].bb_has_dyn_data   = (unsigned char)0x0;
    the_hash_table[i].bb_ends_in_branch = (unsigned char)0x0;

    the_hash_table[i].generation = 0x0;

    the_hash_table[i].base_reg_used    = (unsigned char)0x0;
    the_hash_table[i].scratch_reg_used = (unsigned char)0x0;
    the_hash_table[i].buffer_reg_used  = (unsigned char)0x0;
  }
}


/*******************************************************************
  PROGRAM NAME..... find_hash_table_entry_for
  FUNCTION ........ Finds the location in the hash table to which 
      the given original user program address would correspond.
      If there is an existing (filled-in) entry, then that hash
      table entry is returned.  Otherwise, a empty entry is allocated
      and linked into the hash table as required.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... The starting address of the associated translation
     for the true-path block.
  CALLED-ROUTINES.. None
  CALLED-BY........ aria_interface.c (do_aria_execution)
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... <see above>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
hash_entry *
find_hash_table_entry_for(unsigned *  or_address, 
			  unsigned    generation_flag
			  )
{
  unsigned oaddr = (unsigned) or_address;
  unsigned index = ((oaddr>>2) % NUMBER_OF_HASH_TABLE_ENTRIES);
  /** unsigned index = 1; USED FOR TESTING ONLY **/

  hash_entry * new_optr;
  hash_entry * optr  = &the_hash_table[index];

  /** DEBUG VERSION **
    if (optr->orig_address == NULL) {
    return optr;
    }
    if (HashTable_original_address(*optr) == or_address) {
    if (HashTable_generation(*optr) == generation_flag) {
    return optr;
    }
    else {
    warn("============================== %u  vs  %u\n", HashTable_generation(*optr), generation_flag);
    }
    }
    **/
  if ( (optr->orig_address == NULL) ||
       ((HashTable_original_address(*optr) == or_address) && (HashTable_generation(*optr) == generation_flag))
       ) {
    /* warn("==> address = 0x%08x  index = %d\n", or_address, index); */
    return optr;
  }
  /**/

  while (HashTable_overflow(*optr) != NULL) {
    optr = HashTable_overflow(*optr);

    if ( (HashTable_original_address(*optr) == or_address) && (HashTable_generation(*optr) == generation_flag) ) {
      return optr;
    }
  }

  if (next_overflow == (HASH_TABLE_OVERFLOW_MULTIPLIER*NUMBER_OF_HASH_TABLE_ENTRIES)) {
    fprintf(mystderr, "ARIA: ERROR: Out of overflow space in the hash table in\n\t find_hash_table_entry_for\n");
    fprintf(mystderr, "         Filled all %u overflow entries... [%llu total translations]\n", 
	    (HASH_TABLE_OVERFLOW_MULTIPLIER*NUMBER_OF_HASH_TABLE_ENTRIES), global_bb_index);
    /**
      if (do_hash_tables_dump) {
      debug_dump_hash_tables();
      }
      **/
    /** THIS IS PRETTY HARSH
      abort();
      **/
    /** EMERGENCY (DRASTIC) RECOVERY: dump all translations and continue! **/
    {
      int i;
      for (i = 0; i < NUMBER_OF_HASH_TABLE_ENTRIES; i++) {
	int j;
	the_hash_table[i].orig_address           = NULL;
	the_hash_table[i].xlated_address         = NULL;
	the_hash_table[i].nt_xlated_address      = NULL;
	the_hash_table[i].microtrace_buffer_addr = NULL;
	
	the_hash_table[i].overflow = NULL;
	
	the_hash_table[i].user_stop_iaddr = NULL;
	
	for (j = 0; j < 12; j++) {
	  the_hash_table[i].use_def_vectors[j] = 0x0;
	}
	
	the_hash_table[i].size_of_prolog = (short)0x0;
	the_hash_table[i].size_of_epilog = (short)0x0;
	
	the_hash_table[i].user_bb_size         = (short)0x0;
	the_hash_table[i].taken_xl_bb_size     = (short)0x0;
	the_hash_table[i].not_taken_xl_bb_size = (short)0x0;
	
	the_hash_table[i].bb_has_dyn_data   = (unsigned char)0x0;
	the_hash_table[i].bb_ends_in_branch = (unsigned char)0x0;
	
	the_hash_table[i].generation = 0x0;
	
	the_hash_table[i].base_reg_used    = (unsigned char)0x0;
	the_hash_table[i].scratch_reg_used = (unsigned char)0x0;
	the_hash_table[i].buffer_reg_used  = (unsigned char)0x0;
      }
      next_overflow = 0;
      next_xlate_address = (unsigned *)XLATE_START_SEGMENT;
    }
    /* warn("==> address = 0x%08x  index = %d\n", or_address, index); */
    return &the_hash_table[index];

  } 
  
  /* warn("==> address = 0x%08x  index = %d  next_overflow = %d\n", or_address, index, next_overflow); */
  new_optr = &overflow_space[next_overflow++];

  HashTable_original_address(*new_optr)   = NULL;
  HashTable_xlation_address(*new_optr)    = NULL;  
  HashTable_NT_xlation_address(*new_optr) = NULL;
  HashTable_trace_buffer(*new_optr)       = NULL;
  HashTable_overflow(*new_optr)           = NULL;

  HashTable_overflow(*optr) = new_optr;
  
  return new_optr;
}


/*******************************************************************
  PROGRAM NAME..... dump_hash_table_entry_descriptions
  FUNCTION ........ Dumps the contents of the hash table

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... <see above>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
void
dump_hash_table_entry_descriptions()
{
  int i;
  warn_hash("Entry#  :  OrigAddr   XlatAddr   NTXLAddr   SticTrBf   Overflow flag\n");
  for (i = 0; i < NUMBER_OF_HASH_TABLE_ENTRIES; i++) {
    if (the_hash_table[i].orig_address != 0){
      hash_entry * tptr;
      warn_hash("%6d  :  %08x   %08x   %08x   %08x   %1d\n", i,
		the_hash_table[i].orig_address,
		the_hash_table[i].xlated_address,
		the_hash_table[i].nt_xlated_address,
		the_hash_table[i].microtrace_buffer_addr,
		(the_hash_table[i].overflow != NULL)
		);
      tptr = the_hash_table[i].overflow;
      while (tptr != NULL) {
	warn_hash("   -->  :  %08x   %08x   %08x   %08x   %1d\n", 
		  tptr->orig_address, tptr->xlated_address, tptr->nt_xlated_address, 
		  tptr->microtrace_buffer_addr, tptr->overflow);
	tptr = tptr->overflow;
      }
    }
  }
}

void
dump_hash_table_entry(hash_entry * tptr)
{
  warn("Address   :  OrigAddr   XlatAddr   NTXLAddr   SticTrBf  Usize  Tsize  NTsze  Overflow flag\n");
  warn("%08x  :  %08x   %08x   %08x   %08x  %5u  %5u  %5u  %1d\n",
       tptr,
       HashTable_original_address(*tptr),
       HashTable_xlation_address(*tptr),
       HashTable_NT_xlation_address(*tptr),
       HashTable_trace_buffer(*tptr),
       HashTable_user_bb_size(*tptr),
       HashTable_xlated_bb_size(*tptr),
       HashTable_NT_xlated_bb_size(*tptr),
       (tptr->overflow != NULL)
       );
}



/*******************************************************************
  PROGRAM NAME..... debug_dump_hash_tables
  FUNCTION ........ Dumps the contents of the hash tables to stdout

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... <see above>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
void
debug_dump_hash_tables()
{
  int i;
  warn("Entry#  :  OrigAddr   XlatAddr   NTXLAddr   SticTrBf   O  ubb  xbb  sbb  psz esz  D B  G\n");
  for (i = 0; i < NUMBER_OF_HASH_TABLE_ENTRIES; i++) {
    if (the_hash_table[i].orig_address != 0) {
      hash_entry * tptr = &the_hash_table[i];
      warn("%6d  :  %08x   %08x   %08x   %08x   %1d  %3u  %3u  %3u  %3u %3u  %1u %1u  %1u\n", i,
	     HashTable_original_address(*tptr),
	     HashTable_xlation_address(*tptr),
	     HashTable_NT_xlation_address(*tptr),
	     HashTable_trace_buffer(*tptr),
	     HashTable_overflow(*tptr),
	     HashTable_user_bb_size(*tptr),
	     HashTable_xlated_bb_size(*tptr),
	     HashTable_NT_xlated_bb_size(*tptr),
	     HashTable_prolog_size(*tptr),
	     HashTable_epilog_size(*tptr),
	     HashTable_has_dynamic_data(*tptr),
	     HashTable_ends_in_a_branch(*tptr),
	     HashTable_generation(*tptr)
	     );
      tptr = HashTable_overflow(*tptr);
      while (tptr != NULL) {
	warn("   -->  :  %08x   %08x   %08x   %08x   %1d  %3u  %3u  %3u  %3u %3u  %1u %1u  %1u\n", i,
	       HashTable_original_address(*tptr),
	       HashTable_xlation_address(*tptr),
	       HashTable_NT_xlation_address(*tptr),
	       HashTable_trace_buffer(*tptr),
	       HashTable_overflow(*tptr),
	       HashTable_user_bb_size(*tptr),
	       HashTable_xlated_bb_size(*tptr),
	       HashTable_NT_xlated_bb_size(*tptr),
	       HashTable_prolog_size(*tptr),
	       HashTable_epilog_size(*tptr),
	       HashTable_has_dynamic_data(*tptr),
	       HashTable_ends_in_a_branch(*tptr),
	       HashTable_generation(*tptr)
	       );
	tptr = HashTable_overflow(*tptr);
      }
    }
  }
}


void
debug_dump_hash_table_entry(hash_entry * tptr, char * prefix_str)
{
  warn("%s : Address  :  OrigAddr   XlatAddr   NTXLAddr   SticTrBf   O  ubb  xbb  sbb  psz esz  D B  G\n", 
       prefix_str);
  warn("%s : %8x :  %08x   %08x   %08x   %08x   %1d  %3u  %3u  %3u  %3u %3u  %1u %1u  %1u\n", 
       prefix_str,
       tptr,
       HashTable_original_address(*tptr),
       HashTable_xlation_address(*tptr),
       HashTable_NT_xlation_address(*tptr),
       HashTable_trace_buffer(*tptr),
       HashTable_overflow(*tptr),
       HashTable_user_bb_size(*tptr),
       HashTable_xlated_bb_size(*tptr),
       HashTable_NT_xlated_bb_size(*tptr),
       HashTable_prolog_size(*tptr),
       HashTable_epilog_size(*tptr),
       HashTable_has_dynamic_data(*tptr),
       HashTable_ends_in_a_branch(*tptr),
       HashTable_generation(*tptr)
       ); 
}


/*******************************************************************
  PROGRAM NAME..... verify_hash_table_data
  FUNCTION ........ Checks that entries in the hash table are legal
      Primarily intended for debugging checks. 

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... <see below>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
void
verify_hash_table()
{
  int i;
  int printed = 0;
  for (i = 0; i < NUMBER_OF_HASH_TABLE_ENTRIES; i++) {
    if (HashTable_original_address(the_hash_table[i]) != NULL) {
      hash_entry * tptr = &the_hash_table[i];
      /**
	warn("%6d  :  %08x   %08x   %08x   %08x   %1d  %3u  %3u  %3u  %3u %3u  %1u %1u  %1u\n", i,
	HashTable_original_address(*tptr),
	HashTable_xlation_address(*tptr),
	HashTable_NT_xlation_address(*tptr),
	HashTable_trace_buffer(*tptr),
	HashTable_overflow(*tptr),
	HashTable_user_bb_size(*tptr),
	HashTable_xlated_bb_size(*tptr),
	HashTable_NT_xlated_bb_size(*tptr),
	HashTable_prolog_size(*tptr),
	HashTable_epilog_size(*tptr),
	HashTable_has_dynamic_data(*tptr),
	HashTable_ends_in_a_branch(*tptr),
	HashTable_generation(*tptr)
	);
	**/

      while (tptr != NULL) {
	if (HashTable_original_address(*tptr) < (unsigned *)0x00003380) {
	  if (printed == 0) {
	    warn_hash("\n\n%7u : Entry#  :  OrigAddr   XlatAddr   NTXLAddr   SticTrBf   Overflow flag\n",
		      global_bb_index);
	    printed = 1;
	  }
	  warn("%6d  :  %08x   %08x   %08x   %08x   %1d  %3u  %3u  %3u  %3u %3u  %1u %1u  %1u\n", 
	       i,
	       HashTable_original_address(*tptr),
	       HashTable_xlation_address(*tptr),
	       HashTable_NT_xlation_address(*tptr),
	       HashTable_trace_buffer(*tptr),
	       HashTable_overflow(*tptr),
	       HashTable_user_bb_size(*tptr),
	       HashTable_xlated_bb_size(*tptr),
	       HashTable_NT_xlated_bb_size(*tptr),
	       HashTable_prolog_size(*tptr),
	       HashTable_epilog_size(*tptr),
	       HashTable_has_dynamic_data(*tptr),
	       HashTable_ends_in_a_branch(*tptr),
	       HashTable_generation(*tptr)
	       );
	  if (HashTable_overflow(*tptr) != NULL) {
	    warn(" -> ", i);
	  }
	}	  
	tptr = tptr->overflow;
      }
    }
  }
}

