/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... aria_interface.h
  FUNCTION ........ Declares the functions in the Aria user 
     interface.  Also serves as the description of the 
     program call-interface to Aria.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... <user mode code>
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... May, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
#ifndef ARIA_INTERFACE_H
#define ARIA_INTERFACE_H

#include "proc_state.h"
#include "proc_state_accessors.h"

/** Define to make the compile-time execution environment explicit **/
#ifndef DO_64_BIT_COMPILE
 #define ARIA_EXECUTION_ENVIRONMENT_BITS  32
#else
 #define ARIA_EXECUTION_ENVIRONMENT_BITS  64
#endif

/** Define some macros for user-specified types... **/
/** NOTE: The signs on these values are significant, in that they  **/
/**   can be used to determine whether this is a before-execution  **/
/**   action (BEFORE) or an after-execution action (AFTER)         **/
/**   or no action at all (DO_NO_ARIA_ACTION).                     **/
#define DO_NO_ARIA_ACTION       0x0
#define DO_ARIA_ACTION_BEFORE   0x1
#define DO_ARIA_ACTION_AFTER    0x2

/* This define is used to identify the value returned when Aria */
/*  detects the end of the user program execution. (EOF flag)   */
#define ARIA_EXIT_FLAG_ADDRESS 0x00000000

/** This defines the aria_function_action_type,                 **/
/**  which is a pointer to a user-supplied function.            **/
/** The parameters to the function are:                         **/
/*     unsigned * instruction address                            */
/*     unsigned instruction word                                 */
/*     unsigned instruction memory access data address           */
/*     unsigned instruction memory access data length            */
/*     proc_state_type * : pointer to the processor state space  */
/*     void *            : a user-supplied pointer for user data */
typedef int (*aria_user_function_type)(unsigned *, unsigned, unsigned *, unsigned, proc_state_type *, void *) ;

/* These are symbolic names for trace buffer entry positions/sizes/etc. */
typedef struct aria_trace_buffer_struct {
  unsigned iaddr;
  unsigned iword;
  unsigned ikind;
  unsigned daddr;
  unsigned dlen;
} aria_trace_buffer_entry_type;

#define ARIA_TB_ITEM_SIZE           4

#define ARIA_TB_PER_INSTR_SIZE      (sizeof(aria_trace_buffer_entry_type)>>2) /* divide by size of an unsigned */
#define ARIA_TB_IADDR_OFFSET        (offsetof(aria_trace_buffer_entry_type,iaddr))
#define ARIA_TB_IWORD_OFFSET        (offsetof(aria_trace_buffer_entry_type,iword))
#define ARIA_TB_IKIND_OFFSET        (offsetof(aria_trace_buffer_entry_type,ikind))
#define ARIA_TB_DADDR_OFFSET        (offsetof(aria_trace_buffer_entry_type,daddr))
#define ARIA_TB_DLENGTH_OFFSET      (offsetof(aria_trace_buffer_entry_type,dlen))
#define ARIA_TB_BRTARGET_OFFSET     (offsetof(aria_trace_buffer_entry_type,daddr))


/** The micro-trace buffer accessor macros. **/
/**  The user of these macros provides some independence of the timer **/
/**   model and the actual implementation.                            **/
typedef unsigned aria_Iaddr_type;
typedef unsigned aria_Daddr_type;

#define aria_Trace_Iaddr(trb_ptr,idx)      ((trb_ptr)[ARIA_TB_PER_INSTR_SIZE*(idx) + 0])
#define aria_Trace_Iword(trb_ptr,idx)      ((trb_ptr)[ARIA_TB_PER_INSTR_SIZE*(idx) + 1])
#define aria_Trace_Ikind(trb_ptr,idx)      ((trb_ptr)[ARIA_TB_PER_INSTR_SIZE*(idx) + 2])
#define aria_Trace_Daddr(trb_ptr,idx)      ((trb_ptr)[ARIA_TB_PER_INSTR_SIZE*(idx) + 3])
#define aria_Trace_Dlen(trb_ptr,idx)       ((trb_ptr)[ARIA_TB_PER_INSTR_SIZE*(idx) + 4])

#define aria_Trace_Iaddr_ptr(trb_ptr,idx)  ((trb_ptr)+ARIA_TB_PER_INSTR_SIZE*(idx) + 0)
#define aria_Trace_Iword_ptr(trb_ptr,idx)  ((trb_ptr)+ARIA_TB_PER_INSTR_SIZE*(idx) + 1)
#define aria_Trace_Ikind_ptr(trb_ptr,idx)  ((trb_ptr)+ARIA_TB_PER_INSTR_SIZE*(idx) + 2)
#define aria_Trace_Daddr_ptr(trb_ptr,idx)  ((trb_ptr)+ARIA_TB_PER_INSTR_SIZE*(idx) + 3)
#define aria_Trace_Dlen_ptr(trb_ptr,idx)   ((trb_ptr)+ARIA_TB_PER_INSTR_SIZE*(idx) + 4)

/* These are some potentially helpful macros for analyzing the micro-trace buffer */
#define aria_Trace_has_Daddr(trb_ptr,idx)  (OPC_IS_A_MEMORY_ACCESS_OP(aria_Trace_Ikind(trb_ptr, idx)) | \
					    OPC_IS_A_FLOW_CONTROL_OP(aria_Trace_Ikind(trb_ptr, idx)) )

#define aria_Trace_has_Dlen(trb_ptr,idx)   (OPC_IS_A_MEMORY_ACCESS_OP(aria_Trace_Ikind(trb_ptr, idx)) )


/** Values for the not_taken_execution_flag value to indicate **/
/**      true-taken-path execution or                         **/
/**      mis-speculated not-taken path execution              **/
/**  for the do_aria_execution routine.                       **/
#define ARIA_TAKEN_PATH_EXECUTION      0
#define ARIA_NOT_TAKEN_PATH_EXECUTION  1

#ifndef ARIA_GPR_SIZE
 #ifndef DO_64_BIT_COMPILE
  #define ARIA_GPR_SIZE 4
 #else
  #define ARIA_GPR_SIZE 8
 #endif
#endif

#ifndef ARIA_FPR_SIZE
 #define ARIA_FPR_SIZE 8
#endif


/*******************************/
/**  The interface functions  **/
/*******************************/

unsigned * do_aria_setup_with_blocksize_limit(proc_state_type * saved_proc_state_ptr, int max_block_size_override);

#define do_aria_setup(x) do_aria_setup_with_blocksize_limit((x), 0)

unsigned * do_aria_execution(unsigned *  bb_address, 
			     proc_state_type * proc_state_ptr, 
			     unsigned *  trace_size,
			     unsigned ** static_trbuf_ptr, 
			     unsigned ** not_taken_bb_address,
			     unsigned    not_taken_execution_flag
			     );

unsigned * do_aria_skip_bbs(unsigned * bb_address, 
			    proc_state_type *  proc_state_ptr, 
			    unsigned num_bbs_to_skip
			    );
unsigned * do_aria_skip_bbs_ll(unsigned * bb_address, 
			       proc_state_type *  proc_state_ptr, 
			       unsigned long long num_bbs_to_skip
			       );

unsigned * do_aria_skip_instructions_ll(unsigned *  bb_address, 
					proc_state_type *     proc_state_ptr, 
					unsigned long long    num_instrs_to_skip,
					unsigned long long *  actual_num_instrs_skipped
					);
unsigned * do_aria_skip_instructions(unsigned *  bb_address, 
				     proc_state_type *     proc_state_ptr, 
				     unsigned   num_instrs_to_skip,
				     unsigned * actual_num_instrs_skipped
				     );

unsigned * do_aria_full_execution(unsigned *  next_bb_address, 
				  proc_state_type * saved_proc_state_ptr,
				  void * user_ptr
				  );


proc_state_type * aria_get_new_process_state_space();

void aria_copy_process_state_from_to(proc_state_type * from_state_ptr,
				     proc_state_type * to_state_ptr
				     );

/**
  int aria_register_integer(unsigned instr_index,
  short act_at_time,
  int the_integer_value
  );
  
  int aria_register_itype_integer(unsigned instr_itype,
  short act_at_time,
  int the_integer_value
  );
  **/

int aria_register_function(unsigned instr_index,
			   short act_at_time,
			   aria_user_function_type the_function
			   );

int aria_register_itype_function(unsigned instr_itype,
				 short act_at_time,
				 aria_user_function_type the_function
				 );

void dump_aria_run_statistics(proc_state_type * state_ptr
			      );

void dump_aria_run_statistics_to(proc_state_type * state_ptr,
				 FILE * fptr
				 );

#endif
