/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... aria_state.h
  FUNCTION ........ Defines the type and macros to access the 
     Aria state represetnation.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... analyze_branch.c, aria_interface.c, aria_main.c,
      canned_multiple_code.c, canned_string_code.c, do_svc_translation.c,
      do_translation.c, main.c, save_user_state.c

  MACROS-USED...... None
  INCLUDES......... <see below>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
#ifndef ARIA_STATE_H
#define ARIA_STATE_H

#include <stddef.h>

#include "globtypes.h"

/** NOTE: IF YOU MOVE THE _aria_orig_stack POSITION THEN YOU MUST UPDATE THE   **/
/**  VALUES USED IN THE truemain.s FILE (WHERE THE VALUES ARE STORED AND READ) **/
typedef struct aria_state_type_struct {
  BASE_TYPE _aria_proc_state_ptr; /* Do not move the position of this entry */
  BASE_TYPE _aria_dyn_trbuf_ptr;	 /* Do not move the position of this entry */
  BASE_TYPE _aria_orig_stack;     /* Do not move the position of this entry */
  BASE_TYPE _not_taken_rval;
  BASE_TYPE _aria_lr;
  BASE_TYPE _aria_cr;
  BASE_TYPE _aria_stack;
  BASE_TYPE _aria_TOC;
  BASE_TYPE _aria_CTR;
  BASE_TYPE _aria_XER;
  BASE_TYPE _aria_GPR[32];
  BASE_TYPE _aria_FPSCR;
  BASE_TYPE _aria_FPR[32];
  BASE_TYPE _aria_do_xlate;
  BASE_TYPE _aria_read_real_time[3];
  BASE_TYPE _aria_save_start_time[3];
  BASE_TYPE _aria_save_finish_time[3];
  BASE_TYPE _aria_save_null_time[3];
} aria_state_type;

/* Define the offsets into our local save area for ARIA to use */
#define PROC_STATE_OFFSET      (offsetof(aria_state_type, _aria_proc_state_ptr))
#define PROC_DYN_TRBUF_OFFSET  (offsetof(aria_state_type, _aria_dyn_trbuf_ptr))

#define ARIA_ORIG_STACK_SAVE   (offsetof(aria_state_type, _aria_orig_stack))

#define NOT_TAKEN_RVAL_OFFSET  (offsetof(aria_state_type, _not_taken_rval))

#define ARIA_LR_OFFSET         (offsetof(aria_state_type, _aria_lr))
#define ARIA_CR_OFFSET         (offsetof(aria_state_type, _aria_cr))
#define ARIA_STACK_OFFSET      (offsetof(aria_state_type, _aria_stack))
#define ARIA_TOC_OFFSET        (offsetof(aria_state_type, _aria_TOC))
#define ARIA_CTR_OFFSET        (offsetof(aria_state_type, _aria_CTR))
#define ARIA_XER_OFFSET        (offsetof(aria_state_type, _aria_XER))
#define ARIA_GPR_OFFSET        (offsetof(aria_state_type, _aria_GPR[0]))
#define ARIA_FPSCR_OFFSET      (offsetof(aria_state_type, _aria_FPSCR[0]))
#define ARIA_FPR_OFFSET        (offsetof(aria_state_type, _aria_FPR[0]))
#define ARIA_DO_XLATE_OFFSET   (offsetof(aria_state_type, _aria_do_xlate))

/* These spaces are used in measuring system call execution times */
#define ARIA_READ_REAL_TIME    (offsetof(aria_state_type, _aria_read_real_time[0]))
#define ARIA_SAVE_START_TIME   (offsetof(aria_state_type, _aria_save_start_time[0]))
#define ARIA_SAVE_FINISH_TIME  (offsetof(aria_state_type, _aria_save_finish_time[0]))
#define ARIA_SAVE_NULL_TIME    (offsetof(aria_state_type, _aria_save_null_time[0]))
 
/* The total size of Aria state */
#define ARIA_STATE_SAVE_SPACE_SIZE (sizeof(aria_state_type))



unsigned * insert_aria_save_state_prolog(unsigned * xl_address, 
					 unsigned gprs, unsigned fprs, unsigned crs, unsigned sprs,
					 unsigned * base_register);

unsigned * insert_aria_restore_state_epilog(unsigned * xl_address, 
					    unsigned gprs, unsigned fprs, unsigned crs, unsigned sprs,
					    unsigned * base_register);

unsigned * insert_user_program_exit_cleanup_code(unsigned * xl_address);

#endif
