/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... globdefs.h
  FUNCTION ........ Sets some global macro definition values that 
      are used by Aria.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... aria_interface.c, aria_main.c, global_variables.h,
      spec_test_driver.c, test_2buf_driver.c, test_driver.c

  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/


#ifndef GLOBDEFS_H
#define GLOBDEFS_H

#include <stdio.h>

#include "opc.h"

#include "aria_per_instr_interface.h"
#include "hashing.h"

/********************************************************************/
/** These defines are used to control where the translated program **/
/**  is placed (in memory), and where the user stack should reside **/
/********************************************************************/


/* The memory segment where the translation should be placed */
#define XLATE_START_SEGMENT       0xc0000000
#define XLATE_SPACE_SIZE          0x10000000

/* The last position in which process state space can be saved */
#define PROC_STATE_LAST_POSN      0xcf00c000
#define PROC_STATE_FIRST_POSN     0xcf000000

/* The global flags state space */
#define GLOBAL_FLAGS_STATE_SPACE_START  0xcf00c100

/* The space in which to save aria state */
/*  NOTE: use a HI_16 value to allow it  */
/*   to be set with a single ADDIS       */
#define ARIA_STATE_SAVE_SPACE     0xcfff0000
#define ARIA_STATE_SAVE_HI_16     ((ARIA_STATE_SAVE_SPACE>>16) & 0x0000ffff)

/* The space in which to put the user program stack */
#define ARIA_ALLOCATED_STACK_ADDRESS 0xcffe0000 

/* The maximum number of user instructions we can put in our internal data structures */
#define ABSOLUTE_MAXIMUM_USER_INSTR_IN_A_BLOCK  256

/** The number of hash table entries **/
#define NUMBER_OF_HASH_TABLE_ENTRIES     0x8000
#define HASH_TABLE_OVERFLOW_MULTIPLIER   2


/** Some machine state description defines **/
#define ARIA_GPR_SIZE 4
#define ARIA_FPR_SIZE 8



/** A structure type to hold all of the global variables, **/
/**  so they can be "passed" via a single pointer.        **/

typedef struct global_flags_struct {

  FILE * _mystderr;

  /* The exit code value is used to check for an _exit system call    */
  /* The exit routine address is used to detect invocation of an exit */
  unsigned _the_exit_code_value;
  unsigned _the_exit_routine_address;
  unsigned _the_do_safe_exit_flag;

  unsigned _global_user_iaddr_offset;
  unsigned _global_user_program_iaddr_mask;

  unsigned _the_machine_isa_type;

  unsigned _do_iaddr_fixup;
  unsigned _allow_global_access_permissions;
  unsigned _no_FP_registers_used_in_model_flag;
  unsigned _not_taken_address_required_flag;

  unsigned _global_use_safe_svc_code;
  unsigned _turn_svc_into_a_bclr_instruction;
  unsigned _the_user_program_architecture;

  unsigned long long _global_bb_index;
  unsigned long long _global_bb_executions;

  unsigned long long _global_max_size_ubb_counter;
  unsigned long long _global_addressibility_counter;
  unsigned long long _global_user_stop_condition_counter;

  unsigned _global_get_system_call_execution_times;

  unsigned _do_not_taken_path_translations;
  unsigned _do_canned_string_code;
  unsigned _do_dynamic_trace_generation;
  unsigned _global_do_old_initial_proc_state_code;

  unsigned _the_xlated_code_alignment;

  unsigned _global_max_block_size_limit;
  unsigned long long _use_block_limit_after_static_bb_translation;
  unsigned long long _use_block_unlimited_after_static_bb_translation;

  unsigned _global_return_nop_in_trace_for_illegal_speculatives;
  unsigned _global_spec_nop_for_illegal_iword;
  unsigned _global_spec_nop_for_illegal_itype;

  unsigned _global_do_aria_simulation;

  /** Output control flags **/
  unsigned _do_no_output_during_runs;
  unsigned long long _output_after_static_bb_translation;
  unsigned long long _output_after_dynamic_bb_execution;
  unsigned long long _no_output_after_static_bb_translation;
  unsigned long long _no_output_after_dynamic_bb_execution;
  unsigned _output_at_original_address;

  unsigned _do_full_xlate_dump;
  unsigned _do_full_execute_dump;
  unsigned _do_trace_dump;
  unsigned _do_trace_buffer_dump;
  unsigned _do_bb_regs_use_def_dump;
  unsigned _do_hash_tables_dump;
  unsigned _do_proc_state_dump;
  unsigned _do_show_runtime_warnings;
  unsigned _show_proc_float_state;

  unsigned _show_final_aria_statistics;

  /** Important state information, tables, etc. **/
  unsigned * _next_xlate_address;

  hash_entry _the_hash_table[NUMBER_OF_HASH_TABLE_ENTRIES];
  hash_entry _overflow_space[HASH_TABLE_OVERFLOW_MULTIPLIER*NUMBER_OF_HASH_TABLE_ENTRIES];

  aria_per_instr_entry_type _global_per_instruction_actions[OPC_NUM+1];

  unsigned _do_aix_4_2_run;

} global_flags_type;


#define mystderr                    (the_global_flags->_mystderr)
#define the_exit_code_value         (the_global_flags->_the_exit_code_value)
#define the_exit_routine_address    (the_global_flags-> _the_exit_routine_address)
#define do_safe_exit_flag           (the_global_flags->_the_do_safe_exit_flag)

#define the_machine_isa_type        (the_global_flags->_the_machine_isa_type)

#define global_user_iaddr_offset         (the_global_flags->_global_user_iaddr_offset)
#define global_user_program_iaddr_mask   (the_global_flags->_global_user_program_iaddr_mask)

#define do_iaddr_fixup                      (the_global_flags->_do_iaddr_fixup)
#define allow_global_access_permissions     (the_global_flags->_allow_global_access_permissions)
#define no_FP_registers_used_in_model_flag  (the_global_flags->_no_FP_registers_used_in_model_flag)
#define not_taken_address_required_flag     (the_global_flags->_not_taken_address_required_flag)

#define turn_svc_into_a_bclr_instruction (the_global_flags->_turn_svc_into_a_bclr_instruction)
#define global_use_safe_svc_code        (the_global_flags->_global_use_safe_svc_code)

#define global_get_system_call_execution_times (the_global_flags->_global_get_system_call_execution_times)

#define global_return_nop_in_trace_for_illegal_speculatives (the_global_flags->_global_return_nop_in_trace_for_illegal_speculatives)
#define global_spec_nop_iword (the_global_flags->_global_spec_nop_for_illegal_iword)
#define global_spec_nop_itype (the_global_flags->_global_spec_nop_for_illegal_itype)

#define global_do_aria_simulation       (the_global_flags->_global_do_aria_simulation)

/* The user program Instruction Set Architecture */
#define the_user_program_architecture    (the_global_flags->_the_user_program_architecture)
#define ARIA_POWER_ISA  0
#define ARIA_PPC_ISA    1

#define global_bb_index                  (the_global_flags->_global_bb_index)
#define global_bb_executions             (the_global_flags->_global_bb_executions)

#define global_max_size_ubb_counter         (the_global_flags->_global_max_size_ubb_counter)
#define global_addressibility_counter       (the_global_flags->_global_addressibility_counter)
#define global_user_stop_condition_counter  (the_global_flags->_global_user_stop_condition_counter)

#define do_not_taken_path_translations   (the_global_flags->_do_not_taken_path_translations)
#define do_canned_string_code            (the_global_flags->_do_canned_string_code)
#define do_dynamic_trace_generation      (the_global_flags->_do_dynamic_trace_generation)
#define the_xlated_code_alignment        (the_global_flags->_the_xlated_code_alignment)

#define global_max_block_size_limit                       (the_global_flags->_global_max_block_size_limit)
#define use_block_limit_after_static_bb_translation       (the_global_flags->_use_block_limit_after_static_bb_translation)
#define use_block_unlimited_after_static_bb_translation   (the_global_flags->_use_block_unlimited_after_static_bb_translation)
#define output_at_original_address                        (the_global_flags->_output_at_original_address)

#define global_do_old_initial_proc_state_code     (the_global_flags->_global_do_old_initial_proc_state_code)

/** Output control flags **/
#define do_no_output_during_runs                  (the_global_flags->_do_no_output_during_runs)
#define output_after_static_bb_translation        (the_global_flags->_output_after_static_bb_translation)
#define output_after_dynamic_bb_execution         (the_global_flags->_output_after_dynamic_bb_execution)
#define no_output_after_static_bb_translation     (the_global_flags->_no_output_after_static_bb_translation)
#define no_output_after_dynamic_bb_execution      (the_global_flags->_no_output_after_dynamic_bb_execution)

#define do_full_xlate_dump               (the_global_flags->_do_full_xlate_dump)
#define do_full_execute_dump             (the_global_flags->_do_full_execute_dump)
#define do_trace_dump                    (the_global_flags->_do_trace_dump)
#define do_trace_buffer_dump             (the_global_flags->_do_trace_buffer_dump)
#define do_bb_regs_use_def_dump          (the_global_flags->_do_bb_regs_use_def_dump)
#define do_hash_tables_dump              (the_global_flags->_do_hash_tables_dump)
#define do_proc_state_dump               (the_global_flags->_do_proc_state_dump)
#define do_show_runtime_warnings         (the_global_flags->_do_show_runtime_warnings)
#define show_proc_float_state            (the_global_flags->_show_proc_float_state)

#define show_final_aria_statistics       (the_global_flags->_show_final_aria_statistics)

  /** Important state information, tables, etc. **/

#define next_xlate_address               (the_global_flags->_next_xlate_address)

#define synch_cache_range                (the_global_flags->_synch_cache_range)

#define global_per_instruction_actions   (the_global_flags->_global_per_instruction_actions)

#define the_hash_table    (the_global_flags->_the_hash_table)
#define overflow_space    (the_global_flags->_overflow_space)

#define do_aix_4_2_run    (the_global_flags->_do_aix_4_2_run)

extern int get_mapping(int);
extern int set_mapping(int);
extern int must_save_Aria(int);
extern int find_free_register();

#endif

