/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... hashing.h
  FUNCTION ........ Forward declarations of routines used to 
     build and manipulate the hash tables.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... aria_interface.c, canned_multiple_code.c, 
      canned_string_code.c, do_svc_translation.c, do_translation.c
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

#ifndef HASHING_H
#define HASHING_H

/* Hash Table entry type definition */

typedef struct hash_entry_struct {
  unsigned * orig_address;
  unsigned * xlated_address;
  unsigned * nt_xlated_address;
  unsigned * microtrace_buffer_addr;
  struct hash_entry_struct * overflow;
  unsigned * user_stop_iaddr;
  unsigned use_def_vectors[12];
  short      size_of_prolog;
  short      size_of_epilog;
  short      user_bb_size;
  short      taken_xl_bb_size;
  short      not_taken_xl_bb_size;
  unsigned char bb_has_dyn_data;
  unsigned char bb_ends_in_branch;
  short      generation;
  unsigned char base_reg_used;
  unsigned char scratch_reg_used;
  unsigned char buffer_reg_used;
} hash_entry;

/* Access Macros */
#define HashTable_original_address(x)   ((x).orig_address)
#define HashTable_xlation_address(x)    ((x).xlated_address)
#define HashTable_NT_xlation_address(x) ((x).nt_xlated_address)
#define HashTable_trace_buffer(x)       ((x).microtrace_buffer_addr)
#define HashTable_user_bb_size(x)       ((x).user_bb_size)
#define HashTable_xlated_bb_size(x)     ((x).taken_xl_bb_size)
#define HashTable_NT_xlated_bb_size(x)  ((x).not_taken_xl_bb_size)
#define HashTable_user_stop_iaddr(x)    ((x).user_stop_iaddr)
#define HashTable_prolog_size(x)        ((x).size_of_prolog)
#define HashTable_epilog_size(x)        ((x).size_of_epilog)
#define HashTable_has_dynamic_data(x)   ((x).bb_has_dyn_data)
#define HashTable_ends_in_a_branch(x)   ((x).bb_ends_in_branch)
#define HashTable_generation(x)         ((x).generation)
#define HashTable_base_reg(x)           ((x).base_reg_used)
#define HashTable_scratch_reg(x)        ((x).scratch_reg_used)
#define HashTable_buffer_reg(x)         ((x).buffer_reg_used)
#define HashTable_overflow(x)           ((x).overflow)
#define HashTable_block_gpr_use(x)      ((x).use_def_vectors[0])
#define HashTable_block_fpr_use(x)      ((x).use_def_vectors[1])
#define HashTable_block_cr_use(x)       ((x).use_def_vectors[2])
#define HashTable_block_spr_use(x)      ((x).use_def_vectors[3])
#define HashTable_block_gpr_def(x)      ((x).use_def_vectors[4])
#define HashTable_block_fpr_def(x)      ((x).use_def_vectors[5])
#define HashTable_block_cr_def(x)       ((x).use_def_vectors[6])
#define HashTable_block_spr_def(x)      ((x).use_def_vectors[7])
#define HashTable_top_vis_gpr_use(x)    ((x).use_def_vectors[8])
#define HashTable_top_vis_fpr_use(x)    ((x).use_def_vectors[9])
#define HashTable_top_vis_cr_use(x)     ((x).use_def_vectors[10])
#define HashTable_top_vis_spr_use(x)    ((x).use_def_vectors[11])

/* Interface functions */

void initialize_hash_table();

hash_entry * find_hash_table_entry_for(unsigned * or_address, 
				       unsigned generation_flag
				       );

void dump_hash_table_entry_descriptions();

void dump_hash_table_entry(hash_entry * tptr);

void debug_dump_hash_tables();

void debug_dump_hash_table_entry(hash_entry * tptr, char * prefix_str);

void verify_hash_table();

#endif

