/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... proc_state.h
  FUNCTION ........ Provides the user process state space type 
      definition, plus some access macros for the space, and 
      some other, useful macros for handling user process state.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 
  CALLED-ROUTINES.. 
  CALLED-BY........ None
  INCLUDED-BY...... analyze_branch.c, aria_interface.c, aria_main.c,
     canned_multiple_code.c, canned_string_code.c, display_proc_state.c,
     do_svc_translation.c, do_translation.c, proc_state.c, 
     save_user_state.c, spec_test_driver.c, test_2buf_driver.c,
     test_driver.c, warn_proc_state.c
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

#ifndef PROC_STATE_H
#define PROC_STATE_H

#include <stddef.h>

#include "globtypes.h"

/****************************************************************** 
 *  This file defines the offsets used in the saved processor state 
 *  for the Aria execution simulation/trace generation package.    
 ******************************************************************/
 
/** Some helpful definitions **/
#define STACK_PTR          1
#define TOC_PTR            2

/** You should not move the valid flags from **/ 
/** the front of the proc state space        **/
/**  (The speculation-path lookup uses this  **/
/**   fact implicitly)                       **/
typedef struct proc_state_type_struct {
  unsigned char _aria_valid_segments[16*16];
  BASE_TYPE     _aria_link_reg;
  BASE_TYPE     _aria_count_reg;
  unsigned      _aria_condition_reg_64b;
  unsigned      _aria_condition_reg;
  BASE_TYPE     _aria_XER_reg;
  BASE_TYPE     _aria_not_taken_target;
  BASE_TYPE     _aria_proc_iaddr_offset;
  BASE_TYPE     _aria_GPRS[32];
  unsigned      _aria_fpscr_dummy;
  unsigned      _aria_fpscr;
  double        _aria_FPRS[32];

  unsigned  _aria_MQ_reg;
  unsigned  _aria_memory_reservation_flag;

  unsigned  _last_sc_id_number;
  unsigned  _last_sc_execution_time_high;
  unsigned  _last_sc_execution_time_low;

  unsigned  _scratch_save_area[32];
  double    _fpr_0_value;

  BASE_TYPE _aria_next_target;

  BASE_TYPE _raw_address_of_program_start;
  BASE_TYPE _raw_address_of_program_end;

} proc_state_type;

/* Define the layout of the processor state space */

#define  VALID_SEGMENT_FLAGS     (offsetof(proc_state_type, _aria_valid_segments[0]))
#define  LR_SAVE_OFFSET          (offsetof(proc_state_type, _aria_link_reg))
#define  CTR_SAVE_OFFSET         (offsetof(proc_state_type, _aria_count_reg))
#define  CR_SAVE_OFFSET          (offsetof(proc_state_type, _aria_condition_reg_64b))
#define  CR_VALUE_OFFSET         (offsetof(proc_state_type, _aria_condition_reg))
#define  XER_SAVE_OFFSET         (offsetof(proc_state_type, _aria_XER_reg))
#define  NEXT_TARGET_OFFSET      (offsetof(proc_state_type, _aria_next_target))
#define  NOT_TAKEN_OFFSET        (offsetof(proc_state_type, _aria_not_taken_target))

#define  IADDR_OFFSET            (offsetof(proc_state_type, _aria_proc_iaddr_offset))

#define  GPR_SAVE_OFFSET         (offsetof(proc_state_type, _aria_GPRS[0]))
#define  GPR_REG_SAVE_OFFSET(_y) (GPR_SAVE_OFFSET + (ARIA_GPR_SIZE*(_y)))
#define  FPSCR_64BIT_OFFSET      (offsetof(proc_state_type, _aria_fpscr_dummy))

#ifndef DO_64_BIT_COMPILE
 #define  FPSCR_VALUE_OFFSET      (offsetof(proc_state_type, _aria_fpscr))
#else
 #define  FPSCR_VALUE_OFFSET      (offsetof(proc_state_type, _aria_fpscr_dummy))
#endif

#define  FPR_SAVE_OFFSET         (offsetof(proc_state_type, _aria_FPRS[0]))
#define  FPR_REG_SAVE_OFFSET(_y) (FPR_SAVE_OFFSET + (ARIA_FPR_SIZE*(_y)))
#define  MQR_SAVE_OFFSET         (offsetof(proc_state_type, _aria_MQ_reg))

#define  LWARX_RESERVE_FLAG     (offsetof(proc_state_type, _aria_memory_reservation_flag))

#define  LAST_SC_ID_NUMBER      (offsetof(proc_state_type, _last_sc_id_number))
#define  LAST_SC_EXECUTION_TIME (offsetof(proc_state_type, _last_sc_execution_time_high))

#define  SCRATCH_SAVE_AREA      (offsetof(proc_state_type, _scratch_save_area))
#define  FPR_0_VALUE            (offsetof(proc_state_type, _fpr_0_value))

#define  TOTAL_PROC_STATE_SIZE  (sizeof(proc_state_type))

#define  SCRATCH_SAVE_SIZE      (sizeof(proc_state_type._scratch_save_area))
/* Some access macros for the proc_state.h, both those promoted */
/*  for use by the user (proc_state_accessors.h) and those      */
/*  expected to be more private (below).                        */
#include "proc_state_accessors.h"

#define proc_state_valid_segs(x)        ((x)->_aria_valid_segments)
#define proc_state_valid_seg(x,y)       ((x)->_aria_valid_segments[16*y])
#define proc_state_valid_segment(x,y,z) ((x)->_aria_valid_segments[16*y+z])
#define proc_state_NT_TARGET(x)         ((x)->_aria_not_taken_target)
#define proc_state_iaddr_offset(x)      ((x)->_aria_proc_iaddr_offset)
#define proc_state_GPRs(x)              ((x)->_aria_GPRS)

/* Some forward function declarations */
void copy_processor_state_from_to(proc_state_type * saved_proc_state_ptr, proc_state_type * speculative_saved_proc_state_ptr);

void equal_check_processor_state_from_to(proc_state_type * saved_proc_state_ptr, 
					 proc_state_type * speculative_saved_proc_state_ptr);

void warn_processor_state(proc_state_type * state_ptr, int show_floats);

void display_processor_state(proc_state_type * state_ptr, int show_floats, unsigned bb_execution_number);

void compare_processor_states(proc_state_type * state_ptr_1, proc_state_type * state_ptr_2, int show_floats);

void equal_check_processor_state_from_to(proc_state_type * from_ptr, proc_state_type * to_ptr);

#endif
