/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... io_msgs.c
  FUNCTION ........ Definitions of guarded io routines.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ aria_interface.c, aria_main.c, dump_description.c,
      proc_state.c, warn_proc_state.c
  INCLUDED-BY...... 
  MACROS-USED...... None
  INCLUDES......... <see below>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

#include <stdio.h>
#include <stdarg.h>

#include "global_variables.h"

void
warn(char * fmt, ...)
{
  va_list       args;
  
  va_start(args, fmt);
  vfprintf(mystderr, fmt, args);
  va_end(args);
}



void
warn_exec(char * fmt, ...)
{
  va_list       args;
  
  if ( (do_full_execute_dump) &&
       (output_after_dynamic_bb_execution <= global_bb_executions) && 
       (no_output_after_dynamic_bb_execution >= global_bb_executions) && 
       (output_after_static_bb_translation <= global_bb_index) &&
       (no_output_after_static_bb_translation >= global_bb_index) ) {
    va_start(args, fmt);
    vfprintf(mystderr, fmt, args);
    va_end(args);
  }
}



void
warn_trace(char * fmt, ...)
{
  va_list       args;

  if ( (do_trace_dump) &&
       (output_after_dynamic_bb_execution <= global_bb_executions) && 
       (no_output_after_dynamic_bb_execution >= global_bb_executions) && 
       (output_after_static_bb_translation <= global_bb_index) &&
       (no_output_after_static_bb_translation >= global_bb_index) ) {
    va_start(args, fmt);
    vfprintf(mystderr, fmt, args);
    va_end(args);
  }
}


void
warn_trace_buffer(char * fmt, ...)
{
  va_list       args;

  if ( (do_trace_buffer_dump) &&
       (output_after_dynamic_bb_execution <= global_bb_executions) && 
       (no_output_after_dynamic_bb_execution >= global_bb_executions) && 
       (output_after_static_bb_translation <= global_bb_index) &&
       (no_output_after_static_bb_translation >= global_bb_index) ) {
    va_start(args, fmt);
    vfprintf(mystderr, fmt, args);
    va_end(args);
  }
}


void
warn_xlate(char * fmt, ...)
{
  va_list       args;

  if ( (do_full_xlate_dump) &&
       (output_after_dynamic_bb_execution <= global_bb_executions) && 
       (no_output_after_dynamic_bb_execution >= global_bb_executions) && 
       (output_after_static_bb_translation <= global_bb_index) &&
       (no_output_after_static_bb_translation >= global_bb_index) ) {
    va_start(args, fmt);
    vfprintf(mystderr, fmt, args);
    va_end(args);
  }
}


void
warn_hash(char * fmt, ...)
{
  va_list       args;

  if ( (do_hash_tables_dump) &&
       (output_after_dynamic_bb_execution <= global_bb_executions) && 
       (no_output_after_dynamic_bb_execution >= global_bb_executions) && 
       (output_after_static_bb_translation <= global_bb_index) &&
       (no_output_after_static_bb_translation >= global_bb_index) ) {
    va_start(args, fmt);
    vfprintf(mystderr, fmt, args);
    va_end(args);
  }
}


void
warn_bb_regs(char * fmt, ...)
{
  va_list       args;

  if ( (do_bb_regs_use_def_dump) &&
       (output_after_dynamic_bb_execution <= global_bb_executions) && 
       (no_output_after_dynamic_bb_execution >= global_bb_executions) && 
       (output_after_static_bb_translation <= global_bb_index) &&
       (no_output_after_static_bb_translation >= global_bb_index) ) {
    va_start(args, fmt);
    vfprintf(mystderr, fmt, args);
    va_end(args);
  }
}


void
warn_proc_state(char * fmt, ...)
{
  va_list       args;

  if ( (do_proc_state_dump) && 
       (output_after_dynamic_bb_execution <= global_bb_executions) && 
       (no_output_after_dynamic_bb_execution >= global_bb_executions) && 
       (output_after_static_bb_translation <= global_bb_index) &&
       (no_output_after_static_bb_translation >= global_bb_index) ) {
    va_start(args, fmt);
    vfprintf(mystderr, fmt, args);
    va_end(args);
  }
}


void
warn_warnings(char * fmt, ...)
{
  va_list       args;
  
  if ( (do_show_runtime_warnings) ) {
    va_start(args, fmt);
    vfprintf(mystderr, fmt, args);
    va_end(args);
  }
}



