/*                                                                     
** COMPONENT_NAME: 
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... main.c
  FUNCTION ........ This is the main program.  It is used to 
     provide any initial setup required (by Aria).

  NOTES............ 
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. aria_main
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... <see below>
  INCLUDES......... <see below>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1998
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

#ifdef DEBUG 
#define IFDEBUG(_the_debug_line) _the_debug_line
#else
#define IFDEBUG(_the_debug_line) 
#endif

#include <stdio.h>

#include "ansi_stdlib.h"
#include "aix_errno.h"
#include "aix_mman.h"

#include "ansi_assert.h"

#include "globdefs.h"

#include "aria_main.h"


int read_base_input_flags(int argc, char * argv[]);

/** The MAIN Program **/

#ifdef AS_MAIN 
int
int aria_main(int        argc, 
	      char     * argv[], 
	      char     * envp, 
	      unsigned * stack_addr, 
	      void     * xlate_addr
	      ) 
{
  printf("[as_main] Entered the AS_MAIN aria_main with...\n");
  printf("[as_main]      int        argc       = %d\n"       , argc); 
  printf("[as_main]      char     * argv[]     = 0x%016LLx\n", argv);
  printf("[as_main]      char     * envp       = 0x%016LLx\n", envp);
  printf("[as_main]      unsigned * stack_addr = 0x%016LLx\n", stack_addr);
  printf("[as_main]      void     * xlate_addr = 0x%016LLx\n", xlate_addr);

  return 0;
}
#endif


/* Local Global Variables */
static int use_orig_user_stack = 1;

/** The MAIN Program **/

main( int argc, 
      char * argv[], 
      char * envp
      )
{
  caddr_t     xlate_addr;

  IFDEBUG( printf("[true_main] Scanning the input flags now...\n"); );
  IFDEBUG(fflush(stdout));
  if (read_base_input_flags(argc, argv) != 0) {
    fprintf(stderr, "Problem reading input parameters... exiting\n");
    return -1;
  }

  IFDEBUG(printf("[true_main] Mapping the memory (at 0x%016LLx for 0x%x bytes)\n", 
		 (void *)XLATE_START_SEGMENT, XLATE_SPACE_SIZE); 
	  );
  IFDEBUG(fflush(stdout));

#ifndef DO_64_BIT_COMPILE
  xlate_addr = mmap((caddr_t)XLATE_START_SEGMENT, XLATE_SPACE_SIZE,
		    PROT_READ|PROT_WRITE|PROT_EXEC,
		    MAP_ANONYMOUS|MAP_FIXED|MAP_PRIVATE,
		    -1, 0);
#else
  xlate_addr = mmap64((caddr_t)XLATE_START_SEGMENT, XLATE_SPACE_SIZE,
		    PROT_READ|PROT_WRITE|PROT_EXEC,
		    MAP_ANONYMOUS|MAP_FIXED|MAP_PRIVATE,
		    -1, 0);
#endif

  IFDEBUG(printf("[true_main] Allocated a memory-mapped segment...\n"); 
	  printf("[true_main] the segment is 0x%016LLx\n", xlate_addr); 
	  );
  if( xlate_addr != (caddr_t) XLATE_START_SEGMENT ) {
    fprintf( stderr, "%s: could not mmap seg 0x%08x, errno=%d\n",
	     argv[0], XLATE_START_SEGMENT, errno);
    return EXIT_FAILURE;
  }

  IFDEBUG( printf("[true_main] Calling the aria_main...\n"); );

  if (use_orig_user_stack == 1) {
    IFDEBUG( printf("[true_main] Using the original stack address...\n"); );
    return aria_main(argc, argv, envp, (unsigned *)0, xlate_addr);
  }
  else {
    IFDEBUG( printf("[true_main] Using the aria_allocated stack address...\n"); );
    return aria_main(argc, argv, envp, (unsigned *)ARIA_ALLOCATED_STACK_ADDRESS, xlate_addr);
  }  
}



/** Scan for input parameters that must be handled by this program **/

int read_base_input_flags(int argc, 
			  char * argv[]
			  ) 
{
  int i = 1;
  int ret_val = 0;

  while (i < argc) {
    if (strcmp(argv[i], "--WORKLOAD") == 0) {
      goto exit_this_while_loop;
    }
    else if (strcmp(argv[i], "--DRIVER") == 0) {
      goto exit_this_while_loop;
    }
    else if (strcmp(argv[i], "-orig_user_stack") == 0) {
      use_orig_user_stack = 1;
    }
    else if (strcmp(argv[i], "-allocate_user_stack") == 0) {
      use_orig_user_stack = 0;
    }
    i++;
  }
exit_this_while_loop:
  return ret_val;
}

