/*******************************************************************
  PROGRAM NAME..... print_usage

  FUNCTION ........ Dumps out information about the invocation of 
     aria, and the options it supports.

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

#include "ansi_stdio.h"
#include "globtypes.h"
#include "globdefs.h"

/* External Global Variables */
extern global_flags_type * the_global_flags;


/* The routine */

void
print_usage(int argc,
	    char * argv[]
	    )
{
  fprintf(mystderr, "usage: %s <options> --MODEL <model> --PROGRAM <xcoff>\n", argv[0]);
  fprintf(mystderr, "\n  Where the\n");
  fprintf(mystderr, "      <model> is the consumer of the aria instruction trace buffer data\n");
  fprintf(mystderr, "      <xcoff> is the user program to be traced, and \n");
  fprintf(mystderr, "      <options> can be any of:\n");

  fprintf(mystderr, "%30s : display all available output\n",    "-show_all");
  fprintf(mystderr, "%30s : display all translation output\n",  "-show_all_xlate");
  fprintf(mystderr, "%30s : display all execution output\n",    "-show_all_execute");
  fprintf(mystderr, "%30s : display a trace of program run\n",  "-show_run_trace");
  fprintf(mystderr, "%30s : display trace buffer after each execution\n",  "-show_trace_buffer");
  fprintf(mystderr, "%30s : display a trace of program use/def\n",    "-show_bb_reg_use_def");
  fprintf(mystderr, "%30s : display a trace of hash table status\n",  "-show_hash_table");
  fprintf(mystderr, "%30s : display a trace of hash table status\n",  "-show_no_hash_table");
  fprintf(mystderr, "%30s : display a trace of processor state\n",    "-show_proc_state");
  fprintf(mystderr, "%30s : include the FPR state in the proc state dump\n",    "-show_proc_floats");

  fprintf(mystderr, "%30s : indicate that Aria should print final run stats\n",    "-show_final_stats");
  fprintf(mystderr, "%30s : output a run marker every <n> executed BBs after <M> BBs\n",    "-update_on_dbbs <M> <N>");

  /** This is a secret option that is unsupported...?
    fprintf(mystderr, "%30s : check hash table every <n> executed BBs after <M> BBs\n",    "-verify_hash <N>");
    **/
  
  fprintf(mystderr, "%30s : delay output until <n> BBs translated\n",  "-output_after_sbb <n>");
  fprintf(mystderr, "%30s : delay output until <n> BBs executed\n",    "-output_after_dbb <n>");
  fprintf(mystderr, "%30s : stop output after <n> BBs translated\n",  "-no_output_after_sbb <n>");
  fprintf(mystderr, "%30s : stop output after <n> BBs executed\n",    "-no_output_after_dbb <n>");

  fprintf(mystderr, "%30s : use the original user stack (in segment 0x2) <default>\n",   "-orig_user_stack");
  fprintf(mystderr, "%30s : use the allocated user stack (in segment 0x4)\n",   "-allocate_user_stack");
  fprintf(mystderr, "%30s : align translation code on 2^n word boundaries\n",   "-align_xlated_code <n>");
  fprintf(mystderr, "%30s : limit the xlated BBs to <n> user instructions\n",   "-max_block_size <n>");
  fprintf(mystderr, "%30s : use the original user proc state initialization code\n", "-old_init_proc_state_code");

  fprintf(mystderr, "%30s : use the canned versions of lswx/stswx code <default>\n", "-canned_string_code");
  fprintf(mystderr, "%30s : don't use the canned versions of lswx/stswx code\n",     "-no_canned_string_code");

  fprintf(mystderr, "%30s : include translations for not-taken block executions <default>\n", "-speculation_support");
  fprintf(mystderr, "%30s : don't include translations for not-taken block executions \n",    "-no_speculation_support");

  fprintf(mystderr, "%30s : generate the dynamic trace buffer information <default>\n", "-dynamic_trace");
  fprintf(mystderr, "%30s : don't generate the dynamic trace buffer information\n",     "-no_dynamic_trace");

  fprintf(mystderr, "%30s : generate corrected user-code iaddresses <default>\n", "-iaddr_fixup");
  fprintf(mystderr, "%30s : don't generate corrected user-code iaddresses\n",     "-no_iaddr_fixup");

  fprintf(mystderr, "%30s : allow user code and model to be globally accessible\n",     "-allow_global_access");

  fprintf(mystderr, "%30s : indicates that the model uses floating-point registers <default>\n", "-fpr_use_in_model");
  fprintf(mystderr, "%30s : indicates that the model uses NO floating-point registers\n", "-no_fpr_use_in_model");

  fprintf(mystderr, "%30s : indicates the model needs not-taken address from Aria\n", "-NT_addr_reqd");
  fprintf(mystderr, "%30s : indicates the model needs no not-taken address from Aria <default>\n", "-no_NT_addr_reqd");

  fprintf(mystderr, "%30s : follow system calls to sc/svc instruction\n", "-show_sc_instr");
  fprintf(mystderr, "%30s : use safest form of sc/svc invocation <default>\n", "-safe_sc_code");

  fprintf(mystderr, "%30s : stop executing program at entry to exit routine <default>\n",  "-full_safe_exit");
  fprintf(mystderr, "%30s : stop executing program at exit system call\n",  "-less_safe_exit");

  fprintf(mystderr, "%30s : respresent an sc/svc instruction as a bclr in the trace <default>\n", "-report_bclr_for_sc");
  fprintf(mystderr, "%30s : do not respresent an sc/svc instruction as a bclr in the trace\n", "-report_svc_for_sc");
  fprintf(mystderr, "%30s : do not respresent an sc/svc instruction as a bclr in the trace\n", "-report_sc_for_sc");

  fprintf(mystderr, "%30s : get system call execution time profiles\n",    "-show_sc_times");
  
  fprintf(mystderr, "%30s : return raw iwords for illegal instructions speculated\n", "-raw_spec_code");

  fprintf(mystderr, "%30s : enables aria simulation of non-implemented instructions\n", "-simulate");
  fprintf(mystderr, "%30s : disables aria simulation of non-implemented instructions <default>\n", "-no_simulate");

  fprintf(mystderr, "%30s : indicates that Aria is executing on AIX 4.2\n", "-aix42");
  fprintf(mystderr, "%30s :  This invokes some special processing for AIX 4.2 systems.", " ");

  /** REMOVED FLAGS (Still allowed, but ignored)
    fprintf(mystderr, "%30s : indicates the level of the OS on which Aria is executing\n", "-oslevel <flag>");
    fprintf(mystderr, "%30s :   acceptable values are:\n", " ");
    fprintf(mystderr, "%30s :     AIX4.1 or AIX4.1.4 \n", " ");
    fprintf(mystderr, "%30s :     AIX4.1-L or AIX4.1.4-L\n", " ");
    fprintf(mystderr, "%30s :     AIX4.2 \n", " ");
    fprintf(mystderr, "%30s :     AIX3.2 or AIX3.2.5 (must compile Aria on a 3.2.5 machine) \n", " ");

    fprintf(mystderr, "%30s : indicates the instructon set architecture on which Aria is executing\n", "-arch <flag>");
    fprintf(mystderr, "%30s :   acceptable values are:\n", " ");
    fprintf(mystderr, "%30s :     POWER <default>\n", " ");
    fprintf(mystderr, "%30s :     PPC \n", " ");
    **/
  fprintf(mystderr, "%30s : indicates the instructon set architecture of the user program\n", "-uarch <flag>");
  fprintf(mystderr, "%30s :   acceptable values are:\n", " ");
  fprintf(mystderr, "%30s :     PPC <default>\n", " ");
  fprintf(mystderr, "%30s :     POWER \n", " ");

  fprintf(mystderr, "%30s : perform uniform sampling of the program execution\n", "-usample <c> <s>");
  fprintf(mystderr, "%30s     capture <c> instructions then skip <s> per sample\n", " ");

}

