/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... proc_state.c
  FUNCTION ........ Holds some code provided for users to display, 
      compare, etc. the processor state.  Mostly useful in debugging.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ the model that drives aria
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... <see below>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

#include <stdio.h>

#include "proc_state.h"

extern FILE * mystderr;

/*******************************************************************
  PROGRAM NAME..... display_processor_state
  FUNCTION ........ Displays the user process state associated with 
     the given user process state save space.  This is used 
     primarily in debugging.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ aria_interface.c, spec_test_driver.c, 
      test_2buf_driver.c, test_driver.c
  INCLUDED-BY...... None

  MACROS-USED...... None
  INCLUDES......... <see below>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
#include <stdio.h>

#include "proc_state.h"

void display_processor_state(proc_state_type * state_ptr,
			     int show_floats,
			     unsigned bb_execution_number
			     )
{
  fprintf(stderr, "\nThe processor state at BB execution %u contains:\n", bb_execution_number);
  fprintf(stderr, "Offset  Description   Value\n");
  fprintf(stderr, "------  -----------  --------\n");
  fprintf(stderr, "  %3d   %11s  %08x\n", LR_SAVE_OFFSET    , "link reg"    , proc_state_LR(state_ptr));
  fprintf(stderr, "  %3d   %11s  %08x\n", CTR_SAVE_OFFSET   , "count reg"   , proc_state_CTR(state_ptr));
  fprintf(stderr, "  %3d   %11s  %08x\n", CR_SAVE_OFFSET    , "cond reg"    , proc_state_CR(state_ptr));
  fprintf(stderr, "  %3d   %11s  %08x\n", XER_SAVE_OFFSET   , "XER reg"     , proc_state_XER(state_ptr));
  fprintf(stderr, "  %3d   %11s  %08x\n", NEXT_TARGET_OFFSET, "next pc"     , proc_state_TARGET(state_ptr));
  
  fprintf(stderr, "  %3d   %11s  %08x  %08x  %08x  %08x  %08x\n", GPR_SAVE_OFFSET   , "gpr  0- 4",
		  proc_state_GPR(state_ptr, 0), proc_state_GPR(state_ptr, 1), proc_state_GPR(state_ptr, 2), 
		  proc_state_GPR(state_ptr, 3), proc_state_GPR(state_ptr, 4)
		  );

  fprintf(stderr, "        %11s  %08x  %08x  %08x  %08x  %08x\n", "gpr  5- 9",
		  proc_state_GPR(state_ptr, 5), proc_state_GPR(state_ptr, 6), proc_state_GPR(state_ptr, 7), 
		  proc_state_GPR(state_ptr, 8), proc_state_GPR(state_ptr, 9)
		  );

  fprintf(stderr, "        %11s  %08x  %08x  %08x  %08x  %08x\n", "gpr 10-14",
		  proc_state_GPR(state_ptr, 10), proc_state_GPR(state_ptr, 11), proc_state_GPR(state_ptr, 12), 
		  proc_state_GPR(state_ptr, 13), proc_state_GPR(state_ptr, 14)
		  );

  fprintf(stderr, "        %11s  %08x  %08x  %08x  %08x  %08x\n", "gpr 15-19",
		  proc_state_GPR(state_ptr, 15), proc_state_GPR(state_ptr, 16), proc_state_GPR(state_ptr, 17), 
		  proc_state_GPR(state_ptr, 18), proc_state_GPR(state_ptr, 19)
		  );

  fprintf(stderr, "        %11s  %08x  %08x  %08x  %08x  %08x\n", "gpr 20-24",
		  proc_state_GPR(state_ptr, 20), proc_state_GPR(state_ptr, 21), proc_state_GPR(state_ptr, 22), 
		  proc_state_GPR(state_ptr, 23), proc_state_GPR(state_ptr, 24)
		  );

  fprintf(stderr, "        %11s  %08x  %08x  %08x  %08x  %08x\n", "gpr 25-29",
		  proc_state_GPR(state_ptr, 25), proc_state_GPR(state_ptr, 26), proc_state_GPR(state_ptr, 27), 
		  proc_state_GPR(state_ptr, 28), proc_state_GPR(state_ptr, 29)
		  );

  fprintf(stderr, "        %11s  %08x  %08x\n", "gpr 30-31",
		  proc_state_GPR(state_ptr, 30), proc_state_GPR(state_ptr, 31)
		  );

  /** Now output the Floating-Point State **/
  if (show_floats) {
    fprintf(stderr, "  %3d   %11s  %08x\n", FPSCR_VALUE_OFFSET    , "fpscr val", proc_state_FPSCR(state_ptr));

    fprintf(stderr, "  %3d   %11s  %lf  %lf  %lf\n", FPR_SAVE_OFFSET   , "fpr  0- 2",
		    proc_state_FPR(state_ptr, 0), proc_state_FPR(state_ptr, 1), proc_state_FPR(state_ptr, 2)
		    );
    fprintf(stderr, "        %11s  %lf  %lf  %lf\n",  "fpr  3- 5",
		    proc_state_FPR(state_ptr, 3), proc_state_FPR(state_ptr, 4), proc_state_FPR(state_ptr, 5) 
		    );
    fprintf(stderr, "        %11s  %lf  %lf  %lf\n",  "fpr  6- 8",
		    proc_state_FPR(state_ptr, 6), proc_state_FPR(state_ptr, 7), proc_state_FPR(state_ptr, 8) 
		    );
    fprintf(stderr, "        %11s  %lf  %lf  %lf\n",  "fpr  9-11",
		    proc_state_FPR(state_ptr, 9), proc_state_FPR(state_ptr, 10), proc_state_FPR(state_ptr, 11) 
		    );
    fprintf(stderr, "        %11s  %lf  %lf  %lf\n",  "fpr 12-14",
		    proc_state_FPR(state_ptr, 12), proc_state_FPR(state_ptr, 13), proc_state_FPR(state_ptr, 14) 
		    );
    fprintf(stderr, "        %11s  %lf  %lf  %lf\n",  "fpr 15-17",
		    proc_state_FPR(state_ptr, 15), proc_state_FPR(state_ptr, 16), proc_state_FPR(state_ptr, 17) 
		    );
    fprintf(stderr, "        %11s  %lf  %lf  %lf\n",  "fpr 18-20",
		    proc_state_FPR(state_ptr, 18), proc_state_FPR(state_ptr, 19), proc_state_FPR(state_ptr, 20) 
		    );
    fprintf(stderr, "        %11s  %lf  %lf  %lf\n",  "fpr 21-23",
		    proc_state_FPR(state_ptr, 21), proc_state_FPR(state_ptr, 22), proc_state_FPR(state_ptr, 23) 
		    );
    fprintf(stderr, "        %11s  %lf  %lf  %lf\n",  "fpr 24-26",
		    proc_state_FPR(state_ptr, 24), proc_state_FPR(state_ptr, 25), proc_state_FPR(state_ptr, 26) 
		    );
    fprintf(stderr, "        %11s  %lf  %lf  %lf\n",  "fpr 27-29",
		    proc_state_FPR(state_ptr, 27), proc_state_FPR(state_ptr, 28), proc_state_FPR(state_ptr, 29)
		    );
    fprintf(stderr, "        %11s  %lf  %lf\n",  "fpr 30-31",
		    proc_state_FPR(state_ptr, 30), proc_state_FPR(state_ptr, 31)
		    );
  }
}


void 
equal_check_processor_state_from_to(proc_state_type * from_ptr,
				    proc_state_type * to_ptr
				    )
{
  unsigned mismatch = 0;
  int i;
  int j;
  for (j = 0; j < 16; j++) {
    for (i = 0; i < 16; i++) {
      if ( proc_state_valid_segment(to_ptr, j, i) != proc_state_valid_segment(from_ptr, j, i) ) {
	mismatch = 1;
      }
    }
  }

  if ( proc_state_LR(to_ptr)  != proc_state_LR(from_ptr) ) {
      mismatch = 1;
  }  
  if ( proc_state_CTR(to_ptr) != proc_state_CTR(from_ptr) ) {
    mismatch = 1;
  }
  if ( proc_state_CR(to_ptr)  != proc_state_CR(from_ptr) ) {
    mismatch = 1;
  }
  if ( proc_state_XER(to_ptr) != proc_state_XER(from_ptr) ) {
    mismatch = 1;
  }
  if ( proc_state_TARGET(to_ptr) != proc_state_TARGET(from_ptr) ) {
    mismatch = 1;
  }
  if ( proc_state_NT_TARGET(to_ptr)  != proc_state_NT_TARGET(from_ptr) ) {
    mismatch = 1;
  }
  if ( proc_state_FPSCR(to_ptr) != proc_state_FPSCR(from_ptr) ) {
    mismatch = 1;
  }

  for (i = 0; i < 32; i++) {
    if ( proc_state_GPR(to_ptr,i) != proc_state_GPR(from_ptr,i) ) {
      mismatch = 1;
    }
    if ( proc_state_FPR(to_ptr,i) != proc_state_FPR(from_ptr,i) ) {
      mismatch = 1;
    }
  }

  if (mismatch) {
    fprintf(mystderr, "--------------------------------------------------------------------------------\n");
    fprintf(mystderr, " Comparison of the processor states:\n");
    compare_processor_states(from_ptr, to_ptr, 0);
    fprintf(mystderr, "\n--------------------------------------------------------------------------------\n");
    fprintf(mystderr, "--------------------------------------------------------------------------------\n");
  }
}

void compare_processor_states(proc_state_type * state_ptr_1,
			      proc_state_type * state_ptr_2,
			      int show_floats
			      )
{
  fprintf(mystderr, "\nComparing processor states:\n");
  fprintf(mystderr, "Offset  Description   Value1    Value2 \n");
  fprintf(mystderr, "------  -----------  --------  --------\n");
  fprintf(mystderr, "  %3d   %11s  %08x  %08x", LR_SAVE_OFFSET    , "link reg" , 
		  proc_state_LR(state_ptr_1), proc_state_LR(state_ptr_2));
  if (proc_state_LR(state_ptr_1) != proc_state_LR(state_ptr_2)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", CTR_SAVE_OFFSET   , "count reg", 
		  proc_state_CTR(state_ptr_1), proc_state_CTR(state_ptr_2));
  if (proc_state_CTR(state_ptr_1) != proc_state_CTR(state_ptr_2)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", CR_SAVE_OFFSET    , "cond reg" , 
		  proc_state_CR(state_ptr_1), proc_state_CR(state_ptr_2));
  if (proc_state_CR(state_ptr_1) != proc_state_CR(state_ptr_2)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", XER_SAVE_OFFSET   , "XER reg"  , 
		  proc_state_XER(state_ptr_1), proc_state_XER(state_ptr_2));
  if (proc_state_XER(state_ptr_1) != proc_state_XER(state_ptr_2)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", NEXT_TARGET_OFFSET, "next pc"  , 
		  proc_state_TARGET(state_ptr_1), proc_state_TARGET(state_ptr_2));
  if (proc_state_TARGET(state_ptr_1) != proc_state_TARGET(state_ptr_2)) { 
    fprintf(mystderr, " ****");
  }
  
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr  0",
       proc_state_GPR(state_ptr_1, 0), proc_state_GPR(state_ptr_2, 0));
  if (proc_state_GPR(state_ptr_1, 0) != proc_state_GPR(state_ptr_2, 0)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr  1",
       proc_state_GPR(state_ptr_1, 1), proc_state_GPR(state_ptr_2, 1));
  if (proc_state_GPR(state_ptr_1, 1) != proc_state_GPR(state_ptr_2, 1)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr  2",
       proc_state_GPR(state_ptr_1, 2), proc_state_GPR(state_ptr_2, 2));
  if (proc_state_GPR(state_ptr_1, 2) != proc_state_GPR(state_ptr_2, 2)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr  3",
       proc_state_GPR(state_ptr_1, 3), proc_state_GPR(state_ptr_2, 3));
  if (proc_state_GPR(state_ptr_1, 3) != proc_state_GPR(state_ptr_2, 3)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr  4",
       proc_state_GPR(state_ptr_1, 4), proc_state_GPR(state_ptr_2, 4));
  if (proc_state_GPR(state_ptr_1, 4) != proc_state_GPR(state_ptr_2, 4)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr  5",
       proc_state_GPR(state_ptr_1, 5), proc_state_GPR(state_ptr_2, 5));
  if (proc_state_GPR(state_ptr_1, 5) != proc_state_GPR(state_ptr_2, 5)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr  6",
       proc_state_GPR(state_ptr_1, 6), proc_state_GPR(state_ptr_2, 6));
  if (proc_state_GPR(state_ptr_1, 6) != proc_state_GPR(state_ptr_2, 6)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr  7",
       proc_state_GPR(state_ptr_1, 7), proc_state_GPR(state_ptr_2, 7));
  if (proc_state_GPR(state_ptr_1, 7) != proc_state_GPR(state_ptr_2, 7)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr  8",
       proc_state_GPR(state_ptr_1, 8), proc_state_GPR(state_ptr_2, 8));
  if (proc_state_GPR(state_ptr_1, 8) != proc_state_GPR(state_ptr_2, 8)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr  9",
       proc_state_GPR(state_ptr_1, 9), proc_state_GPR(state_ptr_2, 9));
  if (proc_state_GPR(state_ptr_1, 9) != proc_state_GPR(state_ptr_2, 9)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 10",
       proc_state_GPR(state_ptr_1, 10), proc_state_GPR(state_ptr_2, 10));
  if (proc_state_GPR(state_ptr_1, 10) != proc_state_GPR(state_ptr_2, 10)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 11",
       proc_state_GPR(state_ptr_1, 11), proc_state_GPR(state_ptr_2, 11));
  if (proc_state_GPR(state_ptr_1, 11) != proc_state_GPR(state_ptr_2, 11)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 12",
       proc_state_GPR(state_ptr_1, 12), proc_state_GPR(state_ptr_2, 12));
  if (proc_state_GPR(state_ptr_1, 12) != proc_state_GPR(state_ptr_2, 12)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 13",
       proc_state_GPR(state_ptr_1, 13), proc_state_GPR(state_ptr_2, 13));
  if (proc_state_GPR(state_ptr_1, 13) != proc_state_GPR(state_ptr_2, 13)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 14",
       proc_state_GPR(state_ptr_1, 14), proc_state_GPR(state_ptr_2, 14));
  if (proc_state_GPR(state_ptr_1, 14) != proc_state_GPR(state_ptr_2, 14)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 15",
       proc_state_GPR(state_ptr_1, 15), proc_state_GPR(state_ptr_2, 15));
  if (proc_state_GPR(state_ptr_1, 15) != proc_state_GPR(state_ptr_2, 15)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 16",
       proc_state_GPR(state_ptr_1, 16), proc_state_GPR(state_ptr_2, 16));
  if (proc_state_GPR(state_ptr_1, 16) != proc_state_GPR(state_ptr_2, 16)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 17",
       proc_state_GPR(state_ptr_1, 17), proc_state_GPR(state_ptr_2, 17));
  if (proc_state_GPR(state_ptr_1, 17) != proc_state_GPR(state_ptr_2, 17)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 18",
       proc_state_GPR(state_ptr_1, 18), proc_state_GPR(state_ptr_2, 18));
  if (proc_state_GPR(state_ptr_1, 18) != proc_state_GPR(state_ptr_2, 18)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 19",
       proc_state_GPR(state_ptr_1, 19), proc_state_GPR(state_ptr_2, 19));
  if (proc_state_GPR(state_ptr_1, 19) != proc_state_GPR(state_ptr_2, 19)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 20",
       proc_state_GPR(state_ptr_1, 20), proc_state_GPR(state_ptr_2, 20));
  if (proc_state_GPR(state_ptr_1, 20) != proc_state_GPR(state_ptr_2, 20)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 21",
       proc_state_GPR(state_ptr_1, 21), proc_state_GPR(state_ptr_2, 21));
  if (proc_state_GPR(state_ptr_1, 21) != proc_state_GPR(state_ptr_2, 21)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 22",
       proc_state_GPR(state_ptr_1, 22), proc_state_GPR(state_ptr_2, 22));
  if (proc_state_GPR(state_ptr_1, 22) != proc_state_GPR(state_ptr_2, 22)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 23",
       proc_state_GPR(state_ptr_1, 23), proc_state_GPR(state_ptr_2, 23));
  if (proc_state_GPR(state_ptr_1, 23) != proc_state_GPR(state_ptr_2, 23)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 24",
       proc_state_GPR(state_ptr_1, 24), proc_state_GPR(state_ptr_2, 24));
  if (proc_state_GPR(state_ptr_1, 24) != proc_state_GPR(state_ptr_2, 24)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 25",
       proc_state_GPR(state_ptr_1, 25), proc_state_GPR(state_ptr_2, 25));
  if (proc_state_GPR(state_ptr_1, 25) != proc_state_GPR(state_ptr_2, 25)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 26",
       proc_state_GPR(state_ptr_1, 26), proc_state_GPR(state_ptr_2, 26));
  if (proc_state_GPR(state_ptr_1, 26) != proc_state_GPR(state_ptr_2, 26)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 27",
       proc_state_GPR(state_ptr_1, 27), proc_state_GPR(state_ptr_2, 27));
  if (proc_state_GPR(state_ptr_1, 27) != proc_state_GPR(state_ptr_2, 27)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 28",
       proc_state_GPR(state_ptr_1, 28), proc_state_GPR(state_ptr_2, 28));
  if (proc_state_GPR(state_ptr_1, 28) != proc_state_GPR(state_ptr_2, 28)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 29",
       proc_state_GPR(state_ptr_1, 29), proc_state_GPR(state_ptr_2, 29));
  if (proc_state_GPR(state_ptr_1, 29) != proc_state_GPR(state_ptr_2, 29)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 30",
       proc_state_GPR(state_ptr_1, 30), proc_state_GPR(state_ptr_2, 30));
  if (proc_state_GPR(state_ptr_1, 30) != proc_state_GPR(state_ptr_2, 30)) { 
    fprintf(mystderr, " ****");
  }
  fprintf(mystderr, "\n  %3d   %11s  %08x  %08x", GPR_SAVE_OFFSET   , "gpr 31",
       proc_state_GPR(state_ptr_1, 31), proc_state_GPR(state_ptr_2, 31));
  if (proc_state_GPR(state_ptr_1, 31) != proc_state_GPR(state_ptr_2, 31)) { 
    fprintf(mystderr, " ****");
  }

  /** Now output the Floating-Point State **/
  if (show_floats) {
    fprintf(mystderr, "\n  %3d   %11s  %08x", FPSCR_VALUE_OFFSET    , "fpscr val", proc_state_FPSCR(state_ptr_1));
  }
}
