/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... restore_aria_state_code.c
  FUNCTION ........ A C code include file to import code that will 
    restore the Aria state after the state has been swapped to the 
    user process state, a translated basic block executed, and the 
    new user state saved off.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... canned_multiple_code.c, canned_string_code.c,
      do_svc_translation.c, do_translation.c
  MACROS-USED...... None
  INCLUDES......... <see below>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

/** Define some mnemonic names for registers **/
#define STACK_PTR   1
#define TOC_PTR     2

{
  int i;

  /* Load GPRbase_register with the address of our ARIA state saving space    */               
  
  /* Restore the Aria link register value (so we can BCLR to it)   */       
  /**
    if (the_full_block.touch_vectors[3] & OPC_LR_DEF_USE_MASK)
    **/
  {
    *next_xlate_address = LOAD_REG  (scratch_reg, ARIA_LR_OFFSET, base_register);    next_xlate_address++; 
    *next_xlate_address = OPC_ASM_MTSPR( 8, scratch_reg);                           next_xlate_address++; 
  }

  /* Restore the Aria count register value                         */       
  /**
    if (sprs & OPC_CTR_DEF_USE_MASK) {
    *next_xlate_address = LOAD_REG  (scratch_reg, ARIA_CTR_OFFSET, base_register);       next_xlate_address++; 
    *next_xlate_address = OPC_ASM_MTSPR( 9, scratch_reg);                      next_xlate_address++; 
    }
    **/

  /* Restore the Aria XER register value                           */
  /**
    if (sprs & OPC_XER_DEF_USE_MASK) {
    *next_xlate_address = LOAD_REG  (scratch_reg, ARIA_XER_OFFSET, base_register);       next_xlate_address++; 
    *next_xlate_address = OPC_ASM_MTSPR( 1, scratch_reg);                      next_xlate_address++; 
    }
    **/

  /* Restore the Aria condition register (in case we need it)      */       
  if (crs) {
    *next_xlate_address = LOAD_REG  (scratch_reg,  ARIA_CR_OFFSET, base_register);       next_xlate_address++; 
    *next_xlate_address = OPC_ASM_MTCRF(0xff, scratch_reg);                                 next_xlate_address++; 
  }


  /** Restore the saved value of the FPSCR **/
  /**
    if (sprs & OPC_FPSCR_DEF_USE_MASK) {
    *next_xlate_address = OPC_ASM_LFD  (fscratch_reg, ARIA_FPSCR_OFFSET, base_register);  next_xlate_address++;
    *next_xlate_address = OPC_ASM_MTFSF( 0x0f, 0x1e, fscratch_reg);              next_xlate_address++; 
    }
    **/

  /** Loop through the set of FPRs and see if we need to load them **/
  if (no_FP_registers_used_in_model_flag == 0) {
    for (i = 14; i < 32; i++) {
      if (fprs & (1 << i)) {
	*next_xlate_address = OPC_ASM_LFD  ( i, ARIA_FPR_OFFSET+ARIA_FPR_SIZE*i,  base_register);   next_xlate_address++;
      }
    }
  }

  /* Restore the general purpose registers in the aria state space   */            
  for (i = 13; i < 32; i++) {
    if (gprs & (1 << i)) {
      if (i != base_register) {
	/* Read in the register value to the aria save space */
	*next_xlate_address = LOAD_REG( i, ARIA_GPR_OFFSET+(i-13)*4, base_register);   next_xlate_address++;
      }
    }
  }

  *next_xlate_address = LOAD_REG  ( STACK_PTR, ARIA_STACK_OFFSET, base_register);    next_xlate_address++; 
  *next_xlate_address = LOAD_REG  ( 2, ARIA_TOC_OFFSET, base_register);              next_xlate_address++; 

  if (base_register > 12) {
    i = base_register;
    *next_xlate_address = LOAD_REG( i, ARIA_GPR_OFFSET+(i-13)*4, base_register);       next_xlate_address++;
  }
}

