/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... save_aria_state_code.c
  FUNCTION ........ A C code include file to import code that will 
    save the Aria state before the state is swapped to the user 
    process state so that a translated basic block can be executed.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... canned_multiple_code.c, canned_string_code.c,
      do_svc_translation.c, do_translation.c
  MACROS-USED...... None
  INCLUDES......... <see below>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

/** Define some mnemonic names for registers **/
#define STACK_PTR   1
#define TOC_PTR     2


{
  int i;

  /* Save the general purpose registers in the aria state space */  
  *next_xlate_address = STORE_REG  (STACK_PTR, ARIA_STACK_OFFSET, base_register);   next_xlate_address++; 
  *next_xlate_address = STORE_REG  (TOC_PTR  , ARIA_TOC_OFFSET, base_register);     next_xlate_address++; 

  for (i = 13; i < 32; i++) {
    if (gprs & (1 << i)) {
      /* Write out the register value to the aria save space */
      *next_xlate_address = STORE_REG( i, ARIA_GPR_OFFSET+(i-13)*4, base_register); next_xlate_address++;
    }
  }

  /* Save Aria's LR so we can later return to it               */  
  *next_xlate_address = OPC_ASM_MFSPR(scratch_reg,  8);                          next_xlate_address++; 
  *next_xlate_address = STORE_REG  (scratch_reg, ARIA_LR_OFFSET, base_register);   next_xlate_address++; 

  /* Save Aria's CTR */ 
  /**
    if (sprs & OPC_CTR_DEF_USE_MASK) {
    *next_xlate_address = OPC_ASM_MFSPR(scratch_reg,  9);                    next_xlate_address++; 
    *next_xlate_address = STORE_REG  (scratch_reg, ARIA_CTR_OFFSET, base_register);     next_xlate_address++; 
    }
    **/

  /* Save Aria's XER */ 
  /**
    if (sprs & OPC_XER_DEF_USE_MASK) {
    *next_xlate_address = OPC_ASM_MFSPR(scratch_reg,  1);                    next_xlate_address++; 
    *next_xlate_address = STORE_REG  (scratch_reg, ARIA_XER_OFFSET, base_register);     next_xlate_address++; 
    }
    **/

  /* Save Aria's CR so we can change the CR freely              */  
  if (crs) {
    *next_xlate_address = OPC_ASM_MFCR (scratch_reg);                            next_xlate_address++;  
    *next_xlate_address = STORE_REG  (scratch_reg,  ARIA_CR_OFFSET, base_register);     next_xlate_address++;
  }

  /** Loop through the set of FPRs and see if we need to store them **/
  if (no_FP_registers_used_in_model_flag == 0) {
    for (i = 14; i < 32; i++) {
      if (fprs & (1 << i)) {
	*next_xlate_address = OPC_ASM_STFD ( i, ARIA_FPR_OFFSET+ARIA_FPR_SIZE*i,  base_register);   next_xlate_address++;
      }
    }
  }
  /** Save the (user) value of the FPSCR **/
  /**
    if (sprs & OPC_FPSCR_DEF_USE_MASK) {
    *next_xlate_address = OPC_ASM_MFFS (fscratch_reg);                           next_xlate_address++; 
    *next_xlate_address = OPC_ASM_STFD (fscratch_reg, ARIA_FPSCR_OFFSET, base_register);  next_xlate_address++;
    }
    **/
}
