/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... insert_user_proc_state_save_code
  FUNCTION ........ Creates translation code to copy the initial 
     user process state into the provided user process state save
     space, to set the initial process state for Aria execution.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... the next available translation address
  CALLED-ROUTINES.. None
  CALLED-BY........ do_aria_setup (in aria_interface.c)
  INCLUDED-BY...... None
  MACROS-USED...... the opc_asm macros (see opc_asm.gen)
                    STACK_PTR, TOC_PTR (see proc_state.h)
  INCLUDES......... <see below>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
#include "aria_xlate_macros.h"
#include "aria_debug.h"

#include "proc_state.h"
#include "aria_state.h"
#include "opc.h"
#include "dump_description.h"

#include "global_variables.h"

/********************************************************************/
/** THE FOLLOWING ARE THE 32-BIT MACROS FOR SAVING USER STATE CODE **/
/********************************************************************/
#ifndef DO_64_BIT_COMPILE

#define ADDRESS_STORAGE_DECLARES \
  unsigned user_SP_hi = ((((unsigned)user_stackp) & 0xffff0000) >> 16); \
  unsigned user_SP_lo = (((unsigned)user_stackp) & 0x0000ffff)

/** Reset the user stack pointer to start at the user_stackp parameter value **/
#define RESET_USER_STACK_POINTER \
  *next_xlate_address = OPC_ASM_ADDIS(STACK_PTR,  0, user_SP_hi);           next_xlate_address++; \
  *next_xlate_address = OPC_ASM_ORI  (STACK_PTR, STACK_PTR, user_SP_lo);    next_xlate_address++;
 

/********************************************************************/
/** THE FOLLOWING ARE THE 64-BIT MACROS FOR SAVING USER STATE CODE **/
/********************************************************************/
#else

#define ADDRESS_STORAGE_DECLARES \
  unsigned addr_0_15   = (((BASE_TYPE)user_stackp) >> 48) & 0xffffLL; \
  unsigned addr_16_31  = (((BASE_TYPE)user_stackp) >> 32) & 0xffffLL; \
  unsigned addr_32_47  = (((BASE_TYPE)user_stackp) >> 16) & 0xffffLL; \
  unsigned addr_48_63  = (((BASE_TYPE)user_stackp)      ) & 0xffffLL

/** Reset the user stack pointer to start at the user_stackp parameter value **/
#define RESET_USER_STACK_POINTER \
  *next_xlate_address = OPC_ASM_ADDI  ( 6, 0, addr_0_15);    next_xlate_address++; \
  *next_xlate_address = OPC_ASM_RLDICL( 6, 6, 16, 0);        next_xlate_address++; \
  *next_xlate_address = OPC_ASM_ORI   ( 6, 6, addr_16_31);   next_xlate_address++; \
  *next_xlate_address = OPC_ASM_RLDICL( 6, 6, 16, 0);        next_xlate_address++; \
  *next_xlate_address = OPC_ASM_ORI   ( 6, 6, addr_32_47);   next_xlate_address++; \
  *next_xlate_address = OPC_ASM_RLDICL( 6, 6, 16, 0);        next_xlate_address++; \
  *next_xlate_address = OPC_ASM_ORI   ( 6, 6, addr_48_63);   next_xlate_address++; \
  *next_xlate_address = OPC_ASM_ORI   (STACK_PTR, 6, 0);     next_xlate_address++; 

#endif


/***************************************************************/
/** THE SUBROUTINE TO INSERT THE INITIAL SAVE USER STATE CODE **/
/***************************************************************/
unsigned *
insert_user_proc_state_save_code(proc_state_type * saved_proc_state_ptr,
				 unsigned * user_stackp
				 )
{
  unsigned * temp_ptr = next_xlate_address;

  unsigned base_register = 12;
  unsigned scratch_reg   = 11;

  ADDRESS_STORAGE_DECLARES;

  /** Save the user proc state address in the aria state **/ 
  /*   Note: the call to this code is:                    */
  /*       prog_ptr(argc, argv, envp, proc_state_address) */
  /*       so state address is in GPR 6.                  */
  *next_xlate_address = OPC_ASM_ADDIS(base_register,  0, ARIA_STATE_SAVE_HI_16);     next_xlate_address++; 
  *next_xlate_address = STORE_REG(6, PROC_STATE_OFFSET, base_register);     next_xlate_address++; 

  /* Save the Aria stack pointer */
  *next_xlate_address = STORE_REG(STACK_PTR, ARIA_STACK_OFFSET, base_register);   next_xlate_address++; 
  /**
   *next_xlate_address = STORE_REG(TOC_PTR  , ARIA_TOC_OFFSET,   base);   next_xlate_address++; 
   **/

  /** Reset the user stack pointer to start at the user_stackp parameter value **/
  RESET_USER_STACK_POINTER;
  
  {
    unsigned gpr_defs   = 0xffffffff;
    unsigned fpr_defs   = 0xffffffff;
    unsigned cr_touches = 0xffffffff;
    unsigned spr_uses   = 0xffffffff;
    unsigned spr_defs   = 0xffffffff;

    #include "bb_epilog_code.c"
  }

  /* Load GPRbase with the address of our ARIA state saving space    */               
  *next_xlate_address = OPC_ASM_ADDIS(base_register,  0, ARIA_STATE_SAVE_HI_16);        next_xlate_address++; 

  /** restore the ARIA stack and TOC ptrs **/
  *next_xlate_address = LOAD_REG( STACK_PTR, ARIA_STACK_OFFSET, base_register);    next_xlate_address++; 
  *next_xlate_address = LOAD_REG( 2, ARIA_TOC_OFFSET, base_register);              next_xlate_address++; 

  /** return to the ARIA main program **/
  *next_xlate_address = OPC_ASM_BCLR (0x14, 0);        next_xlate_address++;
  
  DUMP_DESCRIPTION(temp_ptr, next_xlate_address);

  /** Dump the translated code out of the data cache **/
  _sync_cache_range(temp_ptr, ((unsigned)next_xlate_address - (unsigned)temp_ptr));

  IFDEBUG(warn_xlate("[save_user_state]  Returning to caller...\n"); ); 
  return next_xlate_address;
}


