/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... trace_processor_state
  FUNCTION ........ Displays changes in the user process state 
     as a result of execution.  Useful only in debugging.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ aria_interface.c, spec_test_driver.c, 
      test_2buf_driver.c, test_driver.c
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... <see below>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... August, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
#include "proc_state.h"
#include "global_variables.h"
#include "io_msgs.h"

void trace_processor_state(proc_state_type * state_ptr,
			   proc_state_type * old_state_ptr,
			   int show_floats
			   )
{
  int ii;
  warn_trace("\nThe processor state changes due to BB execution %u:\n", global_bb_executions);

  if (proc_state_LR(state_ptr) != proc_state_LR(old_state_ptr)) {
    warn_trace("  %3d   %11s  %08x\n", LR_SAVE_OFFSET    , "link reg"    , proc_state_LR(state_ptr));
  }
  if (proc_state_CTR(state_ptr) != proc_state_CTR(old_state_ptr)) {
    warn_trace("  %3d   %11s  %08x\n", CTR_SAVE_OFFSET   , "count reg"   , proc_state_CTR(state_ptr));
  }
  if (proc_state_CR(state_ptr) != proc_state_CR(old_state_ptr)) {
    warn_trace("  %3d   %11s  %08x\n", CR_SAVE_OFFSET    , "cond reg"    , proc_state_CR(state_ptr));
  }
  if (proc_state_XER(state_ptr) != proc_state_XER(old_state_ptr)) {
    warn_trace("  %3d   %11s  %08x\n", XER_SAVE_OFFSET   , "XER reg"     , proc_state_XER(state_ptr));
  }
  if (proc_state_TARGET(state_ptr) != proc_state_TARGET(old_state_ptr)) {
    warn_trace("  %3d   %11s  %08x\n", NEXT_TARGET_OFFSET, "next pc"     , proc_state_TARGET(state_ptr));
  }

  for (ii = 0; ii < 32; ii++) {
    if (proc_state_GPR(state_ptr, ii) != proc_state_GPR(old_state_ptr, ii)) {
      char pr_str[20];
      sprintf(pr_str, "gpr %u", ii);
      warn_trace("  %3d   %11s  %08x\n", GPR_SAVE_OFFSET+ARIA_GPR_SIZE*ii, pr_str, proc_state_GPR(state_ptr, ii));
    }
  }
  /** Now output the Floating-Point State **/
  if (1 || show_floats) {
    if (proc_state_FPSCR(state_ptr) != proc_state_FPSCR(old_state_ptr)) {
      warn_trace("  %3d   %11s  %08x\n", FPSCR_VALUE_OFFSET, "fpscr val", proc_state_FPSCR(state_ptr));
    }
    for (ii = 0; ii < 32; ii++) {
      if (proc_state_FPR(state_ptr, ii) != proc_state_FPR(old_state_ptr, ii)) {
	char pr_str[20];
	sprintf(pr_str, "fpr %u", ii);
	warn_trace("  %3d   %11s  0x%08x%08x\n",  FPR_SAVE_OFFSET+ARIA_FPR_SIZE*ii, pr_str, 
		   *(unsigned *)(FPR_SAVE_OFFSET+ARIA_FPR_SIZE*ii), *(unsigned *)(FPR_SAVE_OFFSET+ARIA_FPR_SIZE*ii+4));
      }
    }
  }
}


