/*                                                                     
** COMPONENT_NAME: MET / Aria
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... warn_processor_state
  FUNCTION ........ Displays the user process state associated with 
     the given user process state save space.  This is used 
     primarily in debugging.

  NOTES............
  ASSUMPTIONS... None
  GLOBAL-VARS... None
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... None
  CALLED-ROUTINES.. None
  CALLED-BY........ aria_interface.c, spec_test_driver.c, 
      test_2buf_driver.c, test_driver.c
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... <see below>

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/
#include "proc_state.h"
#include "global_variables.h"
#include "io_msgs.h"

void warn_processor_state(proc_state_type * state_ptr,
			  int show_floats
			  )
{
  warn_proc_state("\nThe processor state at BB execution %d contains:\n", global_bb_executions);
  warn_proc_state("Offset  Description   Value\n");
  warn_proc_state("------  -----------  --------\n");
  warn_proc_state("  %3d   %11s  %08x\n", LR_SAVE_OFFSET    , "link reg"    , proc_state_LR(state_ptr));
  warn_proc_state("  %3d   %11s  %08x\n", CTR_SAVE_OFFSET   , "count reg"   , proc_state_CTR(state_ptr));
  warn_proc_state("  %3d   %11s  %08x\n", CR_SAVE_OFFSET    , "cond reg"    , proc_state_CR(state_ptr));
  warn_proc_state("  %3d   %11s  %08x\n", XER_SAVE_OFFSET   , "XER reg"     , proc_state_XER(state_ptr));
  warn_proc_state("  %3d   %11s  %08x\n", NEXT_TARGET_OFFSET, "next pc"     , proc_state_TARGET(state_ptr));
  
  warn_proc_state("  %3d   %11s  %08x  %08x  %08x  %08x  %08x\n", GPR_SAVE_OFFSET   , "gpr  0- 4",
		  proc_state_GPR(state_ptr, 0), proc_state_GPR(state_ptr, 1), proc_state_GPR(state_ptr, 2), 
		  proc_state_GPR(state_ptr, 3), proc_state_GPR(state_ptr, 4)
		  );

  warn_proc_state("        %11s  %08x  %08x  %08x  %08x  %08x\n", "gpr  5- 9",
		  proc_state_GPR(state_ptr, 5), proc_state_GPR(state_ptr, 6), proc_state_GPR(state_ptr, 7), 
		  proc_state_GPR(state_ptr, 8), proc_state_GPR(state_ptr, 9)
		  );

  warn_proc_state("        %11s  %08x  %08x  %08x  %08x  %08x\n", "gpr 10-14",
		  proc_state_GPR(state_ptr, 10), proc_state_GPR(state_ptr, 11), proc_state_GPR(state_ptr, 12), 
		  proc_state_GPR(state_ptr, 13), proc_state_GPR(state_ptr, 14)
		  );

  warn_proc_state("        %11s  %08x  %08x  %08x  %08x  %08x\n", "gpr 15-19",
		  proc_state_GPR(state_ptr, 15), proc_state_GPR(state_ptr, 16), proc_state_GPR(state_ptr, 17), 
		  proc_state_GPR(state_ptr, 18), proc_state_GPR(state_ptr, 19)
		  );

  warn_proc_state("        %11s  %08x  %08x  %08x  %08x  %08x\n", "gpr 20-24",
		  proc_state_GPR(state_ptr, 20), proc_state_GPR(state_ptr, 21), proc_state_GPR(state_ptr, 22), 
		  proc_state_GPR(state_ptr, 23), proc_state_GPR(state_ptr, 24)
		  );

  warn_proc_state("        %11s  %08x  %08x  %08x  %08x  %08x\n", "gpr 25-29",
		  proc_state_GPR(state_ptr, 25), proc_state_GPR(state_ptr, 26), proc_state_GPR(state_ptr, 27), 
		  proc_state_GPR(state_ptr, 28), proc_state_GPR(state_ptr, 29)
		  );

  warn_proc_state("        %11s  %08x  %08x\n", "gpr 30-31",
		  proc_state_GPR(state_ptr, 30), proc_state_GPR(state_ptr, 31)
		  );

  /** Now output the Floating-Point State **/
  if (show_floats) {
    int j;
    warn_proc_state("  %3d   %11s  %08x\n", FPSCR_VALUE_OFFSET    , "fpscr val", proc_state_FPSCR(state_ptr));
    for (j = 0; j < 32; j++) {
      double t1 = proc_state_FPR(state_ptr, j+0);
      unsigned * pt1 = (unsigned *)&t1;
      fprintf(mystderr, "%2d: %08x%08x  =  %f\n", j,
	      *pt1, *(pt1+1), t1);
    }

    /**
      warn_proc_state("  %3d   %11s  %lf  %lf  %lf\n",  FPR_SAVE_OFFSET, "fpr  0- 2",
		    proc_state_FPR(state_ptr, 0), proc_state_FPR(state_ptr, 1), proc_state_FPR(state_ptr, 2)
		    );
    warn_proc_state("        %11s  %lf  %lf  %lf\n",  "fpr  3- 5",
		    proc_state_FPR(state_ptr, 3), proc_state_FPR(state_ptr, 4), proc_state_FPR(state_ptr, 5) 
		    );
    warn_proc_state("        %11s  %lf  %lf  %lf\n",  "fpr  6- 8",
		    proc_state_FPR(state_ptr, 6), proc_state_FPR(state_ptr, 7), proc_state_FPR(state_ptr, 8) 
		    );
    warn_proc_state("        %11s  %lf  %lf  %lf\n",  "fpr  9-11",
		    proc_state_FPR(state_ptr, 9), proc_state_FPR(state_ptr, 10), proc_state_FPR(state_ptr, 11) 
		    );
    warn_proc_state("        %11s  %lf  %lf  %lf\n",  "fpr 12-14",
		    proc_state_FPR(state_ptr, 12), proc_state_FPR(state_ptr, 13), proc_state_FPR(state_ptr, 14) 
		    );
    warn_proc_state("        %11s  %lf  %lf  %lf\n",  "fpr 15-17",
		    proc_state_FPR(state_ptr, 15), proc_state_FPR(state_ptr, 16), proc_state_FPR(state_ptr, 17) 
		    );
    warn_proc_state("        %11s  %lf  %lf  %lf\n",  "fpr 18-20",
		    proc_state_FPR(state_ptr, 18), proc_state_FPR(state_ptr, 19), proc_state_FPR(state_ptr, 20) 
		    );
    warn_proc_state("        %11s  %lf  %lf  %lf\n",  "fpr 21-23",
		    proc_state_FPR(state_ptr, 21), proc_state_FPR(state_ptr, 22), proc_state_FPR(state_ptr, 23) 
		    );
    warn_proc_state("        %11s  %lf  %lf  %lf\n",  "fpr 24-26",
		    proc_state_FPR(state_ptr, 24), proc_state_FPR(state_ptr, 25), proc_state_FPR(state_ptr, 26) 
		    );
    warn_proc_state("        %11s  %lf  %lf  %lf\n",  "fpr 27-29",
		    proc_state_FPR(state_ptr, 27), proc_state_FPR(state_ptr, 28), proc_state_FPR(state_ptr, 29)
		    );
    warn_proc_state("        %11s  %lf  %lf\n",  "fpr 30-31",
		    proc_state_FPR(state_ptr, 30), proc_state_FPR(state_ptr, 31)
		    );
    **/
  }
}



