/*                                                                     
** COMPONENT_NAME: MET / Aria  for watt
**                                                                      
** Licensed Materials - Property of IBM                                  
**                                                                     
** IBM Confidential (IBM Confidential-Restricted when combined          
**                   with aggregated modules for this product)          
**                                                                     
**                                                                     
** OCO Source Materials                                                
**                                                                     
**                                                                     
** GA-0100 (C) Copyright IBM Corp. 1997                                 
**                                                                     
** US Government Users Restricted RIghts - Use, duplication or         
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.   
**                                                                     
** The source code for this program is not published or otherwise      
** divested of its trade secrets, irrespective of what has been        
** deposited within the U.S. Copyright Office.                         
**                                                                     
*/


 
/*******************************************************************
  PROGRAM NAME..... watt_ariareader_init 

  FUNCTION ........ Provides the set of routines comprising the 
                    local aria main program, for use by watt.

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 
  CALLED-ROUTINES.. 
  CALLED-BY........ None
  INCLUDED-BY...... 
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1998
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

#define TRC() (fprintf(mystderr, "@ %s:%u\n", __FILE__, __LINE__))

#include <limits.h>
#include <posix_stat.h>

#include "aix_systemcfg.h"

#include "aix_a.out.h"
#include "aix_errno.h"
#include "aix_ldr.h"
#include "aix_mman.h"
#include "posix_fcntl.h"
#include "ansi_stdio.h"
#include "ansi_stdlib.h"

#include "ansi_assert.h"

#include "aria_state.h"

#include "io_msgs.h"

#include "opc.h"

#include "proc_state.h"

#include "globtypes.h"
#include "globdefs.h"

#include "aria_main.h"

#include "opc_is_implemented_types.h"

#include "watt.h"

/** MACROS **/
#include "aria_debug.h"

/** Types **/
#include "ariareader_types.h"

/** Forward function declarations **/

extern void print_usage(int argc, char * argv[]);

/** (Kernel-code independent) Global Variables **/

global_flags_type * the_global_flags = (global_flags_type *)(GLOBAL_FLAGS_STATE_SPACE_START);

unsigned  * prog_start;
unsigned  * xlate_mem_start;
unsigned    user_program_start_address;

aria_func_desc * prog_desc;
func_ptr    prog_ptr;

int     user_argc;
char ** user_argv;
char  * user_envp = NULL;

unsigned * user_stack_address_value; 

proc_state_type * curr_process_state_space_ptr;
proc_state_type * last_process_state_space_position;

unsigned global_do_show_system_xlation;
unsigned global_do_show_system_pre_run_proc_state;
unsigned global_do_show_system_proc_state;
unsigned do_interactive_aria_system_run;

unsigned show_run_update_every;
unsigned show_run_update_after;

unsigned global_do_verify_hash_after;
unsigned global_do_verify_hash_every;

/*******************************************************************
  PROGRAM NAME..... aria_main

  FUNCTION ........ Serves as teh local main program, setting up 
     essential information, parsing the input flags, and so forth.
     It then invokes the specified model program, returning any 
     value returned by that program.

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... return value determined by called model program.
  CALLED-ROUTINES.. 
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

int
watt_ariareader_init (watt_workload_handle wh, 
		      int        trace_no, 
		      char *     tracename, 
		      int        argc, 
		      char *     argv[],
		      unsigned * stack_address,
		      caddr_t    xlate_addr
		      )
{
  int		 prog_fd;
  char *	 prog_name;
  struct filehdr prog_hdr;
  AOUTHDR	 prog_auxhdr;
  unsigned	 prog_textsize;
  unsigned *	 prog_end;

  func_ptr       model_ptr;
  char *	 model_name;

  int		 n;

  int user_arg_posn = 0;
  int model_arg_posn = 0;

  int my_stderr_fdes = dup(2);

  int return_value = 0; 
  int i;

  struct stat file_status;

  /* Set up the aria extension space for the workload handle */
  aria_extension_type * arex = (aria_extension_type *)malloc(sizeof(aria_extension_type));
  wh->wh_method_specific = (void *)arex;

  /* Get a new (aria-only) copy of the stderr path */
  mystderr    = fdopen(my_stderr_fdes, "w");

  /** Copy the stack address to global variable space **/

  /** If the actual stack address is in 0x2f000000 region,   **/
  /**  the speculation code cannot "see" it, so move it down **/
  /**  a little, otherwise, keep it unchanged                **/
  if (((unsigned)stack_address & 0xff000000) == 0x2f000000) {
    /* Drop the user stack into segment 0x2e000000 */
    user_stack_address_value = (unsigned *) (0x2e000000 | ((unsigned)stack_address & 0x00ffffff));
  }
  else {
    /* Leave the user stack unchanged */
    user_stack_address_value = stack_address;
  }

  /** Set up the global variables' initial values **/
  the_exit_code_value      = 0x17;
  the_exit_routine_address = 0x0;
  do_safe_exit_flag = 1;

  /** Set up to return a NOP in the trace for illegal instructions **/
  global_return_nop_in_trace_for_illegal_speculatives = 1;

  /** Default to no internal (Aria) simulation mode **/
  global_do_aria_simulation = 0;

  /* Allow globally-accessible xcoff/timer files */
  allow_global_access_permissions = 0;

  /* Do iaddress fixing (to "match" xcoff offsets) */
  do_iaddr_fixup = 1;

  /* The timer model uses no floating-point registers (don't save in Aria state) */
  no_FP_registers_used_in_model_flag  = 0;
  
  /* The timer model does not need Aria to calculate a not-taken address for each block */
  not_taken_address_required_flag  = 0;

  /* Don't replace the SVC instructions with BCLR instructions to retain trace semantics consistency */
  turn_svc_into_a_bclr_instruction = 0;
  global_use_safe_svc_code = 0;
  global_get_system_call_execution_times = 0;

  /* Set the user program ISA to be the PowerPC ISA */
  the_user_program_architecture = ARIA_PPC_ISA;

  /* Reset the Arai statistics counters */
  global_bb_index      = 0;
  global_bb_executions = 0;
  global_max_size_ubb_counter  = 0;
  global_user_stop_condition_counter = 0; 
  global_addressibility_counter = 0;

  /* Aria should do not-taken (speculative) translations */
  do_not_taken_path_translations = 1;

  /* Use the Aria canned string instruction code */
  do_canned_string_code = 1;

  /* Generate the dynamic trace buffer information */
  do_dynamic_trace_generation = 1;

  /* Use the older, original user process state save space initialization code option. */
  /*  Note that we DO NOT use this code by default any more                            */
  global_do_old_initial_proc_state_code = 0;

  /** OUTPUT flags: **/

  /* Give no periodic run update messages */
  show_run_update_every = 0;
  show_run_update_after = INT_MAX;

  /* Do verification checks on hash table each cycle */
  global_do_verify_hash_every = 0;
  global_do_verify_hash_after = INT_MAX;

  /* Turn off all output */
  do_no_output_during_runs  = 1;

  /* Set ouput limiters to block all output */
  output_after_static_bb_translation    = INT_MAX;
  no_output_after_static_bb_translation = INT_MAX;

  output_after_dynamic_bb_execution    = INT_MAX;
  no_output_after_dynamic_bb_execution = INT_MAX;

  /* Set all output info flags to OFF */
  do_full_xlate_dump       = 0;
  do_full_execute_dump     = 0;
  do_trace_dump            = 0;
  do_trace_buffer_dump     = 0;
  do_bb_regs_use_def_dump  = 0;
  do_hash_tables_dump      = 0;
  do_proc_state_dump       = 0;
  show_proc_float_state    = 0;
  show_final_aria_statistics = 0;

  /* Kernel mode inform globals */
  global_do_show_system_xlation = 0;
  global_do_show_system_pre_run_proc_state = 0;
  global_do_show_system_proc_state = 0;
  do_interactive_aria_system_run = 0;

  /* Default alignment is on a word-boundary */
  the_xlated_code_alignment   = 4; 

  /* Set max block size limit to default to 32 user instructions */
  global_max_block_size_limit = 32;

  /* Set to use limit for all user basic blocks */
  use_block_limit_after_static_bb_translation     = 0;
  use_block_unlimited_after_static_bb_translation = INT_MAX;

  /* Set the global pointers into the process saved state memory heap space */
  last_process_state_space_position = (proc_state_type *)PROC_STATE_LAST_POSN;
  curr_process_state_space_ptr      = (proc_state_type *)PROC_STATE_FIRST_POSN;

  /* Set the underlying machine ISA type from the  */
  /*  system configuration information             */
  if ( __power_pc() ) {
    the_machine_isa_type = OPC_PPC_ISA;
    IFDEBUG( printf("Setting the_machine_isa_type to OPC_PPC_ISA = 0x%02x\n", OPC_PPC_ISA); );
  }
  else if ( __power_rs() ) {
    if (__power_rs1()) {
      the_machine_isa_type = OPC_POWER_ISA;
      IFDEBUG( printf("Setting the_machine_isa_type to OPC_POWER_ISA = 0x%02x\n", OPC_POWER_ISA); );
    }
    else if (__power_rsc()) {
      the_machine_isa_type = OPC_POWER_ISA;
      IFDEBUG( printf("Setting the_machine_isa_type to OPC_POWER_ISA = 0x%02x\n", OPC_POWER_ISA); );
    }
    else if (__power_rs2()) {
      the_machine_isa_type = OPC_POWER2_ISA;
      IFDEBUG( printf("Setting the_machine_isa_type to OPC_POWER2_ISA = 0x%02x\n", OPC_POWER2_ISA); );
    }
  }

  /***************************************************/
  /** Start analysis of the user input command line **/
  /***************************************************/

  /** Read in the input (command line) switch values **/
  if (read_watt_aria_input_flags(argc, argv, &user_arg_posn, &user_argc) != 0) {
    fprintf(mystderr, "Problem reading the command-line input... Exiting\n");
    print_usage(argc, argv);
    return EXIT_FAILURE;
  }

  prog_name = wh->wh_workload_name;
  user_argv = &argv[user_arg_posn];

  IFDEBUG ({ int jj;
            printf("Back from read_watt_aria_input_flags; user_arg_posn = %d, user_argc = %d\n", user_arg_posn, user_argc);
            for (jj = 0; jj < user_argc; jj++) {
	      fprintf(mystderr, "user_argv[%d] = \"%s\"\n", jj, user_argv[jj]);
	    }
           }
	   );

  /** Set up the hash table **/
  initialize_hash_table();

  /** Set up the disassembler **/
  opc_init();

  IFDEBUG( fprintf(mystderr, "Trying to check out program \"%s\"\n", prog_name); );

  /** Check that these files do not have global access permissions; **/
  /**  NOTE: global permissions allow the file to be loaded into    **/
  /**   segment 0xd, and do not unload the program nicely at exit.  **/
  if (allow_global_access_permissions == 0) {
    /** Check the status of the program file (to see if globally read enabled) **/
    if (stat(prog_name, &file_status) == 0) {
      if ( (file_status.st_mode & S_IROTH) ) {
	fprintf(mystderr, "ARIA: ERROR: You should not run Aria with a globally-read--enabled xcoff file (%s)\n", prog_name);
	return EXIT_FAILURE;
      }
    }
    else {
      fprintf(mystderr, "ARIA: ERROR: Cannot check the status of xcoff file %s, errno=%d \n", prog_name, errno);
      return EXIT_FAILURE;
    }
  }
    
  /*****************************************/
  /** Get the extent of the user program  **/
  /*****************************************/
  prog_fd = open( prog_name, O_RDONLY, 0);
  if( prog_fd == -1 ) {
    fprintf( mystderr, "[aria_reader_init]: could not open program %s\n", prog_name);
    return EXIT_FAILURE;
  }
  
  n = read( prog_fd, &prog_hdr, sizeof( prog_hdr) );
  if( n != sizeof(prog_hdr) ) {
    fprintf( mystderr, "[aria_reader_init]: could not read header for %s\n", prog_name);
    return EXIT_FAILURE;
  }
  
  n = read( prog_fd, &prog_auxhdr, sizeof( prog_auxhdr) );
  if( n != sizeof(prog_auxhdr) ) {
    fprintf( mystderr, "[aria_reader_init]: could not read auxiliary header for %s\n", prog_name);
    return EXIT_FAILURE;
  }

  /** ASSUME that the text_start is the starting address for this program **/
  /**  NOTE: SHOULD CHANGE THIS TO LOAD AND DECODE THE PROGRAM ENTRY DESCRIPTION **/
  /**  NOTE2: If the text_start is in segment 0x0, then we have a problem, so we     **/
  /**           alter the text_start to be in segment 0x1, and following code works. **/
  if ((prog_auxhdr.text_start & 0xf0000000) == 0x00000000) {
    user_program_start_address = 0x10000000 | (prog_auxhdr.text_start & 0x0fffffff);
  }
  else {
    user_program_start_address = prog_auxhdr.text_start;
  }
  
  prog_textsize = prog_auxhdr.tsize / 4;
  close( prog_fd );
  

  IFDEBUG( fprintf(mystderr, "Program is fine... calculating values...\n"); );

  /** Do the load of the program **/
  prog_ptr = load( prog_name, 1, 0);

  prog_desc  = (aria_func_desc *)prog_ptr;
  prog_start = pc_func_desc(prog_desc);
  prog_end   = prog_start + prog_textsize;
  
  /* calculate the global_iaddr_offset value */
  global_user_program_iaddr_mask = ((unsigned)prog_start) & 0xf0000000;
  global_user_iaddr_offset = (unsigned)prog_start - user_program_start_address;

  IFDEBUG( warn("The global_user_iaddr_offset = 0x%08x\n", global_user_iaddr_offset); );

  /** Set the value of some global variables, including **/
  /*    the start of the translation memory              */
  /*    the next translation memory address              */
  /*    the global user line index number                */
  /*    the global translated line index number          */
  /*    the global number of basic blocks translated     */
  xlate_mem_start = (unsigned *)xlate_addr;
  next_xlate_address = xlate_mem_start;

  /** Set the _exit system call value automatically **/
  {
    /* Get the function descriptor for the _exit routine */
    /*  (which does the system call)                     */
    unsigned * taddrp = (unsigned *)_exit;

    /* Use the TOC value (GPR 2) as the exit code value */
    the_exit_code_value = *(taddrp+1);

    /* Print debug information if desired */
    IFDEBUG (fprintf(mystderr, "the _exit descriptor is at 0x%08x\n", taddrp);
	     fprintf(mystderr, "       _exit address 0x%08x\n",*taddrp);
	     fprintf(mystderr, "       _exit TOC     0x%08x\n",*(taddrp+1));
	     fprintf(mystderr, "       _exit other   0x%08x\n",*(taddrp+2));
	     fprintf(mystderr, "       the_exit_code_value = 0x%08x\n", the_exit_code_value);
	     );
  }

  /** Set the exit routine address (automatically) **/
  {
    /* Get the function descriptor for the exit routine */
    unsigned * taddrp = (unsigned *)exit;

    /* Use the TOC value (GPR 2) as the exit code value */
    the_exit_routine_address = *taddrp;

    /* Print debug information if desired */
    IFDEBUG(fprintf(mystderr, "the exit routine is at: 0x%08x\n", taddrp);
	    fprintf(mystderr, "       exit address 0x%08x\n",*taddrp);
	    fprintf(mystderr, "       exit TOC     0x%08x\n",*(taddrp+1));
	    fprintf(mystderr, "       exit other   0x%08x\n",*(taddrp+2));
	    );
  }

  /** If the user wants timings of the system calls, **/
  /**  set the address of the read_real_time routine **/
  if (global_get_system_call_execution_times == 1) {
    /* Get the address of the function descriptor save space */
    unsigned * Aaddr  = (unsigned *) (ARIA_STATE_SAVE_SPACE + ARIA_READ_REAL_TIME);
    /* get the read_real_time routine function descriptor */
    unsigned * taddrp = (unsigned *)read_real_time;

    /* Save (copy) the function descriptor values */
    *Aaddr  =*taddrp;
    *(Aaddr+1) = *(taddrp+1);
    *(Aaddr+2) = *(taddrp+2);

    IFDEBUG(fprintf(mystderr, "Saving read_real_time descriptor to 0x%08x\n", Aaddr);
	    fprintf(mystderr, "       read_real_time address 0x%08x to 0x%08x = 0x%08x\n",*taddrp, Aaddr, *Aaddr);
	    fprintf(mystderr, "       read_real_time TOC     0x%08x to 0x%08x = 0x%08x\n",*(taddrp+1), (Aaddr+1), *(Aaddr+1));
	    fprintf(mystderr, "       read_real_time other   0x%08x to 0x%08x = 0x%08x\n",*(taddrp+2), (Aaddr+2), *(Aaddr+2));
	    fflush(mystderr);
	    );

    /* Pre-set the saved null time to all zeros */
    Aaddr      = (unsigned *) (ARIA_STATE_SAVE_SPACE + ARIA_SAVE_NULL_TIME);
    *Aaddr     = 0x00000000;
    *(Aaddr+1) = 0x00000000;
    *(Aaddr+2) = 0x00000000;
  }    

  /** Invoke the user model (driver) program;
   ** NOTE: we have to zap r30 and r31 because of some eccentricity
   ** in the startup code (DONT ASK!!!) (look at __start())
   **/
  zap3031();

  /* Set up the aria extension space */
  arex->proc_space = aria_get_new_process_state_space();
  arex->bb_address = do_aria_setup(arex->proc_space);
  arex->trace_size = 0;
  arex->trace_posn = 0;
  arex->trace_buffer = NULL;
  arex->data = (watt_data_handle)malloc(sizeof(workload_data));
  
  /* Set the workload handle initial address */
  wh->wh_initial_address = (unsigned)arex->bb_address;

  return return_value;
}






/*******************************************************************
  PROGRAM NAME..... read_watt_aria_input_flags

  FUNCTION ........ Parses the input command-line arguments, and 
     sets (global) variables to indicate the flag conditions that 
     are requested by the user.

  NOTES............
  ASSUMPTIONS... 
  GLOBAL-VARS... 
  RESTRICTIONS.. None
  DEVIATIONS.... None
  PROGRAM TYPE..... C
  RETURN CODES..... 0 = good, other = error in parsing
  CALLED-ROUTINES.. None
  CALLED-BY........ None
  INCLUDED-BY...... None
  MACROS-USED...... None
  INCLUDES......... None

  AUTHOR........... John-David Wellman, IBM Yorktown
  CREATION DATE.... April, 1997
  MODIFIED......... Who       Date      What
  
  ********************************************************************/

int
read_watt_aria_input_flags(int    argc, 
			   char * argv[],
			   int  * user_arg_posn,
			   int  * user_argc
			   ) 
{
  int ret_val = 0;
  int flag_type = 0;
  int user_argc_count  = 0;

  int i = 1;
  IFDEBUG( fprintf(mystderr, "Checking the input parameters... (there are %d of them)\n", argc);
	   for (i = 0; i < argc; i++) {
	     fprintf(mystderr, "[aria_read_parms] argc[%d] = \"%s\"\n", i, argv[i]);
	   }
	   );
  i = 1;
  while (i < argc) {
    IFDEBUG( fprintf(mystderr, "[Aria] Considering parameter argv[%d] = \"%s\" (flag = %d)\n", i, argv[i], flag_type); );
    switch (flag_type) {
    case 0:
      if (strcmp(argv[i], "--ARIAparms") != 0) {
	IFDEBUG( fprintf(mystderr, "[Aria] Skipping parameter \"%s\"\n", argv[i]); );
      }
      else {
	flag_type++;
	IFDEBUG( fprintf(mystderr, "[Aria] Skipping parameter \"%s\" and setting flag to %d\n", argv[i], flag_type); );
      }
      break;
      
    case 2:
      IFDEBUG( fprintf(mystderr, "[Aria] Found program parameter \"%s\"\n", argv[i]); );
      user_argc_count++;
      break;
      
    case 1:
      IFDEBUG( fprintf(mystderr, "[Aria] Found Aria parameter \"%s\"\n", argv[i]); );
      if (strcmp(argv[i], "--PROGparms") == 0) {
	IFDEBUG( fprintf(mystderr, "[Aria] user_arg_posn = %d (argv[%d] = \"%s\") setting flag type to %d\n", i, i, argv[i], flag_type); );
	*user_arg_posn = i;
	flag_type++;
	user_argc_count = 1;
      }
      else if (strcmp(argv[i], "-show_all") == 0) {
	IFDEBUG( fprintf(mystderr, "[Aria] Setting Aria parameter \"%s\"\n", "-show_all"); );
	do_full_xlate_dump      = 1;
	do_full_execute_dump    = 1;
	do_hash_tables_dump     = 1;
	do_proc_state_dump      = 1;
	do_bb_regs_use_def_dump = 1;
	do_trace_dump           = 1;
	do_trace_buffer_dump    = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }
      else if (strcmp(argv[i], "-show_all_xlate") == 0) {
	IFDEBUG( fprintf(mystderr, "[Aria] Setting Aria parameter \"%s\"\n", "-show_all_xlate"); );
	do_full_xlate_dump      = 1;
	do_hash_tables_dump     = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }
      else if (strcmp(argv[i], "-show_all_execute") == 0) {
	IFDEBUG( fprintf(mystderr, "[Aria] Setting Aria parameter \"%s\"\n", "-show_all_execute"); );
	do_full_execute_dump    = 1;
	do_proc_state_dump      = 1;
	do_bb_regs_use_def_dump = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }
      else if (strcmp(argv[i], "-show_run_trace") == 0) {
	IFDEBUG( fprintf(mystderr, "[Aria] Setting Aria parameter \"%s\"\n", "-show_run_trace"); );
	do_trace_dump            = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }
      else if (strcmp(argv[i], "-show_trace_buffer") == 0) {
	IFDEBUG( fprintf(mystderr, "[Aria] Setting Aria parameter \"%s\"\n", "-show_trace_buffer"); );
	do_trace_buffer_dump     = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }
      else if (strcmp(argv[i], "-show_bb_reg_use_def") == 0) {
	IFDEBUG( fprintf(mystderr, "[Aria] Setting Aria parameter \"%s\"\n", "-show_bb_reg_use_def"); );
	do_bb_regs_use_def_dump  = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }
      else if (strcmp(argv[i], "-show_hash_table") == 0) {
	IFDEBUG( fprintf(mystderr, "[Aria] Setting Aria parameter \"%s\"\n", "-show_hash_table"); );
	do_hash_tables_dump      = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }
      else if (strcmp(argv[i], "-show_no_hash_table") == 0) {
	IFDEBUG( fprintf(mystderr, "[Aria] Setting Aria parameter \"%s\"\n", "-show_no_hash_table"); );
	do_hash_tables_dump      = 0;
      }
      else if (strcmp(argv[i], "-show_proc_state") == 0) {
	IFDEBUG( fprintf(mystderr, "[Aria] Setting Aria parameter \"%s\"\n", "-show_proc_state"); );
	do_proc_state_dump       = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }
      else if (strcmp(argv[i], "-show_proc_floats") == 0) {
	IFDEBUG( fprintf(mystderr, "[Aria] Setting Aria parameter \"%s\"\n", "-show_proc_floats"); );
	show_proc_float_state   = 1;
	do_no_output_during_runs = 0;
	if (output_after_static_bb_translation == INT_MAX) {
	  output_after_static_bb_translation = 0;
	}
	if (output_after_dynamic_bb_execution == INT_MAX) {
	  output_after_dynamic_bb_execution  = 0;
	}
      }

      else if (strcmp(argv[i], "-show_final_stats") == 0) {
	IFDEBUG( fprintf(mystderr, "[Aria] Setting Aria parameter \"%s\"\n", "-show_final_stats"); );
	show_final_aria_statistics = 1;
      }

      else if (strcmp(argv[i], "-verify_hash") == 0) {
#ifndef FULL_DEVELOPMENT_DEBUG_CODE
	warn("Aria does not generally support the -verify_hash option...\n");
	warn("  This option is only for developmental debugging. \n");
	warn("  If you need access to this option, contact the author by email:\n");
	warn("    John-David Wellman : wellman@watson.ibm.com\n");
#else
	i++;
	if (i < argc) {
	  global_do_verify_hash_after = strtoul(argv[i], 0, 0);
	}
	else {
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
	i++;
	if (i < argc) {
	  global_do_verify_hash_every = strtoul(argv[i], 0, 0);
	}
	else {
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
#endif
      }

      else if (strcmp(argv[i], "-update_on_dbbs") == 0) {
	i++;
	if (i < argc) {
	  show_run_update_after = strtoul(argv[i], 0, 0);
	}
	else {
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
	i++;
	if (i < argc) {
	  show_run_update_every = strtoul(argv[i], 0, 0);
	}
	else {
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-max_block_size") == 0) {
	i++;      
	if (i < argc) {
	  global_max_block_size_limit = strtoul(argv[i], 0, 0);
	  if ( global_max_block_size_limit > ABSOLUTE_MAXIMUM_USER_INSTR_IN_A_BLOCK ) {
	    warn("Cannot specify a max block size larger than %u\n", ABSOLUTE_MAXIMUM_USER_INSTR_IN_A_BLOCK);
	    ret_val = 1;
	    goto exit_this_while_loop;
	  }
	}
	else {
	  warn("Must specify a max block size value with -max_block_size\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }
    
      else if (strcmp(argv[i], "-output_after_sbb") == 0) {
	i++;
	if (i < argc) {
	  output_after_static_bb_translation = strtoul(argv[i], 0, 0);
	}
	else {
	  warn("Must specify a number of static block translations with -output_after_sbb\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-output_after_dbb") == 0) {
	i++;      
	if (i < argc) {
	  output_after_dynamic_bb_execution = strtoul(argv[i], 0, 0);
	}
	else {
	  warn("Must specify a number of dynamic block executions with -output_after_dbb\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-no_output_after_sbb") == 0) {
	i++;
	if (i < argc) {
	  no_output_after_static_bb_translation = strtoul(argv[i], 0, 0);
	}
	else {
	  warn("Must specify a number of static block translations with -no_output_after_sbb\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-no_output_after_dbb") == 0) {
	i++;      
	if (i < argc) {
	  no_output_after_dynamic_bb_execution = strtoul(argv[i], 0, 0);
	}
	else {
	  warn("Must specify a number of dynamic block executions with -no_output_after_dbb\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-orig_user_stack") == 0) {
	; /** Handled by the main.c code **/
      }

      else if (strcmp(argv[i], "-align_xlated_code") == 0) {
	i++;      
	if (i < argc) {
	  unsigned power_of_two = (strtoul(argv[i], 0, 0) + 2);
	  the_xlated_code_alignment = 1 << power_of_two;
	}
	else {
	  warn("Must specify a word alignment (power of two) with -align_xlated_code\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-block_lim_after_sbb") == 0) {
	i++;
	if (i < argc) {
	  use_block_limit_after_static_bb_translation = strtoul(argv[i], 0, 0); 
	}
	else {
	  warn("Must specify a number of static block translations with -block_lim_after_sbb\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }
      else if (strcmp(argv[i], "-block_unlim_after_sbb") == 0) {
	i++;
	if (i < argc) {
	  use_block_unlimited_after_static_bb_translation = strtoul(argv[i], 0, 0); 
	}
	else {
	  warn("Must specify a number of static block translations with -no_block_lim_after_sbb\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-speculation_support") == 0) {
	do_not_taken_path_translations = 1;
      }
      else if (strcmp(argv[i], "-no_speculation_support") == 0) {
	do_not_taken_path_translations = 0;
      }

      else if (strcmp(argv[i], "-canned_string_code") == 0) {
	do_canned_string_code = 1;
      }
      else if (strcmp(argv[i], "-no_canned_string_code") == 0) {
	do_canned_string_code = 0;
      }

      else if (strcmp(argv[i], "-dynamic_trace") == 0) {
	do_dynamic_trace_generation = 1;
      }
      else if (strcmp(argv[i], "-no_dynamic_trace") == 0) {
	do_dynamic_trace_generation = 0;
      }

      else if (strcmp(argv[i], "-old_init_proc_state_code") == 0) {
	global_do_old_initial_proc_state_code = 1;
      }

      else if (strcmp(argv[i], "-iaddr_fixup") == 0) {
	do_iaddr_fixup = 1;
      }
      else if (strcmp(argv[i], "-no_iaddr_fixup") == 0) {
	do_iaddr_fixup = 0;
      }

      else if (strcmp(argv[i], "-report_bclr_for_sc") == 0) {
	turn_svc_into_a_bclr_instruction = 1;
      }
      else if (strcmp(argv[i], "-report_svc_for_sc") == 0) {
	turn_svc_into_a_bclr_instruction = 0;
      }
      else if (strcmp(argv[i], "-report_sc_for_sc") == 0) {
	turn_svc_into_a_bclr_instruction = 0;
      }

      else if (strcmp(argv[i], "-raw_spec_code") == 0) {
	global_return_nop_in_trace_for_illegal_speculatives = 0;
      }


      else if (strcmp(argv[i], "-simulate") == 0) {
	global_do_aria_simulation = 1;
      }
      else if (strcmp(argv[i], "-no_simulate") == 0) {
	global_do_aria_simulation = 0;
      }

      else if (strcmp(argv[i], "-show_sc_instr") == 0) {
	global_use_safe_svc_code = 0;
      }
      else if (strcmp(argv[i], "-safe_sc_code") == 0) {
	global_use_safe_svc_code = 1;
      }

      else if (strcmp(argv[i], "-show_sc_times") == 0) {
	global_get_system_call_execution_times = 1;
      }

      else if (strcmp(argv[i], "-fpr_use_in_model") == 0) {
	no_FP_registers_used_in_model_flag = 0;
      }
      else if (strcmp(argv[i], "-no_fpr_use_in_model") == 0) {
	no_FP_registers_used_in_model_flag = 1;
      }

      else if (strcmp(argv[i], "-no_NT_addr_reqd") == 0) {
	not_taken_address_required_flag = 0;
      }
      else if (strcmp(argv[i], "-NT_addr_reqd") == 0) {
	not_taken_address_required_flag = 1;
      }
      else if (strcmp(argv[i], "-allow_global_access") == 0) {
	allow_global_access_permissions = 1;
      }
    
      else if (strcmp(argv[i], "-full_safe_exit") == 0) {
	do_safe_exit_flag = 1;
      }
      else if (strcmp(argv[i], "-less_safe_exit") == 0) {
	do_safe_exit_flag = 0;
      }
    
      else if (strcmp(argv[i], "-oslevel") == 0) {
	i++;
	if (i < argc) {
	  if ((strcmp(argv[i], "AIX4.1-L") == 0) || (strcmp(argv[i], "AIX4.1.4-L") == 0)) {
	    /*
	      the_exit_code_value = 0x16;
	      */
	  }
	  if ((strcmp(argv[i], "AIX4.1") == 0) || (strcmp(argv[i], "AIX4.1.4") == 0)) {
	    /*
	      the_exit_code_value = 0x17;
	      */
	  }
	  else if (strcmp(argv[i], "AIX4.2") == 0) {
	    /*
	      the_exit_code_value = 0x19;
	      */
	  }
	  else if ((strcmp(argv[i], "AIX3.2") == 0) || (strcmp(argv[i], "AIX3.2.5") == 0)) {
	    /*
	      the_exit_code_value = 0x0d;
	      */
	  }
	  else {
	    fprintf(mystderr, "Illegal flag type in read_input_flags of Aria\n");
	  }
	}
	else {
	  fprintf(mystderr, "Need a flag value for the -oslevel switch...\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-arch") == 0) {
	i++;
	if (i < argc) {
	  if ( (strcmp(argv[i], "POWER") == 0) || (strcmp(argv[i], "PPC") == 0) ) {
	    ; /* Do nothing; this flag is now set automatically */
	  }
	  else {
	    fprintf(mystderr, "Illegal flag type in read_input_flags of Aria:\n");
	    fprintf(mystderr, "   -arch flag cannot have %s flag\n", argv[i]);
	  }
	}
	else {
	  fprintf(mystderr, "Need a flag value for the -arch switch...\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else if (strcmp(argv[i], "-uarch") == 0) {
	i++;
	if (i < argc) {
	  if (strcmp(argv[i], "POWER") == 0) {
	    the_user_program_architecture = ARIA_POWER_ISA;
	  }
	  else if (strcmp(argv[i], "PPC") == 0) {
	    the_user_program_architecture = ARIA_PPC_ISA;
	  }
	  else {
	    fprintf(mystderr, "Illegal flag type in read_input_flags of Aria:\n");
	    fprintf(mystderr, "   -uarch flag cannot have %s flag\n", argv[i]);
	  }
	}
	else {
	  fprintf(mystderr, "Need a flag value for the -arch switch...\n");
	  ret_val = 1;
	  goto exit_this_while_loop;
	}
      }

      else {
	fprintf(mystderr, "Unrecognized switch in Aria read_input_flags: '%s'\n", argv[i]);
	ret_val = 1;
	goto exit_this_while_loop;
      }
      break;

    default:
      fprintf(mystderr, "Illegal flag type in read_input_flags of Aria\n");
      fprintf(mystderr, "   the flag is: '%s'\n", argv[i]);
      ret_val = 1;
      break;
    }
    i++;
  }

exit_this_while_loop:
  
  if (user_argc_count == 0) {
    fprintf(mystderr, "NOTE: Did not find a --PROGparms entry...\n");
    ret_val = 1;
  }

  *user_argc  = user_argc_count;
  
  return ret_val;
}



  
