#include "ansi_assert.h"
#include "ansi_stdio.h"
#include "aria_reader.h"
#include "opc.h"

#if( !defined(TRACE) )
#define TRACE		0
#endif

#if( !defined(DUMP_ALL))
#define DUMP_ALL	0
#endif

#if( !defined(COUNT) )
#define COUNT		0
#endif

#define COND(x)		0
/* #define COND(x)		(((x) > 27399400) || (((x)%0x10000) == 0) ) */

#if( TRACE )
#define TRC(x)	((void)(x))
#else
#define TRC(x)	((void)(0))
#endif

#define min(x,y)	(((x)<(y))?(x):(y))
#define is_valid_address(a,mn,mx)	\
	( ((mn) <= (a) && (a) < (mx) ) ||\
	  (((a) & 0xf0000000) == 0xd0000000 ) )

extern FILE * the_stderr;
static unsigned	last_pc = 0;

void
aria_reader_setup(
	aria_reader *	reader,
	unsigned	min_pc,
	unsigned	max_pc,
	unsigned *	first_pc
	)
{
  unsigned *	init;

  TRC( fprintf(the_stderr, "entering setup...%p %x %x %p\n",
	reader, min_pc, max_pc, first_pc) );

  init = do_aria_setup(aria_reader_taken_state(reader));
  TRC( fprintf(the_stderr, "\tinit = %p\n", init) );
/***
last_pc = (unsigned) init;
***/

  aria_reader_first_pc(reader)		= (unsigned) init;
  aria_reader_min_pc(reader)		= min_pc;
  aria_reader_max_pc(reader)		= max_pc;

  aria_reader_sample_total(reader)	= 0;
  aria_reader_sample_skip(reader)	= 0;
  aria_reader_sample_read(reader)	= ~0;

  aria_reader_taken_last_pc(reader)	= (unsigned) init;
  aria_reader_taken_left(reader)	= 0;

  *first_pc				= (unsigned) init;
  TRC( fprintf(the_stderr, "\tleaving setup\n") );
  TRC( opc_init() );
}

void
aria_sample_setup(
	aria_reader *	reader,
	unsigned	skip,
	unsigned	read
	)
{
  aria_reader_sample_skip(reader)	= skip;
  aria_reader_sample_read(reader)	= read;
}

int
aria_get_taken(
	aria_reader *	reader,
	unsigned * 	parm_pc,
	int		max_info,
	xinsn_info	info[]
	)
{
  unsigned	min_pc;
  unsigned	num_skip;
  unsigned	max_pc;
  unsigned	pc;
  unsigned	next_pc;
  unsigned	num;
  unsigned *	dummy;
  unsigned *	itrace;
  unsigned	left;
  unsigned	start;
  unsigned	i;
  unsigned	begin;
  unsigned	dummy_left;
  unsigned *	dummy_itrace;
  char		buf[512];
  #if( COUNT )
  static unsigned count = 0;
  #endif

/* fprintf(the_stderr, "ENTERING: get_taken %08x\n", *parm_pc); */

  if( aria_reader_is_done(reader) ) {
    fprintf(the_stderr, "already done\n");
    return 0;
  }

  TRC( fprintf(the_stderr, "============================================================\n") );
  TRC( fprintf(the_stderr, "entering get taken...%p pc [@%p]=%08x max=%d info=%p\n",
	reader, parm_pc, *parm_pc, max_info, info) );

  pc = *parm_pc;
  left = aria_reader_taken_left(reader);
  start = aria_reader_taken_start(reader);

  TRC( fprintf(the_stderr, "\tpc=%x left=%d start=%d last=%x\n",
	pc, left, start, aria_reader_taken_last_pc(reader) ) );

  min_pc = aria_reader_min_pc(reader);
  max_pc = aria_reader_max_pc(reader);

  if( left == 0 ) {

    TRC( fprintf(the_stderr, "\tgetting more stuff from %08x...min=%x max=%x\n",
	pc, min_pc, max_pc) );
    next_pc = pc;

    do {

      if( next_pc == 0 ) {
	#if( COUNT )
	fprintf( the_stderr, "dbb_count[final0] =%d\n", count);
	#endif
	aria_reader_is_done(reader) = 1;
	return 0;
      }

      #if( COUNT )
	if( COND(count) ) {
	  fprintf(the_stderr, "dbb_count[pre main...] =%d, next= %08x\n", count, next_pc);
	  fflush( the_stderr );
	}
      #endif

/***
fprintf(the_stderr, "fetching from %08x [last %08x]\n", next_pc, last_pc);
assert( next_pc == last_pc );
***/

      if( aria_reader_sample_total(reader) >
		aria_reader_sample_read(reader) ) {

	fprintf(the_stderr, "skipping %u...\n", aria_reader_sample_skip(reader));
	next_pc = (unsigned) do_aria_skip_instructions((unsigned *)next_pc,
			aria_reader_taken_state(reader),
			aria_reader_sample_skip(reader),
			&num_skip);
	fprintf(the_stderr, "... done %u\n", num_skip);
			
	aria_reader_sample_total(reader) = 0;

	if( next_pc == 0 ) {
	  #if( COUNT )
	  fprintf( the_stderr, "dbb_count[final0] =%d\n", count);
	  #endif
	  aria_reader_is_done(reader) = 1;
	  return 0;
	}
      }

      next_pc = (unsigned) do_aria_execution( (unsigned *) next_pc,
			aria_reader_taken_state(reader),
			&left,
			&aria_reader_taken_itrace(reader),
			&dummy,
			0);

/***
last_pc = (unsigned) next_pc;
fprintf(the_stderr, "\t..fetched [next %08x]\n", next_pc);
***/

      #if( COUNT )
	count++;
	if( COND(count) ) {
	  fprintf(the_stderr, "dbb_count[main] =%d, next= %08x\n", count, next_pc);
	}
      #endif
      #if( DUMP_ALL )
	for( i = 0; i < left; i++  ) {
	  opc_dis(aria_reader_taken_itrace(reader)[5*i+1], buf);
	  fprintf(the_stderr, "%08x %08x=%s\n",
		aria_reader_taken_itrace(reader)[5*i],
		aria_reader_taken_itrace(reader)[5*i+1], buf);
	}
      #endif

      TRC( fprintf(the_stderr, "\t\tnext_pc = %x, %d, itrace=%x\n",
	next_pc, left, aria_reader_taken_itrace(reader)) );

      begin = aria_reader_taken_itrace(reader)[0];
    } while( !is_valid_address(begin, min_pc, max_pc ) || left == 0 );

    while( !is_valid_address(next_pc, min_pc, max_pc) && next_pc != 0 ) {
    TRC( fprintf(the_stderr, "\tgetting lib stuff...%x\n", next_pc) );
      #if( COUNT )
	if( COND(count) ) {
	  fprintf(the_stderr, "dbb_count[pre...] =%d, next= %08x\n", count, next_pc);
	}
      #endif
/***
fprintf(the_stderr, "fetching from %08x [last %08x]\n", next_pc, last_pc);
assert( next_pc == last_pc );
***/

      if( aria_reader_sample_total(reader) >
		aria_reader_sample_read(reader) ) {

	fprintf(the_stderr, "skipping %u...\n", aria_reader_sample_skip(reader));
	next_pc = (unsigned) do_aria_skip_instructions((unsigned *)next_pc,
			aria_reader_taken_state(reader),
			aria_reader_sample_skip(reader),
			&num_skip);
	fprintf(the_stderr, "... done %u\n", num_skip);
			
	aria_reader_sample_total(reader) = 0;

	if( next_pc == 0 ) {
	  #if( COUNT )
	  fprintf( the_stderr, "dbb_count[final0] =%d\n", count);
	  #endif
	  aria_reader_is_done(reader) = 1;
	  return 0;
	}
      }


      next_pc = (unsigned) do_aria_execution( (unsigned *) next_pc,
			aria_reader_taken_state(reader),
			&dummy_left,
			&dummy_itrace,
			&dummy,
			0);

/****
last_pc = (unsigned) next_pc;
fprintf(the_stderr, "\t..fetched [next %08x]\n", next_pc);
****/

      #if( COUNT )
	count++;
	if( COND(count) ){
	  fprintf(the_stderr, "dbb_count[lib] =%d, next= %08x\n", count, next_pc);
	}
      #endif
      #if( DUMP_ALL )
	for( i = 0; i < dummy_left; i++  ) {
	  opc_dis(dummy_itrace[5*i+1], buf);
	  fprintf(the_stderr, "%08x %08x=%s\n",
		dummy_itrace[5*i], dummy_itrace[5*i+1], buf);
	}
      #endif
    }

    #if( COUNT )
      if( next_pc == 0 ) {
	fprintf(the_stderr, "dbb_count[final] =%d\n", count);
      }
    #endif


    aria_reader_sample_total(reader) += left;
    start = 0;
    aria_reader_taken_last_pc(reader) = next_pc;
    TRC( fprintf(the_stderr, "\t\tfinally %x..%x %d %d\n", begin, next_pc, left, start) );
  }

  itrace = aria_reader_taken_itrace(reader)+5*start;
  num = min( max_info, left);

  for( i = 0; i < num; i++ ) {
    xinsn_info_iaddr(info[i]) = itrace[0];
    xinsn_info_iword(info[i]) = itrace[1];
    xinsn_info_daddr(info[i]) = itrace[3];
    xinsn_info_dlength(info[i]) = itrace[4];
    #if( TRACE )
      opc_dis(itrace[1], buf);
      fprintf(the_stderr, "[%08x] %08x %08x=%s\n", itrace, itrace[0], itrace[1], buf);
    #endif
    assert( is_valid_address( itrace[0], min_pc, max_pc) );
    itrace += 5;
  }

  start += num;
  left -= num;
  assert( left >= 0 );

  aria_reader_taken_left(reader) = left;
  aria_reader_taken_start(reader) = start;

  if( left == 0 ) {
    *parm_pc = aria_reader_taken_last_pc(reader);
  }
  else {
    *parm_pc = itrace[0];
  }

  TRC( fprintf( the_stderr, "next pc is %x, left=%d start=%d num=%d\n", *parm_pc, left, start, num));

/* fprintf(the_stderr, "LEAVING: get_taken %08x\n", *parm_pc); */
  return num;
}

void
aria_swap_to_not_taken(
	aria_reader *	reader,
	unsigned	spec_pc
	)
{
  TRC( fprintf(the_stderr, "swapping to not taken %08x\n", spec_pc ) );
  *aria_reader_nottaken_state(reader) = *aria_reader_taken_state(reader);
  aria_reader_nottaken_left(reader) = 0;
  aria_reader_nottaken_last_pc(reader) = spec_pc;
}

int
aria_get_not_taken(
	aria_reader *	reader,
	unsigned * 	parm_pc,
	int		max_info,
	xinsn_info	info[]
	)
{
  unsigned	min_pc;
  unsigned	max_pc;
  unsigned	pc;
  unsigned	next_pc;
  unsigned	num;
  unsigned *	dummy;
  unsigned *	itrace;
  unsigned	left;
  unsigned	start;
  unsigned	i;
  unsigned	begin;
  unsigned	dummy_left;
  unsigned *	dummy_itrace;
  char		buf[512];

  TRC( fprintf(the_stderr, "============================================================\n") );

  TRC( fprintf(the_stderr, "entering get not taken...%p %p=%x %d %p\n",
	reader, parm_pc, *parm_pc, max_info, info) );

  pc = *parm_pc;
  left = aria_reader_nottaken_left(reader);
  start = aria_reader_nottaken_start(reader);

  TRC( fprintf(the_stderr, "\tpc=%x left=%d start=%d last=%x\n",
	pc, left, start, aria_reader_nottaken_last_pc(reader) ) );


  min_pc = aria_reader_min_pc(reader);
  max_pc = aria_reader_max_pc(reader);

  if( left == 0 || aria_reader_nottaken_last_pc(reader) != pc ) {

    if( !is_valid_address( pc, min_pc, max_pc ) ) {
      return 0;
    }

    TRC( fprintf(the_stderr, "\tgetting more stuff from %08x...min=%x max=%x\n",
	pc, min_pc, max_pc) );
    next_pc = pc;


    TRC( fprintf(the_stderr, "\t next_pc = %08x\n", next_pc ) );
    next_pc = (unsigned) do_aria_execution( (unsigned *) next_pc,
		      aria_reader_nottaken_state(reader),
		      &left,
		      &aria_reader_nottaken_itrace(reader),
		      &dummy,
		      1);
    TRC( fprintf(the_stderr, "\t....back; new next_pc = %08x\n", next_pc) );

    #if( DUMP_ALL )
      for( i = 0; i < left; i++  ) {
	opc_dis(aria_reader_nottaken_itrace(reader)[5*i+1], buf);
	fprintf(the_stderr, "%d/%d: %08x %08x=%s\n",
	      i, left,
	      aria_reader_nottaken_itrace(reader)[5*i],
	      aria_reader_nottaken_itrace(reader)[5*i+1], buf);
      }
    #endif

    TRC( fprintf(the_stderr, "\t\tnext_pc = %x, %d, itrace=%x\n",
      next_pc, left, aria_reader_nottaken_itrace(reader)) );



    start = 0;
    aria_reader_nottaken_last_pc(reader) = next_pc;
    TRC( fprintf(the_stderr, "\t\tfinally %x..%x %d %d\n", begin, next_pc, left, start) );
  }

  itrace = aria_reader_nottaken_itrace(reader)+5*start;
  num = min( max_info, left);

  for( i = 0; i < num && is_valid_address(itrace[0], min_pc, max_pc); i++ ) {
    xinsn_info_iaddr(info[i]) = itrace[0];
    xinsn_info_iword(info[i]) = itrace[1];
    xinsn_info_daddr(info[i]) = itrace[3];
    xinsn_info_dlength(info[i]) = itrace[4];
    #if( TRACE )
      opc_dis(itrace[1], buf);
      fprintf(the_stderr, "[%08x] %08x %08x=%s\n", itrace, itrace[0], itrace[1], buf);
    #endif
    assert( is_valid_address(itrace[0], min_pc, max_pc) );
    itrace += 5;
  }

  start += i;
  left -= i;
  assert( left >= 0 );

  aria_reader_nottaken_left(reader) = left;
  aria_reader_nottaken_start(reader) = start;

  if( left == 0 ) {
    *parm_pc = aria_reader_nottaken_last_pc(reader);
  }
  else {
    *parm_pc = itrace[0];
  }

  TRC( fprintf( the_stderr, "next pc is %x, left=%d start=%d\n", *parm_pc, left, start));

  return i;
}
