#include "ansi_assert.h"
#include "ansi_stdio.h"
#include "ansi_stdlib.h"
#include "dep_prep_info.h"

#define MAGIC_NUMBER	0x1e

void
write_dep_prep(	FILE * fp,
		struct xdep_prep_ctl *	ctl,
		unsigned first_addr,
		unsigned min_addr,
		unsigned size,
		xdep_prep_info * info,

		unsigned osize,
		xarch * overflow,

		unsigned xsize,
		xdep_prep_info * expand
	)
{
  unsigned		total = size + xsize;
  size_t		ret;
  unsigned		offset;
  xarch *	ptr;
  xdep_prep_info *	eptr;
  unsigned		i;
  xdep_prep_info *	entry;
  unsigned		first = MAGIC_NUMBER;

  ret = fwrite( &first, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "write_dep_prep: error writing first word\n");
    exit( EXIT_FAILURE );
  }

  ret = fwrite( ctl, sizeof( struct xdep_prep_ctl), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "write_dep_prep: error writing xdep_prep_ctl\n");
    exit( EXIT_FAILURE );
  }

  ret = fwrite( &first_addr, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "write_dep_prep: error writing first addr\n");
    exit( EXIT_FAILURE );
  }

  ret = fwrite( &min_addr, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "write_dep_prep: error writing min addr\n");
    exit( EXIT_FAILURE );
  }

  ret = fwrite( &size, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "write_dep_prep: error writing size\n");
    exit( EXIT_FAILURE );
  }

  ret = fwrite( &xsize, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "write_dep_prep: error writing expand size\n");
    exit( EXIT_FAILURE );
  }

  ret = fwrite( &osize, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "write_dep_prep: error writing reg over size\n");
    exit( EXIT_FAILURE );
  }

  for( i = 0; i < size; i++ ) {
    entry = &info[i];
    assert( xdep_prep_info_fetch(entry) >= 1 );
    if( xdep_prep_info_is_over(entry) ) {
      offset = xdep_prep_info_overflow(entry) - overflow;
      assert( offset >= 0 );
      assert( offset < osize );
      xdep_prep_info_over_off(entry) = offset;
    }
    if( xdep_prep_info_is_expand(entry) ) {
      offset = xdep_prep_info_expands(entry) - expand;
      assert( offset >= 0 );
      assert( offset < xsize );
      xdep_prep_info_exp_off(entry) = offset+size;
    }
  }

  for( i = 0; i < xsize; i++ ) {
    entry = &expand[i];
    if( xdep_prep_info_is_over(entry) ) {
      offset = xdep_prep_info_overflow(entry) - overflow;
      assert( offset >= 0 );
      assert( offset < osize );
      xdep_prep_info_over_off(entry) = offset;
    }

    assert(!xdep_prep_info_is_expand(entry) );
    xdep_prep_info_exp_off(entry) = 0;
  }

  ret = fwrite( info, sizeof( xdep_prep_info ), size, fp );
  if( ret != size ) {
    fprintf(stderr, "write_dep_prep: error writing %d entries\n", size);
    exit( EXIT_FAILURE );
  }

  ret = fwrite( expand, sizeof( xdep_prep_info ), xsize, fp );
  if( ret != xsize ) {
    fprintf(stderr, "write_dep_prep: error writing %d expands\n", xsize);
    exit( EXIT_FAILURE );
  }

  ret = fwrite( overflow , sizeof( xarch ), osize, fp );
  if( ret != osize ) {
    fprintf(stderr, "write_dep_prep: error writing %d reg overs\n", osize);
    exit( EXIT_FAILURE );
  }

  for( i = 0; i < size; i++ ) {
    entry = &info[i];
    if( xdep_prep_info_is_over(entry) ) {
      ptr = overflow + xdep_prep_info_over_off(entry);
      xdep_prep_info_overflow(entry) = ptr;
    }
    if( xdep_prep_info_is_expand(entry) ) {
      eptr = expand + (xdep_prep_info_exp_off(entry)-size);
      xdep_prep_info_expands(entry) = eptr;
    }
    assert( xdep_prep_info_res(entry) != 0 || xdep_prep_info_res_compl(entry) == 0x80808080 );
    assert( xdep_prep_info_res(entry) == 0 || xdep_prep_info_res_compl(entry) != 0x80808080 );
  }

  for( i = 0; i < xsize; i++ ) {
    entry = &expand[i];
    if( xdep_prep_info_is_over(entry) ) {
      ptr = overflow + xdep_prep_info_over_off(entry);
      xdep_prep_info_overflow(entry) = ptr;
    }
    assert( xdep_prep_info_res(entry) != 0 || xdep_prep_info_res_compl(entry) == 0x80808080 );
    assert( xdep_prep_info_res(entry) == 0 || xdep_prep_info_res_compl(entry) != 0x80808080 );
  }

}

void
read_dep_prep(	FILE * fp,
		struct xdep_prep_ctl *	ctl,
		unsigned * pfirst_addr,
		unsigned * pmin_addr,
		unsigned * psize,
		unsigned * pxsize,
		xdep_prep_info ** pinfo,
		unsigned * posize,
		xarch ** poverflow
	)
{
  int			ret;
  unsigned		size;
  unsigned		xsize;
  xdep_prep_info *	info;
  xdep_prep_info *	dbg;
  unsigned		osize;
  xarch *	overflow;
  unsigned		i;
  xarch *	ptr;
  xdep_prep_info *	entry;
  unsigned		first;

  ret = fread( &first, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "read_dep_prep: error reading first word\n");
    exit( EXIT_FAILURE );
  }

  if(  first  != MAGIC_NUMBER ) {
    fprintf(stderr, "magic number mismatch; expected %x, got %x\n", MAGIC_NUMBER, first );
    exit( EXIT_FAILURE );
  }

  ret = fread( ctl, sizeof( struct xdep_prep_ctl ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "read_dep_prep: error reading xdep_prep_ctl\n");
    exit( EXIT_FAILURE );
  }


  ret = fread( pfirst_addr, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "read_dep_prep: error reading first addr\n");
    exit( EXIT_FAILURE );
  }

  ret = fread( pmin_addr, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "read_dep_prep: error reading min\n");
    exit( EXIT_FAILURE );
  }

  ret = fread( psize, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "read_dep_prep: error reading size\n");
    exit( EXIT_FAILURE );
  }

  ret = fread( pxsize, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "read_dep_prep: error reading expand size\n");
    exit( EXIT_FAILURE );
  }

  ret = fread( posize, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "read_dep_prep: error reading osize\n");
    exit( EXIT_FAILURE );
  }

  size = *psize;
  xsize = *pxsize;
  osize = *posize;

  info = (xdep_prep_info *)malloc( sizeof( xdep_prep_info )*(size+xsize) );
  if( info == 0 ) {
    fprintf(stderr, "read_dep_prep: out of memory allocating entries\n");
    exit( EXIT_FAILURE );
  }

  overflow = (xarch *)malloc( sizeof( xarch )*osize );
  if( osize != 0 && overflow == 0 ) {
    fprintf(stderr, "read_dep_prep: out of memory allocating overflow\n");
    exit( EXIT_FAILURE );
  }

  ret = fread( info, sizeof( xdep_prep_info ), size, fp);
  if( ret != size ) {
    fprintf(stderr, "read_dep_prep: error reading %d entries\n", size);
    exit( EXIT_FAILURE );
  }

  ret = fread( &info[size], sizeof( xdep_prep_info ), xsize, fp);
  if( ret != xsize ) {
    fprintf(stderr, "read_dep_prep: error reading %d expands\n", xsize);
    exit( EXIT_FAILURE );
  }

  ret = fread( overflow, sizeof( xarch ), osize, fp);
  if( ret != osize ) {
    fprintf(stderr, "read_dep_prep: error reading %d reg overs\n", osize);
    exit( EXIT_FAILURE );
  }

  for( i = 0; i < (size+xsize); i++ ) {
    entry = &info[i];
    assert( xdep_prep_info_fetch(entry) >= 1 );
    if( xdep_prep_info_is_over(entry) ) {
      ptr = overflow + xdep_prep_info_over_off(entry);
      xdep_prep_info_overflow(entry) = ptr;
    }
  }

  *pinfo = info;
  *poverflow = overflow;
}
