#include "ansi_assert.h"
#include "ansi_stdio.h"
#include "ansi_stdlib.h"

#define USING_FF	1
#define MAGIC_NUMBER	0x1f

#include "dep_prep_info.h"

void
write_ffdep_prep(
	FILE *			fp,
	struct xdep_prep_ctl *  ctl,
	unsigned		ifetch_max,
	unsigned		first_addr,
	xdep_prep_info_table *	table,
	unsigned		xsize,
	xdep_prep_info *	expand,
	unsigned		osize,
	xarch *			overflow
	)
{
  int			status;
  unsigned		offset;
  xarch *	ptr;
  unsigned		i;
  xdep_prep_info *	array;
  xdep_prep_info *	entry;
  xdep_prep_info *	eptr;
  unsigned		size;
  unsigned		first = (MAGIC_NUMBER << 16) | ifetch_max;

  status = fwrite( &first, sizeof( unsigned ), 1, fp );
  if( status != 1 ) {
    fprintf(stderr, "write_ffdep_prep: error writing first word\n");
    exit( EXIT_FAILURE );
  }

  status = fwrite( ctl, sizeof( struct xdep_prep_ctl), 1, fp );
  if( status != 1 ) {
    fprintf(stderr, "write_ffdep_prep: error writing xdep_prep_ctl\n");
    exit( EXIT_FAILURE );
  }

  status = fwrite( &first_addr, sizeof( unsigned ), 1, fp );
  if( status != 1 ) {
    fprintf(stderr, "write_ffdep_prep: error writing first addr\n");
    exit( EXIT_FAILURE );
  }

  status = fwrite( &xsize, sizeof( unsigned ), 1, fp );
  if( status != 1 ) {
    fprintf(stderr, "write_ffdep_prep: error writing expand size\n");
    exit( EXIT_FAILURE );
  }

  status = fwrite( &osize, sizeof( unsigned ), 1, fp );
  if( status != 1 ) {
    fprintf(stderr, "write_ffdep_prep: error writing reg over size\n");
    exit( EXIT_FAILURE );
  }

  array = array_xdep_prep_info_table(table);
  size = size_xdep_prep_info_table(table);
  for( i = 0; i < size; i++ ) {
    entry = &array[i];
    if( xdep_prep_info_is_over(entry) ) {
      offset = xdep_prep_info_overflow(entry) - overflow;
      assert( offset >= 0 );
      assert( offset < osize );
      xdep_prep_info_over_off(entry) = offset;
    }
    if( xdep_prep_info_is_expand(entry) ) {
      offset = xdep_prep_info_expands(entry) - expand;
      assert( offset >= 0 );
      assert( offset < xsize );
      xdep_prep_info_exp_off(entry) = offset;
    }
  }

  for( i = 0; i < xsize; i++ ) {
    entry = &expand[i];
    if( xdep_prep_info_is_over(entry) ) {
      offset = xdep_prep_info_overflow(entry) - overflow;
      assert( offset >= 0 );
      assert( offset < osize );
      xdep_prep_info_over_off(entry) = offset;
    }
    assert( !xdep_prep_info_is_expand(entry) );
    xdep_prep_info_exp_off(entry) = 0;
  }

#if( !NDEBUG )
for( i = 0; i < size; i++ ) {
  entry = &array[i];
  assert( xdep_prep_info_fetch(entry) > 0 );
  assert( xdep_prep_info_res(entry) || xdep_prep_info_res_compl(entry));
}

for( i = 0; i < xsize; i++ ) {
  entry = &expand[i];
  assert( xdep_prep_info_res(entry) || xdep_prep_info_res_compl(entry));
}
#endif

  write_xdep_prep_info_table( fp, table);

  status = fwrite(expand, sizeof( struct xdep_prep_info), xsize, fp);
  if( status != xsize ) {
    fprintf(stderr, "write_ffdep_prep: error writing %d expands\n", xsize);
    exit( EXIT_FAILURE );
  }

  for( i = 0; i < size; i++ ) {
    entry = &array[i];
    if( xdep_prep_info_is_over(entry) ) {
      ptr = overflow + xdep_prep_info_over_off(entry);
      xdep_prep_info_overflow(entry) = ptr;
    }
    if( xdep_prep_info_is_expand(entry) ) {
      eptr = expand + (xdep_prep_info_exp_off(entry));
      xdep_prep_info_expands(entry) = eptr;
    }
  }
  for( i = 0; i < xsize; i++ ) {
    entry = &array[i];
    if( xdep_prep_info_is_over(entry) ) {
      ptr = overflow + xdep_prep_info_over_off(entry);
      xdep_prep_info_overflow(entry) = ptr;
    }
  }

  status = fwrite( overflow , sizeof( xarch ), osize, fp );
  if( status != osize ) {
    fprintf(stderr, "write_ffdep_prep: error writing %d reg overs\n", osize);
    exit( EXIT_FAILURE );
  }
}

void
read_ffdep_prep(
	FILE *			fp,
	struct xdep_prep_ctl *	ctl,
	unsigned *		pifetch_max,
	unsigned *		pfirst_addr,
	xdep_prep_info_table *	table,
	unsigned *		pxsize,
	xdep_prep_info **	pexpand,
	unsigned *		posize,
	xarch **		poverflow
	)
{
  int			ret;
  xdep_prep_info *	array;
  xdep_prep_info *	entry;
  xdep_prep_info *	expand;
  xdep_prep_info *	eptr;
  unsigned		size;
  unsigned		osize;
  unsigned		xsize;
  xarch *		overflow;
  unsigned		i;
  xarch *		ptr;
  int			status;
  unsigned		first;

  status = fread( &first, sizeof( unsigned ), 1, fp );
  if( status != 1 ) {
    fprintf(stderr, "read_ffdep_prep: error reading first word\n");
    exit( EXIT_FAILURE );
  }

  if( (first >> 16) !=  MAGIC_NUMBER ) {
    fprintf(stderr, "magic number mismatch; expected %x, got %x\n", MAGIC_NUMBER, first>>16);
    exit( EXIT_FAILURE );
  }
  *pifetch_max = first & 0xffff;

  ret = fread(ctl, sizeof(struct xdep_prep_ctl), 1, fp);
  if( ret != 1 ) {
    fprintf(stderr, "ffread_dep_prep: error reading xdep_prep_ctl\n");
    exit( EXIT_FAILURE);
  }

  status = fread( pfirst_addr, sizeof( unsigned ), 1, fp );
  if( status != 1 ) {
    fprintf(stderr, "read_ffdep_prep: error reading first addr\n");
    exit( EXIT_FAILURE );
  }

  status = fread( pxsize, sizeof( unsigned ), 1, fp );
  if( status != 1 ) {
    fprintf(stderr, "read_ffdep_prep: error reading expand size\n");
    exit( EXIT_FAILURE );
  }
  xsize = *pxsize;

  status = fread( posize, sizeof( unsigned ), 1, fp );
  if( status != 1 ) {
    fprintf(stderr, "read_ffdep_prep: error reading reg over size\n");
    exit( EXIT_FAILURE );
  }
  osize = *posize;

  read_xdep_prep_info_table( fp, table );

  expand = (xdep_prep_info *)malloc( sizeof( xdep_prep_info )*xsize );
  if( xsize != 0 && expand == 0 ) {
    fprintf(stderr, "read_ffdep_prep: out of memory allocating expands\n");
    exit( EXIT_FAILURE);
  }

  status = fread( expand, sizeof( xdep_prep_info ), xsize, fp);
  if( status != xsize ) {
    fprintf(stderr, "read_ffdep_prep: error reading %d expands\n", xsize);
    exit( EXIT_FAILURE );
  }

  overflow = (xarch *)malloc( sizeof( xarch )*osize );
  if( osize != 0 && overflow == 0 ) {
    fprintf(stderr, "read_ffdep_prep: out of memory allocating reg overs\n");
    exit( EXIT_FAILURE);
  }

  status = fread( overflow, sizeof( xarch ), osize, fp);
  if( status != osize ) {
    fprintf(stderr, "read_ffdep_prep: error reading %d reg overs\n", osize);
    exit( EXIT_FAILURE );
  }

  size = size_xdep_prep_info_table( table );
  array = array_xdep_prep_info_table( table );
  for( i = 0; i < size; i++ ) {
    entry = &array[i];
    if( xdep_prep_info_is_over(entry) ) {
      ptr = overflow + xdep_prep_info_over_off(entry);
      xdep_prep_info_overflow(entry) = ptr;
    }
    if( xdep_prep_info_is_expand(entry) ) {
      eptr = expand + xdep_prep_info_exp_off(entry);
      xdep_prep_info_expands(entry) = eptr;
    }
  }

  for( i = 0; i < xsize; i++ ) {
    entry = &expand[i];
    if( xdep_prep_info_is_over(entry) ) {
      ptr = overflow + xdep_prep_info_over_off(entry);
      xdep_prep_info_overflow(entry) = ptr;
    }
  }

#if( !NDEBUG )
for( i = 0; i < size; i++ ) {
  entry = &array[i];
  assert( xdep_prep_info_fetch(entry) > 0 );
  assert( xdep_prep_info_res(entry) || xdep_prep_info_res_compl(entry));
}

for( i = 0; i < xsize; i++ ) {
  entry = &expand[i];
  assert( xdep_prep_info_res(entry) || xdep_prep_info_res_compl(entry));
}
#endif

  *pexpand = expand;
  *poverflow = overflow;
}
