#include "ansi_assert.h"
#include "ansi_stddef.h"
#include "ansi_stdlib.h"
#include "ffdep_prep_info_table.h"


void
init_xdep_prep_info_table(
	xdep_prep_info_table *	table
	)
{
  int	i;

  for( i = 0; i < FFREADER_PREP_INFO_TABLE_SIZE; i++ ) {
    subtable_xdep_prep_info_table(table, i) = 0;
  }
  array_xdep_prep_info_table(table) = 0;
  addrs_xdep_prep_info_table(table) = 0;
  size_xdep_prep_info_table(table) = 0;
}

xdep_prep_info *
find_xdep_prep_info_table(
	xdep_prep_info_table *	table,
	unsigned	addr,
	unsigned	word
	)
{
  unsigned			hi = FFREADER_PREP_INFO_HI(addr);
  unsigned			lo = FFREADER_PREP_INFO_LO(addr);
  struct subxdep_prep_info_table *	tbl = subtable_xdep_prep_info_table(table, hi);
  xdep_prep_info *	p;

  if( tbl == 0 ) {
    return 0;
  }

  for( p = entry_subxdep_prep_info_table(tbl, lo); p; p = xdep_prep_info_over(p) ) {
    if( xdep_prep_info_iword(p) == word ) {
      return p;
    }
  }
  return 0;
}

xdep_prep_info *
find_1st_xdep_prep_info_table(
	xdep_prep_info_table *	table,
	unsigned	addr
	)
{
  unsigned			hi = FFREADER_PREP_INFO_HI(addr);
  unsigned			lo = FFREADER_PREP_INFO_LO(addr);
  struct subxdep_prep_info_table *	tbl = subtable_xdep_prep_info_table(table, hi);

  if( tbl == 0 ) {
    return 0;
  }

  return entry_subxdep_prep_info_table(tbl, lo);
}

void
add_xdep_prep_info_table(
	xdep_prep_info_table *	table,
	unsigned	addr,
	xdep_prep_info *		entry
	)
{
  unsigned			hi = FFREADER_PREP_INFO_HI(addr);
  unsigned			lo = FFREADER_PREP_INFO_LO(addr);
  struct subxdep_prep_info_table *	tbl = subtable_xdep_prep_info_table(table, hi);
  xdep_prep_info *			p;

  if( tbl == 0 ) {
    tbl = (struct subxdep_prep_info_table*) calloc(sizeof(struct subxdep_prep_info_table), 1);
    subtable_xdep_prep_info_table(table, hi) = tbl;
    if( tbl == 0 ) {
      fprintf(stderr,
	"add_xdep_prep_info_table: out_of_memory allocating subxdep_prep_info_table\n");
      exit( EXIT_FAILURE );
    }
  }

  p = entry_subxdep_prep_info_table(tbl, lo);
  entry_subxdep_prep_info_table(tbl, lo) = entry;

  set_xdep_prep_info_over(entry, p);
}

void
write_xdep_prep_info_table(
	FILE *		fp,
	xdep_prep_info_table *	table
	)
{
  int				i;
  int				j;
  struct subxdep_prep_info_table *	tbl;
  xdep_prep_info *			entry;
  unsigned			size;
  unsigned			bytes;
  unsigned			addr;
  unsigned			word;
  unsigned			num;
  int				status;

  size = 0;
  for( i = 0; i < FFREADER_PREP_INFO_TABLE_SIZE; i++ ) {
    tbl = subtable_xdep_prep_info_table(table, i);
    if( tbl == 0 ) {
      continue;
    }
    for( j = 0; j < FFREADER_PREP_INFO_SUBTABLE_SIZE; j++ ) {
      for( entry = entry_subxdep_prep_info_table(tbl, j); entry; entry = xdep_prep_info_over(entry)) {
	size++;
      }
    }
  }
  status = fwrite(&size, sizeof(unsigned), 1, fp);
  if( status != 1 ) {
    fprintf(stderr,
	"write_xdep_prep_info_table: error writing size\n");
    exit( EXIT_FAILURE );
  }

  bytes = sizeof(xdep_prep_info);
  status = fwrite(&bytes, sizeof(size_t), 1, fp);
  if( status != 1 ) {
    fprintf(stderr,
	"write_xdep_prep_info_table: error writing bytes\n");
    exit( EXIT_FAILURE );
  }

  num = 0;
  for( i = 0; i < FFREADER_PREP_INFO_TABLE_SIZE; i++ ) {
    tbl = subtable_xdep_prep_info_table(table, i);
    if( tbl == 0 ) {
      continue;
    }
    for( j = 0; j < FFREADER_PREP_INFO_SUBTABLE_SIZE; j++ ) {
      addr = (i*FFREADER_PREP_INFO_SUBTABLE_SIZE+j)<<2;
      for( entry = entry_subxdep_prep_info_table(tbl, j); entry; entry = xdep_prep_info_over(entry)) {

	status = fwrite(&addr, sizeof(unsigned), 1, fp);
	if( status != 1 ) {
	  fprintf(stderr,
	      "write_xdep_prep_info_table: error writing addr of entry #%d\n", num);
	  exit( EXIT_FAILURE );
	}

	word = xdep_prep_info_iword(entry);
	status = fwrite(&word, sizeof(unsigned), 1, fp);
	if( status != 1 ) {
	  fprintf(stderr,
	      "write_xdep_prep_info_table: error writing word of entry #%d\n", num);
	  exit( EXIT_FAILURE );
	}

	status = fwrite(entry, sizeof(xdep_prep_info), 1, fp);
	if( status != 1 ) {
	  fprintf(stderr,
	      "write_xdep_prep_info_table: error writing of entry #%d\n", num);
	  exit( EXIT_FAILURE );
	}

	num++;
      }
    }
  }
  assert( num == size);
}

void
read_xdep_prep_info_table(
	FILE *		fp,
	xdep_prep_info_table *	table
	)
{
  xdep_prep_info *	array;
  unsigned *	addrs;
  int		status;
  unsigned	size;
  unsigned	bytes;
  int		i;
  unsigned	addr;
  unsigned	word;
  xdep_prep_info *	entry;

  init_xdep_prep_info_table(table);

  status = fread(&size, sizeof(unsigned), 1, fp);
  if( status != 1 ) {
    fprintf(stderr,
	"read_xdep_prep_info_table: error reading size\n");
    exit( EXIT_FAILURE );
  }
  size_xdep_prep_info_table(table) = size;

  array = (xdep_prep_info *)calloc(sizeof(xdep_prep_info), size);
  if( array == 0 ) {
    fprintf(stderr,
	"read_xdep_prep_info_table: out_of_memory allocating array of %d xdep_prep_info\n",
	size);
    exit( EXIT_FAILURE );
  }
  array_xdep_prep_info_table(table) = array;

  addrs = (unsigned *)calloc(sizeof(unsigned), size);
  if( addrs == 0 ) {
    fprintf(stderr,
	"read_xdep_prep_info_table: out_of_memory allocating array of %d addrs\n",
	size);
    exit( EXIT_FAILURE );
  }
  addrs_xdep_prep_info_table(table) = addrs;

  status = fread(&bytes, sizeof(unsigned), 1, fp);
  if( status != 1 ) {
    fprintf(stderr,
	"read_xdep_prep_info_table: error reading bytes\n");
    exit( EXIT_FAILURE );
  }

  for( i = 0; i < size; i++ ) {
    status = fread(&addr, sizeof(unsigned), 1, fp);
    if( status != 1 ) {
      fprintf(stderr,
	"read_xdep_prep_info_table: error reading addr of entry #%d\n", i);
    }
    addrs[i] = addr;

    status = fread(&word, sizeof(unsigned), 1, fp);
    if( status != 1 ) {
      fprintf(stderr,
	"read_xdep_prep_info_table: error reading word of entry #%d\n", i);
    }

    entry = &array[i];
    status = fread(entry, bytes, 1, fp);
    if( status != 1 ) {
      fprintf(stderr,
	"read_xdep_prep_info_table: error reading entry #%d\n", i);
    }

    assert( xdep_prep_info_iword(entry) == word );
    assert( !find_xdep_prep_info_table(table, addr, word) );
    assert( !find_xdep_prep_info_table(table, addrs[i], word) );
    add_xdep_prep_info_table(table, addr, entry);
    assert( find_xdep_prep_info_table(table, addr, word) == entry);
    assert( get_xdep_prep_info_table(table, addr, word) == entry);
    assert( find_xdep_prep_info_table(table, addrs[i], word) == entry);
    assert( get_xdep_prep_info_table(table, addrs[i], word) == entry);
  }
}

xdep_prep_info *
(array_xdep_prep_info_table)(
	xdep_prep_info_table *	table
	)
{
  return array_xdep_prep_info_table(table);
}


unsigned *
(addrs_xdep_prep_info_table)(
	xdep_prep_info_table *	table
	)
{
  return addrs_xdep_prep_info_table(table);
}

unsigned
(size_xdep_prep_info_table)(
	xdep_prep_info_table *	table
	)
{
  return size_xdep_prep_info_table(table);
}
