/****************************************************************/
/* This is an ADT that Expands into a hash-table indexed by	*/
/* an addresss/word pair.					*/
/*								*/
/* The implementation uses the high order bits of address to	*/
/* index into a subtable, then uses the low order bits to	*/
/* select an entry in the subtable. The lowest 2 bits are 	*/
/* assumed to be 0. The entry is then searched for a matching	*/
/* word.							*/
/*								*/
/* This ADT assumes that the user type xdep_prep_info provides the	*/
/* following interfaces:					*/
/*	xdep_prep_info * xdep_prep_info_over(xdep_prep_info * type)				*/
/*		gets the type that was last set_xdep_prep_info_over (if some set_xdep_prep_info_over	*/
/*		performed), otherwise 0.			*/
/*	void set_xdep_prep_info_over(xdep_prep_info * type, xdep_prep_info * over)			*/
/*		provides value for xdep_prep_info_over to read			*/
/*	unsigned xdep_prep_info_iword(xdep_prep_info type)				*/
/*		returns word for xdep_prep_info				*/
/*								*/
/* Other Expand parameters are:					*/
/*	dep_prep_info.h: an include file, typically where xdep_prep_info is	*/
/*		defined						*/
/*	ffdep_prep_info_table:	the file prefixes for the various files		*/
/*	xdep_prep_info:	the name of the type				*/
/*	FFREADER_PREP_INFO:	a unique distinguishing prefix for the type	*/
/*	10:	the number of bits in the first level table	*/
/*								*/
/* Also:							*/
/*	xdep_prep_info_table: the type of the table (and a suffix for	*/
/*		all functions that use it); _COMPULSORY_ if	*/
/*		xdep_prep_info is not a single word. If appears, must 	*/
/*		be before xdep_prep_info in the .trans file		*/
/****************************************************************/

#ifndef H_ffdep_prep_info_table
#define H_ffdep_prep_info_table

typedef struct xdep_prep_info_table	xdep_prep_info_table;

#include "ansi_stdio.h"
#include "dep_prep_info.h"
#include "ffdep_prep_info_table-private.h"


/****************************************************************/
/* These are the routines to be used while creating the table	*/
/****************************************************************/

void	init_xdep_prep_info_table(xdep_prep_info_table * table);

void	add_xdep_prep_info_table(xdep_prep_info_table * table,
			unsigned addr, xdep_prep_info * entry );

xdep_prep_info *	find_xdep_prep_info_table(xdep_prep_info_table * table,
			unsigned addr, unsigned word );

void	write_xdep_prep_info_table(FILE * fp, xdep_prep_info_table * table);


/****************************************************************/
/* These are the routines to be used while processing an	*/
/* already created table that has been written to disk		*/
/* Note that the following are also available as macros		*/
/*	- array_xdep_prep_info_table					*/
/*	- addrs_xdep_prep_info_table					*/
/*	- size_xdep_prep_info_table					*/
/*	- index_xdep_prep_info_table					*/
/****************************************************************/

void	read_xdep_prep_info_table(FILE * fp, xdep_prep_info_table * table);

xdep_prep_info *	(array_xdep_prep_info_table)(xdep_prep_info_table * table);

unsigned * (addrs_xdep_prep_info_table)(xdep_prep_info_table * table);

unsigned (size_xdep_prep_info_table)(xdep_prep_info_table *table);

xdep_prep_info * (get_xdep_prep_info_table)(xdep_prep_info_table * table,
			unsigned addr, unsigned word );

xdep_prep_info *	find_1st_xdep_prep_info_table(xdep_prep_info_table * table,
			unsigned addr);

#endif
