#include "ansi_assert.h"
#include "ansi_stdio.h"
#include "ansi_stdlib.h"
#include "dep_prep_info.h"

void
write_rochdep_prep(
	FILE * fp,
	unsigned size,
	xdep_prep_info * info,
	unsigned osize,
	xarch * overflow,
	unsigned xsize,
	xdep_prep_info * expand
	)
{
  unsigned		total = size + xsize;
  size_t		ret;
  unsigned		offset;
  xarch *	ptr;
  xdep_prep_info *	eptr;
  unsigned		i;
  xdep_prep_info *	entry;

  ret = fwrite( &size, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "write_rochdep_prep: error writing size\n");
    exit( EXIT_FAILURE );
  }

  ret = fwrite( &xsize, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "write_rochdep_prep: error writing expand size\n");
    exit( EXIT_FAILURE );
  }

  ret = fwrite( &osize, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "write_rochdep_prep: error writing reg over size\n");
    exit( EXIT_FAILURE );
  }

  for( i = 0; i < size; i++ ) {
    entry = &info[i];
    if( xdep_prep_info_is_over(entry) ) {
      offset = xdep_prep_info_overflow(entry) - overflow;
      assert( offset >= 0 );
      assert( offset < osize );
      xdep_prep_info_over_off(entry) = offset;
    }
    if( xdep_prep_info_is_expand(entry) ) {
      offset = xdep_prep_info_expands(entry) - expand;
      assert( offset >= 0 );
      assert( offset < xsize );
      xdep_prep_info_exp_off(entry) = offset+size;
    }
  }

  for( i = 0; i < xsize; i++ ) {
    entry = &expand[i];
    if( xdep_prep_info_is_over(entry) ) {
      offset = xdep_prep_info_overflow(entry) - overflow;
      assert( offset >= 0 );
      assert( offset < osize );
      xdep_prep_info_over_off(entry) = offset;
    }

    assert(!xdep_prep_info_is_expand(entry) );
    xdep_prep_info_exp_off(entry) = 0;
  }

  ret = fwrite( info, sizeof( xdep_prep_info ), size, fp );
  if( ret != size ) {
    fprintf(stderr, "write_rochdep_prep: error writing %d entries\n", size);
    exit( EXIT_FAILURE );
  }

  ret = fwrite( expand, sizeof( xdep_prep_info ), xsize, fp );
  if( ret != xsize ) {
    fprintf(stderr, "write_rochdep_prep: error writing %d expands\n", xsize);
    exit( EXIT_FAILURE );
  }

  ret = fwrite( overflow , sizeof( xarch ), osize, fp );
  if( ret != osize ) {
    fprintf(stderr, "write_rochdep_prep: error writing %d reg overs\n", osize);
    exit( EXIT_FAILURE );
  }

  for( i = 0; i < size; i++ ) {
    entry = &info[i];
    if( xdep_prep_info_is_over(entry) ) {
      ptr = overflow + xdep_prep_info_over_off(entry);
      xdep_prep_info_overflow(entry) = ptr;
    }
    if( xdep_prep_info_is_expand(entry) ) {
      eptr = expand + (xdep_prep_info_exp_off(entry)-size);
      xdep_prep_info_expands(entry) = eptr;
    }
  }

  for( i = 0; i < xsize; i++ ) {
    entry = &info[i];
    if( xdep_prep_info_is_over(entry) ) {
      ptr = overflow + xdep_prep_info_over_off(entry);
      xdep_prep_info_overflow(entry) = ptr;
    }
  }

}

void
read_rochdep_prep(
	FILE * fp,
	unsigned * psize,
	unsigned * pxsize,
	xdep_prep_info ** pinfo,
	unsigned * posize,
	xarch ** poverflow
	)
{
  int			ret;
  unsigned		size;
  unsigned		xsize;
  xdep_prep_info *	info;
  unsigned		osize;
  xarch *	overflow;
  unsigned		i;
  xarch *	ptr;
  xdep_prep_info *	entry;

  ret = fread( psize, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "read_rochdep_prep: error reading size\n");
    exit( EXIT_FAILURE );
  }

  ret = fread( pxsize, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "read_rochdep_prep: error reading expand size\n");
    exit( EXIT_FAILURE );
  }

  ret = fread( posize, sizeof( unsigned ), 1, fp );
  if( ret != 1 ) {
    fprintf(stderr, "read_rochdep_prep: error reading osize\n");
    exit( EXIT_FAILURE );
  }

  size = *psize;
  xsize = *pxsize;
  osize = *posize;

  info = (xdep_prep_info *)malloc( sizeof( xdep_prep_info )*(size+xsize) );
  if( info == 0 ) {
    fprintf(stderr, "read_rochdep_prep: out of memory allocating entries\n");
    exit( EXIT_FAILURE );
  }

  overflow = (xarch *)malloc( sizeof( xarch )*osize );
  if( osize != 0 && overflow == 0 ) {
    fprintf(stderr, "read_rochdep_prep: out of memory allocating overflow\n");
    exit( EXIT_FAILURE );
  }

  ret = fread( info, sizeof( xdep_prep_info ), size, fp);
  if( ret != size ) {
    fprintf(stderr, "read_rochdep_prep: error reading %d entries\n", size);
    exit( EXIT_FAILURE );
  }

  ret = fread( &info[size], sizeof( xdep_prep_info ), xsize, fp);
  if( ret != xsize ) {
    fprintf(stderr, "read_rochdep_prep: error reading %d expands\n", xsize);
    exit( EXIT_FAILURE );
  }

  ret = fread( overflow, sizeof( xarch ), osize, fp);
  if( ret != osize ) {
    fprintf(stderr, "read_rochdep_prep: error reading %d reg overs\n", osize);
    exit( EXIT_FAILURE );
  }

  for( i = 0; i < (size+xsize); i++ ) {
    entry = &info[i];
    if( xdep_prep_info_is_over(entry) ) {
      ptr = overflow + xdep_prep_info_over_off(entry);
      xdep_prep_info_overflow(entry) = ptr;
    }
  }

  *pinfo = info;
  *poverflow = overflow;

}
