#include <time.h>
#include "text_xcoff.h"
#include "ansi_stdlib.h"
#include "ansi_stdio.h"

void
dump_filehdr(
	FILE *			fp,
	struct filehdr *	hdr
	)
{
  fprintf(fp, "%18s: 0%o [0737]\n", "magic number", magic_filehdr(hdr));
  fprintf(fp, "%18s: %u\n", "#sections", nsecns_filehdr(hdr));
  fprintf(fp, "%18s: %u = %s", "time & date", time_filehdr(hdr),
	ctime(&time_filehdr(hdr)));
  fprintf(fp, "%18s: %u\n", "symbol table", symtab_filehdr(hdr));
  fprintf(fp, "%18s: %u\n", "#symbols", nsyms_filehdr(hdr));
  fprintf(fp, "%18s: %u [72]\n", "opt hdr size", optsize_filehdr(hdr));
  fprintf(fp, "%18s: %x =%s%s%s%s%s%s\n", "flags", flags_filehdr(hdr),
  	is_reloc_filehdr(hdr)?" reloc":"",
  	is_exec_filehdr(hdr)?" exec":"",
  	is_linenum_filehdr(hdr)?" linenum":"",
  	is_bigendian_filehdr(hdr)?" bigendian":"",
  	is_dynamic_filehdr(hdr)?" dynamic":"",
  	is_shared_filehdr(hdr)?" shared":"",
  	is_loadonly_filehdr(hdr)?" loadonly":""
	);
}

void
dump_aouthdr(
	FILE *			fp,
	struct aouthdr *	hdr
	)
{
  fprintf(fp, "%18s: 0x%x [0x10b]\n", "magic number", magic_aouthdr(hdr));
  fprintf(fp, "%18s: %u [1]\n", "version number", version_aouthdr(hdr));
  fprintf(fp, "%18s: %u\n", "text size", text_size_aouthdr(hdr));
  fprintf(fp, "%18s: %u\n", "data size", data_size_aouthdr(hdr));
  fprintf(fp, "%18s: %u\n", "bss size", bss_size_aouthdr(hdr));
  fprintf(fp, "%18s: %u\n", "entry", magic_aouthdr(hdr));
  fprintf(fp, "%18s: %08x\n", "text start", text_start_aouthdr(hdr));
  fprintf(fp, "%18s: %08x\n", "data start", data_start_aouthdr(hdr));
  fprintf(fp, "%18s: %08x\n", "toc addr", toc_addr_aouthdr(hdr));
  fprintf(fp, "%18s: %u\n", "entry section", entry_secn_aouthdr(hdr));
  fprintf(fp, "%18s: %u\n", "text section", text_secn_aouthdr(hdr));
  fprintf(fp, "%18s: %u\n", "data section", data_secn_aouthdr(hdr));
  fprintf(fp, "%18s: %u\n", "toc section", toc_secn_aouthdr(hdr));
  fprintf(fp, "%18s: %u\n", "bss section", bss_secn_aouthdr(hdr));
  fprintf(fp, "%18s: %x\n", "text align", text_align_aouthdr(hdr));
  fprintf(fp, "%18s: %x\n", "data align", data_align_aouthdr(hdr));
  fprintf(fp, "%18s: %x %x\n", "module type", module_type_aouthdr(hdr)[0], module_type_aouthdr(hdr)[1]);
  fprintf(fp, "%18s: %x [0]\n", "cpu flag", cpu_flag_aouthdr(hdr));
  fprintf(fp, "%18s: %x [0]\n", "cpu type", cpu_type_aouthdr(hdr));
  fprintf(fp, "%18s: %x\n", "max stack", max_stack_aouthdr(hdr));
  fprintf(fp, "%18s: %x\n", "max data", max_data_aouthdr(hdr));
}

void
dump_secnhdr(
	FILE * fp,
	struct scnhdr *	hdr
	)
{
  if( name_secnhdr(hdr)[7] == 0 ) {
    fprintf(fp, "%18s: %s\n", "name", name_secnhdr(hdr));
  }
  else {
    fprintf(fp, "%18s: %c%c%c%c%c%c%c%c\n", "name",
	name_secnhdr(hdr)[0],
	name_secnhdr(hdr)[1],
	name_secnhdr(hdr)[2],
	name_secnhdr(hdr)[3],
	name_secnhdr(hdr)[4],
	name_secnhdr(hdr)[5],
	name_secnhdr(hdr)[6],
	name_secnhdr(hdr)[7]);
  }
  fprintf(fp, "%18s: %08x\n", "physical address", paddr_secnhdr(hdr));
  fprintf(fp, "%18s: %08x\n", "virtual address", vaddr_secnhdr(hdr));
  fprintf(fp, "%18s: %u\n", "size", size_secnhdr(hdr));
  fprintf(fp, "%18s: %u\n", "data offset", offset_secnhdr(hdr));
  fprintf(fp, "%18s: %u\n", "reloc offset", reloc_secnhdr(hdr));
  fprintf(fp, "%18s: %u\n", "line offset", line_secnhdr(hdr));
  fprintf(fp, "%18s: %u\n", "#relocs", nrelocs_secnhdr(hdr));
  fprintf(fp, "%18s: %u\n", "#lines", nlines_secnhdr(hdr));
  fprintf(fp, "%18s: %x%s%s%s%s%s%s%s%s%s%s\n", "flags", flags_secnhdr(hdr),
	is_pad_secnhdr(hdr)?" pad":"",
	is_text_secnhdr(hdr)?" text":"",
	is_data_secnhdr(hdr)?" data":"",
	is_bss_secnhdr(hdr)?" bss":"",
	is_exception_secnhdr(hdr)?" exception":"",
	is_comment_secnhdr(hdr)?" comment":"",
	is_loader_secnhdr(hdr)?" loader":"",
	is_debug_secnhdr(hdr)?" debug":"",
	is_typecheck_secnhdr(hdr)?" typecheck":"",
	is_overflow_secnhdr(hdr)?" overflow":""
	);

}

void
get_text_xcoff(
	FILE *			fpin,
	struct xcoffhdr **	p_xcoffhdr,
	unsigned *		p_nsecns,
	struct scnhdr **	p_secns,
	unsigned *		p_ntexts,
	struct text_section **	p_text_sections
	)
{
  struct xcoffhdr *	hdr;
  struct filehdr *	filehdr;
  struct aouthdr *	aouthdr;
  struct scnhdr	*	secns;
  struct scnhdr	**	texts;
  struct scnhdr	*	secn;
  unsigned		nsecns;
  unsigned		ntexts;
  int			i;
  int			j;
  int			pos;
  unsigned		offset;
  unsigned		size;
  unsigned *		insns;
  unsigned 		insn;
  unsigned 		addr;
  struct text_section *	text_sections;
  struct text_section *	text_section;
  int			status;

  hdr = (struct xcoffhdr *) malloc( sizeof( struct xcoffhdr) );
  if( hdr == 0 ) {
    fprintf(stderr, "get_text_xcoff: out-of-memory allocating xcoffhdr\n");
    exit( EXIT_FAILURE);
  }
  status = fread( hdr, sizeof(struct xcoffhdr), 1, fpin);
  if( status != 1 ) {
    fprintf(stderr, "get_text_xcoff: error reading xcoffhdr\n");
    exit( EXIT_FAILURE);
  }

  filehdr = filehdr_xcoffhdr(hdr);
  aouthdr = aouthdr_xcoffhdr(hdr);

  nsecns = nsecns_filehdr(filehdr);
  secns = (struct scnhdr *) malloc( sizeof(struct scnhdr) * nsecns );
  if( secns == 0 ) {
    fprintf(stderr, "get_text_xcoff: out-of-memory allocating scnhdr\n");
    exit( EXIT_FAILURE);
  }
  status = fread( secns, sizeof(struct scnhdr), nsecns, fpin);
  if( status != nsecns ) {
    fprintf(stderr, "get_text_xcoff: error reading scnhdr\n");
    exit( EXIT_FAILURE);
  }

  ntexts = 0;
  for( i = 0; i < nsecns; i++ ) {
    secn = &secns[i];
    if( is_text_secnhdr(secn) ) {
      ntexts++;
    }
  }

  texts = (struct scnhdr **) malloc( sizeof(struct scnhdr *) * ntexts );
  if( texts  == 0 ) {
    fprintf(stderr, "get_text_xcoff: out-of-memory allocating text scnhdr\n");
    exit( EXIT_FAILURE);
  }

  ntexts = 0;
  for( i = 0; i < nsecns; i++ ) {
    secn = &secns[i];
    if( is_text_secnhdr(secn) ) {
      texts[ntexts] = secn;
      ntexts++;
    }
  }

  for( i = 0; i < ntexts-1; i++ ) {
    pos = i;
    for( j = i+1; j < ntexts; j++ ) {
      if( offset_secnhdr(texts[pos]) > offset_secnhdr(texts[j]) ) {
	pos = j;
      }
    }
    secn = texts[i];
    texts[i] = texts[pos];
    texts[pos] = secn;
  }

  text_sections = (struct text_section *) malloc( sizeof(struct text_section) * ntexts );
  if( text_sections == 0 ) {
    fprintf(stderr, "get_text_xcoff: out-of-memory allocating text_section\n");
    exit( EXIT_FAILURE);
  }

  for( i = 0; i < ntexts; i++ ) {
    secn = texts[i];
    text_section = &text_sections[i];

    offset = offset_secnhdr(secn);
    status = fseek( fpin, offset, SEEK_SET);
    if( status != 0 ) {
      fprintf(stderr, "ERROR: fseek\n");
      exit( EXIT_FAILURE );
    }
    size = size_secnhdr(secn);
    insns = (unsigned *)malloc( size );
    addr = vaddr_secnhdr(secn);
    fread( insns, 1, size, fpin);

    address_text_section(text_section) = addr;
    size_text_section(text_section) = size;
    insns_text_section(text_section) = insns;
  }
  free( texts );

  *p_xcoffhdr = hdr;
  *p_nsecns = nsecns;
  *p_secns = secns;
  *p_ntexts = ntexts;
  *p_text_sections = text_sections;
}
