#ifndef H_text_xcoff
#define H_text_xcoff

#include <xcoff.h>
#include "ansi_stdio.h"

/****************************************************************/
/* xcoffhdr macros						*/
/****************************************************************/

#define deref_xcoffhdr(x)	(x)
#define filehdr_xcoffhdr(x)	(&(deref_xcoffhdr(x)->filehdr))
#define aouthdr_xcoffhdr(x)	(&(deref_xcoffhdr(x)->aouthdr))

/****************************************************************/
/* filehdr macros						*/
/****************************************************************/

#define deref_filehdr(f)	(f)
#define magic_filehdr(f)	(deref_filehdr(f)->f_magic)
#define nsecns_filehdr(f)	(deref_filehdr(f)->f_nscns)
#define time_filehdr(f)		(deref_filehdr(f)->f_timdat)
#define symtab_filehdr(f)	(deref_filehdr(f)->f_symptr)
#define nsyms_filehdr(f)	(deref_filehdr(f)->f_nsyms)
#define optsize_filehdr(f)	(deref_filehdr(f)->f_opthdr)
#define flags_filehdr(f)	(deref_filehdr(f)->f_flags)

#define is_reloc_filehdr(f)	((flags_filehdr(f)&F_RELFLG)!=0)
#define is_exec_filehdr(f)	((flags_filehdr(f)&F_EXEC)!=0)
#define is_linenum_filehdr(f)	((flags_filehdr(f)&F_LNNO)!=0)
#define is_bigendian_filehdr(f)	((flags_filehdr(f)&F_AR32W)!=0)
#define is_dynamic_filehdr(f)	((flags_filehdr(f)&F_DYNLOAD)!=0)
#define is_shared_filehdr(f)	((flags_filehdr(f)&F_SHROBJ)!=0)
#define is_loadonly_filehdr(f)	((flags_filehdr(f)&F_LOADONLY)!=0)

void dump_filehdr( FILE * fp, struct filehdr * hdr );

/****************************************************************/
/* aouthdr macros						*/
/****************************************************************/
#define deref_aouthdr(a)	(a)
#define magic_aouthdr(a)	(deref_aouthdr(a)->magic)
#define version_aouthdr(a)	(deref_aouthdr(a)->vstamp)
#define text_size_aouthdr(a)	(deref_aouthdr(a)->tsize)
#define data_size_aouthdr(a)	(deref_aouthdr(a)->dsize)
#define bss_size_aouthdr(a)	(deref_aouthdr(a)->bsize)
#define entry_aouthdr(a)	(deref_aouthdr(a)->entry)
#define text_start_aouthdr(a)	(deref_aouthdr(a)->text_start)
#define data_start_aouthdr(a)	(deref_aouthdr(a)->data_start)
#define toc_addr_aouthdr(a)	(deref_aouthdr(a)->o_toc)
#define entry_secn_aouthdr(a)	(deref_aouthdr(a)->o_snentry)
#define text_secn_aouthdr(a)	(deref_aouthdr(a)->o_sntext)
#define data_secn_aouthdr(a)	(deref_aouthdr(a)->o_sndata)
#define toc_secn_aouthdr(a)	(deref_aouthdr(a)->o_sntoc)
#define bss_secn_aouthdr(a)	(deref_aouthdr(a)->o_snbss)
#define text_align_aouthdr(a)	(deref_aouthdr(a)->o_algntext)
#define data_align_aouthdr(a)	(deref_aouthdr(a)->o_algndata)
#define module_type_aouthdr(a)	(deref_aouthdr(a)->o_modtype)
#define cpu_flag_aouthdr(a)	(deref_aouthdr(a)->o_cpuflag)
#define cpu_type_aouthdr(a)	(deref_aouthdr(a)->o_cputype)
#define max_stack_aouthdr(a)	(deref_aouthdr(a)->o_maxstack)
#define max_data_aouthdr(a)	(deref_aouthdr(a)->o_maxdata)
#define resv_aouthdr(a)		(deref_aouthdr(a)->resv2)

void dump_aouthdr( FILE * fp, struct aouthdr * hdr);

/****************************************************************/
/* section hdr macros						*/
/****************************************************************/
#define	deref_secnhdr(s)	(s)
#define name_secnhdr(s)		(deref_secnhdr(s)->s_name)
#define paddr_secnhdr(s)	(deref_secnhdr(s)->s_paddr)
#define vaddr_secnhdr(s)	(deref_secnhdr(s)->s_vaddr)
#define size_secnhdr(s)		(deref_secnhdr(s)->s_size)
#define offset_secnhdr(s)	(deref_secnhdr(s)->s_scnptr)
#define reloc_secnhdr(s)	(deref_secnhdr(s)->s_relptr)
#define line_secnhdr(s)		(deref_secnhdr(s)->s_lnnoptr)
#define nrelocs_secnhdr(s)	(deref_secnhdr(s)->s_nreloc)
#define nlines_secnhdr(s)	(deref_secnhdr(s)->s_nlnno)
#define flags_secnhdr(s)	(deref_secnhdr(s)->s_flags)

#define is_pad_secnhdr(s)	((flags_secnhdr(s)&STYP_PAD)!=0)
#define is_text_secnhdr(s)	((flags_secnhdr(s)&STYP_TEXT)!=0)
#define is_data_secnhdr(s)	((flags_secnhdr(s)&STYP_DATA)!=0)
#define is_bss_secnhdr(s)	((flags_secnhdr(s)&STYP_BSS)!=0)
#define is_exception_secnhdr(s)	((flags_secnhdr(s)&STYP_EXCEPT)!=0)
#define is_comment_secnhdr(s)	((flags_secnhdr(s)&STYP_INFO)!=0)
#define is_loader_secnhdr(s)	((flags_secnhdr(s)&STYP_LOADER)!=0)
#define is_debug_secnhdr(s)	((flags_secnhdr(s)&STYP_DEBUG)!=0)
#define is_typecheck_secnhdr(s)	((flags_secnhdr(s)&STYP_TYPCHK)!=0)
#define is_overflow_secnhdr(s)	((flags_secnhdr(s)&STYP_OVRFLO)!=0)

void dump_secnhdr( FILE * fp, struct scnhdr * hdr);

/****************************************************************/
/* text section declarations					*/
/****************************************************************/
struct text_section {
  unsigned	address;
  unsigned	size;
  unsigned *	insns;
};

#define deref_text_section(t)	(t)
#define address_text_section(t)	(deref_text_section(t)->address)
#define size_text_section(t)	(deref_text_section(t)->size)
#define insns_text_section(t)	(deref_text_section(t)->insns)

void
get_text_xcoff(
	FILE *			fpin,
	struct xcoffhdr **	p_xcoffhdr,
	unsigned *		p_nsecns,
	struct scnhdr **	p_secns,
	unsigned *		p_ntexts,
	struct text_section **	p_text_sections
	);
#endif
