#ifndef H_insn_info
#define H_insn_info

// TT6 structures found from AmberTraceFormats.pdf
//
// Although I could rewrite this to use the field names from the fF
// decoding routines in Turandot, I'm retaining Apple's syntax to
// keep things clear with Amber's specifications. -jdonald

// To avoiding printf() warnings and type definition problems,
// I'm using 'unsigned' instead of 'uint32_t'

// little endian, so I had to reverse the order of these. -jdonald
typedef struct {
#if !defined( SWITCH_ENDIAN ) // i don't completely understand why this reordering is necessary...
	unsigned escape : 6;
	unsigned code : 10;
	unsigned count : 16;
#else // !SWITCH_ENDIAN
	unsigned count : 16;	// 16-32 number of words following escape record
	unsigned code : 10;		// 6-15 code:
							//   0x00 = segment register
							//   0x01 = optional data address for next instruction
							//   0x02 = condition register value
							//   0x03 = real address of branch target for next instruction
							//   0x04 = real data address for next instruction
							//   0x05 = real address of next instruction
							//   0x20-0x3F = thread synchronization
	unsigned escape : 6;	// 0-5 escape record indicator - always zero
#endif // !SWITCH_ENDIAN
} tt6_escape_record_fields_t;

typedef union {
	tt6_escape_record_fields_t field;
	unsigned value;
} tt6_escape_record_t;

typedef enum {
	SYNC_SIGNAL				= 0x20,
	SYNC_BROADCAST_SIGNAL	= 0X21,
	SYNC_WAIT				= 0x30,
	SYNC_TRY_WAIT			= 0x31
} sync_type_t;
// end of Amber spec

typedef struct { // this structure is not from Amber, but rather made by me. -jdonald
	unsigned code;
	unsigned address;
	unsigned count;
} syncrec_t;

typedef struct xinsn_info xinsn_info;

struct xinsn_info {
  unsigned		iaddr;
  unsigned		iword;
  unsigned		daddr;
  unsigned 		dlength;
#if( USING_FF52_SEGS )
  unsigned		dseg;
  unsigned		iseg;
#endif
  syncrec_t syncrec; // jdonald
};

#define xinsn_info_deref(i)		(i)
#define xinsn_info_iaddr(i)		(xinsn_info_deref(i).iaddr)
#define xinsn_info_iword(i)		(xinsn_info_deref(i).iword)
#define xinsn_info_daddr(i)		(xinsn_info_deref(i).daddr)
#define xinsn_info_dlength(i)	(xinsn_info_deref(i).dlength)
#if( USING_FF52_SEGS )
#define xinsn_info_dseg(i)		(xinsn_info_deref(i).dseg)
#define xinsn_info_iseg(i)		(xinsn_info_deref(i).iseg)
#endif
#define xinsn_info_syncrec(i)	(xinsn_info_deref(i).syncrec) // jdonald

#endif
