#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "trace_decompress.h"


#define TRACE_COMPRESS_MAX 256

enum {
  TRACE_COMPRESS_TYPE_NONE = 0,
  TRACE_COMPRESS_TYPE_GZIP,
  TRACE_COMPRESS_TYPE_BZIP2
};

FILE* trace_compress_stream[TRACE_COMPRESS_MAX];
int trace_compress_type[TRACE_COMPRESS_MAX];
int trace_compress_count;

FILE* trace_open(char *fname, char *type)
{
  FILE *trace_stream;
  char cmd_path[256];
  char *ext;
  
  ext = strrchr(fname, '.');
  if (ext != NULL && *ext != '\0') {
    if (!strcmp(ext, ".GZ") || !strcmp(ext, ".gz")) {
#ifdef USE_ZLIB 
      trace_stream = (gzFile) gzopen(fname, "rb");
      trace_compress_stream[trace_compress_count] =
	trace_stream;
      trace_compress_type[trace_compress_count] =
	TRACE_COMPRESS_TYPE_GZIP;
      trace_compress_count++;
#else
      sprintf(cmd_path, "%s -dc %s", GZPATH, fname);
      trace_stream = popen(cmd_path, type);
#endif      

      return trace_stream;
    }
    else if (!strcmp(ext, ".BZ") || !strcmp(ext, ".bz") ||
	     !strcmp(ext, ".BZ2") || !strcmp(ext, ".bz2")) {
#ifdef USE_BZLIB
      trace_stream = (BZFILE*) BZ2_bzopen(fname, "rb");
      trace_compress_stream[trace_compress_count] =
	trace_stream;
      trace_compress_type[trace_compress_count] =
	TRACE_COMPRESS_TYPE_BZIP2;
      trace_compress_count++;
#else
      sprintf(cmd_path, "%s -dc %s", BZ2PATH, fname);
      trace_stream = popen(cmd_path, type);     
#endif            
      return trace_stream;
    }
  }

  trace_stream = fopen(fname, type);
  return trace_stream;
}

// jdonald wrote trace_close()
int trace_close(FILE* stream)
{
  int trace_compress_index;

  for(trace_compress_index = 0; 
      trace_compress_index < trace_compress_count;
      trace_compress_index++) {
    if (trace_compress_stream[trace_compress_index] == stream)
      break;
  }
  if (trace_compress_index < trace_compress_count) {
    assert((trace_compress_type[trace_compress_index] ==
            TRACE_COMPRESS_TYPE_GZIP) ||
           (trace_compress_type[trace_compress_index] ==
            TRACE_COMPRESS_TYPE_BZIP2));
    if (trace_compress_type[trace_compress_index] ==
        TRACE_COMPRESS_TYPE_GZIP) {
#ifdef USE_ZLIB
      return fclose(stream);
#else
      return pclose(stream) != -1;
#endif
    }
    else if (trace_compress_type[trace_compress_index] ==
        TRACE_COMPRESS_TYPE_BZIP2) {
#ifdef USE_BZLIB
      BZ2_bzclose(stream);
      return 0;
#else
      return pclose(stream) != -1;
#endif
    }
    return -1;
  }
  else
    return fclose(stream);
}

int trace_read(void *buf, size_t elem_size, 
	       size_t elem_count, FILE *stream)
{
  int read_count = 0;
  int trace_compress_index;

  for(trace_compress_index = 0; 
      trace_compress_index < trace_compress_count;
      trace_compress_index++) {
    if (trace_compress_stream[trace_compress_index] == stream)
      break;
  }

  if (trace_compress_index < trace_compress_count) {
    assert((trace_compress_type[trace_compress_index] ==
	    TRACE_COMPRESS_TYPE_GZIP) ||
	   (trace_compress_type[trace_compress_index] ==
	    TRACE_COMPRESS_TYPE_BZIP2));


    if (trace_compress_type[trace_compress_index] ==
	TRACE_COMPRESS_TYPE_GZIP) {
#ifdef USE_ZLIB
      read_count = gzread(stream, buf, elem_size * elem_count);
      read_count = read_count / elem_size;
#else
      assert(!"ZLIB used for opening but not reading");
#endif
    }
    else if (trace_compress_type[trace_compress_index] ==
	TRACE_COMPRESS_TYPE_BZIP2) {
#ifdef USE_BZLIB
      read_count = BZ2_bzread(stream, buf, elem_size * elem_count);
      read_count = read_count / elem_size;
#else
      assert(!"BZLIB used for opening but not reading");
#endif
    }
  }
  else {
    read_count = fread(buf, elem_size, elem_count, stream);
  }
  
  return read_count;
}

