/******************************************************************************
 This file contains routines to read an fF trace.
******************************************************************************/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include "opc.h"
#include "insn_info.h"
#include "trace_ffreader.h"
#include "trace_decompress.h"

#if( !defined(DODBG) )
#define DODBG	0
#endif

#if( DODBG )
#define DBG(x)	((void)(x))
#else
#define DBG(x)	((void)0)
#endif

/* switch endian, refer to <<Migarating C and C++ Applications to AIX5L on IA-64>> for more details, zhigangh@1/24/2003 */
#define SWITCH_ENDIAN_32BIT(x)	(((x) << 24) |				\
			 (((x) << 8)  & 0x00ff0000) |		\
			 (((x) >> 8)  & 0x0000ff00) |		\
			 (((x) >> 24) & 0x000000ff))

#define trace_ffreader_get_word(word, ptr, len, buf, reader, lbl)\
  {						\
    if( (ptr) == (len) ) {			\
      (ptr) = 0;				\
      (len) = trace_ffreader_fill_buf(reader);	\
      if( (len) == 0 ) {			\
	goto lbl;				\
      }						\
    }						\
    (word) = (buf)[ptr];			\
	if (SWITCH_ENDIAN)				\
	{								\
		(word) = SWITCH_ENDIAN_32BIT(word);	\
	}								\
    (ptr)++;					\
  }

#define trace_ffreader_peek_word(word, ptr, len, buf, reader, lbl)\
  {						\
    if( (ptr) == (len) ) {			\
      (ptr) = 0;				\
      (len) = trace_ffreader_fill_buf(reader);	\
      if( (len) == 0 ) {			\
	goto lbl;				\
      }						\
    }						\
    (word) = (buf)[ptr];			\
  }


static int
trace_ffreader_fill_buf(
	trace_ffreader * reader
	)
{
  unsigned	num;

  
  num = trace_read(trace_ffreader_buf(reader), sizeof(unsigned),
		   TRACE_FFREADER_BUF_SIZE,
		   trace_ffreader_fp(reader));
DBG(  fprintf(stderr, "reading from file; got %d records\n", num) );
  return num;
}

void
trace_ffreader_setup(
	trace_ffreader * reader,
	FILE * trace_fp
	)
{
  int		status;
#if( USING_FF52_SEGS ) // jdonald
  int		i;
#endif
  unsigned	addr;

  trace_ffreader_fp(reader) = trace_fp;
  trace_ffreader_is_done(reader) = 0;

  status = trace_read( &addr, sizeof(unsigned), 1, trace_fp);
  if (SWITCH_ENDIAN)
  {
  	addr = SWITCH_ENDIAN_32BIT(addr);
  }
  if( status == 0 ) {
    fprintf(stderr, "trace_ffreader_setup: could not read start address\n");
    exit(EXIT_FAILURE);
  }

  trace_ffreader_first_pc(reader) = addr;
  trace_ffreader_last_pc(reader)= trace_ffreader_first_pc(reader);

  trace_ffreader_buflen(reader)	= trace_ffreader_fill_buf(reader);
  trace_ffreader_bufptr(reader) = 0;
  if( trace_ffreader_buflen(reader) == 0 ) {
    fprintf(stderr, "trace_ffreader_setup: empty trace\n");
    exit( EXIT_FAILURE );
  }

#if( USING_FF52_SEGS )
  for( i = 0; i < 16; i++ ) {
    trace_ffreader_segid(reader, i) = i<<16;
  }
  trace_ffreader_pcseg(reader) = (trace_ffreader_first_pc(reader) & 0xf0000000)>>12;
#endif
}

int
trace_ffreader_fill(
	trace_ffreader *	reader,
	int			max_info,
	xinsn_info		info[]
	)
{
  int			i;
  unsigned		pc;
#if( USING_FF52_SEGS ) // jdonald
  unsigned		pcseg;
#endif
  // unsigned		prev_pc; // unused?
  // unsigned		prev_kind; // unused?
  unsigned *	reader_buf;
  unsigned		reader_bufptr;
  unsigned		reader_buflen;
  unsigned		iword;
  unsigned		kind;
  // unsigned		addr; // unused?
  unsigned		dummy;
#if( USING_FF52_SEGS ) // jdonald
  unsigned		val;
  unsigned		seg;
#endif
  // int			disp; // unused?
  opc_idx		idx;

  if( trace_ffreader_is_done(reader) ) {
    DBG( fprintf(stderr, "already done\n") );
    return 0;
  }

  reader_buf = trace_ffreader_buf(reader);
  pc = trace_ffreader_last_pc(reader);
#if( USING_FF52_SEGS )
  pcseg = trace_ffreader_pcseg(reader);
  assert( trace_ffreader_segid(reader, (pc >> 28) ) == pcseg );
#endif
  i = 0;

// synchronized: // unused?
  reader_bufptr = trace_ffreader_bufptr(reader);
  reader_buflen = trace_ffreader_buflen(reader);
  assert( reader_bufptr <= reader_buflen );

  while( i < max_info ) {

	syncrec_t syncrec = { 0, 0, 0 };
	
	trace_ffreader_get_word(iword, reader_bufptr, reader_buflen,
							reader_buf, reader, done_with_trace);
    while( (iword & 0xFC000000) == 0x0 ) { // TT6 escape record
      switch(iword) {
	    case 0x1:
	      	trace_ffreader_get_word(dummy, reader_bufptr, reader_buflen,
									reader_buf, reader, done_with_trace);
#if( USING_FF52_SEGS )
		  	seg = dummy >> 28;
		 	 val = (dummy & 0xffffff);
		  	if( (pc >> 28) == seg ) {
		    	pcseg = val;
		  	}
		  	trace_ffreader_segid(reader, seg) = val;
#endif
			trace_ffreader_get_word(iword, reader_bufptr, reader_buflen,
									reader_buf, reader, done_with_trace);
			break;

      	case 0x00060001: 
			DBG( fprintf(stderr,"pc == %d\n*TRACEBREAK*\n",pc) );
			trace_ffreader_get_word(pc, reader_bufptr, reader_buflen,
									reader_buf, reader, done_with_trace);
			DBG(	fprintf(stderr,"pc == %d\n",pc) );
#if( USING_FF52_SEGS )
			DBG( fprintf(stderr,"using segs pc == %d\n",pc) );
		  	seg = (pc>>28);
			pcseg = trace_ffreader_segid(reader, seg);
		  	xinsn_info_dseg(info[i]) = pcseg;
#endif
			trace_ffreader_get_word(iword, reader_bufptr, reader_buflen,
									reader_buf, reader, done_with_trace);
			break;

      	default: {
	      	tt6_escape_record_t record; // jdonald
	      	record.value = iword;
	      	switch(record.field.code) {
		      	case SYNC_SIGNAL:
		      	case SYNC_BROADCAST_SIGNAL:
		      	case SYNC_WAIT:
		      	case SYNC_TRY_WAIT:
					assert(record.field.count == 2);
      				assert(syncrec.code == 0); // hopefully there won't be more than one synchronization escape code on a single instruction
      				syncrec.code = record.field.code;
      				// fprintf(stderr, "// [Word 0] 0 : %x : %d\n", record.field.code, record.field.count);
      				trace_ffreader_get_word(syncrec.address, reader_bufptr, reader_buflen,
											reader_buf, reader, done_with_trace);
					// fprintf(stderr, "// [Word 1] sync point address: %08x\n", syncrec.address);
					trace_ffreader_get_word(syncrec.count, reader_bufptr, reader_buflen,
											reader_buf, reader, done_with_trace);
		      		// fprintf(stderr, "// [Word 2] sync point count: %d\n", syncrec.count);
      				trace_ffreader_get_word(iword, reader_bufptr, reader_buflen,
									reader_buf, reader, done_with_trace);
		      		break;
		      	case 0x00: // segment register
				// case 0x01: // condition register value, handled above
				case 0x03: // real address of branch target for next instruction
				case 0x04: // real data address for next instruction
				case 0x05: // real address of next instruction
				default: {
		      		fprintf(stderr,"warning: trace_ffreader: unrecognized TT6 escape record %08x, ", iword);
		      		// assert(0); exit(-1); // with NDEBUG, can't rely on assert() -jdonald
		      		if (record.field.count < 10) {
			      		int j;
		      			fprintf(stderr, "skipping %d words\n", record.field.count);
		      			for (j = 0; j < record.field.count; j += 1)
		      				trace_ffreader_get_word(dummy, reader_bufptr, reader_buflen,
													reader_buf, reader, done_with_trace);
					}
					else
						fprintf(stderr, "ignored\n"); 
      				trace_ffreader_get_word(iword, reader_bufptr, reader_buflen,
											reader_buf, reader, done_with_trace);
				}
      		}
		}
      }
    }

    idx = opc_index(iword);
    kind = opc_ff32_cat(idx);

	DBG( fprintf(stderr, "%d: pc is %d [%x], iword is %x, kind is %d\n",
				 i, pc, pc, iword, kind)); // why was the [%x] 4*pc? /jdonald

    assert( reader_buflen != 0 );

    xinsn_info_iaddr(info[i]) = pc;
#if( USING_FF52_SEGS )
	xinsn_info_iseg(info[i]) = pcseg;
#endif
    xinsn_info_iword(info[i]) = iword;
    xinsn_info_dlength(info[i]) = 0;
    xinsn_info_daddr(info[i]) = 0;
    xinsn_info_syncrec(info[i]) = syncrec;

    switch( kind ) {
		case OPC_FF32_OTHER: {
			pc += 4;
			break;
		}
		
		case OPC_FF32_BRANCH: {
			trace_ffreader_get_word(pc, reader_bufptr, reader_buflen,
			    					reader_buf, reader, done_with_trace);
		
#if( USING_FF52_SEGS )
			seg = (pc>>28);
			pcseg = trace_ffreader_segid(reader, seg);
			xinsn_info_dseg(info[i]) = pcseg;
#endif
		
			xinsn_info_daddr(info[i]) = pc;
		
			break;
		}
		
		case OPC_FF32_MEMOP: {
			trace_ffreader_get_word(dummy, reader_bufptr, reader_buflen,
									reader_buf, reader, done_with_trace);
		
#if( USING_FF52_SEGS )
			seg = (dummy>>28);
			xinsn_info_dseg(info[i]) = trace_ffreader_segid(reader, seg);
#endif
		
			xinsn_info_daddr(info[i]) = dummy;
			xinsn_info_dlength(info[i]) = opc_dlength(idx);
		
			pc += 4;
			break;
		}
		
		case OPC_FF32_MULTI: {
			trace_ffreader_get_word(dummy, reader_bufptr, reader_buflen,
									reader_buf, reader, done_with_trace);
		
#if( USING_FF52_SEGS )
			seg = (dummy>>28);
			xinsn_info_dseg(info[i]) = trace_ffreader_segid(reader, seg);
#endif
		
			xinsn_info_daddr(info[i]) = dummy;
		
			trace_ffreader_get_word(dummy, reader_bufptr, reader_buflen,
									reader_buf, reader, done_with_trace);
			xinsn_info_dlength(info[i]) = dummy;
			pc += 4;
			break;
		}
		default: {
			fprintf(stderr, "pc is %d [%x], iword is %x, kind is %d\n",
				 pc, pc, iword, kind); // why was the [%x] 4*pc? /jdonald
			abort();
			break;
		}
    }
    assert( reader_bufptr <= reader_buflen);
    i++;
  }

  trace_ffreader_bufptr(reader) = reader_bufptr;
  trace_ffreader_buflen(reader) = reader_buflen;
  trace_ffreader_last_pc(reader) = pc;
#if( USING_FF52_SEGS )
  trace_ffreader_pcseg(reader) = pcseg;
#endif
  return max_info;

done_with_trace:;
  trace_ffreader_is_done(reader) = 1;
  trace_ffreader_bufptr(reader) = reader_bufptr;
  trace_ffreader_buflen(reader) = reader_buflen;
  trace_ffreader_last_pc(reader) = pc;
  #if( USING_FF52_SEGS )
    trace_ffreader_pcseg(reader) = pcseg;
  #endif
  return i;
}
