/******************************************************************************
 This file contains routines to the work of RefillBuffer from ravi's xtrace
 pacakge, but faster. It uses a preprocessed version of the xcoff file to
 achieve this.
******************************************************************************/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include "reader_prep_info.h"
#include "insn_info.h"
#include "trace_reader.h"

#if( !defined(DODBG) )
#define DODBG	0
#endif

#if( DODBG )
#define DBG(x)	((void)(x))
#else
#define DBG(x)	((void)0)
#endif


#define xtrace_reader_get_word(ptr, len, buf, reader)	\
	( (( ptr == len )? \
	    ( ptr = 0, len = xtrace_reader_fill_buf(reader)): \
	    0), \
	  buf[ptr++])

#define xtrace_reader_peek_word(ptr, len, buf, reader)	\
	( (( ptr == len )? \
	    ( ptr = 0, len = xtrace_reader_fill_buf(reader)): \
	    0), \
	  buf[ptr])

static int xtrace_reader_nbrabs = 0;

static int
xtrace_reader_fill_buf(	xtrace_reader * reader
	)
{
  unsigned	num;
  num = fread(  xtrace_reader_buf(reader), sizeof(unsigned),
		XTRACE_READER_BUF_SIZE, xtrace_reader_fp(reader) );
DBG(  fprintf(stderr, "reading from file; got %d records\n", num) );
  return num;
}

void
xtrace_reader_setup( xtrace_reader * reader,
		FILE * prep_fp,
		FILE * trace_fp
	)
{
  int		status;
  unsigned	addr;
  unsigned	reader_min_pc;
  xtrace_reader_fp(reader) = trace_fp;
  xtrace_reader_is_done(reader) = 0;

  status = read_reader_prep( prep_fp,
      &xtrace_reader_first_pc(reader),
      &xtrace_reader_min_pc(reader),
      &xtrace_reader_size(reader),
      &xtrace_reader_prep(reader));
  if( status != 0 ) {
    fprintf(stderr, "could not read prep file; status %d\n", status);
    exit( EXIT_FAILURE );
  }

  reader_min_pc = xtrace_reader_min_pc(reader);

  xtrace_reader_buflen(reader)	= xtrace_reader_fill_buf(reader);
  xtrace_reader_last_pc(reader)	= xtrace_reader_first_pc(reader);
  xtrace_reader_max_pc(reader)	= xtrace_reader_min_pc(reader) +
				  (xtrace_reader_size(reader)*4);
  xtrace_reader_last_pc(reader) = (xtrace_reader_first_pc(reader) - reader_min_pc)&~0x3;

  addr = (xtrace_reader_buf(reader)[0]);
  assert((addr & ~0x3) == xtrace_reader_last_pc(reader));
 
  fprintf(stderr, "first pc = %08x prep file size is %d\n",
	xtrace_reader_first_pc(reader),
	xtrace_reader_max_pc(reader));

}

int
xtrace_reader_fill(	xtrace_reader *	reader,
		int		max_info,
		xinsn_info	info[]
	)
{
  unsigned	p;
  unsigned	insn;
  xreader_prep_info *	prep;
  int		read_till;
  int		i;
  unsigned	pc;
  unsigned	prev_pc;
  unsigned	prev_kind;
  xreader_prep_info *	reader_prep;
  unsigned *	reader_buf;
  unsigned	reader_bufptr;
  unsigned	reader_buflen;
  unsigned	reader_text_size;
  unsigned	iword;
  unsigned	kind;
  unsigned	addr;
  unsigned	iaddr;
  int		disp;
  int		sign_disp;

  if( xtrace_reader_is_done(reader) ) {
    fprintf(stderr, "already done\n");
    return 0;
  }

  reader_text_size = (xtrace_reader_size(reader)*4);
  reader_prep = xtrace_reader_prep(reader);
  reader_buf = xtrace_reader_buf(reader);
  iaddr = xtrace_reader_last_pc(reader);
  i = 0;

synchronized:
  reader_bufptr = xtrace_reader_bufptr(reader);
  reader_buflen = xtrace_reader_buflen(reader);

  while( i < max_info ) {

    pc = iaddr >> 2;
    iword = xreader_prep_info_iword(reader_prep[pc]);
    kind = xreader_prep_info_kind(reader_prep[pc]);

DBG( fprintf(stderr, "%d: pc is %d [%x], iword is %x, kind is %d %s\n",
i, pc, 4*pc, iword, kind, xreader_prep_info_kind_name[kind]) );
    assert( reader_buflen != 0 );

    xinsn_info_iaddr(info[i]) = iaddr;
    xinsn_info_iword(info[i]) = iword;
    xinsn_info_daddr(info[i]) = 0;
    xinsn_info_dlength(info[i]) = 0;

    switch( kind ) {
      case xreader_prep_info_is_other: {
	iaddr += 4;
	assert( (iaddr & 0x3 ) == 0 );
	break;
      }
      case xreader_prep_info_is_memop: {
	addr = xtrace_reader_get_word(reader_bufptr, reader_buflen,
					  reader_buf, reader);

	#if( !NDEBUG )
	  if( (addr & 1 ) != 1 ) {
	    fprintf(stderr, "failed at %d, iaddr is %x, %d %s\n",
	      i, iaddr, prev_kind, xreader_prep_info_kind_name[prev_kind]);
	    fprintf(stderr, "ptr = %d len = %d\n", reader_bufptr, reader_buflen);
	  }
	#endif
	assert( (addr & 1) == 1 );

	info[i].daddr = xtrace_reader_get_word(reader_bufptr, reader_buflen,
						  reader_buf, reader);
	info[i].dlength = xtrace_reader_get_word(reader_bufptr, reader_buflen,
						  reader_buf, reader);

	assert( (addr &~0x3) == iaddr );
	prev_kind = kind;
	iaddr += 4;
	assert( (iaddr & 0x3 ) == 0 );
	break;
      }

      case xreader_prep_info_is_branch: {
	prev_kind = kind;
	sign_disp = iword<<6;
	disp = sign_disp>>8;
	iaddr += (sign_disp>>6)&~0x3;

	if( reader_bufptr == reader_buflen ) {
	  addr = xtrace_reader_peek_word(reader_bufptr, reader_buflen,
					  reader_buf, reader);
	  if( reader_buflen == 0 ) {
	    xtrace_reader_is_done(reader) = 1;
	    return i+1;
	  }
	}

	info[i].daddr = iaddr;
	assert( (iaddr & 0x3 ) == 0 );
	break;
      }

      case xreader_prep_info_is_brabs: {
	/*** fprintf(stderr, "found brabs\n"); ***/
	xtrace_reader_nbrabs++;
	prev_kind = kind;
	iaddr += 4;

	info[i].daddr = iaddr;
	assert( (iaddr & 0x3 ) == 0 );
	break;
      }

      case xreader_prep_info_is_brcond: {
	addr = xtrace_reader_get_word(reader_bufptr, reader_buflen,
					  reader_buf, reader);
	prev_kind = kind;
	iaddr = addr;

	info[i].daddr = iaddr;
	assert( (iaddr & 0x3 ) == 0 );
	break;
      }

      case xreader_prep_info_is_brreg: {
	addr = xtrace_reader_get_word(reader_bufptr, reader_buflen,
					  reader_buf, reader);

	assert( (addr & 1) == 0 );
  /*** branches into untranslated code; use next valid pc location to synch ***/
	if( addr > reader_text_size ) {
	  addr = xtrace_reader_peek_word(reader_bufptr, reader_buflen,
					  reader_buf, reader);

	  if( reader_buflen == reader_bufptr ) {
	    xtrace_reader_is_done(reader) = 1;
	    return i+1;
	  }
	}

	assert( reader_buflen != 0);
	prev_kind = kind;
	iaddr =  addr & ~0x3;

	info[i].daddr = iaddr;
	break;
      }
      default: {
	fprintf(stderr, "kind is %d; iword is %x\n", kind, iword);
	abort();
	break;
      }
    }

    assert( reader_bufptr <= reader_buflen);
    assert( (iaddr & 0x3 ) == 0 );
    i++;
  }

  xtrace_reader_bufptr(reader) = reader_bufptr;
  xtrace_reader_buflen(reader) = reader_buflen;
  xtrace_reader_last_pc(reader) = iaddr;
  return max_info;
}
