// array.h

#define TEST_CACHE 0

/* array operation constants */
#define ARRAY_LOOKUP		0
#define ARRAY_HIT_UPDATE	1
#define ARRAY_MISS_UPDATE	2
#define ARRAY_READ			4
#define ARRAY_WRITE			8
#define ARRAY_INVALIDATE	16
#define ARRAY_SHARE			32

#define ARRAY_MISS		0
#define ARRAY_HIT		1

#if TEST_CACHE
#if !defined(NULL)
#define NULL			0
#endif /* !defined(NULL) */

#define the_stderr		stderr
#define EXIT_FAILURE	0
#define FALSE			0
#define TRUE			1
#define lg2(x)		( (((x)>>1) == 0) ? 0 : \
			  (((x)>>2) == 0) ? 1 : \
			  (((x)>>3) == 0) ? 2 : \
			  (((x)>>4) == 0) ? 3 : \
			  (((x)>>5) == 0) ? 4 : \
			  (((x)>>6) == 0) ? 5 : \
			  (((x)>>7) == 0) ? 6 : \
			  (((x)>>8) == 0) ? 7 : \
			  (((x)>>9) == 0) ? 8 : \
			  (((x)>>10) == 0) ? 9 : \
			  (((x)>>11) == 0) ? 10 : \
			  (((x)>>12) == 0) ? 11 : \
			  (((x)>>13) == 0) ? 12 : \
			  (((x)>>14) == 0) ? 13 : \
			  (((x)>>15) == 0) ? 14 : \
			  (((x)>>16) == 0) ? 15 : \
			  (((x)>>17) == 0) ? 16 : \
			  (((x)>>18) == 0) ? 17 : \
			  (((x)>>19) == 0) ? 18 : \
			  (((x)>>20) == 0) ? 19 : \
			  (((x)>>21) == 0) ? 20 : \
			  (((x)>>22) == 0) ? 21 : \
			  (((x)>>23) == 0) ? 22 : \
			  (((x)>>24) == 0) ? 23 : \
			  (((x)>>25) == 0) ? 24 : \
			  (((x)>>26) == 0) ? 25 : \
			  (((x)>>27) == 0) ? 26 : \
			  (((x)>>28) == 0) ? 27 : \
			  (((x)>>29) == 0) ? 28 : \
			  (((x)>>30) == 0) ? 29 : \
			  (((x)>>31) == 0) ? 30 : \
			  31)

#endif /* TEST_CACHE */

#define MK_ADDR(ARRAY, TAG, SET, OFFSET) ((TAG<<(ARRAY)->tag_shift) | (SET<<(ARRAY)->set_shift) | OFFSET)
struct xarray_blk
{
    int thread_id;
    int processor_id; /* LYM CMP */ // jdonald made signed, not unsigned, so that we can set it to -1
    unsigned tag;
    unsigned valid : 1;
    unsigned dirty : 1;
    unsigned shared : 1; // jdonald
    unsigned prev_dirty : 1;
	// M = Modified  = valid && dirty; assert(!shared)
	// E = Exclusive = valid && !dirty && !shared
	// S = Shared    = valid && shared; assert(!dirty)
	// I = Invalid   = !valid; assert(!dirty && !shared)

    /* a unified storage for possible data, e.g. target address in BTB */
    union
    {
    	unsigned nfa_next;
    	unsigned counter;
    }data;

    /* for forming linked list of blks */
    struct xarray_blk *next;
};

struct xarray_set
{
    struct xarray_blk *blks;
};

struct xarray
{
	/* basic parameters */
    char *name;
    
    unsigned n_set;
    unsigned n_way;
    unsigned blk_size;
    
    struct xarray_set *sets;
    struct xarray_blk *blks;

    /* derivative parameters for convenience */
    unsigned tag_shift;
    unsigned set_shift;
    unsigned set_mask;
    unsigned offset_mask;
    
};

/* creating a new array structure with appropriate parameters */
inline struct xarray *array_create(char *name, unsigned n_set, unsigned n_way, unsigned blk_size);
inline struct xarray *shared_array_create(char *name, unsigned n_set, unsigned n_way, unsigned blk_size); /* LYM */

/* access an array with a key, and an operation type (lookup/update/probe) */
inline int array_access(int processor_id, int thread_id, struct xarray *array, unsigned key, unsigned op, struct xarray_blk **pp_blk);
inline void free_array(struct xarray* array);
