#!/usr/bin/perl
#
# "Sources/turandot/src"
#
# wow that's a real descriptive header... -jdonald

use Text::Wrap; 

sub round {
    $float = shift; # /* the number to round */
    $intgr = int($float + 0.5);    
    return $intgr;    
}

sub create_parms_header {
	my $knob_list = $_[0];
	print "creating gp_parms.h from '" . substr($knob_list, 0, 120) . "...': ";
	open FOUT, ">gp_parms.h" or die "cannot open gp_parms.h";

	print FOUT "/* Do not edit this file directly. It is created automatically every time\n";
	print FOUT " * Turandot is built. This file is meant to be an easy way to check the\n";
	print FOUT " * final knob values, but unfortunately these values are still set within\n";
	print FOUT " * messy perl scripts such as compile.smt.pl, gp_parms.pl, super_gp_parms.pl,\n";
	print FOUT " * 750_parms.pl and 405_parms.pl\n";
	print FOUT " */\n";
	print FOUT "\n";
	$Text::Wrap::columns = 78;
	print FOUT wrap("/* ", "   ", "Parsing of: $knob_list");
	print FOUT " */\n";
	print FOUT "\n";
	
	foreach $entry (split(/\s+/, $knob_list) ) {
		if( $entry =~ /^-D(\w+)=(\S+)/ ) {
    		print FOUT "#define $1 $2\n";
		}
		elsif( $entry =~ /^-U(\w+)/ ) {
	   		print FOUT "#undef $1\n";
  		}
  		else {
	   		print "cannot parse entry: $entry\n";
    		exit(-1);
		}
	}
	close FOUT;
	print "done\n";
}

	# $SRCDIR = `pwd`;
	# $SRCDIR = substr($SRCDIR, 0, length($SRCDIR) - 1);

	# $MODELDIR="$SRCDIR";
	# $ARIADIR="$SRCDIR/../../aria";
	# $TURDIR="$SRCDIR/..";

	# printf("$SRCDIR\n");
	# printf("$MODELDIR\n");
	# printf("$ARIADIR\n");
	# printf("$TURDIR\n");
	
	$SRCDIR = ".";
	$MODELDIR = $SRCDIR;
	$ARIADIR = "../../aria";
	$TURDIR = "..";
	
	$CC = $ENV{'CC'};
	if (!$CC) {
		$CC = "gcc";
	}
	
	$OPT_FLAGS = $ENV{'GLOBAL_OPT_CFLAGS'} . " " . $ENV{'GLOBAL_LDFLAGS'};

	require("super_gp_parms.pl");
	require("gp_parms.pl");
	require("750_parms.pl");
	require("405_parms.pl");

	$BRANCH_PARMS = "-DBRANCH_MAX_PRED=12 -DEVALUATE_MISPREDICTED_BRANCHES=FALSE -DBP_COND_LG2SIZE=14 -DBP_COND_BITS=1 -DBP_HISTORY_SHIFT=3 -DBP_LINK_SIZE=16 -DBP_COUNTER_LG2SIZE=5 -DBP_COUNTER_ASSOC=1 -DPERFECT_BRANCH_PREDICTION=FALSE";
	$FETCH_PARMS  = "-DIFETCH_MAX=8 -DIBUF_MAX=64 -DFCHBLK_MAX=8 -DNFA_NEXT_SEQ=TRUE -DUSE_BLOCK_NFA=FALSE -DPERFECT_NFA_PREDICTION=FALSE";
	$DECODE_PARMS = "-DDECODE_ONE_BRANCH=TRUE -DDECODE_EXTRA_BRANCH=TRUE -DDECODE_MAX=9 -DDISPATCH_MAX=9 -DDECODE0_MAX=4 -DDISPATCH_GROUPS=TRUE -DGROUPS_MAX=20"; 
	$RENAME_PARMS = "-DRENAME_MAX=9 -DCCR_PHYS_MAX=32 -DSPR_PHYS_MAX=60 -DGPR_PHYS_MAX=80 -DFPR_PHYS_MAX=72 -DINFLIGHT_MAX=150";
	$ISSUEQ_PARMS = "-DBR_RESV=12 -DLOG_RESV=10 -DMEM_RESV=18 -DFPU_RESV=10";
	$EXEC_PARMS   = "-DCOMBINE_FIX_MEM=TRUE -DBR_NUM=1 -DMEM_NUM=2 -DCLUSTER=TRUE -DCOMBINE_FIX_NUM=1 -DCOMBINE_MEM_NUM=1 -DCLUSTER_FPU=TRUE -DFPU_NUM=1 -DHANDLE_NOPS=TRUE -DUSING_LOG_UNIT=TRUE  -DUSE_NONPIPE_FPU_OPS=TRUE -DUSE_NONPIPE_FIX_OPS=TRUE ";
	$RETIRE_PARMS = "-DRETIRE_MAX=9 -DRETIREQ_MAX=100";
	$DCACHE_PARMS = "-DDCACHE_PORTS=2 -DDCACHE_LG2SIZE=7 -DDCACHE_LG2ENTRIES=8 -DDCACHE_ASSOC=2 -DDCACHE_WRITE_PORTS=1";
	$ICACHE_PARMS = "-DICACHE_LG2SIZE=7 -DICACHE_LG2ENTRIES=8 -DICACHE_ASSOC=1";
	
#	$L2CACHE_PARMS = "-DL2CACHE_LG2ENTRIES=14 -DL2CACHE_LG2SIZE=7 -DINFINITE_L2CACHE=TRUE";
	$L2CACHE_PARMS = "-DL2CACHE_LG2ENTRIES=13 -DL2CACHE_LG2SIZE=7";
	
#	$DTLB_PARMS = "-DDTLB_LG2ENTRIES=7 -DD2TLB_LG2ENTRIES=10 -DDTLB_ASSOC=2 -DD2TLB_ASSOC=4 -DUNIFIED_2TLB=TRUE -DINFINITE_DTLB=TRUE -DMTSR_PURGE_TLB=FALSE";
	$DTLB_PARMS = "-DDTLB_LG2ENTRIES=7 -DD2TLB_LG2ENTRIES=10 -DDTLB_ASSOC=2 -DD2TLB_ASSOC=4 -DUNIFIED_2TLB=TRUE -DMTSR_PURGE_TLB=FALSE";
	
#	$ITLB_PARMS = "-DITLB_LG2ENTRIES=7 -DI2TLB_LG2ENTRIES=10 -DITLB_ASSOC=2 -DI2TLB_ASSOC=4 -DINFINITE_ITLB=TRUE";
	$ITLB_PARMS = "-DITLB_LG2ENTRIES=7 -DI2TLB_LG2ENTRIES=10 -DITLB_ASSOC=2 -DI2TLB_ASSOC=4";

	$STOREQ_PARMS = "-DSTORE_RETIRE_SEPARATE=TRUE -DSTORE_RETIRE_NOLOAD=FALSE -DCOMMIT_STORES_DELAY=6 -DDMISS_MAX=8 -DUSE_STOREQ_NODATA_CIRCULATE=TRUE -DSTOREQ_SAME_CYCLE_ABORT=TRUE -DSTOREQ_IGNORE_PARTIAL=FALSE -DSTOREQ_IGNORE_RETIRED=FALSE -DSTOREQ_LENGTH=29 -DUSE_BITS_MEMQ=12 -DSTOREQ_FWD_AVAIL=0";
# 	// above: DSTOREQ_SIZE=32 removed

#	$TRACE_PARMS ="-DUSING_FF52_SEGS=1 -DUSING_FF=1 -DUSE_INSN_CUTOFF=TRUE -DMONITOR_CYCLES=0x1000 -DTRACE=TRUE -UDEBUG -DDEBUG=FALSE -DTIMELINE=FALSE";
	$TRACE_PARMS ="-DUSING_FF52_SEGS=1 -DUSING_FF=1 -DUSE_INSN_CUTOFF=TRUE -DMONITOR_CYCLES=0x1000 -DTRACE=TRUE -UDEBUG -DDEBUG=FALSE -DTIMELINE=FALSE";

#	$POWER_PARMS = "-DPOWER_MODE=TRUE -DPOWER_PERIODIC=FALSE -DPDS_SIM=TRUE -DPOWER_CYCLES=0x100 ";
#	$POWER_PARMS = "-DPOWER_MODE=TRUE -DPOWER_PERIODIC=FALSE -DPDS_SIM=TRUE -DPOWER_CYCLES=100000 ";
	$POWER_PARMS = "-DPOWER_MODE=TRUE -DPOWER_PERIODIC=FALSE -DPDS_SIM=FALSE -DPOWER_CYCLES=100000 ";
	
#	$TEMP_PARMS = "-DTEMPERATURE=FALSE -DRELIABILITY=FALSE";
	$TEMP_PARMS = "-DTEMPERATURE=TRUE -DRELIABILITY=FALSE -DFIXED_FREQUENCY=TRUE -DVARIATION=FALSE";
#	$TEMP_PARMS = $TEMP_PARMS . "-DFETCH_THROTTLE=TRUE";
#	$TEMP_PARMS = $TEMP_PARMS . "-DFETCH_THROTTLE=TRUE -DSELECTIVE_FETCH=TRUE";
#	$TEMP_PARMS = $TEMP_PARMS . "-DRENAME_THROTTLE=TRUE";
#	$TEMP_PARMS = $TEMP_PARMS . "-DRENAME_THROTTLE=TRUE -DSELECTIVE_RENAME=TRUE";
#	$TEMP_PARMS = $TEMP_PARMS . "-DFETCH_THROTTLE=TRUE -DCONTEXT_SWITCH=TRUE";
#	$TEMP_PARMS = $TEMP_PARMS . "-DDYNAMIC_PROFILING=TRUE";
#	$TEMP_PARMS = $TEMP_PARMS . "-DFETCH_THROTTLE=TRUE -DSELECTIVE_FETCH=TRUE -DDYNAMIC_PROFILING=TRUE";
#	$TEMP_PARMS = $TEMP_PARMS . "-DRENAME_THROTTLE=TRUE -DSELECTIVE_RENAME=TRUE -DDYNAMIC_PROFILING=TRUE";
#	$TEMP_PARMS = $TEMP_PARMS . "-DDYNAMIC_PROFILING=TRUE";
#	$TEMP_PARMS  = " ";
	$TEMP_PARMS = $TEMP_PARMS . " -DDYNAMIC_PROFILING=FALSE -DCOARSE_STATS=FALSE";

#	$SMT_PARMS = "-DCCR_PHYS_MAX=64 -DFPR_PHYS_MAX=96 -DGPR_PHYS_MAX=120 -DICACHE_ASSOC=2 -DSPR_PHYS_MAX=90 ";

#	$SMT_PARMS = $SMT_PARMS . "-USTOREQ_LENGTH -USTOREQ_HIWATER -USTOREQ_INORDER_CUTOFF "; 
#	$SMT_PARMS = $SMT_PARMS . "-UREORDERQ_LENGTH -UREORDERQ_HIWATER ";
#	$SMT_PARMS = $SMT_PARMS . "-UDMISS_MAX -UDCASTOUT_MAX ";
#	$SMT_PARMS = $SMT_PARMS . "-DSTOREQ_LENGTH=44 -DSTOREQ_HIWATER=42 -DSTOREQ_INORDER_CUTOFF=6 ";
#	$SMT_PARMS = $SMT_PARMS . "-DREORDERQ_LENGTH=47 -DREORDERQ_HIWATER=45 ";
#	$SMT_PARMS = $SMT_PARMS . "-DDMISS_MAX=12 -DDCASTOUT_MAX=12 ";

# /* Set icache assoc 1 for ST */

# RETRO_PARMS tries to #undef the physical registers here too, jesus
# update: that's a good thing
	$SMT_PARMS = "-UCCR_PHYS_MAX -UFPR_PHYS_MAX -UGPR_PHYS_MAX -UICACHE_ASSOC -USPR_PHYS_MAX -UGROUPS_MAX -UINFLIGHT_MAX -UIQ_SIZE -UGID_MAX -UPHYS_MAX ";
# -USPR_PHYS_MAX was semi-hidden in above line
	$SMT_PARMS = $SMT_PARMS . "-UICACHE_ASSOC -UGROUPS_MAX -UINFLIGHT_MAX -UIQ_SIZE -UGID_MAX -UPHYS_MAX ";
	
	$SMT_PARMS = $SMT_PARMS . "-UIBUF_MAX -URETIREQ_MAX -UFCHBLK_MAX ";
	$SMT_PARMS = $SMT_PARMS . "-UFIX_RESV -UFPU_RESV -UMEM_RESV -ULOG_RESV -UBR_RESV ";
# RETRO_PARMS: may seek to use FPR_PHYS=108, GPR_PHYS=120, (CCR_PHYS=48)
# but otherwise, keep it simple at 32, 72, 80 instead of the SMT-aggressive 48, 108, 120
# Oh, and SPR_PHYS_MAX seems to be kept at 60
# However, the code to change this actually is somewhere else, way below... -jdonald
# whoa, in fact there are three different places setting this, jesus
#	$SMT_PARMS = $SMT_PARMS . "-DCCR_PHYS_MAX=48 -DFPR_PHYS_MAX=108 -DGPR_PHYS_MAX=120 -DICACHE_ASSOC=2 ";
#	$SMT_PARMS = $SMT_PARMS . "-DSPR_PHYS_MAX=90 -DGROUPS_MAX=30 -DINFLIGHT_MAX=225 -DIQ_SIZE=256 -DGID_MAX=64 -DPHYS_MAX=256 ";
#   now increasing to max 4 contexts
	$SMT_PARMS = $SMT_PARMS . "-DCCR_PHYS_MAX=80 -DFPR_PHYS_MAX=180 -DGPR_PHYS_MAX=200 -DICACHE_ASSOC=2 ";
	$SMT_PARMS = $SMT_PARMS . "-DSPR_PHYS_MAX=160 -DGROUPS_MAX=30 -DINFLIGHT_MAX=225 -DIQ_SIZE=256 -DGID_MAX=64 -DPHYS_MAX=256 ";

# hmm, i had to change the SPR_PHYS_MAX from 60 to 90, maybe it should have been 60? /jdonald

# what the!!! registers get set and reset like 3 times -jdonald
# back to normal now. the following two are commented out since we can extend the registers
	# $SMT_PARMS = $SMT_PARMS . "-DICACHE_ASSOC=2 ";
	# $SMT_PARMS = $SMT_PARMS . "-DGROUPS_MAX=30 -DINFLIGHT_MAX=225 -DIQ_SIZE=256 -DGID_MAX=64 -DPHYS_MAX=256 ";
	$SMT_PARMS = $SMT_PARMS . "-DIBUF_MAX=96 -DRETIREQ_MAX=150 -DFCHBLK_MAX=12 ";
	$SMT_PARMS = $SMT_PARMS . "-DFIX_RESV=30 -DFPU_RESV=8 -DMEM_RESV=27 -DLOG_RESV=15 -DBR_RESV=18 ";

	$BASE_PREDECODE_PARMS =  "";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DIFETCH_MAX=8 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DIFETCH_ALIGN=8 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DICACHE_ALIGN=32 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DBRANCH_UNIT=3 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DFPU_UNIT=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DLOAD_UNIT=2 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DSTORE_UNIT=2 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DFIX_UNIT=2 "; 
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DIMUL_UNIT=2 "; 
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DIDIV_UNIT=2 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DUSE_GP=1 "; 
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DSPLIT_ALL=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DSPLIT_FXLDAU=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DSPLIT_FXLDA=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DSPLIT_FXLDU=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DSPLIT_FPLDU=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DSPLIT_FXSTUX=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DSPLIT_FXSTU=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DSPLIT_FPSTU=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DSPLIT_STU3=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DSPLIT_FXST=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DSPLIT_FPST=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DSPLIT_MULT=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DSPLIT_CRL=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DSPLIT_SPR=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DSPLIT_STRING=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DSPLIT_SHOW=0 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DPRED_DO=0 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DPRED_DIST=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DPRED_MODE=1 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DPRED_SHOW=0 ";
	$BASE_PREDECODE_PARMS = $BASE_PREDECODE_PARMS . "-DWIDE_ARCH_REG=0 ";

	$BASEARCH = "GP";

	if($BASEARCH eq "SUPER_GP") {
	    $BASE_PARMS = "$SUPER_GP_PARMS $SMT_PARMS $TRACE_PARMS $POWER_PARMS $TEMP_PARMS";
	    $BASE_PREDECODE_FLAGS = "$SUPER_GP_PREDECODE_FLAGS";	
	}
	elsif($BASEARCH eq "GP") {
	    $BASE_PARMS = "$GP_PARMS $SMT_PARMS $TRACE_PARMS $POWER_PARMS $TEMP_PARMS";
	    $BASE_PREDECODE_FLAGS = "$GP_PREDECODE_FLAGS";
	}
	elsif($BASEARCH eq "705") {
	    $BASE_PARMS = "$M750_PARMS $SMT_PARMS $TRACE_PARMS $POWER_PARMS $TEMP_PARMS";
	    $BASE_PREDECODE_FLAGS = "$M750_PREDECODE_FLAGS";
	}
	elsif($BASEARCH eq "405") {
	    $BASE_PARMS = "$M405_PARMS $SMT_PARMS $TRACE_PARMS $POWER_PARMS $TEMP_PARMS";
	    $BASE_PREDECODE_FLAGS = "$M405_PREDECODE_FLAGS";
	}

	$powerscale = 0;
	$powercoeff = 1.0;
    $POWERSCALE_PARMS = "-DFXU_REG_SCALE=$powerscale -DFPU_REG_SCALE=$powerscale -DFXU_XER_SCALE=$powerscale -DFXU_SPR_SCALE=$powerscale -DIFU_CR_SCALE=$powerscale -DIFU_SPR_SCALE=$powerscale      -DFXQ_SCALE=$powerscale      -DFPQ_SCALE=$powerscale     -DBRQ_SCALE=$powerscale      -DFX_MAPPER_SCALE=$powerscale -DFP_MAPPER_SCALE=$powerscale  -DCR_MAPPER_SCALE=$powerscale -DCTRLR_MAPPER_SCALE=$powerscale -DXER_MAPPER_SCALE=$powerscale -DLSU_SRQ_SCALE=$powerscale -DLSU_SDQ_SCALE=$powerscale -DLSU_LRQ_SCALE=$powerscale -DLSU_LMQ_SCALE=$powerscale  -DGTB_SCALE=$powerscale -DFXU_REG_SCALE_NUM=$powercoeff   -DFPU_REG_SCALE_NUM=$powercoeff      -DFXU_XER_SCALE_NUM=$powercoeff -DFXU_SPR_SCALE_NUM=$powercoeff -DIFU_CR_SCALE_NUM=$powercoeff -DIFU_SPR_SCALE_NUM=$powercoeff -DFXQ_SCALE_NUM=$powercoeff      -DFPQ_SCALE_NUM=$powercoeff     -DBRQ_SCALE_NUM=$powercoeff      -DFX_MAPPER_SCALE_NUM=$powercoeff -DFP_MAPPER_SCALE_NUM=$powercoeff  -DCR_MAPPER_SCALE_NUM=$powercoeff -DCTRLR_MAPPER_SCALE_NUM=$powercoeff -DXER_MAPPER_SCALE_NUM=$powercoeff -DLSU_SRQ_SCALE_NUM=$powercoeff -DLSU_SDQ_SCALE_NUM=$powercoeff -DLSU_LRQ_SCALE_NUM=$powercoeff -DLSU_LMQ_SCALE_NUM=$powercoeff -DGTB_SCALE_NUM=$powercoeff -DSCALEPOWER=1 -DLEAK1=1 -DPOWER_ADJUST_COEFF=1.0 ";
#	print $COMPILE_PARMS;

# /* IDEAL CASES */
	$scale = 1.5;
	while ($scale < 1.51) { # clearly this hack needs to be cleaned up -jdonald
		# RETRO_PARMS would have these register scaling parts turned back on
		# edit: this is no longer retro since these are just *_PHYS_MAX variables
		$GPR_PHYS_MAX = int(80*$scale+0.5);
		$CCR_PHYS_MAX = int(32*$scale+0.5);
		$FPR_PHYS_MAX = int(72*$scale+0.5);
		$SPR_PHYS_MAX = int(60*$scale+0.5);

		$GROUPS_MAX = int(20*$scale+0.5);
		$INFLIGHT_MAX = int(150*$scale+0.5);
		$IBUF_MAX = int(64*$scale+0.5);
		$RETIREQ_MAX = int(100*$scale+0.5);
		$FCHBLK_MAX = int(8*$scale+0.5);

		$FIX_RESV = int(20*$scale+0.5);
		$FPU_RESV = int(5*$scale+0.5);
		$LOG_RESV = int(10*$scale+0.5);
        $MEM_RESV = int(18*$scale+0.5);
        $BR_RESV = int(12*$scale+0.5);

# RETRO_PARMS 5th or 6th instance
        # $FXSCALE_PARMS = "-UCCR_PHYS_MAX -UFPR_PHYS_MAX -UGPR_PHYS_MAX -USPR_PHYS_MAX -UGROUPS_MAX -UINFLIGHT_MAX ";
# the above undefine turned back on... OR NOT /jdonald
        $FXSCALE_PARMS = "-UGROUPS_MAX -UINFLIGHT_MAX ";
        $FXSCALE_PARMS = $FXSCALE_PARMS . "-UIBUF_MAX -URETIREQ_MAX -UFCHBLK_MAX ";
        $FXSCALE_PARMS = $FXSCALE_PARMS . "-UFIX_RESV -UFPU_RESV -UMEM_RESV -ULOG_RESV -UBR_RESV ";
        # $FXSCALE_PARMS = $FXSCALE_PARMS . "-DCCR_PHYS_MAX=$CCR_PHYS_MAX -DFPR_PHYS_MAX=$FPR_PHYS_MAX -DGPR_PHYS_MAX=$GPR_PHYS_MAX -DSPR_PHYS_MAX=$SPR_PHYS_MAX ";
# turn this back on when RETRO_PARMS is off
        $FXSCALE_PARMS = $FXSCALE_PARMS . "-DGROUPS_MAX=$GROUPS_MAX -DINFLIGHT_MAX=$INFLIGHT_MAX ";
        $FXSCALE_PARMS = $FXSCALE_PARMS . "-DIBUF_MAX=$IBUF_MAX -DRETIREQ_MAX=$RETIREQ_MAX -DFCHBLK_MAX=$FCHBLK_MAX ";
        $FXSCALE_PARMS = $FXSCALE_PARMS . "-DFIX_RESV=$FIX_RESV -DFPU_RESV=$FPU_RESV -DMEM_RESV=$MEM_RESV -DLOG_RESV=$LOG_RESV -DBR_RESV=$BR_RESV ";
        $FXSCALE_PARMS = $FXSCALE_PARMS . "-DSCALEPOWER=1 ";

#		$MODELNAME = "turandot-$BASEARCH-BASE";
		$MODELNAME = "gpffturandot";
		$KNOB_PARMS = "$BASE_PARMS $BASE_PREDECODE_FLAGS $POWERSCALE_PARMS $FXSCALE_PARMS";
		$COMPILE_PARMS = "-I$ARIADIR/opcode -I$ARIADIR/opcode/include -I$TURDIR/ffreader -I$TURDIR/deps " . $ENV{'TLS_DEFS'};

		create_parms_header($KNOB_PARMS);
		# chdir($SRCDIR);
		system("rm -f $MODELDIR/$MODELNAME ");
		system("rm -f $MODELDIR/$MODELNAME.o "); # keeping this to avoid running it after a failed build
		
		$cmdstr = "$CC $OPT_FLAGS $COMPILE_PARMS -c turandot.c -o $MODELDIR/$MODELNAME.o ";
		printf("$cmdstr\n");
		$retval = system($cmdstr);
		if ($retval == 0) {
			$cmdstr = "$CC " . $ENV{'GLOBAL_LDFLAGS'} . " -L$ARIADIR/lib $MODELDIR/$MODELNAME.o $TURDIR/ffreader/trace_ff52reader.o $TURDIR/ffreader/trace_decompress.o -lopc -lm -lpthread " . $ENV{'COMPRESS_LIBS'} . " -o $MODELDIR/$MODELNAME";
			printf("$cmdstr\n");
			if (system($cmdstr) != 0) {
				exit(1);
				# we need to return 1 to mark an error for perl, can't just use any nonzero value
			}
			else {
				exit(0);
			}
			# system("rm -fr $MODELDIR/$MODELNAME.o ");
		}
		else {
			exit(1);
		}
		$scale = $scale + 0.1; # no good in our case
	}
