#ifndef __FLP_H_
#define __FLP_H_

#define STR_SIZE	4096	

/* functional unit placement definition	*/
typedef struct unit_t_st
{
	char name[STR_SIZE];
	double width;
	double height;
	double leftx;
	double bottomy;
}unit_t;

/* floorplan definition	*/
typedef struct flp_t_st
{
	unit_t *units;
	int n_units;
	int core_units;
	int attach_units;
} flp_t;

/* routines	*/

/* 
 * print the floorplan in a FIG like format 
 * that can be read by tofig.pl to produce 
 * an xfig output 
 */
void print_flp_fig (const flp_t *flp);

/* read floorplan file and allocate memory	*/
flp_t *read_flp(const char *file);
flp_t *read_cmpflp(const char *file, const char *cmpfile, const char *attachfile);

/* memory uninitialization	*/
void free_flp(flp_t *flp);

/* get unit index from its name	*/
int get_blk_index(const flp_t *flp, char *name);

/* are the units horizontally adjacent?	*/
int is_horiz_adj(const flp_t *flp, int i, int j);

/* are the units vertically adjacent?	*/
int is_vert_adj (const flp_t *flp, int i, int j);

/* shared length between units	*/
double get_shared_len(const flp_t *flp, int i, int j);

/* total chip width	*/
double get_total_width(const flp_t *flp);

/* total chip height */
double get_total_height(const flp_t *flp);

// jdonald putting the following two functions here since there
// wasn't quite any appropriate existing header file.
double calculate_core_power(int corenum, const double* power, const flp_t* flp);
double calculate_core_temperature(int corenum, const double* temp, const flp_t* flp);
#endif // __FLP_H_

