// iq.h

#ifndef H_iq
#define H_iq

#if(!defined(TIMELINE_SIZE))
  #define TIMELINE_SIZE 	1024
#endif

struct xiq_info {
  xdep_prep_info *	entry;
  unsigned		iaddr;
  unsigned		daddr;
  xarch *		out_regs;
  xcycle		cycle;
  unsigned		start;
//  unsigned		early;
 xcycle                  early; /* LYM 64bits adaptation */

  /* for global linked list across threads in SMT mode */
  unsigned		thread_idx;
  struct xiq_info	*gnext;	
  struct xiq_info	*gprev;	
  	
  unsigned		prop0;
  unsigned		prop1;
  unsigned 		prop2;
  unsigned 		propl;
  unsigned		iword;

  xiq_regidx	 	reg_idx;

  xiq_idx		next;	/* pointer to next in (fetch, decode, dmiss, execution, retire, etc) queue */
  unsigned char		trauma;
  unsigned char		slot;
  unsigned char		cluster;


  #if(USING_FF52_SEGS )
  unsigned		dseg;
  #endif

  #if( TRACE )
    xstatus		status;
  #endif
  #if( TRACE || DUMP_DMISS || TIMELINE )
    unsigned		count;
  #endif
  #if( TIMELINE )
    xcycle		tl_start;
    xcycle		tl_complete;
    char		tl_status[TIMELINE_SIZE];
  #endif
};

#define xiq_deref(i)		(i)
#define xiq_iaddr(i)		(xiq_deref(i).iaddr)
#define xiq_daddr(i)		(xiq_deref(i).daddr)
#define xiq_entry(i)		(xiq_deref(i).entry)
#define xiq_start(i)		(xiq_deref(i).start)
#define xiq_early(i)		(xiq_deref(i).early)
#define xiq_reg_idx(i)		(xiq_deref(i).reg_idx)
#define xiq_out_regs(i)		(xiq_deref(i).out_regs)
#define xiq_next(i)		(xiq_deref(i).next)
#define xiq_cycle(i)		(xiq_deref(i).cycle)

#define xiq_trauma(i)		(xiq_deref(i).trauma)
#define xiq_iword(i)		(xiq_deref(i).iword)

#define xiq_thread_id(i) 	(xiq_deref(i).thread_id)

#if( USING_FF52_SEGS )
#define xiq_dseg(i)		(xiq_deref(i).dseg)
#endif
#define xiq_slot(i)		(xiq_deref(i).slot)

#if( TRACE )
  #define xiq_status(i)		(xiq_deref(i).status)
#endif
#if( TRACE || DUMP_DMISS || TIMELINE )
  #define xiq_count(i)		(xiq_deref(i).count)
#endif

#if( TIMELINE )
  #define xiq_tl_start(i)	(xiq_deref(i).tl_start)
  #define xiq_tl_complete(i)	(xiq_deref(i).tl_complete)
  #define xiq_tl_status(i)	(xiq_deref(i).tl_status)
#endif

/****************************************************************/
/* Some properties are precomputed; the rest are computed at	*/
/* run time. The precomputed properties are copied over from	*/
/* precomputed cache into the iq structure.			*/
/* Some precomputed properties will only be accessed from the	*/
/* precomputed cache; some are accessed from both iq and cache;	*/
/****************************************************************/

#include "xprop.h"

#define xiq_prop0(i)		(xiq_deref(i).prop0)

/****************************************************************/
/* precompute-only: max number of instructions that can be	*/	
/* fetched from the I-Cache, assuming this is the first		*/
/* instruction accessed						*/
/****************************************************************/
#define xiq_fetch(i)		xprop_fetch(xiq_prop0(i))

/****************************************************************/
/* issue-retire cycles of an operation				*/
/* (compare with avail below)					*/
/****************************************************************/
#define xiq_latency(i)		xprop_latency(xiq_prop0(i))

/****************************************************************/
/* load/store pipe operation? if so				*/
/*	- is it a store?					*/
/*	- is it a load?						*/
/*	- how many bytes are affected by the memory operation?	*/
/*	(see below for notaddr: non-load/store op that affects	*/
/*	memory opeartion)					*/
/****************************************************************/
#define xiq_is_memop(i)		xprop_is_memop(xiq_prop0(i))
#define xiq_is_store(i)		xprop_is_store(xiq_prop0(i))
#define xiq_is_load(i)		xprop_is_load(xiq_prop0(i))
#define xiq_dlength(i)		xprop_dlength(xiq_prop0(i))

/****************************************************************/
/* branch? if so,						*/
/*	- does it have the LK bit set?				*/
/*	- is it a branch thru link register?			*/
/*	- is it a branch thru counter register?			*/
/*	- is it a branch conditional?				*/
/*	- is it a conditional thru register branch?		*/
/****************************************************************/
#define xiq_is_branch(i)	xprop_is_branch(xiq_prop0(i))
#define xiq_is_brlk(i)		xprop_is_brlk(xiq_prop0(i))
#define xiq_is_brlink(i)	xprop_is_brlink(xiq_prop0(i))
#define xiq_is_brctr(i)		xprop_is_brctr(xiq_prop0(i))
#define xiq_is_brcond(i)	xprop_is_brcond(xiq_prop0(i))
#define xiq_is_brcrg(i)		xprop_is_brcrg(xiq_prop0(i))

/****************************************************************/
/* predication support: should this branch be treated like a	*/
/*	skip? If so, how many subsequent instructions will be	*/
/*	controlled?						*/
/****************************************************************/
#define xiq_is_pred(i)		(xprop_pred(xiq_prop0(i)) != 0)
#define xiq_pred_dist(i)	(xprop_pred(xiq_prop0(i)))

/****************************************************************/
/* what kind of unit (e.g. fpu) is this instruction going to be	*/
/* executed in							*/
/****************************************************************/
#define xiq_unit(i)		xprop_unit(xiq_prop0(i))

/****************************************************************/
/* is the operation expanded: i.e. cracked or millicoded? if so	*/
/* how many operations does it expand into?			*/
/* (see also multi below)					*/
/****************************************************************/
#define xiq_nexpands(i)		xprop_nexpands(xiq_prop0(i))
#define xiq_is_expand(i)	xprop_is_expand(xiq_prop0(i))

/****************************************************************/
/* obsolete?? meant to record that the op was the result of an	*/
/* exapnded op 							*/
/****************************************************************/
#define xiq_is_split(i)		xprop_is_split(xiq_prop0(i))

/****************************************************************/
/* a store can be cracked into at least one data move and one	*/
/* address compute. The transfer flag indicates that the op is	*/
/* the data move component of a cracked store			*/
/****************************************************************/
#define xiq_is_transfer(i)	xprop_is_transfer(xiq_prop0(i))

#define xiq_prop1(i)		(xiq_deref(i).prop1)
#define xiq_prop2(i)		(xiq_deref(i).prop2)

/****************************************************************/
/* ins and outs is the number of input and output dependences 	*/
/* on the operation.						*/
/* is_over is a a special bit that controls how the actual	*/
/* dependences are stored.					*/
/****************************************************************/
#define xiq_ins(i)		xprop_ins(xiq_prop1(i))
#define xiq_outs(i)		xprop_outs(xiq_prop1(i))
#define xiq_is_over(i)		xprop_is_over(xiq_prop1(i))
#define reset_xiq_ins(i,v)	reset_xprop_ins(xiq_prop1(i),(v))
#define reset_xiq_outs(i,v)	reset_xprop_outs(xiq_prop1(i),(v))

/****************************************************************/
/* lswx/stswx/lscbx						*/
/****************************************************************/
#define xiq_is_string(i)	xprop_is_string(xiq_prop1(i))

/****************************************************************/
/* sync instruction						*/
/****************************************************************/
#define xiq_is_sync(i)		xprop_is_sync(xiq_prop1(i))

/****************************************************************/
/* issue only when at head of retirement queue			*/
/****************************************************************/
#define xiq_is_athead(i)	xprop_is_athead(xiq_prop1(i))

/****************************************************************/
/* mtsr	instruction?						*/
/*	- the sr accessed					*/
/* bug??							*/
/****************************************************************/
#define xiq_is_mtsr(i)		xprop_is_mtsr(xiq_prop1(i))
#define xiq_sr(i)		xprop_sr(xiq_prop1(i))

/****************************************************************/
/* non-load/store op that affects memory operation		*/
/****************************************************************/
#define xiq_is_notaddr(i)	xprop_is_notaddr(xiq_prop1(i))

/****************************************************************/
/* operation must be in slot 0 to be decoded			*/
/****************************************************************/
#define xiq_is_slot0(i)		xprop_is_slot0(xiq_prop1(i))

#define xiq_is_slotN(i)		xprop_is_slotN(xiq_prop1(i))

/****************************************************************/
/* the issue-use cycles of an operation				*/
/* (compare with latency above)					*/
/****************************************************************/
#define xiq_avail(i)		xprop_avail(xiq_prop1(i))

/****************************************************************/
/* millicoded as opposed to cracked				*/
/****************************************************************/
#define xiq_is_multi(i)		xprop_is_multi(xiq_prop1(i))

/****************************************************************/
/* nop; may handle specially 					*/
/****************************************************************/
#define xiq_is_nop(i)		xprop_is_nop(xiq_prop2(i))
#define xiq_is_notpipe(i)	xprop_is_notpipe(xiq_prop2(i))
#define xiq_is_nocount(i)	xprop_is_nocount(xiq_prop2(i))
#define xiq_is_reissue(i)	xprop_is_reissue(xiq_prop2(i))

#define	xiq_propl(i)		(xiq_deref(i).propl)

#define XIQ_IS_BUSY_START	0
#define XIQ_IS_BUSY_MASK	0x1
#define xiq_is_busy(i)		((xiq_propl(i)>>XIQ_IS_BUSY_START)&XIQ_IS_BUSY_MASK)
#define set_xiq_is_busy(i,v)	(xiq_propl(i) |= ((v)&XIQ_IS_BUSY_MASK)<<XIQ_IS_BUSY_START)
#define set0_xiq_is_busy(i)	(xiq_propl(i) &= ~(XIQ_IS_BUSY_MASK<<XIQ_IS_BUSY_START))
#define set1_xiq_is_busy(i)	(xiq_propl(i) |= (XIQ_IS_BUSY_MASK<<XIQ_IS_BUSY_START))
#define reset_xiq_is_busy(i,v)	\
	(xiq_propl(i) = (xiq_propl(i)&~(XIQ_IS_BUSY_MASK<<XIQ_IS_BUSY_START)) \
			|(((v)&XIQ_IS_BUSY_MASK)<<XIQ_IS_BUSY_START))

#define XIQ_IS_MISPREDICTED_START	1
#define XIQ_IS_MISPREDICTED_MASK	0x1
#define xiq_is_mispredicted(i)		((xiq_propl(i)>>XIQ_IS_MISPREDICTED_START)&XIQ_IS_MISPREDICTED_MASK)
#define set_xiq_is_mispredicted(i,v)	(xiq_propl(i) |= ((v)&XIQ_IS_MISPREDICTED_MASK)<<XIQ_IS_MISPREDICTED_START)
#define reset_xiq_is_mispredicted(i,v)	\
	(xiq_propl(i) = (xiq_propl(i)&~(XIQ_IS_MISPREDICTED_MASK<<XIQ_IS_MISPREDICTED_START)) \
			|(((v)&XIQ_IS_MISPREDICTED_MASK)<<XIQ_IS_MISPREDICTED_START))

#define XIQ_REORDER_START	2
#define XIQ_REORDER_MASK	0x3
#define xiq_reorder(i)		((xiq_propl(i)>>XIQ_REORDER_START)&XIQ_REORDER_MASK)
#define set_xiq_reorder(i,v)	(xiq_propl(i) |= ((v)&XIQ_REORDER_MASK)<<XIQ_REORDER_START)
#define reset_xiq_reorder(i,v)	\
	(xiq_propl(i) = (xiq_propl(i)&~(XIQ_REORDER_MASK<<XIQ_REORDER_START)) \
			|(((v)&XIQ_REORDER_MASK)<<XIQ_REORDER_START))
	
#define XIQ_FALL_START		4
#define XIQ_FALL_MASK		0x1
#define xiq_fall(i)		((xiq_propl(i)>>XIQ_FALL_START)&XIQ_FALL_MASK)
#define set_xiq_fall(i,v)	(xiq_propl(i) |= ((v)&XIQ_FALL_MASK)<<XIQ_FALL_START)
#define reset_xiq_fall(i,v)	\
	(xiq_propl(i) = (xiq_propl(i)&~(XIQ_FALL_MASK<<XIQ_FALL_START)) \
			|(((v)&XIQ_FALL_MASK)<<XIQ_FALL_START))

#define XIQ_IS_FIRST_START	5
#define XIQ_IS_FIRST_MASK	0x1
#define xiq_is_first(i)		((xiq_propl(i)>>XIQ_IS_FIRST_START)&XIQ_IS_FIRST_MASK)
#define set_xiq_is_first(i,v)	(xiq_propl(i) |= ((v)&XIQ_IS_FIRST_MASK)<<XIQ_IS_FIRST_START)
#define reset_xiq_is_first(i,v)	\
	(xiq_propl(i) = (xiq_propl(i)&~(XIQ_IS_FIRST_MASK<<XIQ_IS_FIRST_START)) \
			|(((v)&XIQ_IS_FIRST_MASK)<<XIQ_IS_FIRST_START))

#define XIQ_IS_GROUP_START	6
#define XIQ_IS_GROUP_MASK	0x1
#define xiq_is_group(i)		((xiq_propl(i)>>XIQ_IS_GROUP_START)&XIQ_IS_GROUP_MASK)
#define set_xiq_is_group(i,v)	(xiq_propl(i) |= ((v)&XIQ_IS_GROUP_MASK)<<XIQ_IS_GROUP_START)
#define reset_xiq_is_group(i,v)	\
	(xiq_propl(i) = (xiq_propl(i)&~(XIQ_IS_GROUP_MASK<<XIQ_IS_GROUP_START)) \
			|(((v)&XIQ_IS_GROUP_MASK)<<XIQ_IS_GROUP_START))

#define XIQ_IS_PREDOP_START	7
#define XIQ_IS_PREDOP_MASK	0x1
#define xiq_is_predop(i)		((xiq_propl(i)>>XIQ_IS_PREDOP_START)&XIQ_IS_PREDOP_MASK)
#define set_xiq_is_predop(i,v)	(xiq_propl(i) |= ((v)&XIQ_IS_PREDOP_MASK)<<XIQ_IS_PREDOP_START)
#define reset_xiq_is_predop(i,v)	\
	(xiq_propl(i) = (xiq_propl(i)&~(XIQ_IS_PREDOP_MASK<<XIQ_IS_PREDOP_START)) \
			|(((v)&XIQ_IS_PREDOP_MASK)<<XIQ_IS_PREDOP_START))

#define XIQ_IS_CRACK_START	8
#define XIQ_IS_CRACK_MASK	0x1
#define xiq_is_crack(i)		((xiq_propl(i)>>XIQ_IS_CRACK_START)&XIQ_IS_CRACK_MASK)
#define set_xiq_is_crack(i,v)	(xiq_propl(i) |= ((v)&XIQ_IS_CRACK_MASK)<<XIQ_IS_CRACK_START)
#define reset_xiq_is_crack(i,v)	\
	(xiq_propl(i) = (xiq_propl(i)&~(XIQ_IS_CRACK_MASK<<XIQ_IS_CRACK_START)) \
			|(((v)&XIQ_IS_CRACK_MASK)<<XIQ_IS_CRACK_START))

#define XIQ_GID_START		9
#define XIQ_GID_MASK		0x1f
#define xiq_gid(i)		((xiq_propl(i)>>XIQ_GID_START)&XIQ_GID_MASK)
#define set_xiq_gid(i,v)	(xiq_propl(i) |= ((v)&XIQ_GID_MASK)<<XIQ_GID_START)
#define reset_xiq_gid(i,v)	\
	(xiq_propl(i) = (xiq_propl(i)&~(XIQ_GID_MASK<<XIQ_GID_START)) \
			|(((v)&XIQ_GID_MASK)<<XIQ_GID_START))

#define XIQ_FCHBLK_START	14
#define XIQ_FCHBLK_MASK		0x1
#define xiq_fchblk(i)		((xiq_propl(i)>>XIQ_FCHBLK_START)&XIQ_FCHBLK_MASK)
#define set_xiq_fchblk(i,v)	(xiq_propl(i) |= ((v)&XIQ_FCHBLK_MASK)<<XIQ_FCHBLK_START)
#define reset_xiq_fchblk(i,v)	\
	(xiq_propl(i) = (xiq_propl(i)&~(XIQ_FCHBLK_MASK<<XIQ_FCHBLK_START)) \
			|(((v)&XIQ_FCHBLK_MASK)<<XIQ_FCHBLK_START))

#define XIQ_L2MISS_START	15
#define XIQ_L2MISS_MASK		0x1
#define xiq_l2miss(i)		((xiq_propl(i)>>XIQ_L2MISS_START)&XIQ_L2MISS_MASK)
#define set_xiq_l2miss(i,v)	(xiq_propl(i) |= ((v)&XIQ_L2MISS_MASK)<<XIQ_L2MISS_START)
#define reset_xiq_l2miss(i,v)	\
	(xiq_propl(i) = (xiq_propl(i)&~(XIQ_L2MISS_MASK<<XIQ_L2MISS_START)) \
			|(((v)&XIQ_L2MISS_MASK)<<XIQ_L2MISS_START))

#define XIQ_LINE_START		16
#define XIQ_LINE_MASK		0x1
#define xiq_line(i)		((xiq_propl(i)>>XIQ_LINE_START)&XIQ_LINE_MASK)
#define set_xiq_line(i,v)	(xiq_propl(i) |= ((v)&XIQ_LINE_MASK)<<XIQ_LINE_START)
#define reset_xiq_line(i,v)	\
	(xiq_propl(i) = (xiq_propl(i)&~(XIQ_LINE_MASK<<XIQ_LINE_START)) \
			|(((v)&XIQ_LINE_MASK)<<XIQ_LINE_START))

#endif
