// jdonald: since this is used only once per POWER_CYCLES interval,
//          their performance-criticality is a bit different from all
//          the other Turandot internals, so I'll leave these functions
//          as non-inline.
//
// other major hotspot routines can be found in sim.init.temp.macros and
// init.temp.macros

/* LYM the new create_RC_matrices */
void create_RC_matrices(flp_t *flp, int omit_lateral)
{
        int i, j, k = 0, n = flp->n_units;
        int **border;
        double **len, *gx, *gy, **g, *c_ver, **t, *gx_sp, *gy_sp;
        double r_sp1, r_sp2, r_hs;      /* lateral resistances to spreader and heatsink */

        /* NOTE: *_mid - the vertical R/C from CENTER nodes of spreader
         * and heatsink. *_per - the vertical R/C from PERIPHERAL (n,s,e,w) nodes
         */
        double r_sp_per, r_hs_mid, r_hs_per, c_sp_per, c_hs_mid, c_hs_per;
        double gn_sp=0, gs_sp=0, ge_sp=0, gw_sp=0;

        double w_chip = get_total_width (flp);  /* x-axis       */
        double l_chip = get_total_height (flp); /* y-axis       */

        border = imatrix(n, 4);
        len = matrix(n, n);             /* len[i][j] = length of shared edge bet. i & j */
        gx = vector(n);                 /* lumped conductances in x direction   */
        gy = vector(n);                 /* lumped conductances in y direction   */
        gx_sp = vector(n);              /* lateral conductances in the spreader layer */
        gy_sp = vector(n);
        g = matrix(NL*n+EXTRA, NL*n+EXTRA);     /* g[i][j] = conductance bet. nodes i & j */
        c_ver = vector(NL*n+EXTRA);     /* vertical capacitance */

        b = matrix(NL*n+EXTRA, NL*n+EXTRA);     /* B, C, INVA  and INVB are (NL*n+EXTRA)x(NL*n+EXTRA) matrices  */
        c = matrix(NL*n+EXTRA, NL*n+EXTRA);
        inva = matrix(NL*n+EXTRA, NL*n+EXTRA);
        invb = matrix(NL*n+EXTRA, NL*n+EXTRA);
        t = matrix (NL*n+EXTRA, NL*n+EXTRA);    /* copy of B    */

        /* compute the silicon fitting factor - see pg 10 of the UVA CS tech report - CS-TR-2003-08     */
        factor_chip = C_FACTOR * ((SPEC_HEAT_INT / SPEC_HEAT_SI) * (w_chip + 0.88 * t_interface) \
                                * (l_chip + 0.88 * t_interface) * t_interface / ( w_chip * l_chip * t_chip) + 1);

        /* fitting factor for interface  - same rationale as above */
        factor_int = C_FACTOR * ((SPEC_HEAT_CU / SPEC_HEAT_INT) * (w_chip + 0.88 * t_spreader) \
                                * (l_chip + 0.88 * t_spreader) * t_spreader / ( w_chip * l_chip * t_interface) + 1);

        /*printf("fitting factors : %lf, %lf\n", factor_chip, factor_int);      */

        /* gx's and gy's of blocks      */
        for (i = 0; i < n; i++) {
                /* at the silicon layer */
                if (omit_lateral) {
                        gx[i] = gy[i] = 0;
                }
                else {
                        gx[i] = 1.0/getr(K_SI, flp->units[i].height, flp->units[i].width, l_chip, t_chip);
                        gy[i] = 1.0/getr(K_SI, flp->units[i].width, flp->units[i].height, w_chip, t_chip);
                }

                /* at the spreader layer        */
                gx_sp[i] = 1.0/getr(K_CU, flp->units[i].height, flp->units[i].width, l_chip, t_spreader);
                gy_sp[i] = 1.0/getr(K_CU, flp->units[i].width, flp->units[i].height, w_chip, t_spreader);
        }

        /* shared lengths between blocks        */
        for (i = 0; i < n; i++)
                for (j = i; j < n; j++)
                        len[i][j] = len[j][i] = get_shared_len(flp, i, j);

        /* lateral R's of spreader and sink */
        r_sp1 = getr(K_CU, (s_spreader+3*w_chip)/4.0, (s_spreader-w_chip)/4.0, w_chip, t_spreader);
        r_sp2 = getr(K_CU, (3*s_spreader+w_chip)/4.0, (s_spreader-w_chip)/4.0, (s_spreader+3*w_chip)/4.0, t_spreader);
        r_hs = getr(K_CU, (s_sink+3*s_spreader)/4.0, (s_sink-s_spreader)/4.0, s_spreader, t_sink);

        /* vertical R's and C's of spreader and sink */
        r_sp_per = RHO_CU * t_spreader * 4.0 / (s_spreader * s_spreader - w_chip*l_chip);
        c_sp_per = factor_pack * SPEC_HEAT_CU * t_spreader * (s_spreader * s_spreader - w_chip*l_chip) / 4.0;
        r_hs_mid = RHO_CU * t_sink / (s_spreader*s_spreader);
        c_hs_mid = factor_pack * SPEC_HEAT_CU * t_sink * (s_spreader * s_spreader);
        r_hs_per = RHO_CU * t_sink * 4.0 / (s_sink * s_sink - s_spreader*s_spreader);
        c_hs_per = factor_pack * SPEC_HEAT_CU * t_sink * (s_sink * s_sink - s_spreader*s_spreader) / 4.0;

        /* short the R's from block centers to a particular chip edge   */
        for (i = 0; i < n; i++) {
                if (eq(flp->units[i].bottomy + flp->units[i].height, l_chip)) {
                        gn_sp += gy_sp[i];
                        border[i][2] = 1;       /* block is on northern border  */
                }
                if (eq(flp->units[i].bottomy, 0)) {
                        gs_sp += gy_sp[i];
                        border[i][3] = 1;       /* block is on southern border  */
                }
                if (eq(flp->units[i].leftx + flp->units[i].width, w_chip)) {
                        ge_sp += gx_sp[i];
                        border[i][1] = 1;       /* block is on eastern border   */
                }
                if (eq(flp->units[i].leftx, 0)) {
                        gw_sp += gx_sp[i];
                        border[i][0] = 1;       /* block is on western border   */
                }
        }
        /* overall R and C between nodes */
        for (i = 0; i < n; i++) {
                double area = (flp->units[i].height * flp->units[i].width);
                /*
                 * amongst functional units     in the various layers
                 * resistances in the interface layer are assumed
                 * to be infinite
                 */
                for (j = 0; j < n; j++) {
                        double part = 0, part_sp = 0;
                        if (is_horiz_adj(flp, i, j)) {
                                part = gx[i] / flp->units[i].height;
                                part_sp = gx_sp[i] / flp->units[i].height;
                        }
                        else if (is_vert_adj(flp, i,j))  {
                                part = gy[i] / flp->units[i].width;
                                part_sp = gy_sp[i] / flp->units[i].width;
                        }
                        g[i][j] = part * len[i][j];
                        g[HSP*n+i][HSP*n+j] = part_sp * len[i][j];
                }

                /* vertical g's in the silicon layer    */
                g[i][IFACE*n+i]=g[IFACE*n+i][i]=2.0/(RHO_SI * t_chip / area);
                /* vertical g's in the interface layer  */
                g[IFACE*n+i][HSP*n+i]=g[HSP*n+i][IFACE*n+i]=2.0/(RHO_INT * t_interface / area);
                /* vertical g's in the spreader layer   */
                g[HSP*n+i][NL*n+SP_B]=g[NL*n+SP_B][HSP*n+i]=2.0/(RHO_CU * t_spreader / area);

                /* C's from functional units to ground  */
                c_ver[i] = factor_chip * SPEC_HEAT_SI * t_chip * area;
                /* C's from interface portion of the functional units to ground */
                c_ver[IFACE*n+i] = factor_int * SPEC_HEAT_INT * t_interface * area;
                /* C's from spreader portion of the functional units to ground  */
                c_ver[HSP*n+i] = factor_pack * SPEC_HEAT_CU * t_spreader * area;

                /* lateral g's from block center (spreader layer) to peripheral (n,s,e,w) spreader nodes        */
                g[HSP*n+i][NL*n+SP_N]=g[NL*n+SP_N][HSP*n+i]=2.0*border[i][2]/((1.0/gy_sp[i])+r_sp1*gn_sp/gy_sp[i]);
                g[HSP*n+i][NL*n+SP_S]=g[NL*n+SP_S][HSP*n+i]=2.0*border[i][3]/((1.0/gy_sp[i])+r_sp1*gs_sp/gy_sp[i]);
                g[HSP*n+i][NL*n+SP_E]=g[NL*n+SP_E][HSP*n+i]=2.0*border[i][1]/((1.0/gx_sp[i])+r_sp1*ge_sp/gx_sp[i]);
                g[HSP*n+i][NL*n+SP_W]=g[NL*n+SP_W][HSP*n+i]=2.0*border[i][0]/((1.0/gx_sp[i])+r_sp1*gw_sp/gx_sp[i]);
        }

        /* max slope (max_power * max_vertical_R / vertical RC time constant) for silicon       */
        max_slope = MAX_PD / (factor_chip * t_chip * SPEC_HEAT_SI);

        /* vertical g's and C's between central nodes   */
        /* between spreader bottom and sink bottom      */
        g[NL*n+SINK_B][NL*n+SP_B]=g[NL*n+SP_B][NL*n+SINK_B]=2.0/r_hs_mid;
        /* from spreader bottom to ground       */
        c_ver[NL*n+SP_B]=c_hs_mid;
        /* from sink bottom to ground   */
        c_ver[NL*n+SINK_B] = factor_pack * c_convec;

        /* g's and C's from peripheral(n,s,e,w) nodes   */
        for (i = 1; i <= 4; i++) {
                /* vertical g's between peripheral spreader nodes and spreader bottom */
                g[NL*n+SP_B-i][NL*n+SP_B]=g[NL*n+SP_B][NL*n+SP_B-i]=2.0/r_sp_per;
                /* lateral g's between peripheral spreader nodes and peripheral sink nodes      */
                g[NL*n+SP_B-i][NL*n+SINK_B-i]=g[NL*n+SINK_B-i][NL*n+SP_B-i]=2.0/(r_hs + r_sp2);
                /* vertical g's between peripheral sink nodes and sink bottom   */
                g[NL*n+SINK_B-i][NL*n+SINK_B]=g[NL*n+SINK_B][NL*n+SINK_B-i]=2.0/r_hs_per;
                /* from peripheral spreader nodes to ground     */
                c_ver[NL*n+SP_B-i]=c_sp_per;
                /* from peripheral sink nodes to ground */
                c_ver[NL*n+SINK_B-i]=c_hs_per;
        }

        /* calculate matrices A, B such that A(dT) + BT = POWER */

        for (i = 0; i < NL*n+EXTRA; i++) {
                for (j = 0; j < NL*n+EXTRA; j++) {
                        if (i==j) {
                                inva[i][j] = 1.0/c_ver[i];
                                if (i == NL*n+SINK_B)   /* sink bottom */
                                        b[i][j] += 1.0 / r_convec;
                                for (k = 0; k < NL*n+EXTRA; k++) {
                                        if ((g[i][k]==0.0)||(g[k][i])==0.0)
                                                continue;
                                        else
                                        /* here is why the 2.0 factor comes when calculating g[][]      */
                                                b[i][j] += 1.0/((1.0/g[i][k])+(1.0/g[k][i]));
                                }
                        } else {
                                inva[i][j]=0.0;
                                if ((g[i][j]==0.0)||(g[j][i])==0.0)
                                        b[i][j]=0.0;
                                else
                                        b[i][j]=-1.0/((1.0/g[i][j])+(1.0/g[j][i]));
                        }
                }
        }

        /* we are always going to use the eqn dT + A^-1 * B T = A^-1 * POWER. so, store  C = A^-1 * B   */
        matmult(c, inva, b, NL*n+EXTRA);
        /* we will also be needing INVB so store it too */
        copy_matrix(t, b, NL*n+EXTRA, NL*n+EXTRA);

        matinv(invb, t, NL*n+EXTRA);
/*      dump_vector(c_ver, NL*n+EXTRA); */
/*      dump_matrix(g, NL*n+EXTRA, NL*n+EXTRA); */
/*      dump_matrix(c, NL*n+EXTRA, NL*n+EXTRA); */

        /* cleanup */
        free_matrix(t, NL*n+EXTRA);
        free_matrix(g, NL*n+EXTRA);
        free_matrix(len, n);
        free_imatrix(border, n);
        free_vector(c_ver);
        free_vector(gx);
        free_vector(gy);
        free_vector(gx_sp);
        free_vector(gy_sp);
}


/* setting internal node power numbers  */
/* LYM */
void set_internal_power1 (double *pow, int n_units)
{
        int i;
        for (i=n_units; i < NL*n_units+SINK_B; i++)
                pow[i] = 0;
        pow[NL*n_units+SINK_B] = ambient / r_convec;
}

/* power and temp should both be alloced using hotspot_vector.
 * 'b' is the 'thermal conductance' matrix. i.e, b * temp = power
 *  => temp = invb * power
 */

/* LYM */
void steady_state_temp(double *power, double *temp, int n_units)
{
        /* set power numbers for the virtual nodes */
        set_internal_power1(power, n_units);

        /* find temperatures    */
        matvectmult1(temp, invb, power, NL*n_units+EXTRA);
}

/* differs from 'vector()' in that memory for internal nodes is also allocated  */
double *hotspot_vector(int n_units)
{
        return vector(NL*n_units+EXTRA);
}

/* sets the temperature of a vector 'temp' allocated using 'hotspot_vector'     */
/* LYM */
void set_temp(double *temp, int n_units, double val)
{
        int i;
        for(i=0; i < NL*n_units + EXTRA; i++)
                temp[i] = val;
}

/* dump temperature vector alloced using 'hotspot_vector' to 'file' */
/* LYM */


void dump_temp(flp_t *flp, double *temp, char *file)
// something strange about this function. it only dumps temperatures for
// the first core, but along with cache temperatures?
{
        int i;
        char str[STR_SIZE];
        FILE *fp = fopen (file, "w");
        if (!fp) {
                sprintf (str,"error: %s could not be opened for writing in dump_temp\n", file);
                fatal(str);
        }
        /* on chip temperatures */
        fprintf(stderr, "dump_temp: flp->core_units = %d\n", flp->core_units);
        for (i=0; i < flp->core_units; i++) {
		        if (strstr(flp->units[i].name, "_p1") != NULL) // jdonald 2-18-2005
		        	flp->units[i].name[strlen(flp->units[i].name)-3] = '\0'; /* jdonald hack 2-16-2005 */
                fprintf(fp, "%s\t%.1f\n", flp->units[i].name, temp[i]);
            }

        for (i=0; i < flp->attach_units; i++)
                fprintf(fp, "%s\t%.1f\n", flp->units[i+flp->n_units-flp->attach_units].name, temp[i+flp->core_units]);
		
        /* interface temperatures       */
        for (i=0; i < flp->core_units; i++)
                fprintf(fp, "iface_%s\t%.1f\n", flp->units[i].name, temp[IFACE*flp->n_units+i]);

        for (i=0; i < flp->attach_units; i++)
                fprintf(fp, "iface_%s\t%.1f\n", flp->units[i+flp->n_units-flp->attach_units].name, temp[i+flp->core_units+IFACE*flp->n_units]);

        /* spreader temperatures        */
        for (i=0; i < flp->core_units; i++)
                fprintf(fp, "hsp_%s\t%.1f\n", flp->units[i].name, temp[HSP*flp->n_units+i]);

        for (i=0; i < flp->attach_units; i++)
                fprintf(fp, "hsp_%s\t%.1f\n", flp->units[i+flp->n_units-flp->attach_units].name, temp[i+flp->core_units+HSP*flp->n_units]);

        /* internal node temperatures   */
        for (i=0; i < EXTRA; i++) {
                sprintf(str, "inode_%d", i);
                fprintf(fp, "%s\t%.1f\n", str, temp[i+NL*flp->n_units]);
        }
        fclose(fp);
}

/* LYM */
void read_temp(flp_t *flp, char *cmpfile, char *attachfile, double *temp, char *file, int clip)
{
        int i, idx;
        double max=0, val;
        char str[STR_SIZE], name[STR_SIZE];
        FILE *fp = fopen (file, "r");
		FILE *cmpfp = fopen (cmpfile, "r"); // LYM */
		FILE *attachfp = fopen (attachfile, "r"); // LYM
        int cmpcount = 0; /* LYM */
        int attachcount = 0; /* LYM */
		int j; /* LYM */
        char *ptr; /* LYM */

		// fprintf(stderr, "read_temp() got \"%s\" for file\n", file);
        while(!feof(cmpfp))
        {
                fgets(str, STR_SIZE, cmpfp);
                if (feof(cmpfp))
                        break;
                ptr = strtok(str, " \t\n");
                if (ptr && ptr[0] != '#')
                        cmpcount++;
        } /* LYM */

        while(!feof(attachfp))
        {
                fgets(str, STR_SIZE, attachfp);
                if (feof(attachfp))
                        break;
                ptr = strtok(str, " \t\n");
                if (ptr && ptr[0] != '#')
                        attachcount++;
        } /* LYM */

        if (!fp) {
                sprintf (str,"error: %s could not be opened for reading in read_temp\n", file);
                fatal(str);
        }

	for (j=0;j<cmpcount;j++){
	
	    fseek(fp, 0, SEEK_SET); /* LYM */
        /* find max temp on the chip    */
        for (i=0; i < flp->n_units; i++) {
                fgets(str, STR_SIZE, fp);
                if (feof(fp))
                        fatal("not enough lines in temperature file1\n");
                if (sscanf(str, "%s%lf", name, &val) != 2)
                        fatal("invalid temperature file format1\n");
                idx = get_blk_index(flp, name);
		idx = idx + j * (flp->n_units); /* LYM */
                if (idx >= 0)
                        temp[idx] = val;
                else    /* since get_blk_index calls fatal, the line below cannot be reached    */
                        fatal ("unit in temperature file not found in floorplan\n");
                if (temp[idx] > max)
                        max = temp[idx];
        }

        for (i=0; i < attachcount; i++) {
                fgets(str, STR_SIZE, fp);
                if (feof(fp))
                        fatal("not enough lines in temperature file1\n");
                if (sscanf(str, "%s%lf", name, &val) != 2)
                        fatal("invalid temperature file format1\n");
/*                idx = get_blk_index(flp, name); */
/*                idx = idx + j * (flp->n_units); */ /* LYM */
/*                if (idx >= 0)  */
		  		idx = cmpcount*flp->n_units+i;
                        temp[idx] = val;
/*                else    */ /* since get_blk_index calls fatal, the line below cannot be reached    */
/*                     fatal ("unit in temperature file not found in floorplan\n"); */
                if (temp[idx] > max)
                        max = temp[idx];
        }

        /* interface material temperatures      */
        for (i=0; i < flp->n_units; i++) {
                fgets(str, STR_SIZE, fp);
                if (feof(fp))
                        fatal("not enough lines in temperature file2\n");
                if (sscanf(str, "iface_%s%lf", name, &val) != 2){
                        fatal("invalid temperature file format2\n");
		}
                idx = get_blk_index(flp, name);
                idx = idx + j * (flp->n_units); /* LYM */
                if (idx >= 0)
                        temp[idx+IFACE*(cmpcount*flp->n_units+attachcount)] = val;
                else    /* since get_blk_index calls fatal, the line below cannot be reached    */
                        fatal ("unit in temperature file not found in floorplan\n");
        }

	for (i=0; i < attachcount; i++) {
                fgets(str, STR_SIZE, fp);
                if (feof(fp))
                        fatal("not enough lines in temperature file2\n");
                if (sscanf(str, "iface_%s%lf", name, &val) != 2)
                        printf("invalid temperature file format2, %d\n",attachcount);
/*                idx = get_blk_index(flp, name);*/
/*                idx = idx + j * (flp->n_units); */ /* LYM */
/*               if (idx >= 0) */
		  		idx =  cmpcount*flp->n_units+i;
                  temp[idx+IFACE*(cmpcount*flp->n_units+attachcount)] = val;
/*                else    */ /* since get_blk_index calls fatal, the line below cannot be reached    */
/*                        fatal ("unit in temperature file not found in floorplan\n");*/
        }

        for (i=0; i < flp->n_units; i++) {
                fgets(str, STR_SIZE, fp);
                if (feof(fp))
                        fatal("not enough lines in temperature file3\n");
                if (sscanf(str, "hsp_%s%lf", name, &val) != 2)
                        fatal("invalid temperature file format3\n");
                idx = get_blk_index(flp, name);
                idx = idx + j * (flp->n_units); /* LYM */
                if (idx >= 0)
                        temp[idx+HSP*(cmpcount*flp->n_units+attachcount)] = val;
                else    /* since get_blk_index calls fatal, the line below cannot be reached    */
                        fatal ("unit in temperature file not found in floorplan\n");
        }

        for (i=0; i < attachcount; i++) {
                fgets(str, STR_SIZE, fp);
                if (feof(fp))
                        fatal("not enough lines in temperature file3\n");
                if (sscanf(str, "hsp_%s%lf", name, &val) != 2)
                        fatal("invalid temperature file format3\n");
/*              idx = get_blk_index(flp, name); */
/*              idx = idx + j * (flp->n_units); */ /* LYM */
		  		idx = cmpcount*flp->n_units+i;
/*              if (idx >= 0) */
                        temp[idx+HSP*(cmpcount*flp->n_units+attachcount)] = val;
/*              else   */ /* since get_blk_index calls fatal, the line below cannot be reached    */
/*                      fatal ("unit in temperature file not found in floorplan\n"); */
        }


} /* LYM */

        /* internal node temperatures   */
        for (i=0; i < EXTRA; i++) {
                fgets(str, STR_SIZE, fp);
                if (feof(fp))
                        fatal("not enough lines in temperature file4\n");
                if (sscanf(str, "%s%lf", name, &val) != 2)
                        fatal("invalid temperature file format4\n");
                sprintf(str, "inode_%d", i);
                if (strcasecmp(str, name))
                        fatal("invalid temperature file format5\n");
                temp[i+ 3 * (cmpcount * flp->n_units+attachcount)] = val;   /* LYM */
        }


        /* clipping     */
        if (clip && (max > thermal_threshold)) {
                /* if max has to be brought down to thermal_threshold,
                 * (w.r.t the ambient) what is the scale down factor?
                 */
                double factor = (thermal_threshold - ambient) / (max - ambient);

                /* scale down all temperature differences (from ambient) by the same factor     */
                for (i=0; i < NL*(flp->n_units*cmpcount+attachcount) + EXTRA; i++)
                        temp[i] = (temp[i]-ambient)*factor + ambient; /* LYM */
        }
		fclose(fp);
		fclose(cmpfp);
		fclose(attachfp);
}

/* 
 * read temperature vector alloced using 'hotspot_vector' from 'file'
 * which was dumped using 'dump_vector'. values are clipped to thermal
 * threshold based on 'clip'
 */ 
int read_temp_classic(flp_t *flp, double *temp, char *file, int clip) // changed to bool by jdonald
{
	int i, idx;
	double max=0, val;
	char str[STR_SIZE], name[STR_SIZE];
	FILE *fp = fopen (file, "r");
	if (!fp) {
		/*
		sprintf (str,"error: %s could not be opened for reading\n", file);
		fatal(str); */
		return 0;
	}	

	/* find max temp on the chip	*/
	for (i=0; i < flp->n_units; i++) {
		fgets(str, STR_SIZE, fp);
		if (feof(fp))
			fatal("not enough lines in temperature file\n");
		if (sscanf(str, "%s%lf", name, &val) != 2)
			fatal("invalid temperature file format\n");
		idx = get_blk_index(flp, name);
		if (idx >= 0)
			temp[idx] = val;
		else	/* since get_blk_index calls fatal, the line below cannot be reached	*/
			fatal ("unit in temperature file not found in floorplan\n");
		if (temp[idx] > max)
			max = temp[idx];
	}

	/* interface material temperatures	*/
	for (i=0; i < flp->n_units; i++) {
		fgets(str, STR_SIZE, fp);
		if (feof(fp))
			fatal("not enough lines in temperature file\n");
		if (sscanf(str, "iface_%s%lf", name, &val) != 2)
			fatal("invalid temperature file format\n");
		idx = get_blk_index(flp, name);
		if (idx >= 0)
			temp[idx+IFACE*flp->n_units] = val;
		else	/* since get_blk_index calls fatal, the line below cannot be reached	*/
			fatal ("unit in temperature file not found in floorplan\n");
	}

	/* heat spreader temperatures	*/
	for (i=0; i < flp->n_units; i++) {
		fgets(str, STR_SIZE, fp);
		if (feof(fp))
			fatal("not enough lines in temperature file\n");
		if (sscanf(str, "hsp_%s%lf", name, &val) != 2)
			fatal("invalid temperature file format\n");
		idx = get_blk_index(flp, name);
		if (idx >= 0)
			temp[idx+HSP*flp->n_units] = val;
		else	/* since get_blk_index calls fatal, the line below cannot be reached	*/
			fatal ("unit in temperature file not found in floorplan\n");
	}

	/* internal node temperatures	*/	
	for (i=0; i < EXTRA; i++) {
		fgets(str, STR_SIZE, fp);
		if (feof(fp))
			fatal("not enough lines in temperature file\n");
		if (sscanf(str, "%s%lf", name, &val) != 2)
			fatal("invalid temperature file format\n");
		sprintf(str, "inode_%d", i);
		if (strcasecmp(str, name))
			fatal("invalid temperature file format\n");
		temp[i+NL*flp->n_units] = val;	
	}

	fclose(fp);	

	/* clipping	*/
	if (clip && (max > thermal_threshold)) {
		/* if max has to be brought down to thermal_threshold, 
		 * (w.r.t the ambient) what is the scale down factor?
		 */
		double factor = (thermal_threshold - ambient) / (max - ambient);
	
		/* scale down all temperature differences (from ambient) by the same factor	*/
		for (i=0; i < NL*flp->n_units + EXTRA; i++)
			temp[i] = (temp[i]-ambient)*factor + ambient;
	}
	return 1;
}

/* LYM */
void cleanup_hotspot(int n_units)
{
        free_matrix(inva, NL*n_units+EXTRA);
        inva = NULL;
        free_matrix(b, NL*n_units+EXTRA);
        b = NULL;
        free_matrix(invb, NL*n_units+EXTRA);
        invb = NULL;
        free_matrix(c, NL*n_units+EXTRA);
        c = NULL;
}





int eq(float x, float y)
{
        return (fabs(x-y) <  DELTA);
}

int le(float x, float y)
{
        return ((x < y) || eq(x,y));
}

int ge(float x, float y)
{
        return ((x > y) || eq(x,y));
}

void print_flp_fig (const flp_t *flp)
{
        int i;
        unit_t *unit;

        fprintf(stderr, "FIG starts\n");
        for (i=0; i< flp->n_units; i++) {
                unit = &flp->units[i];
                fprintf(stderr, "%.5f %.5f %.5f %.5f %.5f %.5f %.5f %.5f %.5f %.5f\n", unit->leftx, unit->bottomy, \
                                unit->leftx, unit->bottomy + unit->height, unit->leftx + unit->width, unit->bottomy + \
                                unit->height, unit->leftx + unit->width, unit->bottomy, unit->leftx, unit->bottomy);
                fprintf(stderr, "%s\n", unit->name);
        }
        fprintf(stderr, "FIG ends\n");
}

/* LYM CMP TEMP */
flp_t *read_cmpflp(const char *file, const char *cmpfile, const char *attachfile)
{
    char str[STR_SIZE], copy[STR_SIZE], processor_name[STR_SIZE];
    FILE *fp = fopen(file, "r");
    FILE *cmpfp = fopen(cmpfile, "r");
	FILE *attachfp = fopen(attachfile, "r");
    flp_t *flp;
    int i=0, count = 0, cmpcount = 0, attachcount = 0, j=0;
    char *ptr;
    double left, bottom;
    double core_width = 0.0; // jdonald

    if (!fp) {
        sprintf(str, "error opening file %s\n", file);
        fatal(str);
    }

    if (!cmpfp) {
        sprintf(str, "error opening file %s\n", cmpfile);
        fatal(str);
    }

	if (!attachfp) {
		sprintf(str, "error opening file %s\n", attachfile);
		fatal(str);
	}

    while(!feof(fp)) {                /* first pass   */
    	fgets(str, STR_SIZE, fp);
        if (feof(fp))
        	break;
        ptr =   strtok(str, " \t\n");
		if ( ptr && ptr[0] != '#')      /* ignore comments and empty lines      */
        count++;
    }

    while(!feof(cmpfp)) {
        fgets(str, STR_SIZE, cmpfp);
        if (feof(cmpfp))
            break;
        ptr = strtok(str, " \t\n");
        if (ptr && ptr[0] != '#')
            cmpcount++;
    }

	while(!feof(attachfp)) {
		fgets(str, STR_SIZE, attachfp);
		if (feof(attachfp))
			break;
		ptr = strtok(str, " \t\n");
		if (ptr && ptr[0] != '#')
			attachcount++;
	}

    if (cmpcount < processor_num - temperature_core_skip)
    	fatal("read_flp(): not enough cores available in floorplan file\n");

    if(!count)
		fatal("no units specified in the floorplan file\n");
    flp = (flp_t *) calloc (1, sizeof(flp_t));
    if(!flp)
    	fatal("memory allocation error\n");
    flp->units = (unit_t *) calloc (count*cmpcount+attachcount, sizeof(unit_t));
    if (!flp->units)
    	fatal("memory allocation error\n");
    flp->n_units = count*cmpcount+attachcount;
    flp->core_units = count; // jdonald
	flp->attach_units = attachcount;

    fseek(fp, 0, SEEK_SET);
    while(!feof(fp)) {               /* second pass  */
        fgets(str, STR_SIZE, fp);
        if (feof(fp))
	        break;
        strcpy(copy, str);
        ptr =   strtok(str, " \t\n");
        if ( ptr && ptr[0] != '#') {    /* ignore comments and empty lines      */
            if (sscanf(copy, "%s%lf%lf%lf%lf", flp->units[i].name, &flp->units[i].width, \
                  &flp->units[i].height, &flp->units[i].leftx, &flp->units[i].bottomy) != 5)
                fatal("invalid floorplan file format\n");
            if (flp->units[i].width + flp->units[i].leftx > core_width)
            	core_width = flp->units[i].width + flp->units[i].leftx;
        	i++;
        }
    }
    fclose(fp);

    fseek(cmpfp, 0 , SEEK_SET);
    i = 0;
    while(!feof(cmpfp)){
        fgets(str, STR_SIZE, cmpfp);
        if (feof(cmpfp))
        	break;
        strcpy(copy, str);
        ptr = strtok(str, " \t\n");
        if ( ptr && ptr[0] != '#'){
	    if (sscanf(copy, "%s%lf%lf", processor_name, &left, &bottom) != 3)
	        fatal("invalid cmp floorplan file format\n");
            if (i>0)
                for (j=0;j< count ;j++){
                    flp->units[i*count + j].name[0] = '\0';
                    strcat(flp->units[i*count + j].name,flp->units[j].name);
                    strcat(flp->units[i*count + j].name,"_");
                    strcat(flp->units[i*count + j].name,processor_name);
                    flp->units[i*count + j].width = flp->units[j].width;
                    flp->units[i*count + j].height = flp->units[j].height;
                    if (flip_core_flp && i >= (cmpcount+1)/2)
                    	flp->units[i*count + j].leftx = left + core_width - flp->units[j].leftx - flp->units[j].width; // horizontally flip the second half of cores
                    else
                        flp->units[i*count + j].leftx = flp->units[j].leftx + left;
                    flp->units[i*count + j].bottomy = flp->units[j].bottomy + bottom;
                }
            i++;
        }
    }

	fseek(attachfp, 0, SEEK_SET);
	i = 0;
    while(!feof(attachfp)) {               /* second pass  */
        fgets(str, STR_SIZE, attachfp);
        if (feof(attachfp))
            break;
        strcpy(copy, str);
        ptr =   strtok(str, " \t\n");
        if ( ptr && ptr[0] != '#') {    /* ignore comments and empty lines      */
            if (sscanf(copy, "%s%lf%lf%lf%lf", flp->units[i+ cmpcount*count].name, &flp->units[i+ cmpcount*count].width, \
                 &flp->units[i+ cmpcount*count].height, &flp->units[i+ cmpcount*count].leftx, &flp->units[i+ cmpcount*count].bottomy) != 5)
            fatal("invalid floorplan file format\n");
            i++;
    	}
    }
	fclose(attachfp);

    fseek(cmpfp, 0 , SEEK_SET);
    i = 0;
    while(!feof(cmpfp)){
        fgets(str, STR_SIZE, cmpfp);
        if (feof(cmpfp))
            break;
        strcpy(copy, str);
        ptr = strtok(str, " \t\n");
        if ( ptr && ptr[0] != '#'){
            if (sscanf(copy, "%s%lf%lf", processor_name, &left, &bottom) != 3)
                fatal("invalid cmp floorplan file format\n");
            for (j=0;j<count;j++){
                strcat(flp->units[i*count + j].name,"_");
                strcat(flp->units[i*count + j].name,processor_name);
				flp->units[i*count + j].leftx = flp->units[j].leftx + left;
				flp->units[i*count + j].bottomy = flp->units[j].bottomy + bottom;
				// these two lines added by jdonald to fix the first core's floorplan.
            }
            break; // it seemsthis statement ensures only the *first core* is affected by this loop. -jdonald
        }
    }

    fclose(cmpfp);

    // print_flp_fig(flp);
    // use the -printflp knob to print the floorplan
    return flp;
}
/* LYM CMP TEMP END */

flp_t *read_flp(const char *file)
{
        char str[STR_SIZE], copy[STR_SIZE];
        FILE *fp = fopen(file, "r");
        flp_t *flp;
        int i=0, count = 0;
        char *ptr;

        if (!fp) {
                sprintf(str, "error opening file %s\n", file);
                fatal(str);
        }

        while(!feof(fp))                /* first pass   */
        {
                fgets(str, STR_SIZE, fp);
                if (feof(fp))
                        break;
                ptr =   strtok(str, " \t\n");
 if ( ptr && ptr[0] != '#')      /* ignore comments and empty lines      */
                        count++;
        }

        if(!count)
                fatal("no units specified in the floorplan file\n");

        flp = (flp_t *) calloc (1, sizeof(flp_t));
        if(!flp)
                fatal("memory allocation error\n");
        flp->units = (unit_t *) calloc (count, sizeof(unit_t));
        if (!flp->units)
                fatal("memory allocation error\n");
        flp->n_units = count;

        fseek(fp, 0, SEEK_SET);
        while(!feof(fp))                /* second pass  */
        {
                fgets(str, STR_SIZE, fp);
                if (feof(fp))
                        break;
                strcpy(copy, str);
                ptr =   strtok(str, " \t\n");
                if ( ptr && ptr[0] != '#') {    /* ignore comments and empty lines      */
                        if (sscanf(copy, "%s%lf%lf%lf%lf", flp->units[i].name, &flp->units[i].width, \
                        &flp->units[i].height, &flp->units[i].leftx, &flp->units[i].bottomy) != 5)
                                fatal("invalid floorplan file format\n");
                        i++;
                }
        }

        fclose(fp);
        // print_flp_fig(flp);
	    // no need to print this out if read_cmpflp() also prints out one.
        return flp;
}


void free_flp(flp_t *flp)
{
        free(flp->units);

        free(flp);
}

int get_blk_index(const flp_t *flp, char *name)
{
        int i;
        char msg[STR_SIZE];

        if (!flp)
                fatal("null pointer in get_blk_index\n");

        for (i = 0; i < flp->n_units; i++) {
                if (!strcasecmp(name, flp->units[i].name)) {
                        return i;
                }
        }

        sprintf(msg, "block %s not found. flp->n_units = %d\n", name, flp->n_units);
        fatal(msg);
        return -1;
}

int is_horiz_adj(const flp_t *flp, int i, int j)
{
        double x1, x2, x3, x4;
        double y1, y2, y3, y4;

        if (i == j)
                return 0;

        x1 = flp->units[i].leftx;
        x2 = x1 + flp->units[i].width;
        x3 = flp->units[j].leftx;
        x4 = x3 + flp->units[j].width;

        y1 = flp->units[i].bottomy;
        y2 = y1 + flp->units[i].height;
        y3 = flp->units[j].bottomy;
        y4 = y3 + flp->units[j].height;
  /* diagonally adjacent => not adjacent */
        if (eq(x2,x3) && eq(y2,y3))
                return 0;
        if (eq(x1,x4) && eq(y1,y4))
                return 0;
        if (eq(x2,x3) && eq(y1,y4))
                return 0;
        if (eq(x1,x4) && eq(y2,y3))
                return 0;

        if (eq(x1,x4) || eq(x2,x3))
                if ((y3 >= y1 && y3 <= y2) || (y4 >= y1 && y4 <= y2) ||
                    (y1 >= y3 && y1 <= y4) || (y2 >= y3 && y2 <= y4))
                        return 1;

        return 0;
}

int is_vert_adj (const flp_t *flp, int i, int j)
{
        double x1, x2, x3, x4;
        double y1, y2, y3, y4;

        if (i == j)
                return 0;

        x1 = flp->units[i].leftx;
        x2 = x1 + flp->units[i].width;
        x3 = flp->units[j].leftx;
        x4 = x3 + flp->units[j].width;

        y1 = flp->units[i].bottomy;
        y2 = y1 + flp->units[i].height;
        y3 = flp->units[j].bottomy;
        y4 = y3 + flp->units[j].height;

        /* diagonally adjacent => not adjacent */
        if (eq(x2,x3) && eq(y2,y3))
    		return 0;
        if (eq(x1,x4) && eq(y1,y4))
                return 0;
        if (eq(x2,x3) && eq(y1,y4))
                return 0;
        if (eq(x1,x4) && eq(y2,y3))
                return 0;

        if (eq(y1,y4) || eq(y2,y3))
                if ((x3 >= x1 && x3 <= x2) || (x4 >= x1 && x4 <= x2) ||
                    (x1 >= x3 && x1 <= x4) || (x2 >= x3 && x2 <= x4))
                        return 1;

        return 0;
}

double get_shared_len(const flp_t *flp, int i, int j)
{
        double p11, p12, p21, p22;
        p11 = p12 = p21 = p22 = 0.0;

        if (i==j)
                return 0;

        if (is_horiz_adj(flp, i, j)) {
                p11 = flp->units[i].bottomy;
                p12 = p11 + flp->units[i].height;
                p21 = flp->units[j].bottomy;
                p22 = p21 + flp->units[j].height;
        }

        if (is_vert_adj(flp, i, j)) {
                p11 = flp->units[i].leftx;
                p12 = p11 + flp->units[i].width;
                p21 = flp->units[j].leftx;
                p22 = p21 + flp->units[j].width;
        }

        return (MIN(p12, p22) - MAX(p11, p21));

}

double get_total_width(const flp_t *flp)
{
        int i;
        double min_x = flp->units[0].leftx;
        double max_x = flp->units[0].leftx + flp->units[0].width;

        for (i=1; i < flp->n_units; i++) {
                if (flp->units[i].leftx < min_x)
                        min_x = flp->units[i].leftx;
                if (flp->units[i].leftx + flp->units[i].width > max_x)
                        max_x = flp->units[i].leftx + flp->units[i].width;
        }

        return (max_x - min_x);
}

double get_total_height(const flp_t *flp)
{
        int i;
        double min_y = flp->units[0].bottomy;
        double max_y = flp->units[0].bottomy + flp->units[0].height;

        for (i=1; i < flp->n_units; i++) {
                if (flp->units[i].bottomy < min_y)
                        min_y = flp->units[i].bottomy;
                if (flp->units[i].bottomy + flp->units[i].height > max_y)
                        max_y = flp->units[i].bottomy + flp->units[i].height;
        }


        return (max_y - min_y);
}


/* get the thermal resistance values. 'k' is the thermal conductivity
 * of the actual material (silicon or copper). Lb and Wb are bulk length
 * and width respectively. Ws is the source width. The calculations are
 * from "Constriction/Spreading Resistance Model for Electronic Packaging",
 * Seri LEE et al. ASME/JSME Thermal Engineering Conference vol. 4 1995.
 */



/* double getr(double k, double Wb, double Lb, double Ws)
{
        double theta, Bi, As, Ab, epsilon, tau, a, b, lamda, phi, psi, Rsprd;
        theta = k*(Lb/2)/(Wb*t_chip);
        As = Ws*t_chip;
        Ab = Wb*t_chip;
        a = sqrt(As/M_PI);
        b = sqrt(Ab/M_PI);
        epsilon = a/b;
        lamda = M_PI+1/(sqrt(M_PI)*epsilon);
        Bi = 1/(M_PI*k*b*theta);
        tau = Lb/(2*b);
        phi = (tanh(lamda*tau)+lamda/Bi)/(1+(lamda/Bi)*tanh(lamda*tau));
        psi = (epsilon*tau+fabs(1-epsilon)*phi)/sqrt(M_PI);
        Rsprd = M_PI*M_PI*psi/(sqrt(M_PI)*k*a);
        return Rsprd;
} */
/* LYM new getr */

double getr(double k, double Wb, double Lb, double Ws, double t)
{
        double theta, Bi, As, Ab, epsilon, tau, a, b, lamda, phi, psi, Rsprd;
        theta = (Lb/2)/(k*Wb*t);
        As = Ws*t;
        Ab = Wb*t;
        a = sqrt(As/M_PI);
        b = sqrt(Ab/M_PI);
        epsilon = a/b;
        lamda = M_PI+1/(sqrt(M_PI)*epsilon);
        Bi = 1/(M_PI*k*b*theta);
        tau = Lb/(2*b);
        phi = (tanh(lamda*tau)+lamda/Bi)/(1+(lamda/Bi)*tanh(lamda*tau));
        psi = (epsilon*tau+fabs(1-epsilon)*phi)/sqrt(M_PI);
        Rsprd = psi/(sqrt(M_PI)*k*a);
        return Rsprd;
}

void iswap (int *a, int *b)
{
        int t = *a;
        *a = *b;
        *b = t;
}

void dswap (double *a, double *b)
{
        double t = *a;
        *a = *b;
        *b = t;
}

/*
 * LUP decomposition from the pseudocode given in the CLR
 * 'Introduction to Algorithms' textbook. The matrix 'a' is
 * transformed into an in-place lower/upper triangular matrix
 * and the vector'p' carries the permutation vector such that
 *  Pa = lu, where 'P' is the matrix form of 'p'
 */

void lupdcmp(double**a, int n, int *p)
{
        int i, j, k, pivot=0;
        double max = 0;

        /* start with identity permutation      */
        for (i=0; i < n; i++)
                p[i] = i;

        for (k=0; k < n-1; k++)  {
                max = 0;
                for (i = k; i < n; i++) {
                        if (fabs(a[i][k]) > max) {
                                max = fabs(a[i][k]);
                                pivot = i;
                        }
                }
                if (eq (max, 0)){
			printf("k=%d ... i=%d!!!\n",k,i);
                        fatal ("singular matrix in lupdcmp\n");
		}

                /* bring pivot element to position      */
                iswap (&p[k], &p[pivot]);
                for (i=0; i < n; i++)
                        dswap (&a[k][i], &a[pivot][i]);
 for (i=k+1; i < n; i++) {
                        a[i][k] /= a[k][k];
                        for (j=k+1; j < n; j++)
                                a[i][j] -= a[i][k] * a[k][j];
                }
        }
}

/*
 * the matrix a is an in-place lower/upper triangular matrix
 * the following macros split them into their constituents
 */

#define LOWER(a, i, j)          ((i > j) ? a[i][j] : 0)
#define UPPER(a, i, j)          ((i <= j) ? a[i][j] : 0)


/*
 * LU forward and backward substitution from the pseudocode given
 * in the CLR 'Introduction to Algorithms' textbook. It solves ax = b
 * where, 'a' is an in-place lower/upper triangular matrix. The vector
 * 'x' carries the solution vector. 'p' is the permutation vector
 */

void lusolve(double **a, int n, int *p, double *b, double *x)
{
        int i, j;
        double *y = vector (n);
        double sum;

        /* forward substitution - solves ly = pb        */
        for (i=0; i < n; i++) {
                for (j=0, sum=0; j < i; j++)
                        sum += y[j] * LOWER(a, i, j);
                y[i] = b[p[i]] - sum;
        }

        /* backward substitution - solves ux = y        */
 for (i=n-1; i >= 0; i--) {
                for (j=i+1, sum=0; j < n; j++)
                        sum += x[j] * UPPER(a, i, j);
                x[i] = (y[i] - sum) / UPPER(a, i, i);
        }

        free_vector(y);
}


double *vector(int n)
{
        double *v;

        v=(double *)calloc(n, sizeof(double));
        if (!v) fatal("allocation failure in vector()");

        return v;
}

void free_vector(double *v)
{
        free(v);
}

void dump_vector (double *v, int n)
{
        int i;
        for (i=0; i < n; i++)
                fprintf(stderr, "%.5f\t", v[i]);
        fprintf(stderr, "\n");
}

int *ivector(int n)
{
        int *v;

        v = (int *)calloc(n, sizeof(int));
        if (!v) fatal("allocation failure in ivector()");

        return v;
}

void free_ivector(int *v)
{
        free(v);
}

void dump_ivector (int *v, int n)
{
        int i;
        for (i=0; i < n; i++)
                fprintf(stderr, "%d\t", v[i]);
        fprintf(stderr, "\n\n");
}

/* allocate a matrix with subscript range m[1...nr][1...nc] */
double **matrix(int nr, int nc)
{
        int i;
        double **m;

        m = (double **) calloc (nr, sizeof(double *));
        assert(m != NULL);

        for (i = 0; i < nr; i++) {
                m[i] = (double *)calloc(nc, sizeof(double));
                assert(m[i] != NULL);
        }

        return m;
}

void free_matrix(double **m, int nr)
{
        int i;
        for (i = 0; i < nr; i++)
                free(m[i]);
    free(m);
}

void dump_matrix (double **m, int nr, int nc)
{
        int i;
        for (i=0; i < nr; i++)
                dump_vector(m[i], nc);
        fprintf(stderr, "\n");
}

void copy_matrix (double **dst, double **src, int nr, int nc)
{
        int i,j;
        for (i=0; i < nr; i++)
                for (j=0; j < nc; j++)
                        dst[i][j] = src[i][j];
}

int **imatrix(int nr, int nc)
{
        int i;
        int **m;

        m = (int **) calloc (nr, sizeof(int *));
        assert(m != NULL);

        for (i = 0; i < nr; i++) {
                m[i] = (int *)calloc(nc, sizeof(int));
                assert(m[i] != NULL);
        }

        return m;
}

void free_imatrix(int **m, int nr)
{
        int i;
        for (i = 0; i < nr; i++)
  free(m[i]);
        free(m);
}

void dump_imatrix (int **m, int nr, int nc)
{
        int i;
        for (i=0; i < nr; i++)
                dump_ivector(m[i], nc);
        fprintf(stderr, "\n");
}

/* matmult: C = AB, A, B are n x n square matrices      */
void matmult(double **c, double **a, double **b, int n)
{
        int i, j, k;

        for (i = 0; i < n; i++)
                for (j = 0; j < n; j++) {
                        c[i][j] = 0;
                        for (k = 0; k < n; k++)
                                c[i][j] += a[i][k] * b[k][j];
                }
}


/* mult of an n x n matrix and an n x 1 column vector   */
void matvectmult1(double *vout, double **m, double *vin, int n)
{
        int i, j;
        for (i = 0; i < n; i++) {
                vout[i] = 0;
                for (j = 0; j < n; j++)
                        vout[i] += m[i][j] * vin[j];
        }
}

/* INV = M^-1, INV, M are n by n matrices */
void matinv(double **INV, double **M, int n)
{
        int *p, i, j;
        double *col, *x;

        p = ivector(n);
        col = vector(n);
        x = vector(n);

        lupdcmp(M, n, p);

        for (j = 0; j < n; j++) {
                for (i = 0; i < n; i++) col[i]=0.0;
                col[j] = 1.0;
                lusolve(M, n, p, col, x);
                for (i = 0; i < n; i++) INV[i][j]=x[i];
        }

        free_ivector(p);
        free_vector(col);
        free_vector(x);
}

double calculate_core_power(int corenum, const double* power, const flp_t* flp) {
/* pass the overalls_power vector from temp_inner.sync.macros
 * to this function.
 */
// this function is no longer called (but it can be used for sanity checks),
// since TEMPERATURE was decoupled from POWER_MODE
	double total = 0.0;
	int i;
	for (i = 0; i < flp->core_units; i += 1)
		total += power[corenum * flp->core_units + i];
	return total;
}


double calculate_core_temperature(int corenum, const double* temp, const flp_t* flp) {
/* pass the most recently calculated temperature matrix (either steady_temp
 * or hotspottemp) to this function.
 */
	int i, n;
	double total_area = 0.0, total_product = 0.0;
	assert(corenum >= temperature_core_skip);
	corenum -= temperature_core_skip; // adjust for ignored cores
	n = flp->core_units;
	for (i = 0; i < n; i += 1) {
		double area;
		unit_t unit;
		unit = flp->units[corenum*flp->core_units + i];
		area = unit.width * unit.height;
		total_area += area;
		total_product += area * temp[corenum*flp->core_units + i];
	}
	return total_product / total_area;
}

double calculate_l2cache_temperature(const double* temp, const flp_t* flp) {
/* pass the most recently calculated temperature matrix (either steady_temp
 * or hotspottemp) to this function.
 */
	int i, n;
	double total_area = 0.0, total_product = 0.0;
	n = flp->attach_units;
	for (i = 0; i < n; i += 1) {
		double area;
		unit_t unit;
		unit = flp->units[flp->n_units-flp->attach_units + i];
		area = unit.width * unit.height;
		total_area += area;
		total_product += area * temp[flp->n_units-flp->attach_units + i];
	}
	return total_product / total_area;
}
