#ifndef H_trauma
#define H_trauma

enum xtrauma {
  TR_ILLEGAL,
  TR_NORMAL,

/*** instruction fetch stalls ***/
  TR_INFA,	/* next fetch address bubble */
  TR_ITLB,	/* ITLB miss bubble */
  TR_I2TLB,	/* ITLB miss bubble */
  TR_IL2,	/* L2 miss bubble */
  TR_IL15,	/* L15 miss bubble */
  TR_IL1,	/* L1 miss bubble */
  TR_IPRE,	/* prefetch hit bubble */
  TR_IPRED,	/* mispredict bubble */
  TR_IFULL,	/* IBuf full */
  TR_IINF,	/* inflight max exceeded */
  TR_INBR,	/* too many predicted branches */
  TR_ILDST,	/* flushed because of load store conflict */

  TR_DEC,	/* decode stage stall */

  TR_REN,	/* rename stage stall */
  #if 0
  TR_RGPR,	/* couldn't rename - GPR */
  TR_RSPR,	/* couldn't rename - SPR */
  TR_RFPR,	/* couldn't rename - FPR */
  TR_RCCR,	/* couldn't rename - CCR */
  #endif

/*** these MUST be in the same order as the units ***/
  TR_DFIX,	/* couldn't dispatch - FIX q full */
  TR_DFPU,	/* couldn't dispatch - FPU q full */
  TR_DMEM,	/* couldn't dispatch - MEM q full */
  TR_DBR,	/* couldn't dispatch - BR q full */
  TR_DLOG,
  TR_DCMPLX,

/*** these MUST be in the same order as the units ***/
  TR_QFIXF,	/* more than one cycle in FIX q */
  TR_QFPUF,	/* more than one cycle in FPU q */
  TR_QMEMF,	/* more than one cycle in MEM q */
  TR_QBRF,	/* more than one cycle in BR q */
  TR_QLOG,
  TR_QCMPLX,

  TR_MSQF,	/* couldn't issue mem op because store q full */
  TR_MSQC,	/* couldn't issue mem op because store q at cutoff */
  TR_MRQF,	/* couldn't issue mem op because reorder q full */
  TR_MDQF,	/* couldn't issue mem op because dmiss q full */
  TR_MCQF,	/* couldn't issue mem op because castout q full */
  TR_MSTD,	/* couldn't issue mem op because data not ready */
  TR_MTLB1,	/* couldn't issue mem op because of ERAT-TLB miss */
  TR_MTLB2,	/* couldn't issue mem op because of D-TLB miss */
  TR_ML2,	/* couldn't issue mem op because of D-L1 miss */
  TR_ML1,	/* couldn't issue mem op because of L2 miss */

/*** these MUST be in the same order as the units ***/
  TR_RFIX,	/* waiting for result from FIX q */
  TR_RFPU,	/* waiting for result from FPU q */
  TR_RMEM,	/* waiting for result from MEM q */
  TR_RBR,	/* waiting for result from BR q */
  TR_LOG,	/* waiting for result from LOG q */
  TR_CMPLX,	/* waiting for result from LOG q */
  TR_STDATA,	/* waiting for data value for store q */

  TR_RST,	/* couldn't retire store op because no ports avail */
  TR_NUM
};

typedef enum xtrauma trauma;

char * trauma_names[] = {
  "illegal",
  "normal ",

  "if_nfa ",
  "if_tlb1",
  "if_tlb2",
  "if_l2  ",
  "if_l15 ",
  "if_l1  ",
  "if_pref",
  "if_pred",
  "if_full",
  "if_flit",
  "if_brch",
  "if_ldst",

  "decode ",
  "rename ",
  #if 0
  "?en_gpr",
  "?en_spr",
  "?en_fpr",
  "?en_ccr",
  #endif

  "diq_fix",
  "diq_fpu",
  "diq_mem",
  "diq_br ",
  "diq_log",
  "diq_cmplx",

  "ful_fix",
  "ful_fpu",
  "ful_mem",
  "ful_br ",
  "ful_log",
  "ful_cmplx",

  "mm_stqf",
  "mm_stqc",
  "mm_roqf",
  "mm_dmqf",
  "mm_dcqf",
  "mm_stnd",
  "mm_tlb1",
  "mm_tlb2",
  "mm_dl2 ",
  "mm_dl1 ",

  "rg_fix ",
  "rg_fpu ",
  "rg_mem ",
  "rg_br  ",
  "rg_log  ",
  "rg_cmplx",

  "st_data",

  "ret_st "
};

#endif
