// dep_prep_process.c

// Three functions: process_illegal(), process_iword(), and process_branch(),
// all non-reentrant because they link with Aria which has global data!
//
// This file itself doesn't have any global variables though.

#include "ansi_assert.h"
#include "ansi_stdlib.h"
#include "ansi_stdio.h"
#include "dep_prep_process.h" // jdonald put back
#include "opc.h"

#define EXTRA0_STRING_SPR	30
#define EXTRA1_STRING_SPR	31
#define EXTRA_PREDICATION_SPR	31
#define EXTRA_LUX_SPR		31
#define EXTRA_LHAU_SPR		31
#define EXTRA_RLWIMI_SPR	31
#define EXTRA_MFCR0_SPR		31
#define EXTRA_MFCR1_SPR		30
#define EXTRA_MFCR2_SPR		29
#define EXTRA_MFSPR0_SPR	31
#define EXTRA_MFSPR1_SPR	30
#define EXTRA_MFSPR2_SPR	29
#define EXTRA_DEFAULT_SPR	26

#ifndef DEBUG
	#define DEBUG 1
#endif // jdonald 2-14-2005, unsure of effect

#if !defined(IN_TRANS) && !defined(_AIX) // jdonald fixing this unusual _fatal() missing
#define fatal(s) {\
	fprintf(stderr, "fatal: %s: line %d: %s: %s", __FILE__, __LINE__, __FUNCTION__, s); \
	exit(EXIT_FAILURE);	\
	}
#endif // !defined(_AIX)

struct xdep_prep_ctl xdep_prep_ctl = { // jdonald: oops this isn't const, altered by dep_prep.C
  IFETCH_MAX,
  IFETCH_ALIGN,
  IFETCH_ALIGN_MASK,
  ICACHE_ALIGN,
  ICACHE_MASK,

  BRANCH_LATENCY,
  CRL_LATENCY,
  FPU_LATENCY,
  LOAD_LATENCY,
  SUBWORD_LOAD_LATENCY,
  ARITHMETIC_LOAD_LATENCY,
  max( SUBWORD_LOAD_LATENCY, ARITHMETIC_LOAD_LATENCY),
  FLOAT_LOAD_LATENCY,
  STORE_LATENCY,
  FIX_LATENCY,
  IMUL_LATENCY,
  IDIV_LATENCY,
  FDIV_LATENCY,
  CMP_LATENCY,

  BRANCH_AVAIL,
  CRL_AVAIL,
  FPU_AVAIL,
  LOAD_AVAIL,
  SUBWORD_LOAD_AVAIL,
  ARITHMETIC_LOAD_AVAIL,
  max( SUBWORD_LOAD_AVAIL, ARITHMETIC_LOAD_AVAIL),
  FLOAT_LOAD_AVAIL,
  STORE_AVAIL,
  FIX_AVAIL,
  IMUL_AVAIL,
  IDIV_AVAIL,
  FDIV_AVAIL,
  CMP_AVAIL,

  BRANCH_UNIT,
  BRANCH_UNIT + 1,
  FPU_UNIT,
  LOAD_UNIT,
  STORE_UNIT,
  FIX_UNIT,
  IMUL_UNIT,
  IDIV_UNIT,

  USE_GP + 1,
  SPLIT_ALL,
  SPLIT_FXLDAU,
  SPLIT_FXLDA,
  SPLIT_FXLDU,
  SPLIT_FPLDU,
  SPLIT_FXSTUX,
  SPLIT_FXSTU,
  SPLIT_FPSTU,
  SPLIT_STU3,
  SPLIT_FXST,
  SPLIT_FPST,
  SPLIT_MULT,
  SPLIT_CRL,
  SPLIT_SPR,
  SPLIT_STRING,
  SPLIT_SHOW,

  PRED_DO,
  PRED_SHOW,
  PRED_MODE,
  PRED_DIST,

  WIDE_ARCH_REG,
  LG2ARCH_MAX
};

#if( !defined(min))
#define min(x,y)	(((x)<(y))?(x):(y))
#endif

#define RES_NONE	0x00000000
#define RESCPL_NONE	0x80808080
#define RES_ONE_GPR	0x00000001
#define RESCPL_ONE_GPR	0x8080807f
#define RES_ONE_FPR	0x00000100
#define RESCPL_ONE_FPR	0x80807f80
#define RES_ONE_CR	0x00010000
#define RESCPL_ONE_CR	0x807f8080
#define RES_ONE_SPR	0x01000000
#define RESCPL_ONE_SPR	0x7f808080

#if( IN_TRANS )
#define cvt_dep_reg( val, reg, lg2max )	\
  {					\
    unsigned	tmp_num;		\
    unsigned	tmp_clss;		\
    if( (reg) < OPC_FIRST_CR) {	        \
      assert( OPC_REG_IS_A_SPR(reg) );	\
      tmp_clss = 3;			\
      tmp_num = (reg);	                \
    }					\
    else if( OPC_REG_IS_A_CR(reg) ) {	\
      tmp_clss = 2;			\
      tmp_num = OPC_CVT_CR_RVAL(reg);	\
    }					\
    else if( OPC_REG_IS_A_FPR(reg) ) {	\
      tmp_clss = 1;			\
      tmp_num = OPC_CVT_FPR_RVAL(reg);	\
    }					\
    else if( OPC_REG_IS_A_GPR(reg) ) {	\
      tmp_clss = 0;			\
      tmp_num = OPC_CVT_GPR_RVAL(reg);	\
    }					\
    else if( OPC_REG_IS_A_SPR(reg) ) {	\
      tmp_clss = 0;			\
      tmp_num = EXTRA_DEFAULT_SPR;	\
    }					\
    else {				\
      fprintf(stderr, "FAILED: [%s:%d] cvt_dep_reg : unexpected register %d\n", __FILE__, __LINE__, (reg) );	\
      exit( EXIT_FAILURE );		\
    }					\
    if( tmp_num >= 32 ) {		\
      fprintf(stderr, "FAILED: [%s:%d] cvt_dep_reg : number %d of reg %d too large\n",  __FILE__, __LINE__,tmp_num, (reg));	\
      exit( EXIT_FAILURE );		\
    }					\
					\
    (val) = (tmp_clss << LG2ARCH_MAX) | tmp_num;	\
  }
#else
#define cvt_dep_reg( val, reg, lg2max )	\
  {					\
    unsigned	tmp_num;		\
    unsigned	tmp_clss;		\
    if( (reg) < OPC_FIRST_CR) {	        \
      assert( OPC_REG_IS_A_SPR(reg) );	\
      tmp_clss = 3;			\
      tmp_num = (reg);	                \
    }					\
    else if( OPC_REG_IS_A_CR(reg) ) {	\
      tmp_clss = 2;			\
      tmp_num = OPC_CVT_CR_RVAL(reg);	\
    }					\
    else if( OPC_REG_IS_A_FPR(reg) ) {	\
      tmp_clss = 1;			\
      tmp_num = OPC_CVT_FPR_RVAL(reg);	\
    }					\
    else if( OPC_REG_IS_A_GPR(reg) ) {	\
      tmp_clss = 0;			\
      tmp_num = OPC_CVT_GPR_RVAL(reg);	\
    }					\
    else if( OPC_REG_IS_A_SPR(reg) ) {	\
      tmp_clss = 0;			\
      tmp_num = EXTRA_DEFAULT_SPR;	\
    }					\
    else {				\
      fprintf(stderr, "FAILED: [%s:%d] cvt_dep_reg : unexpected register %d\n", __FILE__, __LINE__, (reg) );	\
      exit( EXIT_FAILURE );		\
    }					\
    if( tmp_num >= 32 ) {		\
      fprintf(stderr, "FAILED: [%s:%d] cvt_dep_reg : number %d of reg %d too large\n", __FILE__, __LINE__, tmp_num, (reg));	\
      exit( EXIT_FAILURE );		\
    }					\
					\
    (val) = (tmp_clss << lg2max) | tmp_num;	\
  }
#endif

#if( IN_TRANS )
#define get_expands( exp, entry, n )					\
  {									\
    if( n + trans_deps_top > TRANS_DEPS_NUM ) {				\
      fprintf(the_stderr, "FAILED: [%s:%d] too many deps; recompile with TRANS_DEPS_NUM > %d\n", __FILE__, __LINE__, TRANS_DEPS_NUM); \
      exit( EXIT_FAILURE );						\
    }									\
    exp = &trans_deps[trans_deps_top];					\
    trans_deps_top += n;						\
    TRCOUT( fprintf(the_stderr, "DEP_PREP: get_expands: %d starting at %08x\n", n, (unsigned) exp));		\
  }
#else
#define get_expands( exp, entry, n )					\
  {									\
    exp = (xdep_prep_info *) malloc( n*sizeof(xdep_prep_info) ); 	\
    if( exp == 0 ) {							\
      fprintf(stderr, "FAILED: [%s:%d] out of memory expanding instructions\n", __FILE__, __LINE__);	\
      exit( EXIT_FAILURE );						\
    }									\
  }
#endif

#if( IN_TRANS )
#define get_overflows( overflow, entry, n)				\
  {									\
    if( n + trans_overs_top > TRANS_OVERS_NUM ) {			\
      fprintf(the_stderr, "FAILED: [%s:%d], too many deps; recompile with TRANS_OVERS_NUM > %d\n", __FILE__, __LINE__, TRANS_OVERS_NUM); \
      exit( EXIT_FAILURE );						\
    }									\
    overflow = &trans_overs[trans_overs_top];				\
    trans_overs_top += n;						\
    TRCOUT( fprintf(the_stderr, "DEP_PREP: get_overflows: %d starting at %08x\n",  n, (unsigned) overflow)); \
    xdep_prep_info_overflow( entry ) = overflow;			\
  }
#else
#define get_overflows( overflow, entry, n)				\
  {									\
    overflow = (xarch *) malloc( n * sizeof(xarch));			\
    if( overflow == 0 ) {						\
      fprintf(stderr, "FAILED: [%s:%d] process_iword: out of memory allocating overflow\n", __FILE__, __LINE__);	\
      exit( EXIT_FAILURE );						\
    }									\
    xdep_prep_info_overflow( entry ) = overflow;			\
  }
#endif

void
process_illegal(
	unsigned		iaddr,
	xdep_prep_info *	entry
	)
{
  unsigned fetch;
  unsigned cache_fetch;
  unsigned align_fetch;

  xdep_prep_info_res_compl(entry) = 0x80808080;
  xdep_prep_info_res(entry) = 0;

  xdep_prep_info_prop0(entry) = 0;
  xdep_prep_info_prop1(entry) = 0;

  set_xdep_prep_info_unit(entry, xdep_prep_ctl.fix_unit);
  set_xdep_prep_info_latency(entry, xdep_prep_ctl.fix_latency);
  set_xdep_prep_info_avail(entry, xdep_prep_ctl.fix_avail);

/****************************************************************/
/* Compute the number of instructions that would be fetched	*/
/* if this instruction were the target of the branch; this is	*/
/* one of the few fields we care about in an illegal instruction*/
/****************************************************************/

  cache_fetch = xdep_prep_ctl.cache_align - ((iaddr>>2)&xdep_prep_ctl.cache_mask);
  align_fetch = xdep_prep_ctl.fetch_max - ((iaddr>>2)&xdep_prep_ctl.fetch_align_mask);
  fetch = min(cache_fetch, align_fetch);
  assert( fetch >= 1 );
  assert( fetch <= xdep_prep_ctl.fetch_max);

  set_xdep_prep_info_fetch(entry, fetch);
  assert( xdep_prep_info_fetch(entry) == fetch);
}/* process_illegal */

void
process_iword(
	unsigned		iaddr,
	unsigned		iword,
	xdep_prep_info *	entry
#if( IN_TRANS ) // turandot
	, xcycle cycle,
	int processor_id,
	int thread_id
#endif
	)
{
  opc_idx		idx = opc_index(iword);
  unsigned		nuses = 0;
  unsigned		uses[256];
  unsigned		nsets = 0;
  unsigned		sets[256];
  unsigned		is_branch = 0;
  unsigned		is_brlk = 0;
  unsigned		is_brcond = 0;
  unsigned		is_brlink = 0;
  unsigned		is_brctr = 0;
  unsigned		is_brcrg = 0;
  unsigned		latency = 0;
  unsigned		avail = 0;
  unsigned		unit = 0;
  unsigned		is_memop = 0;
  unsigned		dlength = 0;
  unsigned		is_notaddr = 0;
  unsigned		is_load = 0;
  unsigned		is_store = 0;
  unsigned		is_multi = 0;
  unsigned		is_string = 0;
  unsigned		is_sync = 0;
  unsigned		is_athead = 0;
  unsigned		is_mtsr = 0;
  unsigned		sr = 0;
  unsigned		i;
  unsigned		n;
  unsigned		m;
  unsigned		j;
  unsigned		k;
  // unsigned		n_expand; // unused?
  unsigned		spr;
  unsigned		is_over;
  unsigned		fetch;
  unsigned		cache_fetch;
  unsigned		align_fetch;
  unsigned		seen_SO;

  xarch *		overflow;
  xarch *		xoverflow;
  xarch *		in_regs;
  xarch *		out_regs;
  xarch *		regs; // for !defined(NDEBUG)

  unsigned char 	req[4];
  unsigned		whole;
  unsigned		compl;
  extern __thread unsigned	opc_tblsprenc[];
  int			f_crm;
  int			f_si;
  unsigned		f_a;
  unsigned		r_a;
  unsigned		f_s;
  unsigned		f_d;
  unsigned		r_d;
  unsigned		r_s;
  unsigned		tmpn;
  unsigned		exp_word;
  xarch			reg;
  char			buf[256];
  xdep_prep_info *	xentry;
  xdep_prep_info *	expand = 0;
  unsigned		lg2max = xdep_prep_ctl.flag_lg2arch_max; 
  xarch			Treg;
  // xarch		CCreg; // unused?
  xarch			Areg;
  unsigned		mod_avail;
  unsigned		mod_latency;

#if( IN_TRANS ) // turandot
  /* update iword for entry, zhigangh@1/10/2003 */
  entry->iword = iword;
#endif

#if FALSE
#endif /* FALSE */
   
  xdep_prep_info_prop0(entry) = 0;
  xdep_prep_info_prop1(entry) = 0;
  xdep_prep_info_prop2(entry) = 0;
  switch( idx ) {
  case OPC_ILLEGAL: {
      /*** fprintf(stderr, "illegal instruction %08x @ %x\n", iword, iaddr); ***/
      process_illegal(iaddr, entry);
      #if( IN_TRANS )
		TRCOUT( fprintf(the_stderr, "DEP_PREP: [illegal] entry = %08x %08x@%08x %08x %08x\n",
	  (unsigned) entry, (unsigned) iword, iaddr,
	  (unsigned) xdep_prep_info_res(entry), xdep_prep_info_res_compl(entry)) );
      #endif
      return;
    }
/*    
#include "opc_dep.gen"
*/
  case OPC_ABS: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ABSO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ABS_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ABSO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADD: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADD_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDC: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDCO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDC_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDCO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDE: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDEO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDE_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDEO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDI: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDIC: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDIC_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDIS: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDME: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDMEO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDME_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDMEO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDZE: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDZEO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDZE_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ADDZEO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_AND: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_AND_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ANDC: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ANDC_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ANDI_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ANDIS_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_B: {
      is_branch = 1;
      unit = xdep_prep_ctl.branch_unit;
      latency = xdep_prep_ctl.branch_latency;
      avail = xdep_prep_ctl.branch_avail;
      break;
    }
  case OPC_BA: {
      is_branch = 1;
      unit = xdep_prep_ctl.branch_unit;
      latency = xdep_prep_ctl.branch_latency;
      avail = xdep_prep_ctl.branch_avail;
      break;
    }
  case OPC_BL: {
      is_brlk = 1;
      sets[nsets] = OPC_LR;
      nsets += 1;
      is_branch = 1;
      unit = xdep_prep_ctl.branch_unit;
      latency = xdep_prep_ctl.branch_latency;
      avail = xdep_prep_ctl.branch_avail;
      break;
    }
  case OPC_BLA: {
      is_brlk = 1;
      sets[nsets] = OPC_LR;
      nsets += 1;
      is_branch = 1;
      unit = xdep_prep_ctl.branch_unit;
      latency = xdep_prep_ctl.branch_latency;
      avail = xdep_prep_ctl.branch_avail;
      break;
    }
  case OPC_BC: {
      if ((OPC_DEC_BO(iword) & 0x14) != 0x14) {
        uses[nuses++] = OPC_CR(OPC_DEC_BI(iword) >> 2);
      }
      is_brcond = 1;
      uses[nuses] = OPC_CTR;
      nuses += !OPC_DEC_BO2(iword);
      sets[nsets] = OPC_CTR;
      nsets += !OPC_DEC_BO2(iword);
      is_branch = 1;
      unit = xdep_prep_ctl.branch_unit;
      latency = xdep_prep_ctl.branch_latency;
      avail = xdep_prep_ctl.branch_avail;
      break;
    }
  case OPC_BCA: {
      if ((OPC_DEC_BO(iword) & 0x14) != 0x14) {
        uses[nuses++] = OPC_CR(OPC_DEC_BI(iword) >> 2);
      }
      is_brcond = 1;
      uses[nuses] = OPC_CTR;
      nuses += !OPC_DEC_BO2(iword);
      sets[nsets] = OPC_CTR;
      nsets += !OPC_DEC_BO2(iword);
      is_branch = 1;
      unit = xdep_prep_ctl.branch_unit;
      latency = xdep_prep_ctl.branch_latency;
      avail = xdep_prep_ctl.branch_avail;
      break;
    }
  case OPC_BCL: {
      if ((OPC_DEC_BO(iword) & 0x14) != 0x14) {
        uses[nuses++] = OPC_CR(OPC_DEC_BI(iword) >> 2);
      }
      is_brcond = 1;
      is_brlk = 1; 
      uses[nuses] = OPC_CTR;
      nuses += !OPC_DEC_BO2(iword);
      sets[nsets] = OPC_CTR;
      nsets += !OPC_DEC_BO2(iword);
      sets[nsets] = OPC_LR;
      nsets += 1;
      is_branch = 1;
      unit = xdep_prep_ctl.branch_unit;
      latency = xdep_prep_ctl.branch_latency;
      avail = xdep_prep_ctl.branch_avail;
      break;
    }
  case OPC_BCLA: {
      if ((OPC_DEC_BO(iword) & 0x14) != 0x14) {
        uses[nuses++] = OPC_CR(OPC_DEC_BI(iword) >> 2);
      }
      is_brcond = 1;
      is_brlk = 1; 
      uses[nuses] = OPC_CTR;
      nuses += !OPC_DEC_BO2(iword);
      sets[nsets] = OPC_CTR;
      nsets += !OPC_DEC_BO2(iword);
      sets[nsets] = OPC_LR;
      nsets += 1;
      is_branch = 1;
      unit = xdep_prep_ctl.branch_unit;
      latency = xdep_prep_ctl.branch_latency;
      avail = xdep_prep_ctl.branch_avail;
      break;
    }
  case OPC_BCCTR: {
      if ((OPC_DEC_BO(iword) & 0x14) != 0x14) {
        uses[nuses++] = OPC_CR(OPC_DEC_BI(iword) >> 2);
      }
      is_brctr = 1;
      is_brcrg = !OPC_DEC_BO_bit0(iword);
      uses[nuses] = OPC_CTR;
      nuses += 1;
      sets[nsets] = OPC_CTR;
      nsets += !OPC_DEC_BO2(iword);
      is_branch = 1;
      unit = xdep_prep_ctl.branch_unit;
      latency = xdep_prep_ctl.branch_latency;
      avail = xdep_prep_ctl.branch_avail;
      break;
    }
  case OPC_BCCTRL: {
      if ((OPC_DEC_BO(iword) & 0x14) != 0x14) {
        uses[nuses++] = OPC_CR(OPC_DEC_BI(iword) >> 2);
      }
      is_brctr = 1;
      is_brcrg = !OPC_DEC_BO_bit0(iword);
      is_brlk = 1;
      uses[nuses] = OPC_CTR;
      nuses += 1;
      sets[nsets] = OPC_CTR;
      nsets += !OPC_DEC_BO2(iword);
      sets[nsets] = OPC_LR;
      nsets += 1;
      is_branch = 1;
      unit = xdep_prep_ctl.branch_unit;
      latency = xdep_prep_ctl.branch_latency;
      avail = xdep_prep_ctl.branch_avail;
      break;
    }
  case OPC_BCLR: {
      if ((OPC_DEC_BO(iword) & 0x14) != 0x14) {
        uses[nuses++] = OPC_CR(OPC_DEC_BI(iword) >> 2);
      }
      is_brlink = 1;
      is_brcrg = !OPC_DEC_BO_bit0(iword);
      uses[nuses] = OPC_LR;
      nuses += 1;
      uses[nuses] = OPC_CTR;
      nuses += !OPC_DEC_BO2(iword);
      sets[nsets] = OPC_CTR;
      nsets += !OPC_DEC_BO2(iword);
      is_branch = 1;
      unit = xdep_prep_ctl.branch_unit;
      latency = xdep_prep_ctl.branch_latency;
      avail = xdep_prep_ctl.branch_avail;
      break;
    }
  case OPC_BCLRL: {
      if ((OPC_DEC_BO(iword) & 0x14) != 0x14) {
        uses[nuses++] = OPC_CR(OPC_DEC_BI(iword) >> 2);
      }
      is_brlink = 1;
      is_brcrg = !OPC_DEC_BO_bit0(iword);
      is_brlk = 1;
      uses[nuses] = OPC_LR;
      nuses += 1;
      uses[nuses] = OPC_CTR;
      nuses += !OPC_DEC_BO2(iword);
      sets[nsets] = OPC_CTR;
      nsets += !OPC_DEC_BO2(iword);
      sets[nsets] = OPC_LR;
      nsets += 1;
      is_branch = 1;
      unit = xdep_prep_ctl.branch_unit;
      latency = xdep_prep_ctl.branch_latency;
      avail = xdep_prep_ctl.branch_avail;
      break;
    }
  case OPC_CLCS: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_CLF: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_CLI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_CMP: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_CRF(OPC_DEC_BF(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.cmp_latency;
      avail = xdep_prep_ctl.cmp_avail;
      break;
    }
  case OPC_CMPI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_CRF(OPC_DEC_BF(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.cmp_latency;
      avail = xdep_prep_ctl.cmp_avail;
      break;
    }
  case OPC_CMPL: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_CRF(OPC_DEC_BF(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.cmp_latency;
      avail = xdep_prep_ctl.cmp_avail;
      break;
    }
  case OPC_CMPLA: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_CRF(OPC_DEC_BF(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.cmp_latency;
      avail = xdep_prep_ctl.cmp_avail;
      break;
    }
  case OPC_CMPLI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_CRF(OPC_DEC_BF(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.cmp_latency;
      avail = xdep_prep_ctl.cmp_avail;
      break;
    }
  case OPC_CNTLZD: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_CNTLZD_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_CNTLZW: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_CNTLZW_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_CRAND: {
      uses[nuses++] = OPC_CR(OPC_DEC_BA(iword) >> 2);
      uses[nuses++] = OPC_CR(OPC_DEC_BB(iword) >> 2);
      uses[nuses++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      sets[nsets++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      unit = xdep_prep_ctl.crl_unit;
      latency = xdep_prep_ctl.crl_latency;
      avail = xdep_prep_ctl.crl_avail;
      break;
    }
  case OPC_CRANDC: {
      uses[nuses++] = OPC_CR(OPC_DEC_BA(iword) >> 2);
      uses[nuses++] = OPC_CR(OPC_DEC_BB(iword) >> 2);
      uses[nuses++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      sets[nsets++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      unit = xdep_prep_ctl.crl_unit;
      latency = xdep_prep_ctl.crl_latency;
      avail = xdep_prep_ctl.crl_avail;
      break;
    }
  case OPC_CREQV: {
      uses[nuses++] = OPC_CR(OPC_DEC_BA(iword) >> 2);
      uses[nuses++] = OPC_CR(OPC_DEC_BB(iword) >> 2);
      uses[nuses++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      sets[nsets++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      unit = xdep_prep_ctl.crl_unit;
      latency = xdep_prep_ctl.crl_latency;
      avail = xdep_prep_ctl.crl_avail;
      break;
    }
  case OPC_CRNAND: {
      uses[nuses++] = OPC_CR(OPC_DEC_BA(iword) >> 2);
      uses[nuses++] = OPC_CR(OPC_DEC_BB(iword) >> 2);
      uses[nuses++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      sets[nsets++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      unit = xdep_prep_ctl.crl_unit;
      latency = xdep_prep_ctl.crl_latency;
      avail = xdep_prep_ctl.crl_avail;
      break;
    }
  case OPC_CRNOR: {
      uses[nuses++] = OPC_CR(OPC_DEC_BA(iword) >> 2);
      uses[nuses++] = OPC_CR(OPC_DEC_BB(iword) >> 2);
      uses[nuses++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      sets[nsets++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      unit = xdep_prep_ctl.crl_unit;
      latency = xdep_prep_ctl.crl_latency;
      avail = xdep_prep_ctl.crl_avail;
      break;
    }
  case OPC_CROR: {
      uses[nuses++] = OPC_CR(OPC_DEC_BA(iword) >> 2);
      uses[nuses++] = OPC_CR(OPC_DEC_BB(iword) >> 2);
      uses[nuses++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      sets[nsets++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      unit = xdep_prep_ctl.crl_unit;
      latency = xdep_prep_ctl.crl_latency;
      avail = xdep_prep_ctl.crl_avail;
      break;
    }
  case OPC_CRORC: {
      uses[nuses++] = OPC_CR(OPC_DEC_BA(iword) >> 2);
      uses[nuses++] = OPC_CR(OPC_DEC_BB(iword) >> 2);
      uses[nuses++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      sets[nsets++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      unit = xdep_prep_ctl.crl_unit;
      latency = xdep_prep_ctl.crl_latency;
      avail = xdep_prep_ctl.crl_avail;
      break;
    }
  case OPC_CRXOR: {
      uses[nuses++] = OPC_CR(OPC_DEC_BA(iword) >> 2);
      uses[nuses++] = OPC_CR(OPC_DEC_BB(iword) >> 2);
      uses[nuses++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      sets[nsets++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      unit = xdep_prep_ctl.crl_unit;
      latency = xdep_prep_ctl.crl_latency;
      avail = xdep_prep_ctl.crl_avail;
      break;
    }
  case OPC_DCBF: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_DCBI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_DCBST: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_DCBT: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_DCBTST: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_DCBZ: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_DCCCI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_DCREAD: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_DCLST: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_DSIXES: {
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_DTCS_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_DIV: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_MQ;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_MQ;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIV_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_MQ;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_MQ;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVS: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_MQ;
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVSO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVS_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVSO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVD: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVDO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVD_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVDO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVW: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVWO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVW_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVWO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVDU: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVDUO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVDU_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVDUO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVWU: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVWUO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVWU_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DIVWUO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.idiv_unit;
      latency = xdep_prep_ctl.idiv_latency;
      avail = xdep_prep_ctl.idiv_avail;
      break;
    }
  case OPC_DLMZB: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets] = OPC_XER_BCNT;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_DLMZB_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_BCNT;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_DOZ: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_DOZO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_DOZ_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_DOZO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_DOZI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ECIWX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_ECOWX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_EIEIO: {
      is_memop = 1;
      is_notaddr = 1;
      is_athead = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_EQV: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_EQV_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_EXTSB: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_EXTSB_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_EXTSH: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_EXTSH_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_EXTSW: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_EXTSW_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_FABS: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FABS_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FADD: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FADD_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FADDS: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FADDS_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FCMPO: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_CRF(OPC_DEC_BF(iword));
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FCMPU: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_CRF(OPC_DEC_BF(iword));
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FCFID: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FCFID_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FCTID: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FCTID_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FCTIDZ: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FCTIDZ_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FCTIW: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FCTIW_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FCTIWZ: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FCTIWZ_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FDIV: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fdiv_latency;
      avail = xdep_prep_ctl.fdiv_avail;
      break;
    }
  case OPC_FDIV_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fdiv_latency;
      avail = xdep_prep_ctl.fdiv_avail;
      break;
    }
  case OPC_FDIVS: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fdiv_latency;
      avail = xdep_prep_ctl.fdiv_avail;
      break;
    }
  case OPC_FDIVS_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fdiv_latency;
      avail = xdep_prep_ctl.fdiv_avail;
      break;
    }
  case OPC_FMADD: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FMADD_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FMADDS: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FMADDS_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FMR: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FMR_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FMSUB: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FMSUB_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FMSUBS: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FMSUBS_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FMUL: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FMUL_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FMULS: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FMULS_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FNABS: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FNABS_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FNEG: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FNEG_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FNMADD: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FNMADD_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FNMADDS: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FNMADDS_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FNMSUB: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FNMSUB_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FNMSUBS: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FNMSUBS_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FRES: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FRES_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FRSP: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FRSP_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FRSQRTE: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FRSQRTE_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FSEL: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FSEL_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRC(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FSQRT: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FSQRT_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FSQRTS: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FSQRTS_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FSUB: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FSUB_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FSUBS: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_FSUBS_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRA(iword));
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      uses[nuses] = OPC_FPSCR_ENR;
      nuses += 1;
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_ICBI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_ICBT: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_ICCCI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_ICREAD: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_ISYNC: {
      is_sync = 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_LBZ: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 1;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.subword_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.subword_load_avail;
      break;
    }
  case OPC_LBZU: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 1;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.subword_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.subword_load_avail;
      break;
    }
  case OPC_LBZUX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_load = 1;
      dlength = 1;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.subword_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.subword_load_avail;
      break;
    }
  case OPC_LBZX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 1;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.subword_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.subword_load_avail;
      break;
    }
  case OPC_LFD: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      is_load = 1;
      dlength = 8;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.float_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.float_load_avail;
      break;
    }
  case OPC_LFDU: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_load = 1;
      dlength = 8;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.float_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.float_load_avail;
      break;
    }
  case OPC_LFDUX: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_load = 1;
      dlength = 8;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.float_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.float_load_avail;
      break;
    }
  case OPC_LFDX: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      is_load = 1;
      dlength = 8;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.float_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.float_load_avail;
      break;
    }
  case OPC_LQ: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 16;
      is_memop = 1;
      sets[nsets] = OPC_XER_TGCC;
      nsets += 1;
      n = (OPC_DEC_RT(iword)+1)%32;
      sets[nsets++] = OPC_FPR(n);
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LFQ: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      is_load = 1;
      dlength = 16;
      is_memop = 1;
      n = (OPC_DEC_RT(iword)+1)%32;
      sets[nsets++] = OPC_FPR(n);
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.float_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.float_load_avail;
      break;
    }
  case OPC_LFQU: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_load = 1;
      dlength = 16;
      is_memop = 1;
      n = (OPC_DEC_RT(iword)+1)%32;
      sets[nsets++] = OPC_FPR(n);
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.float_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.float_load_avail;
      break;
    }
  case OPC_LFQUX: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_load = 1;
      dlength = 16;
      is_memop = 1;
      n = (OPC_DEC_RT(iword)+1)%32;
      sets[nsets++] = OPC_FPR(n);
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.float_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.float_load_avail;
      break;
    }
  case OPC_LFQX: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      is_load = 1;
      dlength = 16;
      is_memop = 1;
      n = (OPC_DEC_RT(iword)+1)%32;
      sets[nsets++] = OPC_FPR(n);
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.float_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.float_load_avail;
      break;
    }
  case OPC_LFS: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      is_load = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.float_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.float_load_avail;
      break;
    }
  case OPC_LFSU: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_load = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.float_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.float_load_avail;
      break;
    }
  case OPC_LFSUX: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_load = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.float_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.float_load_avail;
      break;
    }
  case OPC_LFSX: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      is_load = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.float_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.float_load_avail;
      break;
    }
  case OPC_LHA: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 2;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.arithmetic_subword_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.arithmetic_subword_load_avail;
      break;
    }
  case OPC_LHAU: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 2;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.arithmetic_subword_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.arithmetic_subword_load_avail;
      break;
    }
  case OPC_LHAUX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_load = 1;
      dlength = 2;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.arithmetic_subword_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.arithmetic_subword_load_avail;
      break;
    }
  case OPC_LHAX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 2;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.arithmetic_subword_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.arithmetic_subword_load_avail;
      break;
    }
  case OPC_LHBRX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 2;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LHZ: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 2;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.subword_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.subword_load_avail;
      break;
    }
  case OPC_LHZU: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 2;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.subword_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.subword_load_avail;
      break;
    }
  case OPC_LHZUX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_load = 1;
      dlength = 2;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.subword_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.subword_load_avail;
      break;
    }
  case OPC_LHZX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 2;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.subword_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.subword_load_avail;
      break;
    }
  case OPC_LMD: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 0;
      is_memop = 1;
      for( i = OPC_DEC_RT(iword)+1; i < 32; i++ ) {
        sets[nsets++] = OPC_GPR(i);
      }
      is_multi = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LMW: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 0;
      is_memop = 1;
      for( i = OPC_DEC_RT(iword)+1; i < 32; i++ ) {
        sets[nsets++] = OPC_GPR(i);
      }
      is_multi = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LSCBX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_load = 1;
      dlength = 0;
      is_memop = 1;
      uses[nuses] = OPC_XER_BCMP;
      nuses += 1;
      uses[nuses] = OPC_XER_BCNT;
      nuses += 1;
      for( i = 0; i < 32; i++ ) {
        uses[nuses++] = OPC_GPR(i);
      }
      sets[nsets] = OPC_XER_BCNT;
      nsets += 1;
      n = OPC_DEC_RB(iword);
      m = OPC_DEC_RA(iword);
      m = (m == 0) ? n : m;
      for( i = 0; i < 32; i++ ) {
        sets[nsets] = OPC_GPR(i);
        nsets += ((i != n) && (i != m));
      }
      is_multi = 1;
      is_string = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LSCBX_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_load = 1;
      dlength = 0;
      is_memop = 1;
      uses[nuses] = OPC_XER_BCMP;
      nuses += 1;
      uses[nuses] = OPC_XER_BCNT;
      nuses += 1;
      for( i = 0; i < 32; i++ ) {
        uses[nuses++] = OPC_GPR(i);
      }
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_BCNT;
      nsets += 1;
      n = OPC_DEC_RB(iword);
      m = OPC_DEC_RA(iword);
      m = (m == 0) ? n : m;
      for( i = 0; i < 32; i++ ) {
        sets[nsets] = OPC_GPR(i);
        nsets += ((i != n) && (i != m));
      }
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      is_multi = 1;
      is_string = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LSDI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 0;
      is_memop = 1;
      n = OPC_DEC_RB(iword)?(OPC_DEC_RB(iword)+3)>>2:8;
      for( i = OPC_DEC_RT(iword)+1; i < OPC_DEC_RT(iword) + n; i++ ) {
        sets[nsets++] = OPC_GPR((i%32));
      }
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LSWI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 0;
      is_memop = 1;
      n = OPC_DEC_RB(iword)?(OPC_DEC_RB(iword)+3)>>2:8;
      for( i = OPC_DEC_RT(iword)+1; i < OPC_DEC_RT(iword) + n; i++ ) {
        sets[nsets++] = OPC_GPR((i%32));
      }
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LSDX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_load = 1;
      dlength = 0;
      is_memop = 1;
      uses[nuses] = OPC_XER_BCNT;
      nuses += 1;
      for( i = 0; i < 32; i++ ) {
        uses[nuses++] = OPC_GPR(i);
      }
      n = OPC_DEC_RB(iword);
      m = OPC_DEC_RA(iword);
      m = (m == 0) ? n : m;
      for( i = 0; i < 32; i++ ) {
        sets[nsets] = OPC_GPR(i);
        nsets += ((i != n) && (i != m));
      }
      is_multi = 1;
      is_string = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LSWX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_load = 1;
      dlength = 0;
      is_memop = 1;
      uses[nuses] = OPC_XER_BCNT;
      nuses += 1;
      for( i = 0; i < 32; i++ ) {
        uses[nuses++] = OPC_GPR(i);
      }
      n = OPC_DEC_RB(iword);
      m = OPC_DEC_RA(iword);
      m = (m == 0) ? n : m;
      for( i = 0; i < 32; i++ ) {
        sets[nsets] = OPC_GPR(i);
        nsets += ((i != n) && (i != m));
      }
      is_multi = 1;
      is_string = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LWARX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LWBRX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LWZ: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LWZU: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LWZUX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_load = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LWZX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LWA: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.arithmetic_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.arithmetic_load_avail;
      break;
    }
  case OPC_LWAUX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_load = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.arithmetic_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.arithmetic_load_avail;
      break;
    }
  case OPC_LWAX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency + xdep_prep_ctl.arithmetic_load_latency;
      avail = xdep_prep_ctl.load_avail + xdep_prep_ctl.arithmetic_load_avail;
      break;
    }
  case OPC_LD: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 8;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LDARX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 8;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LDU: {
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_load = 1;
      dlength = 8;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LDUX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_load = 1;
      dlength = 8;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_LDX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_load = 1;
      dlength = 8;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_MASKG: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MASKG_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MASKIR: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MASKIR_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MCRF: {
      uses[nuses++] = OPC_CRF(OPC_DEC_BFA(iword));
      sets[nsets++] = OPC_CRF(OPC_DEC_BF(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MCRFS: {
      uses[nuses++] = OPC_CRF(OPC_DEC_BFA(iword));
      sets[nsets++] = OPC_CRF(OPC_DEC_BF(iword));
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_MCRXR: {
      sets[nsets++] = OPC_CRF(OPC_DEC_BF(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      uses[nuses] = OPC_XER_OV;
      nuses += 1;
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MCRXRT: {
      sets[nsets++] = OPC_CRF(OPC_DEC_BF(iword));
      uses[nuses] = OPC_XER_TGCC;
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MFCR: {
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_CR(0);
      nuses += 1;
      uses[nuses] = OPC_CR(1);
      nuses += 1;
      uses[nuses] = OPC_CR(2);
      nuses += 1;
      uses[nuses] = OPC_CR(3);
      nuses += 1;
      uses[nuses] = OPC_CR(4);
      nuses += 1;
      uses[nuses] = OPC_CR(5);
      nuses += 1;
      uses[nuses] = OPC_CR(6);
      nuses += 1;
      uses[nuses] = OPC_CR(7);
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MFDCR: {
      spr = OPC_DEC_SPR(iword);
      if( spr == 1 ) {
        uses[nuses] = OPC_XER_SO;
        nuses += 1;
        uses[nuses] = OPC_XER_CA;
        nuses += 1;
        uses[nuses] = OPC_XER_FXCC;
        nuses += 1;
        uses[nuses] = OPC_XER_BCNT;
        nuses += 1;
      }
     else {
        spr = opc_tblsprenc[spr];
        uses[nuses] = spr;
        nuses += spr != 0;
      }
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MFFS: {
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_MFFS_: {
      sets[nsets++] = OPC_FPR(OPC_DEC_FRT(iword));
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_MFMSR: {
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_MSR;
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MFSPR: {
      spr = OPC_DEC_SPR(iword);
      if( spr == 1 ) {
        uses[nuses] = OPC_XER_SO;
        nuses += 1;
        uses[nuses] = OPC_XER_CA;
        nuses += 1;
        uses[nuses] = OPC_XER_FXCC;
        nuses += 1;
        uses[nuses] = OPC_XER_BCNT;
        nuses += 1;
      }
     else {
        spr = opc_tblsprenc[spr];
        uses[nuses] = spr;
        nuses += spr != 0;
      }
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MFSR: {
      is_memop = 1;
      is_notaddr = 1;
      is_athead = 1;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_SR0_15;
      nuses += 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_MFSRI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_SR0_15;
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MFSRIN: {
      is_memop = 1;
      is_notaddr = 1;
      is_athead = 1;
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_SR0_15;
      nuses += 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_MFTB: {
      spr = OPC_DEC_SPR(iword);
      if( spr == 1 ) {
        uses[nuses] = OPC_XER_SO;
        nuses += 1;
        uses[nuses] = OPC_XER_CA;
        nuses += 1;
        uses[nuses] = OPC_XER_FXCC;
        nuses += 1;
        uses[nuses] = OPC_XER_BCNT;
        nuses += 1;
      }
     else {
        spr = opc_tblsprenc[spr];
        uses[nuses] = spr;
        nuses += spr != 0;
      }
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MTCRF: {
      spr = OPC_DEC_SPR(iword);
      if( spr == 1 ) {
        sets[nsets] = OPC_XER_SO;
        nsets += 1;
        sets[nsets] = OPC_XER_CA;
        nsets += 1;
        sets[nsets] = OPC_XER_FXCC;
        nsets += 1;
        sets[nsets] = OPC_XER_BCNT;
        nsets += 1;
      }
     else {
        spr = opc_tblsprenc[spr];
        sets[nsets] = spr;
        nsets += spr != 0;
      }
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      for( i = OPC_DEC_FXM(iword), j = 7; i; i  = i >> 1, j--  ) {
        sets[nsets] = OPC_CR(j);
        nsets += i&1;
      }
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MTDCR: {
      spr = OPC_DEC_SPR(iword);
      if( spr == 1 ) {
        sets[nsets] = OPC_XER_SO;
        nsets += 1;
        sets[nsets] = OPC_XER_CA;
        nsets += 1;
        sets[nsets] = OPC_XER_FXCC;
        nsets += 1;
        sets[nsets] = OPC_XER_BCNT;
        nsets += 1;
      }
     else {
        spr = opc_tblsprenc[spr];
        sets[nsets] = spr;
        nsets += spr != 0;
      }
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MTFSB0: {
      sets[nsets++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_MTFSB0_: {
      sets[nsets++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_MTFSB1: {
      sets[nsets++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_MTFSB1_: {
      sets[nsets++] = OPC_CR(OPC_DEC_BT(iword) >> 2);
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_MTFSF: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_MTFSF_: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRB(iword));
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_MTFSFI: {
      sets[nsets++] = OPC_CRF(OPC_DEC_BF(iword));
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_MTFSFI_: {
      sets[nsets++] = OPC_CRF(OPC_DEC_BF(iword));
      sets[nsets] = OPC_CR(1);
      nsets += 1;
      unit = xdep_prep_ctl.fpu_unit;
      latency = xdep_prep_ctl.fpu_latency;
      avail = xdep_prep_ctl.fpu_avail;
      break;
    }
  case OPC_MTMSRD: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets] = OPC_MSR;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MTMSR: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets] = OPC_MSR;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MTSPR: {
      spr = OPC_DEC_SPR(iword);
      if( spr == 1 ) {
        sets[nsets] = OPC_XER_SO;
        nsets += 1;
        sets[nsets] = OPC_XER_CA;
        nsets += 1;
        sets[nsets] = OPC_XER_FXCC;
        nsets += 1;
        sets[nsets] = OPC_XER_BCNT;
        nsets += 1;
      }
     else {
        spr = opc_tblsprenc[spr];
        sets[nsets] = spr;
        nsets += spr != 0;
      }
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MTSR: {
      is_memop = 1;
      is_notaddr = 1;
      is_athead = 1;
      is_mtsr = 1;
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets] = OPC_SR0_15;
      nsets += 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_MTSRI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets] = OPC_SR0_15;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_MTSRIN: {
      is_memop = 1;
      is_notaddr = 1;
      is_athead = 1;
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets] = OPC_SR0_15;
      nsets += 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_MUL: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_MQ;
      nsets += 1;
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MUL_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULHD: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULHD_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULHW: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULHW_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULHDU: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULHDU_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULHWU: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULHWU_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULLD: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULLDO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULLD_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULLDO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULLW: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULLWO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULLW_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULLWO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_MULLI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.imul_unit;
      latency = xdep_prep_ctl.imul_latency;
      avail = xdep_prep_ctl.imul_avail;
      break;
    }
  case OPC_NABS: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_NABSO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_NABS_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_NABSO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_NAND: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_NAND_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_NEG: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_NEGO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_NEG_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_NEGO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_NOR: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_NOR_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_OR: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_OR_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ORC: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ORC_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ORI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
/* commented out by zhigangh 'cause RA is output, not input       
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
*/      
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_ORIS: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
/* commented out by zhigangh 'cause RA is output, not input       
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
*/
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RAC: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RAC_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RFSC: {
      uses[nuses] = OPC_SRR0;
      nuses += 1;
      uses[nuses] = OPC_SRR1;
      nuses += 1;
      sets[nsets] = OPC_MSR;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RFID: {
      uses[nuses] = OPC_SRR0;
      nuses += 1;
      uses[nuses] = OPC_SRR1;
      nuses += 1;
      sets[nsets] = OPC_MSR;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RFCI: {
      uses[nuses] = OPC_SRR0;
      nuses += 1;
      uses[nuses] = OPC_SRR1;
      nuses += 1;
      sets[nsets] = OPC_MSR;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RFI: {
      uses[nuses] = OPC_SRR0;
      nuses += 1;
      uses[nuses] = OPC_SRR1;
      nuses += 1;
      sets[nsets] = OPC_MSR;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLMI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLMI_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLDCL: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLDCL_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLDCR: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLDCR_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLDIC: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLDIC_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLDICL: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLDICL_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLDICR: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLDICR_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLDIMI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLDIMI_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLWIMI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLWIMI_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLWINM: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLWINM_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLWNM: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RLWNM_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RRIB: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_RRIB_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SC: {
      uses[nuses] = OPC_MSR;
      nuses += 1;
      sets[nsets] = OPC_SRR0;
      nsets += 1;
      sets[nsets] = OPC_SRR1;
      nsets += 1;
      sets[nsets] = OPC_MSR;
      nsets += 1;
      is_branch = 1;
      unit = xdep_prep_ctl.branch_unit;
      latency = xdep_prep_ctl.branch_latency;
      avail = xdep_prep_ctl.branch_avail;
      break;
    }
  case OPC_SVC: {
      uses[nuses] = OPC_MSR;
      nuses += 1;
      sets[nsets] = OPC_SRR0;
      nsets += 1;
      sets[nsets] = OPC_SRR1;
      nsets += 1;
      sets[nsets] = OPC_MSR;
      nsets += 1;
      is_branch = 1;
      unit = xdep_prep_ctl.branch_unit;
      latency = xdep_prep_ctl.branch_latency;
      avail = xdep_prep_ctl.branch_avail;
      break;
    }
  case OPC_SVCL: {
      uses[nuses] = OPC_MSR;
      nuses += 1;
      sets[nsets] = OPC_SRR0;
      nsets += 1;
      sets[nsets] = OPC_SRR1;
      nsets += 1;
      sets[nsets] = OPC_MSR;
      nsets += 1;
      sets[nsets] = OPC_LR;
      nsets += 1;
      is_branch = 1;
      unit = xdep_prep_ctl.branch_unit;
      latency = xdep_prep_ctl.branch_latency;
      avail = xdep_prep_ctl.branch_avail;
      break;
    }
  case OPC_SELIIn: {
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SELIIn_: {
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SELIRn: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SELIRn_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SELRIn: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SELRIn_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SELRRn: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SELRRn_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SELII: {
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
  /*** NEED TO FIGURE THIS ONE OUT (Time Base?) ***/ 
  /*    tbr = OPC_DEC_TBR(iword);  */
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SELII_: {
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
  /*** NEED TO FIGURE THIS ONE OUT (Time Base?) ***/ 
  /*    tbr = OPC_DEC_TBR(iword);  */
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SELIR: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
  /*** NEED TO FIGURE THIS ONE OUT (Time Base?) ***/ 
  /*    tbr = OPC_DEC_TBR(iword);  */
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SELIR_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
  /*** NEED TO FIGURE THIS ONE OUT (Time Base?) ***/ 
  /*    tbr = OPC_DEC_TBR(iword);  */
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SELRI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
  /*** NEED TO FIGURE THIS ONE OUT (Time Base?) ***/ 
  /*    tbr = OPC_DEC_TBR(iword);  */
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SELRI_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
  /*** NEED TO FIGURE THIS ONE OUT (Time Base?) ***/ 
  /*    tbr = OPC_DEC_TBR(iword);  */
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SELRR: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
  /*** NEED TO FIGURE THIS ONE OUT (Time Base?) ***/ 
  /*    tbr = OPC_DEC_TBR(iword);  */
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SELRR_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
  /*** NEED TO FIGURE THIS ONE OUT (Time Base?) ***/ 
  /*    tbr = OPC_DEC_TBR(iword);  */
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SETTAG: {
      sets[nsets] = OPC_XER_TGCC;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SLBIA: {
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SLBIE: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_SLBIEX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_SLE: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SLE_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SLEQ: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_MQ;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SLEQ_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_MQ;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SLIQ: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets] = OPC_MQ;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SLIQ_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SLLIQ: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_MQ;
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SLLIQ_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_MQ;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SLLQ: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_MQ;
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SLLQ_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_MQ;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SLQ: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets] = OPC_MQ;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SLQ_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SLD: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SLD_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SLW: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SLW_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRAIQ: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRAIQ_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRAQ: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRAQ_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRAD: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRAD_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRAW: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRAW_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRADI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRADI_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRAWI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRAWI_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRE: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets] = OPC_MQ;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRE_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SREA: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SREA_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SREQ: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_MQ;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SREQ_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_MQ;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRIQ: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets] = OPC_MQ;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRIQ_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRLIQ: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_MQ;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRLIQ_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_MQ;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRLQ: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_MQ;
      nuses += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRLQ_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_MQ;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRQ: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets] = OPC_MQ;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRQ_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_MQ;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRD: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRD_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRW: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SRW_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_STB: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 1;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STBU: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 1;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STBUX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 1;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STBX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 1;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STFD: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 8;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STFDU: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 8;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STFDUX: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 8;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STFDX: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      is_store = 1;
      dlength = 8;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STFIWX: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      is_store = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STFQ: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 16;
      is_memop = 1;
        n = (OPC_DEC_RS(iword)+1)%32;
        uses[nuses++] = OPC_FPR(n);
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STFQU: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 16;
      is_memop = 1;
        n = (OPC_DEC_RS(iword)+1)%32;
        uses[nuses++] = OPC_FPR(n);
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STFQUX: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 16;
      is_memop = 1;
        n = (OPC_DEC_RS(iword)+1)%32;
        uses[nuses++] = OPC_FPR(n);
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STFQX: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      is_store = 1;
      dlength = 16;
      is_memop = 1;
        n = (OPC_DEC_RS(iword)+1)%32;
        uses[nuses++] = OPC_FPR(n);
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STFS: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STFSU: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STFSUX: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STFSX: {
      uses[nuses++] = OPC_FPR(OPC_DEC_FRS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      is_store = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STH: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 2;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STHBRX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 2;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STHU: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 2;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STHUX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 2;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STHX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 2;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STMD: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 0;
      is_memop = 1;
      for( i = OPC_DEC_RS(iword)+1; i < 32; i++ ) {
        uses[nuses++] = OPC_GPR(i);
      }
      is_multi = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STMW: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 0;
      is_memop = 1;
      for( i = OPC_DEC_RS(iword)+1; i < 32; i++ ) {
        uses[nuses++] = OPC_GPR(i);
      }
      is_multi = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STSDI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      is_store = 1;
      dlength = 0;
      is_memop = 1;
      n = OPC_DEC_NB(iword)?(OPC_DEC_NB(iword)+3)>>2:8;
      for( i = OPC_DEC_RS(iword)+1; i < OPC_DEC_RS(iword)+n; i++ ) {
        unsigned reg = i%32;
        uses[nuses++] = OPC_GPR(reg);
      }
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STSWI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      is_store = 1;
      dlength = 0;
      is_memop = 1;
      n = OPC_DEC_NB(iword)?(OPC_DEC_NB(iword)+3)>>2:8;
      for( i = OPC_DEC_RS(iword)+1; i < OPC_DEC_RS(iword)+n; i++ ) {
        unsigned reg = i%32;
        uses[nuses++] = OPC_GPR(reg);
      }
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STSDX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 0;
      is_memop = 1;
      uses[nuses] = OPC_XER_BCNT;
      nuses += 1;
      for( i = 0; i < 32; i++ ) {
        uses[nuses++] = OPC_GPR(i);
      }
      is_multi = 1;
      is_string = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STSWX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 0;
      is_memop = 1;
      uses[nuses] = OPC_XER_BCNT;
      nuses += 1;
      for( i = 0; i < 32; i++ ) {
        uses[nuses++] = OPC_GPR(i);
      }
      is_multi = 1;
      is_string = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STW: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STWBRX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STWCX_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 4;
      is_memop = 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STWU: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STWUX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STWX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 4;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STD: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 8;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STDCX_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 8;
      is_memop = 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STDU: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 8;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STDUX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets] = OPC_GPR(OPC_DEC_RA(iword));
      nsets += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 8;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STDX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 8;
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_STQ: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      is_store = 1;
      dlength = 16;
      is_memop = 1;
      sets[nsets] = OPC_XER_TGCC;
      nsets += 1;
      n = (OPC_DEC_RT(iword)+1)%32;
      sets[nsets++] = OPC_FPR(n);
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_SUBF: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBF_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFC: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFCO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFC_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFCO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFE: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFEO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFE_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFEO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFIC: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFME: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFMEO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFME_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFMEO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFZE: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFZEO: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFZE_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SUBFZEO_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      uses[nuses] = OPC_XER_CA;
      nuses += 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_XER_CA;
      nsets += 1;
      sets[nsets] = OPC_XER_OV;
      nsets += 1;
      sets[nsets] = OPC_XER_SO;
      nsets += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_SYNC: {
      is_memop = 1;
      is_notaddr = 1;
      is_athead = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_TLBIA: {
      is_memop = 1;
      is_notaddr = 1;
      is_athead = 1;
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_TLBIE: {
      is_memop = 1;
      is_notaddr = 1;
      is_athead = 1;
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      is_memop = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_TLBIX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_TLBIR: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_TLBRE: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_TLBSX: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_TLBSX_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      uses[nuses] = OPC_GPR(OPC_DEC_RA(iword));
      nuses += OPC_DEC_RA(iword) != 0;
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_memop = 1;
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_TLBSYNC: {
      is_memop = 1;
      is_notaddr = 1;
      is_athead = 1;
      unit = xdep_prep_ctl.load_unit;
      latency = xdep_prep_ctl.load_latency;
      avail = xdep_prep_ctl.load_avail;
      break;
    }
  case OPC_TLBWE: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RT(iword));
      is_memop = 1;
      unit = xdep_prep_ctl.store_unit;
      latency = xdep_prep_ctl.store_latency;
      avail = xdep_prep_ctl.store_avail;
      break;
    }
  case OPC_TD: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_TW: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_TDI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_TWI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_TXER: {
  /*** NEED TO FIGURE THIS ONE OUT (Time Base?) ***/ 
  /*    tbr = OPC_DEC_TBR(iword);  */
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_TXER_: {
  /*** NEED TO FIGURE THIS ONE OUT (Time Base?) ***/ 
  /*    tbr = OPC_DEC_TBR(iword);  */
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_VSYNC: {
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_WRTEE: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      sets[nsets] = OPC_MSR;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_WRTEEI: {
      sets[nsets] = OPC_MSR;
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_XOR: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_XOR_: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      uses[nuses] = OPC_XER_SO;
      nuses += 1;
      sets[nsets] = OPC_CR(0);
      nsets += 1;
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_XORI: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }
  case OPC_XORIS: {
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RA(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
      break;
    }

  default: {
      // fprintf(stderr, "FAILED: illegal instruction %08x @ %x\n", iword, iaddr);
      // assert(0); exit( EXIT_FAILURE);
      // break;

      // fprintf(stderr, "warning: unrecognized instruction %08x @ %08x on core %d\n", iword, iaddr, processor_id); 
#if( IN_TRANS ) // turandot
      fprintf(stderr, "warning: unrecognized instruction %08x @ %08x on core %d thread %d at cycle %lld\n", iword, iaddr, processor_id, thread_id, cycle); // jdonald avoiding thread-local storage the old fashioned way: passing arguments...
#endif
      // interpret as an OR instruction
      uses[nuses++] = OPC_GPR(OPC_DEC_RS(iword));
      uses[nuses++] = OPC_GPR(OPC_DEC_RB(iword));
      sets[nsets++] = OPC_GPR(OPC_DEC_RA(iword));
      unit = xdep_prep_ctl.fix_unit;
      latency = xdep_prep_ctl.fix_latency;
      avail = xdep_prep_ctl.fix_avail;
	  break;
    }
  }

  j = 0;
  for( i = 0; i < nuses; i++ ) {
    unsigned use = uses[i];
    if( use != OPC_XER_SO ) {
      uses[j] = use;
      j++;
    }
  }
  nuses = j;

  seen_SO = 0;
  j = 0;
  for( i = 0; i < nsets; i++ ) {
    unsigned set = sets[i];
    if( seen_SO == 0 ) {
      if( set == OPC_XER_OV || set == OPC_XER_SO ) {
	sets[i] = OPC_XER_SO;
	seen_SO = 1;
      }
    }
    else if( seen_SO == 1 ) {
      if (set == OPC_XER_OV || set == OPC_XER_SO ) {
	seen_SO = 2;
	j = 1;
      }
    }
    else {
      if (set == OPC_XER_OV || set == OPC_XER_SO ) {
	j++;
      }
      else {
	sets[i - j] = sets[i];
      }
    }
  }
  nsets -= j;

  if( is_branch ) {
    assert( !(is_brctr  && is_brlink) );
    set_xdep_prep_info_is_branch(entry,  1);
    set_xdep_prep_info_is_brlk(entry, is_brlk);
    set_xdep_prep_info_is_brcond(entry, is_brcond);
    set_xdep_prep_info_is_brctr(entry, is_brctr);
    set_xdep_prep_info_is_brlink(entry, is_brlink);
    set_xdep_prep_info_is_brcrg(entry, is_brcrg);

    assert( xdep_prep_info_is_branch(entry) ==  1);
    assert( xdep_prep_info_is_brlk(entry) == is_brlk);
    assert( xdep_prep_info_is_brcond(entry) == is_brcond);
    assert( xdep_prep_info_is_brctr(entry) == is_brctr);
    assert( xdep_prep_info_is_brlink(entry) == is_brlink);
    assert( xdep_prep_info_is_brcrg(entry) == is_brcrg);
  }
  else if( is_memop ) {
    set_xdep_prep_info_is_memop(entry, 1);
    set_xdep_prep_info_is_store(entry, is_store);
    set_xdep_prep_info_is_load(entry, is_load);
    set_xdep_prep_info_is_multi(entry, is_multi);
    set_xdep_prep_info_is_string(entry, is_string);
    is_notaddr = is_notaddr || (!is_load && !is_store);
    set_xdep_prep_info_is_notaddr(entry, is_notaddr);
    set_xdep_prep_info_dlength(entry, dlength);

    assert( xdep_prep_info_is_memop(entry) ==  1);
    assert( xdep_prep_info_is_store(entry) ==  is_store);
    assert( xdep_prep_info_is_load(entry) ==  is_load);
    assert( xdep_prep_info_is_notaddr(entry) ==  is_notaddr);
    assert( xdep_prep_info_is_multi(entry) ==  is_multi);
    assert( xdep_prep_info_is_string(entry) ==  is_string);
    assert( xdep_prep_info_dlength(entry) ==  dlength);

  }

  set_xdep_prep_info_is_sync(entry, is_sync);
  set_xdep_prep_info_is_athead(entry, is_athead);
  assert( xdep_prep_info_is_sync(entry) ==  is_sync);
  assert( xdep_prep_info_is_athead(entry) ==  is_athead);

  if( is_mtsr ) {
    sr = OPC_DEC_RA( iword );

    if( sr ) {
      assert( sr < 16 );
      set_xdep_prep_info_is_mtsr(entry, 1);
      set_xdep_prep_info_sr(entry, sr);
    }
    else {
      set_xdep_prep_info_is_mtsr(entry, 0);
    }
  }
  else {
    set_xdep_prep_info_is_mtsr(entry, 0);
  }

  is_over = nuses + nsets > 4;

  set_xdep_prep_info_unit(entry, unit);
  latency = min( 63, latency );
  avail = min( 63, avail );
  set_xdep_prep_info_latency(entry, latency);
  set_xdep_prep_info_avail(entry, avail);
  set_xdep_prep_info_ins(entry, nuses);
  set_xdep_prep_info_outs(entry, nsets);
  set_xdep_prep_info_is_over(entry, is_over);

  assert( xdep_prep_info_latency(entry) == latency);
  assert( xdep_prep_info_avail(entry) == avail);
  assert( xdep_prep_info_nexpands(entry) == 0);
  assert( xdep_prep_info_ins(entry) == nuses);
  assert( xdep_prep_info_outs(entry) == nsets);
  assert( xdep_prep_info_is_over(entry) == is_over);

/****************************************************************/
/* Compute the number of instructions that would be fetched	*/
/* if this instruction were the target of the branch		*/
/****************************************************************/
  assert( (iaddr & 0x3) == 0 );
  cache_fetch = xdep_prep_ctl.cache_align - ((iaddr>>2)&xdep_prep_ctl.cache_mask);
  align_fetch = xdep_prep_ctl.fetch_max - ((iaddr>>2)&xdep_prep_ctl.fetch_align_mask);
  fetch = min(cache_fetch, align_fetch);
  assert( fetch >= 1 );
  assert( fetch <= xdep_prep_ctl.fetch_max);

  set_xdep_prep_info_fetch(entry, fetch);
  assert( xdep_prep_info_fetch(entry) == fetch);

/****************************************************************/
/* Check if there is an overflow; i.e. nuses+nsets > 4		*/
/****************************************************************/
  if( is_over ) {
    get_overflows( overflow, entry, (nuses+nsets));
  }

/****************************************************************/
/* Setup the input registers					*/
/****************************************************************/

  in_regs = is_over? overflow : xdep_prep_info_regs( entry );
  for( j = 0; j < nuses; j++ ) {
    unsigned use = uses[j];
    cvt_dep_reg( reg, use, lg2max );
    assert( reg < (1<<(2+lg2max)) );
    in_regs[j] = reg;
  }

/****************************************************************/
/* Setup the output registers					*/
/****************************************************************/

  out_regs = in_regs + nuses;
  if( nsets ) {
    for( j = 0; j < nsets-1; j++ ) {
      for( k = j+1; k < nsets; k++ ) {
	if( sets[j] == sets[k] ) {
	  #if( IN_TRANS )
	    fprintf(the_stderr, "FAILED: multiple output sets %08x @ %08x\n", iword, iaddr);
	    exit( EXIT_FAILURE );
	  #else
	    fprintf(stderr, "WARNING: multiple output sets %08x @ %08x\n", iword, iaddr);
	  #endif
	}
      }
    }
  }

  for( j = 0; j < nsets; j++ ) {
    // unsigned set = sets[j];
    // unused variable? -jdonald
    
    cvt_dep_reg( reg, sets[j], lg2max );
    if( reg >= (1<<(2+lg2max)) ) {
      fprintf(stderr, "reg %d, lg2max %d, val %d\n", reg, lg2max, 1<<(2+lg2max));
    }
    assert( reg < (1<<(2+lg2max)) );
    out_regs[j] = reg;
  }

/****************************************************************/
/* Compute the number of rename registers of each class 	*/
/* required by the instruction.					*/
/****************************************************************/

  for( j = 0; j < 4; j++ ) {
    req[j] = 0;
  }

  for( j = 0; j < nsets; j++ ) {
    unsigned clss = xdep_prep_reg_class( out_regs[j], lg2max );
    assert( clss < 4 );
    req[clss]++;
  }

/****************************************************************/
/* Compute the byte vectors for the rename registers requrired	*/
/* The whole vector is a concatencation of the registers reqd.	*/
/* The compl vector is a concatencation of the 7-bit 2's	*/
/* complement (roughly: -0 is represented as 128) of the reqd.	*/
/****************************************************************/

  whole = (req[3]<<24)|(req[2]<<16)|(req[1]<<8)|req[0];
  compl = ((~whole)&0x7f7f7f7f)+0x01010101;

  xdep_prep_info_res( entry ) = whole;
  xdep_prep_info_res_compl( entry ) = compl;

  assert( xdep_prep_info_res(entry) || xdep_prep_info_res_compl(entry));

/****************************************************************/
/* Other GP properties						*/
/****************************************************************/
  if( xdep_prep_ctl.flag_use_gp ) {
    switch( idx ) {
    case OPC_MTSPR: {
	avail ++;
	reset_xdep_prep_info_avail(entry, avail);
	break;
      }
    case OPC_DIVD: {
	reset_xdep_prep_info_latency(entry, 63);
	assert( xdep_prep_info_latency(entry) == 63 );
	latency = 63;
	break;
      }
    case OPC_FSQRT:
    case OPC_FSQRT_:
    case OPC_FSQRTS:
    case OPC_FSQRTS_: {
	reset_xdep_prep_info_latency(entry, 35);
	assert( xdep_prep_info_latency(entry) == 35 );
	reset_xdep_prep_info_is_notpipe(entry, 1);
	latency = 35;
	break;
      }
    case OPC_SC: {
	reset_xdep_prep_info_is_slot0(entry, 1);
	reset_xdep_prep_info_is_slotN(entry, 1);
	reset_xdep_prep_info_is_sync(entry, 1);
	break;
      }
    case OPC_RFI: {
	reset_xdep_prep_info_is_slot0(entry, 1);
	reset_xdep_prep_info_is_slotN(entry, 1);
	reset_xdep_prep_info_is_sync(entry, 1);
	break;
      }
    case OPC_MULLI: {
	reset_xdep_prep_info_latency(entry, latency - 1 );
	reset_xdep_prep_info_avail(entry, avail - 1 );
	reset_xdep_prep_info_is_notpipe(entry, 1);
	break;
      }

    case OPC_MCRF: {
       reset_xdep_prep_info_unit(entry, xdep_prep_ctl.crl_unit);
       break;
      }

    case OPC_FDIV:
    case OPC_FDIVS:
    case OPC_FDIV_:
    case OPC_FDIVS_: {
	reset_xdep_prep_info_is_notpipe(entry, 1);
	break;
      }

    case OPC_FCMPU:
    case OPC_FCMPO: {
	reset_xdep_prep_info_avail(entry, avail + 1 );
	break;
      }

    case OPC_MULLWO:
    case OPC_MULLWO_: {
	reset_xdep_prep_info_latency(entry, latency + 1 );
	reset_xdep_prep_info_avail(entry, avail + 1 );
	reset_xdep_prep_info_is_notpipe(entry, 1);
	break;
      }

    case OPC_LFS:
    case OPC_LFSU:
    case OPC_LFSUX:
    case OPC_LFSX:
    case OPC_LFD:
    case OPC_LFDU:
    case OPC_LFDUX:
    case OPC_LFDX: {
	reset_xdep_prep_info_avail(entry, avail + 1 );
	break;
      }

    case OPC_DIVW:
    case OPC_DIVWU:
    case OPC_DIVW_:
    case OPC_DIVWU_:
    case OPC_DIVWO:
    case OPC_DIVWUO:
    case OPC_DIVWO_:
    case OPC_DIVWUO_:
    case OPC_MULLW:
    case OPC_MULHW:
    case OPC_MULHWU:
    case OPC_MULLW_:
    case OPC_MULHW_:
    case OPC_MULHWU_: {
	reset_xdep_prep_info_is_notpipe(entry, 1);
	break;
      }
    case OPC_ORI: {
	assert( 0x60000000 == OPC_ASM_ORI(0, 0, 0) );
	if( iword == 0x60000000 ) {
	  reset_xdep_prep_info_is_nop(entry, 1);
	  reset_xdep_prep_info_ins(entry, 0);
	  reset_xdep_prep_info_outs(entry, 0);
	  xdep_prep_info_res( entry ) = RES_NONE;
	  xdep_prep_info_res_compl( entry ) = RESCPL_NONE;
	}
	break;
      }
    case OPC_MTFSB0:
    case OPC_MTFSB1:
    case OPC_MTFSF:
    case OPC_MTFSFI: {
	reset_xdep_prep_info_is_slot0(entry, 1);
	break;
      }
    }
  }


/****************************************************************/
/* Preliminary mechanism for splitting instructions		*/
/****************************************************************/

  n = 0;
  if( xdep_prep_ctl.flag_all_split ) {
    switch( idx ) {

    case OPC_ADDC:
    case OPC_SUBFC: {
	if( xdep_prep_ctl.flag_use_gp ) {
	  n = 2;
	  get_expands(expand, entry, n);

	  xentry = &expand[0];
	  exp_word = OPC_ASM_ORI(  0, 0, 0 );
	  process_iword( iaddr, exp_word, xentry
#if( IN_TRANS ) // turandot
		, cycle, processor_id, thread_id
#endif
		);
	  reset_xdep_prep_info_is_nop(xentry, 1);
	  reset_xdep_prep_info_ins(xentry, 0);
	  reset_xdep_prep_info_outs(xentry, 0);
	  xdep_prep_info_res( xentry ) = RES_NONE;
	  xdep_prep_info_res_compl( xentry ) = RESCPL_NONE;

	  xentry = &expand[1];
	  *xentry = *entry;

	}
	break;
      }

    case OPC_ADDC_:
    case OPC_SUBFC_: {
	if( xdep_prep_ctl.flag_use_gp ) {
	  n = 3;
	  get_expands(expand, entry, n);

	  xentry = &expand[0];
	  *xentry = *entry;
	  reset_xdep_prep_info_outs(xentry, 0);
	  xdep_prep_info_res( xentry ) = RES_NONE;
	  xdep_prep_info_res_compl( xentry ) = RESCPL_NONE;
	  set_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[1];
	  *xentry = *entry;
	  reset_xdep_prep_info_outs(xentry, nsets-1);
	  xdep_prep_info_res(xentry) &= 0xff00ffff;
	  xdep_prep_info_res_compl(xentry) &= 0xff00ffff;
	  xdep_prep_info_res_compl(xentry) |= 0x00800000;
	  set_xdep_prep_info_is_multi(xentry, 1);

	  if( nuses + nsets == 5 ) {
	    reset_xdep_prep_info_is_over(xentry, 0);
	    for( i = 0; i < nuses; i++ ) {
	      xdep_prep_info_regs(xentry)[i] = in_regs[i];
	    }
	    for( i = 0; i < nsets-1; i++ ) {
	      xdep_prep_info_regs(xentry)[i+nuses] = out_regs[i];
	    }
	  }
	  else if( nuses + nsets > 5 ) {
	    get_overflows( xoverflow, xentry, (nuses+nsets-1) );
	    for( i = 0; i < nuses; i++ ) {
	      xoverflow[i] = in_regs[i];
	    }
	    for( i = 0; i < nsets-1; i++ ) {
	      xoverflow[i+nuses] = out_regs[i];
	    }
	  }

	  xentry = &expand[2];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, 1);
	  reset_xdep_prep_info_outs(xentry, 1);
	  reset_xdep_prep_info_is_over(xentry, 0);

	  assert( xdep_prep_info_ins(xentry) == 1 );
	  assert( xdep_prep_info_outs(xentry) == 1 );

	  xdep_prep_info_regs(xentry)[0] = out_regs[0];
	  xdep_prep_info_regs(xentry)[1] = out_regs[nsets-1];
	  xdep_prep_info_res(xentry) = RES_ONE_CR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_CR;
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.cmp_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.cmp_avail);
	  set_xdep_prep_info_is_multi(xentry, 1);
	  set_xdep_prep_info_is_slotN(xentry, 1);


	}
	break;
      }

    case OPC_ADDCO:
    case OPC_SUBFCO: {
	if( xdep_prep_ctl.flag_use_gp ) {
	  n = 3;
	  get_expands(expand, entry, n);

	  assert( nuses + 1 <= 4 );
	  assert( nsets == 3 );

	  xentry = &expand[0];
	  *xentry = *entry;
	  reset_xdep_prep_info_is_over(xentry, 0);
	  reset_xdep_prep_info_outs(xentry, 0);
	  xdep_prep_info_res( xentry ) = RES_NONE;
	  xdep_prep_info_res_compl( xentry ) = RESCPL_NONE;
	  set_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[1];
	  *xentry = *entry;
	  reset_xdep_prep_info_is_over(xentry, 0);
	  reset_xdep_prep_info_outs(xentry, 1);
	  xdep_prep_info_res(xentry) = RES_ONE_SPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	  set_xdep_prep_info_is_multi(xentry, 1);
	  for( i = 0; i < nuses; i++ ) {
	    xdep_prep_info_regs(xentry)[i] = in_regs[i];
	  }
	  xdep_prep_info_regs(xentry)[nuses] = out_regs[2];

	  xentry = &expand[2];
	  *xentry = *entry;
	  reset_xdep_prep_info_outs(xentry, 2);
	  xdep_prep_info_res(xentry) = 0x01000001;
	  xdep_prep_info_res_compl(xentry) = 0x7f80807f;
	  set_xdep_prep_info_is_multi(xentry, 1);

	  assert( (out_regs[0]>>lg2max) == 0 );
	  assert( (out_regs[1]>>lg2max) == 3 );

	  if( nuses + 2 <= 4 ) {
	    reset_xdep_prep_info_is_over(xentry, 0);
	    for( i = 0; i < nuses; i++ ) {
	      xdep_prep_info_regs(xentry)[i] = in_regs[i];
	    }
	    xdep_prep_info_regs(xentry)[nuses] = out_regs[0];
	    xdep_prep_info_regs(xentry)[nuses+1] = out_regs[1];
	  }
	  else {
	    get_overflows( xoverflow, xentry, (nuses+2) );
	    for( i = 0; i < nuses; i++ ) {
	      xoverflow[i] = in_regs[i];
	    }
	    xoverflow[nuses] = out_regs[0];
	    xoverflow[nuses+1] = out_regs[1];
	  }

	}
	break;
      }

    case OPC_ADDCO_:
    case OPC_SUBFCO_: {
	if( xdep_prep_ctl.flag_use_gp ) {
	  n = 4;
	  get_expands(expand, entry, n);

	  assert( nuses + 1 <= 4 );
	  assert( nsets == 4 );

	  xentry = &expand[0];
	  *xentry = *entry;
	  reset_xdep_prep_info_is_over(xentry, 0);
	  reset_xdep_prep_info_outs(xentry, 0);
	  xdep_prep_info_res( xentry ) = RES_NONE;
	  xdep_prep_info_res_compl( xentry ) = RESCPL_NONE;
	  set_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[1];
	  *xentry = *entry;
	  reset_xdep_prep_info_is_over(xentry, 0);
	  reset_xdep_prep_info_outs(xentry, 1);
	  xdep_prep_info_res(xentry) = RES_ONE_SPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	  set_xdep_prep_info_is_multi(xentry, 1);
	  for( i = 0; i < nuses; i++ ) {
	    xdep_prep_info_regs(xentry)[i] = in_regs[i];
	  }
	  xdep_prep_info_regs(xentry)[nuses] = out_regs[2];

	  xentry = &expand[2];
	  *xentry = *entry;
	  reset_xdep_prep_info_outs(xentry, 2);
	  xdep_prep_info_res(xentry) = 0x01000001;
	  xdep_prep_info_res_compl(xentry) = 0x7f80807f;
	  set_xdep_prep_info_is_multi(xentry, 1);

	  assert( (out_regs[0]>>lg2max) == 0 );
	  assert( (out_regs[1]>>lg2max) == 3 );

	  if( nuses + 2 <= 4 ) {
	    reset_xdep_prep_info_is_over(xentry, 0);
	    for( i = 0; i < nuses; i++ ) {
	      xdep_prep_info_regs(xentry)[i] = in_regs[i];
	    }
	    xdep_prep_info_regs(xentry)[nuses] = out_regs[0];
	    xdep_prep_info_regs(xentry)[nuses+1] = out_regs[1];
	  }
	  else {
	    get_overflows( xoverflow, xentry, (nuses+2) );
	    for( i = 0; i < nuses; i++ ) {
	      xoverflow[i] = in_regs[i];
	    }
	    xoverflow[nuses] = out_regs[0];
	    xoverflow[nuses+1] = out_regs[1];
	  }

	  xentry = &expand[3];
	  *xentry = *entry;

	  reset_xdep_prep_info_ins(xentry, 1);
	  reset_xdep_prep_info_outs(xentry, 1);
	  reset_xdep_prep_info_is_over(xentry, 0);

	  assert( xdep_prep_info_ins(xentry) == 1 );
	  assert( xdep_prep_info_outs(xentry) == 1 );

	  xdep_prep_info_regs(xentry)[0] = out_regs[0];
	  xdep_prep_info_regs(xentry)[1] = out_regs[nsets-1];
	  xdep_prep_info_res(xentry) = RES_ONE_CR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_CR;
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.cmp_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.cmp_avail);
	  set_xdep_prep_info_is_multi(xentry, 1);
	  set_xdep_prep_info_is_slotN(xentry, 1);

	}
	break;
      }

    case OPC_ADDIC_:
    case OPC_ADDE_:
    case OPC_SUBFE_:
    case OPC_ADDME_:
    case OPC_SUBFME_:
    case OPC_ADDZE_:
    case OPC_SUBFZE_:
    case OPC_ADDO_:
    case OPC_SUBFO_:
    case OPC_NEGO_:
    case OPC_CNTLZW_:
    case OPC_EXTSB_:
    case OPC_EXTSH_:
    case OPC_RLWINM_:
    case OPC_RLWNM_:
    case OPC_SLW_:
    case OPC_SRW_:
    case OPC_SRAW_:
    case OPC_SRAWI_:
    case OPC_MULLW_:
    case OPC_MULHW_:
    case OPC_MULHWU_:
    case OPC_MULLWO_:
	{

	if( xdep_prep_ctl.flag_use_gp == 2 && idx == OPC_CNTLZW_ ) {
	  break;
	}

	if( xdep_prep_ctl.flag_use_gp ) {

	  n = 2;
	  get_expands(expand, entry, n);

	  xentry = &expand[0];
	  *xentry = *entry;
	  reset_xdep_prep_info_outs(xentry, nsets-1);
	  xdep_prep_info_res(xentry) &= 0xff00ffff;
	  xdep_prep_info_res_compl(xentry) &= 0xff00ffff;
	  xdep_prep_info_res_compl(xentry) |= 0x00800000;

	  if( nuses + nsets == 5 ) {
	    reset_xdep_prep_info_is_over(xentry, 0);
	    for( i = 0; i < nuses; i++ ) {
	      xdep_prep_info_regs(xentry)[i] = in_regs[i];
	    }
	    for( i = 0; i < nsets-1; i++ ) {
	      xdep_prep_info_regs(xentry)[i+nuses] = out_regs[i];
	    }
	  }
	  else if( nuses + nsets > 5 ) {
	    get_overflows( xoverflow, xentry, (nuses+nsets-1) );
	    for( i = 0; i < nuses; i++ ) {
	      xoverflow[i] = in_regs[i];
	    }
	    for( i = 0; i < nsets-1; i++ ) {
	      xoverflow[i+nuses] = out_regs[i];
	    }
	  }

	  xentry = &expand[1];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, 1);
	  reset_xdep_prep_info_outs(xentry, 1);
	  reset_xdep_prep_info_is_over(xentry, 0);

	  assert( xdep_prep_info_ins(xentry) == 1 );
	  assert( xdep_prep_info_outs(xentry) == 1 );

	  xdep_prep_info_regs(xentry)[0] = out_regs[0];
	  xdep_prep_info_regs(xentry)[1] = out_regs[nsets-1];
	  xdep_prep_info_res(xentry) = RES_ONE_CR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_CR;
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.cmp_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.cmp_avail);
	}

	break;
      }

    case OPC_RLWIMI: {
	if( xdep_prep_ctl.flag_use_gp ) {
	  n = 2;
	  get_expands(expand, entry, n);

	  assert( xdep_prep_info_ins(entry) == 2 );
	  assert( xdep_prep_info_outs(entry) == 1 );

	  xentry = &expand[0];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, 1);
	  cvt_dep_reg(r_d, EXTRA_RLWIMI_SPR, lg2max);
	  xdep_prep_info_regs(xentry)[1] = r_d;
	  xdep_prep_info_res(xentry) = RES_ONE_SPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;

	  xentry = &expand[1];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, 2);
	  xdep_prep_info_regs(xentry)[0] = r_d;
	  reset_xdep_prep_info_outs(xentry, 1);
	}

	break;
      }

    case OPC_RLWIMI_: {
	if( xdep_prep_ctl.flag_use_gp ) {
	  n = 3;
	  get_expands(expand, entry, n);

	  assert( xdep_prep_info_ins(entry) == 2 );
	  assert( xdep_prep_info_outs(entry) == 2 );

	  xentry = &expand[0];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, 1);

	  reset_xdep_prep_info_outs(xentry, 1);
	  cvt_dep_reg(r_d, EXTRA_RLWIMI_SPR, lg2max);
	  xdep_prep_info_regs(xentry)[1] = r_d;
	  xdep_prep_info_res(xentry) = RES_ONE_SPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	  set_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[1];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, 2);
	  xdep_prep_info_regs(xentry)[0] = r_d;
	  reset_xdep_prep_info_outs(xentry, 1);
	  xdep_prep_info_res(xentry) = RES_ONE_GPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;
	  set_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[2];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, 1);
	  reset_xdep_prep_info_outs(xentry, 1);
	  reset_xdep_prep_info_is_over(xentry, 0);

	  assert( xdep_prep_info_ins(xentry) == 1 );
	  assert( xdep_prep_info_outs(xentry) == 1 );

	  xdep_prep_info_regs(xentry)[0] = out_regs[0];
	  xdep_prep_info_regs(xentry)[1] = out_regs[nsets-1];
	  xdep_prep_info_res(xentry) = RES_ONE_CR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_CR;
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.cmp_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.cmp_avail);
	  set_xdep_prep_info_is_multi(xentry, 1);
	  set_xdep_prep_info_is_slotN(xentry, 1);
	}

	break;
      }

    case OPC_DIVWO:
    case OPC_DIVWUO:
    case OPC_DIVW:
    case OPC_DIVWU: {
	if( xdep_prep_ctl.flag_use_gp ) {
	  n = 2;
	  get_expands(expand, entry, n);

	  xentry = &expand[0];
	  exp_word = OPC_ASM_ORI(  0, 0, 0 );
	  process_iword( iaddr, exp_word, xentry
#if( IN_TRANS ) // turandot
		, cycle, processor_id, thread_id
#endif
		);
		
	  reset_xdep_prep_info_is_nop(xentry, 1);
	  reset_xdep_prep_info_ins(xentry, 0);
	  reset_xdep_prep_info_outs(xentry, 0);
	  xdep_prep_info_res( xentry ) = RES_NONE;
	  xdep_prep_info_res_compl( xentry ) = RESCPL_NONE;

	  xentry = &expand[1];
	  *xentry = *entry;

	}
	break;
      }

    case OPC_DIVWO_:
    case OPC_DIVWUO_:
    case OPC_DIVW_:
    case OPC_DIVWU_: {
	if( xdep_prep_ctl.flag_use_gp ) {
	  n = 3;
	  get_expands(expand, entry, n);

	  xentry = &expand[0];
	  exp_word = OPC_ASM_ORI(  0, 0, 0 );
	  process_iword( iaddr, exp_word, xentry
#if( IN_TRANS ) // turandot
		, cycle, processor_id, thread_id
#endif
		);
	  reset_xdep_prep_info_is_nop(xentry, 1);
	  reset_xdep_prep_info_ins(xentry, 0);
	  reset_xdep_prep_info_outs(xentry, 0);
	  xdep_prep_info_res( xentry ) = RES_NONE;
	  xdep_prep_info_res_compl( xentry ) = RESCPL_NONE;
	  set_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[1];
	  *xentry = *entry;
	  reset_xdep_prep_info_outs(xentry, nsets-1);
	  xdep_prep_info_res(xentry) &= 0xff00ffff;
	  xdep_prep_info_res_compl(xentry) &= 0xff00ffff;
	  xdep_prep_info_res_compl(xentry) |= 0x00800000;

	  if( nuses + nsets == 5 ) {
	    reset_xdep_prep_info_is_over(xentry, 0);
	    for( i = 0; i < nuses; i++ ) {
	      xdep_prep_info_regs(xentry)[i] = in_regs[i];
	    }
	    for( i = 0; i < nsets-1; i++ ) {
	      xdep_prep_info_regs(xentry)[i+nuses] = out_regs[i];
	    }
	  }
	  else if( nuses + nsets > 5 ) {
	    get_overflows( xoverflow, xentry, (nuses+nsets-1) );
	    for( i = 0; i < nuses; i++ ) {
	      xoverflow[i] = in_regs[i];
	    }
	    for( i = 0; i < nsets-1; i++ ) {
	      xoverflow[i+nuses] = out_regs[i];
	    }
	  }
	  set_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[2];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, 1);
	  reset_xdep_prep_info_outs(xentry, 1);
	  reset_xdep_prep_info_is_over(xentry, 0);

	  assert( xdep_prep_info_ins(xentry) == 1 );
	  assert( xdep_prep_info_outs(xentry) == 1 );

	  xdep_prep_info_regs(xentry)[0] = out_regs[0];
	  xdep_prep_info_regs(xentry)[1] = out_regs[nsets-1];
	  xdep_prep_info_res(xentry) = RES_ONE_CR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_CR;
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.cmp_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.cmp_avail);
	  set_xdep_prep_info_is_multi(xentry, 1);
	  reset_xdep_prep_info_is_slotN(xentry, 1);
	}
	break;
      }

    case OPC_ADDEO:
    case OPC_SUBFEO:
    case OPC_ADDMEO:
    case OPC_SUBFMEO:
    case OPC_ADDZEO:
    case OPC_SUBFZEO: {
	if( xdep_prep_ctl.flag_use_gp ) {
	  n = 2;
	  get_expands(expand, entry, n);

	  xentry = &expand[0];
	  *xentry = *entry;

	  cvt_dep_reg(reg, OPC_XER_SO, lg2max);
	  assert( reg < (1<<(2+lg2max)) );

	  reset_xdep_prep_info_outs(xentry, 1);
	  if( nuses + 1 <= 4 ) {
	    reset_xdep_prep_info_is_over(xentry, 0);
	    for( i = 0; i < nuses; i++ ) {
	      xdep_prep_info_regs(xentry)[i] = in_regs[i];
	      assert( in_regs[i] < (1<<(2+lg2max)) );
	      assert( xdep_prep_info_regs(xentry)[i] < (1<<(2+lg2max)) );
	    }
	    xdep_prep_info_regs(xentry)[nuses] = reg;
	  }
	  else {
	    get_overflows( xoverflow, xentry, (nuses+1) );
	    for( i = 0; i < nuses; i++ ) {
	      xoverflow[i] = in_regs[i];
	    }
	    xoverflow[nuses] = reg;
	  }
	  xdep_prep_info_res(xentry) = RES_ONE_SPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;

	  xentry = &expand[1];
	  *xentry = *entry;

	  reset_xdep_prep_info_outs(xentry, (nsets-1));
	  if( nuses + nsets - 1 <= 4 ) {
	    reset_xdep_prep_info_is_over(xentry, 0);
	    for( i = 0; i < nuses; i++ ) {
	      xdep_prep_info_regs(xentry)[i] = in_regs[i];
	    }
	    j = nuses;
	    for( i = 0; i < nsets; i++ ) {
	      if( out_regs[i] != reg ) {
		xdep_prep_info_regs(xentry)[j] = out_regs[i];
		j++;
	      }
	    }
	    assert( j == nuses + nsets - 1 );
	  }
	  else {
	    get_overflows( xoverflow, xentry, (nuses+nsets-1) );
	    for( i = 0; i < nuses; i++ ) {
	      xoverflow[i] = in_regs[i];
	    }
	    j = nuses;
	    for( i = 0; i < nsets; i++ ) {
	      if( out_regs[i] != reg ) {
		xoverflow[j] = out_regs[i];
		j++;
	      }
	    }
	    assert( j == nuses + nsets - 1 );
	  }
	  xdep_prep_info_res(xentry) &= 0x00ffffff;
	  xdep_prep_info_res(xentry) |= 0x01000000;
	  xdep_prep_info_res_compl(xentry) &= 0x00ffffff;
	  xdep_prep_info_res_compl(xentry) |= 0x7f000000;

	}
	break;
      }


    case OPC_ADDEO_:
    case OPC_SUBFEO_:
    case OPC_ADDMEO_:
    case OPC_SUBFMEO_:
    case OPC_ADDZEO_:
    case OPC_SUBFZEO_: {
	if( xdep_prep_ctl.flag_use_gp ) {
	  n = 3;
	  get_expands(expand, entry, n);

	  xentry = &expand[0];
	  *xentry = *entry;

	  cvt_dep_reg(reg, OPC_XER_SO, lg2max);
	  assert( reg < (1<<(2+lg2max)) );

	  reset_xdep_prep_info_outs(xentry, 1);
	  if( nuses + 1 <= 4 ) {
	    reset_xdep_prep_info_is_over(xentry, 0);
	    for( i = 0; i < nuses; i++ ) {
	      xdep_prep_info_regs(xentry)[i] = in_regs[i];
	      assert( in_regs[i] < (1<<(2+lg2max)) );
	      assert( xdep_prep_info_regs(xentry)[i] < (1<<(2+lg2max)) );
	    }
	    xdep_prep_info_regs(xentry)[nuses] = reg;
	  }
	  else {
	    get_overflows( xoverflow, xentry, (nuses+1) );
	    for( i = 0; i < nuses; i++ ) {
	      xoverflow[i] = in_regs[i];
	    }
	    xoverflow[nuses] = reg;
	  }
	  xdep_prep_info_res(xentry) = RES_ONE_SPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	  set_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[1];
	  *xentry = *entry;

	  reset_xdep_prep_info_outs(xentry, (nsets-2));
	  if( nuses + nsets - 2 <= 4 ) {
	    reset_xdep_prep_info_is_over(xentry, 0);
	    for( i = 0; i < nuses; i++ ) {
	      xdep_prep_info_regs(xentry)[i] = in_regs[i];
	    }
	    j = nuses;
	    for( i = 0; i < nsets - 1; i++ ) {
	      if( out_regs[i] != reg ) {
		xdep_prep_info_regs(xentry)[j] = out_regs[i];
		j++;
	      }
	    }
	    assert( j == nuses + nsets - 2 );
	  }
	  else {
	    get_overflows( xoverflow, xentry, (nuses+nsets-1) );
	    for( i = 0; i < nuses; i++ ) {
	      xoverflow[i] = in_regs[i];
	    }
	    j = nuses;
	    for( i = 0; i < nsets - 1; i++ ) {
	      if( out_regs[i] != reg ) {
		xoverflow[j] = out_regs[i];
		j++;
	      }
	    }
	    assert( j == nuses + nsets - 2 );
	  }
	  xdep_prep_info_res(xentry) &= 0x0000ffff;
	  xdep_prep_info_res(xentry) |= 0x01000000;
	  xdep_prep_info_res_compl(xentry) &= 0x0000ffff;
	  xdep_prep_info_res_compl(xentry) |= 0x7f800000;
	  set_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[2];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, 1);
	  reset_xdep_prep_info_outs(xentry, 1);
	  reset_xdep_prep_info_is_over(xentry, 0);

	  assert( xdep_prep_info_ins(xentry) == 1 );
	  assert( xdep_prep_info_outs(xentry) == 1 );

	  xdep_prep_info_regs(xentry)[0] = out_regs[0];
	  xdep_prep_info_regs(xentry)[1] = out_regs[nsets-1];
	  xdep_prep_info_res(xentry) = RES_ONE_CR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_CR;
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.cmp_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.cmp_avail);
	  set_xdep_prep_info_is_multi(xentry, 1);
	  reset_xdep_prep_info_is_slotN(xentry, 1);

	}
	break;
      }
    case OPC_MFSPR: {
	if( xdep_prep_ctl.flag_spr_split ) {
	  spr = (iword >> 11) & 0x3ff;
	  if( spr == 0x020 ) {
	    if( xdep_prep_ctl.flag_use_gp ) {
	      n = 7;
	      get_expands( expand, entry, n);

	      f_d = OPC_GPR(OPC_DEC_RT(iword));
	      cvt_dep_reg(r_d, f_d, lg2max);
	      assert( xdep_prep_reg_class( out_regs[j], lg2max ) == 0 );
	      assert( xdep_prep_info_res(entry) == RES_ONE_GPR );
	      assert( xdep_prep_info_res_compl(entry) == RESCPL_ONE_GPR );

	      xentry = &expand[0];
	      *xentry = *entry;

	      reset_xdep_prep_info_is_over(xentry, 0);

	      reset_xdep_prep_info_ins(xentry, 1);
	      cvt_dep_reg(r_s, OPC_XER_BCNT, lg2max);
	      xdep_prep_info_regs(xentry)[0] = r_s;

	      reset_xdep_prep_info_outs(xentry, 1);
	      cvt_dep_reg(r_d, EXTRA_MFSPR0_SPR, lg2max);
	      xdep_prep_info_regs(xentry)[1] = r_d;
	      xdep_prep_info_res(xentry) = RES_ONE_SPR;
	      xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	      set_xdep_prep_info_is_multi(xentry, 1);

	      xentry = &expand[1];
	      *xentry = *entry;

	      reset_xdep_prep_info_is_over(xentry, 0);

	      reset_xdep_prep_info_ins(xentry, 1);
	      cvt_dep_reg(r_s, OPC_XER_SO, lg2max);
	      xdep_prep_info_regs(xentry)[0] = r_s;

	      reset_xdep_prep_info_outs(xentry, 1);
	      cvt_dep_reg(r_d, EXTRA_MFSPR1_SPR, lg2max);
	      xdep_prep_info_regs(xentry)[1] = r_d;
	      xdep_prep_info_res(xentry) = RES_ONE_SPR;
	      xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	      set_xdep_prep_info_is_multi(xentry, 1);

	      xentry = &expand[2];
	      *xentry = *entry;

	      reset_xdep_prep_info_is_over(xentry, 0);

	      reset_xdep_prep_info_ins(xentry, 1);
	      cvt_dep_reg(r_s, OPC_XER_CA, lg2max);
	      xdep_prep_info_regs(xentry)[0] = r_s;

	      reset_xdep_prep_info_outs(xentry, 1);
	      cvt_dep_reg(r_d, EXTRA_MFSPR2_SPR, lg2max);
	      xdep_prep_info_regs(xentry)[1] = r_d;
	      xdep_prep_info_res(xentry) = RES_ONE_SPR;
	      xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	      set_xdep_prep_info_is_multi(xentry, 1);

	      xentry = &expand[3];
	      *xentry = *entry;

	      reset_xdep_prep_info_is_over(xentry, 0);

	      reset_xdep_prep_info_ins(xentry, 2);
	      cvt_dep_reg(r_s, EXTRA_MFSPR0_SPR, lg2max);
	      xdep_prep_info_regs(xentry)[0] = r_s;
	      cvt_dep_reg(r_s, EXTRA_MFSPR1_SPR, lg2max);
	      xdep_prep_info_regs(xentry)[1] = r_s;

	      reset_xdep_prep_info_outs(xentry, 1);
	      xdep_prep_info_regs(xentry)[2] = out_regs[0];
	      xdep_prep_info_res(xentry) = RES_ONE_GPR;
	      xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;
	      set_xdep_prep_info_is_multi(xentry, 1);
	      set_xdep_prep_info_is_slotN(xentry, 1);

	      xentry = &expand[4];
	      *xentry = *entry;

	      reset_xdep_prep_info_is_over(xentry, 0);

	      reset_xdep_prep_info_ins(xentry, 1);
	      cvt_dep_reg(r_s, OPC_XER_FXCC, lg2max);
	      xdep_prep_info_regs(xentry)[0] = r_s;

	      reset_xdep_prep_info_outs(xentry, 1);
	      cvt_dep_reg(r_d, EXTRA_MFSPR0_SPR, lg2max);
	      xdep_prep_info_regs(xentry)[1] = r_d;
	      xdep_prep_info_res(xentry) = RES_ONE_SPR;
	      xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	      set_xdep_prep_info_is_multi(xentry, 1);

	      xentry = &expand[5];
	      *xentry = *entry;

	      reset_xdep_prep_info_is_over(xentry, 0);
	      reset_xdep_prep_info_ins(xentry, 2);
	      cvt_dep_reg(r_s, EXTRA_MFSPR0_SPR, lg2max);
	      xdep_prep_info_regs(xentry)[0] = r_s;
	      cvt_dep_reg(r_s, EXTRA_MFSPR2_SPR, lg2max);
	      xdep_prep_info_regs(xentry)[1] = r_s;

	      reset_xdep_prep_info_outs(xentry, 1);
	      cvt_dep_reg(r_d, EXTRA_MFSPR0_SPR, lg2max);
	      xdep_prep_info_regs(xentry)[2] = r_d;
	      xdep_prep_info_res(xentry) = RES_ONE_SPR;
	      xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	      set_xdep_prep_info_is_multi(xentry, 1);

	      xentry = &expand[6];
	      *xentry = *entry;

	      reset_xdep_prep_info_is_over(xentry, 0);
	      reset_xdep_prep_info_ins(xentry, 2);
	      cvt_dep_reg(r_s, EXTRA_MFSPR0_SPR, lg2max);
	      xdep_prep_info_regs(xentry)[0] = r_s;
	      xdep_prep_info_regs(xentry)[1] = out_regs[0];

	      reset_xdep_prep_info_outs(xentry, 1);
	      xdep_prep_info_regs(xentry)[2] = out_regs[0];
	      xdep_prep_info_res(xentry) = RES_ONE_GPR;
	      xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;
	      set_xdep_prep_info_is_multi(xentry, 1);

	      set_xdep_prep_info_is_slotN(&expand[n-1], 1);
	    }
	    else {
	      n = 4;
	      f_d = OPC_GPR(OPC_DEC_RT(iword));

	      get_expands(expand, entry, n);

	      xentry = &expand[0];
	      *xentry = *entry;
	      reset_xdep_prep_info_is_over(xentry, 0);
	      reset_xdep_prep_info_ins(xentry, 1);
	      cvt_dep_reg(reg, OPC_XER_BCNT, lg2max);
	      xdep_prep_info_regs(xentry)[0] = reg;
	      reset_xdep_prep_info_outs(xentry, 1);
	      cvt_dep_reg(reg, f_d, lg2max);
	      xdep_prep_info_regs(xentry)[1] = reg;
	      set_xdep_prep_info_is_multi(xentry, 1);

	      xentry = &expand[1];
	      *xentry = *entry;
	      reset_xdep_prep_info_is_over(xentry, 0);
	      reset_xdep_prep_info_ins(xentry, 1);
	      cvt_dep_reg(reg, OPC_XER_SO, lg2max);
	      xdep_prep_info_regs(xentry)[0] = reg;
	      reset_xdep_prep_info_outs(xentry, 1);
	      cvt_dep_reg(reg, f_d, lg2max);
	      xdep_prep_info_regs(xentry)[1] = reg;
	      set_xdep_prep_info_is_multi(xentry, 1);

	      xentry = &expand[2];
	      *xentry = *entry;
	      reset_xdep_prep_info_is_over(xentry, 0);
	      reset_xdep_prep_info_ins(xentry, 1);
	      cvt_dep_reg(reg, OPC_XER_CA, lg2max);
	      xdep_prep_info_regs(xentry)[0] = reg;
	      reset_xdep_prep_info_outs(xentry, 1);
	      cvt_dep_reg(reg, f_d, lg2max);
	      xdep_prep_info_regs(xentry)[1] = reg;
	      set_xdep_prep_info_is_multi(xentry, 1);

	      xentry = &expand[3];
	      *xentry = *entry;
	      reset_xdep_prep_info_is_over(xentry, 0);
	      reset_xdep_prep_info_ins(xentry, 1);
	      cvt_dep_reg(reg, OPC_XER_FXCC, lg2max);
	      xdep_prep_info_regs(xentry)[0] = reg;
	      reset_xdep_prep_info_outs(xentry, 1);
	      cvt_dep_reg(reg, f_d, lg2max);
	      xdep_prep_info_regs(xentry)[1] = reg;
	      set_xdep_prep_info_is_multi(xentry, 1);
	      reset_xdep_prep_info_is_slotN(xentry, 1);
	    }
	  }
	}
	break;
      }

    case OPC_MTSPR: {
	if( xdep_prep_ctl.flag_spr_split ) {
	  spr = (iword >> 11) & 0x3ff;

	  /**** XER ****/
	  if( spr == 0x020 ) {
	    if( xdep_prep_ctl.flag_use_gp ) {
	      i = 0;
	      n = 8;
	    }
	    else {
	      i = 0;
	      n = 4;
	    }

	    r_s = OPC_GPR(OPC_DEC_RS(iword));
	    get_expands(expand, entry, n);

	    xentry = &expand[i];
	    i++;
	    *xentry = *entry;
	    reset_xdep_prep_info_is_over(xentry, 0);
	    reset_xdep_prep_info_ins(xentry, 1);
	    cvt_dep_reg(reg, r_s, lg2max);
	    xdep_prep_info_regs(xentry)[0] = reg;
	    reset_xdep_prep_info_outs(xentry, 1);
	    cvt_dep_reg(reg, OPC_XER_BCNT, lg2max);
	    xdep_prep_info_regs(xentry)[1] = reg;
	    set_xdep_prep_info_is_multi(xentry, 1);
	    xdep_prep_info_res(xentry) = RES_ONE_SPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;

	    if( xdep_prep_ctl.flag_use_gp ) {
	      xentry = & expand[i];
	      i++;
	      exp_word = OPC_ASM_ORI(  0, 0, 0 );
	      process_iword( iaddr, exp_word, xentry
#if( IN_TRANS ) // turandot
			, cycle, processor_id, thread_id
#endif
			);
	      reset_xdep_prep_info_is_nop(xentry, 1);
	      reset_xdep_prep_info_ins(xentry, 0);
	      reset_xdep_prep_info_outs(xentry, 0);
	      xdep_prep_info_res( xentry ) = RES_NONE;
	      xdep_prep_info_res_compl( xentry ) = RESCPL_NONE;
	      set_xdep_prep_info_is_multi(xentry, 1);

	      xentry = & expand[i];
	      i++;
	      exp_word = OPC_ASM_ORI(  0, 0, 0 );
	      process_iword( iaddr, exp_word, xentry
#if( IN_TRANS ) // turandot
			, cycle, processor_id, thread_id
#endif
			);
	      reset_xdep_prep_info_is_nop(xentry, 1);
	      reset_xdep_prep_info_ins(xentry, 0);
	      reset_xdep_prep_info_outs(xentry, 0);
	      xdep_prep_info_res( xentry ) = RES_NONE;
	      xdep_prep_info_res_compl( xentry ) = RESCPL_NONE;
	      set_xdep_prep_info_is_multi(xentry, 1);
	    }

	    xentry = &expand[i];
	    i++;
	    *xentry = *entry;
	    reset_xdep_prep_info_is_over(xentry, 0);
	    reset_xdep_prep_info_ins(xentry, 1);
	    reset_xdep_prep_info_outs(xentry, 1);
	    cvt_dep_reg(reg, r_s, lg2max);
	    xdep_prep_info_regs(xentry)[0] = reg;
	    cvt_dep_reg(reg, OPC_XER_SO, lg2max);
	    xdep_prep_info_regs(xentry)[1] = reg;
	    set_xdep_prep_info_is_multi(xentry, 1);
	    xdep_prep_info_res(xentry) = RES_ONE_SPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;

	    xentry = &expand[i];
	    i++;
	    *xentry = *entry;
	    reset_xdep_prep_info_is_over(xentry, 0);
	    reset_xdep_prep_info_ins(xentry, 1);
	    reset_xdep_prep_info_outs(xentry, 1);
	    cvt_dep_reg(reg, r_s, lg2max);
	    xdep_prep_info_regs(xentry)[0] = reg;
	    cvt_dep_reg(reg, OPC_XER_CA, lg2max);
	    xdep_prep_info_regs(xentry)[1] = reg;
	    set_xdep_prep_info_is_multi(xentry, 1);
	    xdep_prep_info_res(xentry) = RES_ONE_SPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;

	    if( xdep_prep_ctl.flag_use_gp ) {
	      xentry = & expand[i];
	      i++;
	      exp_word = OPC_ASM_ORI(  0, 0, 0 );
	      process_iword( iaddr, exp_word, xentry
#if( IN_TRANS ) // turandot
			, cycle, processor_id, thread_id
#endif
			);
	      reset_xdep_prep_info_is_nop(xentry, 1);
	      reset_xdep_prep_info_ins(xentry, 0);
	      reset_xdep_prep_info_outs(xentry, 0);
	      xdep_prep_info_res( xentry ) = RES_NONE;
	      xdep_prep_info_res_compl( xentry ) = RESCPL_NONE;
	      set_xdep_prep_info_is_multi(xentry, 1);

	      xentry = & expand[i];
	      i++;
	      exp_word = OPC_ASM_ORI(  0, 0, 0 );
	      process_iword( iaddr, exp_word, xentry
#if( IN_TRANS ) // turandot
			, cycle, processor_id, thread_id
#endif
			);
	      reset_xdep_prep_info_is_nop(xentry, 1);
	      reset_xdep_prep_info_ins(xentry, 0);
	      reset_xdep_prep_info_outs(xentry, 0);
	      xdep_prep_info_res( xentry ) = RES_NONE;
	      xdep_prep_info_res_compl( xentry ) = RESCPL_NONE;
	      set_xdep_prep_info_is_multi(xentry, 1);
	    }

	    xentry = &expand[i];
	    i++;
	    *xentry = *entry;
	    reset_xdep_prep_info_is_over(xentry, 0);
	    reset_xdep_prep_info_ins(xentry, 1);
	    reset_xdep_prep_info_outs(xentry, 1);
	    cvt_dep_reg(reg, r_s, lg2max);
	    xdep_prep_info_regs(xentry)[0] = reg;
	    cvt_dep_reg(reg, OPC_XER_FXCC, lg2max);
	    xdep_prep_info_regs(xentry)[1] = reg;
	    set_xdep_prep_info_is_multi(xentry, 1);
	    xdep_prep_info_res(xentry) = RES_ONE_SPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	    reset_xdep_prep_info_is_slotN(xentry, 1);
	  }
	  else {
	    /* other than XER; CTR, LR */
	    if( xdep_prep_ctl.flag_use_gp ) {
	      reset_xdep_prep_info_is_slot0(entry, 1);
	    }
	  }
	}
	break;
      }

    case OPC_LHA:
    case OPC_LWA:
    case OPC_LHAX:
    case OPC_LWAX: {
	if( xdep_prep_ctl.flag_fxlda_split  ) {
	  n = 2;
	  get_expands(expand, entry, n);
	  Treg = out_regs[0];

	  xentry = &expand[0];
	  *xentry = *entry;

	  xentry = &expand[1];
	  *xentry = *entry;

	  reset_xdep_prep_info_ins(xentry, 1);
	  xdep_prep_info_regs(xentry)[0] = Treg;
	  xdep_prep_info_regs(xentry)[1] = Treg;
	  reset_xdep_prep_info_is_memop(xentry, 0);
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	}

	break;
      }


    case OPC_LBZU:
    case OPC_LHZU:
    case OPC_LWZU:
    case OPC_LDU: {
	if( xdep_prep_ctl.flag_fxldu_split  ) {
	  n = 2;
	  get_expands(expand, entry, n);
	  Treg = out_regs[1];
	  Areg = out_regs[0];

	  xentry = &expand[0];
	  *xentry = *entry;
	  reset_xdep_prep_info_outs(xentry, 1);
	  xdep_prep_info_regs(xentry)[nuses] = Treg;
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.load_unit);
	  xdep_prep_info_res(xentry) = RES_ONE_GPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;

	  xentry = &expand[1];
	  *xentry = *entry;
	  reset_xdep_prep_info_outs(xentry, 1);
	  xdep_prep_info_regs(xentry)[nuses] = Areg;
	  reset_xdep_prep_info_is_memop(xentry, 0);
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	  xdep_prep_info_res(xentry) = RES_ONE_GPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;

	}

	break;
      }

    case OPC_LBZUX:
    case OPC_LHZUX:
    case OPC_LWZUX:
    case OPC_LDUX: {
	if( xdep_prep_ctl.flag_fxldu_split  ) {
	  Treg = out_regs[0];
	  Areg = out_regs[1];


	  if( Treg != in_regs[0] && Treg != in_regs[1]
		&& xdep_prep_ctl.flag_use_gp != 2 ) {
	    n = 2;
	    get_expands(expand, entry, n);

	    xentry = &expand[0];
	    *xentry = *entry;
	    reset_xdep_prep_info_outs(xentry, 1);
	    xdep_prep_info_regs(xentry)[nuses] = Treg;
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.load_unit);
	    xdep_prep_info_res(xentry) = RES_ONE_GPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;

	    xentry = &expand[1];
	    *xentry = *entry;
	    reset_xdep_prep_info_outs(xentry, 1);
	    xdep_prep_info_regs(xentry)[nuses] = Areg;
	    reset_xdep_prep_info_is_memop(xentry, 0);
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	    reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	    reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	    xdep_prep_info_res(xentry) = RES_ONE_GPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;
	  }
	  else if( !xdep_prep_ctl.flag_use_gp ) {
	    n = 2;
	    get_expands(expand, entry, n);

	    xentry = &expand[0];
	    *xentry = *entry;
	    reset_xdep_prep_info_outs(xentry, 1);
	    xdep_prep_info_regs(xentry)[nuses] = Areg;
	    reset_xdep_prep_info_is_memop(xentry, 0);
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	    reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	    reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	    xdep_prep_info_res(xentry) = RES_ONE_GPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;

	    xentry = &expand[1];
	    *xentry = *entry;
	    reset_xdep_prep_info_ins(xentry, 1);
	    reset_xdep_prep_info_outs(xentry, 1);
	    xdep_prep_info_regs(xentry)[0] = Areg;
	    xdep_prep_info_regs(xentry)[1] = Treg;
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.load_unit);
	    xdep_prep_info_res(xentry) = RES_ONE_GPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;

	  }
	  else {
	    n = 3;
	    get_expands(expand, entry, n);

	    cvt_dep_reg(reg, EXTRA_LUX_SPR, lg2max);

	    xentry = &expand[0];
	    *xentry = *entry;
	    reset_xdep_prep_info_outs(xentry, 1);
	    xdep_prep_info_regs(xentry)[nuses] = reg;
	    reset_xdep_prep_info_is_memop(xentry, 0);
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	    reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	    reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	    xdep_prep_info_res(xentry) = RES_ONE_SPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	    reset_xdep_prep_info_is_multi(xentry, 1);

	    xentry = &expand[1];
	    *xentry = *entry;
	    reset_xdep_prep_info_outs(xentry, 1);
	    xdep_prep_info_regs(xentry)[nuses] = Treg;
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.load_unit);
	    xdep_prep_info_res(xentry) = RES_ONE_GPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;
	    set_xdep_prep_info_is_multi(xentry, 1);

	    xentry = &expand[2];
	    *xentry = *entry;
	    reset_xdep_prep_info_ins(xentry, 1);
	    reset_xdep_prep_info_outs(xentry, 1);
	    xdep_prep_info_regs(xentry)[0] = reg;
	    xdep_prep_info_regs(xentry)[1] = Areg;
	    reset_xdep_prep_info_is_memop(xentry, 0);
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	    reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	    reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	    xdep_prep_info_res(xentry) = RES_ONE_GPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;
	    reset_xdep_prep_info_is_multi(xentry, 1);

	    reset_xdep_prep_info_is_slotN(&expand[n-1], 1);
	  }
	}

	break;
      }

    case OPC_LHAU:
    case OPC_LHAUX:
    case OPC_LWAUX: {

	if( idx == OPC_LHAU  ) {
	  Treg = out_regs[1];
	  Areg = out_regs[0];
	}
	else {
	  Treg = out_regs[0];
	  Areg = out_regs[1];
	}

	if( xdep_prep_ctl.flag_fxldau_split  ) {
	  n = 3;
	  get_expands(expand, entry, n);

	  xentry = &expand[0];
	  *xentry = *entry;
	  reset_xdep_prep_info_outs(xentry, 1);
	  cvt_dep_reg(reg, EXTRA0_STRING_SPR, lg2max);
	  xdep_prep_info_regs(xentry)[nuses] = reg;
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.load_unit);
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.load_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.load_avail);
	  xdep_prep_info_res(xentry) = RES_ONE_SPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	  set_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[1];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, 1);
	  reset_xdep_prep_info_outs(xentry, 1);
	  xdep_prep_info_regs(xentry)[0] = reg;
	  xdep_prep_info_regs(xentry)[1] = Treg;
	  reset_xdep_prep_info_is_memop(xentry, 0);
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	  xdep_prep_info_res(xentry) = RES_ONE_GPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;
	  set_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[2];
	  *xentry = *entry;
	  reset_xdep_prep_info_outs(xentry, 1);
	  xdep_prep_info_regs(xentry)[1] = Areg;
	  reset_xdep_prep_info_is_memop(xentry, 0);
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	  xdep_prep_info_res(xentry) = RES_ONE_GPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;
	  set_xdep_prep_info_is_multi(xentry, 1);

	  reset_xdep_prep_info_is_slotN(&expand[n-1], 1);
	}
	else if( xdep_prep_ctl.flag_fxldu_split  ) {

	  if( (idx == OPC_LHAU || (Treg != in_regs[0] && Treg != in_regs[1]))
		&& xdep_prep_ctl.flag_use_gp != 2 ) {

	    n = 2;
	    get_expands(expand, entry, n);

	    xentry = &expand[0];
	    *xentry = *entry;
	    reset_xdep_prep_info_outs(xentry, 1);
	    xdep_prep_info_regs(xentry)[nuses] = Treg;
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.load_unit);
	    xdep_prep_info_res(xentry) = RES_ONE_GPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;

	    xentry = &expand[1];
	    *xentry = *entry;
	    reset_xdep_prep_info_outs(xentry, 1);
	    xdep_prep_info_regs(xentry)[nuses] = Areg;
	    reset_xdep_prep_info_is_memop(xentry, 0);
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	    reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	    reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	    xdep_prep_info_res(xentry) = RES_ONE_GPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;
	  }
	  else {

	    n = 2;
	    get_expands(expand, entry, n);

	    xentry = &expand[0];
	    *xentry = *entry;
	    reset_xdep_prep_info_outs(xentry, 1);
	    xdep_prep_info_regs(xentry)[nuses] = Areg;
	    reset_xdep_prep_info_is_memop(xentry, 0);
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	    reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	    reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	    xdep_prep_info_res(xentry) = RES_ONE_GPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;
	    set_xdep_prep_info_is_multi(xentry, 1);

	    xentry = &expand[1];
	    *xentry = *entry;
	    reset_xdep_prep_info_ins(xentry, 1);
	    reset_xdep_prep_info_outs(xentry, 1);
	    xdep_prep_info_regs(xentry)[0] = Areg;
	    xdep_prep_info_regs(xentry)[1] = Treg;
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.load_unit);
	    xdep_prep_info_res(xentry) = RES_ONE_GPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;
	    set_xdep_prep_info_is_multi(xentry, 1);

	    reset_xdep_prep_info_is_slotN(&expand[n-1], 1);
	  }
	}
	else if( xdep_prep_ctl.flag_fxlda_split  ) {
	  n = 2;
	  get_expands(expand, entry, n);

	  xentry = &expand[0];
	  *xentry = *entry;

	  xentry = &expand[1];
	  *xentry = *entry;


	  reset_xdep_prep_info_ins(xentry, 1);
	  xdep_prep_info_regs(xentry)[0] = Treg;
	  xdep_prep_info_regs(xentry)[1] = Treg;
	  reset_xdep_prep_info_is_memop(xentry, 0);
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	}
	break;
      }

    case OPC_LFDU:
    case OPC_LFDUX:
    case OPC_LFSU:
    case OPC_LFSUX: {
	  if( xdep_prep_ctl.flag_fpldu_split ) {
	    n = 2;
	    get_expands(expand, entry, n);

	    xentry = &expand[0];
	    *xentry = *entry;
	    reset_xdep_prep_info_outs(xentry, 1);
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.load_unit);
	    mod_latency = xdep_prep_info_latency(entry);
	    reset_xdep_prep_info_latency(xentry, mod_latency);
	    mod_avail = xdep_prep_info_avail(entry);
	    reset_xdep_prep_info_avail(xentry, mod_avail);
	    xdep_prep_info_res(xentry) = RES_ONE_FPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_FPR;

	    xentry = &expand[1];
	    *xentry = *entry;
	    reset_xdep_prep_info_outs(xentry, 1);
	    reset_xdep_prep_info_is_memop(xentry, 0);
	    xdep_prep_info_regs(xentry)[nuses] = out_regs[1];
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	    reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	    reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	    xdep_prep_info_res(xentry) = RES_ONE_GPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;
	  }

	  break;
	}

    case OPC_STFS:
    case OPC_STFSX:
    case OPC_STFD:
    case OPC_STFDX: {
	if( xdep_prep_ctl.flag_fpst_split ) {
	  n = 2;
	  get_expands( expand, entry, n);

	  xentry = &expand[0];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, nuses-1);
	  for( i = 1; i < nuses; i++ ) {
	    xdep_prep_info_regs(xentry)[i-1] = in_regs[i];
	  }
	  reset_xdep_prep_info_is_split(xentry, 1);
	  reset_xdep_prep_info_is_nocount(xentry, 1);

	  xentry = &expand[1];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, 1);
	  reset_xdep_prep_info_is_store(xentry, 0);
	  reset_xdep_prep_info_is_memop(xentry, 0);
	  reset_xdep_prep_info_is_transfer(xentry, 1);
	  reset_xdep_prep_info_is_reissue(xentry, 1);
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fpu_unit);
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency+1);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail); /* transfer */
	}

	break;
      }

    case OPC_STB:
    case OPC_STH:
    case OPC_STW:
    case OPC_STD: {
	if( xdep_prep_ctl.flag_fxst_split ) {
	  n = 2;
	  get_expands( expand, entry, n);

	  xentry = &expand[0];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, nuses-1);
	  for( i = 1; i < nuses; i++ ) {
	    xdep_prep_info_regs(xentry)[i-1] = in_regs[i];
	  }
	  reset_xdep_prep_info_is_split(xentry, 1);
	  reset_xdep_prep_info_is_nocount(xentry, 1);
	  assert( xdep_prep_info_res(xentry) == 0x0 );
	  assert( xdep_prep_info_res_compl(xentry) == 0x80808080 );

	  xentry = &expand[1];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, 1);
	  reset_xdep_prep_info_is_store(xentry, 0);
	  reset_xdep_prep_info_is_memop(xentry, 0);
	  reset_xdep_prep_info_is_transfer(xentry, 1);
	  reset_xdep_prep_info_is_reissue(xentry, 1);
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	  assert( xdep_prep_info_res(xentry) == 0x0 );
	  assert( xdep_prep_info_res_compl(xentry) == 0x80808080 );
	}

	break;
      }

    case OPC_STBX:
    case OPC_STHX:
    case OPC_STWBRX:
    case OPC_STWX:
    case OPC_STDX: {
	if( xdep_prep_ctl.flag_fxst_split ) {
	  n = 2;
	  get_expands( expand, entry, n);

	  xentry = &expand[0];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, nuses-1);
	  for( i = 1; i < nuses; i++ ) {
	    xdep_prep_info_regs(xentry)[i-1] = in_regs[i];
	  }
	  reset_xdep_prep_info_is_split(xentry, 1);
	  assert( xdep_prep_info_res(xentry) == 0x0 );
	  assert( xdep_prep_info_res_compl(xentry) == 0x80808080 );

	  xentry = &expand[1];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, 1);
	  reset_xdep_prep_info_is_store(xentry, 0);
	  reset_xdep_prep_info_is_memop(xentry, 0);
	  reset_xdep_prep_info_is_transfer(xentry, 1);
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	  assert( xdep_prep_info_res(xentry) == 0x0 );
	  assert( xdep_prep_info_res_compl(xentry) == 0x80808080 );
	}

	break;
      }

    case OPC_STFSU:
    case OPC_STFSUX:
    case OPC_STFDU:
    case OPC_STFDUX: {
	if( xdep_prep_ctl.flag_fpstu_split ) {
	  if( xdep_prep_ctl.flag_stu3_split ) {
	    n = 3;
	    get_expands( expand, entry, n);

	    xentry = &expand[0];
	    *xentry = *entry;
	    reset_xdep_prep_info_ins(xentry, nuses-1);
	    for( i = 1; i < nuses; i++ ) {
	      xdep_prep_info_regs(xentry)[i-1] = in_regs[i];
	    }
	    reset_xdep_prep_info_outs(xentry, 0);
	    reset_xdep_prep_info_is_split(xentry, 1);
	    reset_xdep_prep_info_is_nocount(xentry, 1);
	    xdep_prep_info_res(xentry) = 0;
	    xdep_prep_info_res_compl(xentry) = 0x80808080;
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.store_unit);
	    reset_xdep_prep_info_is_memop(xentry, 1);
	    reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.store_latency);
	    reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.store_avail);

	    xentry = &expand[2];
	    *xentry = *entry;
	    reset_xdep_prep_info_ins(xentry, nuses-1);
	    for( i = 1; i < nuses; i++ ) {
	      xdep_prep_info_regs(xentry)[i-1] = in_regs[i];
	    }
	    xdep_prep_info_regs(xentry)[nuses-1] = out_regs[0];
	    reset_xdep_prep_info_is_store(xentry, 0);
	    reset_xdep_prep_info_is_memop(xentry, 0);
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	    reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	    reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	    m = 2;
	  }
	  else {
	    n = 2;
	    get_expands( expand, entry, n);
	    xentry = &expand[0];
	    *xentry = *entry;
	    reset_xdep_prep_info_ins(xentry, nuses-1);
	    for( i = 1; i < nuses; i++ ) {
	      xdep_prep_info_regs(xentry)[i-1] = in_regs[i];
	    }
	    reset_xdep_prep_info_outs(xentry, 1);
	    xdep_prep_info_regs(xentry)[nuses-1] = out_regs[0];
	    reset_xdep_prep_info_is_split(xentry, 1);
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.store_unit);
	    reset_xdep_prep_info_is_memop(xentry, 1);
	    reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.store_latency);
	    reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.store_avail);
	    m = 1;
	  }

	  xentry = &expand[1];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, 1);
	  reset_xdep_prep_info_outs(xentry, 0);
	  reset_xdep_prep_info_is_store(xentry, 0);
	  reset_xdep_prep_info_is_memop(xentry, 0);
	  reset_xdep_prep_info_is_transfer(xentry, 1);
	  reset_xdep_prep_info_is_reissue(xentry, 1);
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fpu_unit);
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency+1);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail); /* transfer */
	  xdep_prep_info_res(xentry) =  0;
	  xdep_prep_info_res_compl(xentry) =  0x80808080;
	}

	break;
      }

    case OPC_STBUX:
    case OPC_STHUX:
    case OPC_STWUX:
    case OPC_STDUX: {
	if( xdep_prep_ctl.flag_fxstux_split ) {
	  n = 3;
	  get_expands( expand, entry, n);

	  xentry = &expand[0];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, nuses-1);
	  for( i = 1; i < nuses; i++ ) {
	    xdep_prep_info_regs(xentry)[i-1] = in_regs[i];
	  }
	  reset_xdep_prep_info_outs(xentry, 0);
	  reset_xdep_prep_info_is_split(xentry, 1);
	  xdep_prep_info_res(xentry) = 0;
	  xdep_prep_info_res_compl(xentry) = 0x80808080;
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.store_unit);
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.store_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.store_avail);
	  reset_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[1];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, 1);
	  reset_xdep_prep_info_outs(xentry, 0);
	  reset_xdep_prep_info_is_store(xentry, 0);
	  reset_xdep_prep_info_is_memop(xentry, 0);
	  reset_xdep_prep_info_is_transfer(xentry, 1);
	  xdep_prep_info_res(xentry) = 0;
	  xdep_prep_info_res_compl(xentry) = 0x80808080;
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	  reset_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[2];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, nuses-1);
	  for( i = 1; i < nuses; i++ ) {
	    xdep_prep_info_regs(xentry)[i-1] = in_regs[i];
	  }
	  xdep_prep_info_regs(xentry)[nuses-1] = out_regs[0];
	  reset_xdep_prep_info_is_store(xentry, 0);
	  reset_xdep_prep_info_is_memop(xentry, 0);
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	  reset_xdep_prep_info_is_multi(xentry, 1);

	  reset_xdep_prep_info_is_slotN(&expand[n-1], 1);
      }
      break;
    }

  case OPC_STBU:
  case OPC_STHU:
  case OPC_STWU:
  case OPC_STDU: {
      if( xdep_prep_ctl.flag_fxstu_split ) {
	if( xdep_prep_ctl.flag_stu3_split ) {
	    n = 3;
	    get_expands( expand, entry, n);

	    xentry = &expand[0];
	    *xentry = *entry;
	    reset_xdep_prep_info_ins(xentry, nuses-1);
	    for( i = 1; i < nuses; i++ ) {
	      xdep_prep_info_regs(xentry)[i-1] = in_regs[i];
	    }
	    reset_xdep_prep_info_outs(xentry, 0);
	    reset_xdep_prep_info_is_split(xentry, 1);
	    reset_xdep_prep_info_is_nocount(xentry, 1);
	    xdep_prep_info_res(xentry) = 0;
	    xdep_prep_info_res_compl(xentry) = 0x80808080;
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.store_unit);
	    reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.store_latency);
	    reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.store_avail);

	    xentry = &expand[2];
	    *xentry = *entry;
	    reset_xdep_prep_info_ins(xentry, nuses-1);
	    for( i = 1; i < nuses; i++ ) {
	      xdep_prep_info_regs(xentry)[i-1] = in_regs[i];
	    }
	    xdep_prep_info_regs(xentry)[nuses-1] = out_regs[0];
	    reset_xdep_prep_info_is_store(xentry, 0);
	    reset_xdep_prep_info_is_memop(xentry, 0);
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	    reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	    reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	  }
	  else {
	    n = 2;
	    get_expands( expand, entry, n);

	    xentry = &expand[0];
	    *xentry = *entry;
	    reset_xdep_prep_info_ins(xentry, nuses-1);
	    for( i = 1; i < nuses; i++ ) {
	      xdep_prep_info_regs(xentry)[i-1] = in_regs[i];
	    }
	    reset_xdep_prep_info_outs(xentry, 1);
	    xdep_prep_info_regs(xentry)[nuses-1] = out_regs[0];
	    reset_xdep_prep_info_is_split(xentry, 1);
	    reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.store_unit);
	    reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.store_latency);
	    reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.store_avail);
	  }

	  xentry = &expand[1];
	  *xentry = *entry;
	  reset_xdep_prep_info_ins(xentry, 1);
	  reset_xdep_prep_info_outs(xentry, 0);
	  reset_xdep_prep_info_is_store(xentry, 0);
	  reset_xdep_prep_info_is_memop(xentry, 0);
	  reset_xdep_prep_info_is_transfer(xentry, 1);
	  reset_xdep_prep_info_is_reissue(xentry, 1);
	  xdep_prep_info_res(xentry) = 0;
	  xdep_prep_info_res_compl(xentry) = 0x80808080;
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	  reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	  reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	}

	break;
      }

    case OPC_STMW:
    case OPC_STMD: {
	if( xdep_prep_ctl.flag_mult_split ) {
	  n = 32 - OPC_DEC_RS(iword);
	  if( xdep_prep_ctl.flag_fxst_split ) {
	    n *= 2;
	    get_expands( expand, entry, n);

	    f_a = OPC_DEC_RA( iword );
	    f_a = OPC_GPR( f_a );
	    cvt_dep_reg(r_a, f_a, lg2max);

	    for( i = 0, f_s = OPC_DEC_RS(iword); f_s < 32; i++, f_s++ ) {

	      xentry = &expand[2*i];
	      *xentry = *entry;
	      xdep_prep_info_regs(xentry)[0] = r_a;
	      reset_xdep_prep_info_ins(xentry, 1);
	      reset_xdep_prep_info_is_split(xentry, 1);
	      reset_xdep_prep_info_is_over(xentry, 0);
	      reset_xdep_prep_info_dlength(xentry, 4);

	      assert( xdep_prep_info_ins(xentry) == 1);
	      assert( xdep_prep_info_is_split(xentry) == 1);
	      assert( xdep_prep_info_is_over(xentry) == 0);
	      assert( xdep_prep_info_dlength(xentry) == 4);

	      xentry = &expand[2*i+1];
	      *xentry = *entry;
	      f_d = OPC_GPR( f_s );
	      cvt_dep_reg( xdep_prep_info_regs(xentry)[0], f_d, lg2max );

	      reset_xdep_prep_info_ins(xentry, 1);
	      reset_xdep_prep_info_is_store(xentry, 0);
	      reset_xdep_prep_info_is_memop(xentry, 0);
	      reset_xdep_prep_info_is_over(xentry, 0);
	      reset_xdep_prep_info_is_transfer(xentry, 1);
	      reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	      reset_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	      reset_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);

	      assert( xdep_prep_info_ins(xentry) == 1);
	      assert( xdep_prep_info_is_store(xentry) == 0);
	      assert( xdep_prep_info_is_memop(xentry) == 0);
	      assert( xdep_prep_info_is_over(xentry) == 0);
	      assert( xdep_prep_info_is_transfer(xentry) == 1);
	      assert( xdep_prep_info_unit(xentry) == xdep_prep_ctl.fix_unit);
	      assert( xdep_prep_info_latency(xentry) == xdep_prep_ctl.fix_latency);
	      assert( xdep_prep_info_avail(xentry) == xdep_prep_ctl.fix_avail);
	    }
	    assert( n == 2*i );
	  }
	  else {
	    get_expands( expand, entry, n);
	    f_a = OPC_DEC_RA( iword );
	    f_a = OPC_GPR( f_a );
	    cvt_dep_reg(r_a, f_a, lg2max );

	    for( i = 0, f_s = OPC_DEC_RS(iword); f_s < 32; i++, f_s++ ) {

	      xentry = &expand[i];
	      *xentry = *entry;
	      xdep_prep_info_regs(xentry)[0] = r_a;
	      f_d = OPC_GPR(f_s);
	      cvt_dep_reg( xdep_prep_info_regs(xentry)[1], f_d, lg2max );

	      reset_xdep_prep_info_ins(xentry, 2);
	      reset_xdep_prep_info_is_split(xentry, 0);
	      reset_xdep_prep_info_is_over(xentry, 0);
	      reset_xdep_prep_info_dlength(xentry, 4);

	      assert( xdep_prep_info_ins(xentry) == 2);
	      assert( xdep_prep_info_is_split(xentry) == 0);
	      assert( xdep_prep_info_is_over(xentry) == 0);
	      assert( xdep_prep_info_dlength(xentry) == 4);
	    }
	    assert( n == i );
	  }
	}

	break;
      }

    case OPC_LMW:
    case OPC_LMD: {
	if( xdep_prep_ctl.flag_mult_split ) {
	  n = 32 - OPC_DEC_RT(iword);
	  get_expands( expand, entry, n);

	  f_a = OPC_DEC_RA( iword );
	  f_si = OPC_DEC_D( iword );

	  for( i = 0, f_d = OPC_DEC_RT(iword); f_d < 32; i++, f_d++ ) {
	    exp_word = OPC_ASM_LWZ(  f_d, f_si, f_a );
	    assert( i < n );
	    process_iword( iaddr, exp_word, &expand[i]
#if( IN_TRANS ) // turandot
			, cycle, processor_id, thread_id
#endif
			);
	    reset_xdep_prep_info_dlength( &expand[i], 4);
	    set_xdep_prep_info_is_multi(&expand[i], 1);
	  }
	  reset_xdep_prep_info_is_slotN(&expand[n-1], 1);
	  assert( n == i );
	}

	break;
      }

    case OPC_CROR:
    case OPC_CRORC:
    case OPC_CRNOR:
    case OPC_CRAND:
    case OPC_CRANDC:
    case OPC_CRNAND:
    case OPC_CRXOR:
    case OPC_CREQV: {
	if( xdep_prep_ctl.flag_crl_split ) {

	  if( !xdep_prep_ctl.flag_use_gp ||
		( in_regs[1] != out_regs[0] &&
		( in_regs[0] != out_regs[0] || xdep_prep_ctl.flag_use_gp == 3 )) ) {
	    unsigned split_avail = xdep_prep_ctl.flag_use_gp?2:1;
	    unsigned split_latency = xdep_prep_ctl.flag_use_gp?1:1;

	    n = 2;
	    get_expands(expand, entry, n);

	    xentry = &expand[0];
	    *xentry = *entry;

	    assert( xdep_prep_info_ins(xentry) == 3 );
	    reset_xdep_prep_info_ins(xentry, 2);
	    xdep_prep_info_regs(xentry)[1] = xdep_prep_info_regs(xentry)[2];
	    xdep_prep_info_regs(xentry)[2] = xdep_prep_info_regs(xentry)[3];
	    reset_xdep_prep_info_is_slot0(xentry, 1);
	    reset_xdep_prep_info_nexpands(xentry, 0);

	    reset_xdep_prep_info_avail(xentry, split_avail);
	    reset_xdep_prep_info_latency(xentry, split_latency);

	    xentry = &expand[1];
	    *xentry = *entry;

	    assert( xdep_prep_info_ins(xentry) == 3 );
	    reset_xdep_prep_info_ins(xentry, 2);
	    xdep_prep_info_regs(xentry)[0] = xdep_prep_info_regs(xentry)[1];
	    xdep_prep_info_regs(xentry)[1] = xdep_prep_info_regs(xentry)[2];
	    xdep_prep_info_regs(xentry)[2] = xdep_prep_info_regs(xentry)[3];
	    reset_xdep_prep_info_nexpands(xentry, 0);

	    tmpn = xdep_prep_info_avail(xentry) - split_avail;
	    if( tmpn < 1 ) {
	      tmpn = 1;
	    }
	    reset_xdep_prep_info_avail(xentry, tmpn);

	    tmpn = xdep_prep_info_latency(entry) - split_latency;
	    if( tmpn < 1 ) {
	      tmpn = 1;
	    }
	    reset_xdep_prep_info_latency(xentry, tmpn);
	  }
	}
	break;
       }

    case OPC_MFCR: {
	if( xdep_prep_ctl.flag_mult_split ) {
	  n = 7;
	  get_expands( expand, entry, n);

	  f_d = OPC_GPR(OPC_DEC_RT(iword));
	  cvt_dep_reg(r_d, f_d, lg2max);
	  assert( xdep_prep_reg_class( out_regs[j], lg2max ) == 0 );
	  assert( xdep_prep_info_res(entry) == RES_ONE_GPR );
	  assert( xdep_prep_info_res_compl(entry) == RESCPL_ONE_GPR );

	  xentry = &expand[0];
	  *xentry = *entry;

	  reset_xdep_prep_info_is_over(xentry, 0);

	  reset_xdep_prep_info_ins(xentry, 2);
	  cvt_dep_reg(r_s, OPC_CR(0), lg2max);
	  xdep_prep_info_regs(xentry)[0] = r_s;
	  cvt_dep_reg(r_s, OPC_CR(1), lg2max);
	  xdep_prep_info_regs(xentry)[1] = r_s;

	  reset_xdep_prep_info_outs(xentry, 1);
	  cvt_dep_reg(r_d, EXTRA_MFCR0_SPR, lg2max);
	  xdep_prep_info_regs(xentry)[2] = r_d;
	  xdep_prep_info_res(xentry) = RES_ONE_SPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.crl_unit);
	  set_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[1];
	  *xentry = *entry;

	  reset_xdep_prep_info_is_over(xentry, 0);

	  reset_xdep_prep_info_ins(xentry, 2);
	  cvt_dep_reg(r_s, OPC_CR(2), lg2max);
	  xdep_prep_info_regs(xentry)[0] = r_s;
	  cvt_dep_reg(r_s, OPC_CR(3), lg2max);
	  xdep_prep_info_regs(xentry)[1] = r_s;

	  reset_xdep_prep_info_outs(xentry, 1);
	  cvt_dep_reg(r_d, EXTRA_MFCR1_SPR, lg2max);
	  xdep_prep_info_regs(xentry)[2] = r_d;
	  xdep_prep_info_res(xentry) = RES_ONE_SPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.crl_unit);
	  set_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[2];
	  *xentry = *entry;

	  reset_xdep_prep_info_is_over(xentry, 0);

	  reset_xdep_prep_info_ins(xentry, 2);
	  cvt_dep_reg(r_s, EXTRA_MFCR0_SPR, lg2max);
	  xdep_prep_info_regs(xentry)[0] = r_s;
	  cvt_dep_reg(r_s, EXTRA_MFCR1_SPR, lg2max);
	  xdep_prep_info_regs(xentry)[1] = r_s;

	  reset_xdep_prep_info_outs(xentry, 1);
	  xdep_prep_info_regs(xentry)[2] = out_regs[0];
	  xdep_prep_info_res(xentry) = RES_ONE_GPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;
	  set_xdep_prep_info_is_multi(xentry, 1);
	  set_xdep_prep_info_is_slotN(xentry, 1);

	  xentry = &expand[3];
	  *xentry = *entry;

	  reset_xdep_prep_info_is_over(xentry, 0);

	  reset_xdep_prep_info_ins(xentry, 2);
	  cvt_dep_reg(r_s, OPC_CR(4), lg2max);
	  xdep_prep_info_regs(xentry)[0] = r_s;
	  cvt_dep_reg(r_s, OPC_CR(5), lg2max);
	  xdep_prep_info_regs(xentry)[1] = r_s;

	  reset_xdep_prep_info_outs(xentry, 1);
	  cvt_dep_reg(r_d, EXTRA_MFCR0_SPR, lg2max);
	  xdep_prep_info_regs(xentry)[2] = r_d;
	  xdep_prep_info_res(xentry) = RES_ONE_SPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.crl_unit);
	  set_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[4];
	  *xentry = *entry;

	  reset_xdep_prep_info_is_over(xentry, 0);

	  reset_xdep_prep_info_ins(xentry, 2);
	  cvt_dep_reg(r_s, OPC_CR(6), lg2max);
	  xdep_prep_info_regs(xentry)[0] = r_s;
	  cvt_dep_reg(r_s, OPC_CR(7), lg2max);
	  xdep_prep_info_regs(xentry)[1] = r_s;

	  reset_xdep_prep_info_outs(xentry, 1);
	  cvt_dep_reg(r_d, EXTRA_MFCR1_SPR, lg2max);
	  xdep_prep_info_regs(xentry)[2] = r_d;
	  xdep_prep_info_res(xentry) = RES_ONE_SPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	  reset_xdep_prep_info_unit(xentry, xdep_prep_ctl.crl_unit);
	  set_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[5];
	  *xentry = *entry;

	  reset_xdep_prep_info_is_over(xentry, 0);
	  reset_xdep_prep_info_ins(xentry, 2);
	  cvt_dep_reg(r_s, EXTRA_MFCR0_SPR, lg2max);
	  xdep_prep_info_regs(xentry)[0] = r_s;
	  cvt_dep_reg(r_s, EXTRA_MFCR1_SPR, lg2max);
	  xdep_prep_info_regs(xentry)[1] = r_s;

	  reset_xdep_prep_info_outs(xentry, 1);
	  cvt_dep_reg(r_d, EXTRA_MFCR0_SPR, lg2max);
	  xdep_prep_info_regs(xentry)[2] = r_d;
	  xdep_prep_info_res(xentry) = RES_ONE_SPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	  set_xdep_prep_info_is_multi(xentry, 1);

	  xentry = &expand[6];
	  *xentry = *entry;

	  reset_xdep_prep_info_is_over(xentry, 0);
	  reset_xdep_prep_info_ins(xentry, 2);
	  cvt_dep_reg(r_s, EXTRA_MFCR0_SPR, lg2max);
	  xdep_prep_info_regs(xentry)[0] = r_s;
	  xdep_prep_info_regs(xentry)[1] = out_regs[0];

	  reset_xdep_prep_info_outs(xentry, 1);
	  xdep_prep_info_regs(xentry)[2] = out_regs[0];
	  xdep_prep_info_res(xentry) = RES_ONE_GPR;
	  xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;
	  set_xdep_prep_info_is_multi(xentry, 1);

	  set_xdep_prep_info_is_slotN(&expand[n-1], 1);
	}
	break;
      }

    case OPC_MTCRF: {
	if( xdep_prep_ctl.flag_mult_split ) {
	  f_crm = OPC_DEC_FXM( iword );
	  /* if f_crm is a power of 2, skip */
	  if( (f_crm - 1) & f_crm) {
	    n = 0;
	    for( i = 1, j = f_crm; j;  i <<= 1, j >>= 1 ) {
	      if( j & 1 ) {
		n++;
	      }
	    }
	    get_expands( expand, entry, n);

	    assert( f_crm < (1<<8) );
	    n = 0;
	    for( i = 1; f_crm; f_crm >>= 1, i <<= 1 ) {
	      assert( i < (1<<8));
	      if( f_crm & 1 ) {
		exp_word = OPC_INS_FXM( iword, i );
		process_iword( iaddr, exp_word, &expand[n]
#if( IN_TRANS ) // turandot
			, cycle, processor_id, thread_id
#endif
		);
		n++;
	      }
	    }
	  }
	}
	break;
      }

    case OPC_LSCBX:
    case OPC_LSCBX_:
    case OPC_LSWX:
    case OPC_LSDX: {
	/* an lswx gets broken into 32 sets of instructions, each
	 * set consisting of:
	 *	string_spr0	<- check_set# BCNT
	 *	string_spr1	<- lwz rA, rB
	 *	rD		<- combine string_spr0, string_spr1, rD
	 */
	if( xdep_prep_ctl.flag_string_split ) {
	  n = 32*3;
	  get_expands( expand, entry, n);

	  f_d = OPC_DEC_RT(iword);
	  for( i = 0; i < 32; i++ ) {

	    xentry = &expand[3*i+0];
	    xdep_prep_info_prop0(xentry) = 0;
	    xdep_prep_info_prop1(xentry) = 0;
	    set_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	    set_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	    set_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	    set_xdep_prep_info_ins(xentry, 1);
	    set_xdep_prep_info_outs(xentry, 1);
	    set_xdep_prep_info_fetch(xentry, fetch);
	    set_xdep_prep_info_is_multi(xentry, 1);
	    set_xdep_prep_info_is_string(xentry, 1);
	    cvt_dep_reg(reg, OPC_XER_BCNT, lg2max);
	    xdep_prep_info_regs(xentry)[0] = reg;
	    cvt_dep_reg(reg, EXTRA0_STRING_SPR, lg2max);
	    xdep_prep_info_regs(xentry)[1] = reg;
	    xdep_prep_info_res(xentry) = RES_ONE_SPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	    set_xdep_prep_info_nexpands(xentry, 0);

	    xentry = &expand[3*i+1];
	    xdep_prep_info_prop0(xentry) = 0;
	    xdep_prep_info_prop1(xentry) = 0;
	    set_xdep_prep_info_is_memop(xentry, 1);
	    set_xdep_prep_info_is_load(xentry, 1);
	    set_xdep_prep_info_unit(xentry, xdep_prep_ctl.load_unit);
	    set_xdep_prep_info_latency(xentry, xdep_prep_ctl.load_latency);
	    set_xdep_prep_info_avail(xentry, xdep_prep_ctl.load_avail);
	    reset_xdep_prep_info_dlength( xentry, 4);
	    set_xdep_prep_info_is_multi(xentry, 1);
	    set_xdep_prep_info_is_string(xentry, 1);
	    if( OPC_DEC_RA(iword) == 0 ) {
	      set_xdep_prep_info_ins(xentry, 1);
	      set_xdep_prep_info_outs(xentry, 1);
	      set_xdep_prep_info_fetch(xentry, fetch);
	      xdep_prep_info_regs(xentry)[0] = in_regs[0];
	      cvt_dep_reg(reg, EXTRA1_STRING_SPR, lg2max);
	      xdep_prep_info_regs(xentry)[1] = reg;
	    }
	    else {
	      set_xdep_prep_info_ins(xentry, 2);
	      set_xdep_prep_info_outs(xentry, 1);
	      set_xdep_prep_info_fetch(xentry, fetch);
	      xdep_prep_info_regs(xentry)[0] = in_regs[0];
	      xdep_prep_info_regs(xentry)[1] = in_regs[1];
	      cvt_dep_reg(reg, EXTRA1_STRING_SPR, lg2max);
	      xdep_prep_info_regs(xentry)[2] = reg;
	    }
	    xdep_prep_info_res(xentry) = RES_ONE_SPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	    set_xdep_prep_info_nexpands(xentry, 0);

	    xentry = &expand[3*i+2];
	    xdep_prep_info_prop0(xentry) = 0;
	    xdep_prep_info_prop1(xentry) = 0;
	    set_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	    set_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	    set_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	    set_xdep_prep_info_ins(xentry, 3);
	    set_xdep_prep_info_outs(xentry, 1);
	    set_xdep_prep_info_fetch(xentry, fetch);
	    set_xdep_prep_info_is_multi(xentry, 1);
	    set_xdep_prep_info_is_string(xentry, 1);
	    cvt_dep_reg(reg, EXTRA0_STRING_SPR, lg2max);
	    xdep_prep_info_regs(xentry)[0] = reg;
	    cvt_dep_reg(reg, EXTRA1_STRING_SPR, lg2max);
	    xdep_prep_info_regs(xentry)[1] = reg;

	    r_d = (f_d + i)%32;
	    r_d += OPC_FIRST_GPR;
	    cvt_dep_reg(reg, r_d, lg2max);

	    xdep_prep_info_regs(xentry)[2] = reg;
	    xdep_prep_info_regs(xentry)[3] = reg;
	    xdep_prep_info_res(xentry) = RES_ONE_GPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_GPR;
	    set_xdep_prep_info_nexpands(xentry, 0);
	  }
	}
	break;
      }

    case OPC_LSWI:
    case OPC_LSDI: {
	if( xdep_prep_ctl.flag_string_split ) {
	  n = nsets;
	  get_expands( expand, entry, n);

	  f_a = OPC_DEC_RA( iword );
	  f_d = OPC_DEC_RT(iword);
	  for( i = 0; i < nsets; i++ ) {

	    f_si = 4*i;
	    exp_word = OPC_ASM_LWZ(  f_d, f_si, f_a );

	    if( f_d == f_a && f_a != 0) {
	      f_d = (f_d + 2)%32;
	    }
	    else {
	      f_d = (f_d + 1)%32;
	    }

	    assert( i < n );
	    process_iword( iaddr, exp_word, &expand[i]
#if( IN_TRANS ) // turandot
			, cycle, processor_id, thread_id
#endif
			);
	    reset_xdep_prep_info_dlength( &expand[i], 4);
	    set_xdep_prep_info_is_multi(&expand[i], 1);
	  }
	  reset_xdep_prep_info_is_slotN(&expand[n-1], 1);
	  assert( n == i );
	}

	break;
      }

    case OPC_STSWX:
    case OPC_STSDX: {
	/* an stswx gets broken into 32 sets of instructions, each
	 * set consisting of:
	 *	string_spr0	<- check_set# BCNT
	 *			<- agen rA, rB
	 *			<- dmove string_spr0, rS
	 */
	if( xdep_prep_ctl.flag_string_split ) {
	  n = 32*3;
	  f_s = OPC_DEC_RS(iword);
	  get_expands( expand, entry, n);

	  for( i = 0; i < 32; i++ ) {

	    xentry = &expand[3*i+0];
	    xdep_prep_info_prop0(xentry) = 0;
	    xdep_prep_info_prop1(xentry) = 0;
	    set_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	    set_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	    set_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	    set_xdep_prep_info_ins(xentry, 1);
	    set_xdep_prep_info_outs(xentry, 1);
	    set_xdep_prep_info_fetch(xentry, fetch);
	    set_xdep_prep_info_is_multi(xentry, 1);
	    set_xdep_prep_info_is_string(xentry, 1);
	    cvt_dep_reg(reg, OPC_XER_BCNT, lg2max);
	    xdep_prep_info_regs(xentry)[0] = reg;
	    cvt_dep_reg(reg, EXTRA0_STRING_SPR, lg2max);
	    xdep_prep_info_regs(xentry)[1] = reg;
	    xdep_prep_info_res(xentry) = RES_ONE_SPR;
	    xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;
	    set_xdep_prep_info_nexpands(xentry, 0);

	    xentry = &expand[3*i+1];
	    xdep_prep_info_prop0(xentry) = 0;
	    xdep_prep_info_prop1(xentry) = 0;
	    set_xdep_prep_info_is_memop(xentry, 1);
	    set_xdep_prep_info_is_store(xentry, 1);
	    set_xdep_prep_info_is_split(xentry, 1);
	    set_xdep_prep_info_is_multi(xentry, 1);
	    set_xdep_prep_info_is_string(xentry, 1);
	    reset_xdep_prep_info_dlength( xentry, 4);
	    set_xdep_prep_info_unit(xentry, xdep_prep_ctl.store_unit);
	    set_xdep_prep_info_latency(xentry, xdep_prep_ctl.store_latency);
	    set_xdep_prep_info_avail(xentry, xdep_prep_ctl.store_avail);
	    set_xdep_prep_info_fetch(xentry, fetch);

	    if( OPC_DEC_RA(iword) == 0 ) {
	      set_xdep_prep_info_ins(xentry, 1);
	      xdep_prep_info_regs(xentry)[0] = in_regs[0];
	    }
	    else {
	      set_xdep_prep_info_ins(xentry, 2);
	      xdep_prep_info_regs(xentry)[0] = in_regs[0];
	      xdep_prep_info_regs(xentry)[1] = in_regs[1];
	    }
	    xdep_prep_info_res(xentry) = RES_NONE;
	    xdep_prep_info_res_compl(xentry) = RESCPL_NONE;
	    set_xdep_prep_info_nexpands(xentry, 0);

	    xentry = &expand[3*i+2];
	    xdep_prep_info_prop0(xentry) = 0;
	    xdep_prep_info_prop1(xentry) = 0;
	    set_xdep_prep_info_is_store(xentry, 0);
	    set_xdep_prep_info_is_transfer(xentry, 1);
	    set_xdep_prep_info_is_multi(xentry, 1);
	    set_xdep_prep_info_is_string(xentry, 1);
	    set_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	    set_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	    set_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	    set_xdep_prep_info_ins(xentry, 2);
	    set_xdep_prep_info_fetch(xentry, fetch);

	    cvt_dep_reg(reg, EXTRA0_STRING_SPR, lg2max);
	    xdep_prep_info_regs(xentry)[0] = reg;

	    r_s = (i+f_s)%32;
	    r_s += OPC_FIRST_GPR;
	    cvt_dep_reg(reg, r_s, lg2max);
	    xdep_prep_info_regs(xentry)[1] = reg;

	    xdep_prep_info_res(xentry) = RES_NONE;
	    xdep_prep_info_res_compl(xentry) = RESCPL_NONE;
	    set_xdep_prep_info_nexpands(xentry, 0);
	  }
	}
	break;
      }

    case OPC_STSWI:
    case OPC_STSDI: {
	if( xdep_prep_ctl.flag_string_split ) {
	  f_a = OPC_DEC_RA( iword );

	  if( f_a ) {
	    m = nuses-1;
	  }
	  else {
	    m = nuses;
	  }

	  #if( !IN_TRANS )
	    if( m <= 0 ) {
	      opc_dis(iword, buf);
	      fprintf(stderr, "instruction has 0 uses: %s\n", buf );
	    }
	  #endif
	  assert( m > 0 );


	  n = 2*m;
	  get_expands( expand, entry, n);

	  f_s = OPC_DEC_RS(iword);
	  for( i = 0; i < m; i++ ) {

	    xentry = &expand[2*i+0];
	    xdep_prep_info_prop0(xentry) = 0;
	    xdep_prep_info_prop1(xentry) = 0;
	    set_xdep_prep_info_is_memop(xentry, 1);
	    set_xdep_prep_info_is_store(xentry, 1);
	    set_xdep_prep_info_is_split(xentry, 1);
	    set_xdep_prep_info_is_multi(xentry, 1);
	    set_xdep_prep_info_unit(xentry, xdep_prep_ctl.store_unit);
	    reset_xdep_prep_info_dlength( xentry, 4);
	    set_xdep_prep_info_latency(xentry, xdep_prep_ctl.store_latency);
	    set_xdep_prep_info_avail(xentry, xdep_prep_ctl.store_avail);
	    set_xdep_prep_info_fetch(xentry, fetch);
	    if( f_a == 0 ) {
	      set_xdep_prep_info_ins(xentry, 0);
	    }
	    else {
	      set_xdep_prep_info_ins(xentry, 1);
	      xdep_prep_info_regs(xentry)[0] = in_regs[0];
	    }
	    xdep_prep_info_res(xentry) = RES_NONE;
	    xdep_prep_info_res_compl(xentry) = RESCPL_NONE;
	    set_xdep_prep_info_nexpands(xentry, 0);

	    xentry = &expand[2*i+1];
	    xdep_prep_info_prop0(xentry) = 0;
	    xdep_prep_info_prop1(xentry) = 0;
	    reset_xdep_prep_info_is_store(xentry, 0);
	    reset_xdep_prep_info_is_transfer(xentry, 1);
	    set_xdep_prep_info_is_multi(xentry, 1);
	    set_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	    set_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	    set_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	    set_xdep_prep_info_ins(xentry, 1);
	    set_xdep_prep_info_fetch(xentry, fetch);

	    r_s = OPC_FIRST_GPR+f_s;
	    cvt_dep_reg(reg, r_s, lg2max);
	    xdep_prep_info_regs(xentry)[0] = reg;

	    xdep_prep_info_res(xentry) = RES_NONE;
	    xdep_prep_info_res_compl(xentry) = RESCPL_NONE;
	    set_xdep_prep_info_nexpands(xentry, 0);

	    f_s = (f_s + 1)%32;
	  }
	  reset_xdep_prep_info_is_slotN(&expand[n-1], 1);
	  assert( n == 2*i );
	}

	break;
      }

    }/* switch (idx) */


    /* if the instruction is expanded into multiple operations, zhigangh@1/10/2003 */		
    if( n ) {
      set_xdep_prep_info_nexpands(entry, n);
      assert( xdep_prep_info_nexpands(entry) == n );
      xdep_prep_info_expands(entry) = expand;

      if( xdep_prep_ctl.flag_show_split ) {
	opc_dis(iword, buf);
	fprintf(stderr, "expanding [%08x@%08x] %s into %d expands\n", iword, iaddr, buf, n);
      }

      for( i = 0; i < n; i++ ) {
	int is_bad;

	xentry = &expand[i];

	nuses = xdep_prep_info_ins(xentry);
	nsets = xdep_prep_info_outs(xentry);
	in_regs = (nuses+nsets)>4? xdep_prep_info_overflow(xentry) : xdep_prep_info_regs( xentry );
	out_regs = in_regs + nuses;

	for( j = 0; j < 4; j++ ) {
	  req[j] = 0;
	}

	for( j = 0; j < nsets; j++ ) {
	  unsigned clss = xdep_prep_reg_class( out_regs[j], lg2max );
	  assert( clss < 4 );
	  req[clss]++;
	}
	whole = (req[3]<<24)|(req[2]<<16)|(req[1]<<8)|req[0];
	is_bad =  whole != xdep_prep_info_res(xentry);
	is_bad |= req[0] != (xdep_prep_info_res(xentry) & 0xff);
	is_bad |= req[1] != ((xdep_prep_info_res(xentry)>>8) & 0xff);
	is_bad |= req[2] != ((xdep_prep_info_res(xentry)>>16) & 0xff);
	is_bad |= req[3] != ((xdep_prep_info_res(xentry)>>24) & 0xff);

	if( is_bad ) {
	  fprintf(stderr, "expand #%d: uses = %d, sets = %d\n", i, nuses, nsets);
	  for( j = 0; j < 4; j++ ) {
	    fprintf(stderr, "class %d, res_vec = 0x%02x, out_reg = 0x%02x\n", j, (xdep_prep_info_res(xentry)>>(8*j))&0xff, req[j]);
	  }
	  fatal("bad instruction detected\n");
	}
	assert( req[0] == (xdep_prep_info_res(xentry) & 0xff) );
	assert( req[1] == ((xdep_prep_info_res(xentry)>>8) & 0xff) );
	assert( req[2] == ((xdep_prep_info_res(xentry)>>16) & 0xff) );
	assert( req[3] == ((xdep_prep_info_res(xentry)>>24) & 0xff) );
      }

      #if( !NDEBUG )
	for( i = 0; i < n; i++ ) {
	  xentry = &expand[i];
	  if( !xdep_prep_info_res(xentry) && !xdep_prep_info_res_compl(xentry) ) {
	    fprintf(stderr, "expand #%d: res = 0x%08x compl = 0x%08x\n",
		i, xdep_prep_info_res(xentry), !xdep_prep_info_res_compl(xentry));
		fatal("bad instruction detected\n");
	  }
	  assert( xdep_prep_info_res(xentry) || xdep_prep_info_res_compl(xentry));
	  assert( xdep_prep_info_res(xentry) != 0 || xdep_prep_info_res_compl(xentry) == 0x80808080 );
	  assert( xdep_prep_info_res(xentry) == 0 || xdep_prep_info_res_compl(xentry) != 0x80808080 );
	  /* change this check if number of classes changes from 1<<2 */
  	  regs = xdep_prep_info_is_over(xentry)? xdep_prep_info_overflow(xentry) : xdep_prep_info_regs( xentry );
	  for( j = 0; j < xdep_prep_info_ins(xentry) + xdep_prep_info_outs(xentry); j++ ) {
	    if(  regs[j] >= (1<<(2+lg2max)) ) {
	      fprintf(stderr, "expand #%d: (ov=%d) regs #%d = %d: max = %d (%d)\n",
		i, xdep_prep_info_is_over(xentry), j, regs[j], 1<<(2+lg2max), lg2max);
		  fatal("bad instruction detected\n");
	    }
	    assert( regs[j] < (1<<(2+lg2max)) );
	  }
	}
      #endif
    }/* if (n) */
  }/* if( xdep_prep_ctl.flag_all_split ) */

/*****
opc_dis(iword, buf);
fprintf(stderr, "[%08x] %s = (%d %d) %08x %08x\n", iword, buf, xdep_prep_info_ins(entry), xdep_prep_info_outs(entry),
	xdep_prep_info_res(entry), xdep_prep_info_res_compl(entry));
****/

  assert( xdep_prep_info_res(entry) || xdep_prep_info_res_compl(entry));
  assert( xdep_prep_info_res(entry) != 0 || xdep_prep_info_res_compl(entry) == 0x80808080 );
  assert( xdep_prep_info_res(entry) == 0 || xdep_prep_info_res_compl(entry) != 0x80808080 );
  #if( IN_TRANS )
    TRCOUT( fprintf(the_stderr, "DEP_PREP [final]: entry = %08x %08x@%08x res=%08x %08x\n",
	(unsigned) entry, iword, iaddr,
	(unsigned) xdep_prep_info_res(entry), (unsigned) xdep_prep_info_res_compl(entry)) );
  #endif
}/* process_iword */

void
process_branch(
	unsigned		iaddr,
	unsigned		iword,
	xdep_prep_info *	entry,
	unsigned		max_dist,
	unsigned		niwords[],
	xdep_prep_info 		nentrys[]
#if( IN_TRANS ) // turandot
	, xcycle cycle, // jdonald adding the next three just for debugging purposes
	int processor_id,
	int thread_id
#endif
	)
{
  int			i;
  int			j;
  int			k;
  xdep_prep_info *	xentry;
  xdep_prep_info *	atentry;
  // xdep_prep_info *	atflush; // unused?
  xdep_prep_info *	nentry;
  xdep_prep_info *	nexpand;
  xdep_prep_info *	expand;
  unsigned		niword;
  int			ndist;
  int			nexps;
  int			nexpands;
  int			nat;
  int			n;
  char			buf[512];
  unsigned		lg2max = xdep_prep_ctl.flag_lg2arch_max; // for !defined(NDEBUG)
  unsigned		bicr;
  unsigned		cr;
  xarch			cvtcr;
  // xarch			dep; // unused?
  unsigned		nins = 0; // initialize
  unsigned		nouts = 0; // initialize
  int			is_store;
  int			is_load;
  int			is_transfer;
  unsigned		unit;
  // xarch			out0; // unused?
  // unsigned		sel; // unused?
  unsigned		tot;
  xarch *		overflow;
  xarch *		nregs;
  xarch *		xregs;
  // unsigned		res; // unused?
  // unsigned		compl; // unused?
  unsigned		dist;
  xarch			crs[512];
  unsigned		ncrs;
  int			seen_only_bcs;
  // opc_idx		idx; // unused?
  unsigned		xins;
  unsigned		xouts;
  unsigned		req[4]; // for !defined(NDEBUG)
  unsigned	whole; // for !defined(NDEBUG)
  int		is_bad; // for !defined(NDEBUG)

  if( xdep_prep_ctl.flag_show_pred ) {
    opc_dis(iword, buf);
    fprintf(stderr, "trying to predicate 0x%08x: 0x%08x %s\n", iaddr, iword, buf);
  }

  if( (iword & 0xfec00003) != 0x40c00000 ) {
    if( xdep_prep_ctl.flag_show_pred ) {
      fprintf(stderr, "ignoring: not predicated branch\n");
    }
    return;
  }

  dist = OPC_DEC_BD(iword) - 1;
  if( dist >= max_dist ) {
    if( xdep_prep_ctl.flag_show_pred ) {
      fprintf(stderr, "ignoring: dist too large (%d)\n", dist);
    }
    return;
  }

  bicr = OPC_DEC_BI(iword) >> 2;
  cr = OPC_FIRST_CR + bicr;
  cvt_dep_reg(cvtcr, cr, lg2max);
  crs[0] = cvtcr;
  ncrs = 1;

  nexps = 0;
  seen_only_bcs = 1;
  for( i = 0; i < dist; i++ ) {
    niword = niwords[i];
    nentry = &nentrys[i];

    if( xdep_prep_ctl.flag_show_pred ) {
      opc_dis(niword, buf);
      fprintf(stderr, "\t\tentry 0x%p: 0x%08x %s\n", nentry, niword, buf);
    }

/* look for skip skip ... */
    if( xdep_prep_info_is_branch(nentry) ) {
      if( !seen_only_bcs ) {
	if( xdep_prep_ctl.flag_show_pred ) {
	  fprintf(stderr, "\t\tignoring: do not handle branches after other ops\n");
	}
	return;
      }
      if( ncrs == 512 ) {
	if( xdep_prep_ctl.flag_show_pred ) {
	  fprintf(stderr, "\t\tignoring: too many bcs\n");
	}
	return;
      }

      if( (niword & 0xfec00003) != 0x40c00000 ) {
	if( xdep_prep_ctl.flag_show_pred ) {
	  fprintf(stderr, "\t\tignoring: predicates non-predicated branch\n");
	}
	return;
      }

      ndist = OPC_DEC_BD(niword) - 1;
      if( ndist + i + 1 != dist ) {
	if( xdep_prep_ctl.flag_show_pred ) {
	  fprintf(stderr, "\t\tdo not handle bcs that do not go to end %d vs %d\n",
		dist, ndist + i + 1 );
	}
	return;
      }

      bicr = OPC_DEC_BI(iword) >> 2;
      cr = OPC_FIRST_CR + bicr;
      cvt_dep_reg(cvtcr, cr, lg2max);
      crs[ncrs] = cvtcr;
      ncrs++;

      continue;
    }

/* seen skip ... skip op */

    seen_only_bcs = 0;

    if( xdep_prep_info_nexpands(nentry) ) {
      n = xdep_prep_info_nexpands(nentry);
      nexpand = xdep_prep_info_expands(nentry);
      #if( DEBUG )
      fprintf(stderr, "\talready has %d expands\n", n);
      #endif
    }
    else {
      n = 1;
      nexpand = nentry;
      #if( DEBUG )
      fprintf(stderr, "\tusing instruction itself\n"); // extra input: n /jdonald
      #endif
    }

    for( j = 0; j < n; j++ ) {
      atentry = &nexpand[j];

      if( xdep_prep_info_is_branch(atentry) ) {
	if( xdep_prep_ctl.flag_show_pred ) {
	  fprintf(stderr, "\t\tdo not handle predicated branches\n");
	}
	return;
      }

      nins = xdep_prep_info_ins(atentry);
      nouts = xdep_prep_info_outs(atentry);
      is_store = xdep_prep_info_is_store(atentry);
      is_load = xdep_prep_info_is_load(atentry);
      is_transfer = xdep_prep_info_is_transfer(atentry);
      unit = xdep_prep_info_unit(atentry);

      #if( DEBUG )
      fprintf(stderr, "\tins = %d outs = %d\n", nins, nouts);
      #endif

      nexps++;
    }

  }


/* actually do the predication */
  if( xdep_prep_ctl.pred_mode == 1 ) {
    nexpands = nexps*2;
  }
  else {
    nexpands = nexps;
  }
  get_expands(expand, entry, nexpands);
  nat = 0;

  for( i = ncrs-1; i < dist; i++ ) {
    niword = niwords[i];
    nentry = &nentrys[i];

    if( xdep_prep_ctl.flag_show_pred ) {
      opc_dis(niword, buf);
      fprintf(stderr, "\t\tprocessing entry 0x%p: 0x%08x %s\n", nentry, niword, buf);
    }

    if( xdep_prep_info_nexpands(nentry) ) {
      n = xdep_prep_info_nexpands(nentry);
      nexpand = xdep_prep_info_expands(nentry);
    }
    else {
      n = 1;
      nexpand = nentry;
    }

    for( j = 0; j < n; j++ ) {
      atentry = &nexpand[j];

      nins = xdep_prep_info_ins(atentry);
      nouts = xdep_prep_info_outs(atentry);
      is_store = xdep_prep_info_is_store(atentry);
      is_load = xdep_prep_info_is_load(atentry);
      is_transfer = xdep_prep_info_is_transfer(atentry);
      unit = xdep_prep_info_unit(atentry);

      if( xdep_prep_ctl.pred_mode == 1 ) {
	/* will expand as flush */
	xentry = &expand[nat+nexps];
	*xentry = *atentry;
	xins = nouts + nouts + ncrs;
	xouts = nouts;
	reset_xdep_prep_info_ins(xentry, xins);
	reset_xdep_prep_info_outs(xentry, xouts);
	tot = xins + xouts;
	if( xdep_prep_info_is_over(atentry) ) {
	  nregs = xdep_prep_info_overflow(atentry);
	}
	else {
	  nregs = xdep_prep_info_regs(atentry);
	}

	if(  tot > 4 ) {
	  reset_xdep_prep_info_is_over(xentry, 1);
	  get_overflows( overflow, xentry, tot);
	  xregs = overflow;
	}
	else {
	  xregs = xdep_prep_info_regs( xentry );
	}

	for( k = 0; k < ncrs; k++ ) {
	  xregs[k] = crs[k];
	}
	for( k = 0; k < nouts; k++ ) {
	  xregs[k+ncrs] = nregs[nins+k];
	}
	for( k = 0; k < nouts; k++ ) {
	  xregs[k+xins] = nregs[nins+k];
	}
      } 

      {
	/* will expand as a conditional */

	xentry = &expand[nat];
	nat++;
	*xentry = *atentry;

	xins = nins + nouts + ncrs;
	xouts = nouts;
	reset_xdep_prep_info_ins(xentry, xins);
	reset_xdep_prep_info_outs(xentry, xouts);

	tot = xins + xouts;

	if( xdep_prep_info_is_over(atentry) ) {
	  nregs = xdep_prep_info_overflow(atentry);
	}
	else {
	  nregs = xdep_prep_info_regs(atentry);
	}

	if( tot > 4 ) {
	  reset_xdep_prep_info_is_over(xentry, 1);
	  get_overflows( overflow, xentry, tot);
	  xregs = overflow;
	}
	else {
	  xregs = xdep_prep_info_regs( xentry );
	}

	for( k = 0; k < ncrs; k++ ) {
	  xregs[k] = crs[k];
	}
	for( k = 0; k < nins; k++ ) {
	  xregs[k+ncrs] = nregs[k];
	}
	for( k = 0; k < nouts; k++ ) {
	  xregs[k+ncrs+nins] = nregs[nins+k];
	}
	for( k = 0; k < nouts; k++ ) {
	  xregs[k+xins] = nregs[nins+k];
	}

      } /* end expand as conditional */
    }
  }

  assert( nat == nexps );
  reset_xdep_prep_info_nexpands(entry, nexpands);
  xdep_prep_info_expands(entry) = expand;

/* done; reset is just debug and tracing */

  assert( dist );
  reset_xdep_prep_info_pred(entry, dist);
  assert( xdep_prep_info_pred_dist(entry) == dist );
  assert( xdep_prep_info_is_pred(entry) );

  #if( !NDEBUG )
    for( i = 0; i < nexpands; i++ ) {
      xentry = &expand[i];
      assert( xdep_prep_info_res(xentry) || xdep_prep_info_res_compl(xentry));
      assert( xdep_prep_info_res(xentry) != 0 || xdep_prep_info_res_compl(xentry) == 0x80808080 );
      assert( xdep_prep_info_res(xentry) == 0 || xdep_prep_info_res_compl(xentry) != 0x80808080 );
      /* change this check if number of classes changes from 2 */

      if( xdep_prep_info_is_over(xentry) ) {
	xregs = xdep_prep_info_overflow(xentry);
      }
      else {
	xregs = xdep_prep_info_regs(xentry);
      }
      xins = xdep_prep_info_ins(xentry);
      xouts = xdep_prep_info_outs(xentry);
      #if( DEBUG )
      fprintf(stderr, "checking %d: %d, %d\n", i, nins, nouts );
      #endif

      for( j = 0; j < xins + xouts; j++ ) {
	#if( DEBUG )
	fprintf(stderr, "\t%d/%d=%d+%d, %d, %d (%d)\n", j, nins+nouts, nins, nouts, xregs[j], 1<<(2+lg2max), lg2max);
	#endif
	assert( xregs[j] < (1<<(2+lg2max)) );

      }

      for( j = 0; j < 4; j++ ) {
	req[j] = 0;
      }

      for( j = 0; j < xouts; j++ ) {
	unsigned clss = xdep_prep_reg_class( xregs[xins+j], lg2max );
	assert( clss < 4 );
	req[clss]++;
      }
      whole = (req[3]<<24)|(req[2]<<16)|(req[1]<<8)|req[0];
      is_bad =  whole != xdep_prep_info_res(xentry);
      is_bad |= req[0] != (xdep_prep_info_res(xentry) & 0xff);
      is_bad |= req[1] != ((xdep_prep_info_res(xentry)>>8) & 0xff);
      is_bad |= req[2] != ((xdep_prep_info_res(xentry)>>16) & 0xff);
      is_bad |= req[3] != ((xdep_prep_info_res(xentry)>>24) & 0xff);

      if( is_bad ) {
	fprintf(stderr, "expand #%d: uses = %d, sets = %d\n", i, xins, xouts);
	for( j = 0; j < 4; j++ ) {
	  fprintf(stderr, "class %d, by res_vec = 0x%02x, by out_reg = 0x%02x\n",
		j, (xdep_prep_info_res(xentry)>>(8*j))&0xff, req[j]);
	}
	  fatal("bad instruction detected\n");
      }
      assert( req[0] == (xdep_prep_info_res(xentry) & 0xff) );
      assert( req[1] == ((xdep_prep_info_res(xentry)>>8) & 0xff) );
      assert( req[2] == ((xdep_prep_info_res(xentry)>>16) & 0xff) );
      assert( req[3] == ((xdep_prep_info_res(xentry)>>24) & 0xff) );

    }
  #endif /* !NDEBUG */

  assert( xdep_prep_info_nexpands(entry) == nexpands );
  if( xdep_prep_ctl.flag_show_pred ) {
    fprintf(stderr, "entry 0x%p at 0x%08x has %d expands controlled by %d crs\n", entry, iaddr, xdep_prep_info_nexpands(entry), ncrs);
  }
}

/************************************************************************/
/* Code from first which I'm keeping around in case I want to		*/
/* implement speculative execution, then select				*/
/************************************************************************/

#if 0
      if( nouts == 1 && unit == xdep_prep_ctl.fix_unit
	&& !is_store && !is_transfer && !is_load ) {
      /* will expand with a select */

	#if( DEBUG )
	fprintf(stderr, "\t%d: +%d\n", j, 2);
	#endif

	nexps += 2;
      }
      else {
      /* will expand as a conditional */

	#if( DEBUG )
	fprintf(stderr, "\t%d: +%d\n", j, 1);
	#endif
	nexps++;
      }
#endif
#if 0
{
      if( nouts == 1 && unit == xdep_prep_ctl.fix_unit
	&& !is_store && !is_transfer && !is_load) {
	/* expand with select */
	xentry = &expand[nat];
	nat++;
	*xentry = *atentry;
	reset_xdep_prep_info_is_multi(xentry, 1);

	if( xdep_prep_info_is_over(nentry) ) {
	  nregs = xdep_prep_info_overflow(nentry);
	  reset_xdep_prep_info_is_over(xentry, 1);
	  get_overflows( overflow, xentry, tot);
	  xregs = overflow;
	  for( k = 0;  k < nins + nouts; k++ ) {
	    xregs[k] = nregs[k];
	  }
	}
	else {
	  xregs = xdep_prep_info_regs(xentry);
	}

	res = xdep_prep_info_res(xentry);
	compl = xdep_prep_info_res_compl(xentry);
	out0 = xdep_prep_info_regs(atentry)[nins];

	cvt_dep_reg(dep, EXTRA_PREDICATION_SPR, lg2max);
	xregs[nins] = dep;
	xdep_prep_info_res(xentry) = RES_ONE_SPR;
	xdep_prep_info_res_compl(xentry) = RESCPL_ONE_SPR;

	xentry = &expand[nat];
	nat++;

	xdep_prep_info_prop0(xentry) = 0;
	xdep_prep_info_prop1(xentry) = 0;
	set_xdep_prep_info_fetch(xentry, xdep_prep_info_fetch(nentry));
	set_xdep_prep_info_ins(xentry, 2+ncrs);
	set_xdep_prep_info_outs(xentry, 1);
	set_xdep_prep_info_unit(xentry, xdep_prep_ctl.fix_unit);
	set_xdep_prep_info_latency(xentry, xdep_prep_ctl.fix_latency);
	set_xdep_prep_info_avail(xentry, xdep_prep_ctl.fix_avail);
	set_xdep_prep_info_is_multi(xentry, 1);

	if( 2+ncrs+1 > 4 ) {
	  reset_xdep_prep_info_is_over(xentry, 1);
	  get_overflows( overflow, xentry, 3 + ncrs);
	  xregs = overflow;
	}
	else {
	  xregs = xdep_prep_info_regs(xentry);
	}

	for( k = 0; k < ncrs; k++ ) {
	  xregs[k] = crs[k];
	}
	xregs[ncrs+0] = dep;
	xregs[ncrs+1] = out0;
	xregs[ncrs+2] = out0;

	xdep_prep_info_res(xentry) = res;
	xdep_prep_info_res_compl(xentry) = compl;
	xdep_prep_info_expands(xentry) = 0;
      }
      else
}
#endif
